
#ifndef ELF_RELOC
#error "ELF_RELOC must be defined"
#endif

/* TODO: this is just a subset */
ELF_RELOC(X86_NONE,           0)
ELF_RELOC(X86_32,             1)
ELF_RELOC(X86_PC32,           2)
ELF_RELOC(X86_GOT32,          3)
ELF_RELOC(X86_PLT32,          4)
ELF_RELOC(X86_COPY,           5)
ELF_RELOC(X86_GLOB_DAT,       6)
ELF_RELOC(X86_JUMP_SLOT,      7)
ELF_RELOC(X86_RELATIVE,       8)
ELF_RELOC(X86_GOTOFF,         9)
ELF_RELOC(X86_GOTPC,          10)
ELF_RELOC(X86_32PLT,          11)
ELF_RELOC(X86_TLS_TPOFF,      14)
ELF_RELOC(X86_TLS_IE,         15)
ELF_RELOC(X86_TLS_GOTIE,      16)
ELF_RELOC(X86_TLS_LE,         17)
ELF_RELOC(X86_TLS_GD,         18)
ELF_RELOC(X86_TLS_LDM,        19)
ELF_RELOC(X86_16,             20)
ELF_RELOC(X86_PC16,           21)
ELF_RELOC(X86_8,              22)
ELF_RELOC(X86_PC8,            23)
ELF_RELOC(X86_TLS_GD_32,      24)
ELF_RELOC(X86_TLS_GD_PUSH,    25)
ELF_RELOC(X86_TLS_GD_CALL,    26)
ELF_RELOC(X86_TLS_GD_POP,     27)
ELF_RELOC(X86_TLS_LDM_32,     28)
ELF_RELOC(X86_TLS_LDM_PUSH,   29)
ELF_RELOC(X86_TLS_LDM_CALL,   30)
ELF_RELOC(X86_TLS_LDM_POP,    31)
ELF_RELOC(X86_TLS_LDO_32,     32)
ELF_RELOC(X86_TLS_IE_32,      33)
ELF_RELOC(X86_TLS_LE_32,      34)
ELF_RELOC(X86_TLS_DTPMOD32,   35)
ELF_RELOC(X86_TLS_DTPOFF32,   36)
ELF_RELOC(X86_TLS_TPOFF32,    37)
ELF_RELOC(X86_TLS_GOTDESC,    39)
ELF_RELOC(X86_TLS_DESC_CALL,  40)
ELF_RELOC(X86_TLS_DESC,       41)
ELF_RELOC(X86_IRELATIVE,      42)
