/* Copyright 2022 - 2025 R. Thomas
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef LIEF_DWARF_TYPES_H
#define LIEF_DWARF_TYPES_H
#include <LIEF/DWARF/Type.hpp>
#include <LIEF/DWARF/types/ClassLike.hpp>
#include <LIEF/DWARF/types/Pointer.hpp>
#include <LIEF/DWARF/types/Const.hpp>
#include <LIEF/DWARF/types/Base.hpp>
#include <LIEF/DWARF/types/Array.hpp>
#include <LIEF/DWARF/types/Typedef.hpp>
#include <LIEF/DWARF/types/Atomic.hpp>
#include <LIEF/DWARF/types/Coarray.hpp>
#include <LIEF/DWARF/types/Dynamic.hpp>
#include <LIEF/DWARF/types/File.hpp>
#include <LIEF/DWARF/types/Immutable.hpp>
#include <LIEF/DWARF/types/Interface.hpp>
#include <LIEF/DWARF/types/PointerToMember.hpp>
#include <LIEF/DWARF/types/RValueRef.hpp>
#include <LIEF/DWARF/types/Reference.hpp>
#include <LIEF/DWARF/types/Restrict.hpp>
#include <LIEF/DWARF/types/SetTy.hpp>
#include <LIEF/DWARF/types/Shared.hpp>
#include <LIEF/DWARF/types/StringTy.hpp>
#include <LIEF/DWARF/types/Subroutine.hpp>
#include <LIEF/DWARF/types/TemplateAlias.hpp>
#include <LIEF/DWARF/types/Thrown.hpp>
#include <LIEF/DWARF/types/Volatile.hpp>
#include <LIEF/DWARF/types/Enum.hpp>
#endif
