#ifndef FUNCTION_VARIANT_FLAG
#error "FUNCTION_VARIANT_FLAG must be defined"
#endif

FUNCTION_VARIANT_FLAG(SYSTEM_WIDE_DEFAULT      , 0  , "default"     )

FUNCTION_VARIANT_FLAG(SYSTEM_WIDE_INTERNAL_INSTALL   , 1  , "internalInstall"  )
FUNCTION_VARIANT_FLAG(SYSTEM_WIDE_CUSTOMER_INSTALL   , 2  , "customerInstall"  )
FUNCTION_VARIANT_FLAG(SYSTEM_WIDE_LOCKDOWN           , 3  , "lockdown"         )
