#ifndef FUNCTION_VARIANT_FLAG
#error "FUNCTION_VARIANT_FLAG must be defined"
#endif

FUNCTION_VARIANT_FLAG(X86_64_DEFAULT   , 0  , "default"   )

FUNCTION_VARIANT_FLAG(X86_64_SSE41     , 1  , "sse41"     )
FUNCTION_VARIANT_FLAG(X86_64_FMA       , 2  , "fma"       )
FUNCTION_VARIANT_FLAG(X86_64_AVX       , 3  , "avx"       )
FUNCTION_VARIANT_FLAG(X86_64_AVX2      , 4  , "avx2"      )
FUNCTION_VARIANT_FLAG(X86_64_AVX512F   , 5  , "avx512f"   )
FUNCTION_VARIANT_FLAG(X86_64_AVX512BW  , 6  , "avx512bw"  )
FUNCTION_VARIANT_FLAG(X86_64_BMI1      , 7  , "bmi1"      )
FUNCTION_VARIANT_FLAG(X86_64_ROSETTA   , 8  , "rosetta"   )
FUNCTION_VARIANT_FLAG(X86_64_HASWELL   , 9  , "haswell"   )
FUNCTION_VARIANT_FLAG(X86_64_IVYBRIDGE , 10 , "ivybridge" )
FUNCTION_VARIANT_FLAG(X86_64_NEHALEM   , 11 , "nehalem"   )
