
#ifndef ELF_RELOC
#error "ELF_RELOC must be defined"
#endif

ELF_RELOC(SH_NONE, 0)
ELF_RELOC(SH_DIR32, 1)
ELF_RELOC(SH_REL32, 2)
ELF_RELOC(SH_DIR8WPN, 3)
ELF_RELOC(SH_IND12W, 4)
ELF_RELOC(SH_DIR8WPL, 5)
ELF_RELOC(SH_DIR8WPZ, 6)
ELF_RELOC(SH_DIR8BP, 7)
ELF_RELOC(SH_DIR8W, 8)
ELF_RELOC(SH_DIR8L, 9)
ELF_RELOC(SH_LOOP_START, 10)
ELF_RELOC(SH_LOOP_END, 11)
ELF_RELOC(SH_GNU_VTINHERIT, 22)
ELF_RELOC(SH_GNU_VTENTRY, 23)
ELF_RELOC(SH_SWITCH8, 24)
ELF_RELOC(SH_SWITCH16, 25)
ELF_RELOC(SH_SWITCH32, 26)
ELF_RELOC(SH_USES, 27)
ELF_RELOC(SH_COUNT, 28)
ELF_RELOC(SH_ALIGN, 29)
ELF_RELOC(SH_CODE, 30)
ELF_RELOC(SH_DATA, 31)
ELF_RELOC(SH_LABEL, 32)
ELF_RELOC(SH_DIR16, 33)
ELF_RELOC(SH_DIR8, 34)
ELF_RELOC(SH_DIR8UL, 35)
ELF_RELOC(SH_DIR8UW, 36)
ELF_RELOC(SH_DIR8U, 37)
ELF_RELOC(SH_DIR8SW, 38)
ELF_RELOC(SH_DIR8S, 39)
ELF_RELOC(SH_DIR4UL, 40)
ELF_RELOC(SH_DIR4UW, 41)
ELF_RELOC(SH_DIR4U, 42)
ELF_RELOC(SH_PSHA, 43)
ELF_RELOC(SH_PSHL, 44)
ELF_RELOC(SH_DIR5U, 45)
ELF_RELOC(SH_DIR6U, 46)
ELF_RELOC(SH_DIR6S, 47)
ELF_RELOC(SH_DIR10S, 48)
ELF_RELOC(SH_DIR10SW, 49)
ELF_RELOC(SH_DIR10SL, 50)
ELF_RELOC(SH_DIR10SQ, 51)
ELF_RELOC(SH_DIR16S, 53)
ELF_RELOC(SH_TLS_GD_32, 144)
ELF_RELOC(SH_TLS_LD_32, 145)
ELF_RELOC(SH_TLS_LDO_32, 146)
ELF_RELOC(SH_TLS_IE_32, 147)
ELF_RELOC(SH_TLS_LE_32, 148)
ELF_RELOC(SH_TLS_DTPMOD32, 149)
ELF_RELOC(SH_TLS_DTPOFF32, 150)
ELF_RELOC(SH_TLS_TPOFF32, 151)
ELF_RELOC(SH_GOT32, 160)
ELF_RELOC(SH_PLT32, 161)
ELF_RELOC(SH_COPY, 162)
ELF_RELOC(SH_GLOB_DAT, 163)
ELF_RELOC(SH_JMP_SLOT, 164)
ELF_RELOC(SH_RELATIVE, 165)
ELF_RELOC(SH_GOTOFF, 166)
ELF_RELOC(SH_GOTPC, 167)
ELF_RELOC(SH_GOTPLT32, 168)
ELF_RELOC(SH_GOT_LOW16, 169)
ELF_RELOC(SH_GOT_MEDLOW16, 170)
ELF_RELOC(SH_GOT_MEDHI16, 171)
ELF_RELOC(SH_GOT_HI16, 172)
ELF_RELOC(SH_GOTPLT_LOW16, 173)
ELF_RELOC(SH_GOTPLT_MEDLOW16, 174)
ELF_RELOC(SH_GOTPLT_MEDHI16, 175)
ELF_RELOC(SH_GOTPLT_HI16, 176)
ELF_RELOC(SH_PLT_LOW16, 177)
ELF_RELOC(SH_PLT_MEDLOW16, 178)
ELF_RELOC(SH_PLT_MEDHI16, 179)
ELF_RELOC(SH_PLT_HI16, 180)
ELF_RELOC(SH_GOTOFF_LOW16, 181)
ELF_RELOC(SH_GOTOFF_MEDLOW16, 182)
ELF_RELOC(SH_GOTOFF_MEDHI16, 183)
ELF_RELOC(SH_GOTOFF_HI16, 184)
ELF_RELOC(SH_GOTPC_LOW16, 185)
ELF_RELOC(SH_GOTPC_MEDLOW16, 186)
ELF_RELOC(SH_GOTPC_MEDHI16, 187)
ELF_RELOC(SH_GOTPC_HI16, 188)
ELF_RELOC(SH_GOT10BY4, 189)
ELF_RELOC(SH_GOTPLT10BY4, 190)
ELF_RELOC(SH_GOT10BY8, 191)
ELF_RELOC(SH_GOTPLT10BY8, 192)
ELF_RELOC(SH_COPY64, 193)
ELF_RELOC(SH_GLOB_DAT64, 194)
ELF_RELOC(SH_JMP_SLOT64, 195)
ELF_RELOC(SH_RELATIVE64, 196)
ELF_RELOC(SH_GOT20, 201)
ELF_RELOC(SH_GOTOFF20, 202)
ELF_RELOC(SH_GOTFUNCDESC, 203)
ELF_RELOC(SH_GOTFUNCDESC20, 204)
ELF_RELOC(SH_GOTOFFFUNCDESC, 205)
ELF_RELOC(SH_GOTOFFFUNCDESC20, 206)
ELF_RELOC(SH_FUNCDESC, 207)
ELF_RELOC(SH_FUNCDESC_VALUE, 208)
ELF_RELOC(SH_SHMEDIA_CODE, 242)
ELF_RELOC(SH_PT_16, 243)
ELF_RELOC(SH_IMMS16, 244)
ELF_RELOC(SH_IMMU16, 245)
ELF_RELOC(SH_IMM_LOW16, 246)
ELF_RELOC(SH_IMM_LOW16_PCREL, 247)
ELF_RELOC(SH_IMM_MEDLOW16, 248)
ELF_RELOC(SH_IMM_MEDLOW16_PCREL, 249)
ELF_RELOC(SH_IMM_MEDHI16, 250)
ELF_RELOC(SH_IMM_MEDHI16_PCREL, 251)
ELF_RELOC(SH_IMM_HI16, 252)
ELF_RELOC(SH_IMM_HI16_PCREL, 253)
ELF_RELOC(SH_64, 254)
ELF_RELOC(SH_64_PCREL, 255)
