/* Copyright 2017 - 2025 R. Thomas
 * Copyright 2017 - 2025 Quarkslab
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef LIEF_PE_DEBUG_INC_H
#define LIEF_PE_DEBUG_INC_H
#include <LIEF/PE/debug/Debug.hpp>
#include <LIEF/PE/debug/CodeView.hpp>
#include <LIEF/PE/debug/Pogo.hpp>
#include <LIEF/PE/debug/PogoEntry.hpp>
#include <LIEF/PE/debug/Repro.hpp>
#include <LIEF/PE/debug/CodeViewPDB.hpp>
#include <LIEF/PE/debug/VCFeature.hpp>
#include <LIEF/PE/debug/ExDllCharacteristics.hpp>
#include <LIEF/PE/debug/FPO.hpp>
#endif
