/* Copyright 2022 - 2025 R. Thomas
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef LIEF_ASM_X86_OPCODE_H
#define LIEF_ASM_X86_OPCODE_H

/* Generated from LLVM: 21.1.0 */

namespace LIEF {
namespace assembly {
namespace x86 {
enum class OPCODE {
  PHI = 0,
  INLINEASM = 1,
  INLINEASM_BR = 2,
  CFI_INSTRUCTION = 3,
  EH_LABEL = 4,
  GC_LABEL = 5,
  ANNOTATION_LABEL = 6,
  KILL = 7,
  EXTRACT_SUBREG = 8,
  INSERT_SUBREG = 9,
  IMPLICIT_DEF = 10,
  INIT_UNDEF = 11,
  SUBREG_TO_REG = 12,
  COPY_TO_REGCLASS = 13,
  DBG_VALUE = 14,
  DBG_VALUE_LIST = 15,
  DBG_INSTR_REF = 16,
  DBG_PHI = 17,
  DBG_LABEL = 18,
  REG_SEQUENCE = 19,
  COPY = 20,
  BUNDLE = 21,
  LIFETIME_START = 22,
  LIFETIME_END = 23,
  PSEUDO_PROBE = 24,
  ARITH_FENCE = 25,
  STACKMAP = 26,
  FENTRY_CALL = 27,
  PATCHPOINT = 28,
  LOAD_STACK_GUARD = 29,
  PREALLOCATED_SETUP = 30,
  PREALLOCATED_ARG = 31,
  STATEPOINT = 32,
  LOCAL_ESCAPE = 33,
  FAULTING_OP = 34,
  PATCHABLE_OP = 35,
  PATCHABLE_FUNCTION_ENTER = 36,
  PATCHABLE_RET = 37,
  PATCHABLE_FUNCTION_EXIT = 38,
  PATCHABLE_TAIL_CALL = 39,
  PATCHABLE_EVENT_CALL = 40,
  PATCHABLE_TYPED_EVENT_CALL = 41,
  ICALL_BRANCH_FUNNEL = 42,
  FAKE_USE = 43,
  MEMBARRIER = 44,
  JUMP_TABLE_DEBUG_INFO = 45,
  CONVERGENCECTRL_ENTRY = 46,
  CONVERGENCECTRL_ANCHOR = 47,
  CONVERGENCECTRL_LOOP = 48,
  CONVERGENCECTRL_GLUE = 49,
  G_ASSERT_SEXT = 50,
  G_ASSERT_ZEXT = 51,
  G_ASSERT_ALIGN = 52,
  G_ADD = 53,
  G_SUB = 54,
  G_MUL = 55,
  G_SDIV = 56,
  G_UDIV = 57,
  G_SREM = 58,
  G_UREM = 59,
  G_SDIVREM = 60,
  G_UDIVREM = 61,
  G_AND = 62,
  G_OR = 63,
  G_XOR = 64,
  G_ABDS = 65,
  G_ABDU = 66,
  G_IMPLICIT_DEF = 67,
  G_PHI = 68,
  G_FRAME_INDEX = 69,
  G_GLOBAL_VALUE = 70,
  G_PTRAUTH_GLOBAL_VALUE = 71,
  G_CONSTANT_POOL = 72,
  G_EXTRACT = 73,
  G_UNMERGE_VALUES = 74,
  G_INSERT = 75,
  G_MERGE_VALUES = 76,
  G_BUILD_VECTOR = 77,
  G_BUILD_VECTOR_TRUNC = 78,
  G_CONCAT_VECTORS = 79,
  G_PTRTOINT = 80,
  G_INTTOPTR = 81,
  G_BITCAST = 82,
  G_FREEZE = 83,
  G_CONSTANT_FOLD_BARRIER = 84,
  G_INTRINSIC_FPTRUNC_ROUND = 85,
  G_INTRINSIC_TRUNC = 86,
  G_INTRINSIC_ROUND = 87,
  G_INTRINSIC_LRINT = 88,
  G_INTRINSIC_LLRINT = 89,
  G_INTRINSIC_ROUNDEVEN = 90,
  G_READCYCLECOUNTER = 91,
  G_READSTEADYCOUNTER = 92,
  G_LOAD = 93,
  G_SEXTLOAD = 94,
  G_ZEXTLOAD = 95,
  G_INDEXED_LOAD = 96,
  G_INDEXED_SEXTLOAD = 97,
  G_INDEXED_ZEXTLOAD = 98,
  G_STORE = 99,
  G_INDEXED_STORE = 100,
  G_ATOMIC_CMPXCHG_WITH_SUCCESS = 101,
  G_ATOMIC_CMPXCHG = 102,
  G_ATOMICRMW_XCHG = 103,
  G_ATOMICRMW_ADD = 104,
  G_ATOMICRMW_SUB = 105,
  G_ATOMICRMW_AND = 106,
  G_ATOMICRMW_NAND = 107,
  G_ATOMICRMW_OR = 108,
  G_ATOMICRMW_XOR = 109,
  G_ATOMICRMW_MAX = 110,
  G_ATOMICRMW_MIN = 111,
  G_ATOMICRMW_UMAX = 112,
  G_ATOMICRMW_UMIN = 113,
  G_ATOMICRMW_FADD = 114,
  G_ATOMICRMW_FSUB = 115,
  G_ATOMICRMW_FMAX = 116,
  G_ATOMICRMW_FMIN = 117,
  G_ATOMICRMW_FMAXIMUM = 118,
  G_ATOMICRMW_FMINIMUM = 119,
  G_ATOMICRMW_UINC_WRAP = 120,
  G_ATOMICRMW_UDEC_WRAP = 121,
  G_ATOMICRMW_USUB_COND = 122,
  G_ATOMICRMW_USUB_SAT = 123,
  G_FENCE = 124,
  G_PREFETCH = 125,
  G_BRCOND = 126,
  G_BRINDIRECT = 127,
  G_INVOKE_REGION_START = 128,
  G_INTRINSIC = 129,
  G_INTRINSIC_W_SIDE_EFFECTS = 130,
  G_INTRINSIC_CONVERGENT = 131,
  G_INTRINSIC_CONVERGENT_W_SIDE_EFFECTS = 132,
  G_ANYEXT = 133,
  G_TRUNC = 134,
  G_TRUNC_SSAT_S = 135,
  G_TRUNC_SSAT_U = 136,
  G_TRUNC_USAT_U = 137,
  G_CONSTANT = 138,
  G_FCONSTANT = 139,
  G_VASTART = 140,
  G_VAARG = 141,
  G_SEXT = 142,
  G_SEXT_INREG = 143,
  G_ZEXT = 144,
  G_SHL = 145,
  G_LSHR = 146,
  G_ASHR = 147,
  G_FSHL = 148,
  G_FSHR = 149,
  G_ROTR = 150,
  G_ROTL = 151,
  G_ICMP = 152,
  G_FCMP = 153,
  G_SCMP = 154,
  G_UCMP = 155,
  G_SELECT = 156,
  G_UADDO = 157,
  G_UADDE = 158,
  G_USUBO = 159,
  G_USUBE = 160,
  G_SADDO = 161,
  G_SADDE = 162,
  G_SSUBO = 163,
  G_SSUBE = 164,
  G_UMULO = 165,
  G_SMULO = 166,
  G_UMULH = 167,
  G_SMULH = 168,
  G_UADDSAT = 169,
  G_SADDSAT = 170,
  G_USUBSAT = 171,
  G_SSUBSAT = 172,
  G_USHLSAT = 173,
  G_SSHLSAT = 174,
  G_SMULFIX = 175,
  G_UMULFIX = 176,
  G_SMULFIXSAT = 177,
  G_UMULFIXSAT = 178,
  G_SDIVFIX = 179,
  G_UDIVFIX = 180,
  G_SDIVFIXSAT = 181,
  G_UDIVFIXSAT = 182,
  G_FADD = 183,
  G_FSUB = 184,
  G_FMUL = 185,
  G_FMA = 186,
  G_FMAD = 187,
  G_FDIV = 188,
  G_FREM = 189,
  G_FPOW = 190,
  G_FPOWI = 191,
  G_FEXP = 192,
  G_FEXP2 = 193,
  G_FEXP10 = 194,
  G_FLOG = 195,
  G_FLOG2 = 196,
  G_FLOG10 = 197,
  G_FLDEXP = 198,
  G_FFREXP = 199,
  G_FNEG = 200,
  G_FPEXT = 201,
  G_FPTRUNC = 202,
  G_FPTOSI = 203,
  G_FPTOUI = 204,
  G_SITOFP = 205,
  G_UITOFP = 206,
  G_FPTOSI_SAT = 207,
  G_FPTOUI_SAT = 208,
  G_FABS = 209,
  G_FCOPYSIGN = 210,
  G_IS_FPCLASS = 211,
  G_FCANONICALIZE = 212,
  G_FMINNUM = 213,
  G_FMAXNUM = 214,
  G_FMINNUM_IEEE = 215,
  G_FMAXNUM_IEEE = 216,
  G_FMINIMUM = 217,
  G_FMAXIMUM = 218,
  G_FMINIMUMNUM = 219,
  G_FMAXIMUMNUM = 220,
  G_GET_FPENV = 221,
  G_SET_FPENV = 222,
  G_RESET_FPENV = 223,
  G_GET_FPMODE = 224,
  G_SET_FPMODE = 225,
  G_RESET_FPMODE = 226,
  G_GET_ROUNDING = 227,
  G_PTR_ADD = 228,
  G_PTRMASK = 229,
  G_SMIN = 230,
  G_SMAX = 231,
  G_UMIN = 232,
  G_UMAX = 233,
  G_ABS = 234,
  G_LROUND = 235,
  G_LLROUND = 236,
  G_BR = 237,
  G_BRJT = 238,
  G_VSCALE = 239,
  G_INSERT_SUBVECTOR = 240,
  G_EXTRACT_SUBVECTOR = 241,
  G_INSERT_VECTOR_ELT = 242,
  G_EXTRACT_VECTOR_ELT = 243,
  G_SHUFFLE_VECTOR = 244,
  G_SPLAT_VECTOR = 245,
  G_STEP_VECTOR = 246,
  G_VECTOR_COMPRESS = 247,
  G_CTTZ = 248,
  G_CTTZ_ZERO_UNDEF = 249,
  G_CTLZ = 250,
  G_CTLZ_ZERO_UNDEF = 251,
  G_CTPOP = 252,
  G_BSWAP = 253,
  G_BITREVERSE = 254,
  G_FCEIL = 255,
  G_FCOS = 256,
  G_FSIN = 257,
  G_FSINCOS = 258,
  G_FTAN = 259,
  G_FACOS = 260,
  G_FASIN = 261,
  G_FATAN = 262,
  G_FATAN2 = 263,
  G_FCOSH = 264,
  G_FSINH = 265,
  G_FTANH = 266,
  G_FSQRT = 267,
  G_FFLOOR = 268,
  G_FRINT = 269,
  G_FNEARBYINT = 270,
  G_ADDRSPACE_CAST = 271,
  G_BLOCK_ADDR = 272,
  G_JUMP_TABLE = 273,
  G_DYN_STACKALLOC = 274,
  G_STACKSAVE = 275,
  G_STACKRESTORE = 276,
  G_STRICT_FADD = 277,
  G_STRICT_FSUB = 278,
  G_STRICT_FMUL = 279,
  G_STRICT_FDIV = 280,
  G_STRICT_FREM = 281,
  G_STRICT_FMA = 282,
  G_STRICT_FSQRT = 283,
  G_STRICT_FLDEXP = 284,
  G_READ_REGISTER = 285,
  G_WRITE_REGISTER = 286,
  G_MEMCPY = 287,
  G_MEMCPY_INLINE = 288,
  G_MEMMOVE = 289,
  G_MEMSET = 290,
  G_BZERO = 291,
  G_TRAP = 292,
  G_DEBUGTRAP = 293,
  G_UBSANTRAP = 294,
  G_VECREDUCE_SEQ_FADD = 295,
  G_VECREDUCE_SEQ_FMUL = 296,
  G_VECREDUCE_FADD = 297,
  G_VECREDUCE_FMUL = 298,
  G_VECREDUCE_FMAX = 299,
  G_VECREDUCE_FMIN = 300,
  G_VECREDUCE_FMAXIMUM = 301,
  G_VECREDUCE_FMINIMUM = 302,
  G_VECREDUCE_ADD = 303,
  G_VECREDUCE_MUL = 304,
  G_VECREDUCE_AND = 305,
  G_VECREDUCE_OR = 306,
  G_VECREDUCE_XOR = 307,
  G_VECREDUCE_SMAX = 308,
  G_VECREDUCE_SMIN = 309,
  G_VECREDUCE_UMAX = 310,
  G_VECREDUCE_UMIN = 311,
  G_SBFX = 312,
  G_UBFX = 313,
  ADD16ri_DB = 314,
  ADD16rr_DB = 315,
  ADD32ri_DB = 316,
  ADD32rr_DB = 317,
  ADD64ri32_DB = 318,
  ADD64rr_DB = 319,
  ADD8ri_DB = 320,
  ADD8rr_DB = 321,
  AVX1_SETALLONES = 322,
  AVX2_SETALLONES = 323,
  AVX512_128_SET0 = 324,
  AVX512_256_SET0 = 325,
  AVX512_512_SET0 = 326,
  AVX512_512_SETALLONES = 327,
  AVX512_512_SEXT_MASK_32 = 328,
  AVX512_512_SEXT_MASK_64 = 329,
  AVX512_FsFLD0F128 = 330,
  AVX512_FsFLD0SD = 331,
  AVX512_FsFLD0SH = 332,
  AVX512_FsFLD0SS = 333,
  AVX_SET0 = 334,
  CALL64m_RVMARKER = 335,
  CALL64pcrel32_RVMARKER = 336,
  CALL64r_ImpCall = 337,
  CALL64r_RVMARKER = 338,
  FsFLD0F128 = 339,
  FsFLD0SD = 340,
  FsFLD0SH = 341,
  FsFLD0SS = 342,
  G_FILD = 343,
  G_FIST = 344,
  G_FNSTCW16 = 345,
  INDIRECT_THUNK_CALL32 = 346,
  INDIRECT_THUNK_CALL64 = 347,
  INDIRECT_THUNK_TCRETURN32 = 348,
  INDIRECT_THUNK_TCRETURN64 = 349,
  KSET0D = 350,
  KSET0Q = 351,
  KSET0W = 352,
  KSET1D = 353,
  KSET1Q = 354,
  KSET1W = 355,
  LCMPXCHG16B_NO_RBX = 356,
  LCMPXCHG16B_SAVE_RBX = 357,
  MMX_SET0 = 358,
  MORESTACK_RET = 359,
  MORESTACK_RET_RESTORE_R10 = 360,
  MOV32ImmSExti8 = 361,
  MOV32r0 = 362,
  MOV32r1 = 363,
  MOV32r_1 = 364,
  MOV32ri64 = 365,
  MOV64ImmSExti8 = 366,
  MWAITX = 367,
  MWAITX_SAVE_RBX = 368,
  PLDTILECFGV = 369,
  PLEA32r = 370,
  PLEA64r = 371,
  PT2RPNTLVWZ0RST1V = 372,
  PT2RPNTLVWZ0RSV = 373,
  PT2RPNTLVWZ0T1V = 374,
  PT2RPNTLVWZ0V = 375,
  PT2RPNTLVWZ1RST1V = 376,
  PT2RPNTLVWZ1RSV = 377,
  PT2RPNTLVWZ1T1V = 378,
  PT2RPNTLVWZ1V = 379,
  PTDPBF16PSV = 380,
  PTDPBSSDV = 381,
  PTDPBSUDV = 382,
  PTDPBUSDV = 383,
  PTDPBUUDV = 384,
  PTDPFP16PSV = 385,
  PTILELOADDRST1V = 386,
  PTILELOADDRSV = 387,
  PTILELOADDT1V = 388,
  PTILELOADDV = 389,
  PTILEPAIRLOAD = 390,
  PTILEPAIRSTORE = 391,
  PTILESTOREDV = 392,
  PTILEZEROV = 393,
  RDFLAGS32 = 394,
  RDFLAGS64 = 395,
  SEH_BeginEpilogue = 396,
  SEH_EndEpilogue = 397,
  SEH_EndPrologue = 398,
  SEH_PushFrame = 399,
  SEH_PushReg = 400,
  SEH_SaveReg = 401,
  SEH_SaveXMM = 402,
  SEH_SetFrame = 403,
  SEH_StackAlign = 404,
  SEH_StackAlloc = 405,
  SEH_UnwindV2Start = 406,
  SEH_UnwindVersion = 407,
  SETB_C32r = 408,
  SETB_C64r = 409,
  SHLDROT32ri = 410,
  SHLDROT64ri = 411,
  SHRDROT32ri = 412,
  SHRDROT64ri = 413,
  VMOVAPSZ128mr_NOVLX = 414,
  VMOVAPSZ128rm_NOVLX = 415,
  VMOVAPSZ256mr_NOVLX = 416,
  VMOVAPSZ256rm_NOVLX = 417,
  VMOVUPSZ128mr_NOVLX = 418,
  VMOVUPSZ128rm_NOVLX = 419,
  VMOVUPSZ256mr_NOVLX = 420,
  VMOVUPSZ256rm_NOVLX = 421,
  V_SET0 = 422,
  V_SETALLONES = 423,
  WRFLAGS32 = 424,
  WRFLAGS64 = 425,
  XABORT_DEF = 426,
  XOR32_FP = 427,
  XOR64_FP = 428,
  AAA = 429,
  AAD8i8 = 430,
  AADD32mr = 431,
  AADD32mr_EVEX = 432,
  AADD64mr = 433,
  AADD64mr_EVEX = 434,
  AAM8i8 = 435,
  AAND32mr = 436,
  AAND32mr_EVEX = 437,
  AAND64mr = 438,
  AAND64mr_EVEX = 439,
  AAS = 440,
  ABS_F = 441,
  ABS_Fp32 = 442,
  ABS_Fp64 = 443,
  ABS_Fp80 = 444,
  ADC16i16 = 445,
  ADC16mi = 446,
  ADC16mi8 = 447,
  ADC16mi8_EVEX = 448,
  ADC16mi8_ND = 449,
  ADC16mi_EVEX = 450,
  ADC16mi_ND = 451,
  ADC16mr = 452,
  ADC16mr_EVEX = 453,
  ADC16mr_ND = 454,
  ADC16ri = 455,
  ADC16ri8 = 456,
  ADC16ri8_EVEX = 457,
  ADC16ri8_ND = 458,
  ADC16ri_EVEX = 459,
  ADC16ri_ND = 460,
  ADC16rm = 461,
  ADC16rm_EVEX = 462,
  ADC16rm_ND = 463,
  ADC16rr = 464,
  ADC16rr_EVEX = 465,
  ADC16rr_EVEX_REV = 466,
  ADC16rr_ND = 467,
  ADC16rr_ND_REV = 468,
  ADC16rr_REV = 469,
  ADC32i32 = 470,
  ADC32mi = 471,
  ADC32mi8 = 472,
  ADC32mi8_EVEX = 473,
  ADC32mi8_ND = 474,
  ADC32mi_EVEX = 475,
  ADC32mi_ND = 476,
  ADC32mr = 477,
  ADC32mr_EVEX = 478,
  ADC32mr_ND = 479,
  ADC32ri = 480,
  ADC32ri8 = 481,
  ADC32ri8_EVEX = 482,
  ADC32ri8_ND = 483,
  ADC32ri_EVEX = 484,
  ADC32ri_ND = 485,
  ADC32rm = 486,
  ADC32rm_EVEX = 487,
  ADC32rm_ND = 488,
  ADC32rr = 489,
  ADC32rr_EVEX = 490,
  ADC32rr_EVEX_REV = 491,
  ADC32rr_ND = 492,
  ADC32rr_ND_REV = 493,
  ADC32rr_REV = 494,
  ADC64i32 = 495,
  ADC64mi32 = 496,
  ADC64mi32_EVEX = 497,
  ADC64mi32_ND = 498,
  ADC64mi8 = 499,
  ADC64mi8_EVEX = 500,
  ADC64mi8_ND = 501,
  ADC64mr = 502,
  ADC64mr_EVEX = 503,
  ADC64mr_ND = 504,
  ADC64ri32 = 505,
  ADC64ri32_EVEX = 506,
  ADC64ri32_ND = 507,
  ADC64ri8 = 508,
  ADC64ri8_EVEX = 509,
  ADC64ri8_ND = 510,
  ADC64rm = 511,
  ADC64rm_EVEX = 512,
  ADC64rm_ND = 513,
  ADC64rr = 514,
  ADC64rr_EVEX = 515,
  ADC64rr_EVEX_REV = 516,
  ADC64rr_ND = 517,
  ADC64rr_ND_REV = 518,
  ADC64rr_REV = 519,
  ADC8i8 = 520,
  ADC8mi = 521,
  ADC8mi8 = 522,
  ADC8mi_EVEX = 523,
  ADC8mi_ND = 524,
  ADC8mr = 525,
  ADC8mr_EVEX = 526,
  ADC8mr_ND = 527,
  ADC8ri = 528,
  ADC8ri8 = 529,
  ADC8ri_EVEX = 530,
  ADC8ri_ND = 531,
  ADC8rm = 532,
  ADC8rm_EVEX = 533,
  ADC8rm_ND = 534,
  ADC8rr = 535,
  ADC8rr_EVEX = 536,
  ADC8rr_EVEX_REV = 537,
  ADC8rr_ND = 538,
  ADC8rr_ND_REV = 539,
  ADC8rr_REV = 540,
  ADCX32rm = 541,
  ADCX32rm_EVEX = 542,
  ADCX32rm_ND = 543,
  ADCX32rr = 544,
  ADCX32rr_EVEX = 545,
  ADCX32rr_ND = 546,
  ADCX64rm = 547,
  ADCX64rm_EVEX = 548,
  ADCX64rm_ND = 549,
  ADCX64rr = 550,
  ADCX64rr_EVEX = 551,
  ADCX64rr_ND = 552,
  ADD16i16 = 553,
  ADD16mi = 554,
  ADD16mi8 = 555,
  ADD16mi8_EVEX = 556,
  ADD16mi8_ND = 557,
  ADD16mi8_NF = 558,
  ADD16mi8_NF_ND = 559,
  ADD16mi_EVEX = 560,
  ADD16mi_ND = 561,
  ADD16mi_NF = 562,
  ADD16mi_NF_ND = 563,
  ADD16mr = 564,
  ADD16mr_EVEX = 565,
  ADD16mr_ND = 566,
  ADD16mr_NF = 567,
  ADD16mr_NF_ND = 568,
  ADD16ri = 569,
  ADD16ri8 = 570,
  ADD16ri8_EVEX = 571,
  ADD16ri8_ND = 572,
  ADD16ri8_NF = 573,
  ADD16ri8_NF_ND = 574,
  ADD16ri_EVEX = 575,
  ADD16ri_ND = 576,
  ADD16ri_NF = 577,
  ADD16ri_NF_ND = 578,
  ADD16rm = 579,
  ADD16rm_EVEX = 580,
  ADD16rm_ND = 581,
  ADD16rm_NF = 582,
  ADD16rm_NF_ND = 583,
  ADD16rr = 584,
  ADD16rr_EVEX = 585,
  ADD16rr_EVEX_REV = 586,
  ADD16rr_ND = 587,
  ADD16rr_ND_REV = 588,
  ADD16rr_NF = 589,
  ADD16rr_NF_ND = 590,
  ADD16rr_NF_ND_REV = 591,
  ADD16rr_NF_REV = 592,
  ADD16rr_REV = 593,
  ADD32i32 = 594,
  ADD32mi = 595,
  ADD32mi8 = 596,
  ADD32mi8_EVEX = 597,
  ADD32mi8_ND = 598,
  ADD32mi8_NF = 599,
  ADD32mi8_NF_ND = 600,
  ADD32mi_EVEX = 601,
  ADD32mi_ND = 602,
  ADD32mi_NF = 603,
  ADD32mi_NF_ND = 604,
  ADD32mr = 605,
  ADD32mr_EVEX = 606,
  ADD32mr_ND = 607,
  ADD32mr_NF = 608,
  ADD32mr_NF_ND = 609,
  ADD32ri = 610,
  ADD32ri8 = 611,
  ADD32ri8_EVEX = 612,
  ADD32ri8_ND = 613,
  ADD32ri8_NF = 614,
  ADD32ri8_NF_ND = 615,
  ADD32ri_EVEX = 616,
  ADD32ri_ND = 617,
  ADD32ri_NF = 618,
  ADD32ri_NF_ND = 619,
  ADD32rm = 620,
  ADD32rm_EVEX = 621,
  ADD32rm_ND = 622,
  ADD32rm_NF = 623,
  ADD32rm_NF_ND = 624,
  ADD32rr = 625,
  ADD32rr_EVEX = 626,
  ADD32rr_EVEX_REV = 627,
  ADD32rr_ND = 628,
  ADD32rr_ND_REV = 629,
  ADD32rr_NF = 630,
  ADD32rr_NF_ND = 631,
  ADD32rr_NF_ND_REV = 632,
  ADD32rr_NF_REV = 633,
  ADD32rr_REV = 634,
  ADD64i32 = 635,
  ADD64mi32 = 636,
  ADD64mi32_EVEX = 637,
  ADD64mi32_ND = 638,
  ADD64mi32_NF = 639,
  ADD64mi32_NF_ND = 640,
  ADD64mi8 = 641,
  ADD64mi8_EVEX = 642,
  ADD64mi8_ND = 643,
  ADD64mi8_NF = 644,
  ADD64mi8_NF_ND = 645,
  ADD64mr = 646,
  ADD64mr_EVEX = 647,
  ADD64mr_ND = 648,
  ADD64mr_NF = 649,
  ADD64mr_NF_ND = 650,
  ADD64ri32 = 651,
  ADD64ri32_EVEX = 652,
  ADD64ri32_ND = 653,
  ADD64ri32_NF = 654,
  ADD64ri32_NF_ND = 655,
  ADD64ri8 = 656,
  ADD64ri8_EVEX = 657,
  ADD64ri8_ND = 658,
  ADD64ri8_NF = 659,
  ADD64ri8_NF_ND = 660,
  ADD64rm = 661,
  ADD64rm_EVEX = 662,
  ADD64rm_ND = 663,
  ADD64rm_NF = 664,
  ADD64rm_NF_ND = 665,
  ADD64rr = 666,
  ADD64rr_EVEX = 667,
  ADD64rr_EVEX_REV = 668,
  ADD64rr_ND = 669,
  ADD64rr_ND_REV = 670,
  ADD64rr_NF = 671,
  ADD64rr_NF_ND = 672,
  ADD64rr_NF_ND_REV = 673,
  ADD64rr_NF_REV = 674,
  ADD64rr_REV = 675,
  ADD8i8 = 676,
  ADD8mi = 677,
  ADD8mi8 = 678,
  ADD8mi_EVEX = 679,
  ADD8mi_ND = 680,
  ADD8mi_NF = 681,
  ADD8mi_NF_ND = 682,
  ADD8mr = 683,
  ADD8mr_EVEX = 684,
  ADD8mr_ND = 685,
  ADD8mr_NF = 686,
  ADD8mr_NF_ND = 687,
  ADD8ri = 688,
  ADD8ri8 = 689,
  ADD8ri_EVEX = 690,
  ADD8ri_ND = 691,
  ADD8ri_NF = 692,
  ADD8ri_NF_ND = 693,
  ADD8rm = 694,
  ADD8rm_EVEX = 695,
  ADD8rm_ND = 696,
  ADD8rm_NF = 697,
  ADD8rm_NF_ND = 698,
  ADD8rr = 699,
  ADD8rr_EVEX = 700,
  ADD8rr_EVEX_REV = 701,
  ADD8rr_ND = 702,
  ADD8rr_ND_REV = 703,
  ADD8rr_NF = 704,
  ADD8rr_NF_ND = 705,
  ADD8rr_NF_ND_REV = 706,
  ADD8rr_NF_REV = 707,
  ADD8rr_REV = 708,
  ADDPDrm = 709,
  ADDPDrr = 710,
  ADDPSrm = 711,
  ADDPSrr = 712,
  ADDR16_PREFIX = 713,
  ADDR32_PREFIX = 714,
  ADDSDrm = 715,
  ADDSDrm_Int = 716,
  ADDSDrr = 717,
  ADDSDrr_Int = 718,
  ADDSSrm = 719,
  ADDSSrm_Int = 720,
  ADDSSrr = 721,
  ADDSSrr_Int = 722,
  ADDSUBPDrm = 723,
  ADDSUBPDrr = 724,
  ADDSUBPSrm = 725,
  ADDSUBPSrr = 726,
  ADD_F32m = 727,
  ADD_F64m = 728,
  ADD_FI16m = 729,
  ADD_FI32m = 730,
  ADD_FPrST0 = 731,
  ADD_FST0r = 732,
  ADD_Fp32 = 733,
  ADD_Fp32m = 734,
  ADD_Fp64 = 735,
  ADD_Fp64m = 736,
  ADD_Fp64m32 = 737,
  ADD_Fp80 = 738,
  ADD_Fp80m32 = 739,
  ADD_Fp80m64 = 740,
  ADD_FpI16m32 = 741,
  ADD_FpI16m64 = 742,
  ADD_FpI16m80 = 743,
  ADD_FpI32m32 = 744,
  ADD_FpI32m64 = 745,
  ADD_FpI32m80 = 746,
  ADD_FrST0 = 747,
  ADJCALLSTACKDOWN32 = 748,
  ADJCALLSTACKDOWN64 = 749,
  ADJCALLSTACKUP32 = 750,
  ADJCALLSTACKUP64 = 751,
  ADOX32rm = 752,
  ADOX32rm_EVEX = 753,
  ADOX32rm_ND = 754,
  ADOX32rr = 755,
  ADOX32rr_EVEX = 756,
  ADOX32rr_ND = 757,
  ADOX64rm = 758,
  ADOX64rm_EVEX = 759,
  ADOX64rm_ND = 760,
  ADOX64rr = 761,
  ADOX64rr_EVEX = 762,
  ADOX64rr_ND = 763,
  AESDEC128KL = 764,
  AESDEC256KL = 765,
  AESDECLASTrm = 766,
  AESDECLASTrr = 767,
  AESDECWIDE128KL = 768,
  AESDECWIDE256KL = 769,
  AESDECrm = 770,
  AESDECrr = 771,
  AESENC128KL = 772,
  AESENC256KL = 773,
  AESENCLASTrm = 774,
  AESENCLASTrr = 775,
  AESENCWIDE128KL = 776,
  AESENCWIDE256KL = 777,
  AESENCrm = 778,
  AESENCrr = 779,
  AESIMCrm = 780,
  AESIMCrr = 781,
  AESKEYGENASSIST128rm = 782,
  AESKEYGENASSIST128rr = 783,
  AND16i16 = 784,
  AND16mi = 785,
  AND16mi8 = 786,
  AND16mi8_EVEX = 787,
  AND16mi8_ND = 788,
  AND16mi8_NF = 789,
  AND16mi8_NF_ND = 790,
  AND16mi_EVEX = 791,
  AND16mi_ND = 792,
  AND16mi_NF = 793,
  AND16mi_NF_ND = 794,
  AND16mr = 795,
  AND16mr_EVEX = 796,
  AND16mr_ND = 797,
  AND16mr_NF = 798,
  AND16mr_NF_ND = 799,
  AND16ri = 800,
  AND16ri8 = 801,
  AND16ri8_EVEX = 802,
  AND16ri8_ND = 803,
  AND16ri8_NF = 804,
  AND16ri8_NF_ND = 805,
  AND16ri_EVEX = 806,
  AND16ri_ND = 807,
  AND16ri_NF = 808,
  AND16ri_NF_ND = 809,
  AND16rm = 810,
  AND16rm_EVEX = 811,
  AND16rm_ND = 812,
  AND16rm_NF = 813,
  AND16rm_NF_ND = 814,
  AND16rr = 815,
  AND16rr_EVEX = 816,
  AND16rr_EVEX_REV = 817,
  AND16rr_ND = 818,
  AND16rr_ND_REV = 819,
  AND16rr_NF = 820,
  AND16rr_NF_ND = 821,
  AND16rr_NF_ND_REV = 822,
  AND16rr_NF_REV = 823,
  AND16rr_REV = 824,
  AND32i32 = 825,
  AND32mi = 826,
  AND32mi8 = 827,
  AND32mi8_EVEX = 828,
  AND32mi8_ND = 829,
  AND32mi8_NF = 830,
  AND32mi8_NF_ND = 831,
  AND32mi_EVEX = 832,
  AND32mi_ND = 833,
  AND32mi_NF = 834,
  AND32mi_NF_ND = 835,
  AND32mr = 836,
  AND32mr_EVEX = 837,
  AND32mr_ND = 838,
  AND32mr_NF = 839,
  AND32mr_NF_ND = 840,
  AND32ri = 841,
  AND32ri8 = 842,
  AND32ri8_EVEX = 843,
  AND32ri8_ND = 844,
  AND32ri8_NF = 845,
  AND32ri8_NF_ND = 846,
  AND32ri_EVEX = 847,
  AND32ri_ND = 848,
  AND32ri_NF = 849,
  AND32ri_NF_ND = 850,
  AND32rm = 851,
  AND32rm_EVEX = 852,
  AND32rm_ND = 853,
  AND32rm_NF = 854,
  AND32rm_NF_ND = 855,
  AND32rr = 856,
  AND32rr_EVEX = 857,
  AND32rr_EVEX_REV = 858,
  AND32rr_ND = 859,
  AND32rr_ND_REV = 860,
  AND32rr_NF = 861,
  AND32rr_NF_ND = 862,
  AND32rr_NF_ND_REV = 863,
  AND32rr_NF_REV = 864,
  AND32rr_REV = 865,
  AND64i32 = 866,
  AND64mi32 = 867,
  AND64mi32_EVEX = 868,
  AND64mi32_ND = 869,
  AND64mi32_NF = 870,
  AND64mi32_NF_ND = 871,
  AND64mi8 = 872,
  AND64mi8_EVEX = 873,
  AND64mi8_ND = 874,
  AND64mi8_NF = 875,
  AND64mi8_NF_ND = 876,
  AND64mr = 877,
  AND64mr_EVEX = 878,
  AND64mr_ND = 879,
  AND64mr_NF = 880,
  AND64mr_NF_ND = 881,
  AND64ri32 = 882,
  AND64ri32_EVEX = 883,
  AND64ri32_ND = 884,
  AND64ri32_NF = 885,
  AND64ri32_NF_ND = 886,
  AND64ri8 = 887,
  AND64ri8_EVEX = 888,
  AND64ri8_ND = 889,
  AND64ri8_NF = 890,
  AND64ri8_NF_ND = 891,
  AND64rm = 892,
  AND64rm_EVEX = 893,
  AND64rm_ND = 894,
  AND64rm_NF = 895,
  AND64rm_NF_ND = 896,
  AND64rr = 897,
  AND64rr_EVEX = 898,
  AND64rr_EVEX_REV = 899,
  AND64rr_ND = 900,
  AND64rr_ND_REV = 901,
  AND64rr_NF = 902,
  AND64rr_NF_ND = 903,
  AND64rr_NF_ND_REV = 904,
  AND64rr_NF_REV = 905,
  AND64rr_REV = 906,
  AND8i8 = 907,
  AND8mi = 908,
  AND8mi8 = 909,
  AND8mi_EVEX = 910,
  AND8mi_ND = 911,
  AND8mi_NF = 912,
  AND8mi_NF_ND = 913,
  AND8mr = 914,
  AND8mr_EVEX = 915,
  AND8mr_ND = 916,
  AND8mr_NF = 917,
  AND8mr_NF_ND = 918,
  AND8ri = 919,
  AND8ri8 = 920,
  AND8ri_EVEX = 921,
  AND8ri_ND = 922,
  AND8ri_NF = 923,
  AND8ri_NF_ND = 924,
  AND8rm = 925,
  AND8rm_EVEX = 926,
  AND8rm_ND = 927,
  AND8rm_NF = 928,
  AND8rm_NF_ND = 929,
  AND8rr = 930,
  AND8rr_EVEX = 931,
  AND8rr_EVEX_REV = 932,
  AND8rr_ND = 933,
  AND8rr_ND_REV = 934,
  AND8rr_NF = 935,
  AND8rr_NF_ND = 936,
  AND8rr_NF_ND_REV = 937,
  AND8rr_NF_REV = 938,
  AND8rr_REV = 939,
  ANDN32rm = 940,
  ANDN32rm_EVEX = 941,
  ANDN32rm_NF = 942,
  ANDN32rr = 943,
  ANDN32rr_EVEX = 944,
  ANDN32rr_NF = 945,
  ANDN64rm = 946,
  ANDN64rm_EVEX = 947,
  ANDN64rm_NF = 948,
  ANDN64rr = 949,
  ANDN64rr_EVEX = 950,
  ANDN64rr_NF = 951,
  ANDNPDrm = 952,
  ANDNPDrr = 953,
  ANDNPSrm = 954,
  ANDNPSrr = 955,
  ANDPDrm = 956,
  ANDPDrr = 957,
  ANDPSrm = 958,
  ANDPSrr = 959,
  AOR32mr = 960,
  AOR32mr_EVEX = 961,
  AOR64mr = 962,
  AOR64mr_EVEX = 963,
  ARPL16mr = 964,
  ARPL16rr = 965,
  ASAN_CHECK_MEMACCESS = 966,
  AXOR32mr = 967,
  AXOR32mr_EVEX = 968,
  AXOR64mr = 969,
  AXOR64mr_EVEX = 970,
  BEXTR32rm = 971,
  BEXTR32rm_EVEX = 972,
  BEXTR32rm_NF = 973,
  BEXTR32rr = 974,
  BEXTR32rr_EVEX = 975,
  BEXTR32rr_NF = 976,
  BEXTR64rm = 977,
  BEXTR64rm_EVEX = 978,
  BEXTR64rm_NF = 979,
  BEXTR64rr = 980,
  BEXTR64rr_EVEX = 981,
  BEXTR64rr_NF = 982,
  BEXTRI32mi = 983,
  BEXTRI32ri = 984,
  BEXTRI64mi = 985,
  BEXTRI64ri = 986,
  BLCFILL32rm = 987,
  BLCFILL32rr = 988,
  BLCFILL64rm = 989,
  BLCFILL64rr = 990,
  BLCI32rm = 991,
  BLCI32rr = 992,
  BLCI64rm = 993,
  BLCI64rr = 994,
  BLCIC32rm = 995,
  BLCIC32rr = 996,
  BLCIC64rm = 997,
  BLCIC64rr = 998,
  BLCMSK32rm = 999,
  BLCMSK32rr = 1000,
  BLCMSK64rm = 1001,
  BLCMSK64rr = 1002,
  BLCS32rm = 1003,
  BLCS32rr = 1004,
  BLCS64rm = 1005,
  BLCS64rr = 1006,
  BLENDPDrmi = 1007,
  BLENDPDrri = 1008,
  BLENDPSrmi = 1009,
  BLENDPSrri = 1010,
  BLENDVPDrm0 = 1011,
  BLENDVPDrr0 = 1012,
  BLENDVPSrm0 = 1013,
  BLENDVPSrr0 = 1014,
  BLSFILL32rm = 1015,
  BLSFILL32rr = 1016,
  BLSFILL64rm = 1017,
  BLSFILL64rr = 1018,
  BLSI32rm = 1019,
  BLSI32rm_EVEX = 1020,
  BLSI32rm_NF = 1021,
  BLSI32rr = 1022,
  BLSI32rr_EVEX = 1023,
  BLSI32rr_NF = 1024,
  BLSI64rm = 1025,
  BLSI64rm_EVEX = 1026,
  BLSI64rm_NF = 1027,
  BLSI64rr = 1028,
  BLSI64rr_EVEX = 1029,
  BLSI64rr_NF = 1030,
  BLSIC32rm = 1031,
  BLSIC32rr = 1032,
  BLSIC64rm = 1033,
  BLSIC64rr = 1034,
  BLSMSK32rm = 1035,
  BLSMSK32rm_EVEX = 1036,
  BLSMSK32rm_NF = 1037,
  BLSMSK32rr = 1038,
  BLSMSK32rr_EVEX = 1039,
  BLSMSK32rr_NF = 1040,
  BLSMSK64rm = 1041,
  BLSMSK64rm_EVEX = 1042,
  BLSMSK64rm_NF = 1043,
  BLSMSK64rr = 1044,
  BLSMSK64rr_EVEX = 1045,
  BLSMSK64rr_NF = 1046,
  BLSR32rm = 1047,
  BLSR32rm_EVEX = 1048,
  BLSR32rm_NF = 1049,
  BLSR32rr = 1050,
  BLSR32rr_EVEX = 1051,
  BLSR32rr_NF = 1052,
  BLSR64rm = 1053,
  BLSR64rm_EVEX = 1054,
  BLSR64rm_NF = 1055,
  BLSR64rr = 1056,
  BLSR64rr_EVEX = 1057,
  BLSR64rr_NF = 1058,
  BOUNDS16rm = 1059,
  BOUNDS32rm = 1060,
  BSF16rm = 1061,
  BSF16rr = 1062,
  BSF32rm = 1063,
  BSF32rr = 1064,
  BSF64rm = 1065,
  BSF64rr = 1066,
  BSR16rm = 1067,
  BSR16rr = 1068,
  BSR32rm = 1069,
  BSR32rr = 1070,
  BSR64rm = 1071,
  BSR64rr = 1072,
  BSWAP16r_BAD = 1073,
  BSWAP32r = 1074,
  BSWAP64r = 1075,
  BT16mi8 = 1076,
  BT16mr = 1077,
  BT16ri8 = 1078,
  BT16rr = 1079,
  BT32mi8 = 1080,
  BT32mr = 1081,
  BT32ri8 = 1082,
  BT32rr = 1083,
  BT64mi8 = 1084,
  BT64mr = 1085,
  BT64ri8 = 1086,
  BT64rr = 1087,
  BTC16mi8 = 1088,
  BTC16mr = 1089,
  BTC16ri8 = 1090,
  BTC16rr = 1091,
  BTC32mi8 = 1092,
  BTC32mr = 1093,
  BTC32ri8 = 1094,
  BTC32rr = 1095,
  BTC64mi8 = 1096,
  BTC64mr = 1097,
  BTC64ri8 = 1098,
  BTC64rr = 1099,
  BTR16mi8 = 1100,
  BTR16mr = 1101,
  BTR16ri8 = 1102,
  BTR16rr = 1103,
  BTR32mi8 = 1104,
  BTR32mr = 1105,
  BTR32ri8 = 1106,
  BTR32rr = 1107,
  BTR64mi8 = 1108,
  BTR64mr = 1109,
  BTR64ri8 = 1110,
  BTR64rr = 1111,
  BTS16mi8 = 1112,
  BTS16mr = 1113,
  BTS16ri8 = 1114,
  BTS16rr = 1115,
  BTS32mi8 = 1116,
  BTS32mr = 1117,
  BTS32ri8 = 1118,
  BTS32rr = 1119,
  BTS64mi8 = 1120,
  BTS64mr = 1121,
  BTS64ri8 = 1122,
  BTS64rr = 1123,
  BZHI32rm = 1124,
  BZHI32rm_EVEX = 1125,
  BZHI32rm_NF = 1126,
  BZHI32rr = 1127,
  BZHI32rr_EVEX = 1128,
  BZHI32rr_NF = 1129,
  BZHI64rm = 1130,
  BZHI64rm_EVEX = 1131,
  BZHI64rm_NF = 1132,
  BZHI64rr = 1133,
  BZHI64rr_EVEX = 1134,
  BZHI64rr_NF = 1135,
  CALL16m = 1136,
  CALL16m_NT = 1137,
  CALL16r = 1138,
  CALL16r_NT = 1139,
  CALL32m = 1140,
  CALL32m_NT = 1141,
  CALL32r = 1142,
  CALL32r_NT = 1143,
  CALL64m = 1144,
  CALL64m_NT = 1145,
  CALL64pcrel32 = 1146,
  CALL64r = 1147,
  CALL64r_NT = 1148,
  CALLpcrel16 = 1149,
  CALLpcrel32 = 1150,
  CATCHRET = 1151,
  CBW = 1152,
  CCMP16mi = 1153,
  CCMP16mi8 = 1154,
  CCMP16mr = 1155,
  CCMP16ri = 1156,
  CCMP16ri8 = 1157,
  CCMP16rm = 1158,
  CCMP16rr = 1159,
  CCMP16rr_REV = 1160,
  CCMP32mi = 1161,
  CCMP32mi8 = 1162,
  CCMP32mr = 1163,
  CCMP32ri = 1164,
  CCMP32ri8 = 1165,
  CCMP32rm = 1166,
  CCMP32rr = 1167,
  CCMP32rr_REV = 1168,
  CCMP64mi32 = 1169,
  CCMP64mi8 = 1170,
  CCMP64mr = 1171,
  CCMP64ri32 = 1172,
  CCMP64ri8 = 1173,
  CCMP64rm = 1174,
  CCMP64rr = 1175,
  CCMP64rr_REV = 1176,
  CCMP8mi = 1177,
  CCMP8mr = 1178,
  CCMP8ri = 1179,
  CCMP8rm = 1180,
  CCMP8rr = 1181,
  CCMP8rr_REV = 1182,
  CDQ = 1183,
  CDQE = 1184,
  CFCMOV16mr = 1185,
  CFCMOV16rm = 1186,
  CFCMOV16rm_ND = 1187,
  CFCMOV16rr = 1188,
  CFCMOV16rr_ND = 1189,
  CFCMOV16rr_REV = 1190,
  CFCMOV32mr = 1191,
  CFCMOV32rm = 1192,
  CFCMOV32rm_ND = 1193,
  CFCMOV32rr = 1194,
  CFCMOV32rr_ND = 1195,
  CFCMOV32rr_REV = 1196,
  CFCMOV64mr = 1197,
  CFCMOV64rm = 1198,
  CFCMOV64rm_ND = 1199,
  CFCMOV64rr = 1200,
  CFCMOV64rr_ND = 1201,
  CFCMOV64rr_REV = 1202,
  CHS_F = 1203,
  CHS_Fp32 = 1204,
  CHS_Fp64 = 1205,
  CHS_Fp80 = 1206,
  CLAC = 1207,
  CLC = 1208,
  CLD = 1209,
  CLDEMOTE = 1210,
  CLEANUPRET = 1211,
  CLFLUSH = 1212,
  CLFLUSHOPT = 1213,
  CLGI = 1214,
  CLI = 1215,
  CLRSSBSY = 1216,
  CLTS = 1217,
  CLUI = 1218,
  CLWB = 1219,
  CLZERO32r = 1220,
  CLZERO64r = 1221,
  CMC = 1222,
  CMOV16rm = 1223,
  CMOV16rm_ND = 1224,
  CMOV16rr = 1225,
  CMOV16rr_ND = 1226,
  CMOV32rm = 1227,
  CMOV32rm_ND = 1228,
  CMOV32rr = 1229,
  CMOV32rr_ND = 1230,
  CMOV64rm = 1231,
  CMOV64rm_ND = 1232,
  CMOV64rr = 1233,
  CMOV64rr_ND = 1234,
  CMOVBE_F = 1235,
  CMOVBE_Fp32 = 1236,
  CMOVBE_Fp64 = 1237,
  CMOVBE_Fp80 = 1238,
  CMOVB_F = 1239,
  CMOVB_Fp32 = 1240,
  CMOVB_Fp64 = 1241,
  CMOVB_Fp80 = 1242,
  CMOVE_F = 1243,
  CMOVE_Fp32 = 1244,
  CMOVE_Fp64 = 1245,
  CMOVE_Fp80 = 1246,
  CMOVNBE_F = 1247,
  CMOVNBE_Fp32 = 1248,
  CMOVNBE_Fp64 = 1249,
  CMOVNBE_Fp80 = 1250,
  CMOVNB_F = 1251,
  CMOVNB_Fp32 = 1252,
  CMOVNB_Fp64 = 1253,
  CMOVNB_Fp80 = 1254,
  CMOVNE_F = 1255,
  CMOVNE_Fp32 = 1256,
  CMOVNE_Fp64 = 1257,
  CMOVNE_Fp80 = 1258,
  CMOVNP_F = 1259,
  CMOVNP_Fp32 = 1260,
  CMOVNP_Fp64 = 1261,
  CMOVNP_Fp80 = 1262,
  CMOVP_F = 1263,
  CMOVP_Fp32 = 1264,
  CMOVP_Fp64 = 1265,
  CMOVP_Fp80 = 1266,
  CMOV_FR16 = 1267,
  CMOV_FR16X = 1268,
  CMOV_FR32 = 1269,
  CMOV_FR32X = 1270,
  CMOV_FR64 = 1271,
  CMOV_FR64X = 1272,
  CMOV_GR16 = 1273,
  CMOV_GR32 = 1274,
  CMOV_GR8 = 1275,
  CMOV_RFP32 = 1276,
  CMOV_RFP64 = 1277,
  CMOV_RFP80 = 1278,
  CMOV_VK1 = 1279,
  CMOV_VK16 = 1280,
  CMOV_VK2 = 1281,
  CMOV_VK32 = 1282,
  CMOV_VK4 = 1283,
  CMOV_VK64 = 1284,
  CMOV_VK8 = 1285,
  CMOV_VR128 = 1286,
  CMOV_VR128X = 1287,
  CMOV_VR256 = 1288,
  CMOV_VR256X = 1289,
  CMOV_VR512 = 1290,
  CMOV_VR64 = 1291,
  CMP16i16 = 1292,
  CMP16mi = 1293,
  CMP16mi8 = 1294,
  CMP16mr = 1295,
  CMP16ri = 1296,
  CMP16ri8 = 1297,
  CMP16rm = 1298,
  CMP16rr = 1299,
  CMP16rr_REV = 1300,
  CMP32i32 = 1301,
  CMP32mi = 1302,
  CMP32mi8 = 1303,
  CMP32mr = 1304,
  CMP32ri = 1305,
  CMP32ri8 = 1306,
  CMP32rm = 1307,
  CMP32rr = 1308,
  CMP32rr_REV = 1309,
  CMP64i32 = 1310,
  CMP64mi32 = 1311,
  CMP64mi8 = 1312,
  CMP64mr = 1313,
  CMP64ri32 = 1314,
  CMP64ri8 = 1315,
  CMP64rm = 1316,
  CMP64rr = 1317,
  CMP64rr_REV = 1318,
  CMP8i8 = 1319,
  CMP8mi = 1320,
  CMP8mi8 = 1321,
  CMP8mr = 1322,
  CMP8ri = 1323,
  CMP8ri8 = 1324,
  CMP8rm = 1325,
  CMP8rr = 1326,
  CMP8rr_REV = 1327,
  CMPCCXADDmr32 = 1328,
  CMPCCXADDmr32_EVEX = 1329,
  CMPCCXADDmr64 = 1330,
  CMPCCXADDmr64_EVEX = 1331,
  CMPPDrmi = 1332,
  CMPPDrri = 1333,
  CMPPSrmi = 1334,
  CMPPSrri = 1335,
  CMPSB = 1336,
  CMPSDrmi = 1337,
  CMPSDrmi_Int = 1338,
  CMPSDrri = 1339,
  CMPSDrri_Int = 1340,
  CMPSL = 1341,
  CMPSQ = 1342,
  CMPSSrmi = 1343,
  CMPSSrmi_Int = 1344,
  CMPSSrri = 1345,
  CMPSSrri_Int = 1346,
  CMPSW = 1347,
  CMPXCHG16B = 1348,
  CMPXCHG16rm = 1349,
  CMPXCHG16rr = 1350,
  CMPXCHG32rm = 1351,
  CMPXCHG32rr = 1352,
  CMPXCHG64rm = 1353,
  CMPXCHG64rr = 1354,
  CMPXCHG8B = 1355,
  CMPXCHG8rm = 1356,
  CMPXCHG8rr = 1357,
  COMISDrm = 1358,
  COMISDrm_Int = 1359,
  COMISDrr = 1360,
  COMISDrr_Int = 1361,
  COMISSrm = 1362,
  COMISSrm_Int = 1363,
  COMISSrr = 1364,
  COMISSrr_Int = 1365,
  COMP_FST0r = 1366,
  COM_FIPr = 1367,
  COM_FIr = 1368,
  COM_FST0r = 1369,
  COM_FpIr32 = 1370,
  COM_FpIr64 = 1371,
  COM_FpIr80 = 1372,
  COM_Fpr32 = 1373,
  COM_Fpr64 = 1374,
  COM_Fpr80 = 1375,
  CPUID = 1376,
  CQO = 1377,
  CRC32r32m16 = 1378,
  CRC32r32m16_EVEX = 1379,
  CRC32r32m32 = 1380,
  CRC32r32m32_EVEX = 1381,
  CRC32r32m8 = 1382,
  CRC32r32m8_EVEX = 1383,
  CRC32r32r16 = 1384,
  CRC32r32r16_EVEX = 1385,
  CRC32r32r32 = 1386,
  CRC32r32r32_EVEX = 1387,
  CRC32r32r8 = 1388,
  CRC32r32r8_EVEX = 1389,
  CRC32r64m64 = 1390,
  CRC32r64m64_EVEX = 1391,
  CRC32r64m8 = 1392,
  CRC32r64m8_EVEX = 1393,
  CRC32r64r64 = 1394,
  CRC32r64r64_EVEX = 1395,
  CRC32r64r8 = 1396,
  CRC32r64r8_EVEX = 1397,
  CS_PREFIX = 1398,
  CTEST16mi = 1399,
  CTEST16mr = 1400,
  CTEST16ri = 1401,
  CTEST16rr = 1402,
  CTEST32mi = 1403,
  CTEST32mr = 1404,
  CTEST32ri = 1405,
  CTEST32rr = 1406,
  CTEST64mi32 = 1407,
  CTEST64mr = 1408,
  CTEST64ri32 = 1409,
  CTEST64rr = 1410,
  CTEST8mi = 1411,
  CTEST8mr = 1412,
  CTEST8ri = 1413,
  CTEST8rr = 1414,
  CVTDQ2PDrm = 1415,
  CVTDQ2PDrr = 1416,
  CVTDQ2PSrm = 1417,
  CVTDQ2PSrr = 1418,
  CVTPD2DQrm = 1419,
  CVTPD2DQrr = 1420,
  CVTPD2PSrm = 1421,
  CVTPD2PSrr = 1422,
  CVTPS2DQrm = 1423,
  CVTPS2DQrr = 1424,
  CVTPS2PDrm = 1425,
  CVTPS2PDrr = 1426,
  CVTSD2SI64rm = 1427,
  CVTSD2SI64rm_Int = 1428,
  CVTSD2SI64rr = 1429,
  CVTSD2SI64rr_Int = 1430,
  CVTSD2SIrm = 1431,
  CVTSD2SIrm_Int = 1432,
  CVTSD2SIrr = 1433,
  CVTSD2SIrr_Int = 1434,
  CVTSD2SSrm = 1435,
  CVTSD2SSrm_Int = 1436,
  CVTSD2SSrr = 1437,
  CVTSD2SSrr_Int = 1438,
  CVTSI2SDrm = 1439,
  CVTSI2SDrm_Int = 1440,
  CVTSI2SDrr = 1441,
  CVTSI2SDrr_Int = 1442,
  CVTSI2SSrm = 1443,
  CVTSI2SSrm_Int = 1444,
  CVTSI2SSrr = 1445,
  CVTSI2SSrr_Int = 1446,
  CVTSI642SDrm = 1447,
  CVTSI642SDrm_Int = 1448,
  CVTSI642SDrr = 1449,
  CVTSI642SDrr_Int = 1450,
  CVTSI642SSrm = 1451,
  CVTSI642SSrm_Int = 1452,
  CVTSI642SSrr = 1453,
  CVTSI642SSrr_Int = 1454,
  CVTSS2SDrm = 1455,
  CVTSS2SDrm_Int = 1456,
  CVTSS2SDrr = 1457,
  CVTSS2SDrr_Int = 1458,
  CVTSS2SI64rm = 1459,
  CVTSS2SI64rm_Int = 1460,
  CVTSS2SI64rr = 1461,
  CVTSS2SI64rr_Int = 1462,
  CVTSS2SIrm = 1463,
  CVTSS2SIrm_Int = 1464,
  CVTSS2SIrr = 1465,
  CVTSS2SIrr_Int = 1466,
  CVTTPD2DQrm = 1467,
  CVTTPD2DQrr = 1468,
  CVTTPS2DQrm = 1469,
  CVTTPS2DQrr = 1470,
  CVTTSD2SI64rm = 1471,
  CVTTSD2SI64rm_Int = 1472,
  CVTTSD2SI64rr = 1473,
  CVTTSD2SI64rr_Int = 1474,
  CVTTSD2SIrm = 1475,
  CVTTSD2SIrm_Int = 1476,
  CVTTSD2SIrr = 1477,
  CVTTSD2SIrr_Int = 1478,
  CVTTSS2SI64rm = 1479,
  CVTTSS2SI64rm_Int = 1480,
  CVTTSS2SI64rr = 1481,
  CVTTSS2SI64rr_Int = 1482,
  CVTTSS2SIrm = 1483,
  CVTTSS2SIrm_Int = 1484,
  CVTTSS2SIrr = 1485,
  CVTTSS2SIrr_Int = 1486,
  CWD = 1487,
  CWDE = 1488,
  DAA = 1489,
  DAS = 1490,
  DATA16_PREFIX = 1491,
  DEC16m = 1492,
  DEC16m_EVEX = 1493,
  DEC16m_ND = 1494,
  DEC16m_NF = 1495,
  DEC16m_NF_ND = 1496,
  DEC16r = 1497,
  DEC16r_EVEX = 1498,
  DEC16r_ND = 1499,
  DEC16r_NF = 1500,
  DEC16r_NF_ND = 1501,
  DEC16r_alt = 1502,
  DEC32m = 1503,
  DEC32m_EVEX = 1504,
  DEC32m_ND = 1505,
  DEC32m_NF = 1506,
  DEC32m_NF_ND = 1507,
  DEC32r = 1508,
  DEC32r_EVEX = 1509,
  DEC32r_ND = 1510,
  DEC32r_NF = 1511,
  DEC32r_NF_ND = 1512,
  DEC32r_alt = 1513,
  DEC64m = 1514,
  DEC64m_EVEX = 1515,
  DEC64m_ND = 1516,
  DEC64m_NF = 1517,
  DEC64m_NF_ND = 1518,
  DEC64r = 1519,
  DEC64r_EVEX = 1520,
  DEC64r_ND = 1521,
  DEC64r_NF = 1522,
  DEC64r_NF_ND = 1523,
  DEC8m = 1524,
  DEC8m_EVEX = 1525,
  DEC8m_ND = 1526,
  DEC8m_NF = 1527,
  DEC8m_NF_ND = 1528,
  DEC8r = 1529,
  DEC8r_EVEX = 1530,
  DEC8r_ND = 1531,
  DEC8r_NF = 1532,
  DEC8r_NF_ND = 1533,
  DIV16m = 1534,
  DIV16m_EVEX = 1535,
  DIV16m_NF = 1536,
  DIV16r = 1537,
  DIV16r_EVEX = 1538,
  DIV16r_NF = 1539,
  DIV32m = 1540,
  DIV32m_EVEX = 1541,
  DIV32m_NF = 1542,
  DIV32r = 1543,
  DIV32r_EVEX = 1544,
  DIV32r_NF = 1545,
  DIV64m = 1546,
  DIV64m_EVEX = 1547,
  DIV64m_NF = 1548,
  DIV64r = 1549,
  DIV64r_EVEX = 1550,
  DIV64r_NF = 1551,
  DIV8m = 1552,
  DIV8m_EVEX = 1553,
  DIV8m_NF = 1554,
  DIV8r = 1555,
  DIV8r_EVEX = 1556,
  DIV8r_NF = 1557,
  DIVPDrm = 1558,
  DIVPDrr = 1559,
  DIVPSrm = 1560,
  DIVPSrr = 1561,
  DIVR_F32m = 1562,
  DIVR_F64m = 1563,
  DIVR_FI16m = 1564,
  DIVR_FI32m = 1565,
  DIVR_FPrST0 = 1566,
  DIVR_FST0r = 1567,
  DIVR_Fp32m = 1568,
  DIVR_Fp64m = 1569,
  DIVR_Fp64m32 = 1570,
  DIVR_Fp80m32 = 1571,
  DIVR_Fp80m64 = 1572,
  DIVR_FpI16m32 = 1573,
  DIVR_FpI16m64 = 1574,
  DIVR_FpI16m80 = 1575,
  DIVR_FpI32m32 = 1576,
  DIVR_FpI32m64 = 1577,
  DIVR_FpI32m80 = 1578,
  DIVR_FrST0 = 1579,
  DIVSDrm = 1580,
  DIVSDrm_Int = 1581,
  DIVSDrr = 1582,
  DIVSDrr_Int = 1583,
  DIVSSrm = 1584,
  DIVSSrm_Int = 1585,
  DIVSSrr = 1586,
  DIVSSrr_Int = 1587,
  DIV_F32m = 1588,
  DIV_F64m = 1589,
  DIV_FI16m = 1590,
  DIV_FI32m = 1591,
  DIV_FPrST0 = 1592,
  DIV_FST0r = 1593,
  DIV_Fp32 = 1594,
  DIV_Fp32m = 1595,
  DIV_Fp64 = 1596,
  DIV_Fp64m = 1597,
  DIV_Fp64m32 = 1598,
  DIV_Fp80 = 1599,
  DIV_Fp80m32 = 1600,
  DIV_Fp80m64 = 1601,
  DIV_FpI16m32 = 1602,
  DIV_FpI16m64 = 1603,
  DIV_FpI16m80 = 1604,
  DIV_FpI32m32 = 1605,
  DIV_FpI32m64 = 1606,
  DIV_FpI32m80 = 1607,
  DIV_FrST0 = 1608,
  DPPDrmi = 1609,
  DPPDrri = 1610,
  DPPSrmi = 1611,
  DPPSrri = 1612,
  DS_PREFIX = 1613,
  DYN_ALLOCA_32 = 1614,
  DYN_ALLOCA_64 = 1615,
  EH_RETURN = 1616,
  EH_RETURN64 = 1617,
  EH_SjLj_LongJmp32 = 1618,
  EH_SjLj_LongJmp64 = 1619,
  EH_SjLj_SetJmp32 = 1620,
  EH_SjLj_SetJmp64 = 1621,
  EH_SjLj_Setup = 1622,
  ENCLS = 1623,
  ENCLU = 1624,
  ENCLV = 1625,
  ENCODEKEY128 = 1626,
  ENCODEKEY256 = 1627,
  ENDBR32 = 1628,
  ENDBR64 = 1629,
  ENQCMD16 = 1630,
  ENQCMD32 = 1631,
  ENQCMD32_EVEX = 1632,
  ENQCMD64 = 1633,
  ENQCMD64_EVEX = 1634,
  ENQCMDS16 = 1635,
  ENQCMDS32 = 1636,
  ENQCMDS32_EVEX = 1637,
  ENQCMDS64 = 1638,
  ENQCMDS64_EVEX = 1639,
  ENTER = 1640,
  ERETS = 1641,
  ERETU = 1642,
  ES_PREFIX = 1643,
  EXTRACTPSmri = 1644,
  EXTRACTPSrri = 1645,
  EXTRQ = 1646,
  EXTRQI = 1647,
  F2XM1 = 1648,
  FARCALL16i = 1649,
  FARCALL16m = 1650,
  FARCALL32i = 1651,
  FARCALL32m = 1652,
  FARCALL64m = 1653,
  FARJMP16i = 1654,
  FARJMP16m = 1655,
  FARJMP32i = 1656,
  FARJMP32m = 1657,
  FARJMP64m = 1658,
  FBLDm = 1659,
  FBSTPm = 1660,
  FCOM32m = 1661,
  FCOM64m = 1662,
  FCOMP32m = 1663,
  FCOMP64m = 1664,
  FCOMPP = 1665,
  FCOS = 1666,
  FDECSTP = 1667,
  FEMMS = 1668,
  FFREE = 1669,
  FFREEP = 1670,
  FICOM16m = 1671,
  FICOM32m = 1672,
  FICOMP16m = 1673,
  FICOMP32m = 1674,
  FINCSTP = 1675,
  FLDCW16m = 1676,
  FLDENVm = 1677,
  FLDL2E = 1678,
  FLDL2T = 1679,
  FLDLG2 = 1680,
  FLDLN2 = 1681,
  FLDPI = 1682,
  FNCLEX = 1683,
  FNINIT = 1684,
  FNOP = 1685,
  FNSTCW16m = 1686,
  FNSTSW16r = 1687,
  FNSTSWm = 1688,
  FP32_TO_INT16_IN_MEM = 1689,
  FP32_TO_INT32_IN_MEM = 1690,
  FP32_TO_INT64_IN_MEM = 1691,
  FP64_TO_INT16_IN_MEM = 1692,
  FP64_TO_INT32_IN_MEM = 1693,
  FP64_TO_INT64_IN_MEM = 1694,
  FP80_ADDm32 = 1695,
  FP80_ADDr = 1696,
  FP80_TO_INT16_IN_MEM = 1697,
  FP80_TO_INT32_IN_MEM = 1698,
  FP80_TO_INT64_IN_MEM = 1699,
  FPATAN = 1700,
  FPREM = 1701,
  FPREM1 = 1702,
  FPTAN = 1703,
  FRNDINT = 1704,
  FRSTORm = 1705,
  FSAVEm = 1706,
  FSCALE = 1707,
  FSIN = 1708,
  FSINCOS = 1709,
  FSTENVm = 1710,
  FS_PREFIX = 1711,
  FXRSTOR = 1712,
  FXRSTOR64 = 1713,
  FXSAVE = 1714,
  FXSAVE64 = 1715,
  FXTRACT = 1716,
  FYL2X = 1717,
  FYL2XP1 = 1718,
  GETSEC = 1719,
  GF2P8AFFINEINVQBrmi = 1720,
  GF2P8AFFINEINVQBrri = 1721,
  GF2P8AFFINEQBrmi = 1722,
  GF2P8AFFINEQBrri = 1723,
  GF2P8MULBrm = 1724,
  GF2P8MULBrr = 1725,
  GS_PREFIX = 1726,
  HADDPDrm = 1727,
  HADDPDrr = 1728,
  HADDPSrm = 1729,
  HADDPSrr = 1730,
  HLT = 1731,
  HRESET = 1732,
  HSUBPDrm = 1733,
  HSUBPDrr = 1734,
  HSUBPSrm = 1735,
  HSUBPSrr = 1736,
  IDIV16m = 1737,
  IDIV16m_EVEX = 1738,
  IDIV16m_NF = 1739,
  IDIV16r = 1740,
  IDIV16r_EVEX = 1741,
  IDIV16r_NF = 1742,
  IDIV32m = 1743,
  IDIV32m_EVEX = 1744,
  IDIV32m_NF = 1745,
  IDIV32r = 1746,
  IDIV32r_EVEX = 1747,
  IDIV32r_NF = 1748,
  IDIV64m = 1749,
  IDIV64m_EVEX = 1750,
  IDIV64m_NF = 1751,
  IDIV64r = 1752,
  IDIV64r_EVEX = 1753,
  IDIV64r_NF = 1754,
  IDIV8m = 1755,
  IDIV8m_EVEX = 1756,
  IDIV8m_NF = 1757,
  IDIV8r = 1758,
  IDIV8r_EVEX = 1759,
  IDIV8r_NF = 1760,
  ILD_F16m = 1761,
  ILD_F32m = 1762,
  ILD_F64m = 1763,
  ILD_Fp16m32 = 1764,
  ILD_Fp16m64 = 1765,
  ILD_Fp16m80 = 1766,
  ILD_Fp32m32 = 1767,
  ILD_Fp32m64 = 1768,
  ILD_Fp32m80 = 1769,
  ILD_Fp64m32 = 1770,
  ILD_Fp64m64 = 1771,
  ILD_Fp64m80 = 1772,
  IMUL16m = 1773,
  IMUL16m_EVEX = 1774,
  IMUL16m_NF = 1775,
  IMUL16r = 1776,
  IMUL16r_EVEX = 1777,
  IMUL16r_NF = 1778,
  IMUL16rm = 1779,
  IMUL16rm_EVEX = 1780,
  IMUL16rm_ND = 1781,
  IMUL16rm_NF = 1782,
  IMUL16rm_NF_ND = 1783,
  IMUL16rmi = 1784,
  IMUL16rmi8 = 1785,
  IMUL16rmi8_EVEX = 1786,
  IMUL16rmi8_NF = 1787,
  IMUL16rmi_EVEX = 1788,
  IMUL16rmi_NF = 1789,
  IMUL16rr = 1790,
  IMUL16rr_EVEX = 1791,
  IMUL16rr_ND = 1792,
  IMUL16rr_NF = 1793,
  IMUL16rr_NF_ND = 1794,
  IMUL16rri = 1795,
  IMUL16rri8 = 1796,
  IMUL16rri8_EVEX = 1797,
  IMUL16rri8_NF = 1798,
  IMUL16rri_EVEX = 1799,
  IMUL16rri_NF = 1800,
  IMUL32m = 1801,
  IMUL32m_EVEX = 1802,
  IMUL32m_NF = 1803,
  IMUL32r = 1804,
  IMUL32r_EVEX = 1805,
  IMUL32r_NF = 1806,
  IMUL32rm = 1807,
  IMUL32rm_EVEX = 1808,
  IMUL32rm_ND = 1809,
  IMUL32rm_NF = 1810,
  IMUL32rm_NF_ND = 1811,
  IMUL32rmi = 1812,
  IMUL32rmi8 = 1813,
  IMUL32rmi8_EVEX = 1814,
  IMUL32rmi8_NF = 1815,
  IMUL32rmi_EVEX = 1816,
  IMUL32rmi_NF = 1817,
  IMUL32rr = 1818,
  IMUL32rr_EVEX = 1819,
  IMUL32rr_ND = 1820,
  IMUL32rr_NF = 1821,
  IMUL32rr_NF_ND = 1822,
  IMUL32rri = 1823,
  IMUL32rri8 = 1824,
  IMUL32rri8_EVEX = 1825,
  IMUL32rri8_NF = 1826,
  IMUL32rri_EVEX = 1827,
  IMUL32rri_NF = 1828,
  IMUL64m = 1829,
  IMUL64m_EVEX = 1830,
  IMUL64m_NF = 1831,
  IMUL64r = 1832,
  IMUL64r_EVEX = 1833,
  IMUL64r_NF = 1834,
  IMUL64rm = 1835,
  IMUL64rm_EVEX = 1836,
  IMUL64rm_ND = 1837,
  IMUL64rm_NF = 1838,
  IMUL64rm_NF_ND = 1839,
  IMUL64rmi32 = 1840,
  IMUL64rmi32_EVEX = 1841,
  IMUL64rmi32_NF = 1842,
  IMUL64rmi8 = 1843,
  IMUL64rmi8_EVEX = 1844,
  IMUL64rmi8_NF = 1845,
  IMUL64rr = 1846,
  IMUL64rr_EVEX = 1847,
  IMUL64rr_ND = 1848,
  IMUL64rr_NF = 1849,
  IMUL64rr_NF_ND = 1850,
  IMUL64rri32 = 1851,
  IMUL64rri32_EVEX = 1852,
  IMUL64rri32_NF = 1853,
  IMUL64rri8 = 1854,
  IMUL64rri8_EVEX = 1855,
  IMUL64rri8_NF = 1856,
  IMUL8m = 1857,
  IMUL8m_EVEX = 1858,
  IMUL8m_NF = 1859,
  IMUL8r = 1860,
  IMUL8r_EVEX = 1861,
  IMUL8r_NF = 1862,
  IMULZU16rmi = 1863,
  IMULZU16rmi8 = 1864,
  IMULZU16rri = 1865,
  IMULZU16rri8 = 1866,
  IMULZU32rmi = 1867,
  IMULZU32rmi8 = 1868,
  IMULZU32rri = 1869,
  IMULZU32rri8 = 1870,
  IMULZU64rmi32 = 1871,
  IMULZU64rmi8 = 1872,
  IMULZU64rri32 = 1873,
  IMULZU64rri8 = 1874,
  IN16ri = 1875,
  IN16rr = 1876,
  IN32ri = 1877,
  IN32rr = 1878,
  IN8ri = 1879,
  IN8rr = 1880,
  INC16m = 1881,
  INC16m_EVEX = 1882,
  INC16m_ND = 1883,
  INC16m_NF = 1884,
  INC16m_NF_ND = 1885,
  INC16r = 1886,
  INC16r_EVEX = 1887,
  INC16r_ND = 1888,
  INC16r_NF = 1889,
  INC16r_NF_ND = 1890,
  INC16r_alt = 1891,
  INC32m = 1892,
  INC32m_EVEX = 1893,
  INC32m_ND = 1894,
  INC32m_NF = 1895,
  INC32m_NF_ND = 1896,
  INC32r = 1897,
  INC32r_EVEX = 1898,
  INC32r_ND = 1899,
  INC32r_NF = 1900,
  INC32r_NF_ND = 1901,
  INC32r_alt = 1902,
  INC64m = 1903,
  INC64m_EVEX = 1904,
  INC64m_ND = 1905,
  INC64m_NF = 1906,
  INC64m_NF_ND = 1907,
  INC64r = 1908,
  INC64r_EVEX = 1909,
  INC64r_ND = 1910,
  INC64r_NF = 1911,
  INC64r_NF_ND = 1912,
  INC8m = 1913,
  INC8m_EVEX = 1914,
  INC8m_ND = 1915,
  INC8m_NF = 1916,
  INC8m_NF_ND = 1917,
  INC8r = 1918,
  INC8r_EVEX = 1919,
  INC8r_ND = 1920,
  INC8r_NF = 1921,
  INC8r_NF_ND = 1922,
  INCSSPD = 1923,
  INCSSPQ = 1924,
  INSB = 1925,
  INSERTPSrmi = 1926,
  INSERTPSrri = 1927,
  INSERTQ = 1928,
  INSERTQI = 1929,
  INSL = 1930,
  INSW = 1931,
  INT = 1932,
  INT3 = 1933,
  INTO = 1934,
  INVD = 1935,
  INVEPT32 = 1936,
  INVEPT64 = 1937,
  INVEPT64_EVEX = 1938,
  INVLPG = 1939,
  INVLPGA32 = 1940,
  INVLPGA64 = 1941,
  INVLPGB32 = 1942,
  INVLPGB64 = 1943,
  INVPCID32 = 1944,
  INVPCID64 = 1945,
  INVPCID64_EVEX = 1946,
  INVVPID32 = 1947,
  INVVPID64 = 1948,
  INVVPID64_EVEX = 1949,
  IRET = 1950,
  IRET16 = 1951,
  IRET32 = 1952,
  IRET64 = 1953,
  ISTT_FP16m = 1954,
  ISTT_FP32m = 1955,
  ISTT_FP64m = 1956,
  ISTT_Fp16m32 = 1957,
  ISTT_Fp16m64 = 1958,
  ISTT_Fp16m80 = 1959,
  ISTT_Fp32m32 = 1960,
  ISTT_Fp32m64 = 1961,
  ISTT_Fp32m80 = 1962,
  ISTT_Fp64m32 = 1963,
  ISTT_Fp64m64 = 1964,
  ISTT_Fp64m80 = 1965,
  IST_F16m = 1966,
  IST_F32m = 1967,
  IST_FP16m = 1968,
  IST_FP32m = 1969,
  IST_FP64m = 1970,
  IST_Fp16m32 = 1971,
  IST_Fp16m64 = 1972,
  IST_Fp16m80 = 1973,
  IST_Fp32m32 = 1974,
  IST_Fp32m64 = 1975,
  IST_Fp32m80 = 1976,
  IST_Fp64m32 = 1977,
  IST_Fp64m64 = 1978,
  IST_Fp64m80 = 1979,
  Int_eh_sjlj_setup_dispatch = 1980,
  JCC_1 = 1981,
  JCC_2 = 1982,
  JCC_4 = 1983,
  JCXZ = 1984,
  JECXZ = 1985,
  JMP16m = 1986,
  JMP16m_NT = 1987,
  JMP16r = 1988,
  JMP16r_NT = 1989,
  JMP32m = 1990,
  JMP32m_NT = 1991,
  JMP32r = 1992,
  JMP32r_NT = 1993,
  JMP64m = 1994,
  JMP64m_NT = 1995,
  JMP64m_REX = 1996,
  JMP64r = 1997,
  JMP64r_NT = 1998,
  JMP64r_REX = 1999,
  JMPABS64i = 2000,
  JMP_1 = 2001,
  JMP_2 = 2002,
  JMP_4 = 2003,
  JRCXZ = 2004,
  KADDBkk = 2005,
  KADDDkk = 2006,
  KADDQkk = 2007,
  KADDWkk = 2008,
  KANDBkk = 2009,
  KANDDkk = 2010,
  KANDNBkk = 2011,
  KANDNDkk = 2012,
  KANDNQkk = 2013,
  KANDNWkk = 2014,
  KANDQkk = 2015,
  KANDWkk = 2016,
  KCFI_CHECK = 2017,
  KMOVBkk = 2018,
  KMOVBkk_EVEX = 2019,
  KMOVBkm = 2020,
  KMOVBkm_EVEX = 2021,
  KMOVBkr = 2022,
  KMOVBkr_EVEX = 2023,
  KMOVBmk = 2024,
  KMOVBmk_EVEX = 2025,
  KMOVBrk = 2026,
  KMOVBrk_EVEX = 2027,
  KMOVDkk = 2028,
  KMOVDkk_EVEX = 2029,
  KMOVDkm = 2030,
  KMOVDkm_EVEX = 2031,
  KMOVDkr = 2032,
  KMOVDkr_EVEX = 2033,
  KMOVDmk = 2034,
  KMOVDmk_EVEX = 2035,
  KMOVDrk = 2036,
  KMOVDrk_EVEX = 2037,
  KMOVQkk = 2038,
  KMOVQkk_EVEX = 2039,
  KMOVQkm = 2040,
  KMOVQkm_EVEX = 2041,
  KMOVQkr = 2042,
  KMOVQkr_EVEX = 2043,
  KMOVQmk = 2044,
  KMOVQmk_EVEX = 2045,
  KMOVQrk = 2046,
  KMOVQrk_EVEX = 2047,
  KMOVWkk = 2048,
  KMOVWkk_EVEX = 2049,
  KMOVWkm = 2050,
  KMOVWkm_EVEX = 2051,
  KMOVWkr = 2052,
  KMOVWkr_EVEX = 2053,
  KMOVWmk = 2054,
  KMOVWmk_EVEX = 2055,
  KMOVWrk = 2056,
  KMOVWrk_EVEX = 2057,
  KNOTBkk = 2058,
  KNOTDkk = 2059,
  KNOTQkk = 2060,
  KNOTWkk = 2061,
  KORBkk = 2062,
  KORDkk = 2063,
  KORQkk = 2064,
  KORTESTBkk = 2065,
  KORTESTDkk = 2066,
  KORTESTQkk = 2067,
  KORTESTWkk = 2068,
  KORWkk = 2069,
  KSHIFTLBki = 2070,
  KSHIFTLDki = 2071,
  KSHIFTLQki = 2072,
  KSHIFTLWki = 2073,
  KSHIFTRBki = 2074,
  KSHIFTRDki = 2075,
  KSHIFTRQki = 2076,
  KSHIFTRWki = 2077,
  KTESTBkk = 2078,
  KTESTDkk = 2079,
  KTESTQkk = 2080,
  KTESTWkk = 2081,
  KUNPCKBWkk = 2082,
  KUNPCKDQkk = 2083,
  KUNPCKWDkk = 2084,
  KXNORBkk = 2085,
  KXNORDkk = 2086,
  KXNORQkk = 2087,
  KXNORWkk = 2088,
  KXORBkk = 2089,
  KXORDkk = 2090,
  KXORQkk = 2091,
  KXORWkk = 2092,
  LAHF = 2093,
  LAR16rm = 2094,
  LAR16rr = 2095,
  LAR32rm = 2096,
  LAR32rr = 2097,
  LAR64rm = 2098,
  LAR64rr = 2099,
  LCMPXCHG16 = 2100,
  LCMPXCHG16B = 2101,
  LCMPXCHG32 = 2102,
  LCMPXCHG64 = 2103,
  LCMPXCHG8 = 2104,
  LCMPXCHG8B = 2105,
  LDDQUrm = 2106,
  LDMXCSR = 2107,
  LDS16rm = 2108,
  LDS32rm = 2109,
  LDTILECFG = 2110,
  LDTILECFG_EVEX = 2111,
  LD_F0 = 2112,
  LD_F1 = 2113,
  LD_F32m = 2114,
  LD_F64m = 2115,
  LD_F80m = 2116,
  LD_Fp032 = 2117,
  LD_Fp064 = 2118,
  LD_Fp080 = 2119,
  LD_Fp132 = 2120,
  LD_Fp164 = 2121,
  LD_Fp180 = 2122,
  LD_Fp32m = 2123,
  LD_Fp32m64 = 2124,
  LD_Fp32m80 = 2125,
  LD_Fp64m = 2126,
  LD_Fp64m80 = 2127,
  LD_Fp80m = 2128,
  LD_Frr = 2129,
  LEA16r = 2130,
  LEA32r = 2131,
  LEA64_16r = 2132,
  LEA64_32r = 2133,
  LEA64_8r = 2134,
  LEA64r = 2135,
  LEAVE = 2136,
  LEAVE64 = 2137,
  LES16rm = 2138,
  LES32rm = 2139,
  LFENCE = 2140,
  LFS16rm = 2141,
  LFS32rm = 2142,
  LFS64rm = 2143,
  LGDT16m = 2144,
  LGDT32m = 2145,
  LGDT64m = 2146,
  LGS16rm = 2147,
  LGS32rm = 2148,
  LGS64rm = 2149,
  LIDT16m = 2150,
  LIDT32m = 2151,
  LIDT64m = 2152,
  LKGS16m = 2153,
  LKGS16r = 2154,
  LLDT16m = 2155,
  LLDT16r = 2156,
  LLWPCB = 2157,
  LLWPCB64 = 2158,
  LMSW16m = 2159,
  LMSW16r = 2160,
  LOADIWKEY = 2161,
  LOCK_ADD16mi = 2162,
  LOCK_ADD16mi8 = 2163,
  LOCK_ADD16mr = 2164,
  LOCK_ADD32mi = 2165,
  LOCK_ADD32mi8 = 2166,
  LOCK_ADD32mr = 2167,
  LOCK_ADD64mi32 = 2168,
  LOCK_ADD64mi8 = 2169,
  LOCK_ADD64mr = 2170,
  LOCK_ADD8mi = 2171,
  LOCK_ADD8mr = 2172,
  LOCK_AND16mi = 2173,
  LOCK_AND16mi8 = 2174,
  LOCK_AND16mr = 2175,
  LOCK_AND32mi = 2176,
  LOCK_AND32mi8 = 2177,
  LOCK_AND32mr = 2178,
  LOCK_AND64mi32 = 2179,
  LOCK_AND64mi8 = 2180,
  LOCK_AND64mr = 2181,
  LOCK_AND8mi = 2182,
  LOCK_AND8mr = 2183,
  LOCK_BTC16m = 2184,
  LOCK_BTC32m = 2185,
  LOCK_BTC64m = 2186,
  LOCK_BTC_RM16rm = 2187,
  LOCK_BTC_RM32rm = 2188,
  LOCK_BTC_RM64rm = 2189,
  LOCK_BTR16m = 2190,
  LOCK_BTR32m = 2191,
  LOCK_BTR64m = 2192,
  LOCK_BTR_RM16rm = 2193,
  LOCK_BTR_RM32rm = 2194,
  LOCK_BTR_RM64rm = 2195,
  LOCK_BTS16m = 2196,
  LOCK_BTS32m = 2197,
  LOCK_BTS64m = 2198,
  LOCK_BTS_RM16rm = 2199,
  LOCK_BTS_RM32rm = 2200,
  LOCK_BTS_RM64rm = 2201,
  LOCK_DEC16m = 2202,
  LOCK_DEC32m = 2203,
  LOCK_DEC64m = 2204,
  LOCK_DEC8m = 2205,
  LOCK_INC16m = 2206,
  LOCK_INC32m = 2207,
  LOCK_INC64m = 2208,
  LOCK_INC8m = 2209,
  LOCK_OR16mi = 2210,
  LOCK_OR16mi8 = 2211,
  LOCK_OR16mr = 2212,
  LOCK_OR32mi = 2213,
  LOCK_OR32mi8 = 2214,
  LOCK_OR32mr = 2215,
  LOCK_OR64mi32 = 2216,
  LOCK_OR64mi8 = 2217,
  LOCK_OR64mr = 2218,
  LOCK_OR8mi = 2219,
  LOCK_OR8mr = 2220,
  LOCK_PREFIX = 2221,
  LOCK_SUB16mi = 2222,
  LOCK_SUB16mi8 = 2223,
  LOCK_SUB16mr = 2224,
  LOCK_SUB32mi = 2225,
  LOCK_SUB32mi8 = 2226,
  LOCK_SUB32mr = 2227,
  LOCK_SUB64mi32 = 2228,
  LOCK_SUB64mi8 = 2229,
  LOCK_SUB64mr = 2230,
  LOCK_SUB8mi = 2231,
  LOCK_SUB8mr = 2232,
  LOCK_XOR16mi = 2233,
  LOCK_XOR16mi8 = 2234,
  LOCK_XOR16mr = 2235,
  LOCK_XOR32mi = 2236,
  LOCK_XOR32mi8 = 2237,
  LOCK_XOR32mr = 2238,
  LOCK_XOR64mi32 = 2239,
  LOCK_XOR64mi8 = 2240,
  LOCK_XOR64mr = 2241,
  LOCK_XOR8mi = 2242,
  LOCK_XOR8mr = 2243,
  LODSB = 2244,
  LODSL = 2245,
  LODSQ = 2246,
  LODSW = 2247,
  LOOP = 2248,
  LOOPE = 2249,
  LOOPNE = 2250,
  LRET16 = 2251,
  LRET32 = 2252,
  LRET64 = 2253,
  LRETI16 = 2254,
  LRETI32 = 2255,
  LRETI64 = 2256,
  LSL16rm = 2257,
  LSL16rr = 2258,
  LSL32rm = 2259,
  LSL32rr = 2260,
  LSL64rm = 2261,
  LSL64rr = 2262,
  LSS16rm = 2263,
  LSS32rm = 2264,
  LSS64rm = 2265,
  LTRm = 2266,
  LTRr = 2267,
  LWPINS32rmi = 2268,
  LWPINS32rri = 2269,
  LWPINS64rmi = 2270,
  LWPINS64rri = 2271,
  LWPVAL32rmi = 2272,
  LWPVAL32rri = 2273,
  LWPVAL64rmi = 2274,
  LWPVAL64rri = 2275,
  LXADD16 = 2276,
  LXADD32 = 2277,
  LXADD64 = 2278,
  LXADD8 = 2279,
  LZCNT16rm = 2280,
  LZCNT16rm_EVEX = 2281,
  LZCNT16rm_NF = 2282,
  LZCNT16rr = 2283,
  LZCNT16rr_EVEX = 2284,
  LZCNT16rr_NF = 2285,
  LZCNT32rm = 2286,
  LZCNT32rm_EVEX = 2287,
  LZCNT32rm_NF = 2288,
  LZCNT32rr = 2289,
  LZCNT32rr_EVEX = 2290,
  LZCNT32rr_NF = 2291,
  LZCNT64rm = 2292,
  LZCNT64rm_EVEX = 2293,
  LZCNT64rm_NF = 2294,
  LZCNT64rr = 2295,
  LZCNT64rr_EVEX = 2296,
  LZCNT64rr_NF = 2297,
  MASKMOVDQU = 2298,
  MASKMOVDQU64 = 2299,
  MASKPAIR16LOAD = 2300,
  MASKPAIR16STORE = 2301,
  MAXCPDrm = 2302,
  MAXCPDrr = 2303,
  MAXCPSrm = 2304,
  MAXCPSrr = 2305,
  MAXCSDrm = 2306,
  MAXCSDrr = 2307,
  MAXCSSrm = 2308,
  MAXCSSrr = 2309,
  MAXPDrm = 2310,
  MAXPDrr = 2311,
  MAXPSrm = 2312,
  MAXPSrr = 2313,
  MAXSDrm = 2314,
  MAXSDrm_Int = 2315,
  MAXSDrr = 2316,
  MAXSDrr_Int = 2317,
  MAXSSrm = 2318,
  MAXSSrm_Int = 2319,
  MAXSSrr = 2320,
  MAXSSrr_Int = 2321,
  MFENCE = 2322,
  MINCPDrm = 2323,
  MINCPDrr = 2324,
  MINCPSrm = 2325,
  MINCPSrr = 2326,
  MINCSDrm = 2327,
  MINCSDrr = 2328,
  MINCSSrm = 2329,
  MINCSSrr = 2330,
  MINPDrm = 2331,
  MINPDrr = 2332,
  MINPSrm = 2333,
  MINPSrr = 2334,
  MINSDrm = 2335,
  MINSDrm_Int = 2336,
  MINSDrr = 2337,
  MINSDrr_Int = 2338,
  MINSSrm = 2339,
  MINSSrm_Int = 2340,
  MINSSrr = 2341,
  MINSSrr_Int = 2342,
  MMX_CVTPD2PIrm = 2343,
  MMX_CVTPD2PIrr = 2344,
  MMX_CVTPI2PDrm = 2345,
  MMX_CVTPI2PDrr = 2346,
  MMX_CVTPI2PSrm = 2347,
  MMX_CVTPI2PSrr = 2348,
  MMX_CVTPS2PIrm = 2349,
  MMX_CVTPS2PIrr = 2350,
  MMX_CVTTPD2PIrm = 2351,
  MMX_CVTTPD2PIrr = 2352,
  MMX_CVTTPS2PIrm = 2353,
  MMX_CVTTPS2PIrr = 2354,
  MMX_EMMS = 2355,
  MMX_MASKMOVQ = 2356,
  MMX_MASKMOVQ64 = 2357,
  MMX_MOVD64from64mr = 2358,
  MMX_MOVD64from64rr = 2359,
  MMX_MOVD64grr = 2360,
  MMX_MOVD64mr = 2361,
  MMX_MOVD64rm = 2362,
  MMX_MOVD64rr = 2363,
  MMX_MOVD64to64rm = 2364,
  MMX_MOVD64to64rr = 2365,
  MMX_MOVDQ2Qrr = 2366,
  MMX_MOVFR642Qrr = 2367,
  MMX_MOVNTQmr = 2368,
  MMX_MOVQ2DQrr = 2369,
  MMX_MOVQ2FR64rr = 2370,
  MMX_MOVQ64mr = 2371,
  MMX_MOVQ64rm = 2372,
  MMX_MOVQ64rr = 2373,
  MMX_MOVQ64rr_REV = 2374,
  MMX_PABSBrm = 2375,
  MMX_PABSBrr = 2376,
  MMX_PABSDrm = 2377,
  MMX_PABSDrr = 2378,
  MMX_PABSWrm = 2379,
  MMX_PABSWrr = 2380,
  MMX_PACKSSDWrm = 2381,
  MMX_PACKSSDWrr = 2382,
  MMX_PACKSSWBrm = 2383,
  MMX_PACKSSWBrr = 2384,
  MMX_PACKUSWBrm = 2385,
  MMX_PACKUSWBrr = 2386,
  MMX_PADDBrm = 2387,
  MMX_PADDBrr = 2388,
  MMX_PADDDrm = 2389,
  MMX_PADDDrr = 2390,
  MMX_PADDQrm = 2391,
  MMX_PADDQrr = 2392,
  MMX_PADDSBrm = 2393,
  MMX_PADDSBrr = 2394,
  MMX_PADDSWrm = 2395,
  MMX_PADDSWrr = 2396,
  MMX_PADDUSBrm = 2397,
  MMX_PADDUSBrr = 2398,
  MMX_PADDUSWrm = 2399,
  MMX_PADDUSWrr = 2400,
  MMX_PADDWrm = 2401,
  MMX_PADDWrr = 2402,
  MMX_PALIGNRrmi = 2403,
  MMX_PALIGNRrri = 2404,
  MMX_PANDNrm = 2405,
  MMX_PANDNrr = 2406,
  MMX_PANDrm = 2407,
  MMX_PANDrr = 2408,
  MMX_PAVGBrm = 2409,
  MMX_PAVGBrr = 2410,
  MMX_PAVGWrm = 2411,
  MMX_PAVGWrr = 2412,
  MMX_PCMPEQBrm = 2413,
  MMX_PCMPEQBrr = 2414,
  MMX_PCMPEQDrm = 2415,
  MMX_PCMPEQDrr = 2416,
  MMX_PCMPEQWrm = 2417,
  MMX_PCMPEQWrr = 2418,
  MMX_PCMPGTBrm = 2419,
  MMX_PCMPGTBrr = 2420,
  MMX_PCMPGTDrm = 2421,
  MMX_PCMPGTDrr = 2422,
  MMX_PCMPGTWrm = 2423,
  MMX_PCMPGTWrr = 2424,
  MMX_PEXTRWrri = 2425,
  MMX_PHADDDrm = 2426,
  MMX_PHADDDrr = 2427,
  MMX_PHADDSWrm = 2428,
  MMX_PHADDSWrr = 2429,
  MMX_PHADDWrm = 2430,
  MMX_PHADDWrr = 2431,
  MMX_PHSUBDrm = 2432,
  MMX_PHSUBDrr = 2433,
  MMX_PHSUBSWrm = 2434,
  MMX_PHSUBSWrr = 2435,
  MMX_PHSUBWrm = 2436,
  MMX_PHSUBWrr = 2437,
  MMX_PINSRWrmi = 2438,
  MMX_PINSRWrri = 2439,
  MMX_PMADDUBSWrm = 2440,
  MMX_PMADDUBSWrr = 2441,
  MMX_PMADDWDrm = 2442,
  MMX_PMADDWDrr = 2443,
  MMX_PMAXSWrm = 2444,
  MMX_PMAXSWrr = 2445,
  MMX_PMAXUBrm = 2446,
  MMX_PMAXUBrr = 2447,
  MMX_PMINSWrm = 2448,
  MMX_PMINSWrr = 2449,
  MMX_PMINUBrm = 2450,
  MMX_PMINUBrr = 2451,
  MMX_PMOVMSKBrr = 2452,
  MMX_PMULHRSWrm = 2453,
  MMX_PMULHRSWrr = 2454,
  MMX_PMULHUWrm = 2455,
  MMX_PMULHUWrr = 2456,
  MMX_PMULHWrm = 2457,
  MMX_PMULHWrr = 2458,
  MMX_PMULLWrm = 2459,
  MMX_PMULLWrr = 2460,
  MMX_PMULUDQrm = 2461,
  MMX_PMULUDQrr = 2462,
  MMX_PORrm = 2463,
  MMX_PORrr = 2464,
  MMX_PSADBWrm = 2465,
  MMX_PSADBWrr = 2466,
  MMX_PSHUFBrm = 2467,
  MMX_PSHUFBrr = 2468,
  MMX_PSHUFWmi = 2469,
  MMX_PSHUFWri = 2470,
  MMX_PSIGNBrm = 2471,
  MMX_PSIGNBrr = 2472,
  MMX_PSIGNDrm = 2473,
  MMX_PSIGNDrr = 2474,
  MMX_PSIGNWrm = 2475,
  MMX_PSIGNWrr = 2476,
  MMX_PSLLDri = 2477,
  MMX_PSLLDrm = 2478,
  MMX_PSLLDrr = 2479,
  MMX_PSLLQri = 2480,
  MMX_PSLLQrm = 2481,
  MMX_PSLLQrr = 2482,
  MMX_PSLLWri = 2483,
  MMX_PSLLWrm = 2484,
  MMX_PSLLWrr = 2485,
  MMX_PSRADri = 2486,
  MMX_PSRADrm = 2487,
  MMX_PSRADrr = 2488,
  MMX_PSRAWri = 2489,
  MMX_PSRAWrm = 2490,
  MMX_PSRAWrr = 2491,
  MMX_PSRLDri = 2492,
  MMX_PSRLDrm = 2493,
  MMX_PSRLDrr = 2494,
  MMX_PSRLQri = 2495,
  MMX_PSRLQrm = 2496,
  MMX_PSRLQrr = 2497,
  MMX_PSRLWri = 2498,
  MMX_PSRLWrm = 2499,
  MMX_PSRLWrr = 2500,
  MMX_PSUBBrm = 2501,
  MMX_PSUBBrr = 2502,
  MMX_PSUBDrm = 2503,
  MMX_PSUBDrr = 2504,
  MMX_PSUBQrm = 2505,
  MMX_PSUBQrr = 2506,
  MMX_PSUBSBrm = 2507,
  MMX_PSUBSBrr = 2508,
  MMX_PSUBSWrm = 2509,
  MMX_PSUBSWrr = 2510,
  MMX_PSUBUSBrm = 2511,
  MMX_PSUBUSBrr = 2512,
  MMX_PSUBUSWrm = 2513,
  MMX_PSUBUSWrr = 2514,
  MMX_PSUBWrm = 2515,
  MMX_PSUBWrr = 2516,
  MMX_PUNPCKHBWrm = 2517,
  MMX_PUNPCKHBWrr = 2518,
  MMX_PUNPCKHDQrm = 2519,
  MMX_PUNPCKHDQrr = 2520,
  MMX_PUNPCKHWDrm = 2521,
  MMX_PUNPCKHWDrr = 2522,
  MMX_PUNPCKLBWrm = 2523,
  MMX_PUNPCKLBWrr = 2524,
  MMX_PUNPCKLDQrm = 2525,
  MMX_PUNPCKLDQrr = 2526,
  MMX_PUNPCKLWDrm = 2527,
  MMX_PUNPCKLWDrr = 2528,
  MMX_PXORrm = 2529,
  MMX_PXORrr = 2530,
  MONITOR32rrr = 2531,
  MONITOR64rrr = 2532,
  MONITORX32rrr = 2533,
  MONITORX64rrr = 2534,
  MONTMUL = 2535,
  MOV16ao16 = 2536,
  MOV16ao32 = 2537,
  MOV16ao64 = 2538,
  MOV16mi = 2539,
  MOV16mr = 2540,
  MOV16ms = 2541,
  MOV16o16a = 2542,
  MOV16o32a = 2543,
  MOV16o64a = 2544,
  MOV16ri = 2545,
  MOV16ri_alt = 2546,
  MOV16rm = 2547,
  MOV16rr = 2548,
  MOV16rr_REV = 2549,
  MOV16rs = 2550,
  MOV16sm = 2551,
  MOV16sr = 2552,
  MOV32ao16 = 2553,
  MOV32ao32 = 2554,
  MOV32ao64 = 2555,
  MOV32cr = 2556,
  MOV32dr = 2557,
  MOV32mi = 2558,
  MOV32mr = 2559,
  MOV32o16a = 2560,
  MOV32o32a = 2561,
  MOV32o64a = 2562,
  MOV32rc = 2563,
  MOV32rd = 2564,
  MOV32ri = 2565,
  MOV32ri_alt = 2566,
  MOV32rm = 2567,
  MOV32rr = 2568,
  MOV32rr_REV = 2569,
  MOV32rs = 2570,
  MOV32sr = 2571,
  MOV64ao32 = 2572,
  MOV64ao64 = 2573,
  MOV64cr = 2574,
  MOV64dr = 2575,
  MOV64mi32 = 2576,
  MOV64mr = 2577,
  MOV64o32a = 2578,
  MOV64o64a = 2579,
  MOV64rc = 2580,
  MOV64rd = 2581,
  MOV64ri = 2582,
  MOV64ri32 = 2583,
  MOV64rm = 2584,
  MOV64rr = 2585,
  MOV64rr_REV = 2586,
  MOV64rs = 2587,
  MOV64sr = 2588,
  MOV64toPQIrm = 2589,
  MOV64toPQIrr = 2590,
  MOV64toSDrr = 2591,
  MOV8ao16 = 2592,
  MOV8ao32 = 2593,
  MOV8ao64 = 2594,
  MOV8mi = 2595,
  MOV8mr = 2596,
  MOV8mr_NOREX = 2597,
  MOV8o16a = 2598,
  MOV8o32a = 2599,
  MOV8o64a = 2600,
  MOV8ri = 2601,
  MOV8ri_alt = 2602,
  MOV8rm = 2603,
  MOV8rm_NOREX = 2604,
  MOV8rr = 2605,
  MOV8rr_NOREX = 2606,
  MOV8rr_REV = 2607,
  MOVAPDmr = 2608,
  MOVAPDrm = 2609,
  MOVAPDrr = 2610,
  MOVAPDrr_REV = 2611,
  MOVAPSmr = 2612,
  MOVAPSrm = 2613,
  MOVAPSrr = 2614,
  MOVAPSrr_REV = 2615,
  MOVBE16mr = 2616,
  MOVBE16mr_EVEX = 2617,
  MOVBE16rm = 2618,
  MOVBE16rm_EVEX = 2619,
  MOVBE16rr = 2620,
  MOVBE16rr_REV = 2621,
  MOVBE32mr = 2622,
  MOVBE32mr_EVEX = 2623,
  MOVBE32rm = 2624,
  MOVBE32rm_EVEX = 2625,
  MOVBE32rr = 2626,
  MOVBE32rr_REV = 2627,
  MOVBE64mr = 2628,
  MOVBE64mr_EVEX = 2629,
  MOVBE64rm = 2630,
  MOVBE64rm_EVEX = 2631,
  MOVBE64rr = 2632,
  MOVBE64rr_REV = 2633,
  MOVDDUPrm = 2634,
  MOVDDUPrr = 2635,
  MOVDI2PDIrm = 2636,
  MOVDI2PDIrr = 2637,
  MOVDI2SSrr = 2638,
  MOVDIR64B16 = 2639,
  MOVDIR64B32 = 2640,
  MOVDIR64B32_EVEX = 2641,
  MOVDIR64B64 = 2642,
  MOVDIR64B64_EVEX = 2643,
  MOVDIRI32 = 2644,
  MOVDIRI32_EVEX = 2645,
  MOVDIRI64 = 2646,
  MOVDIRI64_EVEX = 2647,
  MOVDQAmr = 2648,
  MOVDQArm = 2649,
  MOVDQArr = 2650,
  MOVDQArr_REV = 2651,
  MOVDQUmr = 2652,
  MOVDQUrm = 2653,
  MOVDQUrr = 2654,
  MOVDQUrr_REV = 2655,
  MOVHLPSrr = 2656,
  MOVHPDmr = 2657,
  MOVHPDrm = 2658,
  MOVHPSmr = 2659,
  MOVHPSrm = 2660,
  MOVLHPSrr = 2661,
  MOVLPDmr = 2662,
  MOVLPDrm = 2663,
  MOVLPSmr = 2664,
  MOVLPSrm = 2665,
  MOVMSKPDrr = 2666,
  MOVMSKPSrr = 2667,
  MOVNTDQArm = 2668,
  MOVNTDQmr = 2669,
  MOVNTI_64mr = 2670,
  MOVNTImr = 2671,
  MOVNTPDmr = 2672,
  MOVNTPSmr = 2673,
  MOVNTSD = 2674,
  MOVNTSS = 2675,
  MOVPC32r = 2676,
  MOVPDI2DImr = 2677,
  MOVPDI2DIrr = 2678,
  MOVPQI2QImr = 2679,
  MOVPQI2QIrr = 2680,
  MOVPQIto64mr = 2681,
  MOVPQIto64rr = 2682,
  MOVQI2PQIrm = 2683,
  MOVRS16rm = 2684,
  MOVRS16rm_EVEX = 2685,
  MOVRS32rm = 2686,
  MOVRS32rm_EVEX = 2687,
  MOVRS64rm = 2688,
  MOVRS64rm_EVEX = 2689,
  MOVRS8rm = 2690,
  MOVRS8rm_EVEX = 2691,
  MOVSB = 2692,
  MOVSDmr = 2693,
  MOVSDrm = 2694,
  MOVSDrm_alt = 2695,
  MOVSDrr = 2696,
  MOVSDrr_REV = 2697,
  MOVSDto64rr = 2698,
  MOVSHDUPrm = 2699,
  MOVSHDUPrr = 2700,
  MOVSL = 2701,
  MOVSLDUPrm = 2702,
  MOVSLDUPrr = 2703,
  MOVSQ = 2704,
  MOVSS2DIrr = 2705,
  MOVSSmr = 2706,
  MOVSSrm = 2707,
  MOVSSrm_alt = 2708,
  MOVSSrr = 2709,
  MOVSSrr_REV = 2710,
  MOVSW = 2711,
  MOVSX16rm16 = 2712,
  MOVSX16rm32 = 2713,
  MOVSX16rm8 = 2714,
  MOVSX16rr16 = 2715,
  MOVSX16rr32 = 2716,
  MOVSX16rr8 = 2717,
  MOVSX32rm16 = 2718,
  MOVSX32rm32 = 2719,
  MOVSX32rm8 = 2720,
  MOVSX32rm8_NOREX = 2721,
  MOVSX32rr16 = 2722,
  MOVSX32rr32 = 2723,
  MOVSX32rr8 = 2724,
  MOVSX32rr8_NOREX = 2725,
  MOVSX64rm16 = 2726,
  MOVSX64rm32 = 2727,
  MOVSX64rm8 = 2728,
  MOVSX64rr16 = 2729,
  MOVSX64rr32 = 2730,
  MOVSX64rr8 = 2731,
  MOVUPDmr = 2732,
  MOVUPDrm = 2733,
  MOVUPDrr = 2734,
  MOVUPDrr_REV = 2735,
  MOVUPSmr = 2736,
  MOVUPSrm = 2737,
  MOVUPSrr = 2738,
  MOVUPSrr_REV = 2739,
  MOVZPQILo2PQIrr = 2740,
  MOVZX16rm16 = 2741,
  MOVZX16rm8 = 2742,
  MOVZX16rr16 = 2743,
  MOVZX16rr8 = 2744,
  MOVZX32rm16 = 2745,
  MOVZX32rm8 = 2746,
  MOVZX32rm8_NOREX = 2747,
  MOVZX32rr16 = 2748,
  MOVZX32rr8 = 2749,
  MOVZX32rr8_NOREX = 2750,
  MOVZX64rm16 = 2751,
  MOVZX64rm8 = 2752,
  MOVZX64rr16 = 2753,
  MOVZX64rr8 = 2754,
  MPSADBWrmi = 2755,
  MPSADBWrri = 2756,
  MUL16m = 2757,
  MUL16m_EVEX = 2758,
  MUL16m_NF = 2759,
  MUL16r = 2760,
  MUL16r_EVEX = 2761,
  MUL16r_NF = 2762,
  MUL32m = 2763,
  MUL32m_EVEX = 2764,
  MUL32m_NF = 2765,
  MUL32r = 2766,
  MUL32r_EVEX = 2767,
  MUL32r_NF = 2768,
  MUL64m = 2769,
  MUL64m_EVEX = 2770,
  MUL64m_NF = 2771,
  MUL64r = 2772,
  MUL64r_EVEX = 2773,
  MUL64r_NF = 2774,
  MUL8m = 2775,
  MUL8m_EVEX = 2776,
  MUL8m_NF = 2777,
  MUL8r = 2778,
  MUL8r_EVEX = 2779,
  MUL8r_NF = 2780,
  MULPDrm = 2781,
  MULPDrr = 2782,
  MULPSrm = 2783,
  MULPSrr = 2784,
  MULSDrm = 2785,
  MULSDrm_Int = 2786,
  MULSDrr = 2787,
  MULSDrr_Int = 2788,
  MULSSrm = 2789,
  MULSSrm_Int = 2790,
  MULSSrr = 2791,
  MULSSrr_Int = 2792,
  MULX32Hrm = 2793,
  MULX32Hrr = 2794,
  MULX32rm = 2795,
  MULX32rm_EVEX = 2796,
  MULX32rr = 2797,
  MULX32rr_EVEX = 2798,
  MULX64Hrm = 2799,
  MULX64Hrr = 2800,
  MULX64rm = 2801,
  MULX64rm_EVEX = 2802,
  MULX64rr = 2803,
  MULX64rr_EVEX = 2804,
  MUL_F32m = 2805,
  MUL_F64m = 2806,
  MUL_FI16m = 2807,
  MUL_FI32m = 2808,
  MUL_FPrST0 = 2809,
  MUL_FST0r = 2810,
  MUL_Fp32 = 2811,
  MUL_Fp32m = 2812,
  MUL_Fp64 = 2813,
  MUL_Fp64m = 2814,
  MUL_Fp64m32 = 2815,
  MUL_Fp80 = 2816,
  MUL_Fp80m32 = 2817,
  MUL_Fp80m64 = 2818,
  MUL_FpI16m32 = 2819,
  MUL_FpI16m64 = 2820,
  MUL_FpI16m80 = 2821,
  MUL_FpI32m32 = 2822,
  MUL_FpI32m64 = 2823,
  MUL_FpI32m80 = 2824,
  MUL_FrST0 = 2825,
  MWAITXrrr = 2826,
  MWAITrr = 2827,
  NEG16m = 2828,
  NEG16m_EVEX = 2829,
  NEG16m_ND = 2830,
  NEG16m_NF = 2831,
  NEG16m_NF_ND = 2832,
  NEG16r = 2833,
  NEG16r_EVEX = 2834,
  NEG16r_ND = 2835,
  NEG16r_NF = 2836,
  NEG16r_NF_ND = 2837,
  NEG32m = 2838,
  NEG32m_EVEX = 2839,
  NEG32m_ND = 2840,
  NEG32m_NF = 2841,
  NEG32m_NF_ND = 2842,
  NEG32r = 2843,
  NEG32r_EVEX = 2844,
  NEG32r_ND = 2845,
  NEG32r_NF = 2846,
  NEG32r_NF_ND = 2847,
  NEG64m = 2848,
  NEG64m_EVEX = 2849,
  NEG64m_ND = 2850,
  NEG64m_NF = 2851,
  NEG64m_NF_ND = 2852,
  NEG64r = 2853,
  NEG64r_EVEX = 2854,
  NEG64r_ND = 2855,
  NEG64r_NF = 2856,
  NEG64r_NF_ND = 2857,
  NEG8m = 2858,
  NEG8m_EVEX = 2859,
  NEG8m_ND = 2860,
  NEG8m_NF = 2861,
  NEG8m_NF_ND = 2862,
  NEG8r = 2863,
  NEG8r_EVEX = 2864,
  NEG8r_ND = 2865,
  NEG8r_NF = 2866,
  NEG8r_NF_ND = 2867,
  NOOP = 2868,
  NOOPL = 2869,
  NOOPLr = 2870,
  NOOPQ = 2871,
  NOOPQr = 2872,
  NOOPW = 2873,
  NOOPWr = 2874,
  NOT16m = 2875,
  NOT16m_EVEX = 2876,
  NOT16m_ND = 2877,
  NOT16r = 2878,
  NOT16r_EVEX = 2879,
  NOT16r_ND = 2880,
  NOT32m = 2881,
  NOT32m_EVEX = 2882,
  NOT32m_ND = 2883,
  NOT32r = 2884,
  NOT32r_EVEX = 2885,
  NOT32r_ND = 2886,
  NOT64m = 2887,
  NOT64m_EVEX = 2888,
  NOT64m_ND = 2889,
  NOT64r = 2890,
  NOT64r_EVEX = 2891,
  NOT64r_ND = 2892,
  NOT8m = 2893,
  NOT8m_EVEX = 2894,
  NOT8m_ND = 2895,
  NOT8r = 2896,
  NOT8r_EVEX = 2897,
  NOT8r_ND = 2898,
  OR16i16 = 2899,
  OR16mi = 2900,
  OR16mi8 = 2901,
  OR16mi8_EVEX = 2902,
  OR16mi8_ND = 2903,
  OR16mi8_NF = 2904,
  OR16mi8_NF_ND = 2905,
  OR16mi_EVEX = 2906,
  OR16mi_ND = 2907,
  OR16mi_NF = 2908,
  OR16mi_NF_ND = 2909,
  OR16mr = 2910,
  OR16mr_EVEX = 2911,
  OR16mr_ND = 2912,
  OR16mr_NF = 2913,
  OR16mr_NF_ND = 2914,
  OR16ri = 2915,
  OR16ri8 = 2916,
  OR16ri8_EVEX = 2917,
  OR16ri8_ND = 2918,
  OR16ri8_NF = 2919,
  OR16ri8_NF_ND = 2920,
  OR16ri_EVEX = 2921,
  OR16ri_ND = 2922,
  OR16ri_NF = 2923,
  OR16ri_NF_ND = 2924,
  OR16rm = 2925,
  OR16rm_EVEX = 2926,
  OR16rm_ND = 2927,
  OR16rm_NF = 2928,
  OR16rm_NF_ND = 2929,
  OR16rr = 2930,
  OR16rr_EVEX = 2931,
  OR16rr_EVEX_REV = 2932,
  OR16rr_ND = 2933,
  OR16rr_ND_REV = 2934,
  OR16rr_NF = 2935,
  OR16rr_NF_ND = 2936,
  OR16rr_NF_ND_REV = 2937,
  OR16rr_NF_REV = 2938,
  OR16rr_REV = 2939,
  OR32i32 = 2940,
  OR32mi = 2941,
  OR32mi8 = 2942,
  OR32mi8Locked = 2943,
  OR32mi8_EVEX = 2944,
  OR32mi8_ND = 2945,
  OR32mi8_NF = 2946,
  OR32mi8_NF_ND = 2947,
  OR32mi_EVEX = 2948,
  OR32mi_ND = 2949,
  OR32mi_NF = 2950,
  OR32mi_NF_ND = 2951,
  OR32mr = 2952,
  OR32mr_EVEX = 2953,
  OR32mr_ND = 2954,
  OR32mr_NF = 2955,
  OR32mr_NF_ND = 2956,
  OR32ri = 2957,
  OR32ri8 = 2958,
  OR32ri8_EVEX = 2959,
  OR32ri8_ND = 2960,
  OR32ri8_NF = 2961,
  OR32ri8_NF_ND = 2962,
  OR32ri_EVEX = 2963,
  OR32ri_ND = 2964,
  OR32ri_NF = 2965,
  OR32ri_NF_ND = 2966,
  OR32rm = 2967,
  OR32rm_EVEX = 2968,
  OR32rm_ND = 2969,
  OR32rm_NF = 2970,
  OR32rm_NF_ND = 2971,
  OR32rr = 2972,
  OR32rr_EVEX = 2973,
  OR32rr_EVEX_REV = 2974,
  OR32rr_ND = 2975,
  OR32rr_ND_REV = 2976,
  OR32rr_NF = 2977,
  OR32rr_NF_ND = 2978,
  OR32rr_NF_ND_REV = 2979,
  OR32rr_NF_REV = 2980,
  OR32rr_REV = 2981,
  OR64i32 = 2982,
  OR64mi32 = 2983,
  OR64mi32_EVEX = 2984,
  OR64mi32_ND = 2985,
  OR64mi32_NF = 2986,
  OR64mi32_NF_ND = 2987,
  OR64mi8 = 2988,
  OR64mi8_EVEX = 2989,
  OR64mi8_ND = 2990,
  OR64mi8_NF = 2991,
  OR64mi8_NF_ND = 2992,
  OR64mr = 2993,
  OR64mr_EVEX = 2994,
  OR64mr_ND = 2995,
  OR64mr_NF = 2996,
  OR64mr_NF_ND = 2997,
  OR64ri32 = 2998,
  OR64ri32_EVEX = 2999,
  OR64ri32_ND = 3000,
  OR64ri32_NF = 3001,
  OR64ri32_NF_ND = 3002,
  OR64ri8 = 3003,
  OR64ri8_EVEX = 3004,
  OR64ri8_ND = 3005,
  OR64ri8_NF = 3006,
  OR64ri8_NF_ND = 3007,
  OR64rm = 3008,
  OR64rm_EVEX = 3009,
  OR64rm_ND = 3010,
  OR64rm_NF = 3011,
  OR64rm_NF_ND = 3012,
  OR64rr = 3013,
  OR64rr_EVEX = 3014,
  OR64rr_EVEX_REV = 3015,
  OR64rr_ND = 3016,
  OR64rr_ND_REV = 3017,
  OR64rr_NF = 3018,
  OR64rr_NF_ND = 3019,
  OR64rr_NF_ND_REV = 3020,
  OR64rr_NF_REV = 3021,
  OR64rr_REV = 3022,
  OR8i8 = 3023,
  OR8mi = 3024,
  OR8mi8 = 3025,
  OR8mi_EVEX = 3026,
  OR8mi_ND = 3027,
  OR8mi_NF = 3028,
  OR8mi_NF_ND = 3029,
  OR8mr = 3030,
  OR8mr_EVEX = 3031,
  OR8mr_ND = 3032,
  OR8mr_NF = 3033,
  OR8mr_NF_ND = 3034,
  OR8ri = 3035,
  OR8ri8 = 3036,
  OR8ri_EVEX = 3037,
  OR8ri_ND = 3038,
  OR8ri_NF = 3039,
  OR8ri_NF_ND = 3040,
  OR8rm = 3041,
  OR8rm_EVEX = 3042,
  OR8rm_ND = 3043,
  OR8rm_NF = 3044,
  OR8rm_NF_ND = 3045,
  OR8rr = 3046,
  OR8rr_EVEX = 3047,
  OR8rr_EVEX_REV = 3048,
  OR8rr_ND = 3049,
  OR8rr_ND_REV = 3050,
  OR8rr_NF = 3051,
  OR8rr_NF_ND = 3052,
  OR8rr_NF_ND_REV = 3053,
  OR8rr_NF_REV = 3054,
  OR8rr_REV = 3055,
  ORPDrm = 3056,
  ORPDrr = 3057,
  ORPSrm = 3058,
  ORPSrr = 3059,
  OUT16ir = 3060,
  OUT16rr = 3061,
  OUT32ir = 3062,
  OUT32rr = 3063,
  OUT8ir = 3064,
  OUT8rr = 3065,
  OUTSB = 3066,
  OUTSL = 3067,
  OUTSW = 3068,
  PABSBrm = 3069,
  PABSBrr = 3070,
  PABSDrm = 3071,
  PABSDrr = 3072,
  PABSWrm = 3073,
  PABSWrr = 3074,
  PACKSSDWrm = 3075,
  PACKSSDWrr = 3076,
  PACKSSWBrm = 3077,
  PACKSSWBrr = 3078,
  PACKUSDWrm = 3079,
  PACKUSDWrr = 3080,
  PACKUSWBrm = 3081,
  PACKUSWBrr = 3082,
  PADDBrm = 3083,
  PADDBrr = 3084,
  PADDDrm = 3085,
  PADDDrr = 3086,
  PADDQrm = 3087,
  PADDQrr = 3088,
  PADDSBrm = 3089,
  PADDSBrr = 3090,
  PADDSWrm = 3091,
  PADDSWrr = 3092,
  PADDUSBrm = 3093,
  PADDUSBrr = 3094,
  PADDUSWrm = 3095,
  PADDUSWrr = 3096,
  PADDWrm = 3097,
  PADDWrr = 3098,
  PALIGNRrmi = 3099,
  PALIGNRrri = 3100,
  PANDNrm = 3101,
  PANDNrr = 3102,
  PANDrm = 3103,
  PANDrr = 3104,
  PAUSE = 3105,
  PAVGBrm = 3106,
  PAVGBrr = 3107,
  PAVGUSBrm = 3108,
  PAVGUSBrr = 3109,
  PAVGWrm = 3110,
  PAVGWrr = 3111,
  PBLENDVBrm0 = 3112,
  PBLENDVBrr0 = 3113,
  PBLENDWrmi = 3114,
  PBLENDWrri = 3115,
  PBNDKB = 3116,
  PCLMULQDQrmi = 3117,
  PCLMULQDQrri = 3118,
  PCMPEQBrm = 3119,
  PCMPEQBrr = 3120,
  PCMPEQDrm = 3121,
  PCMPEQDrr = 3122,
  PCMPEQQrm = 3123,
  PCMPEQQrr = 3124,
  PCMPEQWrm = 3125,
  PCMPEQWrr = 3126,
  PCMPESTRIrmi = 3127,
  PCMPESTRIrri = 3128,
  PCMPESTRMrmi = 3129,
  PCMPESTRMrri = 3130,
  PCMPGTBrm = 3131,
  PCMPGTBrr = 3132,
  PCMPGTDrm = 3133,
  PCMPGTDrr = 3134,
  PCMPGTQrm = 3135,
  PCMPGTQrr = 3136,
  PCMPGTWrm = 3137,
  PCMPGTWrr = 3138,
  PCMPISTRIrmi = 3139,
  PCMPISTRIrri = 3140,
  PCMPISTRMrmi = 3141,
  PCMPISTRMrri = 3142,
  PCONFIG = 3143,
  PDEP32rm = 3144,
  PDEP32rm_EVEX = 3145,
  PDEP32rr = 3146,
  PDEP32rr_EVEX = 3147,
  PDEP64rm = 3148,
  PDEP64rm_EVEX = 3149,
  PDEP64rr = 3150,
  PDEP64rr_EVEX = 3151,
  PEXT32rm = 3152,
  PEXT32rm_EVEX = 3153,
  PEXT32rr = 3154,
  PEXT32rr_EVEX = 3155,
  PEXT64rm = 3156,
  PEXT64rm_EVEX = 3157,
  PEXT64rr = 3158,
  PEXT64rr_EVEX = 3159,
  PEXTRBmri = 3160,
  PEXTRBrri = 3161,
  PEXTRDmri = 3162,
  PEXTRDrri = 3163,
  PEXTRQmri = 3164,
  PEXTRQrri = 3165,
  PEXTRWmri = 3166,
  PEXTRWrri = 3167,
  PEXTRWrri_REV = 3168,
  PF2IDrm = 3169,
  PF2IDrr = 3170,
  PF2IWrm = 3171,
  PF2IWrr = 3172,
  PFACCrm = 3173,
  PFACCrr = 3174,
  PFADDrm = 3175,
  PFADDrr = 3176,
  PFCMPEQrm = 3177,
  PFCMPEQrr = 3178,
  PFCMPGErm = 3179,
  PFCMPGErr = 3180,
  PFCMPGTrm = 3181,
  PFCMPGTrr = 3182,
  PFMAXrm = 3183,
  PFMAXrr = 3184,
  PFMINrm = 3185,
  PFMINrr = 3186,
  PFMULrm = 3187,
  PFMULrr = 3188,
  PFNACCrm = 3189,
  PFNACCrr = 3190,
  PFPNACCrm = 3191,
  PFPNACCrr = 3192,
  PFRCPIT1rm = 3193,
  PFRCPIT1rr = 3194,
  PFRCPIT2rm = 3195,
  PFRCPIT2rr = 3196,
  PFRCPrm = 3197,
  PFRCPrr = 3198,
  PFRSQIT1rm = 3199,
  PFRSQIT1rr = 3200,
  PFRSQRTrm = 3201,
  PFRSQRTrr = 3202,
  PFSUBRrm = 3203,
  PFSUBRrr = 3204,
  PFSUBrm = 3205,
  PFSUBrr = 3206,
  PHADDDrm = 3207,
  PHADDDrr = 3208,
  PHADDSWrm = 3209,
  PHADDSWrr = 3210,
  PHADDWrm = 3211,
  PHADDWrr = 3212,
  PHMINPOSUWrm = 3213,
  PHMINPOSUWrr = 3214,
  PHSUBDrm = 3215,
  PHSUBDrr = 3216,
  PHSUBSWrm = 3217,
  PHSUBSWrr = 3218,
  PHSUBWrm = 3219,
  PHSUBWrr = 3220,
  PI2FDrm = 3221,
  PI2FDrr = 3222,
  PI2FWrm = 3223,
  PI2FWrr = 3224,
  PINSRBrmi = 3225,
  PINSRBrri = 3226,
  PINSRDrmi = 3227,
  PINSRDrri = 3228,
  PINSRQrmi = 3229,
  PINSRQrri = 3230,
  PINSRWrmi = 3231,
  PINSRWrri = 3232,
  PMADDUBSWrm = 3233,
  PMADDUBSWrr = 3234,
  PMADDWDrm = 3235,
  PMADDWDrr = 3236,
  PMAXSBrm = 3237,
  PMAXSBrr = 3238,
  PMAXSDrm = 3239,
  PMAXSDrr = 3240,
  PMAXSWrm = 3241,
  PMAXSWrr = 3242,
  PMAXUBrm = 3243,
  PMAXUBrr = 3244,
  PMAXUDrm = 3245,
  PMAXUDrr = 3246,
  PMAXUWrm = 3247,
  PMAXUWrr = 3248,
  PMINSBrm = 3249,
  PMINSBrr = 3250,
  PMINSDrm = 3251,
  PMINSDrr = 3252,
  PMINSWrm = 3253,
  PMINSWrr = 3254,
  PMINUBrm = 3255,
  PMINUBrr = 3256,
  PMINUDrm = 3257,
  PMINUDrr = 3258,
  PMINUWrm = 3259,
  PMINUWrr = 3260,
  PMOVMSKBrr = 3261,
  PMOVSXBDrm = 3262,
  PMOVSXBDrr = 3263,
  PMOVSXBQrm = 3264,
  PMOVSXBQrr = 3265,
  PMOVSXBWrm = 3266,
  PMOVSXBWrr = 3267,
  PMOVSXDQrm = 3268,
  PMOVSXDQrr = 3269,
  PMOVSXWDrm = 3270,
  PMOVSXWDrr = 3271,
  PMOVSXWQrm = 3272,
  PMOVSXWQrr = 3273,
  PMOVZXBDrm = 3274,
  PMOVZXBDrr = 3275,
  PMOVZXBQrm = 3276,
  PMOVZXBQrr = 3277,
  PMOVZXBWrm = 3278,
  PMOVZXBWrr = 3279,
  PMOVZXDQrm = 3280,
  PMOVZXDQrr = 3281,
  PMOVZXWDrm = 3282,
  PMOVZXWDrr = 3283,
  PMOVZXWQrm = 3284,
  PMOVZXWQrr = 3285,
  PMULDQrm = 3286,
  PMULDQrr = 3287,
  PMULHRSWrm = 3288,
  PMULHRSWrr = 3289,
  PMULHRWrm = 3290,
  PMULHRWrr = 3291,
  PMULHUWrm = 3292,
  PMULHUWrr = 3293,
  PMULHWrm = 3294,
  PMULHWrr = 3295,
  PMULLDrm = 3296,
  PMULLDrr = 3297,
  PMULLWrm = 3298,
  PMULLWrr = 3299,
  PMULUDQrm = 3300,
  PMULUDQrr = 3301,
  POP16r = 3302,
  POP16rmm = 3303,
  POP16rmr = 3304,
  POP2 = 3305,
  POP2P = 3306,
  POP32r = 3307,
  POP32rmm = 3308,
  POP32rmr = 3309,
  POP64r = 3310,
  POP64rmm = 3311,
  POP64rmr = 3312,
  POPA16 = 3313,
  POPA32 = 3314,
  POPCNT16rm = 3315,
  POPCNT16rm_EVEX = 3316,
  POPCNT16rm_NF = 3317,
  POPCNT16rr = 3318,
  POPCNT16rr_EVEX = 3319,
  POPCNT16rr_NF = 3320,
  POPCNT32rm = 3321,
  POPCNT32rm_EVEX = 3322,
  POPCNT32rm_NF = 3323,
  POPCNT32rr = 3324,
  POPCNT32rr_EVEX = 3325,
  POPCNT32rr_NF = 3326,
  POPCNT64rm = 3327,
  POPCNT64rm_EVEX = 3328,
  POPCNT64rm_NF = 3329,
  POPCNT64rr = 3330,
  POPCNT64rr_EVEX = 3331,
  POPCNT64rr_NF = 3332,
  POPDS16 = 3333,
  POPDS32 = 3334,
  POPES16 = 3335,
  POPES32 = 3336,
  POPF16 = 3337,
  POPF32 = 3338,
  POPF64 = 3339,
  POPFS16 = 3340,
  POPFS32 = 3341,
  POPFS64 = 3342,
  POPGS16 = 3343,
  POPGS32 = 3344,
  POPGS64 = 3345,
  POPP64r = 3346,
  POPSS16 = 3347,
  POPSS32 = 3348,
  PORrm = 3349,
  PORrr = 3350,
  PREFETCH = 3351,
  PREFETCHIT0 = 3352,
  PREFETCHIT1 = 3353,
  PREFETCHNTA = 3354,
  PREFETCHRST2 = 3355,
  PREFETCHT0 = 3356,
  PREFETCHT1 = 3357,
  PREFETCHT2 = 3358,
  PREFETCHW = 3359,
  PREFETCHWT1 = 3360,
  PROBED_ALLOCA_32 = 3361,
  PROBED_ALLOCA_64 = 3362,
  PSADBWrm = 3363,
  PSADBWrr = 3364,
  PSHUFBrm = 3365,
  PSHUFBrr = 3366,
  PSHUFDmi = 3367,
  PSHUFDri = 3368,
  PSHUFHWmi = 3369,
  PSHUFHWri = 3370,
  PSHUFLWmi = 3371,
  PSHUFLWri = 3372,
  PSIGNBrm = 3373,
  PSIGNBrr = 3374,
  PSIGNDrm = 3375,
  PSIGNDrr = 3376,
  PSIGNWrm = 3377,
  PSIGNWrr = 3378,
  PSLLDQri = 3379,
  PSLLDri = 3380,
  PSLLDrm = 3381,
  PSLLDrr = 3382,
  PSLLQri = 3383,
  PSLLQrm = 3384,
  PSLLQrr = 3385,
  PSLLWri = 3386,
  PSLLWrm = 3387,
  PSLLWrr = 3388,
  PSMASH = 3389,
  PSRADri = 3390,
  PSRADrm = 3391,
  PSRADrr = 3392,
  PSRAWri = 3393,
  PSRAWrm = 3394,
  PSRAWrr = 3395,
  PSRLDQri = 3396,
  PSRLDri = 3397,
  PSRLDrm = 3398,
  PSRLDrr = 3399,
  PSRLQri = 3400,
  PSRLQrm = 3401,
  PSRLQrr = 3402,
  PSRLWri = 3403,
  PSRLWrm = 3404,
  PSRLWrr = 3405,
  PSUBBrm = 3406,
  PSUBBrr = 3407,
  PSUBDrm = 3408,
  PSUBDrr = 3409,
  PSUBQrm = 3410,
  PSUBQrr = 3411,
  PSUBSBrm = 3412,
  PSUBSBrr = 3413,
  PSUBSWrm = 3414,
  PSUBSWrr = 3415,
  PSUBUSBrm = 3416,
  PSUBUSBrr = 3417,
  PSUBUSWrm = 3418,
  PSUBUSWrr = 3419,
  PSUBWrm = 3420,
  PSUBWrr = 3421,
  PSWAPDrm = 3422,
  PSWAPDrr = 3423,
  PT2RPNTLVWZ0 = 3424,
  PT2RPNTLVWZ0RS = 3425,
  PT2RPNTLVWZ0RST1 = 3426,
  PT2RPNTLVWZ0T1 = 3427,
  PT2RPNTLVWZ1 = 3428,
  PT2RPNTLVWZ1RS = 3429,
  PT2RPNTLVWZ1RST1 = 3430,
  PT2RPNTLVWZ1T1 = 3431,
  PTCMMIMFP16PS = 3432,
  PTCMMIMFP16PSV = 3433,
  PTCMMRLFP16PS = 3434,
  PTCMMRLFP16PSV = 3435,
  PTCONJTCMMIMFP16PS = 3436,
  PTCONJTCMMIMFP16PSV = 3437,
  PTCONJTFP16 = 3438,
  PTCONJTFP16V = 3439,
  PTCVTROWD2PSrre = 3440,
  PTCVTROWD2PSrreV = 3441,
  PTCVTROWD2PSrri = 3442,
  PTCVTROWD2PSrriV = 3443,
  PTCVTROWPS2BF16Hrre = 3444,
  PTCVTROWPS2BF16HrreV = 3445,
  PTCVTROWPS2BF16Hrri = 3446,
  PTCVTROWPS2BF16HrriV = 3447,
  PTCVTROWPS2BF16Lrre = 3448,
  PTCVTROWPS2BF16LrreV = 3449,
  PTCVTROWPS2BF16Lrri = 3450,
  PTCVTROWPS2BF16LrriV = 3451,
  PTCVTROWPS2PHHrre = 3452,
  PTCVTROWPS2PHHrreV = 3453,
  PTCVTROWPS2PHHrri = 3454,
  PTCVTROWPS2PHHrriV = 3455,
  PTCVTROWPS2PHLrre = 3456,
  PTCVTROWPS2PHLrreV = 3457,
  PTCVTROWPS2PHLrri = 3458,
  PTCVTROWPS2PHLrriV = 3459,
  PTDPBF16PS = 3460,
  PTDPBF8PS = 3461,
  PTDPBF8PSV = 3462,
  PTDPBHF8PS = 3463,
  PTDPBHF8PSV = 3464,
  PTDPBSSD = 3465,
  PTDPBSUD = 3466,
  PTDPBUSD = 3467,
  PTDPBUUD = 3468,
  PTDPFP16PS = 3469,
  PTDPHBF8PS = 3470,
  PTDPHBF8PSV = 3471,
  PTDPHF8PS = 3472,
  PTDPHF8PSV = 3473,
  PTESTrm = 3474,
  PTESTrr = 3475,
  PTILELOADD = 3476,
  PTILELOADDRS = 3477,
  PTILELOADDRST1 = 3478,
  PTILELOADDT1 = 3479,
  PTILEMOVROWrre = 3480,
  PTILEMOVROWrreV = 3481,
  PTILEMOVROWrri = 3482,
  PTILEMOVROWrriV = 3483,
  PTILESTORED = 3484,
  PTILEZERO = 3485,
  PTMMULTF32PS = 3486,
  PTMMULTF32PSV = 3487,
  PTTCMMIMFP16PS = 3488,
  PTTCMMIMFP16PSV = 3489,
  PTTCMMRLFP16PS = 3490,
  PTTCMMRLFP16PSV = 3491,
  PTTDPBF16PS = 3492,
  PTTDPBF16PSV = 3493,
  PTTDPFP16PS = 3494,
  PTTDPFP16PSV = 3495,
  PTTMMULTF32PS = 3496,
  PTTMMULTF32PSV = 3497,
  PTTRANSPOSED = 3498,
  PTTRANSPOSEDV = 3499,
  PTWRITE64m = 3500,
  PTWRITE64r = 3501,
  PTWRITEm = 3502,
  PTWRITEr = 3503,
  PUNPCKHBWrm = 3504,
  PUNPCKHBWrr = 3505,
  PUNPCKHDQrm = 3506,
  PUNPCKHDQrr = 3507,
  PUNPCKHQDQrm = 3508,
  PUNPCKHQDQrr = 3509,
  PUNPCKHWDrm = 3510,
  PUNPCKHWDrr = 3511,
  PUNPCKLBWrm = 3512,
  PUNPCKLBWrr = 3513,
  PUNPCKLDQrm = 3514,
  PUNPCKLDQrr = 3515,
  PUNPCKLQDQrm = 3516,
  PUNPCKLQDQrr = 3517,
  PUNPCKLWDrm = 3518,
  PUNPCKLWDrr = 3519,
  PUSH16i = 3520,
  PUSH16i8 = 3521,
  PUSH16r = 3522,
  PUSH16rmm = 3523,
  PUSH16rmr = 3524,
  PUSH2 = 3525,
  PUSH2P = 3526,
  PUSH32i = 3527,
  PUSH32i8 = 3528,
  PUSH32r = 3529,
  PUSH32rmm = 3530,
  PUSH32rmr = 3531,
  PUSH64i32 = 3532,
  PUSH64i8 = 3533,
  PUSH64r = 3534,
  PUSH64rmm = 3535,
  PUSH64rmr = 3536,
  PUSHA16 = 3537,
  PUSHA32 = 3538,
  PUSHCS16 = 3539,
  PUSHCS32 = 3540,
  PUSHDS16 = 3541,
  PUSHDS32 = 3542,
  PUSHES16 = 3543,
  PUSHES32 = 3544,
  PUSHF16 = 3545,
  PUSHF32 = 3546,
  PUSHF64 = 3547,
  PUSHFS16 = 3548,
  PUSHFS32 = 3549,
  PUSHFS64 = 3550,
  PUSHGS16 = 3551,
  PUSHGS32 = 3552,
  PUSHGS64 = 3553,
  PUSHP64r = 3554,
  PUSHSS16 = 3555,
  PUSHSS32 = 3556,
  PVALIDATE32 = 3557,
  PVALIDATE64 = 3558,
  PXORrm = 3559,
  PXORrr = 3560,
  RCL16m1 = 3561,
  RCL16m1_EVEX = 3562,
  RCL16m1_ND = 3563,
  RCL16mCL = 3564,
  RCL16mCL_EVEX = 3565,
  RCL16mCL_ND = 3566,
  RCL16mi = 3567,
  RCL16mi_EVEX = 3568,
  RCL16mi_ND = 3569,
  RCL16r1 = 3570,
  RCL16r1_EVEX = 3571,
  RCL16r1_ND = 3572,
  RCL16rCL = 3573,
  RCL16rCL_EVEX = 3574,
  RCL16rCL_ND = 3575,
  RCL16ri = 3576,
  RCL16ri_EVEX = 3577,
  RCL16ri_ND = 3578,
  RCL32m1 = 3579,
  RCL32m1_EVEX = 3580,
  RCL32m1_ND = 3581,
  RCL32mCL = 3582,
  RCL32mCL_EVEX = 3583,
  RCL32mCL_ND = 3584,
  RCL32mi = 3585,
  RCL32mi_EVEX = 3586,
  RCL32mi_ND = 3587,
  RCL32r1 = 3588,
  RCL32r1_EVEX = 3589,
  RCL32r1_ND = 3590,
  RCL32rCL = 3591,
  RCL32rCL_EVEX = 3592,
  RCL32rCL_ND = 3593,
  RCL32ri = 3594,
  RCL32ri_EVEX = 3595,
  RCL32ri_ND = 3596,
  RCL64m1 = 3597,
  RCL64m1_EVEX = 3598,
  RCL64m1_ND = 3599,
  RCL64mCL = 3600,
  RCL64mCL_EVEX = 3601,
  RCL64mCL_ND = 3602,
  RCL64mi = 3603,
  RCL64mi_EVEX = 3604,
  RCL64mi_ND = 3605,
  RCL64r1 = 3606,
  RCL64r1_EVEX = 3607,
  RCL64r1_ND = 3608,
  RCL64rCL = 3609,
  RCL64rCL_EVEX = 3610,
  RCL64rCL_ND = 3611,
  RCL64ri = 3612,
  RCL64ri_EVEX = 3613,
  RCL64ri_ND = 3614,
  RCL8m1 = 3615,
  RCL8m1_EVEX = 3616,
  RCL8m1_ND = 3617,
  RCL8mCL = 3618,
  RCL8mCL_EVEX = 3619,
  RCL8mCL_ND = 3620,
  RCL8mi = 3621,
  RCL8mi_EVEX = 3622,
  RCL8mi_ND = 3623,
  RCL8r1 = 3624,
  RCL8r1_EVEX = 3625,
  RCL8r1_ND = 3626,
  RCL8rCL = 3627,
  RCL8rCL_EVEX = 3628,
  RCL8rCL_ND = 3629,
  RCL8ri = 3630,
  RCL8ri_EVEX = 3631,
  RCL8ri_ND = 3632,
  RCPPSm = 3633,
  RCPPSr = 3634,
  RCPSSm = 3635,
  RCPSSm_Int = 3636,
  RCPSSr = 3637,
  RCPSSr_Int = 3638,
  RCR16m1 = 3639,
  RCR16m1_EVEX = 3640,
  RCR16m1_ND = 3641,
  RCR16mCL = 3642,
  RCR16mCL_EVEX = 3643,
  RCR16mCL_ND = 3644,
  RCR16mi = 3645,
  RCR16mi_EVEX = 3646,
  RCR16mi_ND = 3647,
  RCR16r1 = 3648,
  RCR16r1_EVEX = 3649,
  RCR16r1_ND = 3650,
  RCR16rCL = 3651,
  RCR16rCL_EVEX = 3652,
  RCR16rCL_ND = 3653,
  RCR16ri = 3654,
  RCR16ri_EVEX = 3655,
  RCR16ri_ND = 3656,
  RCR32m1 = 3657,
  RCR32m1_EVEX = 3658,
  RCR32m1_ND = 3659,
  RCR32mCL = 3660,
  RCR32mCL_EVEX = 3661,
  RCR32mCL_ND = 3662,
  RCR32mi = 3663,
  RCR32mi_EVEX = 3664,
  RCR32mi_ND = 3665,
  RCR32r1 = 3666,
  RCR32r1_EVEX = 3667,
  RCR32r1_ND = 3668,
  RCR32rCL = 3669,
  RCR32rCL_EVEX = 3670,
  RCR32rCL_ND = 3671,
  RCR32ri = 3672,
  RCR32ri_EVEX = 3673,
  RCR32ri_ND = 3674,
  RCR64m1 = 3675,
  RCR64m1_EVEX = 3676,
  RCR64m1_ND = 3677,
  RCR64mCL = 3678,
  RCR64mCL_EVEX = 3679,
  RCR64mCL_ND = 3680,
  RCR64mi = 3681,
  RCR64mi_EVEX = 3682,
  RCR64mi_ND = 3683,
  RCR64r1 = 3684,
  RCR64r1_EVEX = 3685,
  RCR64r1_ND = 3686,
  RCR64rCL = 3687,
  RCR64rCL_EVEX = 3688,
  RCR64rCL_ND = 3689,
  RCR64ri = 3690,
  RCR64ri_EVEX = 3691,
  RCR64ri_ND = 3692,
  RCR8m1 = 3693,
  RCR8m1_EVEX = 3694,
  RCR8m1_ND = 3695,
  RCR8mCL = 3696,
  RCR8mCL_EVEX = 3697,
  RCR8mCL_ND = 3698,
  RCR8mi = 3699,
  RCR8mi_EVEX = 3700,
  RCR8mi_ND = 3701,
  RCR8r1 = 3702,
  RCR8r1_EVEX = 3703,
  RCR8r1_ND = 3704,
  RCR8rCL = 3705,
  RCR8rCL_EVEX = 3706,
  RCR8rCL_ND = 3707,
  RCR8ri = 3708,
  RCR8ri_EVEX = 3709,
  RCR8ri_ND = 3710,
  RDFSBASE = 3711,
  RDFSBASE64 = 3712,
  RDGSBASE = 3713,
  RDGSBASE64 = 3714,
  RDMSR = 3715,
  RDMSRLIST = 3716,
  RDMSRri = 3717,
  RDMSRri_EVEX = 3718,
  RDPID32 = 3719,
  RDPID64 = 3720,
  RDPKRUr = 3721,
  RDPMC = 3722,
  RDPRU = 3723,
  RDRAND16r = 3724,
  RDRAND32r = 3725,
  RDRAND64r = 3726,
  RDSEED16r = 3727,
  RDSEED32r = 3728,
  RDSEED64r = 3729,
  RDSSPD = 3730,
  RDSSPQ = 3731,
  RDTSC = 3732,
  RDTSCP = 3733,
  REPNE_PREFIX = 3734,
  REP_MOVSB_32 = 3735,
  REP_MOVSB_64 = 3736,
  REP_MOVSD_32 = 3737,
  REP_MOVSD_64 = 3738,
  REP_MOVSQ_32 = 3739,
  REP_MOVSQ_64 = 3740,
  REP_MOVSW_32 = 3741,
  REP_MOVSW_64 = 3742,
  REP_PREFIX = 3743,
  REP_STOSB_32 = 3744,
  REP_STOSB_64 = 3745,
  REP_STOSD_32 = 3746,
  REP_STOSD_64 = 3747,
  REP_STOSQ_32 = 3748,
  REP_STOSQ_64 = 3749,
  REP_STOSW_32 = 3750,
  REP_STOSW_64 = 3751,
  RET = 3752,
  RET16 = 3753,
  RET32 = 3754,
  RET64 = 3755,
  RETI16 = 3756,
  RETI32 = 3757,
  RETI64 = 3758,
  REX64_PREFIX = 3759,
  RMPADJUST = 3760,
  RMPQUERY = 3761,
  RMPUPDATE = 3762,
  ROL16m1 = 3763,
  ROL16m1_EVEX = 3764,
  ROL16m1_ND = 3765,
  ROL16m1_NF = 3766,
  ROL16m1_NF_ND = 3767,
  ROL16mCL = 3768,
  ROL16mCL_EVEX = 3769,
  ROL16mCL_ND = 3770,
  ROL16mCL_NF = 3771,
  ROL16mCL_NF_ND = 3772,
  ROL16mi = 3773,
  ROL16mi_EVEX = 3774,
  ROL16mi_ND = 3775,
  ROL16mi_NF = 3776,
  ROL16mi_NF_ND = 3777,
  ROL16r1 = 3778,
  ROL16r1_EVEX = 3779,
  ROL16r1_ND = 3780,
  ROL16r1_NF = 3781,
  ROL16r1_NF_ND = 3782,
  ROL16rCL = 3783,
  ROL16rCL_EVEX = 3784,
  ROL16rCL_ND = 3785,
  ROL16rCL_NF = 3786,
  ROL16rCL_NF_ND = 3787,
  ROL16ri = 3788,
  ROL16ri_EVEX = 3789,
  ROL16ri_ND = 3790,
  ROL16ri_NF = 3791,
  ROL16ri_NF_ND = 3792,
  ROL32m1 = 3793,
  ROL32m1_EVEX = 3794,
  ROL32m1_ND = 3795,
  ROL32m1_NF = 3796,
  ROL32m1_NF_ND = 3797,
  ROL32mCL = 3798,
  ROL32mCL_EVEX = 3799,
  ROL32mCL_ND = 3800,
  ROL32mCL_NF = 3801,
  ROL32mCL_NF_ND = 3802,
  ROL32mi = 3803,
  ROL32mi_EVEX = 3804,
  ROL32mi_ND = 3805,
  ROL32mi_NF = 3806,
  ROL32mi_NF_ND = 3807,
  ROL32r1 = 3808,
  ROL32r1_EVEX = 3809,
  ROL32r1_ND = 3810,
  ROL32r1_NF = 3811,
  ROL32r1_NF_ND = 3812,
  ROL32rCL = 3813,
  ROL32rCL_EVEX = 3814,
  ROL32rCL_ND = 3815,
  ROL32rCL_NF = 3816,
  ROL32rCL_NF_ND = 3817,
  ROL32ri = 3818,
  ROL32ri_EVEX = 3819,
  ROL32ri_ND = 3820,
  ROL32ri_NF = 3821,
  ROL32ri_NF_ND = 3822,
  ROL64m1 = 3823,
  ROL64m1_EVEX = 3824,
  ROL64m1_ND = 3825,
  ROL64m1_NF = 3826,
  ROL64m1_NF_ND = 3827,
  ROL64mCL = 3828,
  ROL64mCL_EVEX = 3829,
  ROL64mCL_ND = 3830,
  ROL64mCL_NF = 3831,
  ROL64mCL_NF_ND = 3832,
  ROL64mi = 3833,
  ROL64mi_EVEX = 3834,
  ROL64mi_ND = 3835,
  ROL64mi_NF = 3836,
  ROL64mi_NF_ND = 3837,
  ROL64r1 = 3838,
  ROL64r1_EVEX = 3839,
  ROL64r1_ND = 3840,
  ROL64r1_NF = 3841,
  ROL64r1_NF_ND = 3842,
  ROL64rCL = 3843,
  ROL64rCL_EVEX = 3844,
  ROL64rCL_ND = 3845,
  ROL64rCL_NF = 3846,
  ROL64rCL_NF_ND = 3847,
  ROL64ri = 3848,
  ROL64ri_EVEX = 3849,
  ROL64ri_ND = 3850,
  ROL64ri_NF = 3851,
  ROL64ri_NF_ND = 3852,
  ROL8m1 = 3853,
  ROL8m1_EVEX = 3854,
  ROL8m1_ND = 3855,
  ROL8m1_NF = 3856,
  ROL8m1_NF_ND = 3857,
  ROL8mCL = 3858,
  ROL8mCL_EVEX = 3859,
  ROL8mCL_ND = 3860,
  ROL8mCL_NF = 3861,
  ROL8mCL_NF_ND = 3862,
  ROL8mi = 3863,
  ROL8mi_EVEX = 3864,
  ROL8mi_ND = 3865,
  ROL8mi_NF = 3866,
  ROL8mi_NF_ND = 3867,
  ROL8r1 = 3868,
  ROL8r1_EVEX = 3869,
  ROL8r1_ND = 3870,
  ROL8r1_NF = 3871,
  ROL8r1_NF_ND = 3872,
  ROL8rCL = 3873,
  ROL8rCL_EVEX = 3874,
  ROL8rCL_ND = 3875,
  ROL8rCL_NF = 3876,
  ROL8rCL_NF_ND = 3877,
  ROL8ri = 3878,
  ROL8ri_EVEX = 3879,
  ROL8ri_ND = 3880,
  ROL8ri_NF = 3881,
  ROL8ri_NF_ND = 3882,
  ROR16m1 = 3883,
  ROR16m1_EVEX = 3884,
  ROR16m1_ND = 3885,
  ROR16m1_NF = 3886,
  ROR16m1_NF_ND = 3887,
  ROR16mCL = 3888,
  ROR16mCL_EVEX = 3889,
  ROR16mCL_ND = 3890,
  ROR16mCL_NF = 3891,
  ROR16mCL_NF_ND = 3892,
  ROR16mi = 3893,
  ROR16mi_EVEX = 3894,
  ROR16mi_ND = 3895,
  ROR16mi_NF = 3896,
  ROR16mi_NF_ND = 3897,
  ROR16r1 = 3898,
  ROR16r1_EVEX = 3899,
  ROR16r1_ND = 3900,
  ROR16r1_NF = 3901,
  ROR16r1_NF_ND = 3902,
  ROR16rCL = 3903,
  ROR16rCL_EVEX = 3904,
  ROR16rCL_ND = 3905,
  ROR16rCL_NF = 3906,
  ROR16rCL_NF_ND = 3907,
  ROR16ri = 3908,
  ROR16ri_EVEX = 3909,
  ROR16ri_ND = 3910,
  ROR16ri_NF = 3911,
  ROR16ri_NF_ND = 3912,
  ROR32m1 = 3913,
  ROR32m1_EVEX = 3914,
  ROR32m1_ND = 3915,
  ROR32m1_NF = 3916,
  ROR32m1_NF_ND = 3917,
  ROR32mCL = 3918,
  ROR32mCL_EVEX = 3919,
  ROR32mCL_ND = 3920,
  ROR32mCL_NF = 3921,
  ROR32mCL_NF_ND = 3922,
  ROR32mi = 3923,
  ROR32mi_EVEX = 3924,
  ROR32mi_ND = 3925,
  ROR32mi_NF = 3926,
  ROR32mi_NF_ND = 3927,
  ROR32r1 = 3928,
  ROR32r1_EVEX = 3929,
  ROR32r1_ND = 3930,
  ROR32r1_NF = 3931,
  ROR32r1_NF_ND = 3932,
  ROR32rCL = 3933,
  ROR32rCL_EVEX = 3934,
  ROR32rCL_ND = 3935,
  ROR32rCL_NF = 3936,
  ROR32rCL_NF_ND = 3937,
  ROR32ri = 3938,
  ROR32ri_EVEX = 3939,
  ROR32ri_ND = 3940,
  ROR32ri_NF = 3941,
  ROR32ri_NF_ND = 3942,
  ROR64m1 = 3943,
  ROR64m1_EVEX = 3944,
  ROR64m1_ND = 3945,
  ROR64m1_NF = 3946,
  ROR64m1_NF_ND = 3947,
  ROR64mCL = 3948,
  ROR64mCL_EVEX = 3949,
  ROR64mCL_ND = 3950,
  ROR64mCL_NF = 3951,
  ROR64mCL_NF_ND = 3952,
  ROR64mi = 3953,
  ROR64mi_EVEX = 3954,
  ROR64mi_ND = 3955,
  ROR64mi_NF = 3956,
  ROR64mi_NF_ND = 3957,
  ROR64r1 = 3958,
  ROR64r1_EVEX = 3959,
  ROR64r1_ND = 3960,
  ROR64r1_NF = 3961,
  ROR64r1_NF_ND = 3962,
  ROR64rCL = 3963,
  ROR64rCL_EVEX = 3964,
  ROR64rCL_ND = 3965,
  ROR64rCL_NF = 3966,
  ROR64rCL_NF_ND = 3967,
  ROR64ri = 3968,
  ROR64ri_EVEX = 3969,
  ROR64ri_ND = 3970,
  ROR64ri_NF = 3971,
  ROR64ri_NF_ND = 3972,
  ROR8m1 = 3973,
  ROR8m1_EVEX = 3974,
  ROR8m1_ND = 3975,
  ROR8m1_NF = 3976,
  ROR8m1_NF_ND = 3977,
  ROR8mCL = 3978,
  ROR8mCL_EVEX = 3979,
  ROR8mCL_ND = 3980,
  ROR8mCL_NF = 3981,
  ROR8mCL_NF_ND = 3982,
  ROR8mi = 3983,
  ROR8mi_EVEX = 3984,
  ROR8mi_ND = 3985,
  ROR8mi_NF = 3986,
  ROR8mi_NF_ND = 3987,
  ROR8r1 = 3988,
  ROR8r1_EVEX = 3989,
  ROR8r1_ND = 3990,
  ROR8r1_NF = 3991,
  ROR8r1_NF_ND = 3992,
  ROR8rCL = 3993,
  ROR8rCL_EVEX = 3994,
  ROR8rCL_ND = 3995,
  ROR8rCL_NF = 3996,
  ROR8rCL_NF_ND = 3997,
  ROR8ri = 3998,
  ROR8ri_EVEX = 3999,
  ROR8ri_ND = 4000,
  ROR8ri_NF = 4001,
  ROR8ri_NF_ND = 4002,
  RORX32mi = 4003,
  RORX32mi_EVEX = 4004,
  RORX32ri = 4005,
  RORX32ri_EVEX = 4006,
  RORX64mi = 4007,
  RORX64mi_EVEX = 4008,
  RORX64ri = 4009,
  RORX64ri_EVEX = 4010,
  ROUNDPDmi = 4011,
  ROUNDPDri = 4012,
  ROUNDPSmi = 4013,
  ROUNDPSri = 4014,
  ROUNDSDmi = 4015,
  ROUNDSDmi_Int = 4016,
  ROUNDSDri = 4017,
  ROUNDSDri_Int = 4018,
  ROUNDSSmi = 4019,
  ROUNDSSmi_Int = 4020,
  ROUNDSSri = 4021,
  ROUNDSSri_Int = 4022,
  RSM = 4023,
  RSQRTPSm = 4024,
  RSQRTPSr = 4025,
  RSQRTSSm = 4026,
  RSQRTSSm_Int = 4027,
  RSQRTSSr = 4028,
  RSQRTSSr_Int = 4029,
  RSTORSSP = 4030,
  SAHF = 4031,
  SALC = 4032,
  SAR16m1 = 4033,
  SAR16m1_EVEX = 4034,
  SAR16m1_ND = 4035,
  SAR16m1_NF = 4036,
  SAR16m1_NF_ND = 4037,
  SAR16mCL = 4038,
  SAR16mCL_EVEX = 4039,
  SAR16mCL_ND = 4040,
  SAR16mCL_NF = 4041,
  SAR16mCL_NF_ND = 4042,
  SAR16mi = 4043,
  SAR16mi_EVEX = 4044,
  SAR16mi_ND = 4045,
  SAR16mi_NF = 4046,
  SAR16mi_NF_ND = 4047,
  SAR16r1 = 4048,
  SAR16r1_EVEX = 4049,
  SAR16r1_ND = 4050,
  SAR16r1_NF = 4051,
  SAR16r1_NF_ND = 4052,
  SAR16rCL = 4053,
  SAR16rCL_EVEX = 4054,
  SAR16rCL_ND = 4055,
  SAR16rCL_NF = 4056,
  SAR16rCL_NF_ND = 4057,
  SAR16ri = 4058,
  SAR16ri_EVEX = 4059,
  SAR16ri_ND = 4060,
  SAR16ri_NF = 4061,
  SAR16ri_NF_ND = 4062,
  SAR32m1 = 4063,
  SAR32m1_EVEX = 4064,
  SAR32m1_ND = 4065,
  SAR32m1_NF = 4066,
  SAR32m1_NF_ND = 4067,
  SAR32mCL = 4068,
  SAR32mCL_EVEX = 4069,
  SAR32mCL_ND = 4070,
  SAR32mCL_NF = 4071,
  SAR32mCL_NF_ND = 4072,
  SAR32mi = 4073,
  SAR32mi_EVEX = 4074,
  SAR32mi_ND = 4075,
  SAR32mi_NF = 4076,
  SAR32mi_NF_ND = 4077,
  SAR32r1 = 4078,
  SAR32r1_EVEX = 4079,
  SAR32r1_ND = 4080,
  SAR32r1_NF = 4081,
  SAR32r1_NF_ND = 4082,
  SAR32rCL = 4083,
  SAR32rCL_EVEX = 4084,
  SAR32rCL_ND = 4085,
  SAR32rCL_NF = 4086,
  SAR32rCL_NF_ND = 4087,
  SAR32ri = 4088,
  SAR32ri_EVEX = 4089,
  SAR32ri_ND = 4090,
  SAR32ri_NF = 4091,
  SAR32ri_NF_ND = 4092,
  SAR64m1 = 4093,
  SAR64m1_EVEX = 4094,
  SAR64m1_ND = 4095,
  SAR64m1_NF = 4096,
  SAR64m1_NF_ND = 4097,
  SAR64mCL = 4098,
  SAR64mCL_EVEX = 4099,
  SAR64mCL_ND = 4100,
  SAR64mCL_NF = 4101,
  SAR64mCL_NF_ND = 4102,
  SAR64mi = 4103,
  SAR64mi_EVEX = 4104,
  SAR64mi_ND = 4105,
  SAR64mi_NF = 4106,
  SAR64mi_NF_ND = 4107,
  SAR64r1 = 4108,
  SAR64r1_EVEX = 4109,
  SAR64r1_ND = 4110,
  SAR64r1_NF = 4111,
  SAR64r1_NF_ND = 4112,
  SAR64rCL = 4113,
  SAR64rCL_EVEX = 4114,
  SAR64rCL_ND = 4115,
  SAR64rCL_NF = 4116,
  SAR64rCL_NF_ND = 4117,
  SAR64ri = 4118,
  SAR64ri_EVEX = 4119,
  SAR64ri_ND = 4120,
  SAR64ri_NF = 4121,
  SAR64ri_NF_ND = 4122,
  SAR8m1 = 4123,
  SAR8m1_EVEX = 4124,
  SAR8m1_ND = 4125,
  SAR8m1_NF = 4126,
  SAR8m1_NF_ND = 4127,
  SAR8mCL = 4128,
  SAR8mCL_EVEX = 4129,
  SAR8mCL_ND = 4130,
  SAR8mCL_NF = 4131,
  SAR8mCL_NF_ND = 4132,
  SAR8mi = 4133,
  SAR8mi_EVEX = 4134,
  SAR8mi_ND = 4135,
  SAR8mi_NF = 4136,
  SAR8mi_NF_ND = 4137,
  SAR8r1 = 4138,
  SAR8r1_EVEX = 4139,
  SAR8r1_ND = 4140,
  SAR8r1_NF = 4141,
  SAR8r1_NF_ND = 4142,
  SAR8rCL = 4143,
  SAR8rCL_EVEX = 4144,
  SAR8rCL_ND = 4145,
  SAR8rCL_NF = 4146,
  SAR8rCL_NF_ND = 4147,
  SAR8ri = 4148,
  SAR8ri_EVEX = 4149,
  SAR8ri_ND = 4150,
  SAR8ri_NF = 4151,
  SAR8ri_NF_ND = 4152,
  SARX32rm = 4153,
  SARX32rm_EVEX = 4154,
  SARX32rr = 4155,
  SARX32rr_EVEX = 4156,
  SARX64rm = 4157,
  SARX64rm_EVEX = 4158,
  SARX64rr = 4159,
  SARX64rr_EVEX = 4160,
  SAVEPREVSSP = 4161,
  SBB16i16 = 4162,
  SBB16mi = 4163,
  SBB16mi8 = 4164,
  SBB16mi8_EVEX = 4165,
  SBB16mi8_ND = 4166,
  SBB16mi_EVEX = 4167,
  SBB16mi_ND = 4168,
  SBB16mr = 4169,
  SBB16mr_EVEX = 4170,
  SBB16mr_ND = 4171,
  SBB16ri = 4172,
  SBB16ri8 = 4173,
  SBB16ri8_EVEX = 4174,
  SBB16ri8_ND = 4175,
  SBB16ri_EVEX = 4176,
  SBB16ri_ND = 4177,
  SBB16rm = 4178,
  SBB16rm_EVEX = 4179,
  SBB16rm_ND = 4180,
  SBB16rr = 4181,
  SBB16rr_EVEX = 4182,
  SBB16rr_EVEX_REV = 4183,
  SBB16rr_ND = 4184,
  SBB16rr_ND_REV = 4185,
  SBB16rr_REV = 4186,
  SBB32i32 = 4187,
  SBB32mi = 4188,
  SBB32mi8 = 4189,
  SBB32mi8_EVEX = 4190,
  SBB32mi8_ND = 4191,
  SBB32mi_EVEX = 4192,
  SBB32mi_ND = 4193,
  SBB32mr = 4194,
  SBB32mr_EVEX = 4195,
  SBB32mr_ND = 4196,
  SBB32ri = 4197,
  SBB32ri8 = 4198,
  SBB32ri8_EVEX = 4199,
  SBB32ri8_ND = 4200,
  SBB32ri_EVEX = 4201,
  SBB32ri_ND = 4202,
  SBB32rm = 4203,
  SBB32rm_EVEX = 4204,
  SBB32rm_ND = 4205,
  SBB32rr = 4206,
  SBB32rr_EVEX = 4207,
  SBB32rr_EVEX_REV = 4208,
  SBB32rr_ND = 4209,
  SBB32rr_ND_REV = 4210,
  SBB32rr_REV = 4211,
  SBB64i32 = 4212,
  SBB64mi32 = 4213,
  SBB64mi32_EVEX = 4214,
  SBB64mi32_ND = 4215,
  SBB64mi8 = 4216,
  SBB64mi8_EVEX = 4217,
  SBB64mi8_ND = 4218,
  SBB64mr = 4219,
  SBB64mr_EVEX = 4220,
  SBB64mr_ND = 4221,
  SBB64ri32 = 4222,
  SBB64ri32_EVEX = 4223,
  SBB64ri32_ND = 4224,
  SBB64ri8 = 4225,
  SBB64ri8_EVEX = 4226,
  SBB64ri8_ND = 4227,
  SBB64rm = 4228,
  SBB64rm_EVEX = 4229,
  SBB64rm_ND = 4230,
  SBB64rr = 4231,
  SBB64rr_EVEX = 4232,
  SBB64rr_EVEX_REV = 4233,
  SBB64rr_ND = 4234,
  SBB64rr_ND_REV = 4235,
  SBB64rr_REV = 4236,
  SBB8i8 = 4237,
  SBB8mi = 4238,
  SBB8mi8 = 4239,
  SBB8mi_EVEX = 4240,
  SBB8mi_ND = 4241,
  SBB8mr = 4242,
  SBB8mr_EVEX = 4243,
  SBB8mr_ND = 4244,
  SBB8ri = 4245,
  SBB8ri8 = 4246,
  SBB8ri_EVEX = 4247,
  SBB8ri_ND = 4248,
  SBB8rm = 4249,
  SBB8rm_EVEX = 4250,
  SBB8rm_ND = 4251,
  SBB8rr = 4252,
  SBB8rr_EVEX = 4253,
  SBB8rr_EVEX_REV = 4254,
  SBB8rr_ND = 4255,
  SBB8rr_ND_REV = 4256,
  SBB8rr_REV = 4257,
  SCASB = 4258,
  SCASL = 4259,
  SCASQ = 4260,
  SCASW = 4261,
  SEAMCALL = 4262,
  SEAMOPS = 4263,
  SEAMRET = 4264,
  SEG_ALLOCA_32 = 4265,
  SEG_ALLOCA_64 = 4266,
  SENDUIPI = 4267,
  SERIALIZE = 4268,
  SETCCm = 4269,
  SETCCm_EVEX = 4270,
  SETCCr = 4271,
  SETCCr_EVEX = 4272,
  SETSSBSY = 4273,
  SETZUCCm = 4274,
  SETZUCCr = 4275,
  SFENCE = 4276,
  SGDT16m = 4277,
  SGDT32m = 4278,
  SGDT64m = 4279,
  SHA1MSG1rm = 4280,
  SHA1MSG1rr = 4281,
  SHA1MSG2rm = 4282,
  SHA1MSG2rr = 4283,
  SHA1NEXTErm = 4284,
  SHA1NEXTErr = 4285,
  SHA1RNDS4rmi = 4286,
  SHA1RNDS4rri = 4287,
  SHA256MSG1rm = 4288,
  SHA256MSG1rr = 4289,
  SHA256MSG2rm = 4290,
  SHA256MSG2rr = 4291,
  SHA256RNDS2rm = 4292,
  SHA256RNDS2rr = 4293,
  SHL16m1 = 4294,
  SHL16m1_EVEX = 4295,
  SHL16m1_ND = 4296,
  SHL16m1_NF = 4297,
  SHL16m1_NF_ND = 4298,
  SHL16mCL = 4299,
  SHL16mCL_EVEX = 4300,
  SHL16mCL_ND = 4301,
  SHL16mCL_NF = 4302,
  SHL16mCL_NF_ND = 4303,
  SHL16mi = 4304,
  SHL16mi_EVEX = 4305,
  SHL16mi_ND = 4306,
  SHL16mi_NF = 4307,
  SHL16mi_NF_ND = 4308,
  SHL16r1 = 4309,
  SHL16r1_EVEX = 4310,
  SHL16r1_ND = 4311,
  SHL16r1_NF = 4312,
  SHL16r1_NF_ND = 4313,
  SHL16rCL = 4314,
  SHL16rCL_EVEX = 4315,
  SHL16rCL_ND = 4316,
  SHL16rCL_NF = 4317,
  SHL16rCL_NF_ND = 4318,
  SHL16ri = 4319,
  SHL16ri_EVEX = 4320,
  SHL16ri_ND = 4321,
  SHL16ri_NF = 4322,
  SHL16ri_NF_ND = 4323,
  SHL32m1 = 4324,
  SHL32m1_EVEX = 4325,
  SHL32m1_ND = 4326,
  SHL32m1_NF = 4327,
  SHL32m1_NF_ND = 4328,
  SHL32mCL = 4329,
  SHL32mCL_EVEX = 4330,
  SHL32mCL_ND = 4331,
  SHL32mCL_NF = 4332,
  SHL32mCL_NF_ND = 4333,
  SHL32mi = 4334,
  SHL32mi_EVEX = 4335,
  SHL32mi_ND = 4336,
  SHL32mi_NF = 4337,
  SHL32mi_NF_ND = 4338,
  SHL32r1 = 4339,
  SHL32r1_EVEX = 4340,
  SHL32r1_ND = 4341,
  SHL32r1_NF = 4342,
  SHL32r1_NF_ND = 4343,
  SHL32rCL = 4344,
  SHL32rCL_EVEX = 4345,
  SHL32rCL_ND = 4346,
  SHL32rCL_NF = 4347,
  SHL32rCL_NF_ND = 4348,
  SHL32ri = 4349,
  SHL32ri_EVEX = 4350,
  SHL32ri_ND = 4351,
  SHL32ri_NF = 4352,
  SHL32ri_NF_ND = 4353,
  SHL64m1 = 4354,
  SHL64m1_EVEX = 4355,
  SHL64m1_ND = 4356,
  SHL64m1_NF = 4357,
  SHL64m1_NF_ND = 4358,
  SHL64mCL = 4359,
  SHL64mCL_EVEX = 4360,
  SHL64mCL_ND = 4361,
  SHL64mCL_NF = 4362,
  SHL64mCL_NF_ND = 4363,
  SHL64mi = 4364,
  SHL64mi_EVEX = 4365,
  SHL64mi_ND = 4366,
  SHL64mi_NF = 4367,
  SHL64mi_NF_ND = 4368,
  SHL64r1 = 4369,
  SHL64r1_EVEX = 4370,
  SHL64r1_ND = 4371,
  SHL64r1_NF = 4372,
  SHL64r1_NF_ND = 4373,
  SHL64rCL = 4374,
  SHL64rCL_EVEX = 4375,
  SHL64rCL_ND = 4376,
  SHL64rCL_NF = 4377,
  SHL64rCL_NF_ND = 4378,
  SHL64ri = 4379,
  SHL64ri_EVEX = 4380,
  SHL64ri_ND = 4381,
  SHL64ri_NF = 4382,
  SHL64ri_NF_ND = 4383,
  SHL8m1 = 4384,
  SHL8m1_EVEX = 4385,
  SHL8m1_ND = 4386,
  SHL8m1_NF = 4387,
  SHL8m1_NF_ND = 4388,
  SHL8mCL = 4389,
  SHL8mCL_EVEX = 4390,
  SHL8mCL_ND = 4391,
  SHL8mCL_NF = 4392,
  SHL8mCL_NF_ND = 4393,
  SHL8mi = 4394,
  SHL8mi_EVEX = 4395,
  SHL8mi_ND = 4396,
  SHL8mi_NF = 4397,
  SHL8mi_NF_ND = 4398,
  SHL8r1 = 4399,
  SHL8r1_EVEX = 4400,
  SHL8r1_ND = 4401,
  SHL8r1_NF = 4402,
  SHL8r1_NF_ND = 4403,
  SHL8rCL = 4404,
  SHL8rCL_EVEX = 4405,
  SHL8rCL_ND = 4406,
  SHL8rCL_NF = 4407,
  SHL8rCL_NF_ND = 4408,
  SHL8ri = 4409,
  SHL8ri_EVEX = 4410,
  SHL8ri_ND = 4411,
  SHL8ri_NF = 4412,
  SHL8ri_NF_ND = 4413,
  SHLD16mrCL = 4414,
  SHLD16mrCL_EVEX = 4415,
  SHLD16mrCL_ND = 4416,
  SHLD16mrCL_NF = 4417,
  SHLD16mrCL_NF_ND = 4418,
  SHLD16mri8 = 4419,
  SHLD16mri8_EVEX = 4420,
  SHLD16mri8_ND = 4421,
  SHLD16mri8_NF = 4422,
  SHLD16mri8_NF_ND = 4423,
  SHLD16rrCL = 4424,
  SHLD16rrCL_EVEX = 4425,
  SHLD16rrCL_ND = 4426,
  SHLD16rrCL_NF = 4427,
  SHLD16rrCL_NF_ND = 4428,
  SHLD16rri8 = 4429,
  SHLD16rri8_EVEX = 4430,
  SHLD16rri8_ND = 4431,
  SHLD16rri8_NF = 4432,
  SHLD16rri8_NF_ND = 4433,
  SHLD32mrCL = 4434,
  SHLD32mrCL_EVEX = 4435,
  SHLD32mrCL_ND = 4436,
  SHLD32mrCL_NF = 4437,
  SHLD32mrCL_NF_ND = 4438,
  SHLD32mri8 = 4439,
  SHLD32mri8_EVEX = 4440,
  SHLD32mri8_ND = 4441,
  SHLD32mri8_NF = 4442,
  SHLD32mri8_NF_ND = 4443,
  SHLD32rrCL = 4444,
  SHLD32rrCL_EVEX = 4445,
  SHLD32rrCL_ND = 4446,
  SHLD32rrCL_NF = 4447,
  SHLD32rrCL_NF_ND = 4448,
  SHLD32rri8 = 4449,
  SHLD32rri8_EVEX = 4450,
  SHLD32rri8_ND = 4451,
  SHLD32rri8_NF = 4452,
  SHLD32rri8_NF_ND = 4453,
  SHLD64mrCL = 4454,
  SHLD64mrCL_EVEX = 4455,
  SHLD64mrCL_ND = 4456,
  SHLD64mrCL_NF = 4457,
  SHLD64mrCL_NF_ND = 4458,
  SHLD64mri8 = 4459,
  SHLD64mri8_EVEX = 4460,
  SHLD64mri8_ND = 4461,
  SHLD64mri8_NF = 4462,
  SHLD64mri8_NF_ND = 4463,
  SHLD64rrCL = 4464,
  SHLD64rrCL_EVEX = 4465,
  SHLD64rrCL_ND = 4466,
  SHLD64rrCL_NF = 4467,
  SHLD64rrCL_NF_ND = 4468,
  SHLD64rri8 = 4469,
  SHLD64rri8_EVEX = 4470,
  SHLD64rri8_ND = 4471,
  SHLD64rri8_NF = 4472,
  SHLD64rri8_NF_ND = 4473,
  SHLX32rm = 4474,
  SHLX32rm_EVEX = 4475,
  SHLX32rr = 4476,
  SHLX32rr_EVEX = 4477,
  SHLX64rm = 4478,
  SHLX64rm_EVEX = 4479,
  SHLX64rr = 4480,
  SHLX64rr_EVEX = 4481,
  SHR16m1 = 4482,
  SHR16m1_EVEX = 4483,
  SHR16m1_ND = 4484,
  SHR16m1_NF = 4485,
  SHR16m1_NF_ND = 4486,
  SHR16mCL = 4487,
  SHR16mCL_EVEX = 4488,
  SHR16mCL_ND = 4489,
  SHR16mCL_NF = 4490,
  SHR16mCL_NF_ND = 4491,
  SHR16mi = 4492,
  SHR16mi_EVEX = 4493,
  SHR16mi_ND = 4494,
  SHR16mi_NF = 4495,
  SHR16mi_NF_ND = 4496,
  SHR16r1 = 4497,
  SHR16r1_EVEX = 4498,
  SHR16r1_ND = 4499,
  SHR16r1_NF = 4500,
  SHR16r1_NF_ND = 4501,
  SHR16rCL = 4502,
  SHR16rCL_EVEX = 4503,
  SHR16rCL_ND = 4504,
  SHR16rCL_NF = 4505,
  SHR16rCL_NF_ND = 4506,
  SHR16ri = 4507,
  SHR16ri_EVEX = 4508,
  SHR16ri_ND = 4509,
  SHR16ri_NF = 4510,
  SHR16ri_NF_ND = 4511,
  SHR32m1 = 4512,
  SHR32m1_EVEX = 4513,
  SHR32m1_ND = 4514,
  SHR32m1_NF = 4515,
  SHR32m1_NF_ND = 4516,
  SHR32mCL = 4517,
  SHR32mCL_EVEX = 4518,
  SHR32mCL_ND = 4519,
  SHR32mCL_NF = 4520,
  SHR32mCL_NF_ND = 4521,
  SHR32mi = 4522,
  SHR32mi_EVEX = 4523,
  SHR32mi_ND = 4524,
  SHR32mi_NF = 4525,
  SHR32mi_NF_ND = 4526,
  SHR32r1 = 4527,
  SHR32r1_EVEX = 4528,
  SHR32r1_ND = 4529,
  SHR32r1_NF = 4530,
  SHR32r1_NF_ND = 4531,
  SHR32rCL = 4532,
  SHR32rCL_EVEX = 4533,
  SHR32rCL_ND = 4534,
  SHR32rCL_NF = 4535,
  SHR32rCL_NF_ND = 4536,
  SHR32ri = 4537,
  SHR32ri_EVEX = 4538,
  SHR32ri_ND = 4539,
  SHR32ri_NF = 4540,
  SHR32ri_NF_ND = 4541,
  SHR64m1 = 4542,
  SHR64m1_EVEX = 4543,
  SHR64m1_ND = 4544,
  SHR64m1_NF = 4545,
  SHR64m1_NF_ND = 4546,
  SHR64mCL = 4547,
  SHR64mCL_EVEX = 4548,
  SHR64mCL_ND = 4549,
  SHR64mCL_NF = 4550,
  SHR64mCL_NF_ND = 4551,
  SHR64mi = 4552,
  SHR64mi_EVEX = 4553,
  SHR64mi_ND = 4554,
  SHR64mi_NF = 4555,
  SHR64mi_NF_ND = 4556,
  SHR64r1 = 4557,
  SHR64r1_EVEX = 4558,
  SHR64r1_ND = 4559,
  SHR64r1_NF = 4560,
  SHR64r1_NF_ND = 4561,
  SHR64rCL = 4562,
  SHR64rCL_EVEX = 4563,
  SHR64rCL_ND = 4564,
  SHR64rCL_NF = 4565,
  SHR64rCL_NF_ND = 4566,
  SHR64ri = 4567,
  SHR64ri_EVEX = 4568,
  SHR64ri_ND = 4569,
  SHR64ri_NF = 4570,
  SHR64ri_NF_ND = 4571,
  SHR8m1 = 4572,
  SHR8m1_EVEX = 4573,
  SHR8m1_ND = 4574,
  SHR8m1_NF = 4575,
  SHR8m1_NF_ND = 4576,
  SHR8mCL = 4577,
  SHR8mCL_EVEX = 4578,
  SHR8mCL_ND = 4579,
  SHR8mCL_NF = 4580,
  SHR8mCL_NF_ND = 4581,
  SHR8mi = 4582,
  SHR8mi_EVEX = 4583,
  SHR8mi_ND = 4584,
  SHR8mi_NF = 4585,
  SHR8mi_NF_ND = 4586,
  SHR8r1 = 4587,
  SHR8r1_EVEX = 4588,
  SHR8r1_ND = 4589,
  SHR8r1_NF = 4590,
  SHR8r1_NF_ND = 4591,
  SHR8rCL = 4592,
  SHR8rCL_EVEX = 4593,
  SHR8rCL_ND = 4594,
  SHR8rCL_NF = 4595,
  SHR8rCL_NF_ND = 4596,
  SHR8ri = 4597,
  SHR8ri_EVEX = 4598,
  SHR8ri_ND = 4599,
  SHR8ri_NF = 4600,
  SHR8ri_NF_ND = 4601,
  SHRD16mrCL = 4602,
  SHRD16mrCL_EVEX = 4603,
  SHRD16mrCL_ND = 4604,
  SHRD16mrCL_NF = 4605,
  SHRD16mrCL_NF_ND = 4606,
  SHRD16mri8 = 4607,
  SHRD16mri8_EVEX = 4608,
  SHRD16mri8_ND = 4609,
  SHRD16mri8_NF = 4610,
  SHRD16mri8_NF_ND = 4611,
  SHRD16rrCL = 4612,
  SHRD16rrCL_EVEX = 4613,
  SHRD16rrCL_ND = 4614,
  SHRD16rrCL_NF = 4615,
  SHRD16rrCL_NF_ND = 4616,
  SHRD16rri8 = 4617,
  SHRD16rri8_EVEX = 4618,
  SHRD16rri8_ND = 4619,
  SHRD16rri8_NF = 4620,
  SHRD16rri8_NF_ND = 4621,
  SHRD32mrCL = 4622,
  SHRD32mrCL_EVEX = 4623,
  SHRD32mrCL_ND = 4624,
  SHRD32mrCL_NF = 4625,
  SHRD32mrCL_NF_ND = 4626,
  SHRD32mri8 = 4627,
  SHRD32mri8_EVEX = 4628,
  SHRD32mri8_ND = 4629,
  SHRD32mri8_NF = 4630,
  SHRD32mri8_NF_ND = 4631,
  SHRD32rrCL = 4632,
  SHRD32rrCL_EVEX = 4633,
  SHRD32rrCL_ND = 4634,
  SHRD32rrCL_NF = 4635,
  SHRD32rrCL_NF_ND = 4636,
  SHRD32rri8 = 4637,
  SHRD32rri8_EVEX = 4638,
  SHRD32rri8_ND = 4639,
  SHRD32rri8_NF = 4640,
  SHRD32rri8_NF_ND = 4641,
  SHRD64mrCL = 4642,
  SHRD64mrCL_EVEX = 4643,
  SHRD64mrCL_ND = 4644,
  SHRD64mrCL_NF = 4645,
  SHRD64mrCL_NF_ND = 4646,
  SHRD64mri8 = 4647,
  SHRD64mri8_EVEX = 4648,
  SHRD64mri8_ND = 4649,
  SHRD64mri8_NF = 4650,
  SHRD64mri8_NF_ND = 4651,
  SHRD64rrCL = 4652,
  SHRD64rrCL_EVEX = 4653,
  SHRD64rrCL_ND = 4654,
  SHRD64rrCL_NF = 4655,
  SHRD64rrCL_NF_ND = 4656,
  SHRD64rri8 = 4657,
  SHRD64rri8_EVEX = 4658,
  SHRD64rri8_ND = 4659,
  SHRD64rri8_NF = 4660,
  SHRD64rri8_NF_ND = 4661,
  SHRX32rm = 4662,
  SHRX32rm_EVEX = 4663,
  SHRX32rr = 4664,
  SHRX32rr_EVEX = 4665,
  SHRX64rm = 4666,
  SHRX64rm_EVEX = 4667,
  SHRX64rr = 4668,
  SHRX64rr_EVEX = 4669,
  SHUFPDrmi = 4670,
  SHUFPDrri = 4671,
  SHUFPSrmi = 4672,
  SHUFPSrri = 4673,
  SIDT16m = 4674,
  SIDT32m = 4675,
  SIDT64m = 4676,
  SKINIT = 4677,
  SLDT16m = 4678,
  SLDT16r = 4679,
  SLDT32r = 4680,
  SLDT64r = 4681,
  SLWPCB = 4682,
  SLWPCB64 = 4683,
  SMSW16m = 4684,
  SMSW16r = 4685,
  SMSW32r = 4686,
  SMSW64r = 4687,
  SQRTPDm = 4688,
  SQRTPDr = 4689,
  SQRTPSm = 4690,
  SQRTPSr = 4691,
  SQRTSDm = 4692,
  SQRTSDm_Int = 4693,
  SQRTSDr = 4694,
  SQRTSDr_Int = 4695,
  SQRTSSm = 4696,
  SQRTSSm_Int = 4697,
  SQRTSSr = 4698,
  SQRTSSr_Int = 4699,
  SQRT_F = 4700,
  SQRT_Fp32 = 4701,
  SQRT_Fp64 = 4702,
  SQRT_Fp80 = 4703,
  SS_PREFIX = 4704,
  STAC = 4705,
  STACKALLOC_W_PROBING = 4706,
  STC = 4707,
  STD = 4708,
  STGI = 4709,
  STI = 4710,
  STMXCSR = 4711,
  STOSB = 4712,
  STOSL = 4713,
  STOSQ = 4714,
  STOSW = 4715,
  STR16r = 4716,
  STR32r = 4717,
  STR64r = 4718,
  STRm = 4719,
  STTILECFG = 4720,
  STTILECFG_EVEX = 4721,
  STUI = 4722,
  ST_F32m = 4723,
  ST_F64m = 4724,
  ST_FP32m = 4725,
  ST_FP64m = 4726,
  ST_FP80m = 4727,
  ST_FPrr = 4728,
  ST_Fp32m = 4729,
  ST_Fp64m = 4730,
  ST_Fp64m32 = 4731,
  ST_Fp80m32 = 4732,
  ST_Fp80m64 = 4733,
  ST_FpP32m = 4734,
  ST_FpP64m = 4735,
  ST_FpP64m32 = 4736,
  ST_FpP80m = 4737,
  ST_FpP80m32 = 4738,
  ST_FpP80m64 = 4739,
  ST_Frr = 4740,
  SUB16i16 = 4741,
  SUB16mi = 4742,
  SUB16mi8 = 4743,
  SUB16mi8_EVEX = 4744,
  SUB16mi8_ND = 4745,
  SUB16mi8_NF = 4746,
  SUB16mi8_NF_ND = 4747,
  SUB16mi_EVEX = 4748,
  SUB16mi_ND = 4749,
  SUB16mi_NF = 4750,
  SUB16mi_NF_ND = 4751,
  SUB16mr = 4752,
  SUB16mr_EVEX = 4753,
  SUB16mr_ND = 4754,
  SUB16mr_NF = 4755,
  SUB16mr_NF_ND = 4756,
  SUB16ri = 4757,
  SUB16ri8 = 4758,
  SUB16ri8_EVEX = 4759,
  SUB16ri8_ND = 4760,
  SUB16ri8_NF = 4761,
  SUB16ri8_NF_ND = 4762,
  SUB16ri_EVEX = 4763,
  SUB16ri_ND = 4764,
  SUB16ri_NF = 4765,
  SUB16ri_NF_ND = 4766,
  SUB16rm = 4767,
  SUB16rm_EVEX = 4768,
  SUB16rm_ND = 4769,
  SUB16rm_NF = 4770,
  SUB16rm_NF_ND = 4771,
  SUB16rr = 4772,
  SUB16rr_EVEX = 4773,
  SUB16rr_EVEX_REV = 4774,
  SUB16rr_ND = 4775,
  SUB16rr_ND_REV = 4776,
  SUB16rr_NF = 4777,
  SUB16rr_NF_ND = 4778,
  SUB16rr_NF_ND_REV = 4779,
  SUB16rr_NF_REV = 4780,
  SUB16rr_REV = 4781,
  SUB32i32 = 4782,
  SUB32mi = 4783,
  SUB32mi8 = 4784,
  SUB32mi8_EVEX = 4785,
  SUB32mi8_ND = 4786,
  SUB32mi8_NF = 4787,
  SUB32mi8_NF_ND = 4788,
  SUB32mi_EVEX = 4789,
  SUB32mi_ND = 4790,
  SUB32mi_NF = 4791,
  SUB32mi_NF_ND = 4792,
  SUB32mr = 4793,
  SUB32mr_EVEX = 4794,
  SUB32mr_ND = 4795,
  SUB32mr_NF = 4796,
  SUB32mr_NF_ND = 4797,
  SUB32ri = 4798,
  SUB32ri8 = 4799,
  SUB32ri8_EVEX = 4800,
  SUB32ri8_ND = 4801,
  SUB32ri8_NF = 4802,
  SUB32ri8_NF_ND = 4803,
  SUB32ri_EVEX = 4804,
  SUB32ri_ND = 4805,
  SUB32ri_NF = 4806,
  SUB32ri_NF_ND = 4807,
  SUB32rm = 4808,
  SUB32rm_EVEX = 4809,
  SUB32rm_ND = 4810,
  SUB32rm_NF = 4811,
  SUB32rm_NF_ND = 4812,
  SUB32rr = 4813,
  SUB32rr_EVEX = 4814,
  SUB32rr_EVEX_REV = 4815,
  SUB32rr_ND = 4816,
  SUB32rr_ND_REV = 4817,
  SUB32rr_NF = 4818,
  SUB32rr_NF_ND = 4819,
  SUB32rr_NF_ND_REV = 4820,
  SUB32rr_NF_REV = 4821,
  SUB32rr_REV = 4822,
  SUB64i32 = 4823,
  SUB64mi32 = 4824,
  SUB64mi32_EVEX = 4825,
  SUB64mi32_ND = 4826,
  SUB64mi32_NF = 4827,
  SUB64mi32_NF_ND = 4828,
  SUB64mi8 = 4829,
  SUB64mi8_EVEX = 4830,
  SUB64mi8_ND = 4831,
  SUB64mi8_NF = 4832,
  SUB64mi8_NF_ND = 4833,
  SUB64mr = 4834,
  SUB64mr_EVEX = 4835,
  SUB64mr_ND = 4836,
  SUB64mr_NF = 4837,
  SUB64mr_NF_ND = 4838,
  SUB64ri32 = 4839,
  SUB64ri32_EVEX = 4840,
  SUB64ri32_ND = 4841,
  SUB64ri32_NF = 4842,
  SUB64ri32_NF_ND = 4843,
  SUB64ri8 = 4844,
  SUB64ri8_EVEX = 4845,
  SUB64ri8_ND = 4846,
  SUB64ri8_NF = 4847,
  SUB64ri8_NF_ND = 4848,
  SUB64rm = 4849,
  SUB64rm_EVEX = 4850,
  SUB64rm_ND = 4851,
  SUB64rm_NF = 4852,
  SUB64rm_NF_ND = 4853,
  SUB64rr = 4854,
  SUB64rr_EVEX = 4855,
  SUB64rr_EVEX_REV = 4856,
  SUB64rr_ND = 4857,
  SUB64rr_ND_REV = 4858,
  SUB64rr_NF = 4859,
  SUB64rr_NF_ND = 4860,
  SUB64rr_NF_ND_REV = 4861,
  SUB64rr_NF_REV = 4862,
  SUB64rr_REV = 4863,
  SUB8i8 = 4864,
  SUB8mi = 4865,
  SUB8mi8 = 4866,
  SUB8mi_EVEX = 4867,
  SUB8mi_ND = 4868,
  SUB8mi_NF = 4869,
  SUB8mi_NF_ND = 4870,
  SUB8mr = 4871,
  SUB8mr_EVEX = 4872,
  SUB8mr_ND = 4873,
  SUB8mr_NF = 4874,
  SUB8mr_NF_ND = 4875,
  SUB8ri = 4876,
  SUB8ri8 = 4877,
  SUB8ri_EVEX = 4878,
  SUB8ri_ND = 4879,
  SUB8ri_NF = 4880,
  SUB8ri_NF_ND = 4881,
  SUB8rm = 4882,
  SUB8rm_EVEX = 4883,
  SUB8rm_ND = 4884,
  SUB8rm_NF = 4885,
  SUB8rm_NF_ND = 4886,
  SUB8rr = 4887,
  SUB8rr_EVEX = 4888,
  SUB8rr_EVEX_REV = 4889,
  SUB8rr_ND = 4890,
  SUB8rr_ND_REV = 4891,
  SUB8rr_NF = 4892,
  SUB8rr_NF_ND = 4893,
  SUB8rr_NF_ND_REV = 4894,
  SUB8rr_NF_REV = 4895,
  SUB8rr_REV = 4896,
  SUBPDrm = 4897,
  SUBPDrr = 4898,
  SUBPSrm = 4899,
  SUBPSrr = 4900,
  SUBR_F32m = 4901,
  SUBR_F64m = 4902,
  SUBR_FI16m = 4903,
  SUBR_FI32m = 4904,
  SUBR_FPrST0 = 4905,
  SUBR_FST0r = 4906,
  SUBR_Fp32m = 4907,
  SUBR_Fp64m = 4908,
  SUBR_Fp64m32 = 4909,
  SUBR_Fp80m32 = 4910,
  SUBR_Fp80m64 = 4911,
  SUBR_FpI16m32 = 4912,
  SUBR_FpI16m64 = 4913,
  SUBR_FpI16m80 = 4914,
  SUBR_FpI32m32 = 4915,
  SUBR_FpI32m64 = 4916,
  SUBR_FpI32m80 = 4917,
  SUBR_FrST0 = 4918,
  SUBSDrm = 4919,
  SUBSDrm_Int = 4920,
  SUBSDrr = 4921,
  SUBSDrr_Int = 4922,
  SUBSSrm = 4923,
  SUBSSrm_Int = 4924,
  SUBSSrr = 4925,
  SUBSSrr_Int = 4926,
  SUB_F32m = 4927,
  SUB_F64m = 4928,
  SUB_FI16m = 4929,
  SUB_FI32m = 4930,
  SUB_FPrST0 = 4931,
  SUB_FST0r = 4932,
  SUB_Fp32 = 4933,
  SUB_Fp32m = 4934,
  SUB_Fp64 = 4935,
  SUB_Fp64m = 4936,
  SUB_Fp64m32 = 4937,
  SUB_Fp80 = 4938,
  SUB_Fp80m32 = 4939,
  SUB_Fp80m64 = 4940,
  SUB_FpI16m32 = 4941,
  SUB_FpI16m64 = 4942,
  SUB_FpI16m80 = 4943,
  SUB_FpI32m32 = 4944,
  SUB_FpI32m64 = 4945,
  SUB_FpI32m80 = 4946,
  SUB_FrST0 = 4947,
  SWAPGS = 4948,
  SYSCALL = 4949,
  SYSENTER = 4950,
  SYSEXIT = 4951,
  SYSEXIT64 = 4952,
  SYSRET = 4953,
  SYSRET64 = 4954,
  T1MSKC32rm = 4955,
  T1MSKC32rr = 4956,
  T1MSKC64rm = 4957,
  T1MSKC64rr = 4958,
  T2RPNTLVWZ0 = 4959,
  T2RPNTLVWZ0RS = 4960,
  T2RPNTLVWZ0RST1 = 4961,
  T2RPNTLVWZ0RST1_EVEX = 4962,
  T2RPNTLVWZ0RS_EVEX = 4963,
  T2RPNTLVWZ0T1 = 4964,
  T2RPNTLVWZ0T1_EVEX = 4965,
  T2RPNTLVWZ0_EVEX = 4966,
  T2RPNTLVWZ1 = 4967,
  T2RPNTLVWZ1RS = 4968,
  T2RPNTLVWZ1RST1 = 4969,
  T2RPNTLVWZ1RST1_EVEX = 4970,
  T2RPNTLVWZ1RS_EVEX = 4971,
  T2RPNTLVWZ1T1 = 4972,
  T2RPNTLVWZ1T1_EVEX = 4973,
  T2RPNTLVWZ1_EVEX = 4974,
  TAILJMPd = 4975,
  TAILJMPd64 = 4976,
  TAILJMPd64_CC = 4977,
  TAILJMPd_CC = 4978,
  TAILJMPm = 4979,
  TAILJMPm64 = 4980,
  TAILJMPm64_REX = 4981,
  TAILJMPr = 4982,
  TAILJMPr64 = 4983,
  TAILJMPr64_REX = 4984,
  TCMMIMFP16PS = 4985,
  TCMMRLFP16PS = 4986,
  TCONJTCMMIMFP16PS = 4987,
  TCONJTFP16 = 4988,
  TCRETURNdi = 4989,
  TCRETURNdi64 = 4990,
  TCRETURNdi64cc = 4991,
  TCRETURNdicc = 4992,
  TCRETURNmi = 4993,
  TCRETURNmi64 = 4994,
  TCRETURNri = 4995,
  TCRETURNri64 = 4996,
  TCRETURNri64_ImpCall = 4997,
  TCVTROWD2PSrre = 4998,
  TCVTROWD2PSrri = 4999,
  TCVTROWPS2BF16Hrre = 5000,
  TCVTROWPS2BF16Hrri = 5001,
  TCVTROWPS2BF16Lrre = 5002,
  TCVTROWPS2BF16Lrri = 5003,
  TCVTROWPS2PHHrre = 5004,
  TCVTROWPS2PHHrri = 5005,
  TCVTROWPS2PHLrre = 5006,
  TCVTROWPS2PHLrri = 5007,
  TDCALL = 5008,
  TDPBF16PS = 5009,
  TDPBF8PS = 5010,
  TDPBHF8PS = 5011,
  TDPBSSD = 5012,
  TDPBSUD = 5013,
  TDPBUSD = 5014,
  TDPBUUD = 5015,
  TDPFP16PS = 5016,
  TDPHBF8PS = 5017,
  TDPHF8PS = 5018,
  TEST16i16 = 5019,
  TEST16mi = 5020,
  TEST16mr = 5021,
  TEST16ri = 5022,
  TEST16rr = 5023,
  TEST32i32 = 5024,
  TEST32mi = 5025,
  TEST32mr = 5026,
  TEST32ri = 5027,
  TEST32rr = 5028,
  TEST64i32 = 5029,
  TEST64mi32 = 5030,
  TEST64mr = 5031,
  TEST64ri32 = 5032,
  TEST64rr = 5033,
  TEST8i8 = 5034,
  TEST8mi = 5035,
  TEST8mr = 5036,
  TEST8ri = 5037,
  TEST8rr = 5038,
  TESTUI = 5039,
  TILELOADD = 5040,
  TILELOADDRS = 5041,
  TILELOADDRST1 = 5042,
  TILELOADDRST1_EVEX = 5043,
  TILELOADDRS_EVEX = 5044,
  TILELOADDT1 = 5045,
  TILELOADDT1_EVEX = 5046,
  TILELOADD_EVEX = 5047,
  TILEMOVROWrre = 5048,
  TILEMOVROWrri = 5049,
  TILERELEASE = 5050,
  TILESTORED = 5051,
  TILESTORED_EVEX = 5052,
  TILEZERO = 5053,
  TLBSYNC = 5054,
  TLSCall_32 = 5055,
  TLSCall_64 = 5056,
  TLS_addr32 = 5057,
  TLS_addr64 = 5058,
  TLS_addrX32 = 5059,
  TLS_base_addr32 = 5060,
  TLS_base_addr64 = 5061,
  TLS_base_addrX32 = 5062,
  TLS_desc32 = 5063,
  TLS_desc64 = 5064,
  TMMULTF32PS = 5065,
  TPAUSE = 5066,
  TRAP = 5067,
  TST_F = 5068,
  TST_Fp32 = 5069,
  TST_Fp64 = 5070,
  TST_Fp80 = 5071,
  TTCMMIMFP16PS = 5072,
  TTCMMRLFP16PS = 5073,
  TTDPBF16PS = 5074,
  TTDPFP16PS = 5075,
  TTMMULTF32PS = 5076,
  TTRANSPOSED = 5077,
  TZCNT16rm = 5078,
  TZCNT16rm_EVEX = 5079,
  TZCNT16rm_NF = 5080,
  TZCNT16rr = 5081,
  TZCNT16rr_EVEX = 5082,
  TZCNT16rr_NF = 5083,
  TZCNT32rm = 5084,
  TZCNT32rm_EVEX = 5085,
  TZCNT32rm_NF = 5086,
  TZCNT32rr = 5087,
  TZCNT32rr_EVEX = 5088,
  TZCNT32rr_NF = 5089,
  TZCNT64rm = 5090,
  TZCNT64rm_EVEX = 5091,
  TZCNT64rm_NF = 5092,
  TZCNT64rr = 5093,
  TZCNT64rr_EVEX = 5094,
  TZCNT64rr_NF = 5095,
  TZMSK32rm = 5096,
  TZMSK32rr = 5097,
  TZMSK64rm = 5098,
  TZMSK64rr = 5099,
  UBSAN_UD1 = 5100,
  UCOMISDrm = 5101,
  UCOMISDrm_Int = 5102,
  UCOMISDrr = 5103,
  UCOMISDrr_Int = 5104,
  UCOMISSrm = 5105,
  UCOMISSrm_Int = 5106,
  UCOMISSrr = 5107,
  UCOMISSrr_Int = 5108,
  UCOM_FIPr = 5109,
  UCOM_FIr = 5110,
  UCOM_FPPr = 5111,
  UCOM_FPr = 5112,
  UCOM_FpIr32 = 5113,
  UCOM_FpIr64 = 5114,
  UCOM_FpIr80 = 5115,
  UCOM_Fpr32 = 5116,
  UCOM_Fpr64 = 5117,
  UCOM_Fpr80 = 5118,
  UCOM_Fr = 5119,
  UD1Lm = 5120,
  UD1Lr = 5121,
  UD1Qm = 5122,
  UD1Qr = 5123,
  UD1Wm = 5124,
  UD1Wr = 5125,
  UIRET = 5126,
  UMONITOR16 = 5127,
  UMONITOR32 = 5128,
  UMONITOR64 = 5129,
  UMWAIT = 5130,
  UNPCKHPDrm = 5131,
  UNPCKHPDrr = 5132,
  UNPCKHPSrm = 5133,
  UNPCKHPSrr = 5134,
  UNPCKLPDrm = 5135,
  UNPCKLPDrr = 5136,
  UNPCKLPSrm = 5137,
  UNPCKLPSrr = 5138,
  URDMSRri = 5139,
  URDMSRri_EVEX = 5140,
  URDMSRrr = 5141,
  URDMSRrr_EVEX = 5142,
  UWRMSRir = 5143,
  UWRMSRir_EVEX = 5144,
  UWRMSRrr = 5145,
  UWRMSRrr_EVEX = 5146,
  V4FMADDPSrm = 5147,
  V4FMADDPSrmk = 5148,
  V4FMADDPSrmkz = 5149,
  V4FMADDSSrm = 5150,
  V4FMADDSSrmk = 5151,
  V4FMADDSSrmkz = 5152,
  V4FNMADDPSrm = 5153,
  V4FNMADDPSrmk = 5154,
  V4FNMADDPSrmkz = 5155,
  V4FNMADDSSrm = 5156,
  V4FNMADDSSrmk = 5157,
  V4FNMADDSSrmkz = 5158,
  VAARG_64 = 5159,
  VAARG_X32 = 5160,
  VADDBF16Z128rm = 5161,
  VADDBF16Z128rmb = 5162,
  VADDBF16Z128rmbk = 5163,
  VADDBF16Z128rmbkz = 5164,
  VADDBF16Z128rmk = 5165,
  VADDBF16Z128rmkz = 5166,
  VADDBF16Z128rr = 5167,
  VADDBF16Z128rrk = 5168,
  VADDBF16Z128rrkz = 5169,
  VADDBF16Z256rm = 5170,
  VADDBF16Z256rmb = 5171,
  VADDBF16Z256rmbk = 5172,
  VADDBF16Z256rmbkz = 5173,
  VADDBF16Z256rmk = 5174,
  VADDBF16Z256rmkz = 5175,
  VADDBF16Z256rr = 5176,
  VADDBF16Z256rrk = 5177,
  VADDBF16Z256rrkz = 5178,
  VADDBF16Zrm = 5179,
  VADDBF16Zrmb = 5180,
  VADDBF16Zrmbk = 5181,
  VADDBF16Zrmbkz = 5182,
  VADDBF16Zrmk = 5183,
  VADDBF16Zrmkz = 5184,
  VADDBF16Zrr = 5185,
  VADDBF16Zrrk = 5186,
  VADDBF16Zrrkz = 5187,
  VADDPDYrm = 5188,
  VADDPDYrr = 5189,
  VADDPDZ128rm = 5190,
  VADDPDZ128rmb = 5191,
  VADDPDZ128rmbk = 5192,
  VADDPDZ128rmbkz = 5193,
  VADDPDZ128rmk = 5194,
  VADDPDZ128rmkz = 5195,
  VADDPDZ128rr = 5196,
  VADDPDZ128rrk = 5197,
  VADDPDZ128rrkz = 5198,
  VADDPDZ256rm = 5199,
  VADDPDZ256rmb = 5200,
  VADDPDZ256rmbk = 5201,
  VADDPDZ256rmbkz = 5202,
  VADDPDZ256rmk = 5203,
  VADDPDZ256rmkz = 5204,
  VADDPDZ256rr = 5205,
  VADDPDZ256rrk = 5206,
  VADDPDZ256rrkz = 5207,
  VADDPDZrm = 5208,
  VADDPDZrmb = 5209,
  VADDPDZrmbk = 5210,
  VADDPDZrmbkz = 5211,
  VADDPDZrmk = 5212,
  VADDPDZrmkz = 5213,
  VADDPDZrr = 5214,
  VADDPDZrrb = 5215,
  VADDPDZrrbk = 5216,
  VADDPDZrrbkz = 5217,
  VADDPDZrrk = 5218,
  VADDPDZrrkz = 5219,
  VADDPDrm = 5220,
  VADDPDrr = 5221,
  VADDPHZ128rm = 5222,
  VADDPHZ128rmb = 5223,
  VADDPHZ128rmbk = 5224,
  VADDPHZ128rmbkz = 5225,
  VADDPHZ128rmk = 5226,
  VADDPHZ128rmkz = 5227,
  VADDPHZ128rr = 5228,
  VADDPHZ128rrk = 5229,
  VADDPHZ128rrkz = 5230,
  VADDPHZ256rm = 5231,
  VADDPHZ256rmb = 5232,
  VADDPHZ256rmbk = 5233,
  VADDPHZ256rmbkz = 5234,
  VADDPHZ256rmk = 5235,
  VADDPHZ256rmkz = 5236,
  VADDPHZ256rr = 5237,
  VADDPHZ256rrk = 5238,
  VADDPHZ256rrkz = 5239,
  VADDPHZrm = 5240,
  VADDPHZrmb = 5241,
  VADDPHZrmbk = 5242,
  VADDPHZrmbkz = 5243,
  VADDPHZrmk = 5244,
  VADDPHZrmkz = 5245,
  VADDPHZrr = 5246,
  VADDPHZrrb = 5247,
  VADDPHZrrbk = 5248,
  VADDPHZrrbkz = 5249,
  VADDPHZrrk = 5250,
  VADDPHZrrkz = 5251,
  VADDPSYrm = 5252,
  VADDPSYrr = 5253,
  VADDPSZ128rm = 5254,
  VADDPSZ128rmb = 5255,
  VADDPSZ128rmbk = 5256,
  VADDPSZ128rmbkz = 5257,
  VADDPSZ128rmk = 5258,
  VADDPSZ128rmkz = 5259,
  VADDPSZ128rr = 5260,
  VADDPSZ128rrk = 5261,
  VADDPSZ128rrkz = 5262,
  VADDPSZ256rm = 5263,
  VADDPSZ256rmb = 5264,
  VADDPSZ256rmbk = 5265,
  VADDPSZ256rmbkz = 5266,
  VADDPSZ256rmk = 5267,
  VADDPSZ256rmkz = 5268,
  VADDPSZ256rr = 5269,
  VADDPSZ256rrk = 5270,
  VADDPSZ256rrkz = 5271,
  VADDPSZrm = 5272,
  VADDPSZrmb = 5273,
  VADDPSZrmbk = 5274,
  VADDPSZrmbkz = 5275,
  VADDPSZrmk = 5276,
  VADDPSZrmkz = 5277,
  VADDPSZrr = 5278,
  VADDPSZrrb = 5279,
  VADDPSZrrbk = 5280,
  VADDPSZrrbkz = 5281,
  VADDPSZrrk = 5282,
  VADDPSZrrkz = 5283,
  VADDPSrm = 5284,
  VADDPSrr = 5285,
  VADDSDZrm = 5286,
  VADDSDZrm_Int = 5287,
  VADDSDZrmk_Int = 5288,
  VADDSDZrmkz_Int = 5289,
  VADDSDZrr = 5290,
  VADDSDZrr_Int = 5291,
  VADDSDZrrb_Int = 5292,
  VADDSDZrrbk_Int = 5293,
  VADDSDZrrbkz_Int = 5294,
  VADDSDZrrk_Int = 5295,
  VADDSDZrrkz_Int = 5296,
  VADDSDrm = 5297,
  VADDSDrm_Int = 5298,
  VADDSDrr = 5299,
  VADDSDrr_Int = 5300,
  VADDSHZrm = 5301,
  VADDSHZrm_Int = 5302,
  VADDSHZrmk_Int = 5303,
  VADDSHZrmkz_Int = 5304,
  VADDSHZrr = 5305,
  VADDSHZrr_Int = 5306,
  VADDSHZrrb_Int = 5307,
  VADDSHZrrbk_Int = 5308,
  VADDSHZrrbkz_Int = 5309,
  VADDSHZrrk_Int = 5310,
  VADDSHZrrkz_Int = 5311,
  VADDSSZrm = 5312,
  VADDSSZrm_Int = 5313,
  VADDSSZrmk_Int = 5314,
  VADDSSZrmkz_Int = 5315,
  VADDSSZrr = 5316,
  VADDSSZrr_Int = 5317,
  VADDSSZrrb_Int = 5318,
  VADDSSZrrbk_Int = 5319,
  VADDSSZrrbkz_Int = 5320,
  VADDSSZrrk_Int = 5321,
  VADDSSZrrkz_Int = 5322,
  VADDSSrm = 5323,
  VADDSSrm_Int = 5324,
  VADDSSrr = 5325,
  VADDSSrr_Int = 5326,
  VADDSUBPDYrm = 5327,
  VADDSUBPDYrr = 5328,
  VADDSUBPDrm = 5329,
  VADDSUBPDrr = 5330,
  VADDSUBPSYrm = 5331,
  VADDSUBPSYrr = 5332,
  VADDSUBPSrm = 5333,
  VADDSUBPSrr = 5334,
  VAESDECLASTYrm = 5335,
  VAESDECLASTYrr = 5336,
  VAESDECLASTZ128rm = 5337,
  VAESDECLASTZ128rr = 5338,
  VAESDECLASTZ256rm = 5339,
  VAESDECLASTZ256rr = 5340,
  VAESDECLASTZrm = 5341,
  VAESDECLASTZrr = 5342,
  VAESDECLASTrm = 5343,
  VAESDECLASTrr = 5344,
  VAESDECYrm = 5345,
  VAESDECYrr = 5346,
  VAESDECZ128rm = 5347,
  VAESDECZ128rr = 5348,
  VAESDECZ256rm = 5349,
  VAESDECZ256rr = 5350,
  VAESDECZrm = 5351,
  VAESDECZrr = 5352,
  VAESDECrm = 5353,
  VAESDECrr = 5354,
  VAESENCLASTYrm = 5355,
  VAESENCLASTYrr = 5356,
  VAESENCLASTZ128rm = 5357,
  VAESENCLASTZ128rr = 5358,
  VAESENCLASTZ256rm = 5359,
  VAESENCLASTZ256rr = 5360,
  VAESENCLASTZrm = 5361,
  VAESENCLASTZrr = 5362,
  VAESENCLASTrm = 5363,
  VAESENCLASTrr = 5364,
  VAESENCYrm = 5365,
  VAESENCYrr = 5366,
  VAESENCZ128rm = 5367,
  VAESENCZ128rr = 5368,
  VAESENCZ256rm = 5369,
  VAESENCZ256rr = 5370,
  VAESENCZrm = 5371,
  VAESENCZrr = 5372,
  VAESENCrm = 5373,
  VAESENCrr = 5374,
  VAESIMCrm = 5375,
  VAESIMCrr = 5376,
  VAESKEYGENASSIST128rm = 5377,
  VAESKEYGENASSIST128rr = 5378,
  VALIGNDZ128rmbi = 5379,
  VALIGNDZ128rmbik = 5380,
  VALIGNDZ128rmbikz = 5381,
  VALIGNDZ128rmi = 5382,
  VALIGNDZ128rmik = 5383,
  VALIGNDZ128rmikz = 5384,
  VALIGNDZ128rri = 5385,
  VALIGNDZ128rrik = 5386,
  VALIGNDZ128rrikz = 5387,
  VALIGNDZ256rmbi = 5388,
  VALIGNDZ256rmbik = 5389,
  VALIGNDZ256rmbikz = 5390,
  VALIGNDZ256rmi = 5391,
  VALIGNDZ256rmik = 5392,
  VALIGNDZ256rmikz = 5393,
  VALIGNDZ256rri = 5394,
  VALIGNDZ256rrik = 5395,
  VALIGNDZ256rrikz = 5396,
  VALIGNDZrmbi = 5397,
  VALIGNDZrmbik = 5398,
  VALIGNDZrmbikz = 5399,
  VALIGNDZrmi = 5400,
  VALIGNDZrmik = 5401,
  VALIGNDZrmikz = 5402,
  VALIGNDZrri = 5403,
  VALIGNDZrrik = 5404,
  VALIGNDZrrikz = 5405,
  VALIGNQZ128rmbi = 5406,
  VALIGNQZ128rmbik = 5407,
  VALIGNQZ128rmbikz = 5408,
  VALIGNQZ128rmi = 5409,
  VALIGNQZ128rmik = 5410,
  VALIGNQZ128rmikz = 5411,
  VALIGNQZ128rri = 5412,
  VALIGNQZ128rrik = 5413,
  VALIGNQZ128rrikz = 5414,
  VALIGNQZ256rmbi = 5415,
  VALIGNQZ256rmbik = 5416,
  VALIGNQZ256rmbikz = 5417,
  VALIGNQZ256rmi = 5418,
  VALIGNQZ256rmik = 5419,
  VALIGNQZ256rmikz = 5420,
  VALIGNQZ256rri = 5421,
  VALIGNQZ256rrik = 5422,
  VALIGNQZ256rrikz = 5423,
  VALIGNQZrmbi = 5424,
  VALIGNQZrmbik = 5425,
  VALIGNQZrmbikz = 5426,
  VALIGNQZrmi = 5427,
  VALIGNQZrmik = 5428,
  VALIGNQZrmikz = 5429,
  VALIGNQZrri = 5430,
  VALIGNQZrrik = 5431,
  VALIGNQZrrikz = 5432,
  VANDNPDYrm = 5433,
  VANDNPDYrr = 5434,
  VANDNPDZ128rm = 5435,
  VANDNPDZ128rmb = 5436,
  VANDNPDZ128rmbk = 5437,
  VANDNPDZ128rmbkz = 5438,
  VANDNPDZ128rmk = 5439,
  VANDNPDZ128rmkz = 5440,
  VANDNPDZ128rr = 5441,
  VANDNPDZ128rrk = 5442,
  VANDNPDZ128rrkz = 5443,
  VANDNPDZ256rm = 5444,
  VANDNPDZ256rmb = 5445,
  VANDNPDZ256rmbk = 5446,
  VANDNPDZ256rmbkz = 5447,
  VANDNPDZ256rmk = 5448,
  VANDNPDZ256rmkz = 5449,
  VANDNPDZ256rr = 5450,
  VANDNPDZ256rrk = 5451,
  VANDNPDZ256rrkz = 5452,
  VANDNPDZrm = 5453,
  VANDNPDZrmb = 5454,
  VANDNPDZrmbk = 5455,
  VANDNPDZrmbkz = 5456,
  VANDNPDZrmk = 5457,
  VANDNPDZrmkz = 5458,
  VANDNPDZrr = 5459,
  VANDNPDZrrk = 5460,
  VANDNPDZrrkz = 5461,
  VANDNPDrm = 5462,
  VANDNPDrr = 5463,
  VANDNPSYrm = 5464,
  VANDNPSYrr = 5465,
  VANDNPSZ128rm = 5466,
  VANDNPSZ128rmb = 5467,
  VANDNPSZ128rmbk = 5468,
  VANDNPSZ128rmbkz = 5469,
  VANDNPSZ128rmk = 5470,
  VANDNPSZ128rmkz = 5471,
  VANDNPSZ128rr = 5472,
  VANDNPSZ128rrk = 5473,
  VANDNPSZ128rrkz = 5474,
  VANDNPSZ256rm = 5475,
  VANDNPSZ256rmb = 5476,
  VANDNPSZ256rmbk = 5477,
  VANDNPSZ256rmbkz = 5478,
  VANDNPSZ256rmk = 5479,
  VANDNPSZ256rmkz = 5480,
  VANDNPSZ256rr = 5481,
  VANDNPSZ256rrk = 5482,
  VANDNPSZ256rrkz = 5483,
  VANDNPSZrm = 5484,
  VANDNPSZrmb = 5485,
  VANDNPSZrmbk = 5486,
  VANDNPSZrmbkz = 5487,
  VANDNPSZrmk = 5488,
  VANDNPSZrmkz = 5489,
  VANDNPSZrr = 5490,
  VANDNPSZrrk = 5491,
  VANDNPSZrrkz = 5492,
  VANDNPSrm = 5493,
  VANDNPSrr = 5494,
  VANDPDYrm = 5495,
  VANDPDYrr = 5496,
  VANDPDZ128rm = 5497,
  VANDPDZ128rmb = 5498,
  VANDPDZ128rmbk = 5499,
  VANDPDZ128rmbkz = 5500,
  VANDPDZ128rmk = 5501,
  VANDPDZ128rmkz = 5502,
  VANDPDZ128rr = 5503,
  VANDPDZ128rrk = 5504,
  VANDPDZ128rrkz = 5505,
  VANDPDZ256rm = 5506,
  VANDPDZ256rmb = 5507,
  VANDPDZ256rmbk = 5508,
  VANDPDZ256rmbkz = 5509,
  VANDPDZ256rmk = 5510,
  VANDPDZ256rmkz = 5511,
  VANDPDZ256rr = 5512,
  VANDPDZ256rrk = 5513,
  VANDPDZ256rrkz = 5514,
  VANDPDZrm = 5515,
  VANDPDZrmb = 5516,
  VANDPDZrmbk = 5517,
  VANDPDZrmbkz = 5518,
  VANDPDZrmk = 5519,
  VANDPDZrmkz = 5520,
  VANDPDZrr = 5521,
  VANDPDZrrk = 5522,
  VANDPDZrrkz = 5523,
  VANDPDrm = 5524,
  VANDPDrr = 5525,
  VANDPSYrm = 5526,
  VANDPSYrr = 5527,
  VANDPSZ128rm = 5528,
  VANDPSZ128rmb = 5529,
  VANDPSZ128rmbk = 5530,
  VANDPSZ128rmbkz = 5531,
  VANDPSZ128rmk = 5532,
  VANDPSZ128rmkz = 5533,
  VANDPSZ128rr = 5534,
  VANDPSZ128rrk = 5535,
  VANDPSZ128rrkz = 5536,
  VANDPSZ256rm = 5537,
  VANDPSZ256rmb = 5538,
  VANDPSZ256rmbk = 5539,
  VANDPSZ256rmbkz = 5540,
  VANDPSZ256rmk = 5541,
  VANDPSZ256rmkz = 5542,
  VANDPSZ256rr = 5543,
  VANDPSZ256rrk = 5544,
  VANDPSZ256rrkz = 5545,
  VANDPSZrm = 5546,
  VANDPSZrmb = 5547,
  VANDPSZrmbk = 5548,
  VANDPSZrmbkz = 5549,
  VANDPSZrmk = 5550,
  VANDPSZrmkz = 5551,
  VANDPSZrr = 5552,
  VANDPSZrrk = 5553,
  VANDPSZrrkz = 5554,
  VANDPSrm = 5555,
  VANDPSrr = 5556,
  VASTART_SAVE_XMM_REGS = 5557,
  VBCSTNEBF162PSYrm = 5558,
  VBCSTNEBF162PSrm = 5559,
  VBCSTNESH2PSYrm = 5560,
  VBCSTNESH2PSrm = 5561,
  VBLENDMPDZ128rm = 5562,
  VBLENDMPDZ128rmb = 5563,
  VBLENDMPDZ128rmbk = 5564,
  VBLENDMPDZ128rmbkz = 5565,
  VBLENDMPDZ128rmk = 5566,
  VBLENDMPDZ128rmkz = 5567,
  VBLENDMPDZ128rr = 5568,
  VBLENDMPDZ128rrk = 5569,
  VBLENDMPDZ128rrkz = 5570,
  VBLENDMPDZ256rm = 5571,
  VBLENDMPDZ256rmb = 5572,
  VBLENDMPDZ256rmbk = 5573,
  VBLENDMPDZ256rmbkz = 5574,
  VBLENDMPDZ256rmk = 5575,
  VBLENDMPDZ256rmkz = 5576,
  VBLENDMPDZ256rr = 5577,
  VBLENDMPDZ256rrk = 5578,
  VBLENDMPDZ256rrkz = 5579,
  VBLENDMPDZrm = 5580,
  VBLENDMPDZrmb = 5581,
  VBLENDMPDZrmbk = 5582,
  VBLENDMPDZrmbkz = 5583,
  VBLENDMPDZrmk = 5584,
  VBLENDMPDZrmkz = 5585,
  VBLENDMPDZrr = 5586,
  VBLENDMPDZrrk = 5587,
  VBLENDMPDZrrkz = 5588,
  VBLENDMPSZ128rm = 5589,
  VBLENDMPSZ128rmb = 5590,
  VBLENDMPSZ128rmbk = 5591,
  VBLENDMPSZ128rmbkz = 5592,
  VBLENDMPSZ128rmk = 5593,
  VBLENDMPSZ128rmkz = 5594,
  VBLENDMPSZ128rr = 5595,
  VBLENDMPSZ128rrk = 5596,
  VBLENDMPSZ128rrkz = 5597,
  VBLENDMPSZ256rm = 5598,
  VBLENDMPSZ256rmb = 5599,
  VBLENDMPSZ256rmbk = 5600,
  VBLENDMPSZ256rmbkz = 5601,
  VBLENDMPSZ256rmk = 5602,
  VBLENDMPSZ256rmkz = 5603,
  VBLENDMPSZ256rr = 5604,
  VBLENDMPSZ256rrk = 5605,
  VBLENDMPSZ256rrkz = 5606,
  VBLENDMPSZrm = 5607,
  VBLENDMPSZrmb = 5608,
  VBLENDMPSZrmbk = 5609,
  VBLENDMPSZrmbkz = 5610,
  VBLENDMPSZrmk = 5611,
  VBLENDMPSZrmkz = 5612,
  VBLENDMPSZrr = 5613,
  VBLENDMPSZrrk = 5614,
  VBLENDMPSZrrkz = 5615,
  VBLENDPDYrmi = 5616,
  VBLENDPDYrri = 5617,
  VBLENDPDrmi = 5618,
  VBLENDPDrri = 5619,
  VBLENDPSYrmi = 5620,
  VBLENDPSYrri = 5621,
  VBLENDPSrmi = 5622,
  VBLENDPSrri = 5623,
  VBLENDVPDYrmr = 5624,
  VBLENDVPDYrrr = 5625,
  VBLENDVPDrmr = 5626,
  VBLENDVPDrrr = 5627,
  VBLENDVPSYrmr = 5628,
  VBLENDVPSYrrr = 5629,
  VBLENDVPSrmr = 5630,
  VBLENDVPSrrr = 5631,
  VBROADCASTF128rm = 5632,
  VBROADCASTF32X2Z256rm = 5633,
  VBROADCASTF32X2Z256rmk = 5634,
  VBROADCASTF32X2Z256rmkz = 5635,
  VBROADCASTF32X2Z256rr = 5636,
  VBROADCASTF32X2Z256rrk = 5637,
  VBROADCASTF32X2Z256rrkz = 5638,
  VBROADCASTF32X2Zrm = 5639,
  VBROADCASTF32X2Zrmk = 5640,
  VBROADCASTF32X2Zrmkz = 5641,
  VBROADCASTF32X2Zrr = 5642,
  VBROADCASTF32X2Zrrk = 5643,
  VBROADCASTF32X2Zrrkz = 5644,
  VBROADCASTF32X4Z256rm = 5645,
  VBROADCASTF32X4Z256rmk = 5646,
  VBROADCASTF32X4Z256rmkz = 5647,
  VBROADCASTF32X4Zrm = 5648,
  VBROADCASTF32X4Zrmk = 5649,
  VBROADCASTF32X4Zrmkz = 5650,
  VBROADCASTF32X8Zrm = 5651,
  VBROADCASTF32X8Zrmk = 5652,
  VBROADCASTF32X8Zrmkz = 5653,
  VBROADCASTF64X2Z256rm = 5654,
  VBROADCASTF64X2Z256rmk = 5655,
  VBROADCASTF64X2Z256rmkz = 5656,
  VBROADCASTF64X2Zrm = 5657,
  VBROADCASTF64X2Zrmk = 5658,
  VBROADCASTF64X2Zrmkz = 5659,
  VBROADCASTF64X4Zrm = 5660,
  VBROADCASTF64X4Zrmk = 5661,
  VBROADCASTF64X4Zrmkz = 5662,
  VBROADCASTI128rm = 5663,
  VBROADCASTI32X2Z128rm = 5664,
  VBROADCASTI32X2Z128rmk = 5665,
  VBROADCASTI32X2Z128rmkz = 5666,
  VBROADCASTI32X2Z128rr = 5667,
  VBROADCASTI32X2Z128rrk = 5668,
  VBROADCASTI32X2Z128rrkz = 5669,
  VBROADCASTI32X2Z256rm = 5670,
  VBROADCASTI32X2Z256rmk = 5671,
  VBROADCASTI32X2Z256rmkz = 5672,
  VBROADCASTI32X2Z256rr = 5673,
  VBROADCASTI32X2Z256rrk = 5674,
  VBROADCASTI32X2Z256rrkz = 5675,
  VBROADCASTI32X2Zrm = 5676,
  VBROADCASTI32X2Zrmk = 5677,
  VBROADCASTI32X2Zrmkz = 5678,
  VBROADCASTI32X2Zrr = 5679,
  VBROADCASTI32X2Zrrk = 5680,
  VBROADCASTI32X2Zrrkz = 5681,
  VBROADCASTI32X4Z256rm = 5682,
  VBROADCASTI32X4Z256rmk = 5683,
  VBROADCASTI32X4Z256rmkz = 5684,
  VBROADCASTI32X4Zrm = 5685,
  VBROADCASTI32X4Zrmk = 5686,
  VBROADCASTI32X4Zrmkz = 5687,
  VBROADCASTI32X8Zrm = 5688,
  VBROADCASTI32X8Zrmk = 5689,
  VBROADCASTI32X8Zrmkz = 5690,
  VBROADCASTI64X2Z256rm = 5691,
  VBROADCASTI64X2Z256rmk = 5692,
  VBROADCASTI64X2Z256rmkz = 5693,
  VBROADCASTI64X2Zrm = 5694,
  VBROADCASTI64X2Zrmk = 5695,
  VBROADCASTI64X2Zrmkz = 5696,
  VBROADCASTI64X4Zrm = 5697,
  VBROADCASTI64X4Zrmk = 5698,
  VBROADCASTI64X4Zrmkz = 5699,
  VBROADCASTSDYrm = 5700,
  VBROADCASTSDYrr = 5701,
  VBROADCASTSDZ256rm = 5702,
  VBROADCASTSDZ256rmk = 5703,
  VBROADCASTSDZ256rmkz = 5704,
  VBROADCASTSDZ256rr = 5705,
  VBROADCASTSDZ256rrk = 5706,
  VBROADCASTSDZ256rrkz = 5707,
  VBROADCASTSDZrm = 5708,
  VBROADCASTSDZrmk = 5709,
  VBROADCASTSDZrmkz = 5710,
  VBROADCASTSDZrr = 5711,
  VBROADCASTSDZrrk = 5712,
  VBROADCASTSDZrrkz = 5713,
  VBROADCASTSSYrm = 5714,
  VBROADCASTSSYrr = 5715,
  VBROADCASTSSZ128rm = 5716,
  VBROADCASTSSZ128rmk = 5717,
  VBROADCASTSSZ128rmkz = 5718,
  VBROADCASTSSZ128rr = 5719,
  VBROADCASTSSZ128rrk = 5720,
  VBROADCASTSSZ128rrkz = 5721,
  VBROADCASTSSZ256rm = 5722,
  VBROADCASTSSZ256rmk = 5723,
  VBROADCASTSSZ256rmkz = 5724,
  VBROADCASTSSZ256rr = 5725,
  VBROADCASTSSZ256rrk = 5726,
  VBROADCASTSSZ256rrkz = 5727,
  VBROADCASTSSZrm = 5728,
  VBROADCASTSSZrmk = 5729,
  VBROADCASTSSZrmkz = 5730,
  VBROADCASTSSZrr = 5731,
  VBROADCASTSSZrrk = 5732,
  VBROADCASTSSZrrkz = 5733,
  VBROADCASTSSrm = 5734,
  VBROADCASTSSrr = 5735,
  VCMPBF16Z128rmbi = 5736,
  VCMPBF16Z128rmbik = 5737,
  VCMPBF16Z128rmi = 5738,
  VCMPBF16Z128rmik = 5739,
  VCMPBF16Z128rri = 5740,
  VCMPBF16Z128rrik = 5741,
  VCMPBF16Z256rmbi = 5742,
  VCMPBF16Z256rmbik = 5743,
  VCMPBF16Z256rmi = 5744,
  VCMPBF16Z256rmik = 5745,
  VCMPBF16Z256rri = 5746,
  VCMPBF16Z256rrik = 5747,
  VCMPBF16Zrmbi = 5748,
  VCMPBF16Zrmbik = 5749,
  VCMPBF16Zrmi = 5750,
  VCMPBF16Zrmik = 5751,
  VCMPBF16Zrri = 5752,
  VCMPBF16Zrrik = 5753,
  VCMPPDYrmi = 5754,
  VCMPPDYrri = 5755,
  VCMPPDZ128rmbi = 5756,
  VCMPPDZ128rmbik = 5757,
  VCMPPDZ128rmi = 5758,
  VCMPPDZ128rmik = 5759,
  VCMPPDZ128rri = 5760,
  VCMPPDZ128rrik = 5761,
  VCMPPDZ256rmbi = 5762,
  VCMPPDZ256rmbik = 5763,
  VCMPPDZ256rmi = 5764,
  VCMPPDZ256rmik = 5765,
  VCMPPDZ256rri = 5766,
  VCMPPDZ256rrik = 5767,
  VCMPPDZrmbi = 5768,
  VCMPPDZrmbik = 5769,
  VCMPPDZrmi = 5770,
  VCMPPDZrmik = 5771,
  VCMPPDZrri = 5772,
  VCMPPDZrrib = 5773,
  VCMPPDZrribk = 5774,
  VCMPPDZrrik = 5775,
  VCMPPDrmi = 5776,
  VCMPPDrri = 5777,
  VCMPPHZ128rmbi = 5778,
  VCMPPHZ128rmbik = 5779,
  VCMPPHZ128rmi = 5780,
  VCMPPHZ128rmik = 5781,
  VCMPPHZ128rri = 5782,
  VCMPPHZ128rrik = 5783,
  VCMPPHZ256rmbi = 5784,
  VCMPPHZ256rmbik = 5785,
  VCMPPHZ256rmi = 5786,
  VCMPPHZ256rmik = 5787,
  VCMPPHZ256rri = 5788,
  VCMPPHZ256rrik = 5789,
  VCMPPHZrmbi = 5790,
  VCMPPHZrmbik = 5791,
  VCMPPHZrmi = 5792,
  VCMPPHZrmik = 5793,
  VCMPPHZrri = 5794,
  VCMPPHZrrib = 5795,
  VCMPPHZrribk = 5796,
  VCMPPHZrrik = 5797,
  VCMPPSYrmi = 5798,
  VCMPPSYrri = 5799,
  VCMPPSZ128rmbi = 5800,
  VCMPPSZ128rmbik = 5801,
  VCMPPSZ128rmi = 5802,
  VCMPPSZ128rmik = 5803,
  VCMPPSZ128rri = 5804,
  VCMPPSZ128rrik = 5805,
  VCMPPSZ256rmbi = 5806,
  VCMPPSZ256rmbik = 5807,
  VCMPPSZ256rmi = 5808,
  VCMPPSZ256rmik = 5809,
  VCMPPSZ256rri = 5810,
  VCMPPSZ256rrik = 5811,
  VCMPPSZrmbi = 5812,
  VCMPPSZrmbik = 5813,
  VCMPPSZrmi = 5814,
  VCMPPSZrmik = 5815,
  VCMPPSZrri = 5816,
  VCMPPSZrrib = 5817,
  VCMPPSZrribk = 5818,
  VCMPPSZrrik = 5819,
  VCMPPSrmi = 5820,
  VCMPPSrri = 5821,
  VCMPSDZrmi = 5822,
  VCMPSDZrmi_Int = 5823,
  VCMPSDZrmik_Int = 5824,
  VCMPSDZrri = 5825,
  VCMPSDZrri_Int = 5826,
  VCMPSDZrrib_Int = 5827,
  VCMPSDZrribk_Int = 5828,
  VCMPSDZrrik_Int = 5829,
  VCMPSDrmi = 5830,
  VCMPSDrmi_Int = 5831,
  VCMPSDrri = 5832,
  VCMPSDrri_Int = 5833,
  VCMPSHZrmi = 5834,
  VCMPSHZrmi_Int = 5835,
  VCMPSHZrmik_Int = 5836,
  VCMPSHZrri = 5837,
  VCMPSHZrri_Int = 5838,
  VCMPSHZrrib_Int = 5839,
  VCMPSHZrribk_Int = 5840,
  VCMPSHZrrik_Int = 5841,
  VCMPSSZrmi = 5842,
  VCMPSSZrmi_Int = 5843,
  VCMPSSZrmik_Int = 5844,
  VCMPSSZrri = 5845,
  VCMPSSZrri_Int = 5846,
  VCMPSSZrrib_Int = 5847,
  VCMPSSZrribk_Int = 5848,
  VCMPSSZrrik_Int = 5849,
  VCMPSSrmi = 5850,
  VCMPSSrmi_Int = 5851,
  VCMPSSrri = 5852,
  VCMPSSrri_Int = 5853,
  VCOMISBF16Zrm = 5854,
  VCOMISBF16Zrm_Int = 5855,
  VCOMISBF16Zrr = 5856,
  VCOMISBF16Zrr_Int = 5857,
  VCOMISDZrm = 5858,
  VCOMISDZrm_Int = 5859,
  VCOMISDZrr = 5860,
  VCOMISDZrr_Int = 5861,
  VCOMISDZrrb = 5862,
  VCOMISDrm = 5863,
  VCOMISDrm_Int = 5864,
  VCOMISDrr = 5865,
  VCOMISDrr_Int = 5866,
  VCOMISHZrm = 5867,
  VCOMISHZrm_Int = 5868,
  VCOMISHZrr = 5869,
  VCOMISHZrr_Int = 5870,
  VCOMISHZrrb = 5871,
  VCOMISSZrm = 5872,
  VCOMISSZrm_Int = 5873,
  VCOMISSZrr = 5874,
  VCOMISSZrr_Int = 5875,
  VCOMISSZrrb = 5876,
  VCOMISSrm = 5877,
  VCOMISSrm_Int = 5878,
  VCOMISSrr = 5879,
  VCOMISSrr_Int = 5880,
  VCOMPRESSPDZ128mr = 5881,
  VCOMPRESSPDZ128mrk = 5882,
  VCOMPRESSPDZ128rr = 5883,
  VCOMPRESSPDZ128rrk = 5884,
  VCOMPRESSPDZ128rrkz = 5885,
  VCOMPRESSPDZ256mr = 5886,
  VCOMPRESSPDZ256mrk = 5887,
  VCOMPRESSPDZ256rr = 5888,
  VCOMPRESSPDZ256rrk = 5889,
  VCOMPRESSPDZ256rrkz = 5890,
  VCOMPRESSPDZmr = 5891,
  VCOMPRESSPDZmrk = 5892,
  VCOMPRESSPDZrr = 5893,
  VCOMPRESSPDZrrk = 5894,
  VCOMPRESSPDZrrkz = 5895,
  VCOMPRESSPSZ128mr = 5896,
  VCOMPRESSPSZ128mrk = 5897,
  VCOMPRESSPSZ128rr = 5898,
  VCOMPRESSPSZ128rrk = 5899,
  VCOMPRESSPSZ128rrkz = 5900,
  VCOMPRESSPSZ256mr = 5901,
  VCOMPRESSPSZ256mrk = 5902,
  VCOMPRESSPSZ256rr = 5903,
  VCOMPRESSPSZ256rrk = 5904,
  VCOMPRESSPSZ256rrkz = 5905,
  VCOMPRESSPSZmr = 5906,
  VCOMPRESSPSZmrk = 5907,
  VCOMPRESSPSZrr = 5908,
  VCOMPRESSPSZrrk = 5909,
  VCOMPRESSPSZrrkz = 5910,
  VCOMXSDZrm_Int = 5911,
  VCOMXSDZrr_Int = 5912,
  VCOMXSDZrrb_Int = 5913,
  VCOMXSHZrm_Int = 5914,
  VCOMXSHZrr_Int = 5915,
  VCOMXSHZrrb_Int = 5916,
  VCOMXSSZrm_Int = 5917,
  VCOMXSSZrr_Int = 5918,
  VCOMXSSZrrb_Int = 5919,
  VCVT2PH2BF8SZ128rm = 5920,
  VCVT2PH2BF8SZ128rmb = 5921,
  VCVT2PH2BF8SZ128rmbk = 5922,
  VCVT2PH2BF8SZ128rmbkz = 5923,
  VCVT2PH2BF8SZ128rmk = 5924,
  VCVT2PH2BF8SZ128rmkz = 5925,
  VCVT2PH2BF8SZ128rr = 5926,
  VCVT2PH2BF8SZ128rrk = 5927,
  VCVT2PH2BF8SZ128rrkz = 5928,
  VCVT2PH2BF8SZ256rm = 5929,
  VCVT2PH2BF8SZ256rmb = 5930,
  VCVT2PH2BF8SZ256rmbk = 5931,
  VCVT2PH2BF8SZ256rmbkz = 5932,
  VCVT2PH2BF8SZ256rmk = 5933,
  VCVT2PH2BF8SZ256rmkz = 5934,
  VCVT2PH2BF8SZ256rr = 5935,
  VCVT2PH2BF8SZ256rrk = 5936,
  VCVT2PH2BF8SZ256rrkz = 5937,
  VCVT2PH2BF8SZrm = 5938,
  VCVT2PH2BF8SZrmb = 5939,
  VCVT2PH2BF8SZrmbk = 5940,
  VCVT2PH2BF8SZrmbkz = 5941,
  VCVT2PH2BF8SZrmk = 5942,
  VCVT2PH2BF8SZrmkz = 5943,
  VCVT2PH2BF8SZrr = 5944,
  VCVT2PH2BF8SZrrk = 5945,
  VCVT2PH2BF8SZrrkz = 5946,
  VCVT2PH2BF8Z128rm = 5947,
  VCVT2PH2BF8Z128rmb = 5948,
  VCVT2PH2BF8Z128rmbk = 5949,
  VCVT2PH2BF8Z128rmbkz = 5950,
  VCVT2PH2BF8Z128rmk = 5951,
  VCVT2PH2BF8Z128rmkz = 5952,
  VCVT2PH2BF8Z128rr = 5953,
  VCVT2PH2BF8Z128rrk = 5954,
  VCVT2PH2BF8Z128rrkz = 5955,
  VCVT2PH2BF8Z256rm = 5956,
  VCVT2PH2BF8Z256rmb = 5957,
  VCVT2PH2BF8Z256rmbk = 5958,
  VCVT2PH2BF8Z256rmbkz = 5959,
  VCVT2PH2BF8Z256rmk = 5960,
  VCVT2PH2BF8Z256rmkz = 5961,
  VCVT2PH2BF8Z256rr = 5962,
  VCVT2PH2BF8Z256rrk = 5963,
  VCVT2PH2BF8Z256rrkz = 5964,
  VCVT2PH2BF8Zrm = 5965,
  VCVT2PH2BF8Zrmb = 5966,
  VCVT2PH2BF8Zrmbk = 5967,
  VCVT2PH2BF8Zrmbkz = 5968,
  VCVT2PH2BF8Zrmk = 5969,
  VCVT2PH2BF8Zrmkz = 5970,
  VCVT2PH2BF8Zrr = 5971,
  VCVT2PH2BF8Zrrk = 5972,
  VCVT2PH2BF8Zrrkz = 5973,
  VCVT2PH2HF8SZ128rm = 5974,
  VCVT2PH2HF8SZ128rmb = 5975,
  VCVT2PH2HF8SZ128rmbk = 5976,
  VCVT2PH2HF8SZ128rmbkz = 5977,
  VCVT2PH2HF8SZ128rmk = 5978,
  VCVT2PH2HF8SZ128rmkz = 5979,
  VCVT2PH2HF8SZ128rr = 5980,
  VCVT2PH2HF8SZ128rrk = 5981,
  VCVT2PH2HF8SZ128rrkz = 5982,
  VCVT2PH2HF8SZ256rm = 5983,
  VCVT2PH2HF8SZ256rmb = 5984,
  VCVT2PH2HF8SZ256rmbk = 5985,
  VCVT2PH2HF8SZ256rmbkz = 5986,
  VCVT2PH2HF8SZ256rmk = 5987,
  VCVT2PH2HF8SZ256rmkz = 5988,
  VCVT2PH2HF8SZ256rr = 5989,
  VCVT2PH2HF8SZ256rrk = 5990,
  VCVT2PH2HF8SZ256rrkz = 5991,
  VCVT2PH2HF8SZrm = 5992,
  VCVT2PH2HF8SZrmb = 5993,
  VCVT2PH2HF8SZrmbk = 5994,
  VCVT2PH2HF8SZrmbkz = 5995,
  VCVT2PH2HF8SZrmk = 5996,
  VCVT2PH2HF8SZrmkz = 5997,
  VCVT2PH2HF8SZrr = 5998,
  VCVT2PH2HF8SZrrk = 5999,
  VCVT2PH2HF8SZrrkz = 6000,
  VCVT2PH2HF8Z128rm = 6001,
  VCVT2PH2HF8Z128rmb = 6002,
  VCVT2PH2HF8Z128rmbk = 6003,
  VCVT2PH2HF8Z128rmbkz = 6004,
  VCVT2PH2HF8Z128rmk = 6005,
  VCVT2PH2HF8Z128rmkz = 6006,
  VCVT2PH2HF8Z128rr = 6007,
  VCVT2PH2HF8Z128rrk = 6008,
  VCVT2PH2HF8Z128rrkz = 6009,
  VCVT2PH2HF8Z256rm = 6010,
  VCVT2PH2HF8Z256rmb = 6011,
  VCVT2PH2HF8Z256rmbk = 6012,
  VCVT2PH2HF8Z256rmbkz = 6013,
  VCVT2PH2HF8Z256rmk = 6014,
  VCVT2PH2HF8Z256rmkz = 6015,
  VCVT2PH2HF8Z256rr = 6016,
  VCVT2PH2HF8Z256rrk = 6017,
  VCVT2PH2HF8Z256rrkz = 6018,
  VCVT2PH2HF8Zrm = 6019,
  VCVT2PH2HF8Zrmb = 6020,
  VCVT2PH2HF8Zrmbk = 6021,
  VCVT2PH2HF8Zrmbkz = 6022,
  VCVT2PH2HF8Zrmk = 6023,
  VCVT2PH2HF8Zrmkz = 6024,
  VCVT2PH2HF8Zrr = 6025,
  VCVT2PH2HF8Zrrk = 6026,
  VCVT2PH2HF8Zrrkz = 6027,
  VCVT2PS2PHXZ128rm = 6028,
  VCVT2PS2PHXZ128rmb = 6029,
  VCVT2PS2PHXZ128rmbk = 6030,
  VCVT2PS2PHXZ128rmbkz = 6031,
  VCVT2PS2PHXZ128rmk = 6032,
  VCVT2PS2PHXZ128rmkz = 6033,
  VCVT2PS2PHXZ128rr = 6034,
  VCVT2PS2PHXZ128rrk = 6035,
  VCVT2PS2PHXZ128rrkz = 6036,
  VCVT2PS2PHXZ256rm = 6037,
  VCVT2PS2PHXZ256rmb = 6038,
  VCVT2PS2PHXZ256rmbk = 6039,
  VCVT2PS2PHXZ256rmbkz = 6040,
  VCVT2PS2PHXZ256rmk = 6041,
  VCVT2PS2PHXZ256rmkz = 6042,
  VCVT2PS2PHXZ256rr = 6043,
  VCVT2PS2PHXZ256rrk = 6044,
  VCVT2PS2PHXZ256rrkz = 6045,
  VCVT2PS2PHXZrm = 6046,
  VCVT2PS2PHXZrmb = 6047,
  VCVT2PS2PHXZrmbk = 6048,
  VCVT2PS2PHXZrmbkz = 6049,
  VCVT2PS2PHXZrmk = 6050,
  VCVT2PS2PHXZrmkz = 6051,
  VCVT2PS2PHXZrr = 6052,
  VCVT2PS2PHXZrrb = 6053,
  VCVT2PS2PHXZrrbk = 6054,
  VCVT2PS2PHXZrrbkz = 6055,
  VCVT2PS2PHXZrrk = 6056,
  VCVT2PS2PHXZrrkz = 6057,
  VCVTBF162IBSZ128rm = 6058,
  VCVTBF162IBSZ128rmb = 6059,
  VCVTBF162IBSZ128rmbk = 6060,
  VCVTBF162IBSZ128rmbkz = 6061,
  VCVTBF162IBSZ128rmk = 6062,
  VCVTBF162IBSZ128rmkz = 6063,
  VCVTBF162IBSZ128rr = 6064,
  VCVTBF162IBSZ128rrk = 6065,
  VCVTBF162IBSZ128rrkz = 6066,
  VCVTBF162IBSZ256rm = 6067,
  VCVTBF162IBSZ256rmb = 6068,
  VCVTBF162IBSZ256rmbk = 6069,
  VCVTBF162IBSZ256rmbkz = 6070,
  VCVTBF162IBSZ256rmk = 6071,
  VCVTBF162IBSZ256rmkz = 6072,
  VCVTBF162IBSZ256rr = 6073,
  VCVTBF162IBSZ256rrk = 6074,
  VCVTBF162IBSZ256rrkz = 6075,
  VCVTBF162IBSZrm = 6076,
  VCVTBF162IBSZrmb = 6077,
  VCVTBF162IBSZrmbk = 6078,
  VCVTBF162IBSZrmbkz = 6079,
  VCVTBF162IBSZrmk = 6080,
  VCVTBF162IBSZrmkz = 6081,
  VCVTBF162IBSZrr = 6082,
  VCVTBF162IBSZrrk = 6083,
  VCVTBF162IBSZrrkz = 6084,
  VCVTBF162IUBSZ128rm = 6085,
  VCVTBF162IUBSZ128rmb = 6086,
  VCVTBF162IUBSZ128rmbk = 6087,
  VCVTBF162IUBSZ128rmbkz = 6088,
  VCVTBF162IUBSZ128rmk = 6089,
  VCVTBF162IUBSZ128rmkz = 6090,
  VCVTBF162IUBSZ128rr = 6091,
  VCVTBF162IUBSZ128rrk = 6092,
  VCVTBF162IUBSZ128rrkz = 6093,
  VCVTBF162IUBSZ256rm = 6094,
  VCVTBF162IUBSZ256rmb = 6095,
  VCVTBF162IUBSZ256rmbk = 6096,
  VCVTBF162IUBSZ256rmbkz = 6097,
  VCVTBF162IUBSZ256rmk = 6098,
  VCVTBF162IUBSZ256rmkz = 6099,
  VCVTBF162IUBSZ256rr = 6100,
  VCVTBF162IUBSZ256rrk = 6101,
  VCVTBF162IUBSZ256rrkz = 6102,
  VCVTBF162IUBSZrm = 6103,
  VCVTBF162IUBSZrmb = 6104,
  VCVTBF162IUBSZrmbk = 6105,
  VCVTBF162IUBSZrmbkz = 6106,
  VCVTBF162IUBSZrmk = 6107,
  VCVTBF162IUBSZrmkz = 6108,
  VCVTBF162IUBSZrr = 6109,
  VCVTBF162IUBSZrrk = 6110,
  VCVTBF162IUBSZrrkz = 6111,
  VCVTBIASPH2BF8SZ128rm = 6112,
  VCVTBIASPH2BF8SZ128rmb = 6113,
  VCVTBIASPH2BF8SZ128rmbk = 6114,
  VCVTBIASPH2BF8SZ128rmbkz = 6115,
  VCVTBIASPH2BF8SZ128rmk = 6116,
  VCVTBIASPH2BF8SZ128rmkz = 6117,
  VCVTBIASPH2BF8SZ128rr = 6118,
  VCVTBIASPH2BF8SZ128rrk = 6119,
  VCVTBIASPH2BF8SZ128rrkz = 6120,
  VCVTBIASPH2BF8SZ256rm = 6121,
  VCVTBIASPH2BF8SZ256rmb = 6122,
  VCVTBIASPH2BF8SZ256rmbk = 6123,
  VCVTBIASPH2BF8SZ256rmbkz = 6124,
  VCVTBIASPH2BF8SZ256rmk = 6125,
  VCVTBIASPH2BF8SZ256rmkz = 6126,
  VCVTBIASPH2BF8SZ256rr = 6127,
  VCVTBIASPH2BF8SZ256rrk = 6128,
  VCVTBIASPH2BF8SZ256rrkz = 6129,
  VCVTBIASPH2BF8SZrm = 6130,
  VCVTBIASPH2BF8SZrmb = 6131,
  VCVTBIASPH2BF8SZrmbk = 6132,
  VCVTBIASPH2BF8SZrmbkz = 6133,
  VCVTBIASPH2BF8SZrmk = 6134,
  VCVTBIASPH2BF8SZrmkz = 6135,
  VCVTBIASPH2BF8SZrr = 6136,
  VCVTBIASPH2BF8SZrrk = 6137,
  VCVTBIASPH2BF8SZrrkz = 6138,
  VCVTBIASPH2BF8Z128rm = 6139,
  VCVTBIASPH2BF8Z128rmb = 6140,
  VCVTBIASPH2BF8Z128rmbk = 6141,
  VCVTBIASPH2BF8Z128rmbkz = 6142,
  VCVTBIASPH2BF8Z128rmk = 6143,
  VCVTBIASPH2BF8Z128rmkz = 6144,
  VCVTBIASPH2BF8Z128rr = 6145,
  VCVTBIASPH2BF8Z128rrk = 6146,
  VCVTBIASPH2BF8Z128rrkz = 6147,
  VCVTBIASPH2BF8Z256rm = 6148,
  VCVTBIASPH2BF8Z256rmb = 6149,
  VCVTBIASPH2BF8Z256rmbk = 6150,
  VCVTBIASPH2BF8Z256rmbkz = 6151,
  VCVTBIASPH2BF8Z256rmk = 6152,
  VCVTBIASPH2BF8Z256rmkz = 6153,
  VCVTBIASPH2BF8Z256rr = 6154,
  VCVTBIASPH2BF8Z256rrk = 6155,
  VCVTBIASPH2BF8Z256rrkz = 6156,
  VCVTBIASPH2BF8Zrm = 6157,
  VCVTBIASPH2BF8Zrmb = 6158,
  VCVTBIASPH2BF8Zrmbk = 6159,
  VCVTBIASPH2BF8Zrmbkz = 6160,
  VCVTBIASPH2BF8Zrmk = 6161,
  VCVTBIASPH2BF8Zrmkz = 6162,
  VCVTBIASPH2BF8Zrr = 6163,
  VCVTBIASPH2BF8Zrrk = 6164,
  VCVTBIASPH2BF8Zrrkz = 6165,
  VCVTBIASPH2HF8SZ128rm = 6166,
  VCVTBIASPH2HF8SZ128rmb = 6167,
  VCVTBIASPH2HF8SZ128rmbk = 6168,
  VCVTBIASPH2HF8SZ128rmbkz = 6169,
  VCVTBIASPH2HF8SZ128rmk = 6170,
  VCVTBIASPH2HF8SZ128rmkz = 6171,
  VCVTBIASPH2HF8SZ128rr = 6172,
  VCVTBIASPH2HF8SZ128rrk = 6173,
  VCVTBIASPH2HF8SZ128rrkz = 6174,
  VCVTBIASPH2HF8SZ256rm = 6175,
  VCVTBIASPH2HF8SZ256rmb = 6176,
  VCVTBIASPH2HF8SZ256rmbk = 6177,
  VCVTBIASPH2HF8SZ256rmbkz = 6178,
  VCVTBIASPH2HF8SZ256rmk = 6179,
  VCVTBIASPH2HF8SZ256rmkz = 6180,
  VCVTBIASPH2HF8SZ256rr = 6181,
  VCVTBIASPH2HF8SZ256rrk = 6182,
  VCVTBIASPH2HF8SZ256rrkz = 6183,
  VCVTBIASPH2HF8SZrm = 6184,
  VCVTBIASPH2HF8SZrmb = 6185,
  VCVTBIASPH2HF8SZrmbk = 6186,
  VCVTBIASPH2HF8SZrmbkz = 6187,
  VCVTBIASPH2HF8SZrmk = 6188,
  VCVTBIASPH2HF8SZrmkz = 6189,
  VCVTBIASPH2HF8SZrr = 6190,
  VCVTBIASPH2HF8SZrrk = 6191,
  VCVTBIASPH2HF8SZrrkz = 6192,
  VCVTBIASPH2HF8Z128rm = 6193,
  VCVTBIASPH2HF8Z128rmb = 6194,
  VCVTBIASPH2HF8Z128rmbk = 6195,
  VCVTBIASPH2HF8Z128rmbkz = 6196,
  VCVTBIASPH2HF8Z128rmk = 6197,
  VCVTBIASPH2HF8Z128rmkz = 6198,
  VCVTBIASPH2HF8Z128rr = 6199,
  VCVTBIASPH2HF8Z128rrk = 6200,
  VCVTBIASPH2HF8Z128rrkz = 6201,
  VCVTBIASPH2HF8Z256rm = 6202,
  VCVTBIASPH2HF8Z256rmb = 6203,
  VCVTBIASPH2HF8Z256rmbk = 6204,
  VCVTBIASPH2HF8Z256rmbkz = 6205,
  VCVTBIASPH2HF8Z256rmk = 6206,
  VCVTBIASPH2HF8Z256rmkz = 6207,
  VCVTBIASPH2HF8Z256rr = 6208,
  VCVTBIASPH2HF8Z256rrk = 6209,
  VCVTBIASPH2HF8Z256rrkz = 6210,
  VCVTBIASPH2HF8Zrm = 6211,
  VCVTBIASPH2HF8Zrmb = 6212,
  VCVTBIASPH2HF8Zrmbk = 6213,
  VCVTBIASPH2HF8Zrmbkz = 6214,
  VCVTBIASPH2HF8Zrmk = 6215,
  VCVTBIASPH2HF8Zrmkz = 6216,
  VCVTBIASPH2HF8Zrr = 6217,
  VCVTBIASPH2HF8Zrrk = 6218,
  VCVTBIASPH2HF8Zrrkz = 6219,
  VCVTDQ2PDYrm = 6220,
  VCVTDQ2PDYrr = 6221,
  VCVTDQ2PDZ128rm = 6222,
  VCVTDQ2PDZ128rmb = 6223,
  VCVTDQ2PDZ128rmbk = 6224,
  VCVTDQ2PDZ128rmbkz = 6225,
  VCVTDQ2PDZ128rmk = 6226,
  VCVTDQ2PDZ128rmkz = 6227,
  VCVTDQ2PDZ128rr = 6228,
  VCVTDQ2PDZ128rrk = 6229,
  VCVTDQ2PDZ128rrkz = 6230,
  VCVTDQ2PDZ256rm = 6231,
  VCVTDQ2PDZ256rmb = 6232,
  VCVTDQ2PDZ256rmbk = 6233,
  VCVTDQ2PDZ256rmbkz = 6234,
  VCVTDQ2PDZ256rmk = 6235,
  VCVTDQ2PDZ256rmkz = 6236,
  VCVTDQ2PDZ256rr = 6237,
  VCVTDQ2PDZ256rrk = 6238,
  VCVTDQ2PDZ256rrkz = 6239,
  VCVTDQ2PDZrm = 6240,
  VCVTDQ2PDZrmb = 6241,
  VCVTDQ2PDZrmbk = 6242,
  VCVTDQ2PDZrmbkz = 6243,
  VCVTDQ2PDZrmk = 6244,
  VCVTDQ2PDZrmkz = 6245,
  VCVTDQ2PDZrr = 6246,
  VCVTDQ2PDZrrk = 6247,
  VCVTDQ2PDZrrkz = 6248,
  VCVTDQ2PDrm = 6249,
  VCVTDQ2PDrr = 6250,
  VCVTDQ2PHZ128rm = 6251,
  VCVTDQ2PHZ128rmb = 6252,
  VCVTDQ2PHZ128rmbk = 6253,
  VCVTDQ2PHZ128rmbkz = 6254,
  VCVTDQ2PHZ128rmk = 6255,
  VCVTDQ2PHZ128rmkz = 6256,
  VCVTDQ2PHZ128rr = 6257,
  VCVTDQ2PHZ128rrk = 6258,
  VCVTDQ2PHZ128rrkz = 6259,
  VCVTDQ2PHZ256rm = 6260,
  VCVTDQ2PHZ256rmb = 6261,
  VCVTDQ2PHZ256rmbk = 6262,
  VCVTDQ2PHZ256rmbkz = 6263,
  VCVTDQ2PHZ256rmk = 6264,
  VCVTDQ2PHZ256rmkz = 6265,
  VCVTDQ2PHZ256rr = 6266,
  VCVTDQ2PHZ256rrk = 6267,
  VCVTDQ2PHZ256rrkz = 6268,
  VCVTDQ2PHZrm = 6269,
  VCVTDQ2PHZrmb = 6270,
  VCVTDQ2PHZrmbk = 6271,
  VCVTDQ2PHZrmbkz = 6272,
  VCVTDQ2PHZrmk = 6273,
  VCVTDQ2PHZrmkz = 6274,
  VCVTDQ2PHZrr = 6275,
  VCVTDQ2PHZrrb = 6276,
  VCVTDQ2PHZrrbk = 6277,
  VCVTDQ2PHZrrbkz = 6278,
  VCVTDQ2PHZrrk = 6279,
  VCVTDQ2PHZrrkz = 6280,
  VCVTDQ2PSYrm = 6281,
  VCVTDQ2PSYrr = 6282,
  VCVTDQ2PSZ128rm = 6283,
  VCVTDQ2PSZ128rmb = 6284,
  VCVTDQ2PSZ128rmbk = 6285,
  VCVTDQ2PSZ128rmbkz = 6286,
  VCVTDQ2PSZ128rmk = 6287,
  VCVTDQ2PSZ128rmkz = 6288,
  VCVTDQ2PSZ128rr = 6289,
  VCVTDQ2PSZ128rrk = 6290,
  VCVTDQ2PSZ128rrkz = 6291,
  VCVTDQ2PSZ256rm = 6292,
  VCVTDQ2PSZ256rmb = 6293,
  VCVTDQ2PSZ256rmbk = 6294,
  VCVTDQ2PSZ256rmbkz = 6295,
  VCVTDQ2PSZ256rmk = 6296,
  VCVTDQ2PSZ256rmkz = 6297,
  VCVTDQ2PSZ256rr = 6298,
  VCVTDQ2PSZ256rrk = 6299,
  VCVTDQ2PSZ256rrkz = 6300,
  VCVTDQ2PSZrm = 6301,
  VCVTDQ2PSZrmb = 6302,
  VCVTDQ2PSZrmbk = 6303,
  VCVTDQ2PSZrmbkz = 6304,
  VCVTDQ2PSZrmk = 6305,
  VCVTDQ2PSZrmkz = 6306,
  VCVTDQ2PSZrr = 6307,
  VCVTDQ2PSZrrb = 6308,
  VCVTDQ2PSZrrbk = 6309,
  VCVTDQ2PSZrrbkz = 6310,
  VCVTDQ2PSZrrk = 6311,
  VCVTDQ2PSZrrkz = 6312,
  VCVTDQ2PSrm = 6313,
  VCVTDQ2PSrr = 6314,
  VCVTHF82PHZ128rm = 6315,
  VCVTHF82PHZ128rmk = 6316,
  VCVTHF82PHZ128rmkz = 6317,
  VCVTHF82PHZ128rr = 6318,
  VCVTHF82PHZ128rrk = 6319,
  VCVTHF82PHZ128rrkz = 6320,
  VCVTHF82PHZ256rm = 6321,
  VCVTHF82PHZ256rmk = 6322,
  VCVTHF82PHZ256rmkz = 6323,
  VCVTHF82PHZ256rr = 6324,
  VCVTHF82PHZ256rrk = 6325,
  VCVTHF82PHZ256rrkz = 6326,
  VCVTHF82PHZrm = 6327,
  VCVTHF82PHZrmk = 6328,
  VCVTHF82PHZrmkz = 6329,
  VCVTHF82PHZrr = 6330,
  VCVTHF82PHZrrk = 6331,
  VCVTHF82PHZrrkz = 6332,
  VCVTNE2PS2BF16Z128rm = 6333,
  VCVTNE2PS2BF16Z128rmb = 6334,
  VCVTNE2PS2BF16Z128rmbk = 6335,
  VCVTNE2PS2BF16Z128rmbkz = 6336,
  VCVTNE2PS2BF16Z128rmk = 6337,
  VCVTNE2PS2BF16Z128rmkz = 6338,
  VCVTNE2PS2BF16Z128rr = 6339,
  VCVTNE2PS2BF16Z128rrk = 6340,
  VCVTNE2PS2BF16Z128rrkz = 6341,
  VCVTNE2PS2BF16Z256rm = 6342,
  VCVTNE2PS2BF16Z256rmb = 6343,
  VCVTNE2PS2BF16Z256rmbk = 6344,
  VCVTNE2PS2BF16Z256rmbkz = 6345,
  VCVTNE2PS2BF16Z256rmk = 6346,
  VCVTNE2PS2BF16Z256rmkz = 6347,
  VCVTNE2PS2BF16Z256rr = 6348,
  VCVTNE2PS2BF16Z256rrk = 6349,
  VCVTNE2PS2BF16Z256rrkz = 6350,
  VCVTNE2PS2BF16Zrm = 6351,
  VCVTNE2PS2BF16Zrmb = 6352,
  VCVTNE2PS2BF16Zrmbk = 6353,
  VCVTNE2PS2BF16Zrmbkz = 6354,
  VCVTNE2PS2BF16Zrmk = 6355,
  VCVTNE2PS2BF16Zrmkz = 6356,
  VCVTNE2PS2BF16Zrr = 6357,
  VCVTNE2PS2BF16Zrrk = 6358,
  VCVTNE2PS2BF16Zrrkz = 6359,
  VCVTNEEBF162PSYrm = 6360,
  VCVTNEEBF162PSrm = 6361,
  VCVTNEEPH2PSYrm = 6362,
  VCVTNEEPH2PSrm = 6363,
  VCVTNEOBF162PSYrm = 6364,
  VCVTNEOBF162PSrm = 6365,
  VCVTNEOPH2PSYrm = 6366,
  VCVTNEOPH2PSrm = 6367,
  VCVTNEPS2BF16Yrm = 6368,
  VCVTNEPS2BF16Yrr = 6369,
  VCVTNEPS2BF16Z128rm = 6370,
  VCVTNEPS2BF16Z128rmb = 6371,
  VCVTNEPS2BF16Z128rmbk = 6372,
  VCVTNEPS2BF16Z128rmbkz = 6373,
  VCVTNEPS2BF16Z128rmk = 6374,
  VCVTNEPS2BF16Z128rmkz = 6375,
  VCVTNEPS2BF16Z128rr = 6376,
  VCVTNEPS2BF16Z128rrk = 6377,
  VCVTNEPS2BF16Z128rrkz = 6378,
  VCVTNEPS2BF16Z256rm = 6379,
  VCVTNEPS2BF16Z256rmb = 6380,
  VCVTNEPS2BF16Z256rmbk = 6381,
  VCVTNEPS2BF16Z256rmbkz = 6382,
  VCVTNEPS2BF16Z256rmk = 6383,
  VCVTNEPS2BF16Z256rmkz = 6384,
  VCVTNEPS2BF16Z256rr = 6385,
  VCVTNEPS2BF16Z256rrk = 6386,
  VCVTNEPS2BF16Z256rrkz = 6387,
  VCVTNEPS2BF16Zrm = 6388,
  VCVTNEPS2BF16Zrmb = 6389,
  VCVTNEPS2BF16Zrmbk = 6390,
  VCVTNEPS2BF16Zrmbkz = 6391,
  VCVTNEPS2BF16Zrmk = 6392,
  VCVTNEPS2BF16Zrmkz = 6393,
  VCVTNEPS2BF16Zrr = 6394,
  VCVTNEPS2BF16Zrrk = 6395,
  VCVTNEPS2BF16Zrrkz = 6396,
  VCVTNEPS2BF16rm = 6397,
  VCVTNEPS2BF16rr = 6398,
  VCVTPD2DQYrm = 6399,
  VCVTPD2DQYrr = 6400,
  VCVTPD2DQZ128rm = 6401,
  VCVTPD2DQZ128rmb = 6402,
  VCVTPD2DQZ128rmbk = 6403,
  VCVTPD2DQZ128rmbkz = 6404,
  VCVTPD2DQZ128rmk = 6405,
  VCVTPD2DQZ128rmkz = 6406,
  VCVTPD2DQZ128rr = 6407,
  VCVTPD2DQZ128rrk = 6408,
  VCVTPD2DQZ128rrkz = 6409,
  VCVTPD2DQZ256rm = 6410,
  VCVTPD2DQZ256rmb = 6411,
  VCVTPD2DQZ256rmbk = 6412,
  VCVTPD2DQZ256rmbkz = 6413,
  VCVTPD2DQZ256rmk = 6414,
  VCVTPD2DQZ256rmkz = 6415,
  VCVTPD2DQZ256rr = 6416,
  VCVTPD2DQZ256rrk = 6417,
  VCVTPD2DQZ256rrkz = 6418,
  VCVTPD2DQZrm = 6419,
  VCVTPD2DQZrmb = 6420,
  VCVTPD2DQZrmbk = 6421,
  VCVTPD2DQZrmbkz = 6422,
  VCVTPD2DQZrmk = 6423,
  VCVTPD2DQZrmkz = 6424,
  VCVTPD2DQZrr = 6425,
  VCVTPD2DQZrrb = 6426,
  VCVTPD2DQZrrbk = 6427,
  VCVTPD2DQZrrbkz = 6428,
  VCVTPD2DQZrrk = 6429,
  VCVTPD2DQZrrkz = 6430,
  VCVTPD2DQrm = 6431,
  VCVTPD2DQrr = 6432,
  VCVTPD2PHZ128rm = 6433,
  VCVTPD2PHZ128rmb = 6434,
  VCVTPD2PHZ128rmbk = 6435,
  VCVTPD2PHZ128rmbkz = 6436,
  VCVTPD2PHZ128rmk = 6437,
  VCVTPD2PHZ128rmkz = 6438,
  VCVTPD2PHZ128rr = 6439,
  VCVTPD2PHZ128rrk = 6440,
  VCVTPD2PHZ128rrkz = 6441,
  VCVTPD2PHZ256rm = 6442,
  VCVTPD2PHZ256rmb = 6443,
  VCVTPD2PHZ256rmbk = 6444,
  VCVTPD2PHZ256rmbkz = 6445,
  VCVTPD2PHZ256rmk = 6446,
  VCVTPD2PHZ256rmkz = 6447,
  VCVTPD2PHZ256rr = 6448,
  VCVTPD2PHZ256rrk = 6449,
  VCVTPD2PHZ256rrkz = 6450,
  VCVTPD2PHZrm = 6451,
  VCVTPD2PHZrmb = 6452,
  VCVTPD2PHZrmbk = 6453,
  VCVTPD2PHZrmbkz = 6454,
  VCVTPD2PHZrmk = 6455,
  VCVTPD2PHZrmkz = 6456,
  VCVTPD2PHZrr = 6457,
  VCVTPD2PHZrrb = 6458,
  VCVTPD2PHZrrbk = 6459,
  VCVTPD2PHZrrbkz = 6460,
  VCVTPD2PHZrrk = 6461,
  VCVTPD2PHZrrkz = 6462,
  VCVTPD2PSYrm = 6463,
  VCVTPD2PSYrr = 6464,
  VCVTPD2PSZ128rm = 6465,
  VCVTPD2PSZ128rmb = 6466,
  VCVTPD2PSZ128rmbk = 6467,
  VCVTPD2PSZ128rmbkz = 6468,
  VCVTPD2PSZ128rmk = 6469,
  VCVTPD2PSZ128rmkz = 6470,
  VCVTPD2PSZ128rr = 6471,
  VCVTPD2PSZ128rrk = 6472,
  VCVTPD2PSZ128rrkz = 6473,
  VCVTPD2PSZ256rm = 6474,
  VCVTPD2PSZ256rmb = 6475,
  VCVTPD2PSZ256rmbk = 6476,
  VCVTPD2PSZ256rmbkz = 6477,
  VCVTPD2PSZ256rmk = 6478,
  VCVTPD2PSZ256rmkz = 6479,
  VCVTPD2PSZ256rr = 6480,
  VCVTPD2PSZ256rrk = 6481,
  VCVTPD2PSZ256rrkz = 6482,
  VCVTPD2PSZrm = 6483,
  VCVTPD2PSZrmb = 6484,
  VCVTPD2PSZrmbk = 6485,
  VCVTPD2PSZrmbkz = 6486,
  VCVTPD2PSZrmk = 6487,
  VCVTPD2PSZrmkz = 6488,
  VCVTPD2PSZrr = 6489,
  VCVTPD2PSZrrb = 6490,
  VCVTPD2PSZrrbk = 6491,
  VCVTPD2PSZrrbkz = 6492,
  VCVTPD2PSZrrk = 6493,
  VCVTPD2PSZrrkz = 6494,
  VCVTPD2PSrm = 6495,
  VCVTPD2PSrr = 6496,
  VCVTPD2QQZ128rm = 6497,
  VCVTPD2QQZ128rmb = 6498,
  VCVTPD2QQZ128rmbk = 6499,
  VCVTPD2QQZ128rmbkz = 6500,
  VCVTPD2QQZ128rmk = 6501,
  VCVTPD2QQZ128rmkz = 6502,
  VCVTPD2QQZ128rr = 6503,
  VCVTPD2QQZ128rrk = 6504,
  VCVTPD2QQZ128rrkz = 6505,
  VCVTPD2QQZ256rm = 6506,
  VCVTPD2QQZ256rmb = 6507,
  VCVTPD2QQZ256rmbk = 6508,
  VCVTPD2QQZ256rmbkz = 6509,
  VCVTPD2QQZ256rmk = 6510,
  VCVTPD2QQZ256rmkz = 6511,
  VCVTPD2QQZ256rr = 6512,
  VCVTPD2QQZ256rrk = 6513,
  VCVTPD2QQZ256rrkz = 6514,
  VCVTPD2QQZrm = 6515,
  VCVTPD2QQZrmb = 6516,
  VCVTPD2QQZrmbk = 6517,
  VCVTPD2QQZrmbkz = 6518,
  VCVTPD2QQZrmk = 6519,
  VCVTPD2QQZrmkz = 6520,
  VCVTPD2QQZrr = 6521,
  VCVTPD2QQZrrb = 6522,
  VCVTPD2QQZrrbk = 6523,
  VCVTPD2QQZrrbkz = 6524,
  VCVTPD2QQZrrk = 6525,
  VCVTPD2QQZrrkz = 6526,
  VCVTPD2UDQZ128rm = 6527,
  VCVTPD2UDQZ128rmb = 6528,
  VCVTPD2UDQZ128rmbk = 6529,
  VCVTPD2UDQZ128rmbkz = 6530,
  VCVTPD2UDQZ128rmk = 6531,
  VCVTPD2UDQZ128rmkz = 6532,
  VCVTPD2UDQZ128rr = 6533,
  VCVTPD2UDQZ128rrk = 6534,
  VCVTPD2UDQZ128rrkz = 6535,
  VCVTPD2UDQZ256rm = 6536,
  VCVTPD2UDQZ256rmb = 6537,
  VCVTPD2UDQZ256rmbk = 6538,
  VCVTPD2UDQZ256rmbkz = 6539,
  VCVTPD2UDQZ256rmk = 6540,
  VCVTPD2UDQZ256rmkz = 6541,
  VCVTPD2UDQZ256rr = 6542,
  VCVTPD2UDQZ256rrk = 6543,
  VCVTPD2UDQZ256rrkz = 6544,
  VCVTPD2UDQZrm = 6545,
  VCVTPD2UDQZrmb = 6546,
  VCVTPD2UDQZrmbk = 6547,
  VCVTPD2UDQZrmbkz = 6548,
  VCVTPD2UDQZrmk = 6549,
  VCVTPD2UDQZrmkz = 6550,
  VCVTPD2UDQZrr = 6551,
  VCVTPD2UDQZrrb = 6552,
  VCVTPD2UDQZrrbk = 6553,
  VCVTPD2UDQZrrbkz = 6554,
  VCVTPD2UDQZrrk = 6555,
  VCVTPD2UDQZrrkz = 6556,
  VCVTPD2UQQZ128rm = 6557,
  VCVTPD2UQQZ128rmb = 6558,
  VCVTPD2UQQZ128rmbk = 6559,
  VCVTPD2UQQZ128rmbkz = 6560,
  VCVTPD2UQQZ128rmk = 6561,
  VCVTPD2UQQZ128rmkz = 6562,
  VCVTPD2UQQZ128rr = 6563,
  VCVTPD2UQQZ128rrk = 6564,
  VCVTPD2UQQZ128rrkz = 6565,
  VCVTPD2UQQZ256rm = 6566,
  VCVTPD2UQQZ256rmb = 6567,
  VCVTPD2UQQZ256rmbk = 6568,
  VCVTPD2UQQZ256rmbkz = 6569,
  VCVTPD2UQQZ256rmk = 6570,
  VCVTPD2UQQZ256rmkz = 6571,
  VCVTPD2UQQZ256rr = 6572,
  VCVTPD2UQQZ256rrk = 6573,
  VCVTPD2UQQZ256rrkz = 6574,
  VCVTPD2UQQZrm = 6575,
  VCVTPD2UQQZrmb = 6576,
  VCVTPD2UQQZrmbk = 6577,
  VCVTPD2UQQZrmbkz = 6578,
  VCVTPD2UQQZrmk = 6579,
  VCVTPD2UQQZrmkz = 6580,
  VCVTPD2UQQZrr = 6581,
  VCVTPD2UQQZrrb = 6582,
  VCVTPD2UQQZrrbk = 6583,
  VCVTPD2UQQZrrbkz = 6584,
  VCVTPD2UQQZrrk = 6585,
  VCVTPD2UQQZrrkz = 6586,
  VCVTPH2BF8SZ128rm = 6587,
  VCVTPH2BF8SZ128rmb = 6588,
  VCVTPH2BF8SZ128rmbk = 6589,
  VCVTPH2BF8SZ128rmbkz = 6590,
  VCVTPH2BF8SZ128rmk = 6591,
  VCVTPH2BF8SZ128rmkz = 6592,
  VCVTPH2BF8SZ128rr = 6593,
  VCVTPH2BF8SZ128rrk = 6594,
  VCVTPH2BF8SZ128rrkz = 6595,
  VCVTPH2BF8SZ256rm = 6596,
  VCVTPH2BF8SZ256rmb = 6597,
  VCVTPH2BF8SZ256rmbk = 6598,
  VCVTPH2BF8SZ256rmbkz = 6599,
  VCVTPH2BF8SZ256rmk = 6600,
  VCVTPH2BF8SZ256rmkz = 6601,
  VCVTPH2BF8SZ256rr = 6602,
  VCVTPH2BF8SZ256rrk = 6603,
  VCVTPH2BF8SZ256rrkz = 6604,
  VCVTPH2BF8SZrm = 6605,
  VCVTPH2BF8SZrmb = 6606,
  VCVTPH2BF8SZrmbk = 6607,
  VCVTPH2BF8SZrmbkz = 6608,
  VCVTPH2BF8SZrmk = 6609,
  VCVTPH2BF8SZrmkz = 6610,
  VCVTPH2BF8SZrr = 6611,
  VCVTPH2BF8SZrrk = 6612,
  VCVTPH2BF8SZrrkz = 6613,
  VCVTPH2BF8Z128rm = 6614,
  VCVTPH2BF8Z128rmb = 6615,
  VCVTPH2BF8Z128rmbk = 6616,
  VCVTPH2BF8Z128rmbkz = 6617,
  VCVTPH2BF8Z128rmk = 6618,
  VCVTPH2BF8Z128rmkz = 6619,
  VCVTPH2BF8Z128rr = 6620,
  VCVTPH2BF8Z128rrk = 6621,
  VCVTPH2BF8Z128rrkz = 6622,
  VCVTPH2BF8Z256rm = 6623,
  VCVTPH2BF8Z256rmb = 6624,
  VCVTPH2BF8Z256rmbk = 6625,
  VCVTPH2BF8Z256rmbkz = 6626,
  VCVTPH2BF8Z256rmk = 6627,
  VCVTPH2BF8Z256rmkz = 6628,
  VCVTPH2BF8Z256rr = 6629,
  VCVTPH2BF8Z256rrk = 6630,
  VCVTPH2BF8Z256rrkz = 6631,
  VCVTPH2BF8Zrm = 6632,
  VCVTPH2BF8Zrmb = 6633,
  VCVTPH2BF8Zrmbk = 6634,
  VCVTPH2BF8Zrmbkz = 6635,
  VCVTPH2BF8Zrmk = 6636,
  VCVTPH2BF8Zrmkz = 6637,
  VCVTPH2BF8Zrr = 6638,
  VCVTPH2BF8Zrrk = 6639,
  VCVTPH2BF8Zrrkz = 6640,
  VCVTPH2DQZ128rm = 6641,
  VCVTPH2DQZ128rmb = 6642,
  VCVTPH2DQZ128rmbk = 6643,
  VCVTPH2DQZ128rmbkz = 6644,
  VCVTPH2DQZ128rmk = 6645,
  VCVTPH2DQZ128rmkz = 6646,
  VCVTPH2DQZ128rr = 6647,
  VCVTPH2DQZ128rrk = 6648,
  VCVTPH2DQZ128rrkz = 6649,
  VCVTPH2DQZ256rm = 6650,
  VCVTPH2DQZ256rmb = 6651,
  VCVTPH2DQZ256rmbk = 6652,
  VCVTPH2DQZ256rmbkz = 6653,
  VCVTPH2DQZ256rmk = 6654,
  VCVTPH2DQZ256rmkz = 6655,
  VCVTPH2DQZ256rr = 6656,
  VCVTPH2DQZ256rrk = 6657,
  VCVTPH2DQZ256rrkz = 6658,
  VCVTPH2DQZrm = 6659,
  VCVTPH2DQZrmb = 6660,
  VCVTPH2DQZrmbk = 6661,
  VCVTPH2DQZrmbkz = 6662,
  VCVTPH2DQZrmk = 6663,
  VCVTPH2DQZrmkz = 6664,
  VCVTPH2DQZrr = 6665,
  VCVTPH2DQZrrb = 6666,
  VCVTPH2DQZrrbk = 6667,
  VCVTPH2DQZrrbkz = 6668,
  VCVTPH2DQZrrk = 6669,
  VCVTPH2DQZrrkz = 6670,
  VCVTPH2HF8SZ128rm = 6671,
  VCVTPH2HF8SZ128rmb = 6672,
  VCVTPH2HF8SZ128rmbk = 6673,
  VCVTPH2HF8SZ128rmbkz = 6674,
  VCVTPH2HF8SZ128rmk = 6675,
  VCVTPH2HF8SZ128rmkz = 6676,
  VCVTPH2HF8SZ128rr = 6677,
  VCVTPH2HF8SZ128rrk = 6678,
  VCVTPH2HF8SZ128rrkz = 6679,
  VCVTPH2HF8SZ256rm = 6680,
  VCVTPH2HF8SZ256rmb = 6681,
  VCVTPH2HF8SZ256rmbk = 6682,
  VCVTPH2HF8SZ256rmbkz = 6683,
  VCVTPH2HF8SZ256rmk = 6684,
  VCVTPH2HF8SZ256rmkz = 6685,
  VCVTPH2HF8SZ256rr = 6686,
  VCVTPH2HF8SZ256rrk = 6687,
  VCVTPH2HF8SZ256rrkz = 6688,
  VCVTPH2HF8SZrm = 6689,
  VCVTPH2HF8SZrmb = 6690,
  VCVTPH2HF8SZrmbk = 6691,
  VCVTPH2HF8SZrmbkz = 6692,
  VCVTPH2HF8SZrmk = 6693,
  VCVTPH2HF8SZrmkz = 6694,
  VCVTPH2HF8SZrr = 6695,
  VCVTPH2HF8SZrrk = 6696,
  VCVTPH2HF8SZrrkz = 6697,
  VCVTPH2HF8Z128rm = 6698,
  VCVTPH2HF8Z128rmb = 6699,
  VCVTPH2HF8Z128rmbk = 6700,
  VCVTPH2HF8Z128rmbkz = 6701,
  VCVTPH2HF8Z128rmk = 6702,
  VCVTPH2HF8Z128rmkz = 6703,
  VCVTPH2HF8Z128rr = 6704,
  VCVTPH2HF8Z128rrk = 6705,
  VCVTPH2HF8Z128rrkz = 6706,
  VCVTPH2HF8Z256rm = 6707,
  VCVTPH2HF8Z256rmb = 6708,
  VCVTPH2HF8Z256rmbk = 6709,
  VCVTPH2HF8Z256rmbkz = 6710,
  VCVTPH2HF8Z256rmk = 6711,
  VCVTPH2HF8Z256rmkz = 6712,
  VCVTPH2HF8Z256rr = 6713,
  VCVTPH2HF8Z256rrk = 6714,
  VCVTPH2HF8Z256rrkz = 6715,
  VCVTPH2HF8Zrm = 6716,
  VCVTPH2HF8Zrmb = 6717,
  VCVTPH2HF8Zrmbk = 6718,
  VCVTPH2HF8Zrmbkz = 6719,
  VCVTPH2HF8Zrmk = 6720,
  VCVTPH2HF8Zrmkz = 6721,
  VCVTPH2HF8Zrr = 6722,
  VCVTPH2HF8Zrrk = 6723,
  VCVTPH2HF8Zrrkz = 6724,
  VCVTPH2IBSZ128rm = 6725,
  VCVTPH2IBSZ128rmb = 6726,
  VCVTPH2IBSZ128rmbk = 6727,
  VCVTPH2IBSZ128rmbkz = 6728,
  VCVTPH2IBSZ128rmk = 6729,
  VCVTPH2IBSZ128rmkz = 6730,
  VCVTPH2IBSZ128rr = 6731,
  VCVTPH2IBSZ128rrk = 6732,
  VCVTPH2IBSZ128rrkz = 6733,
  VCVTPH2IBSZ256rm = 6734,
  VCVTPH2IBSZ256rmb = 6735,
  VCVTPH2IBSZ256rmbk = 6736,
  VCVTPH2IBSZ256rmbkz = 6737,
  VCVTPH2IBSZ256rmk = 6738,
  VCVTPH2IBSZ256rmkz = 6739,
  VCVTPH2IBSZ256rr = 6740,
  VCVTPH2IBSZ256rrk = 6741,
  VCVTPH2IBSZ256rrkz = 6742,
  VCVTPH2IBSZrm = 6743,
  VCVTPH2IBSZrmb = 6744,
  VCVTPH2IBSZrmbk = 6745,
  VCVTPH2IBSZrmbkz = 6746,
  VCVTPH2IBSZrmk = 6747,
  VCVTPH2IBSZrmkz = 6748,
  VCVTPH2IBSZrr = 6749,
  VCVTPH2IBSZrrb = 6750,
  VCVTPH2IBSZrrbk = 6751,
  VCVTPH2IBSZrrbkz = 6752,
  VCVTPH2IBSZrrk = 6753,
  VCVTPH2IBSZrrkz = 6754,
  VCVTPH2IUBSZ128rm = 6755,
  VCVTPH2IUBSZ128rmb = 6756,
  VCVTPH2IUBSZ128rmbk = 6757,
  VCVTPH2IUBSZ128rmbkz = 6758,
  VCVTPH2IUBSZ128rmk = 6759,
  VCVTPH2IUBSZ128rmkz = 6760,
  VCVTPH2IUBSZ128rr = 6761,
  VCVTPH2IUBSZ128rrk = 6762,
  VCVTPH2IUBSZ128rrkz = 6763,
  VCVTPH2IUBSZ256rm = 6764,
  VCVTPH2IUBSZ256rmb = 6765,
  VCVTPH2IUBSZ256rmbk = 6766,
  VCVTPH2IUBSZ256rmbkz = 6767,
  VCVTPH2IUBSZ256rmk = 6768,
  VCVTPH2IUBSZ256rmkz = 6769,
  VCVTPH2IUBSZ256rr = 6770,
  VCVTPH2IUBSZ256rrk = 6771,
  VCVTPH2IUBSZ256rrkz = 6772,
  VCVTPH2IUBSZrm = 6773,
  VCVTPH2IUBSZrmb = 6774,
  VCVTPH2IUBSZrmbk = 6775,
  VCVTPH2IUBSZrmbkz = 6776,
  VCVTPH2IUBSZrmk = 6777,
  VCVTPH2IUBSZrmkz = 6778,
  VCVTPH2IUBSZrr = 6779,
  VCVTPH2IUBSZrrb = 6780,
  VCVTPH2IUBSZrrbk = 6781,
  VCVTPH2IUBSZrrbkz = 6782,
  VCVTPH2IUBSZrrk = 6783,
  VCVTPH2IUBSZrrkz = 6784,
  VCVTPH2PDZ128rm = 6785,
  VCVTPH2PDZ128rmb = 6786,
  VCVTPH2PDZ128rmbk = 6787,
  VCVTPH2PDZ128rmbkz = 6788,
  VCVTPH2PDZ128rmk = 6789,
  VCVTPH2PDZ128rmkz = 6790,
  VCVTPH2PDZ128rr = 6791,
  VCVTPH2PDZ128rrk = 6792,
  VCVTPH2PDZ128rrkz = 6793,
  VCVTPH2PDZ256rm = 6794,
  VCVTPH2PDZ256rmb = 6795,
  VCVTPH2PDZ256rmbk = 6796,
  VCVTPH2PDZ256rmbkz = 6797,
  VCVTPH2PDZ256rmk = 6798,
  VCVTPH2PDZ256rmkz = 6799,
  VCVTPH2PDZ256rr = 6800,
  VCVTPH2PDZ256rrk = 6801,
  VCVTPH2PDZ256rrkz = 6802,
  VCVTPH2PDZrm = 6803,
  VCVTPH2PDZrmb = 6804,
  VCVTPH2PDZrmbk = 6805,
  VCVTPH2PDZrmbkz = 6806,
  VCVTPH2PDZrmk = 6807,
  VCVTPH2PDZrmkz = 6808,
  VCVTPH2PDZrr = 6809,
  VCVTPH2PDZrrb = 6810,
  VCVTPH2PDZrrbk = 6811,
  VCVTPH2PDZrrbkz = 6812,
  VCVTPH2PDZrrk = 6813,
  VCVTPH2PDZrrkz = 6814,
  VCVTPH2PSXZ128rm = 6815,
  VCVTPH2PSXZ128rmb = 6816,
  VCVTPH2PSXZ128rmbk = 6817,
  VCVTPH2PSXZ128rmbkz = 6818,
  VCVTPH2PSXZ128rmk = 6819,
  VCVTPH2PSXZ128rmkz = 6820,
  VCVTPH2PSXZ128rr = 6821,
  VCVTPH2PSXZ128rrk = 6822,
  VCVTPH2PSXZ128rrkz = 6823,
  VCVTPH2PSXZ256rm = 6824,
  VCVTPH2PSXZ256rmb = 6825,
  VCVTPH2PSXZ256rmbk = 6826,
  VCVTPH2PSXZ256rmbkz = 6827,
  VCVTPH2PSXZ256rmk = 6828,
  VCVTPH2PSXZ256rmkz = 6829,
  VCVTPH2PSXZ256rr = 6830,
  VCVTPH2PSXZ256rrk = 6831,
  VCVTPH2PSXZ256rrkz = 6832,
  VCVTPH2PSXZrm = 6833,
  VCVTPH2PSXZrmb = 6834,
  VCVTPH2PSXZrmbk = 6835,
  VCVTPH2PSXZrmbkz = 6836,
  VCVTPH2PSXZrmk = 6837,
  VCVTPH2PSXZrmkz = 6838,
  VCVTPH2PSXZrr = 6839,
  VCVTPH2PSXZrrb = 6840,
  VCVTPH2PSXZrrbk = 6841,
  VCVTPH2PSXZrrbkz = 6842,
  VCVTPH2PSXZrrk = 6843,
  VCVTPH2PSXZrrkz = 6844,
  VCVTPH2PSYrm = 6845,
  VCVTPH2PSYrr = 6846,
  VCVTPH2PSZ128rm = 6847,
  VCVTPH2PSZ128rmk = 6848,
  VCVTPH2PSZ128rmkz = 6849,
  VCVTPH2PSZ128rr = 6850,
  VCVTPH2PSZ128rrk = 6851,
  VCVTPH2PSZ128rrkz = 6852,
  VCVTPH2PSZ256rm = 6853,
  VCVTPH2PSZ256rmk = 6854,
  VCVTPH2PSZ256rmkz = 6855,
  VCVTPH2PSZ256rr = 6856,
  VCVTPH2PSZ256rrk = 6857,
  VCVTPH2PSZ256rrkz = 6858,
  VCVTPH2PSZrm = 6859,
  VCVTPH2PSZrmk = 6860,
  VCVTPH2PSZrmkz = 6861,
  VCVTPH2PSZrr = 6862,
  VCVTPH2PSZrrb = 6863,
  VCVTPH2PSZrrbk = 6864,
  VCVTPH2PSZrrbkz = 6865,
  VCVTPH2PSZrrk = 6866,
  VCVTPH2PSZrrkz = 6867,
  VCVTPH2PSrm = 6868,
  VCVTPH2PSrr = 6869,
  VCVTPH2QQZ128rm = 6870,
  VCVTPH2QQZ128rmb = 6871,
  VCVTPH2QQZ128rmbk = 6872,
  VCVTPH2QQZ128rmbkz = 6873,
  VCVTPH2QQZ128rmk = 6874,
  VCVTPH2QQZ128rmkz = 6875,
  VCVTPH2QQZ128rr = 6876,
  VCVTPH2QQZ128rrk = 6877,
  VCVTPH2QQZ128rrkz = 6878,
  VCVTPH2QQZ256rm = 6879,
  VCVTPH2QQZ256rmb = 6880,
  VCVTPH2QQZ256rmbk = 6881,
  VCVTPH2QQZ256rmbkz = 6882,
  VCVTPH2QQZ256rmk = 6883,
  VCVTPH2QQZ256rmkz = 6884,
  VCVTPH2QQZ256rr = 6885,
  VCVTPH2QQZ256rrk = 6886,
  VCVTPH2QQZ256rrkz = 6887,
  VCVTPH2QQZrm = 6888,
  VCVTPH2QQZrmb = 6889,
  VCVTPH2QQZrmbk = 6890,
  VCVTPH2QQZrmbkz = 6891,
  VCVTPH2QQZrmk = 6892,
  VCVTPH2QQZrmkz = 6893,
  VCVTPH2QQZrr = 6894,
  VCVTPH2QQZrrb = 6895,
  VCVTPH2QQZrrbk = 6896,
  VCVTPH2QQZrrbkz = 6897,
  VCVTPH2QQZrrk = 6898,
  VCVTPH2QQZrrkz = 6899,
  VCVTPH2UDQZ128rm = 6900,
  VCVTPH2UDQZ128rmb = 6901,
  VCVTPH2UDQZ128rmbk = 6902,
  VCVTPH2UDQZ128rmbkz = 6903,
  VCVTPH2UDQZ128rmk = 6904,
  VCVTPH2UDQZ128rmkz = 6905,
  VCVTPH2UDQZ128rr = 6906,
  VCVTPH2UDQZ128rrk = 6907,
  VCVTPH2UDQZ128rrkz = 6908,
  VCVTPH2UDQZ256rm = 6909,
  VCVTPH2UDQZ256rmb = 6910,
  VCVTPH2UDQZ256rmbk = 6911,
  VCVTPH2UDQZ256rmbkz = 6912,
  VCVTPH2UDQZ256rmk = 6913,
  VCVTPH2UDQZ256rmkz = 6914,
  VCVTPH2UDQZ256rr = 6915,
  VCVTPH2UDQZ256rrk = 6916,
  VCVTPH2UDQZ256rrkz = 6917,
  VCVTPH2UDQZrm = 6918,
  VCVTPH2UDQZrmb = 6919,
  VCVTPH2UDQZrmbk = 6920,
  VCVTPH2UDQZrmbkz = 6921,
  VCVTPH2UDQZrmk = 6922,
  VCVTPH2UDQZrmkz = 6923,
  VCVTPH2UDQZrr = 6924,
  VCVTPH2UDQZrrb = 6925,
  VCVTPH2UDQZrrbk = 6926,
  VCVTPH2UDQZrrbkz = 6927,
  VCVTPH2UDQZrrk = 6928,
  VCVTPH2UDQZrrkz = 6929,
  VCVTPH2UQQZ128rm = 6930,
  VCVTPH2UQQZ128rmb = 6931,
  VCVTPH2UQQZ128rmbk = 6932,
  VCVTPH2UQQZ128rmbkz = 6933,
  VCVTPH2UQQZ128rmk = 6934,
  VCVTPH2UQQZ128rmkz = 6935,
  VCVTPH2UQQZ128rr = 6936,
  VCVTPH2UQQZ128rrk = 6937,
  VCVTPH2UQQZ128rrkz = 6938,
  VCVTPH2UQQZ256rm = 6939,
  VCVTPH2UQQZ256rmb = 6940,
  VCVTPH2UQQZ256rmbk = 6941,
  VCVTPH2UQQZ256rmbkz = 6942,
  VCVTPH2UQQZ256rmk = 6943,
  VCVTPH2UQQZ256rmkz = 6944,
  VCVTPH2UQQZ256rr = 6945,
  VCVTPH2UQQZ256rrk = 6946,
  VCVTPH2UQQZ256rrkz = 6947,
  VCVTPH2UQQZrm = 6948,
  VCVTPH2UQQZrmb = 6949,
  VCVTPH2UQQZrmbk = 6950,
  VCVTPH2UQQZrmbkz = 6951,
  VCVTPH2UQQZrmk = 6952,
  VCVTPH2UQQZrmkz = 6953,
  VCVTPH2UQQZrr = 6954,
  VCVTPH2UQQZrrb = 6955,
  VCVTPH2UQQZrrbk = 6956,
  VCVTPH2UQQZrrbkz = 6957,
  VCVTPH2UQQZrrk = 6958,
  VCVTPH2UQQZrrkz = 6959,
  VCVTPH2UWZ128rm = 6960,
  VCVTPH2UWZ128rmb = 6961,
  VCVTPH2UWZ128rmbk = 6962,
  VCVTPH2UWZ128rmbkz = 6963,
  VCVTPH2UWZ128rmk = 6964,
  VCVTPH2UWZ128rmkz = 6965,
  VCVTPH2UWZ128rr = 6966,
  VCVTPH2UWZ128rrk = 6967,
  VCVTPH2UWZ128rrkz = 6968,
  VCVTPH2UWZ256rm = 6969,
  VCVTPH2UWZ256rmb = 6970,
  VCVTPH2UWZ256rmbk = 6971,
  VCVTPH2UWZ256rmbkz = 6972,
  VCVTPH2UWZ256rmk = 6973,
  VCVTPH2UWZ256rmkz = 6974,
  VCVTPH2UWZ256rr = 6975,
  VCVTPH2UWZ256rrk = 6976,
  VCVTPH2UWZ256rrkz = 6977,
  VCVTPH2UWZrm = 6978,
  VCVTPH2UWZrmb = 6979,
  VCVTPH2UWZrmbk = 6980,
  VCVTPH2UWZrmbkz = 6981,
  VCVTPH2UWZrmk = 6982,
  VCVTPH2UWZrmkz = 6983,
  VCVTPH2UWZrr = 6984,
  VCVTPH2UWZrrb = 6985,
  VCVTPH2UWZrrbk = 6986,
  VCVTPH2UWZrrbkz = 6987,
  VCVTPH2UWZrrk = 6988,
  VCVTPH2UWZrrkz = 6989,
  VCVTPH2WZ128rm = 6990,
  VCVTPH2WZ128rmb = 6991,
  VCVTPH2WZ128rmbk = 6992,
  VCVTPH2WZ128rmbkz = 6993,
  VCVTPH2WZ128rmk = 6994,
  VCVTPH2WZ128rmkz = 6995,
  VCVTPH2WZ128rr = 6996,
  VCVTPH2WZ128rrk = 6997,
  VCVTPH2WZ128rrkz = 6998,
  VCVTPH2WZ256rm = 6999,
  VCVTPH2WZ256rmb = 7000,
  VCVTPH2WZ256rmbk = 7001,
  VCVTPH2WZ256rmbkz = 7002,
  VCVTPH2WZ256rmk = 7003,
  VCVTPH2WZ256rmkz = 7004,
  VCVTPH2WZ256rr = 7005,
  VCVTPH2WZ256rrk = 7006,
  VCVTPH2WZ256rrkz = 7007,
  VCVTPH2WZrm = 7008,
  VCVTPH2WZrmb = 7009,
  VCVTPH2WZrmbk = 7010,
  VCVTPH2WZrmbkz = 7011,
  VCVTPH2WZrmk = 7012,
  VCVTPH2WZrmkz = 7013,
  VCVTPH2WZrr = 7014,
  VCVTPH2WZrrb = 7015,
  VCVTPH2WZrrbk = 7016,
  VCVTPH2WZrrbkz = 7017,
  VCVTPH2WZrrk = 7018,
  VCVTPH2WZrrkz = 7019,
  VCVTPS2DQYrm = 7020,
  VCVTPS2DQYrr = 7021,
  VCVTPS2DQZ128rm = 7022,
  VCVTPS2DQZ128rmb = 7023,
  VCVTPS2DQZ128rmbk = 7024,
  VCVTPS2DQZ128rmbkz = 7025,
  VCVTPS2DQZ128rmk = 7026,
  VCVTPS2DQZ128rmkz = 7027,
  VCVTPS2DQZ128rr = 7028,
  VCVTPS2DQZ128rrk = 7029,
  VCVTPS2DQZ128rrkz = 7030,
  VCVTPS2DQZ256rm = 7031,
  VCVTPS2DQZ256rmb = 7032,
  VCVTPS2DQZ256rmbk = 7033,
  VCVTPS2DQZ256rmbkz = 7034,
  VCVTPS2DQZ256rmk = 7035,
  VCVTPS2DQZ256rmkz = 7036,
  VCVTPS2DQZ256rr = 7037,
  VCVTPS2DQZ256rrk = 7038,
  VCVTPS2DQZ256rrkz = 7039,
  VCVTPS2DQZrm = 7040,
  VCVTPS2DQZrmb = 7041,
  VCVTPS2DQZrmbk = 7042,
  VCVTPS2DQZrmbkz = 7043,
  VCVTPS2DQZrmk = 7044,
  VCVTPS2DQZrmkz = 7045,
  VCVTPS2DQZrr = 7046,
  VCVTPS2DQZrrb = 7047,
  VCVTPS2DQZrrbk = 7048,
  VCVTPS2DQZrrbkz = 7049,
  VCVTPS2DQZrrk = 7050,
  VCVTPS2DQZrrkz = 7051,
  VCVTPS2DQrm = 7052,
  VCVTPS2DQrr = 7053,
  VCVTPS2IBSZ128rm = 7054,
  VCVTPS2IBSZ128rmb = 7055,
  VCVTPS2IBSZ128rmbk = 7056,
  VCVTPS2IBSZ128rmbkz = 7057,
  VCVTPS2IBSZ128rmk = 7058,
  VCVTPS2IBSZ128rmkz = 7059,
  VCVTPS2IBSZ128rr = 7060,
  VCVTPS2IBSZ128rrk = 7061,
  VCVTPS2IBSZ128rrkz = 7062,
  VCVTPS2IBSZ256rm = 7063,
  VCVTPS2IBSZ256rmb = 7064,
  VCVTPS2IBSZ256rmbk = 7065,
  VCVTPS2IBSZ256rmbkz = 7066,
  VCVTPS2IBSZ256rmk = 7067,
  VCVTPS2IBSZ256rmkz = 7068,
  VCVTPS2IBSZ256rr = 7069,
  VCVTPS2IBSZ256rrk = 7070,
  VCVTPS2IBSZ256rrkz = 7071,
  VCVTPS2IBSZrm = 7072,
  VCVTPS2IBSZrmb = 7073,
  VCVTPS2IBSZrmbk = 7074,
  VCVTPS2IBSZrmbkz = 7075,
  VCVTPS2IBSZrmk = 7076,
  VCVTPS2IBSZrmkz = 7077,
  VCVTPS2IBSZrr = 7078,
  VCVTPS2IBSZrrb = 7079,
  VCVTPS2IBSZrrbk = 7080,
  VCVTPS2IBSZrrbkz = 7081,
  VCVTPS2IBSZrrk = 7082,
  VCVTPS2IBSZrrkz = 7083,
  VCVTPS2IUBSZ128rm = 7084,
  VCVTPS2IUBSZ128rmb = 7085,
  VCVTPS2IUBSZ128rmbk = 7086,
  VCVTPS2IUBSZ128rmbkz = 7087,
  VCVTPS2IUBSZ128rmk = 7088,
  VCVTPS2IUBSZ128rmkz = 7089,
  VCVTPS2IUBSZ128rr = 7090,
  VCVTPS2IUBSZ128rrk = 7091,
  VCVTPS2IUBSZ128rrkz = 7092,
  VCVTPS2IUBSZ256rm = 7093,
  VCVTPS2IUBSZ256rmb = 7094,
  VCVTPS2IUBSZ256rmbk = 7095,
  VCVTPS2IUBSZ256rmbkz = 7096,
  VCVTPS2IUBSZ256rmk = 7097,
  VCVTPS2IUBSZ256rmkz = 7098,
  VCVTPS2IUBSZ256rr = 7099,
  VCVTPS2IUBSZ256rrk = 7100,
  VCVTPS2IUBSZ256rrkz = 7101,
  VCVTPS2IUBSZrm = 7102,
  VCVTPS2IUBSZrmb = 7103,
  VCVTPS2IUBSZrmbk = 7104,
  VCVTPS2IUBSZrmbkz = 7105,
  VCVTPS2IUBSZrmk = 7106,
  VCVTPS2IUBSZrmkz = 7107,
  VCVTPS2IUBSZrr = 7108,
  VCVTPS2IUBSZrrb = 7109,
  VCVTPS2IUBSZrrbk = 7110,
  VCVTPS2IUBSZrrbkz = 7111,
  VCVTPS2IUBSZrrk = 7112,
  VCVTPS2IUBSZrrkz = 7113,
  VCVTPS2PDYrm = 7114,
  VCVTPS2PDYrr = 7115,
  VCVTPS2PDZ128rm = 7116,
  VCVTPS2PDZ128rmb = 7117,
  VCVTPS2PDZ128rmbk = 7118,
  VCVTPS2PDZ128rmbkz = 7119,
  VCVTPS2PDZ128rmk = 7120,
  VCVTPS2PDZ128rmkz = 7121,
  VCVTPS2PDZ128rr = 7122,
  VCVTPS2PDZ128rrk = 7123,
  VCVTPS2PDZ128rrkz = 7124,
  VCVTPS2PDZ256rm = 7125,
  VCVTPS2PDZ256rmb = 7126,
  VCVTPS2PDZ256rmbk = 7127,
  VCVTPS2PDZ256rmbkz = 7128,
  VCVTPS2PDZ256rmk = 7129,
  VCVTPS2PDZ256rmkz = 7130,
  VCVTPS2PDZ256rr = 7131,
  VCVTPS2PDZ256rrk = 7132,
  VCVTPS2PDZ256rrkz = 7133,
  VCVTPS2PDZrm = 7134,
  VCVTPS2PDZrmb = 7135,
  VCVTPS2PDZrmbk = 7136,
  VCVTPS2PDZrmbkz = 7137,
  VCVTPS2PDZrmk = 7138,
  VCVTPS2PDZrmkz = 7139,
  VCVTPS2PDZrr = 7140,
  VCVTPS2PDZrrb = 7141,
  VCVTPS2PDZrrbk = 7142,
  VCVTPS2PDZrrbkz = 7143,
  VCVTPS2PDZrrk = 7144,
  VCVTPS2PDZrrkz = 7145,
  VCVTPS2PDrm = 7146,
  VCVTPS2PDrr = 7147,
  VCVTPS2PHXZ128rm = 7148,
  VCVTPS2PHXZ128rmb = 7149,
  VCVTPS2PHXZ128rmbk = 7150,
  VCVTPS2PHXZ128rmbkz = 7151,
  VCVTPS2PHXZ128rmk = 7152,
  VCVTPS2PHXZ128rmkz = 7153,
  VCVTPS2PHXZ128rr = 7154,
  VCVTPS2PHXZ128rrk = 7155,
  VCVTPS2PHXZ128rrkz = 7156,
  VCVTPS2PHXZ256rm = 7157,
  VCVTPS2PHXZ256rmb = 7158,
  VCVTPS2PHXZ256rmbk = 7159,
  VCVTPS2PHXZ256rmbkz = 7160,
  VCVTPS2PHXZ256rmk = 7161,
  VCVTPS2PHXZ256rmkz = 7162,
  VCVTPS2PHXZ256rr = 7163,
  VCVTPS2PHXZ256rrk = 7164,
  VCVTPS2PHXZ256rrkz = 7165,
  VCVTPS2PHXZrm = 7166,
  VCVTPS2PHXZrmb = 7167,
  VCVTPS2PHXZrmbk = 7168,
  VCVTPS2PHXZrmbkz = 7169,
  VCVTPS2PHXZrmk = 7170,
  VCVTPS2PHXZrmkz = 7171,
  VCVTPS2PHXZrr = 7172,
  VCVTPS2PHXZrrb = 7173,
  VCVTPS2PHXZrrbk = 7174,
  VCVTPS2PHXZrrbkz = 7175,
  VCVTPS2PHXZrrk = 7176,
  VCVTPS2PHXZrrkz = 7177,
  VCVTPS2PHYmr = 7178,
  VCVTPS2PHYrr = 7179,
  VCVTPS2PHZ128mr = 7180,
  VCVTPS2PHZ128mrk = 7181,
  VCVTPS2PHZ128rr = 7182,
  VCVTPS2PHZ128rrk = 7183,
  VCVTPS2PHZ128rrkz = 7184,
  VCVTPS2PHZ256mr = 7185,
  VCVTPS2PHZ256mrk = 7186,
  VCVTPS2PHZ256rr = 7187,
  VCVTPS2PHZ256rrk = 7188,
  VCVTPS2PHZ256rrkz = 7189,
  VCVTPS2PHZmr = 7190,
  VCVTPS2PHZmrk = 7191,
  VCVTPS2PHZrr = 7192,
  VCVTPS2PHZrrb = 7193,
  VCVTPS2PHZrrbk = 7194,
  VCVTPS2PHZrrbkz = 7195,
  VCVTPS2PHZrrk = 7196,
  VCVTPS2PHZrrkz = 7197,
  VCVTPS2PHmr = 7198,
  VCVTPS2PHrr = 7199,
  VCVTPS2QQZ128rm = 7200,
  VCVTPS2QQZ128rmb = 7201,
  VCVTPS2QQZ128rmbk = 7202,
  VCVTPS2QQZ128rmbkz = 7203,
  VCVTPS2QQZ128rmk = 7204,
  VCVTPS2QQZ128rmkz = 7205,
  VCVTPS2QQZ128rr = 7206,
  VCVTPS2QQZ128rrk = 7207,
  VCVTPS2QQZ128rrkz = 7208,
  VCVTPS2QQZ256rm = 7209,
  VCVTPS2QQZ256rmb = 7210,
  VCVTPS2QQZ256rmbk = 7211,
  VCVTPS2QQZ256rmbkz = 7212,
  VCVTPS2QQZ256rmk = 7213,
  VCVTPS2QQZ256rmkz = 7214,
  VCVTPS2QQZ256rr = 7215,
  VCVTPS2QQZ256rrk = 7216,
  VCVTPS2QQZ256rrkz = 7217,
  VCVTPS2QQZrm = 7218,
  VCVTPS2QQZrmb = 7219,
  VCVTPS2QQZrmbk = 7220,
  VCVTPS2QQZrmbkz = 7221,
  VCVTPS2QQZrmk = 7222,
  VCVTPS2QQZrmkz = 7223,
  VCVTPS2QQZrr = 7224,
  VCVTPS2QQZrrb = 7225,
  VCVTPS2QQZrrbk = 7226,
  VCVTPS2QQZrrbkz = 7227,
  VCVTPS2QQZrrk = 7228,
  VCVTPS2QQZrrkz = 7229,
  VCVTPS2UDQZ128rm = 7230,
  VCVTPS2UDQZ128rmb = 7231,
  VCVTPS2UDQZ128rmbk = 7232,
  VCVTPS2UDQZ128rmbkz = 7233,
  VCVTPS2UDQZ128rmk = 7234,
  VCVTPS2UDQZ128rmkz = 7235,
  VCVTPS2UDQZ128rr = 7236,
  VCVTPS2UDQZ128rrk = 7237,
  VCVTPS2UDQZ128rrkz = 7238,
  VCVTPS2UDQZ256rm = 7239,
  VCVTPS2UDQZ256rmb = 7240,
  VCVTPS2UDQZ256rmbk = 7241,
  VCVTPS2UDQZ256rmbkz = 7242,
  VCVTPS2UDQZ256rmk = 7243,
  VCVTPS2UDQZ256rmkz = 7244,
  VCVTPS2UDQZ256rr = 7245,
  VCVTPS2UDQZ256rrk = 7246,
  VCVTPS2UDQZ256rrkz = 7247,
  VCVTPS2UDQZrm = 7248,
  VCVTPS2UDQZrmb = 7249,
  VCVTPS2UDQZrmbk = 7250,
  VCVTPS2UDQZrmbkz = 7251,
  VCVTPS2UDQZrmk = 7252,
  VCVTPS2UDQZrmkz = 7253,
  VCVTPS2UDQZrr = 7254,
  VCVTPS2UDQZrrb = 7255,
  VCVTPS2UDQZrrbk = 7256,
  VCVTPS2UDQZrrbkz = 7257,
  VCVTPS2UDQZrrk = 7258,
  VCVTPS2UDQZrrkz = 7259,
  VCVTPS2UQQZ128rm = 7260,
  VCVTPS2UQQZ128rmb = 7261,
  VCVTPS2UQQZ128rmbk = 7262,
  VCVTPS2UQQZ128rmbkz = 7263,
  VCVTPS2UQQZ128rmk = 7264,
  VCVTPS2UQQZ128rmkz = 7265,
  VCVTPS2UQQZ128rr = 7266,
  VCVTPS2UQQZ128rrk = 7267,
  VCVTPS2UQQZ128rrkz = 7268,
  VCVTPS2UQQZ256rm = 7269,
  VCVTPS2UQQZ256rmb = 7270,
  VCVTPS2UQQZ256rmbk = 7271,
  VCVTPS2UQQZ256rmbkz = 7272,
  VCVTPS2UQQZ256rmk = 7273,
  VCVTPS2UQQZ256rmkz = 7274,
  VCVTPS2UQQZ256rr = 7275,
  VCVTPS2UQQZ256rrk = 7276,
  VCVTPS2UQQZ256rrkz = 7277,
  VCVTPS2UQQZrm = 7278,
  VCVTPS2UQQZrmb = 7279,
  VCVTPS2UQQZrmbk = 7280,
  VCVTPS2UQQZrmbkz = 7281,
  VCVTPS2UQQZrmk = 7282,
  VCVTPS2UQQZrmkz = 7283,
  VCVTPS2UQQZrr = 7284,
  VCVTPS2UQQZrrb = 7285,
  VCVTPS2UQQZrrbk = 7286,
  VCVTPS2UQQZrrbkz = 7287,
  VCVTPS2UQQZrrk = 7288,
  VCVTPS2UQQZrrkz = 7289,
  VCVTQQ2PDZ128rm = 7290,
  VCVTQQ2PDZ128rmb = 7291,
  VCVTQQ2PDZ128rmbk = 7292,
  VCVTQQ2PDZ128rmbkz = 7293,
  VCVTQQ2PDZ128rmk = 7294,
  VCVTQQ2PDZ128rmkz = 7295,
  VCVTQQ2PDZ128rr = 7296,
  VCVTQQ2PDZ128rrk = 7297,
  VCVTQQ2PDZ128rrkz = 7298,
  VCVTQQ2PDZ256rm = 7299,
  VCVTQQ2PDZ256rmb = 7300,
  VCVTQQ2PDZ256rmbk = 7301,
  VCVTQQ2PDZ256rmbkz = 7302,
  VCVTQQ2PDZ256rmk = 7303,
  VCVTQQ2PDZ256rmkz = 7304,
  VCVTQQ2PDZ256rr = 7305,
  VCVTQQ2PDZ256rrk = 7306,
  VCVTQQ2PDZ256rrkz = 7307,
  VCVTQQ2PDZrm = 7308,
  VCVTQQ2PDZrmb = 7309,
  VCVTQQ2PDZrmbk = 7310,
  VCVTQQ2PDZrmbkz = 7311,
  VCVTQQ2PDZrmk = 7312,
  VCVTQQ2PDZrmkz = 7313,
  VCVTQQ2PDZrr = 7314,
  VCVTQQ2PDZrrb = 7315,
  VCVTQQ2PDZrrbk = 7316,
  VCVTQQ2PDZrrbkz = 7317,
  VCVTQQ2PDZrrk = 7318,
  VCVTQQ2PDZrrkz = 7319,
  VCVTQQ2PHZ128rm = 7320,
  VCVTQQ2PHZ128rmb = 7321,
  VCVTQQ2PHZ128rmbk = 7322,
  VCVTQQ2PHZ128rmbkz = 7323,
  VCVTQQ2PHZ128rmk = 7324,
  VCVTQQ2PHZ128rmkz = 7325,
  VCVTQQ2PHZ128rr = 7326,
  VCVTQQ2PHZ128rrk = 7327,
  VCVTQQ2PHZ128rrkz = 7328,
  VCVTQQ2PHZ256rm = 7329,
  VCVTQQ2PHZ256rmb = 7330,
  VCVTQQ2PHZ256rmbk = 7331,
  VCVTQQ2PHZ256rmbkz = 7332,
  VCVTQQ2PHZ256rmk = 7333,
  VCVTQQ2PHZ256rmkz = 7334,
  VCVTQQ2PHZ256rr = 7335,
  VCVTQQ2PHZ256rrk = 7336,
  VCVTQQ2PHZ256rrkz = 7337,
  VCVTQQ2PHZrm = 7338,
  VCVTQQ2PHZrmb = 7339,
  VCVTQQ2PHZrmbk = 7340,
  VCVTQQ2PHZrmbkz = 7341,
  VCVTQQ2PHZrmk = 7342,
  VCVTQQ2PHZrmkz = 7343,
  VCVTQQ2PHZrr = 7344,
  VCVTQQ2PHZrrb = 7345,
  VCVTQQ2PHZrrbk = 7346,
  VCVTQQ2PHZrrbkz = 7347,
  VCVTQQ2PHZrrk = 7348,
  VCVTQQ2PHZrrkz = 7349,
  VCVTQQ2PSZ128rm = 7350,
  VCVTQQ2PSZ128rmb = 7351,
  VCVTQQ2PSZ128rmbk = 7352,
  VCVTQQ2PSZ128rmbkz = 7353,
  VCVTQQ2PSZ128rmk = 7354,
  VCVTQQ2PSZ128rmkz = 7355,
  VCVTQQ2PSZ128rr = 7356,
  VCVTQQ2PSZ128rrk = 7357,
  VCVTQQ2PSZ128rrkz = 7358,
  VCVTQQ2PSZ256rm = 7359,
  VCVTQQ2PSZ256rmb = 7360,
  VCVTQQ2PSZ256rmbk = 7361,
  VCVTQQ2PSZ256rmbkz = 7362,
  VCVTQQ2PSZ256rmk = 7363,
  VCVTQQ2PSZ256rmkz = 7364,
  VCVTQQ2PSZ256rr = 7365,
  VCVTQQ2PSZ256rrk = 7366,
  VCVTQQ2PSZ256rrkz = 7367,
  VCVTQQ2PSZrm = 7368,
  VCVTQQ2PSZrmb = 7369,
  VCVTQQ2PSZrmbk = 7370,
  VCVTQQ2PSZrmbkz = 7371,
  VCVTQQ2PSZrmk = 7372,
  VCVTQQ2PSZrmkz = 7373,
  VCVTQQ2PSZrr = 7374,
  VCVTQQ2PSZrrb = 7375,
  VCVTQQ2PSZrrbk = 7376,
  VCVTQQ2PSZrrbkz = 7377,
  VCVTQQ2PSZrrk = 7378,
  VCVTQQ2PSZrrkz = 7379,
  VCVTSD2SHZrm = 7380,
  VCVTSD2SHZrm_Int = 7381,
  VCVTSD2SHZrmk_Int = 7382,
  VCVTSD2SHZrmkz_Int = 7383,
  VCVTSD2SHZrr = 7384,
  VCVTSD2SHZrr_Int = 7385,
  VCVTSD2SHZrrb_Int = 7386,
  VCVTSD2SHZrrbk_Int = 7387,
  VCVTSD2SHZrrbkz_Int = 7388,
  VCVTSD2SHZrrk_Int = 7389,
  VCVTSD2SHZrrkz_Int = 7390,
  VCVTSD2SI64Zrm = 7391,
  VCVTSD2SI64Zrm_Int = 7392,
  VCVTSD2SI64Zrr = 7393,
  VCVTSD2SI64Zrr_Int = 7394,
  VCVTSD2SI64Zrrb_Int = 7395,
  VCVTSD2SI64rm = 7396,
  VCVTSD2SI64rm_Int = 7397,
  VCVTSD2SI64rr = 7398,
  VCVTSD2SI64rr_Int = 7399,
  VCVTSD2SIZrm = 7400,
  VCVTSD2SIZrm_Int = 7401,
  VCVTSD2SIZrr = 7402,
  VCVTSD2SIZrr_Int = 7403,
  VCVTSD2SIZrrb_Int = 7404,
  VCVTSD2SIrm = 7405,
  VCVTSD2SIrm_Int = 7406,
  VCVTSD2SIrr = 7407,
  VCVTSD2SIrr_Int = 7408,
  VCVTSD2SSZrm = 7409,
  VCVTSD2SSZrm_Int = 7410,
  VCVTSD2SSZrmk_Int = 7411,
  VCVTSD2SSZrmkz_Int = 7412,
  VCVTSD2SSZrr = 7413,
  VCVTSD2SSZrr_Int = 7414,
  VCVTSD2SSZrrb_Int = 7415,
  VCVTSD2SSZrrbk_Int = 7416,
  VCVTSD2SSZrrbkz_Int = 7417,
  VCVTSD2SSZrrk_Int = 7418,
  VCVTSD2SSZrrkz_Int = 7419,
  VCVTSD2SSrm = 7420,
  VCVTSD2SSrm_Int = 7421,
  VCVTSD2SSrr = 7422,
  VCVTSD2SSrr_Int = 7423,
  VCVTSD2USI64Zrm_Int = 7424,
  VCVTSD2USI64Zrr_Int = 7425,
  VCVTSD2USI64Zrrb_Int = 7426,
  VCVTSD2USIZrm_Int = 7427,
  VCVTSD2USIZrr_Int = 7428,
  VCVTSD2USIZrrb_Int = 7429,
  VCVTSH2SDZrm = 7430,
  VCVTSH2SDZrm_Int = 7431,
  VCVTSH2SDZrmk_Int = 7432,
  VCVTSH2SDZrmkz_Int = 7433,
  VCVTSH2SDZrr = 7434,
  VCVTSH2SDZrr_Int = 7435,
  VCVTSH2SDZrrb_Int = 7436,
  VCVTSH2SDZrrbk_Int = 7437,
  VCVTSH2SDZrrbkz_Int = 7438,
  VCVTSH2SDZrrk_Int = 7439,
  VCVTSH2SDZrrkz_Int = 7440,
  VCVTSH2SI64Zrm_Int = 7441,
  VCVTSH2SI64Zrr_Int = 7442,
  VCVTSH2SI64Zrrb_Int = 7443,
  VCVTSH2SIZrm_Int = 7444,
  VCVTSH2SIZrr_Int = 7445,
  VCVTSH2SIZrrb_Int = 7446,
  VCVTSH2SSZrm = 7447,
  VCVTSH2SSZrm_Int = 7448,
  VCVTSH2SSZrmk_Int = 7449,
  VCVTSH2SSZrmkz_Int = 7450,
  VCVTSH2SSZrr = 7451,
  VCVTSH2SSZrr_Int = 7452,
  VCVTSH2SSZrrb_Int = 7453,
  VCVTSH2SSZrrbk_Int = 7454,
  VCVTSH2SSZrrbkz_Int = 7455,
  VCVTSH2SSZrrk_Int = 7456,
  VCVTSH2SSZrrkz_Int = 7457,
  VCVTSH2USI64Zrm_Int = 7458,
  VCVTSH2USI64Zrr_Int = 7459,
  VCVTSH2USI64Zrrb_Int = 7460,
  VCVTSH2USIZrm_Int = 7461,
  VCVTSH2USIZrr_Int = 7462,
  VCVTSH2USIZrrb_Int = 7463,
  VCVTSI2SDZrm = 7464,
  VCVTSI2SDZrm_Int = 7465,
  VCVTSI2SDZrr = 7466,
  VCVTSI2SDZrr_Int = 7467,
  VCVTSI2SDrm = 7468,
  VCVTSI2SDrm_Int = 7469,
  VCVTSI2SDrr = 7470,
  VCVTSI2SDrr_Int = 7471,
  VCVTSI2SHZrm = 7472,
  VCVTSI2SHZrm_Int = 7473,
  VCVTSI2SHZrr = 7474,
  VCVTSI2SHZrr_Int = 7475,
  VCVTSI2SHZrrb_Int = 7476,
  VCVTSI2SSZrm = 7477,
  VCVTSI2SSZrm_Int = 7478,
  VCVTSI2SSZrr = 7479,
  VCVTSI2SSZrr_Int = 7480,
  VCVTSI2SSZrrb_Int = 7481,
  VCVTSI2SSrm = 7482,
  VCVTSI2SSrm_Int = 7483,
  VCVTSI2SSrr = 7484,
  VCVTSI2SSrr_Int = 7485,
  VCVTSI642SDZrm = 7486,
  VCVTSI642SDZrm_Int = 7487,
  VCVTSI642SDZrr = 7488,
  VCVTSI642SDZrr_Int = 7489,
  VCVTSI642SDZrrb_Int = 7490,
  VCVTSI642SDrm = 7491,
  VCVTSI642SDrm_Int = 7492,
  VCVTSI642SDrr = 7493,
  VCVTSI642SDrr_Int = 7494,
  VCVTSI642SHZrm = 7495,
  VCVTSI642SHZrm_Int = 7496,
  VCVTSI642SHZrr = 7497,
  VCVTSI642SHZrr_Int = 7498,
  VCVTSI642SHZrrb_Int = 7499,
  VCVTSI642SSZrm = 7500,
  VCVTSI642SSZrm_Int = 7501,
  VCVTSI642SSZrr = 7502,
  VCVTSI642SSZrr_Int = 7503,
  VCVTSI642SSZrrb_Int = 7504,
  VCVTSI642SSrm = 7505,
  VCVTSI642SSrm_Int = 7506,
  VCVTSI642SSrr = 7507,
  VCVTSI642SSrr_Int = 7508,
  VCVTSS2SDZrm = 7509,
  VCVTSS2SDZrm_Int = 7510,
  VCVTSS2SDZrmk_Int = 7511,
  VCVTSS2SDZrmkz_Int = 7512,
  VCVTSS2SDZrr = 7513,
  VCVTSS2SDZrr_Int = 7514,
  VCVTSS2SDZrrb_Int = 7515,
  VCVTSS2SDZrrbk_Int = 7516,
  VCVTSS2SDZrrbkz_Int = 7517,
  VCVTSS2SDZrrk_Int = 7518,
  VCVTSS2SDZrrkz_Int = 7519,
  VCVTSS2SDrm = 7520,
  VCVTSS2SDrm_Int = 7521,
  VCVTSS2SDrr = 7522,
  VCVTSS2SDrr_Int = 7523,
  VCVTSS2SHZrm = 7524,
  VCVTSS2SHZrm_Int = 7525,
  VCVTSS2SHZrmk_Int = 7526,
  VCVTSS2SHZrmkz_Int = 7527,
  VCVTSS2SHZrr = 7528,
  VCVTSS2SHZrr_Int = 7529,
  VCVTSS2SHZrrb_Int = 7530,
  VCVTSS2SHZrrbk_Int = 7531,
  VCVTSS2SHZrrbkz_Int = 7532,
  VCVTSS2SHZrrk_Int = 7533,
  VCVTSS2SHZrrkz_Int = 7534,
  VCVTSS2SI64Zrm = 7535,
  VCVTSS2SI64Zrm_Int = 7536,
  VCVTSS2SI64Zrr = 7537,
  VCVTSS2SI64Zrr_Int = 7538,
  VCVTSS2SI64Zrrb_Int = 7539,
  VCVTSS2SI64rm = 7540,
  VCVTSS2SI64rm_Int = 7541,
  VCVTSS2SI64rr = 7542,
  VCVTSS2SI64rr_Int = 7543,
  VCVTSS2SIZrm = 7544,
  VCVTSS2SIZrm_Int = 7545,
  VCVTSS2SIZrr = 7546,
  VCVTSS2SIZrr_Int = 7547,
  VCVTSS2SIZrrb_Int = 7548,
  VCVTSS2SIrm = 7549,
  VCVTSS2SIrm_Int = 7550,
  VCVTSS2SIrr = 7551,
  VCVTSS2SIrr_Int = 7552,
  VCVTSS2USI64Zrm_Int = 7553,
  VCVTSS2USI64Zrr_Int = 7554,
  VCVTSS2USI64Zrrb_Int = 7555,
  VCVTSS2USIZrm_Int = 7556,
  VCVTSS2USIZrr_Int = 7557,
  VCVTSS2USIZrrb_Int = 7558,
  VCVTTBF162IBSZ128rm = 7559,
  VCVTTBF162IBSZ128rmb = 7560,
  VCVTTBF162IBSZ128rmbk = 7561,
  VCVTTBF162IBSZ128rmbkz = 7562,
  VCVTTBF162IBSZ128rmk = 7563,
  VCVTTBF162IBSZ128rmkz = 7564,
  VCVTTBF162IBSZ128rr = 7565,
  VCVTTBF162IBSZ128rrk = 7566,
  VCVTTBF162IBSZ128rrkz = 7567,
  VCVTTBF162IBSZ256rm = 7568,
  VCVTTBF162IBSZ256rmb = 7569,
  VCVTTBF162IBSZ256rmbk = 7570,
  VCVTTBF162IBSZ256rmbkz = 7571,
  VCVTTBF162IBSZ256rmk = 7572,
  VCVTTBF162IBSZ256rmkz = 7573,
  VCVTTBF162IBSZ256rr = 7574,
  VCVTTBF162IBSZ256rrk = 7575,
  VCVTTBF162IBSZ256rrkz = 7576,
  VCVTTBF162IBSZrm = 7577,
  VCVTTBF162IBSZrmb = 7578,
  VCVTTBF162IBSZrmbk = 7579,
  VCVTTBF162IBSZrmbkz = 7580,
  VCVTTBF162IBSZrmk = 7581,
  VCVTTBF162IBSZrmkz = 7582,
  VCVTTBF162IBSZrr = 7583,
  VCVTTBF162IBSZrrk = 7584,
  VCVTTBF162IBSZrrkz = 7585,
  VCVTTBF162IUBSZ128rm = 7586,
  VCVTTBF162IUBSZ128rmb = 7587,
  VCVTTBF162IUBSZ128rmbk = 7588,
  VCVTTBF162IUBSZ128rmbkz = 7589,
  VCVTTBF162IUBSZ128rmk = 7590,
  VCVTTBF162IUBSZ128rmkz = 7591,
  VCVTTBF162IUBSZ128rr = 7592,
  VCVTTBF162IUBSZ128rrk = 7593,
  VCVTTBF162IUBSZ128rrkz = 7594,
  VCVTTBF162IUBSZ256rm = 7595,
  VCVTTBF162IUBSZ256rmb = 7596,
  VCVTTBF162IUBSZ256rmbk = 7597,
  VCVTTBF162IUBSZ256rmbkz = 7598,
  VCVTTBF162IUBSZ256rmk = 7599,
  VCVTTBF162IUBSZ256rmkz = 7600,
  VCVTTBF162IUBSZ256rr = 7601,
  VCVTTBF162IUBSZ256rrk = 7602,
  VCVTTBF162IUBSZ256rrkz = 7603,
  VCVTTBF162IUBSZrm = 7604,
  VCVTTBF162IUBSZrmb = 7605,
  VCVTTBF162IUBSZrmbk = 7606,
  VCVTTBF162IUBSZrmbkz = 7607,
  VCVTTBF162IUBSZrmk = 7608,
  VCVTTBF162IUBSZrmkz = 7609,
  VCVTTBF162IUBSZrr = 7610,
  VCVTTBF162IUBSZrrk = 7611,
  VCVTTBF162IUBSZrrkz = 7612,
  VCVTTPD2DQSZ128rm = 7613,
  VCVTTPD2DQSZ128rmb = 7614,
  VCVTTPD2DQSZ128rmbk = 7615,
  VCVTTPD2DQSZ128rmbkz = 7616,
  VCVTTPD2DQSZ128rmk = 7617,
  VCVTTPD2DQSZ128rmkz = 7618,
  VCVTTPD2DQSZ128rr = 7619,
  VCVTTPD2DQSZ128rrk = 7620,
  VCVTTPD2DQSZ128rrkz = 7621,
  VCVTTPD2DQSZ256rm = 7622,
  VCVTTPD2DQSZ256rmb = 7623,
  VCVTTPD2DQSZ256rmbk = 7624,
  VCVTTPD2DQSZ256rmbkz = 7625,
  VCVTTPD2DQSZ256rmk = 7626,
  VCVTTPD2DQSZ256rmkz = 7627,
  VCVTTPD2DQSZ256rr = 7628,
  VCVTTPD2DQSZ256rrb = 7629,
  VCVTTPD2DQSZ256rrbk = 7630,
  VCVTTPD2DQSZ256rrbkz = 7631,
  VCVTTPD2DQSZ256rrk = 7632,
  VCVTTPD2DQSZ256rrkz = 7633,
  VCVTTPD2DQSZrm = 7634,
  VCVTTPD2DQSZrmb = 7635,
  VCVTTPD2DQSZrmbk = 7636,
  VCVTTPD2DQSZrmbkz = 7637,
  VCVTTPD2DQSZrmk = 7638,
  VCVTTPD2DQSZrmkz = 7639,
  VCVTTPD2DQSZrr = 7640,
  VCVTTPD2DQSZrrb = 7641,
  VCVTTPD2DQSZrrbk = 7642,
  VCVTTPD2DQSZrrbkz = 7643,
  VCVTTPD2DQSZrrk = 7644,
  VCVTTPD2DQSZrrkz = 7645,
  VCVTTPD2DQYrm = 7646,
  VCVTTPD2DQYrr = 7647,
  VCVTTPD2DQZ128rm = 7648,
  VCVTTPD2DQZ128rmb = 7649,
  VCVTTPD2DQZ128rmbk = 7650,
  VCVTTPD2DQZ128rmbkz = 7651,
  VCVTTPD2DQZ128rmk = 7652,
  VCVTTPD2DQZ128rmkz = 7653,
  VCVTTPD2DQZ128rr = 7654,
  VCVTTPD2DQZ128rrk = 7655,
  VCVTTPD2DQZ128rrkz = 7656,
  VCVTTPD2DQZ256rm = 7657,
  VCVTTPD2DQZ256rmb = 7658,
  VCVTTPD2DQZ256rmbk = 7659,
  VCVTTPD2DQZ256rmbkz = 7660,
  VCVTTPD2DQZ256rmk = 7661,
  VCVTTPD2DQZ256rmkz = 7662,
  VCVTTPD2DQZ256rr = 7663,
  VCVTTPD2DQZ256rrk = 7664,
  VCVTTPD2DQZ256rrkz = 7665,
  VCVTTPD2DQZrm = 7666,
  VCVTTPD2DQZrmb = 7667,
  VCVTTPD2DQZrmbk = 7668,
  VCVTTPD2DQZrmbkz = 7669,
  VCVTTPD2DQZrmk = 7670,
  VCVTTPD2DQZrmkz = 7671,
  VCVTTPD2DQZrr = 7672,
  VCVTTPD2DQZrrb = 7673,
  VCVTTPD2DQZrrbk = 7674,
  VCVTTPD2DQZrrbkz = 7675,
  VCVTTPD2DQZrrk = 7676,
  VCVTTPD2DQZrrkz = 7677,
  VCVTTPD2DQrm = 7678,
  VCVTTPD2DQrr = 7679,
  VCVTTPD2QQSZ128rm = 7680,
  VCVTTPD2QQSZ128rmb = 7681,
  VCVTTPD2QQSZ128rmbk = 7682,
  VCVTTPD2QQSZ128rmbkz = 7683,
  VCVTTPD2QQSZ128rmk = 7684,
  VCVTTPD2QQSZ128rmkz = 7685,
  VCVTTPD2QQSZ128rr = 7686,
  VCVTTPD2QQSZ128rrk = 7687,
  VCVTTPD2QQSZ128rrkz = 7688,
  VCVTTPD2QQSZ256rm = 7689,
  VCVTTPD2QQSZ256rmb = 7690,
  VCVTTPD2QQSZ256rmbk = 7691,
  VCVTTPD2QQSZ256rmbkz = 7692,
  VCVTTPD2QQSZ256rmk = 7693,
  VCVTTPD2QQSZ256rmkz = 7694,
  VCVTTPD2QQSZ256rr = 7695,
  VCVTTPD2QQSZ256rrb = 7696,
  VCVTTPD2QQSZ256rrbk = 7697,
  VCVTTPD2QQSZ256rrbkz = 7698,
  VCVTTPD2QQSZ256rrk = 7699,
  VCVTTPD2QQSZ256rrkz = 7700,
  VCVTTPD2QQSZrm = 7701,
  VCVTTPD2QQSZrmb = 7702,
  VCVTTPD2QQSZrmbk = 7703,
  VCVTTPD2QQSZrmbkz = 7704,
  VCVTTPD2QQSZrmk = 7705,
  VCVTTPD2QQSZrmkz = 7706,
  VCVTTPD2QQSZrr = 7707,
  VCVTTPD2QQSZrrb = 7708,
  VCVTTPD2QQSZrrbk = 7709,
  VCVTTPD2QQSZrrbkz = 7710,
  VCVTTPD2QQSZrrk = 7711,
  VCVTTPD2QQSZrrkz = 7712,
  VCVTTPD2QQZ128rm = 7713,
  VCVTTPD2QQZ128rmb = 7714,
  VCVTTPD2QQZ128rmbk = 7715,
  VCVTTPD2QQZ128rmbkz = 7716,
  VCVTTPD2QQZ128rmk = 7717,
  VCVTTPD2QQZ128rmkz = 7718,
  VCVTTPD2QQZ128rr = 7719,
  VCVTTPD2QQZ128rrk = 7720,
  VCVTTPD2QQZ128rrkz = 7721,
  VCVTTPD2QQZ256rm = 7722,
  VCVTTPD2QQZ256rmb = 7723,
  VCVTTPD2QQZ256rmbk = 7724,
  VCVTTPD2QQZ256rmbkz = 7725,
  VCVTTPD2QQZ256rmk = 7726,
  VCVTTPD2QQZ256rmkz = 7727,
  VCVTTPD2QQZ256rr = 7728,
  VCVTTPD2QQZ256rrk = 7729,
  VCVTTPD2QQZ256rrkz = 7730,
  VCVTTPD2QQZrm = 7731,
  VCVTTPD2QQZrmb = 7732,
  VCVTTPD2QQZrmbk = 7733,
  VCVTTPD2QQZrmbkz = 7734,
  VCVTTPD2QQZrmk = 7735,
  VCVTTPD2QQZrmkz = 7736,
  VCVTTPD2QQZrr = 7737,
  VCVTTPD2QQZrrb = 7738,
  VCVTTPD2QQZrrbk = 7739,
  VCVTTPD2QQZrrbkz = 7740,
  VCVTTPD2QQZrrk = 7741,
  VCVTTPD2QQZrrkz = 7742,
  VCVTTPD2UDQSZ128rm = 7743,
  VCVTTPD2UDQSZ128rmb = 7744,
  VCVTTPD2UDQSZ128rmbk = 7745,
  VCVTTPD2UDQSZ128rmbkz = 7746,
  VCVTTPD2UDQSZ128rmk = 7747,
  VCVTTPD2UDQSZ128rmkz = 7748,
  VCVTTPD2UDQSZ128rr = 7749,
  VCVTTPD2UDQSZ128rrk = 7750,
  VCVTTPD2UDQSZ128rrkz = 7751,
  VCVTTPD2UDQSZ256rm = 7752,
  VCVTTPD2UDQSZ256rmb = 7753,
  VCVTTPD2UDQSZ256rmbk = 7754,
  VCVTTPD2UDQSZ256rmbkz = 7755,
  VCVTTPD2UDQSZ256rmk = 7756,
  VCVTTPD2UDQSZ256rmkz = 7757,
  VCVTTPD2UDQSZ256rr = 7758,
  VCVTTPD2UDQSZ256rrb = 7759,
  VCVTTPD2UDQSZ256rrbk = 7760,
  VCVTTPD2UDQSZ256rrbkz = 7761,
  VCVTTPD2UDQSZ256rrk = 7762,
  VCVTTPD2UDQSZ256rrkz = 7763,
  VCVTTPD2UDQSZrm = 7764,
  VCVTTPD2UDQSZrmb = 7765,
  VCVTTPD2UDQSZrmbk = 7766,
  VCVTTPD2UDQSZrmbkz = 7767,
  VCVTTPD2UDQSZrmk = 7768,
  VCVTTPD2UDQSZrmkz = 7769,
  VCVTTPD2UDQSZrr = 7770,
  VCVTTPD2UDQSZrrb = 7771,
  VCVTTPD2UDQSZrrbk = 7772,
  VCVTTPD2UDQSZrrbkz = 7773,
  VCVTTPD2UDQSZrrk = 7774,
  VCVTTPD2UDQSZrrkz = 7775,
  VCVTTPD2UDQZ128rm = 7776,
  VCVTTPD2UDQZ128rmb = 7777,
  VCVTTPD2UDQZ128rmbk = 7778,
  VCVTTPD2UDQZ128rmbkz = 7779,
  VCVTTPD2UDQZ128rmk = 7780,
  VCVTTPD2UDQZ128rmkz = 7781,
  VCVTTPD2UDQZ128rr = 7782,
  VCVTTPD2UDQZ128rrk = 7783,
  VCVTTPD2UDQZ128rrkz = 7784,
  VCVTTPD2UDQZ256rm = 7785,
  VCVTTPD2UDQZ256rmb = 7786,
  VCVTTPD2UDQZ256rmbk = 7787,
  VCVTTPD2UDQZ256rmbkz = 7788,
  VCVTTPD2UDQZ256rmk = 7789,
  VCVTTPD2UDQZ256rmkz = 7790,
  VCVTTPD2UDQZ256rr = 7791,
  VCVTTPD2UDQZ256rrk = 7792,
  VCVTTPD2UDQZ256rrkz = 7793,
  VCVTTPD2UDQZrm = 7794,
  VCVTTPD2UDQZrmb = 7795,
  VCVTTPD2UDQZrmbk = 7796,
  VCVTTPD2UDQZrmbkz = 7797,
  VCVTTPD2UDQZrmk = 7798,
  VCVTTPD2UDQZrmkz = 7799,
  VCVTTPD2UDQZrr = 7800,
  VCVTTPD2UDQZrrb = 7801,
  VCVTTPD2UDQZrrbk = 7802,
  VCVTTPD2UDQZrrbkz = 7803,
  VCVTTPD2UDQZrrk = 7804,
  VCVTTPD2UDQZrrkz = 7805,
  VCVTTPD2UQQSZ128rm = 7806,
  VCVTTPD2UQQSZ128rmb = 7807,
  VCVTTPD2UQQSZ128rmbk = 7808,
  VCVTTPD2UQQSZ128rmbkz = 7809,
  VCVTTPD2UQQSZ128rmk = 7810,
  VCVTTPD2UQQSZ128rmkz = 7811,
  VCVTTPD2UQQSZ128rr = 7812,
  VCVTTPD2UQQSZ128rrk = 7813,
  VCVTTPD2UQQSZ128rrkz = 7814,
  VCVTTPD2UQQSZ256rm = 7815,
  VCVTTPD2UQQSZ256rmb = 7816,
  VCVTTPD2UQQSZ256rmbk = 7817,
  VCVTTPD2UQQSZ256rmbkz = 7818,
  VCVTTPD2UQQSZ256rmk = 7819,
  VCVTTPD2UQQSZ256rmkz = 7820,
  VCVTTPD2UQQSZ256rr = 7821,
  VCVTTPD2UQQSZ256rrb = 7822,
  VCVTTPD2UQQSZ256rrbk = 7823,
  VCVTTPD2UQQSZ256rrbkz = 7824,
  VCVTTPD2UQQSZ256rrk = 7825,
  VCVTTPD2UQQSZ256rrkz = 7826,
  VCVTTPD2UQQSZrm = 7827,
  VCVTTPD2UQQSZrmb = 7828,
  VCVTTPD2UQQSZrmbk = 7829,
  VCVTTPD2UQQSZrmbkz = 7830,
  VCVTTPD2UQQSZrmk = 7831,
  VCVTTPD2UQQSZrmkz = 7832,
  VCVTTPD2UQQSZrr = 7833,
  VCVTTPD2UQQSZrrb = 7834,
  VCVTTPD2UQQSZrrbk = 7835,
  VCVTTPD2UQQSZrrbkz = 7836,
  VCVTTPD2UQQSZrrk = 7837,
  VCVTTPD2UQQSZrrkz = 7838,
  VCVTTPD2UQQZ128rm = 7839,
  VCVTTPD2UQQZ128rmb = 7840,
  VCVTTPD2UQQZ128rmbk = 7841,
  VCVTTPD2UQQZ128rmbkz = 7842,
  VCVTTPD2UQQZ128rmk = 7843,
  VCVTTPD2UQQZ128rmkz = 7844,
  VCVTTPD2UQQZ128rr = 7845,
  VCVTTPD2UQQZ128rrk = 7846,
  VCVTTPD2UQQZ128rrkz = 7847,
  VCVTTPD2UQQZ256rm = 7848,
  VCVTTPD2UQQZ256rmb = 7849,
  VCVTTPD2UQQZ256rmbk = 7850,
  VCVTTPD2UQQZ256rmbkz = 7851,
  VCVTTPD2UQQZ256rmk = 7852,
  VCVTTPD2UQQZ256rmkz = 7853,
  VCVTTPD2UQQZ256rr = 7854,
  VCVTTPD2UQQZ256rrk = 7855,
  VCVTTPD2UQQZ256rrkz = 7856,
  VCVTTPD2UQQZrm = 7857,
  VCVTTPD2UQQZrmb = 7858,
  VCVTTPD2UQQZrmbk = 7859,
  VCVTTPD2UQQZrmbkz = 7860,
  VCVTTPD2UQQZrmk = 7861,
  VCVTTPD2UQQZrmkz = 7862,
  VCVTTPD2UQQZrr = 7863,
  VCVTTPD2UQQZrrb = 7864,
  VCVTTPD2UQQZrrbk = 7865,
  VCVTTPD2UQQZrrbkz = 7866,
  VCVTTPD2UQQZrrk = 7867,
  VCVTTPD2UQQZrrkz = 7868,
  VCVTTPH2DQZ128rm = 7869,
  VCVTTPH2DQZ128rmb = 7870,
  VCVTTPH2DQZ128rmbk = 7871,
  VCVTTPH2DQZ128rmbkz = 7872,
  VCVTTPH2DQZ128rmk = 7873,
  VCVTTPH2DQZ128rmkz = 7874,
  VCVTTPH2DQZ128rr = 7875,
  VCVTTPH2DQZ128rrk = 7876,
  VCVTTPH2DQZ128rrkz = 7877,
  VCVTTPH2DQZ256rm = 7878,
  VCVTTPH2DQZ256rmb = 7879,
  VCVTTPH2DQZ256rmbk = 7880,
  VCVTTPH2DQZ256rmbkz = 7881,
  VCVTTPH2DQZ256rmk = 7882,
  VCVTTPH2DQZ256rmkz = 7883,
  VCVTTPH2DQZ256rr = 7884,
  VCVTTPH2DQZ256rrk = 7885,
  VCVTTPH2DQZ256rrkz = 7886,
  VCVTTPH2DQZrm = 7887,
  VCVTTPH2DQZrmb = 7888,
  VCVTTPH2DQZrmbk = 7889,
  VCVTTPH2DQZrmbkz = 7890,
  VCVTTPH2DQZrmk = 7891,
  VCVTTPH2DQZrmkz = 7892,
  VCVTTPH2DQZrr = 7893,
  VCVTTPH2DQZrrb = 7894,
  VCVTTPH2DQZrrbk = 7895,
  VCVTTPH2DQZrrbkz = 7896,
  VCVTTPH2DQZrrk = 7897,
  VCVTTPH2DQZrrkz = 7898,
  VCVTTPH2IBSZ128rm = 7899,
  VCVTTPH2IBSZ128rmb = 7900,
  VCVTTPH2IBSZ128rmbk = 7901,
  VCVTTPH2IBSZ128rmbkz = 7902,
  VCVTTPH2IBSZ128rmk = 7903,
  VCVTTPH2IBSZ128rmkz = 7904,
  VCVTTPH2IBSZ128rr = 7905,
  VCVTTPH2IBSZ128rrk = 7906,
  VCVTTPH2IBSZ128rrkz = 7907,
  VCVTTPH2IBSZ256rm = 7908,
  VCVTTPH2IBSZ256rmb = 7909,
  VCVTTPH2IBSZ256rmbk = 7910,
  VCVTTPH2IBSZ256rmbkz = 7911,
  VCVTTPH2IBSZ256rmk = 7912,
  VCVTTPH2IBSZ256rmkz = 7913,
  VCVTTPH2IBSZ256rr = 7914,
  VCVTTPH2IBSZ256rrk = 7915,
  VCVTTPH2IBSZ256rrkz = 7916,
  VCVTTPH2IBSZrm = 7917,
  VCVTTPH2IBSZrmb = 7918,
  VCVTTPH2IBSZrmbk = 7919,
  VCVTTPH2IBSZrmbkz = 7920,
  VCVTTPH2IBSZrmk = 7921,
  VCVTTPH2IBSZrmkz = 7922,
  VCVTTPH2IBSZrr = 7923,
  VCVTTPH2IBSZrrb = 7924,
  VCVTTPH2IBSZrrbk = 7925,
  VCVTTPH2IBSZrrbkz = 7926,
  VCVTTPH2IBSZrrk = 7927,
  VCVTTPH2IBSZrrkz = 7928,
  VCVTTPH2IUBSZ128rm = 7929,
  VCVTTPH2IUBSZ128rmb = 7930,
  VCVTTPH2IUBSZ128rmbk = 7931,
  VCVTTPH2IUBSZ128rmbkz = 7932,
  VCVTTPH2IUBSZ128rmk = 7933,
  VCVTTPH2IUBSZ128rmkz = 7934,
  VCVTTPH2IUBSZ128rr = 7935,
  VCVTTPH2IUBSZ128rrk = 7936,
  VCVTTPH2IUBSZ128rrkz = 7937,
  VCVTTPH2IUBSZ256rm = 7938,
  VCVTTPH2IUBSZ256rmb = 7939,
  VCVTTPH2IUBSZ256rmbk = 7940,
  VCVTTPH2IUBSZ256rmbkz = 7941,
  VCVTTPH2IUBSZ256rmk = 7942,
  VCVTTPH2IUBSZ256rmkz = 7943,
  VCVTTPH2IUBSZ256rr = 7944,
  VCVTTPH2IUBSZ256rrk = 7945,
  VCVTTPH2IUBSZ256rrkz = 7946,
  VCVTTPH2IUBSZrm = 7947,
  VCVTTPH2IUBSZrmb = 7948,
  VCVTTPH2IUBSZrmbk = 7949,
  VCVTTPH2IUBSZrmbkz = 7950,
  VCVTTPH2IUBSZrmk = 7951,
  VCVTTPH2IUBSZrmkz = 7952,
  VCVTTPH2IUBSZrr = 7953,
  VCVTTPH2IUBSZrrb = 7954,
  VCVTTPH2IUBSZrrbk = 7955,
  VCVTTPH2IUBSZrrbkz = 7956,
  VCVTTPH2IUBSZrrk = 7957,
  VCVTTPH2IUBSZrrkz = 7958,
  VCVTTPH2QQZ128rm = 7959,
  VCVTTPH2QQZ128rmb = 7960,
  VCVTTPH2QQZ128rmbk = 7961,
  VCVTTPH2QQZ128rmbkz = 7962,
  VCVTTPH2QQZ128rmk = 7963,
  VCVTTPH2QQZ128rmkz = 7964,
  VCVTTPH2QQZ128rr = 7965,
  VCVTTPH2QQZ128rrk = 7966,
  VCVTTPH2QQZ128rrkz = 7967,
  VCVTTPH2QQZ256rm = 7968,
  VCVTTPH2QQZ256rmb = 7969,
  VCVTTPH2QQZ256rmbk = 7970,
  VCVTTPH2QQZ256rmbkz = 7971,
  VCVTTPH2QQZ256rmk = 7972,
  VCVTTPH2QQZ256rmkz = 7973,
  VCVTTPH2QQZ256rr = 7974,
  VCVTTPH2QQZ256rrk = 7975,
  VCVTTPH2QQZ256rrkz = 7976,
  VCVTTPH2QQZrm = 7977,
  VCVTTPH2QQZrmb = 7978,
  VCVTTPH2QQZrmbk = 7979,
  VCVTTPH2QQZrmbkz = 7980,
  VCVTTPH2QQZrmk = 7981,
  VCVTTPH2QQZrmkz = 7982,
  VCVTTPH2QQZrr = 7983,
  VCVTTPH2QQZrrb = 7984,
  VCVTTPH2QQZrrbk = 7985,
  VCVTTPH2QQZrrbkz = 7986,
  VCVTTPH2QQZrrk = 7987,
  VCVTTPH2QQZrrkz = 7988,
  VCVTTPH2UDQZ128rm = 7989,
  VCVTTPH2UDQZ128rmb = 7990,
  VCVTTPH2UDQZ128rmbk = 7991,
  VCVTTPH2UDQZ128rmbkz = 7992,
  VCVTTPH2UDQZ128rmk = 7993,
  VCVTTPH2UDQZ128rmkz = 7994,
  VCVTTPH2UDQZ128rr = 7995,
  VCVTTPH2UDQZ128rrk = 7996,
  VCVTTPH2UDQZ128rrkz = 7997,
  VCVTTPH2UDQZ256rm = 7998,
  VCVTTPH2UDQZ256rmb = 7999,
  VCVTTPH2UDQZ256rmbk = 8000,
  VCVTTPH2UDQZ256rmbkz = 8001,
  VCVTTPH2UDQZ256rmk = 8002,
  VCVTTPH2UDQZ256rmkz = 8003,
  VCVTTPH2UDQZ256rr = 8004,
  VCVTTPH2UDQZ256rrk = 8005,
  VCVTTPH2UDQZ256rrkz = 8006,
  VCVTTPH2UDQZrm = 8007,
  VCVTTPH2UDQZrmb = 8008,
  VCVTTPH2UDQZrmbk = 8009,
  VCVTTPH2UDQZrmbkz = 8010,
  VCVTTPH2UDQZrmk = 8011,
  VCVTTPH2UDQZrmkz = 8012,
  VCVTTPH2UDQZrr = 8013,
  VCVTTPH2UDQZrrb = 8014,
  VCVTTPH2UDQZrrbk = 8015,
  VCVTTPH2UDQZrrbkz = 8016,
  VCVTTPH2UDQZrrk = 8017,
  VCVTTPH2UDQZrrkz = 8018,
  VCVTTPH2UQQZ128rm = 8019,
  VCVTTPH2UQQZ128rmb = 8020,
  VCVTTPH2UQQZ128rmbk = 8021,
  VCVTTPH2UQQZ128rmbkz = 8022,
  VCVTTPH2UQQZ128rmk = 8023,
  VCVTTPH2UQQZ128rmkz = 8024,
  VCVTTPH2UQQZ128rr = 8025,
  VCVTTPH2UQQZ128rrk = 8026,
  VCVTTPH2UQQZ128rrkz = 8027,
  VCVTTPH2UQQZ256rm = 8028,
  VCVTTPH2UQQZ256rmb = 8029,
  VCVTTPH2UQQZ256rmbk = 8030,
  VCVTTPH2UQQZ256rmbkz = 8031,
  VCVTTPH2UQQZ256rmk = 8032,
  VCVTTPH2UQQZ256rmkz = 8033,
  VCVTTPH2UQQZ256rr = 8034,
  VCVTTPH2UQQZ256rrk = 8035,
  VCVTTPH2UQQZ256rrkz = 8036,
  VCVTTPH2UQQZrm = 8037,
  VCVTTPH2UQQZrmb = 8038,
  VCVTTPH2UQQZrmbk = 8039,
  VCVTTPH2UQQZrmbkz = 8040,
  VCVTTPH2UQQZrmk = 8041,
  VCVTTPH2UQQZrmkz = 8042,
  VCVTTPH2UQQZrr = 8043,
  VCVTTPH2UQQZrrb = 8044,
  VCVTTPH2UQQZrrbk = 8045,
  VCVTTPH2UQQZrrbkz = 8046,
  VCVTTPH2UQQZrrk = 8047,
  VCVTTPH2UQQZrrkz = 8048,
  VCVTTPH2UWZ128rm = 8049,
  VCVTTPH2UWZ128rmb = 8050,
  VCVTTPH2UWZ128rmbk = 8051,
  VCVTTPH2UWZ128rmbkz = 8052,
  VCVTTPH2UWZ128rmk = 8053,
  VCVTTPH2UWZ128rmkz = 8054,
  VCVTTPH2UWZ128rr = 8055,
  VCVTTPH2UWZ128rrk = 8056,
  VCVTTPH2UWZ128rrkz = 8057,
  VCVTTPH2UWZ256rm = 8058,
  VCVTTPH2UWZ256rmb = 8059,
  VCVTTPH2UWZ256rmbk = 8060,
  VCVTTPH2UWZ256rmbkz = 8061,
  VCVTTPH2UWZ256rmk = 8062,
  VCVTTPH2UWZ256rmkz = 8063,
  VCVTTPH2UWZ256rr = 8064,
  VCVTTPH2UWZ256rrk = 8065,
  VCVTTPH2UWZ256rrkz = 8066,
  VCVTTPH2UWZrm = 8067,
  VCVTTPH2UWZrmb = 8068,
  VCVTTPH2UWZrmbk = 8069,
  VCVTTPH2UWZrmbkz = 8070,
  VCVTTPH2UWZrmk = 8071,
  VCVTTPH2UWZrmkz = 8072,
  VCVTTPH2UWZrr = 8073,
  VCVTTPH2UWZrrb = 8074,
  VCVTTPH2UWZrrbk = 8075,
  VCVTTPH2UWZrrbkz = 8076,
  VCVTTPH2UWZrrk = 8077,
  VCVTTPH2UWZrrkz = 8078,
  VCVTTPH2WZ128rm = 8079,
  VCVTTPH2WZ128rmb = 8080,
  VCVTTPH2WZ128rmbk = 8081,
  VCVTTPH2WZ128rmbkz = 8082,
  VCVTTPH2WZ128rmk = 8083,
  VCVTTPH2WZ128rmkz = 8084,
  VCVTTPH2WZ128rr = 8085,
  VCVTTPH2WZ128rrk = 8086,
  VCVTTPH2WZ128rrkz = 8087,
  VCVTTPH2WZ256rm = 8088,
  VCVTTPH2WZ256rmb = 8089,
  VCVTTPH2WZ256rmbk = 8090,
  VCVTTPH2WZ256rmbkz = 8091,
  VCVTTPH2WZ256rmk = 8092,
  VCVTTPH2WZ256rmkz = 8093,
  VCVTTPH2WZ256rr = 8094,
  VCVTTPH2WZ256rrk = 8095,
  VCVTTPH2WZ256rrkz = 8096,
  VCVTTPH2WZrm = 8097,
  VCVTTPH2WZrmb = 8098,
  VCVTTPH2WZrmbk = 8099,
  VCVTTPH2WZrmbkz = 8100,
  VCVTTPH2WZrmk = 8101,
  VCVTTPH2WZrmkz = 8102,
  VCVTTPH2WZrr = 8103,
  VCVTTPH2WZrrb = 8104,
  VCVTTPH2WZrrbk = 8105,
  VCVTTPH2WZrrbkz = 8106,
  VCVTTPH2WZrrk = 8107,
  VCVTTPH2WZrrkz = 8108,
  VCVTTPS2DQSZ128rm = 8109,
  VCVTTPS2DQSZ128rmb = 8110,
  VCVTTPS2DQSZ128rmbk = 8111,
  VCVTTPS2DQSZ128rmbkz = 8112,
  VCVTTPS2DQSZ128rmk = 8113,
  VCVTTPS2DQSZ128rmkz = 8114,
  VCVTTPS2DQSZ128rr = 8115,
  VCVTTPS2DQSZ128rrk = 8116,
  VCVTTPS2DQSZ128rrkz = 8117,
  VCVTTPS2DQSZ256rm = 8118,
  VCVTTPS2DQSZ256rmb = 8119,
  VCVTTPS2DQSZ256rmbk = 8120,
  VCVTTPS2DQSZ256rmbkz = 8121,
  VCVTTPS2DQSZ256rmk = 8122,
  VCVTTPS2DQSZ256rmkz = 8123,
  VCVTTPS2DQSZ256rr = 8124,
  VCVTTPS2DQSZ256rrk = 8125,
  VCVTTPS2DQSZ256rrkz = 8126,
  VCVTTPS2DQSZrm = 8127,
  VCVTTPS2DQSZrmb = 8128,
  VCVTTPS2DQSZrmbk = 8129,
  VCVTTPS2DQSZrmbkz = 8130,
  VCVTTPS2DQSZrmk = 8131,
  VCVTTPS2DQSZrmkz = 8132,
  VCVTTPS2DQSZrr = 8133,
  VCVTTPS2DQSZrrb = 8134,
  VCVTTPS2DQSZrrbk = 8135,
  VCVTTPS2DQSZrrbkz = 8136,
  VCVTTPS2DQSZrrk = 8137,
  VCVTTPS2DQSZrrkz = 8138,
  VCVTTPS2DQYrm = 8139,
  VCVTTPS2DQYrr = 8140,
  VCVTTPS2DQZ128rm = 8141,
  VCVTTPS2DQZ128rmb = 8142,
  VCVTTPS2DQZ128rmbk = 8143,
  VCVTTPS2DQZ128rmbkz = 8144,
  VCVTTPS2DQZ128rmk = 8145,
  VCVTTPS2DQZ128rmkz = 8146,
  VCVTTPS2DQZ128rr = 8147,
  VCVTTPS2DQZ128rrk = 8148,
  VCVTTPS2DQZ128rrkz = 8149,
  VCVTTPS2DQZ256rm = 8150,
  VCVTTPS2DQZ256rmb = 8151,
  VCVTTPS2DQZ256rmbk = 8152,
  VCVTTPS2DQZ256rmbkz = 8153,
  VCVTTPS2DQZ256rmk = 8154,
  VCVTTPS2DQZ256rmkz = 8155,
  VCVTTPS2DQZ256rr = 8156,
  VCVTTPS2DQZ256rrk = 8157,
  VCVTTPS2DQZ256rrkz = 8158,
  VCVTTPS2DQZrm = 8159,
  VCVTTPS2DQZrmb = 8160,
  VCVTTPS2DQZrmbk = 8161,
  VCVTTPS2DQZrmbkz = 8162,
  VCVTTPS2DQZrmk = 8163,
  VCVTTPS2DQZrmkz = 8164,
  VCVTTPS2DQZrr = 8165,
  VCVTTPS2DQZrrb = 8166,
  VCVTTPS2DQZrrbk = 8167,
  VCVTTPS2DQZrrbkz = 8168,
  VCVTTPS2DQZrrk = 8169,
  VCVTTPS2DQZrrkz = 8170,
  VCVTTPS2DQrm = 8171,
  VCVTTPS2DQrr = 8172,
  VCVTTPS2IBSZ128rm = 8173,
  VCVTTPS2IBSZ128rmb = 8174,
  VCVTTPS2IBSZ128rmbk = 8175,
  VCVTTPS2IBSZ128rmbkz = 8176,
  VCVTTPS2IBSZ128rmk = 8177,
  VCVTTPS2IBSZ128rmkz = 8178,
  VCVTTPS2IBSZ128rr = 8179,
  VCVTTPS2IBSZ128rrk = 8180,
  VCVTTPS2IBSZ128rrkz = 8181,
  VCVTTPS2IBSZ256rm = 8182,
  VCVTTPS2IBSZ256rmb = 8183,
  VCVTTPS2IBSZ256rmbk = 8184,
  VCVTTPS2IBSZ256rmbkz = 8185,
  VCVTTPS2IBSZ256rmk = 8186,
  VCVTTPS2IBSZ256rmkz = 8187,
  VCVTTPS2IBSZ256rr = 8188,
  VCVTTPS2IBSZ256rrk = 8189,
  VCVTTPS2IBSZ256rrkz = 8190,
  VCVTTPS2IBSZrm = 8191,
  VCVTTPS2IBSZrmb = 8192,
  VCVTTPS2IBSZrmbk = 8193,
  VCVTTPS2IBSZrmbkz = 8194,
  VCVTTPS2IBSZrmk = 8195,
  VCVTTPS2IBSZrmkz = 8196,
  VCVTTPS2IBSZrr = 8197,
  VCVTTPS2IBSZrrb = 8198,
  VCVTTPS2IBSZrrbk = 8199,
  VCVTTPS2IBSZrrbkz = 8200,
  VCVTTPS2IBSZrrk = 8201,
  VCVTTPS2IBSZrrkz = 8202,
  VCVTTPS2IUBSZ128rm = 8203,
  VCVTTPS2IUBSZ128rmb = 8204,
  VCVTTPS2IUBSZ128rmbk = 8205,
  VCVTTPS2IUBSZ128rmbkz = 8206,
  VCVTTPS2IUBSZ128rmk = 8207,
  VCVTTPS2IUBSZ128rmkz = 8208,
  VCVTTPS2IUBSZ128rr = 8209,
  VCVTTPS2IUBSZ128rrk = 8210,
  VCVTTPS2IUBSZ128rrkz = 8211,
  VCVTTPS2IUBSZ256rm = 8212,
  VCVTTPS2IUBSZ256rmb = 8213,
  VCVTTPS2IUBSZ256rmbk = 8214,
  VCVTTPS2IUBSZ256rmbkz = 8215,
  VCVTTPS2IUBSZ256rmk = 8216,
  VCVTTPS2IUBSZ256rmkz = 8217,
  VCVTTPS2IUBSZ256rr = 8218,
  VCVTTPS2IUBSZ256rrk = 8219,
  VCVTTPS2IUBSZ256rrkz = 8220,
  VCVTTPS2IUBSZrm = 8221,
  VCVTTPS2IUBSZrmb = 8222,
  VCVTTPS2IUBSZrmbk = 8223,
  VCVTTPS2IUBSZrmbkz = 8224,
  VCVTTPS2IUBSZrmk = 8225,
  VCVTTPS2IUBSZrmkz = 8226,
  VCVTTPS2IUBSZrr = 8227,
  VCVTTPS2IUBSZrrb = 8228,
  VCVTTPS2IUBSZrrbk = 8229,
  VCVTTPS2IUBSZrrbkz = 8230,
  VCVTTPS2IUBSZrrk = 8231,
  VCVTTPS2IUBSZrrkz = 8232,
  VCVTTPS2QQSZ128rm = 8233,
  VCVTTPS2QQSZ128rmb = 8234,
  VCVTTPS2QQSZ128rmbk = 8235,
  VCVTTPS2QQSZ128rmbkz = 8236,
  VCVTTPS2QQSZ128rmk = 8237,
  VCVTTPS2QQSZ128rmkz = 8238,
  VCVTTPS2QQSZ128rr = 8239,
  VCVTTPS2QQSZ128rrk = 8240,
  VCVTTPS2QQSZ128rrkz = 8241,
  VCVTTPS2QQSZ256rm = 8242,
  VCVTTPS2QQSZ256rmb = 8243,
  VCVTTPS2QQSZ256rmbk = 8244,
  VCVTTPS2QQSZ256rmbkz = 8245,
  VCVTTPS2QQSZ256rmk = 8246,
  VCVTTPS2QQSZ256rmkz = 8247,
  VCVTTPS2QQSZ256rr = 8248,
  VCVTTPS2QQSZ256rrb = 8249,
  VCVTTPS2QQSZ256rrbk = 8250,
  VCVTTPS2QQSZ256rrbkz = 8251,
  VCVTTPS2QQSZ256rrk = 8252,
  VCVTTPS2QQSZ256rrkz = 8253,
  VCVTTPS2QQSZrm = 8254,
  VCVTTPS2QQSZrmb = 8255,
  VCVTTPS2QQSZrmbk = 8256,
  VCVTTPS2QQSZrmbkz = 8257,
  VCVTTPS2QQSZrmk = 8258,
  VCVTTPS2QQSZrmkz = 8259,
  VCVTTPS2QQSZrr = 8260,
  VCVTTPS2QQSZrrb = 8261,
  VCVTTPS2QQSZrrbk = 8262,
  VCVTTPS2QQSZrrbkz = 8263,
  VCVTTPS2QQSZrrk = 8264,
  VCVTTPS2QQSZrrkz = 8265,
  VCVTTPS2QQZ128rm = 8266,
  VCVTTPS2QQZ128rmb = 8267,
  VCVTTPS2QQZ128rmbk = 8268,
  VCVTTPS2QQZ128rmbkz = 8269,
  VCVTTPS2QQZ128rmk = 8270,
  VCVTTPS2QQZ128rmkz = 8271,
  VCVTTPS2QQZ128rr = 8272,
  VCVTTPS2QQZ128rrk = 8273,
  VCVTTPS2QQZ128rrkz = 8274,
  VCVTTPS2QQZ256rm = 8275,
  VCVTTPS2QQZ256rmb = 8276,
  VCVTTPS2QQZ256rmbk = 8277,
  VCVTTPS2QQZ256rmbkz = 8278,
  VCVTTPS2QQZ256rmk = 8279,
  VCVTTPS2QQZ256rmkz = 8280,
  VCVTTPS2QQZ256rr = 8281,
  VCVTTPS2QQZ256rrk = 8282,
  VCVTTPS2QQZ256rrkz = 8283,
  VCVTTPS2QQZrm = 8284,
  VCVTTPS2QQZrmb = 8285,
  VCVTTPS2QQZrmbk = 8286,
  VCVTTPS2QQZrmbkz = 8287,
  VCVTTPS2QQZrmk = 8288,
  VCVTTPS2QQZrmkz = 8289,
  VCVTTPS2QQZrr = 8290,
  VCVTTPS2QQZrrb = 8291,
  VCVTTPS2QQZrrbk = 8292,
  VCVTTPS2QQZrrbkz = 8293,
  VCVTTPS2QQZrrk = 8294,
  VCVTTPS2QQZrrkz = 8295,
  VCVTTPS2UDQSZ128rm = 8296,
  VCVTTPS2UDQSZ128rmb = 8297,
  VCVTTPS2UDQSZ128rmbk = 8298,
  VCVTTPS2UDQSZ128rmbkz = 8299,
  VCVTTPS2UDQSZ128rmk = 8300,
  VCVTTPS2UDQSZ128rmkz = 8301,
  VCVTTPS2UDQSZ128rr = 8302,
  VCVTTPS2UDQSZ128rrk = 8303,
  VCVTTPS2UDQSZ128rrkz = 8304,
  VCVTTPS2UDQSZ256rm = 8305,
  VCVTTPS2UDQSZ256rmb = 8306,
  VCVTTPS2UDQSZ256rmbk = 8307,
  VCVTTPS2UDQSZ256rmbkz = 8308,
  VCVTTPS2UDQSZ256rmk = 8309,
  VCVTTPS2UDQSZ256rmkz = 8310,
  VCVTTPS2UDQSZ256rr = 8311,
  VCVTTPS2UDQSZ256rrk = 8312,
  VCVTTPS2UDQSZ256rrkz = 8313,
  VCVTTPS2UDQSZrm = 8314,
  VCVTTPS2UDQSZrmb = 8315,
  VCVTTPS2UDQSZrmbk = 8316,
  VCVTTPS2UDQSZrmbkz = 8317,
  VCVTTPS2UDQSZrmk = 8318,
  VCVTTPS2UDQSZrmkz = 8319,
  VCVTTPS2UDQSZrr = 8320,
  VCVTTPS2UDQSZrrb = 8321,
  VCVTTPS2UDQSZrrbk = 8322,
  VCVTTPS2UDQSZrrbkz = 8323,
  VCVTTPS2UDQSZrrk = 8324,
  VCVTTPS2UDQSZrrkz = 8325,
  VCVTTPS2UDQZ128rm = 8326,
  VCVTTPS2UDQZ128rmb = 8327,
  VCVTTPS2UDQZ128rmbk = 8328,
  VCVTTPS2UDQZ128rmbkz = 8329,
  VCVTTPS2UDQZ128rmk = 8330,
  VCVTTPS2UDQZ128rmkz = 8331,
  VCVTTPS2UDQZ128rr = 8332,
  VCVTTPS2UDQZ128rrk = 8333,
  VCVTTPS2UDQZ128rrkz = 8334,
  VCVTTPS2UDQZ256rm = 8335,
  VCVTTPS2UDQZ256rmb = 8336,
  VCVTTPS2UDQZ256rmbk = 8337,
  VCVTTPS2UDQZ256rmbkz = 8338,
  VCVTTPS2UDQZ256rmk = 8339,
  VCVTTPS2UDQZ256rmkz = 8340,
  VCVTTPS2UDQZ256rr = 8341,
  VCVTTPS2UDQZ256rrk = 8342,
  VCVTTPS2UDQZ256rrkz = 8343,
  VCVTTPS2UDQZrm = 8344,
  VCVTTPS2UDQZrmb = 8345,
  VCVTTPS2UDQZrmbk = 8346,
  VCVTTPS2UDQZrmbkz = 8347,
  VCVTTPS2UDQZrmk = 8348,
  VCVTTPS2UDQZrmkz = 8349,
  VCVTTPS2UDQZrr = 8350,
  VCVTTPS2UDQZrrb = 8351,
  VCVTTPS2UDQZrrbk = 8352,
  VCVTTPS2UDQZrrbkz = 8353,
  VCVTTPS2UDQZrrk = 8354,
  VCVTTPS2UDQZrrkz = 8355,
  VCVTTPS2UQQSZ128rm = 8356,
  VCVTTPS2UQQSZ128rmb = 8357,
  VCVTTPS2UQQSZ128rmbk = 8358,
  VCVTTPS2UQQSZ128rmbkz = 8359,
  VCVTTPS2UQQSZ128rmk = 8360,
  VCVTTPS2UQQSZ128rmkz = 8361,
  VCVTTPS2UQQSZ128rr = 8362,
  VCVTTPS2UQQSZ128rrk = 8363,
  VCVTTPS2UQQSZ128rrkz = 8364,
  VCVTTPS2UQQSZ256rm = 8365,
  VCVTTPS2UQQSZ256rmb = 8366,
  VCVTTPS2UQQSZ256rmbk = 8367,
  VCVTTPS2UQQSZ256rmbkz = 8368,
  VCVTTPS2UQQSZ256rmk = 8369,
  VCVTTPS2UQQSZ256rmkz = 8370,
  VCVTTPS2UQQSZ256rr = 8371,
  VCVTTPS2UQQSZ256rrb = 8372,
  VCVTTPS2UQQSZ256rrbk = 8373,
  VCVTTPS2UQQSZ256rrbkz = 8374,
  VCVTTPS2UQQSZ256rrk = 8375,
  VCVTTPS2UQQSZ256rrkz = 8376,
  VCVTTPS2UQQSZrm = 8377,
  VCVTTPS2UQQSZrmb = 8378,
  VCVTTPS2UQQSZrmbk = 8379,
  VCVTTPS2UQQSZrmbkz = 8380,
  VCVTTPS2UQQSZrmk = 8381,
  VCVTTPS2UQQSZrmkz = 8382,
  VCVTTPS2UQQSZrr = 8383,
  VCVTTPS2UQQSZrrb = 8384,
  VCVTTPS2UQQSZrrbk = 8385,
  VCVTTPS2UQQSZrrbkz = 8386,
  VCVTTPS2UQQSZrrk = 8387,
  VCVTTPS2UQQSZrrkz = 8388,
  VCVTTPS2UQQZ128rm = 8389,
  VCVTTPS2UQQZ128rmb = 8390,
  VCVTTPS2UQQZ128rmbk = 8391,
  VCVTTPS2UQQZ128rmbkz = 8392,
  VCVTTPS2UQQZ128rmk = 8393,
  VCVTTPS2UQQZ128rmkz = 8394,
  VCVTTPS2UQQZ128rr = 8395,
  VCVTTPS2UQQZ128rrk = 8396,
  VCVTTPS2UQQZ128rrkz = 8397,
  VCVTTPS2UQQZ256rm = 8398,
  VCVTTPS2UQQZ256rmb = 8399,
  VCVTTPS2UQQZ256rmbk = 8400,
  VCVTTPS2UQQZ256rmbkz = 8401,
  VCVTTPS2UQQZ256rmk = 8402,
  VCVTTPS2UQQZ256rmkz = 8403,
  VCVTTPS2UQQZ256rr = 8404,
  VCVTTPS2UQQZ256rrk = 8405,
  VCVTTPS2UQQZ256rrkz = 8406,
  VCVTTPS2UQQZrm = 8407,
  VCVTTPS2UQQZrmb = 8408,
  VCVTTPS2UQQZrmbk = 8409,
  VCVTTPS2UQQZrmbkz = 8410,
  VCVTTPS2UQQZrmk = 8411,
  VCVTTPS2UQQZrmkz = 8412,
  VCVTTPS2UQQZrr = 8413,
  VCVTTPS2UQQZrrb = 8414,
  VCVTTPS2UQQZrrbk = 8415,
  VCVTTPS2UQQZrrbkz = 8416,
  VCVTTPS2UQQZrrk = 8417,
  VCVTTPS2UQQZrrkz = 8418,
  VCVTTSD2SI64Srm = 8419,
  VCVTTSD2SI64Srm_Int = 8420,
  VCVTTSD2SI64Srr = 8421,
  VCVTTSD2SI64Srr_Int = 8422,
  VCVTTSD2SI64Srrb_Int = 8423,
  VCVTTSD2SI64Zrm = 8424,
  VCVTTSD2SI64Zrm_Int = 8425,
  VCVTTSD2SI64Zrr = 8426,
  VCVTTSD2SI64Zrr_Int = 8427,
  VCVTTSD2SI64Zrrb_Int = 8428,
  VCVTTSD2SI64rm = 8429,
  VCVTTSD2SI64rm_Int = 8430,
  VCVTTSD2SI64rr = 8431,
  VCVTTSD2SI64rr_Int = 8432,
  VCVTTSD2SISrm = 8433,
  VCVTTSD2SISrm_Int = 8434,
  VCVTTSD2SISrr = 8435,
  VCVTTSD2SISrr_Int = 8436,
  VCVTTSD2SISrrb_Int = 8437,
  VCVTTSD2SIZrm = 8438,
  VCVTTSD2SIZrm_Int = 8439,
  VCVTTSD2SIZrr = 8440,
  VCVTTSD2SIZrr_Int = 8441,
  VCVTTSD2SIZrrb_Int = 8442,
  VCVTTSD2SIrm = 8443,
  VCVTTSD2SIrm_Int = 8444,
  VCVTTSD2SIrr = 8445,
  VCVTTSD2SIrr_Int = 8446,
  VCVTTSD2USI64Srm = 8447,
  VCVTTSD2USI64Srm_Int = 8448,
  VCVTTSD2USI64Srr = 8449,
  VCVTTSD2USI64Srr_Int = 8450,
  VCVTTSD2USI64Srrb_Int = 8451,
  VCVTTSD2USI64Zrm = 8452,
  VCVTTSD2USI64Zrm_Int = 8453,
  VCVTTSD2USI64Zrr = 8454,
  VCVTTSD2USI64Zrr_Int = 8455,
  VCVTTSD2USI64Zrrb_Int = 8456,
  VCVTTSD2USISrm = 8457,
  VCVTTSD2USISrm_Int = 8458,
  VCVTTSD2USISrr = 8459,
  VCVTTSD2USISrr_Int = 8460,
  VCVTTSD2USISrrb_Int = 8461,
  VCVTTSD2USIZrm = 8462,
  VCVTTSD2USIZrm_Int = 8463,
  VCVTTSD2USIZrr = 8464,
  VCVTTSD2USIZrr_Int = 8465,
  VCVTTSD2USIZrrb_Int = 8466,
  VCVTTSH2SI64Zrm = 8467,
  VCVTTSH2SI64Zrm_Int = 8468,
  VCVTTSH2SI64Zrr = 8469,
  VCVTTSH2SI64Zrr_Int = 8470,
  VCVTTSH2SI64Zrrb_Int = 8471,
  VCVTTSH2SIZrm = 8472,
  VCVTTSH2SIZrm_Int = 8473,
  VCVTTSH2SIZrr = 8474,
  VCVTTSH2SIZrr_Int = 8475,
  VCVTTSH2SIZrrb_Int = 8476,
  VCVTTSH2USI64Zrm = 8477,
  VCVTTSH2USI64Zrm_Int = 8478,
  VCVTTSH2USI64Zrr = 8479,
  VCVTTSH2USI64Zrr_Int = 8480,
  VCVTTSH2USI64Zrrb_Int = 8481,
  VCVTTSH2USIZrm = 8482,
  VCVTTSH2USIZrm_Int = 8483,
  VCVTTSH2USIZrr = 8484,
  VCVTTSH2USIZrr_Int = 8485,
  VCVTTSH2USIZrrb_Int = 8486,
  VCVTTSS2SI64Srm = 8487,
  VCVTTSS2SI64Srm_Int = 8488,
  VCVTTSS2SI64Srr = 8489,
  VCVTTSS2SI64Srr_Int = 8490,
  VCVTTSS2SI64Srrb_Int = 8491,
  VCVTTSS2SI64Zrm = 8492,
  VCVTTSS2SI64Zrm_Int = 8493,
  VCVTTSS2SI64Zrr = 8494,
  VCVTTSS2SI64Zrr_Int = 8495,
  VCVTTSS2SI64Zrrb_Int = 8496,
  VCVTTSS2SI64rm = 8497,
  VCVTTSS2SI64rm_Int = 8498,
  VCVTTSS2SI64rr = 8499,
  VCVTTSS2SI64rr_Int = 8500,
  VCVTTSS2SISrm = 8501,
  VCVTTSS2SISrm_Int = 8502,
  VCVTTSS2SISrr = 8503,
  VCVTTSS2SISrr_Int = 8504,
  VCVTTSS2SISrrb_Int = 8505,
  VCVTTSS2SIZrm = 8506,
  VCVTTSS2SIZrm_Int = 8507,
  VCVTTSS2SIZrr = 8508,
  VCVTTSS2SIZrr_Int = 8509,
  VCVTTSS2SIZrrb_Int = 8510,
  VCVTTSS2SIrm = 8511,
  VCVTTSS2SIrm_Int = 8512,
  VCVTTSS2SIrr = 8513,
  VCVTTSS2SIrr_Int = 8514,
  VCVTTSS2USI64Srm = 8515,
  VCVTTSS2USI64Srm_Int = 8516,
  VCVTTSS2USI64Srr = 8517,
  VCVTTSS2USI64Srr_Int = 8518,
  VCVTTSS2USI64Srrb_Int = 8519,
  VCVTTSS2USI64Zrm = 8520,
  VCVTTSS2USI64Zrm_Int = 8521,
  VCVTTSS2USI64Zrr = 8522,
  VCVTTSS2USI64Zrr_Int = 8523,
  VCVTTSS2USI64Zrrb_Int = 8524,
  VCVTTSS2USISrm = 8525,
  VCVTTSS2USISrm_Int = 8526,
  VCVTTSS2USISrr = 8527,
  VCVTTSS2USISrr_Int = 8528,
  VCVTTSS2USISrrb_Int = 8529,
  VCVTTSS2USIZrm = 8530,
  VCVTTSS2USIZrm_Int = 8531,
  VCVTTSS2USIZrr = 8532,
  VCVTTSS2USIZrr_Int = 8533,
  VCVTTSS2USIZrrb_Int = 8534,
  VCVTUDQ2PDZ128rm = 8535,
  VCVTUDQ2PDZ128rmb = 8536,
  VCVTUDQ2PDZ128rmbk = 8537,
  VCVTUDQ2PDZ128rmbkz = 8538,
  VCVTUDQ2PDZ128rmk = 8539,
  VCVTUDQ2PDZ128rmkz = 8540,
  VCVTUDQ2PDZ128rr = 8541,
  VCVTUDQ2PDZ128rrk = 8542,
  VCVTUDQ2PDZ128rrkz = 8543,
  VCVTUDQ2PDZ256rm = 8544,
  VCVTUDQ2PDZ256rmb = 8545,
  VCVTUDQ2PDZ256rmbk = 8546,
  VCVTUDQ2PDZ256rmbkz = 8547,
  VCVTUDQ2PDZ256rmk = 8548,
  VCVTUDQ2PDZ256rmkz = 8549,
  VCVTUDQ2PDZ256rr = 8550,
  VCVTUDQ2PDZ256rrk = 8551,
  VCVTUDQ2PDZ256rrkz = 8552,
  VCVTUDQ2PDZrm = 8553,
  VCVTUDQ2PDZrmb = 8554,
  VCVTUDQ2PDZrmbk = 8555,
  VCVTUDQ2PDZrmbkz = 8556,
  VCVTUDQ2PDZrmk = 8557,
  VCVTUDQ2PDZrmkz = 8558,
  VCVTUDQ2PDZrr = 8559,
  VCVTUDQ2PDZrrk = 8560,
  VCVTUDQ2PDZrrkz = 8561,
  VCVTUDQ2PHZ128rm = 8562,
  VCVTUDQ2PHZ128rmb = 8563,
  VCVTUDQ2PHZ128rmbk = 8564,
  VCVTUDQ2PHZ128rmbkz = 8565,
  VCVTUDQ2PHZ128rmk = 8566,
  VCVTUDQ2PHZ128rmkz = 8567,
  VCVTUDQ2PHZ128rr = 8568,
  VCVTUDQ2PHZ128rrk = 8569,
  VCVTUDQ2PHZ128rrkz = 8570,
  VCVTUDQ2PHZ256rm = 8571,
  VCVTUDQ2PHZ256rmb = 8572,
  VCVTUDQ2PHZ256rmbk = 8573,
  VCVTUDQ2PHZ256rmbkz = 8574,
  VCVTUDQ2PHZ256rmk = 8575,
  VCVTUDQ2PHZ256rmkz = 8576,
  VCVTUDQ2PHZ256rr = 8577,
  VCVTUDQ2PHZ256rrk = 8578,
  VCVTUDQ2PHZ256rrkz = 8579,
  VCVTUDQ2PHZrm = 8580,
  VCVTUDQ2PHZrmb = 8581,
  VCVTUDQ2PHZrmbk = 8582,
  VCVTUDQ2PHZrmbkz = 8583,
  VCVTUDQ2PHZrmk = 8584,
  VCVTUDQ2PHZrmkz = 8585,
  VCVTUDQ2PHZrr = 8586,
  VCVTUDQ2PHZrrb = 8587,
  VCVTUDQ2PHZrrbk = 8588,
  VCVTUDQ2PHZrrbkz = 8589,
  VCVTUDQ2PHZrrk = 8590,
  VCVTUDQ2PHZrrkz = 8591,
  VCVTUDQ2PSZ128rm = 8592,
  VCVTUDQ2PSZ128rmb = 8593,
  VCVTUDQ2PSZ128rmbk = 8594,
  VCVTUDQ2PSZ128rmbkz = 8595,
  VCVTUDQ2PSZ128rmk = 8596,
  VCVTUDQ2PSZ128rmkz = 8597,
  VCVTUDQ2PSZ128rr = 8598,
  VCVTUDQ2PSZ128rrk = 8599,
  VCVTUDQ2PSZ128rrkz = 8600,
  VCVTUDQ2PSZ256rm = 8601,
  VCVTUDQ2PSZ256rmb = 8602,
  VCVTUDQ2PSZ256rmbk = 8603,
  VCVTUDQ2PSZ256rmbkz = 8604,
  VCVTUDQ2PSZ256rmk = 8605,
  VCVTUDQ2PSZ256rmkz = 8606,
  VCVTUDQ2PSZ256rr = 8607,
  VCVTUDQ2PSZ256rrk = 8608,
  VCVTUDQ2PSZ256rrkz = 8609,
  VCVTUDQ2PSZrm = 8610,
  VCVTUDQ2PSZrmb = 8611,
  VCVTUDQ2PSZrmbk = 8612,
  VCVTUDQ2PSZrmbkz = 8613,
  VCVTUDQ2PSZrmk = 8614,
  VCVTUDQ2PSZrmkz = 8615,
  VCVTUDQ2PSZrr = 8616,
  VCVTUDQ2PSZrrb = 8617,
  VCVTUDQ2PSZrrbk = 8618,
  VCVTUDQ2PSZrrbkz = 8619,
  VCVTUDQ2PSZrrk = 8620,
  VCVTUDQ2PSZrrkz = 8621,
  VCVTUQQ2PDZ128rm = 8622,
  VCVTUQQ2PDZ128rmb = 8623,
  VCVTUQQ2PDZ128rmbk = 8624,
  VCVTUQQ2PDZ128rmbkz = 8625,
  VCVTUQQ2PDZ128rmk = 8626,
  VCVTUQQ2PDZ128rmkz = 8627,
  VCVTUQQ2PDZ128rr = 8628,
  VCVTUQQ2PDZ128rrk = 8629,
  VCVTUQQ2PDZ128rrkz = 8630,
  VCVTUQQ2PDZ256rm = 8631,
  VCVTUQQ2PDZ256rmb = 8632,
  VCVTUQQ2PDZ256rmbk = 8633,
  VCVTUQQ2PDZ256rmbkz = 8634,
  VCVTUQQ2PDZ256rmk = 8635,
  VCVTUQQ2PDZ256rmkz = 8636,
  VCVTUQQ2PDZ256rr = 8637,
  VCVTUQQ2PDZ256rrk = 8638,
  VCVTUQQ2PDZ256rrkz = 8639,
  VCVTUQQ2PDZrm = 8640,
  VCVTUQQ2PDZrmb = 8641,
  VCVTUQQ2PDZrmbk = 8642,
  VCVTUQQ2PDZrmbkz = 8643,
  VCVTUQQ2PDZrmk = 8644,
  VCVTUQQ2PDZrmkz = 8645,
  VCVTUQQ2PDZrr = 8646,
  VCVTUQQ2PDZrrb = 8647,
  VCVTUQQ2PDZrrbk = 8648,
  VCVTUQQ2PDZrrbkz = 8649,
  VCVTUQQ2PDZrrk = 8650,
  VCVTUQQ2PDZrrkz = 8651,
  VCVTUQQ2PHZ128rm = 8652,
  VCVTUQQ2PHZ128rmb = 8653,
  VCVTUQQ2PHZ128rmbk = 8654,
  VCVTUQQ2PHZ128rmbkz = 8655,
  VCVTUQQ2PHZ128rmk = 8656,
  VCVTUQQ2PHZ128rmkz = 8657,
  VCVTUQQ2PHZ128rr = 8658,
  VCVTUQQ2PHZ128rrk = 8659,
  VCVTUQQ2PHZ128rrkz = 8660,
  VCVTUQQ2PHZ256rm = 8661,
  VCVTUQQ2PHZ256rmb = 8662,
  VCVTUQQ2PHZ256rmbk = 8663,
  VCVTUQQ2PHZ256rmbkz = 8664,
  VCVTUQQ2PHZ256rmk = 8665,
  VCVTUQQ2PHZ256rmkz = 8666,
  VCVTUQQ2PHZ256rr = 8667,
  VCVTUQQ2PHZ256rrk = 8668,
  VCVTUQQ2PHZ256rrkz = 8669,
  VCVTUQQ2PHZrm = 8670,
  VCVTUQQ2PHZrmb = 8671,
  VCVTUQQ2PHZrmbk = 8672,
  VCVTUQQ2PHZrmbkz = 8673,
  VCVTUQQ2PHZrmk = 8674,
  VCVTUQQ2PHZrmkz = 8675,
  VCVTUQQ2PHZrr = 8676,
  VCVTUQQ2PHZrrb = 8677,
  VCVTUQQ2PHZrrbk = 8678,
  VCVTUQQ2PHZrrbkz = 8679,
  VCVTUQQ2PHZrrk = 8680,
  VCVTUQQ2PHZrrkz = 8681,
  VCVTUQQ2PSZ128rm = 8682,
  VCVTUQQ2PSZ128rmb = 8683,
  VCVTUQQ2PSZ128rmbk = 8684,
  VCVTUQQ2PSZ128rmbkz = 8685,
  VCVTUQQ2PSZ128rmk = 8686,
  VCVTUQQ2PSZ128rmkz = 8687,
  VCVTUQQ2PSZ128rr = 8688,
  VCVTUQQ2PSZ128rrk = 8689,
  VCVTUQQ2PSZ128rrkz = 8690,
  VCVTUQQ2PSZ256rm = 8691,
  VCVTUQQ2PSZ256rmb = 8692,
  VCVTUQQ2PSZ256rmbk = 8693,
  VCVTUQQ2PSZ256rmbkz = 8694,
  VCVTUQQ2PSZ256rmk = 8695,
  VCVTUQQ2PSZ256rmkz = 8696,
  VCVTUQQ2PSZ256rr = 8697,
  VCVTUQQ2PSZ256rrk = 8698,
  VCVTUQQ2PSZ256rrkz = 8699,
  VCVTUQQ2PSZrm = 8700,
  VCVTUQQ2PSZrmb = 8701,
  VCVTUQQ2PSZrmbk = 8702,
  VCVTUQQ2PSZrmbkz = 8703,
  VCVTUQQ2PSZrmk = 8704,
  VCVTUQQ2PSZrmkz = 8705,
  VCVTUQQ2PSZrr = 8706,
  VCVTUQQ2PSZrrb = 8707,
  VCVTUQQ2PSZrrbk = 8708,
  VCVTUQQ2PSZrrbkz = 8709,
  VCVTUQQ2PSZrrk = 8710,
  VCVTUQQ2PSZrrkz = 8711,
  VCVTUSI2SDZrm = 8712,
  VCVTUSI2SDZrm_Int = 8713,
  VCVTUSI2SDZrr = 8714,
  VCVTUSI2SDZrr_Int = 8715,
  VCVTUSI2SHZrm = 8716,
  VCVTUSI2SHZrm_Int = 8717,
  VCVTUSI2SHZrr = 8718,
  VCVTUSI2SHZrr_Int = 8719,
  VCVTUSI2SHZrrb_Int = 8720,
  VCVTUSI2SSZrm = 8721,
  VCVTUSI2SSZrm_Int = 8722,
  VCVTUSI2SSZrr = 8723,
  VCVTUSI2SSZrr_Int = 8724,
  VCVTUSI2SSZrrb_Int = 8725,
  VCVTUSI642SDZrm = 8726,
  VCVTUSI642SDZrm_Int = 8727,
  VCVTUSI642SDZrr = 8728,
  VCVTUSI642SDZrr_Int = 8729,
  VCVTUSI642SDZrrb_Int = 8730,
  VCVTUSI642SHZrm = 8731,
  VCVTUSI642SHZrm_Int = 8732,
  VCVTUSI642SHZrr = 8733,
  VCVTUSI642SHZrr_Int = 8734,
  VCVTUSI642SHZrrb_Int = 8735,
  VCVTUSI642SSZrm = 8736,
  VCVTUSI642SSZrm_Int = 8737,
  VCVTUSI642SSZrr = 8738,
  VCVTUSI642SSZrr_Int = 8739,
  VCVTUSI642SSZrrb_Int = 8740,
  VCVTUW2PHZ128rm = 8741,
  VCVTUW2PHZ128rmb = 8742,
  VCVTUW2PHZ128rmbk = 8743,
  VCVTUW2PHZ128rmbkz = 8744,
  VCVTUW2PHZ128rmk = 8745,
  VCVTUW2PHZ128rmkz = 8746,
  VCVTUW2PHZ128rr = 8747,
  VCVTUW2PHZ128rrk = 8748,
  VCVTUW2PHZ128rrkz = 8749,
  VCVTUW2PHZ256rm = 8750,
  VCVTUW2PHZ256rmb = 8751,
  VCVTUW2PHZ256rmbk = 8752,
  VCVTUW2PHZ256rmbkz = 8753,
  VCVTUW2PHZ256rmk = 8754,
  VCVTUW2PHZ256rmkz = 8755,
  VCVTUW2PHZ256rr = 8756,
  VCVTUW2PHZ256rrk = 8757,
  VCVTUW2PHZ256rrkz = 8758,
  VCVTUW2PHZrm = 8759,
  VCVTUW2PHZrmb = 8760,
  VCVTUW2PHZrmbk = 8761,
  VCVTUW2PHZrmbkz = 8762,
  VCVTUW2PHZrmk = 8763,
  VCVTUW2PHZrmkz = 8764,
  VCVTUW2PHZrr = 8765,
  VCVTUW2PHZrrb = 8766,
  VCVTUW2PHZrrbk = 8767,
  VCVTUW2PHZrrbkz = 8768,
  VCVTUW2PHZrrk = 8769,
  VCVTUW2PHZrrkz = 8770,
  VCVTW2PHZ128rm = 8771,
  VCVTW2PHZ128rmb = 8772,
  VCVTW2PHZ128rmbk = 8773,
  VCVTW2PHZ128rmbkz = 8774,
  VCVTW2PHZ128rmk = 8775,
  VCVTW2PHZ128rmkz = 8776,
  VCVTW2PHZ128rr = 8777,
  VCVTW2PHZ128rrk = 8778,
  VCVTW2PHZ128rrkz = 8779,
  VCVTW2PHZ256rm = 8780,
  VCVTW2PHZ256rmb = 8781,
  VCVTW2PHZ256rmbk = 8782,
  VCVTW2PHZ256rmbkz = 8783,
  VCVTW2PHZ256rmk = 8784,
  VCVTW2PHZ256rmkz = 8785,
  VCVTW2PHZ256rr = 8786,
  VCVTW2PHZ256rrk = 8787,
  VCVTW2PHZ256rrkz = 8788,
  VCVTW2PHZrm = 8789,
  VCVTW2PHZrmb = 8790,
  VCVTW2PHZrmbk = 8791,
  VCVTW2PHZrmbkz = 8792,
  VCVTW2PHZrmk = 8793,
  VCVTW2PHZrmkz = 8794,
  VCVTW2PHZrr = 8795,
  VCVTW2PHZrrb = 8796,
  VCVTW2PHZrrbk = 8797,
  VCVTW2PHZrrbkz = 8798,
  VCVTW2PHZrrk = 8799,
  VCVTW2PHZrrkz = 8800,
  VDBPSADBWZ128rmi = 8801,
  VDBPSADBWZ128rmik = 8802,
  VDBPSADBWZ128rmikz = 8803,
  VDBPSADBWZ128rri = 8804,
  VDBPSADBWZ128rrik = 8805,
  VDBPSADBWZ128rrikz = 8806,
  VDBPSADBWZ256rmi = 8807,
  VDBPSADBWZ256rmik = 8808,
  VDBPSADBWZ256rmikz = 8809,
  VDBPSADBWZ256rri = 8810,
  VDBPSADBWZ256rrik = 8811,
  VDBPSADBWZ256rrikz = 8812,
  VDBPSADBWZrmi = 8813,
  VDBPSADBWZrmik = 8814,
  VDBPSADBWZrmikz = 8815,
  VDBPSADBWZrri = 8816,
  VDBPSADBWZrrik = 8817,
  VDBPSADBWZrrikz = 8818,
  VDIVBF16Z128rm = 8819,
  VDIVBF16Z128rmb = 8820,
  VDIVBF16Z128rmbk = 8821,
  VDIVBF16Z128rmbkz = 8822,
  VDIVBF16Z128rmk = 8823,
  VDIVBF16Z128rmkz = 8824,
  VDIVBF16Z128rr = 8825,
  VDIVBF16Z128rrk = 8826,
  VDIVBF16Z128rrkz = 8827,
  VDIVBF16Z256rm = 8828,
  VDIVBF16Z256rmb = 8829,
  VDIVBF16Z256rmbk = 8830,
  VDIVBF16Z256rmbkz = 8831,
  VDIVBF16Z256rmk = 8832,
  VDIVBF16Z256rmkz = 8833,
  VDIVBF16Z256rr = 8834,
  VDIVBF16Z256rrk = 8835,
  VDIVBF16Z256rrkz = 8836,
  VDIVBF16Zrm = 8837,
  VDIVBF16Zrmb = 8838,
  VDIVBF16Zrmbk = 8839,
  VDIVBF16Zrmbkz = 8840,
  VDIVBF16Zrmk = 8841,
  VDIVBF16Zrmkz = 8842,
  VDIVBF16Zrr = 8843,
  VDIVBF16Zrrk = 8844,
  VDIVBF16Zrrkz = 8845,
  VDIVPDYrm = 8846,
  VDIVPDYrr = 8847,
  VDIVPDZ128rm = 8848,
  VDIVPDZ128rmb = 8849,
  VDIVPDZ128rmbk = 8850,
  VDIVPDZ128rmbkz = 8851,
  VDIVPDZ128rmk = 8852,
  VDIVPDZ128rmkz = 8853,
  VDIVPDZ128rr = 8854,
  VDIVPDZ128rrk = 8855,
  VDIVPDZ128rrkz = 8856,
  VDIVPDZ256rm = 8857,
  VDIVPDZ256rmb = 8858,
  VDIVPDZ256rmbk = 8859,
  VDIVPDZ256rmbkz = 8860,
  VDIVPDZ256rmk = 8861,
  VDIVPDZ256rmkz = 8862,
  VDIVPDZ256rr = 8863,
  VDIVPDZ256rrk = 8864,
  VDIVPDZ256rrkz = 8865,
  VDIVPDZrm = 8866,
  VDIVPDZrmb = 8867,
  VDIVPDZrmbk = 8868,
  VDIVPDZrmbkz = 8869,
  VDIVPDZrmk = 8870,
  VDIVPDZrmkz = 8871,
  VDIVPDZrr = 8872,
  VDIVPDZrrb = 8873,
  VDIVPDZrrbk = 8874,
  VDIVPDZrrbkz = 8875,
  VDIVPDZrrk = 8876,
  VDIVPDZrrkz = 8877,
  VDIVPDrm = 8878,
  VDIVPDrr = 8879,
  VDIVPHZ128rm = 8880,
  VDIVPHZ128rmb = 8881,
  VDIVPHZ128rmbk = 8882,
  VDIVPHZ128rmbkz = 8883,
  VDIVPHZ128rmk = 8884,
  VDIVPHZ128rmkz = 8885,
  VDIVPHZ128rr = 8886,
  VDIVPHZ128rrk = 8887,
  VDIVPHZ128rrkz = 8888,
  VDIVPHZ256rm = 8889,
  VDIVPHZ256rmb = 8890,
  VDIVPHZ256rmbk = 8891,
  VDIVPHZ256rmbkz = 8892,
  VDIVPHZ256rmk = 8893,
  VDIVPHZ256rmkz = 8894,
  VDIVPHZ256rr = 8895,
  VDIVPHZ256rrk = 8896,
  VDIVPHZ256rrkz = 8897,
  VDIVPHZrm = 8898,
  VDIVPHZrmb = 8899,
  VDIVPHZrmbk = 8900,
  VDIVPHZrmbkz = 8901,
  VDIVPHZrmk = 8902,
  VDIVPHZrmkz = 8903,
  VDIVPHZrr = 8904,
  VDIVPHZrrb = 8905,
  VDIVPHZrrbk = 8906,
  VDIVPHZrrbkz = 8907,
  VDIVPHZrrk = 8908,
  VDIVPHZrrkz = 8909,
  VDIVPSYrm = 8910,
  VDIVPSYrr = 8911,
  VDIVPSZ128rm = 8912,
  VDIVPSZ128rmb = 8913,
  VDIVPSZ128rmbk = 8914,
  VDIVPSZ128rmbkz = 8915,
  VDIVPSZ128rmk = 8916,
  VDIVPSZ128rmkz = 8917,
  VDIVPSZ128rr = 8918,
  VDIVPSZ128rrk = 8919,
  VDIVPSZ128rrkz = 8920,
  VDIVPSZ256rm = 8921,
  VDIVPSZ256rmb = 8922,
  VDIVPSZ256rmbk = 8923,
  VDIVPSZ256rmbkz = 8924,
  VDIVPSZ256rmk = 8925,
  VDIVPSZ256rmkz = 8926,
  VDIVPSZ256rr = 8927,
  VDIVPSZ256rrk = 8928,
  VDIVPSZ256rrkz = 8929,
  VDIVPSZrm = 8930,
  VDIVPSZrmb = 8931,
  VDIVPSZrmbk = 8932,
  VDIVPSZrmbkz = 8933,
  VDIVPSZrmk = 8934,
  VDIVPSZrmkz = 8935,
  VDIVPSZrr = 8936,
  VDIVPSZrrb = 8937,
  VDIVPSZrrbk = 8938,
  VDIVPSZrrbkz = 8939,
  VDIVPSZrrk = 8940,
  VDIVPSZrrkz = 8941,
  VDIVPSrm = 8942,
  VDIVPSrr = 8943,
  VDIVSDZrm = 8944,
  VDIVSDZrm_Int = 8945,
  VDIVSDZrmk_Int = 8946,
  VDIVSDZrmkz_Int = 8947,
  VDIVSDZrr = 8948,
  VDIVSDZrr_Int = 8949,
  VDIVSDZrrb_Int = 8950,
  VDIVSDZrrbk_Int = 8951,
  VDIVSDZrrbkz_Int = 8952,
  VDIVSDZrrk_Int = 8953,
  VDIVSDZrrkz_Int = 8954,
  VDIVSDrm = 8955,
  VDIVSDrm_Int = 8956,
  VDIVSDrr = 8957,
  VDIVSDrr_Int = 8958,
  VDIVSHZrm = 8959,
  VDIVSHZrm_Int = 8960,
  VDIVSHZrmk_Int = 8961,
  VDIVSHZrmkz_Int = 8962,
  VDIVSHZrr = 8963,
  VDIVSHZrr_Int = 8964,
  VDIVSHZrrb_Int = 8965,
  VDIVSHZrrbk_Int = 8966,
  VDIVSHZrrbkz_Int = 8967,
  VDIVSHZrrk_Int = 8968,
  VDIVSHZrrkz_Int = 8969,
  VDIVSSZrm = 8970,
  VDIVSSZrm_Int = 8971,
  VDIVSSZrmk_Int = 8972,
  VDIVSSZrmkz_Int = 8973,
  VDIVSSZrr = 8974,
  VDIVSSZrr_Int = 8975,
  VDIVSSZrrb_Int = 8976,
  VDIVSSZrrbk_Int = 8977,
  VDIVSSZrrbkz_Int = 8978,
  VDIVSSZrrk_Int = 8979,
  VDIVSSZrrkz_Int = 8980,
  VDIVSSrm = 8981,
  VDIVSSrm_Int = 8982,
  VDIVSSrr = 8983,
  VDIVSSrr_Int = 8984,
  VDPBF16PSZ128m = 8985,
  VDPBF16PSZ128mb = 8986,
  VDPBF16PSZ128mbk = 8987,
  VDPBF16PSZ128mbkz = 8988,
  VDPBF16PSZ128mk = 8989,
  VDPBF16PSZ128mkz = 8990,
  VDPBF16PSZ128r = 8991,
  VDPBF16PSZ128rk = 8992,
  VDPBF16PSZ128rkz = 8993,
  VDPBF16PSZ256m = 8994,
  VDPBF16PSZ256mb = 8995,
  VDPBF16PSZ256mbk = 8996,
  VDPBF16PSZ256mbkz = 8997,
  VDPBF16PSZ256mk = 8998,
  VDPBF16PSZ256mkz = 8999,
  VDPBF16PSZ256r = 9000,
  VDPBF16PSZ256rk = 9001,
  VDPBF16PSZ256rkz = 9002,
  VDPBF16PSZm = 9003,
  VDPBF16PSZmb = 9004,
  VDPBF16PSZmbk = 9005,
  VDPBF16PSZmbkz = 9006,
  VDPBF16PSZmk = 9007,
  VDPBF16PSZmkz = 9008,
  VDPBF16PSZr = 9009,
  VDPBF16PSZrk = 9010,
  VDPBF16PSZrkz = 9011,
  VDPPDrmi = 9012,
  VDPPDrri = 9013,
  VDPPHPSZ128m = 9014,
  VDPPHPSZ128mb = 9015,
  VDPPHPSZ128mbk = 9016,
  VDPPHPSZ128mbkz = 9017,
  VDPPHPSZ128mk = 9018,
  VDPPHPSZ128mkz = 9019,
  VDPPHPSZ128r = 9020,
  VDPPHPSZ128rk = 9021,
  VDPPHPSZ128rkz = 9022,
  VDPPHPSZ256m = 9023,
  VDPPHPSZ256mb = 9024,
  VDPPHPSZ256mbk = 9025,
  VDPPHPSZ256mbkz = 9026,
  VDPPHPSZ256mk = 9027,
  VDPPHPSZ256mkz = 9028,
  VDPPHPSZ256r = 9029,
  VDPPHPSZ256rk = 9030,
  VDPPHPSZ256rkz = 9031,
  VDPPHPSZm = 9032,
  VDPPHPSZmb = 9033,
  VDPPHPSZmbk = 9034,
  VDPPHPSZmbkz = 9035,
  VDPPHPSZmk = 9036,
  VDPPHPSZmkz = 9037,
  VDPPHPSZr = 9038,
  VDPPHPSZrk = 9039,
  VDPPHPSZrkz = 9040,
  VDPPSYrmi = 9041,
  VDPPSYrri = 9042,
  VDPPSrmi = 9043,
  VDPPSrri = 9044,
  VERRm = 9045,
  VERRr = 9046,
  VERWm = 9047,
  VERWr = 9048,
  VEXP2PDZm = 9049,
  VEXP2PDZmb = 9050,
  VEXP2PDZmbk = 9051,
  VEXP2PDZmbkz = 9052,
  VEXP2PDZmk = 9053,
  VEXP2PDZmkz = 9054,
  VEXP2PDZr = 9055,
  VEXP2PDZrb = 9056,
  VEXP2PDZrbk = 9057,
  VEXP2PDZrbkz = 9058,
  VEXP2PDZrk = 9059,
  VEXP2PDZrkz = 9060,
  VEXP2PSZm = 9061,
  VEXP2PSZmb = 9062,
  VEXP2PSZmbk = 9063,
  VEXP2PSZmbkz = 9064,
  VEXP2PSZmk = 9065,
  VEXP2PSZmkz = 9066,
  VEXP2PSZr = 9067,
  VEXP2PSZrb = 9068,
  VEXP2PSZrbk = 9069,
  VEXP2PSZrbkz = 9070,
  VEXP2PSZrk = 9071,
  VEXP2PSZrkz = 9072,
  VEXPANDPDZ128rm = 9073,
  VEXPANDPDZ128rmk = 9074,
  VEXPANDPDZ128rmkz = 9075,
  VEXPANDPDZ128rr = 9076,
  VEXPANDPDZ128rrk = 9077,
  VEXPANDPDZ128rrkz = 9078,
  VEXPANDPDZ256rm = 9079,
  VEXPANDPDZ256rmk = 9080,
  VEXPANDPDZ256rmkz = 9081,
  VEXPANDPDZ256rr = 9082,
  VEXPANDPDZ256rrk = 9083,
  VEXPANDPDZ256rrkz = 9084,
  VEXPANDPDZrm = 9085,
  VEXPANDPDZrmk = 9086,
  VEXPANDPDZrmkz = 9087,
  VEXPANDPDZrr = 9088,
  VEXPANDPDZrrk = 9089,
  VEXPANDPDZrrkz = 9090,
  VEXPANDPSZ128rm = 9091,
  VEXPANDPSZ128rmk = 9092,
  VEXPANDPSZ128rmkz = 9093,
  VEXPANDPSZ128rr = 9094,
  VEXPANDPSZ128rrk = 9095,
  VEXPANDPSZ128rrkz = 9096,
  VEXPANDPSZ256rm = 9097,
  VEXPANDPSZ256rmk = 9098,
  VEXPANDPSZ256rmkz = 9099,
  VEXPANDPSZ256rr = 9100,
  VEXPANDPSZ256rrk = 9101,
  VEXPANDPSZ256rrkz = 9102,
  VEXPANDPSZrm = 9103,
  VEXPANDPSZrmk = 9104,
  VEXPANDPSZrmkz = 9105,
  VEXPANDPSZrr = 9106,
  VEXPANDPSZrrk = 9107,
  VEXPANDPSZrrkz = 9108,
  VEXTRACTF128mri = 9109,
  VEXTRACTF128rri = 9110,
  VEXTRACTF32X4Z256mri = 9111,
  VEXTRACTF32X4Z256mrik = 9112,
  VEXTRACTF32X4Z256rri = 9113,
  VEXTRACTF32X4Z256rrik = 9114,
  VEXTRACTF32X4Z256rrikz = 9115,
  VEXTRACTF32X4Zmri = 9116,
  VEXTRACTF32X4Zmrik = 9117,
  VEXTRACTF32X4Zrri = 9118,
  VEXTRACTF32X4Zrrik = 9119,
  VEXTRACTF32X4Zrrikz = 9120,
  VEXTRACTF32X8Zmri = 9121,
  VEXTRACTF32X8Zmrik = 9122,
  VEXTRACTF32X8Zrri = 9123,
  VEXTRACTF32X8Zrrik = 9124,
  VEXTRACTF32X8Zrrikz = 9125,
  VEXTRACTF64X2Z256mri = 9126,
  VEXTRACTF64X2Z256mrik = 9127,
  VEXTRACTF64X2Z256rri = 9128,
  VEXTRACTF64X2Z256rrik = 9129,
  VEXTRACTF64X2Z256rrikz = 9130,
  VEXTRACTF64X2Zmri = 9131,
  VEXTRACTF64X2Zmrik = 9132,
  VEXTRACTF64X2Zrri = 9133,
  VEXTRACTF64X2Zrrik = 9134,
  VEXTRACTF64X2Zrrikz = 9135,
  VEXTRACTF64X4Zmri = 9136,
  VEXTRACTF64X4Zmrik = 9137,
  VEXTRACTF64X4Zrri = 9138,
  VEXTRACTF64X4Zrrik = 9139,
  VEXTRACTF64X4Zrrikz = 9140,
  VEXTRACTI128mri = 9141,
  VEXTRACTI128rri = 9142,
  VEXTRACTI32X4Z256mri = 9143,
  VEXTRACTI32X4Z256mrik = 9144,
  VEXTRACTI32X4Z256rri = 9145,
  VEXTRACTI32X4Z256rrik = 9146,
  VEXTRACTI32X4Z256rrikz = 9147,
  VEXTRACTI32X4Zmri = 9148,
  VEXTRACTI32X4Zmrik = 9149,
  VEXTRACTI32X4Zrri = 9150,
  VEXTRACTI32X4Zrrik = 9151,
  VEXTRACTI32X4Zrrikz = 9152,
  VEXTRACTI32X8Zmri = 9153,
  VEXTRACTI32X8Zmrik = 9154,
  VEXTRACTI32X8Zrri = 9155,
  VEXTRACTI32X8Zrrik = 9156,
  VEXTRACTI32X8Zrrikz = 9157,
  VEXTRACTI64X2Z256mri = 9158,
  VEXTRACTI64X2Z256mrik = 9159,
  VEXTRACTI64X2Z256rri = 9160,
  VEXTRACTI64X2Z256rrik = 9161,
  VEXTRACTI64X2Z256rrikz = 9162,
  VEXTRACTI64X2Zmri = 9163,
  VEXTRACTI64X2Zmrik = 9164,
  VEXTRACTI64X2Zrri = 9165,
  VEXTRACTI64X2Zrrik = 9166,
  VEXTRACTI64X2Zrrikz = 9167,
  VEXTRACTI64X4Zmri = 9168,
  VEXTRACTI64X4Zmrik = 9169,
  VEXTRACTI64X4Zrri = 9170,
  VEXTRACTI64X4Zrrik = 9171,
  VEXTRACTI64X4Zrrikz = 9172,
  VEXTRACTPSZmri = 9173,
  VEXTRACTPSZrri = 9174,
  VEXTRACTPSmri = 9175,
  VEXTRACTPSrri = 9176,
  VFCMADDCPHZ128m = 9177,
  VFCMADDCPHZ128mb = 9178,
  VFCMADDCPHZ128mbk = 9179,
  VFCMADDCPHZ128mbkz = 9180,
  VFCMADDCPHZ128mk = 9181,
  VFCMADDCPHZ128mkz = 9182,
  VFCMADDCPHZ128r = 9183,
  VFCMADDCPHZ128rk = 9184,
  VFCMADDCPHZ128rkz = 9185,
  VFCMADDCPHZ256m = 9186,
  VFCMADDCPHZ256mb = 9187,
  VFCMADDCPHZ256mbk = 9188,
  VFCMADDCPHZ256mbkz = 9189,
  VFCMADDCPHZ256mk = 9190,
  VFCMADDCPHZ256mkz = 9191,
  VFCMADDCPHZ256r = 9192,
  VFCMADDCPHZ256rk = 9193,
  VFCMADDCPHZ256rkz = 9194,
  VFCMADDCPHZm = 9195,
  VFCMADDCPHZmb = 9196,
  VFCMADDCPHZmbk = 9197,
  VFCMADDCPHZmbkz = 9198,
  VFCMADDCPHZmk = 9199,
  VFCMADDCPHZmkz = 9200,
  VFCMADDCPHZr = 9201,
  VFCMADDCPHZrb = 9202,
  VFCMADDCPHZrbk = 9203,
  VFCMADDCPHZrbkz = 9204,
  VFCMADDCPHZrk = 9205,
  VFCMADDCPHZrkz = 9206,
  VFCMADDCSHZm = 9207,
  VFCMADDCSHZmk = 9208,
  VFCMADDCSHZmkz = 9209,
  VFCMADDCSHZr = 9210,
  VFCMADDCSHZrb = 9211,
  VFCMADDCSHZrbk = 9212,
  VFCMADDCSHZrbkz = 9213,
  VFCMADDCSHZrk = 9214,
  VFCMADDCSHZrkz = 9215,
  VFCMULCPHZ128rm = 9216,
  VFCMULCPHZ128rmb = 9217,
  VFCMULCPHZ128rmbk = 9218,
  VFCMULCPHZ128rmbkz = 9219,
  VFCMULCPHZ128rmk = 9220,
  VFCMULCPHZ128rmkz = 9221,
  VFCMULCPHZ128rr = 9222,
  VFCMULCPHZ128rrk = 9223,
  VFCMULCPHZ128rrkz = 9224,
  VFCMULCPHZ256rm = 9225,
  VFCMULCPHZ256rmb = 9226,
  VFCMULCPHZ256rmbk = 9227,
  VFCMULCPHZ256rmbkz = 9228,
  VFCMULCPHZ256rmk = 9229,
  VFCMULCPHZ256rmkz = 9230,
  VFCMULCPHZ256rr = 9231,
  VFCMULCPHZ256rrk = 9232,
  VFCMULCPHZ256rrkz = 9233,
  VFCMULCPHZrm = 9234,
  VFCMULCPHZrmb = 9235,
  VFCMULCPHZrmbk = 9236,
  VFCMULCPHZrmbkz = 9237,
  VFCMULCPHZrmk = 9238,
  VFCMULCPHZrmkz = 9239,
  VFCMULCPHZrr = 9240,
  VFCMULCPHZrrb = 9241,
  VFCMULCPHZrrbk = 9242,
  VFCMULCPHZrrbkz = 9243,
  VFCMULCPHZrrk = 9244,
  VFCMULCPHZrrkz = 9245,
  VFCMULCSHZrm = 9246,
  VFCMULCSHZrmk = 9247,
  VFCMULCSHZrmkz = 9248,
  VFCMULCSHZrr = 9249,
  VFCMULCSHZrrb = 9250,
  VFCMULCSHZrrbk = 9251,
  VFCMULCSHZrrbkz = 9252,
  VFCMULCSHZrrk = 9253,
  VFCMULCSHZrrkz = 9254,
  VFIXUPIMMPDZ128rmbi = 9255,
  VFIXUPIMMPDZ128rmbik = 9256,
  VFIXUPIMMPDZ128rmbikz = 9257,
  VFIXUPIMMPDZ128rmi = 9258,
  VFIXUPIMMPDZ128rmik = 9259,
  VFIXUPIMMPDZ128rmikz = 9260,
  VFIXUPIMMPDZ128rri = 9261,
  VFIXUPIMMPDZ128rrik = 9262,
  VFIXUPIMMPDZ128rrikz = 9263,
  VFIXUPIMMPDZ256rmbi = 9264,
  VFIXUPIMMPDZ256rmbik = 9265,
  VFIXUPIMMPDZ256rmbikz = 9266,
  VFIXUPIMMPDZ256rmi = 9267,
  VFIXUPIMMPDZ256rmik = 9268,
  VFIXUPIMMPDZ256rmikz = 9269,
  VFIXUPIMMPDZ256rri = 9270,
  VFIXUPIMMPDZ256rrik = 9271,
  VFIXUPIMMPDZ256rrikz = 9272,
  VFIXUPIMMPDZrmbi = 9273,
  VFIXUPIMMPDZrmbik = 9274,
  VFIXUPIMMPDZrmbikz = 9275,
  VFIXUPIMMPDZrmi = 9276,
  VFIXUPIMMPDZrmik = 9277,
  VFIXUPIMMPDZrmikz = 9278,
  VFIXUPIMMPDZrri = 9279,
  VFIXUPIMMPDZrrib = 9280,
  VFIXUPIMMPDZrribk = 9281,
  VFIXUPIMMPDZrribkz = 9282,
  VFIXUPIMMPDZrrik = 9283,
  VFIXUPIMMPDZrrikz = 9284,
  VFIXUPIMMPSZ128rmbi = 9285,
  VFIXUPIMMPSZ128rmbik = 9286,
  VFIXUPIMMPSZ128rmbikz = 9287,
  VFIXUPIMMPSZ128rmi = 9288,
  VFIXUPIMMPSZ128rmik = 9289,
  VFIXUPIMMPSZ128rmikz = 9290,
  VFIXUPIMMPSZ128rri = 9291,
  VFIXUPIMMPSZ128rrik = 9292,
  VFIXUPIMMPSZ128rrikz = 9293,
  VFIXUPIMMPSZ256rmbi = 9294,
  VFIXUPIMMPSZ256rmbik = 9295,
  VFIXUPIMMPSZ256rmbikz = 9296,
  VFIXUPIMMPSZ256rmi = 9297,
  VFIXUPIMMPSZ256rmik = 9298,
  VFIXUPIMMPSZ256rmikz = 9299,
  VFIXUPIMMPSZ256rri = 9300,
  VFIXUPIMMPSZ256rrik = 9301,
  VFIXUPIMMPSZ256rrikz = 9302,
  VFIXUPIMMPSZrmbi = 9303,
  VFIXUPIMMPSZrmbik = 9304,
  VFIXUPIMMPSZrmbikz = 9305,
  VFIXUPIMMPSZrmi = 9306,
  VFIXUPIMMPSZrmik = 9307,
  VFIXUPIMMPSZrmikz = 9308,
  VFIXUPIMMPSZrri = 9309,
  VFIXUPIMMPSZrrib = 9310,
  VFIXUPIMMPSZrribk = 9311,
  VFIXUPIMMPSZrribkz = 9312,
  VFIXUPIMMPSZrrik = 9313,
  VFIXUPIMMPSZrrikz = 9314,
  VFIXUPIMMSDZrmi = 9315,
  VFIXUPIMMSDZrmik = 9316,
  VFIXUPIMMSDZrmikz = 9317,
  VFIXUPIMMSDZrri = 9318,
  VFIXUPIMMSDZrrib = 9319,
  VFIXUPIMMSDZrribk = 9320,
  VFIXUPIMMSDZrribkz = 9321,
  VFIXUPIMMSDZrrik = 9322,
  VFIXUPIMMSDZrrikz = 9323,
  VFIXUPIMMSSZrmi = 9324,
  VFIXUPIMMSSZrmik = 9325,
  VFIXUPIMMSSZrmikz = 9326,
  VFIXUPIMMSSZrri = 9327,
  VFIXUPIMMSSZrrib = 9328,
  VFIXUPIMMSSZrribk = 9329,
  VFIXUPIMMSSZrribkz = 9330,
  VFIXUPIMMSSZrrik = 9331,
  VFIXUPIMMSSZrrikz = 9332,
  VFMADD132BF16Z128m = 9333,
  VFMADD132BF16Z128mb = 9334,
  VFMADD132BF16Z128mbk = 9335,
  VFMADD132BF16Z128mbkz = 9336,
  VFMADD132BF16Z128mk = 9337,
  VFMADD132BF16Z128mkz = 9338,
  VFMADD132BF16Z128r = 9339,
  VFMADD132BF16Z128rk = 9340,
  VFMADD132BF16Z128rkz = 9341,
  VFMADD132BF16Z256m = 9342,
  VFMADD132BF16Z256mb = 9343,
  VFMADD132BF16Z256mbk = 9344,
  VFMADD132BF16Z256mbkz = 9345,
  VFMADD132BF16Z256mk = 9346,
  VFMADD132BF16Z256mkz = 9347,
  VFMADD132BF16Z256r = 9348,
  VFMADD132BF16Z256rk = 9349,
  VFMADD132BF16Z256rkz = 9350,
  VFMADD132BF16Zm = 9351,
  VFMADD132BF16Zmb = 9352,
  VFMADD132BF16Zmbk = 9353,
  VFMADD132BF16Zmbkz = 9354,
  VFMADD132BF16Zmk = 9355,
  VFMADD132BF16Zmkz = 9356,
  VFMADD132BF16Zr = 9357,
  VFMADD132BF16Zrk = 9358,
  VFMADD132BF16Zrkz = 9359,
  VFMADD132PDYm = 9360,
  VFMADD132PDYr = 9361,
  VFMADD132PDZ128m = 9362,
  VFMADD132PDZ128mb = 9363,
  VFMADD132PDZ128mbk = 9364,
  VFMADD132PDZ128mbkz = 9365,
  VFMADD132PDZ128mk = 9366,
  VFMADD132PDZ128mkz = 9367,
  VFMADD132PDZ128r = 9368,
  VFMADD132PDZ128rk = 9369,
  VFMADD132PDZ128rkz = 9370,
  VFMADD132PDZ256m = 9371,
  VFMADD132PDZ256mb = 9372,
  VFMADD132PDZ256mbk = 9373,
  VFMADD132PDZ256mbkz = 9374,
  VFMADD132PDZ256mk = 9375,
  VFMADD132PDZ256mkz = 9376,
  VFMADD132PDZ256r = 9377,
  VFMADD132PDZ256rk = 9378,
  VFMADD132PDZ256rkz = 9379,
  VFMADD132PDZm = 9380,
  VFMADD132PDZmb = 9381,
  VFMADD132PDZmbk = 9382,
  VFMADD132PDZmbkz = 9383,
  VFMADD132PDZmk = 9384,
  VFMADD132PDZmkz = 9385,
  VFMADD132PDZr = 9386,
  VFMADD132PDZrb = 9387,
  VFMADD132PDZrbk = 9388,
  VFMADD132PDZrbkz = 9389,
  VFMADD132PDZrk = 9390,
  VFMADD132PDZrkz = 9391,
  VFMADD132PDm = 9392,
  VFMADD132PDr = 9393,
  VFMADD132PHZ128m = 9394,
  VFMADD132PHZ128mb = 9395,
  VFMADD132PHZ128mbk = 9396,
  VFMADD132PHZ128mbkz = 9397,
  VFMADD132PHZ128mk = 9398,
  VFMADD132PHZ128mkz = 9399,
  VFMADD132PHZ128r = 9400,
  VFMADD132PHZ128rk = 9401,
  VFMADD132PHZ128rkz = 9402,
  VFMADD132PHZ256m = 9403,
  VFMADD132PHZ256mb = 9404,
  VFMADD132PHZ256mbk = 9405,
  VFMADD132PHZ256mbkz = 9406,
  VFMADD132PHZ256mk = 9407,
  VFMADD132PHZ256mkz = 9408,
  VFMADD132PHZ256r = 9409,
  VFMADD132PHZ256rk = 9410,
  VFMADD132PHZ256rkz = 9411,
  VFMADD132PHZm = 9412,
  VFMADD132PHZmb = 9413,
  VFMADD132PHZmbk = 9414,
  VFMADD132PHZmbkz = 9415,
  VFMADD132PHZmk = 9416,
  VFMADD132PHZmkz = 9417,
  VFMADD132PHZr = 9418,
  VFMADD132PHZrb = 9419,
  VFMADD132PHZrbk = 9420,
  VFMADD132PHZrbkz = 9421,
  VFMADD132PHZrk = 9422,
  VFMADD132PHZrkz = 9423,
  VFMADD132PSYm = 9424,
  VFMADD132PSYr = 9425,
  VFMADD132PSZ128m = 9426,
  VFMADD132PSZ128mb = 9427,
  VFMADD132PSZ128mbk = 9428,
  VFMADD132PSZ128mbkz = 9429,
  VFMADD132PSZ128mk = 9430,
  VFMADD132PSZ128mkz = 9431,
  VFMADD132PSZ128r = 9432,
  VFMADD132PSZ128rk = 9433,
  VFMADD132PSZ128rkz = 9434,
  VFMADD132PSZ256m = 9435,
  VFMADD132PSZ256mb = 9436,
  VFMADD132PSZ256mbk = 9437,
  VFMADD132PSZ256mbkz = 9438,
  VFMADD132PSZ256mk = 9439,
  VFMADD132PSZ256mkz = 9440,
  VFMADD132PSZ256r = 9441,
  VFMADD132PSZ256rk = 9442,
  VFMADD132PSZ256rkz = 9443,
  VFMADD132PSZm = 9444,
  VFMADD132PSZmb = 9445,
  VFMADD132PSZmbk = 9446,
  VFMADD132PSZmbkz = 9447,
  VFMADD132PSZmk = 9448,
  VFMADD132PSZmkz = 9449,
  VFMADD132PSZr = 9450,
  VFMADD132PSZrb = 9451,
  VFMADD132PSZrbk = 9452,
  VFMADD132PSZrbkz = 9453,
  VFMADD132PSZrk = 9454,
  VFMADD132PSZrkz = 9455,
  VFMADD132PSm = 9456,
  VFMADD132PSr = 9457,
  VFMADD132SDZm = 9458,
  VFMADD132SDZm_Int = 9459,
  VFMADD132SDZmk_Int = 9460,
  VFMADD132SDZmkz_Int = 9461,
  VFMADD132SDZr = 9462,
  VFMADD132SDZr_Int = 9463,
  VFMADD132SDZrb = 9464,
  VFMADD132SDZrb_Int = 9465,
  VFMADD132SDZrbk_Int = 9466,
  VFMADD132SDZrbkz_Int = 9467,
  VFMADD132SDZrk_Int = 9468,
  VFMADD132SDZrkz_Int = 9469,
  VFMADD132SDm = 9470,
  VFMADD132SDm_Int = 9471,
  VFMADD132SDr = 9472,
  VFMADD132SDr_Int = 9473,
  VFMADD132SHZm = 9474,
  VFMADD132SHZm_Int = 9475,
  VFMADD132SHZmk_Int = 9476,
  VFMADD132SHZmkz_Int = 9477,
  VFMADD132SHZr = 9478,
  VFMADD132SHZr_Int = 9479,
  VFMADD132SHZrb = 9480,
  VFMADD132SHZrb_Int = 9481,
  VFMADD132SHZrbk_Int = 9482,
  VFMADD132SHZrbkz_Int = 9483,
  VFMADD132SHZrk_Int = 9484,
  VFMADD132SHZrkz_Int = 9485,
  VFMADD132SSZm = 9486,
  VFMADD132SSZm_Int = 9487,
  VFMADD132SSZmk_Int = 9488,
  VFMADD132SSZmkz_Int = 9489,
  VFMADD132SSZr = 9490,
  VFMADD132SSZr_Int = 9491,
  VFMADD132SSZrb = 9492,
  VFMADD132SSZrb_Int = 9493,
  VFMADD132SSZrbk_Int = 9494,
  VFMADD132SSZrbkz_Int = 9495,
  VFMADD132SSZrk_Int = 9496,
  VFMADD132SSZrkz_Int = 9497,
  VFMADD132SSm = 9498,
  VFMADD132SSm_Int = 9499,
  VFMADD132SSr = 9500,
  VFMADD132SSr_Int = 9501,
  VFMADD213BF16Z128m = 9502,
  VFMADD213BF16Z128mb = 9503,
  VFMADD213BF16Z128mbk = 9504,
  VFMADD213BF16Z128mbkz = 9505,
  VFMADD213BF16Z128mk = 9506,
  VFMADD213BF16Z128mkz = 9507,
  VFMADD213BF16Z128r = 9508,
  VFMADD213BF16Z128rk = 9509,
  VFMADD213BF16Z128rkz = 9510,
  VFMADD213BF16Z256m = 9511,
  VFMADD213BF16Z256mb = 9512,
  VFMADD213BF16Z256mbk = 9513,
  VFMADD213BF16Z256mbkz = 9514,
  VFMADD213BF16Z256mk = 9515,
  VFMADD213BF16Z256mkz = 9516,
  VFMADD213BF16Z256r = 9517,
  VFMADD213BF16Z256rk = 9518,
  VFMADD213BF16Z256rkz = 9519,
  VFMADD213BF16Zm = 9520,
  VFMADD213BF16Zmb = 9521,
  VFMADD213BF16Zmbk = 9522,
  VFMADD213BF16Zmbkz = 9523,
  VFMADD213BF16Zmk = 9524,
  VFMADD213BF16Zmkz = 9525,
  VFMADD213BF16Zr = 9526,
  VFMADD213BF16Zrk = 9527,
  VFMADD213BF16Zrkz = 9528,
  VFMADD213PDYm = 9529,
  VFMADD213PDYr = 9530,
  VFMADD213PDZ128m = 9531,
  VFMADD213PDZ128mb = 9532,
  VFMADD213PDZ128mbk = 9533,
  VFMADD213PDZ128mbkz = 9534,
  VFMADD213PDZ128mk = 9535,
  VFMADD213PDZ128mkz = 9536,
  VFMADD213PDZ128r = 9537,
  VFMADD213PDZ128rk = 9538,
  VFMADD213PDZ128rkz = 9539,
  VFMADD213PDZ256m = 9540,
  VFMADD213PDZ256mb = 9541,
  VFMADD213PDZ256mbk = 9542,
  VFMADD213PDZ256mbkz = 9543,
  VFMADD213PDZ256mk = 9544,
  VFMADD213PDZ256mkz = 9545,
  VFMADD213PDZ256r = 9546,
  VFMADD213PDZ256rk = 9547,
  VFMADD213PDZ256rkz = 9548,
  VFMADD213PDZm = 9549,
  VFMADD213PDZmb = 9550,
  VFMADD213PDZmbk = 9551,
  VFMADD213PDZmbkz = 9552,
  VFMADD213PDZmk = 9553,
  VFMADD213PDZmkz = 9554,
  VFMADD213PDZr = 9555,
  VFMADD213PDZrb = 9556,
  VFMADD213PDZrbk = 9557,
  VFMADD213PDZrbkz = 9558,
  VFMADD213PDZrk = 9559,
  VFMADD213PDZrkz = 9560,
  VFMADD213PDm = 9561,
  VFMADD213PDr = 9562,
  VFMADD213PHZ128m = 9563,
  VFMADD213PHZ128mb = 9564,
  VFMADD213PHZ128mbk = 9565,
  VFMADD213PHZ128mbkz = 9566,
  VFMADD213PHZ128mk = 9567,
  VFMADD213PHZ128mkz = 9568,
  VFMADD213PHZ128r = 9569,
  VFMADD213PHZ128rk = 9570,
  VFMADD213PHZ128rkz = 9571,
  VFMADD213PHZ256m = 9572,
  VFMADD213PHZ256mb = 9573,
  VFMADD213PHZ256mbk = 9574,
  VFMADD213PHZ256mbkz = 9575,
  VFMADD213PHZ256mk = 9576,
  VFMADD213PHZ256mkz = 9577,
  VFMADD213PHZ256r = 9578,
  VFMADD213PHZ256rk = 9579,
  VFMADD213PHZ256rkz = 9580,
  VFMADD213PHZm = 9581,
  VFMADD213PHZmb = 9582,
  VFMADD213PHZmbk = 9583,
  VFMADD213PHZmbkz = 9584,
  VFMADD213PHZmk = 9585,
  VFMADD213PHZmkz = 9586,
  VFMADD213PHZr = 9587,
  VFMADD213PHZrb = 9588,
  VFMADD213PHZrbk = 9589,
  VFMADD213PHZrbkz = 9590,
  VFMADD213PHZrk = 9591,
  VFMADD213PHZrkz = 9592,
  VFMADD213PSYm = 9593,
  VFMADD213PSYr = 9594,
  VFMADD213PSZ128m = 9595,
  VFMADD213PSZ128mb = 9596,
  VFMADD213PSZ128mbk = 9597,
  VFMADD213PSZ128mbkz = 9598,
  VFMADD213PSZ128mk = 9599,
  VFMADD213PSZ128mkz = 9600,
  VFMADD213PSZ128r = 9601,
  VFMADD213PSZ128rk = 9602,
  VFMADD213PSZ128rkz = 9603,
  VFMADD213PSZ256m = 9604,
  VFMADD213PSZ256mb = 9605,
  VFMADD213PSZ256mbk = 9606,
  VFMADD213PSZ256mbkz = 9607,
  VFMADD213PSZ256mk = 9608,
  VFMADD213PSZ256mkz = 9609,
  VFMADD213PSZ256r = 9610,
  VFMADD213PSZ256rk = 9611,
  VFMADD213PSZ256rkz = 9612,
  VFMADD213PSZm = 9613,
  VFMADD213PSZmb = 9614,
  VFMADD213PSZmbk = 9615,
  VFMADD213PSZmbkz = 9616,
  VFMADD213PSZmk = 9617,
  VFMADD213PSZmkz = 9618,
  VFMADD213PSZr = 9619,
  VFMADD213PSZrb = 9620,
  VFMADD213PSZrbk = 9621,
  VFMADD213PSZrbkz = 9622,
  VFMADD213PSZrk = 9623,
  VFMADD213PSZrkz = 9624,
  VFMADD213PSm = 9625,
  VFMADD213PSr = 9626,
  VFMADD213SDZm = 9627,
  VFMADD213SDZm_Int = 9628,
  VFMADD213SDZmk_Int = 9629,
  VFMADD213SDZmkz_Int = 9630,
  VFMADD213SDZr = 9631,
  VFMADD213SDZr_Int = 9632,
  VFMADD213SDZrb = 9633,
  VFMADD213SDZrb_Int = 9634,
  VFMADD213SDZrbk_Int = 9635,
  VFMADD213SDZrbkz_Int = 9636,
  VFMADD213SDZrk_Int = 9637,
  VFMADD213SDZrkz_Int = 9638,
  VFMADD213SDm = 9639,
  VFMADD213SDm_Int = 9640,
  VFMADD213SDr = 9641,
  VFMADD213SDr_Int = 9642,
  VFMADD213SHZm = 9643,
  VFMADD213SHZm_Int = 9644,
  VFMADD213SHZmk_Int = 9645,
  VFMADD213SHZmkz_Int = 9646,
  VFMADD213SHZr = 9647,
  VFMADD213SHZr_Int = 9648,
  VFMADD213SHZrb = 9649,
  VFMADD213SHZrb_Int = 9650,
  VFMADD213SHZrbk_Int = 9651,
  VFMADD213SHZrbkz_Int = 9652,
  VFMADD213SHZrk_Int = 9653,
  VFMADD213SHZrkz_Int = 9654,
  VFMADD213SSZm = 9655,
  VFMADD213SSZm_Int = 9656,
  VFMADD213SSZmk_Int = 9657,
  VFMADD213SSZmkz_Int = 9658,
  VFMADD213SSZr = 9659,
  VFMADD213SSZr_Int = 9660,
  VFMADD213SSZrb = 9661,
  VFMADD213SSZrb_Int = 9662,
  VFMADD213SSZrbk_Int = 9663,
  VFMADD213SSZrbkz_Int = 9664,
  VFMADD213SSZrk_Int = 9665,
  VFMADD213SSZrkz_Int = 9666,
  VFMADD213SSm = 9667,
  VFMADD213SSm_Int = 9668,
  VFMADD213SSr = 9669,
  VFMADD213SSr_Int = 9670,
  VFMADD231BF16Z128m = 9671,
  VFMADD231BF16Z128mb = 9672,
  VFMADD231BF16Z128mbk = 9673,
  VFMADD231BF16Z128mbkz = 9674,
  VFMADD231BF16Z128mk = 9675,
  VFMADD231BF16Z128mkz = 9676,
  VFMADD231BF16Z128r = 9677,
  VFMADD231BF16Z128rk = 9678,
  VFMADD231BF16Z128rkz = 9679,
  VFMADD231BF16Z256m = 9680,
  VFMADD231BF16Z256mb = 9681,
  VFMADD231BF16Z256mbk = 9682,
  VFMADD231BF16Z256mbkz = 9683,
  VFMADD231BF16Z256mk = 9684,
  VFMADD231BF16Z256mkz = 9685,
  VFMADD231BF16Z256r = 9686,
  VFMADD231BF16Z256rk = 9687,
  VFMADD231BF16Z256rkz = 9688,
  VFMADD231BF16Zm = 9689,
  VFMADD231BF16Zmb = 9690,
  VFMADD231BF16Zmbk = 9691,
  VFMADD231BF16Zmbkz = 9692,
  VFMADD231BF16Zmk = 9693,
  VFMADD231BF16Zmkz = 9694,
  VFMADD231BF16Zr = 9695,
  VFMADD231BF16Zrk = 9696,
  VFMADD231BF16Zrkz = 9697,
  VFMADD231PDYm = 9698,
  VFMADD231PDYr = 9699,
  VFMADD231PDZ128m = 9700,
  VFMADD231PDZ128mb = 9701,
  VFMADD231PDZ128mbk = 9702,
  VFMADD231PDZ128mbkz = 9703,
  VFMADD231PDZ128mk = 9704,
  VFMADD231PDZ128mkz = 9705,
  VFMADD231PDZ128r = 9706,
  VFMADD231PDZ128rk = 9707,
  VFMADD231PDZ128rkz = 9708,
  VFMADD231PDZ256m = 9709,
  VFMADD231PDZ256mb = 9710,
  VFMADD231PDZ256mbk = 9711,
  VFMADD231PDZ256mbkz = 9712,
  VFMADD231PDZ256mk = 9713,
  VFMADD231PDZ256mkz = 9714,
  VFMADD231PDZ256r = 9715,
  VFMADD231PDZ256rk = 9716,
  VFMADD231PDZ256rkz = 9717,
  VFMADD231PDZm = 9718,
  VFMADD231PDZmb = 9719,
  VFMADD231PDZmbk = 9720,
  VFMADD231PDZmbkz = 9721,
  VFMADD231PDZmk = 9722,
  VFMADD231PDZmkz = 9723,
  VFMADD231PDZr = 9724,
  VFMADD231PDZrb = 9725,
  VFMADD231PDZrbk = 9726,
  VFMADD231PDZrbkz = 9727,
  VFMADD231PDZrk = 9728,
  VFMADD231PDZrkz = 9729,
  VFMADD231PDm = 9730,
  VFMADD231PDr = 9731,
  VFMADD231PHZ128m = 9732,
  VFMADD231PHZ128mb = 9733,
  VFMADD231PHZ128mbk = 9734,
  VFMADD231PHZ128mbkz = 9735,
  VFMADD231PHZ128mk = 9736,
  VFMADD231PHZ128mkz = 9737,
  VFMADD231PHZ128r = 9738,
  VFMADD231PHZ128rk = 9739,
  VFMADD231PHZ128rkz = 9740,
  VFMADD231PHZ256m = 9741,
  VFMADD231PHZ256mb = 9742,
  VFMADD231PHZ256mbk = 9743,
  VFMADD231PHZ256mbkz = 9744,
  VFMADD231PHZ256mk = 9745,
  VFMADD231PHZ256mkz = 9746,
  VFMADD231PHZ256r = 9747,
  VFMADD231PHZ256rk = 9748,
  VFMADD231PHZ256rkz = 9749,
  VFMADD231PHZm = 9750,
  VFMADD231PHZmb = 9751,
  VFMADD231PHZmbk = 9752,
  VFMADD231PHZmbkz = 9753,
  VFMADD231PHZmk = 9754,
  VFMADD231PHZmkz = 9755,
  VFMADD231PHZr = 9756,
  VFMADD231PHZrb = 9757,
  VFMADD231PHZrbk = 9758,
  VFMADD231PHZrbkz = 9759,
  VFMADD231PHZrk = 9760,
  VFMADD231PHZrkz = 9761,
  VFMADD231PSYm = 9762,
  VFMADD231PSYr = 9763,
  VFMADD231PSZ128m = 9764,
  VFMADD231PSZ128mb = 9765,
  VFMADD231PSZ128mbk = 9766,
  VFMADD231PSZ128mbkz = 9767,
  VFMADD231PSZ128mk = 9768,
  VFMADD231PSZ128mkz = 9769,
  VFMADD231PSZ128r = 9770,
  VFMADD231PSZ128rk = 9771,
  VFMADD231PSZ128rkz = 9772,
  VFMADD231PSZ256m = 9773,
  VFMADD231PSZ256mb = 9774,
  VFMADD231PSZ256mbk = 9775,
  VFMADD231PSZ256mbkz = 9776,
  VFMADD231PSZ256mk = 9777,
  VFMADD231PSZ256mkz = 9778,
  VFMADD231PSZ256r = 9779,
  VFMADD231PSZ256rk = 9780,
  VFMADD231PSZ256rkz = 9781,
  VFMADD231PSZm = 9782,
  VFMADD231PSZmb = 9783,
  VFMADD231PSZmbk = 9784,
  VFMADD231PSZmbkz = 9785,
  VFMADD231PSZmk = 9786,
  VFMADD231PSZmkz = 9787,
  VFMADD231PSZr = 9788,
  VFMADD231PSZrb = 9789,
  VFMADD231PSZrbk = 9790,
  VFMADD231PSZrbkz = 9791,
  VFMADD231PSZrk = 9792,
  VFMADD231PSZrkz = 9793,
  VFMADD231PSm = 9794,
  VFMADD231PSr = 9795,
  VFMADD231SDZm = 9796,
  VFMADD231SDZm_Int = 9797,
  VFMADD231SDZmk_Int = 9798,
  VFMADD231SDZmkz_Int = 9799,
  VFMADD231SDZr = 9800,
  VFMADD231SDZr_Int = 9801,
  VFMADD231SDZrb = 9802,
  VFMADD231SDZrb_Int = 9803,
  VFMADD231SDZrbk_Int = 9804,
  VFMADD231SDZrbkz_Int = 9805,
  VFMADD231SDZrk_Int = 9806,
  VFMADD231SDZrkz_Int = 9807,
  VFMADD231SDm = 9808,
  VFMADD231SDm_Int = 9809,
  VFMADD231SDr = 9810,
  VFMADD231SDr_Int = 9811,
  VFMADD231SHZm = 9812,
  VFMADD231SHZm_Int = 9813,
  VFMADD231SHZmk_Int = 9814,
  VFMADD231SHZmkz_Int = 9815,
  VFMADD231SHZr = 9816,
  VFMADD231SHZr_Int = 9817,
  VFMADD231SHZrb = 9818,
  VFMADD231SHZrb_Int = 9819,
  VFMADD231SHZrbk_Int = 9820,
  VFMADD231SHZrbkz_Int = 9821,
  VFMADD231SHZrk_Int = 9822,
  VFMADD231SHZrkz_Int = 9823,
  VFMADD231SSZm = 9824,
  VFMADD231SSZm_Int = 9825,
  VFMADD231SSZmk_Int = 9826,
  VFMADD231SSZmkz_Int = 9827,
  VFMADD231SSZr = 9828,
  VFMADD231SSZr_Int = 9829,
  VFMADD231SSZrb = 9830,
  VFMADD231SSZrb_Int = 9831,
  VFMADD231SSZrbk_Int = 9832,
  VFMADD231SSZrbkz_Int = 9833,
  VFMADD231SSZrk_Int = 9834,
  VFMADD231SSZrkz_Int = 9835,
  VFMADD231SSm = 9836,
  VFMADD231SSm_Int = 9837,
  VFMADD231SSr = 9838,
  VFMADD231SSr_Int = 9839,
  VFMADDCPHZ128m = 9840,
  VFMADDCPHZ128mb = 9841,
  VFMADDCPHZ128mbk = 9842,
  VFMADDCPHZ128mbkz = 9843,
  VFMADDCPHZ128mk = 9844,
  VFMADDCPHZ128mkz = 9845,
  VFMADDCPHZ128r = 9846,
  VFMADDCPHZ128rk = 9847,
  VFMADDCPHZ128rkz = 9848,
  VFMADDCPHZ256m = 9849,
  VFMADDCPHZ256mb = 9850,
  VFMADDCPHZ256mbk = 9851,
  VFMADDCPHZ256mbkz = 9852,
  VFMADDCPHZ256mk = 9853,
  VFMADDCPHZ256mkz = 9854,
  VFMADDCPHZ256r = 9855,
  VFMADDCPHZ256rk = 9856,
  VFMADDCPHZ256rkz = 9857,
  VFMADDCPHZm = 9858,
  VFMADDCPHZmb = 9859,
  VFMADDCPHZmbk = 9860,
  VFMADDCPHZmbkz = 9861,
  VFMADDCPHZmk = 9862,
  VFMADDCPHZmkz = 9863,
  VFMADDCPHZr = 9864,
  VFMADDCPHZrb = 9865,
  VFMADDCPHZrbk = 9866,
  VFMADDCPHZrbkz = 9867,
  VFMADDCPHZrk = 9868,
  VFMADDCPHZrkz = 9869,
  VFMADDCSHZm = 9870,
  VFMADDCSHZmk = 9871,
  VFMADDCSHZmkz = 9872,
  VFMADDCSHZr = 9873,
  VFMADDCSHZrb = 9874,
  VFMADDCSHZrbk = 9875,
  VFMADDCSHZrbkz = 9876,
  VFMADDCSHZrk = 9877,
  VFMADDCSHZrkz = 9878,
  VFMADDPD4Ymr = 9879,
  VFMADDPD4Yrm = 9880,
  VFMADDPD4Yrr = 9881,
  VFMADDPD4Yrr_REV = 9882,
  VFMADDPD4mr = 9883,
  VFMADDPD4rm = 9884,
  VFMADDPD4rr = 9885,
  VFMADDPD4rr_REV = 9886,
  VFMADDPS4Ymr = 9887,
  VFMADDPS4Yrm = 9888,
  VFMADDPS4Yrr = 9889,
  VFMADDPS4Yrr_REV = 9890,
  VFMADDPS4mr = 9891,
  VFMADDPS4rm = 9892,
  VFMADDPS4rr = 9893,
  VFMADDPS4rr_REV = 9894,
  VFMADDSD4mr = 9895,
  VFMADDSD4mr_Int = 9896,
  VFMADDSD4rm = 9897,
  VFMADDSD4rm_Int = 9898,
  VFMADDSD4rr = 9899,
  VFMADDSD4rr_Int = 9900,
  VFMADDSD4rr_Int_REV = 9901,
  VFMADDSD4rr_REV = 9902,
  VFMADDSS4mr = 9903,
  VFMADDSS4mr_Int = 9904,
  VFMADDSS4rm = 9905,
  VFMADDSS4rm_Int = 9906,
  VFMADDSS4rr = 9907,
  VFMADDSS4rr_Int = 9908,
  VFMADDSS4rr_Int_REV = 9909,
  VFMADDSS4rr_REV = 9910,
  VFMADDSUB132PDYm = 9911,
  VFMADDSUB132PDYr = 9912,
  VFMADDSUB132PDZ128m = 9913,
  VFMADDSUB132PDZ128mb = 9914,
  VFMADDSUB132PDZ128mbk = 9915,
  VFMADDSUB132PDZ128mbkz = 9916,
  VFMADDSUB132PDZ128mk = 9917,
  VFMADDSUB132PDZ128mkz = 9918,
  VFMADDSUB132PDZ128r = 9919,
  VFMADDSUB132PDZ128rk = 9920,
  VFMADDSUB132PDZ128rkz = 9921,
  VFMADDSUB132PDZ256m = 9922,
  VFMADDSUB132PDZ256mb = 9923,
  VFMADDSUB132PDZ256mbk = 9924,
  VFMADDSUB132PDZ256mbkz = 9925,
  VFMADDSUB132PDZ256mk = 9926,
  VFMADDSUB132PDZ256mkz = 9927,
  VFMADDSUB132PDZ256r = 9928,
  VFMADDSUB132PDZ256rk = 9929,
  VFMADDSUB132PDZ256rkz = 9930,
  VFMADDSUB132PDZm = 9931,
  VFMADDSUB132PDZmb = 9932,
  VFMADDSUB132PDZmbk = 9933,
  VFMADDSUB132PDZmbkz = 9934,
  VFMADDSUB132PDZmk = 9935,
  VFMADDSUB132PDZmkz = 9936,
  VFMADDSUB132PDZr = 9937,
  VFMADDSUB132PDZrb = 9938,
  VFMADDSUB132PDZrbk = 9939,
  VFMADDSUB132PDZrbkz = 9940,
  VFMADDSUB132PDZrk = 9941,
  VFMADDSUB132PDZrkz = 9942,
  VFMADDSUB132PDm = 9943,
  VFMADDSUB132PDr = 9944,
  VFMADDSUB132PHZ128m = 9945,
  VFMADDSUB132PHZ128mb = 9946,
  VFMADDSUB132PHZ128mbk = 9947,
  VFMADDSUB132PHZ128mbkz = 9948,
  VFMADDSUB132PHZ128mk = 9949,
  VFMADDSUB132PHZ128mkz = 9950,
  VFMADDSUB132PHZ128r = 9951,
  VFMADDSUB132PHZ128rk = 9952,
  VFMADDSUB132PHZ128rkz = 9953,
  VFMADDSUB132PHZ256m = 9954,
  VFMADDSUB132PHZ256mb = 9955,
  VFMADDSUB132PHZ256mbk = 9956,
  VFMADDSUB132PHZ256mbkz = 9957,
  VFMADDSUB132PHZ256mk = 9958,
  VFMADDSUB132PHZ256mkz = 9959,
  VFMADDSUB132PHZ256r = 9960,
  VFMADDSUB132PHZ256rk = 9961,
  VFMADDSUB132PHZ256rkz = 9962,
  VFMADDSUB132PHZm = 9963,
  VFMADDSUB132PHZmb = 9964,
  VFMADDSUB132PHZmbk = 9965,
  VFMADDSUB132PHZmbkz = 9966,
  VFMADDSUB132PHZmk = 9967,
  VFMADDSUB132PHZmkz = 9968,
  VFMADDSUB132PHZr = 9969,
  VFMADDSUB132PHZrb = 9970,
  VFMADDSUB132PHZrbk = 9971,
  VFMADDSUB132PHZrbkz = 9972,
  VFMADDSUB132PHZrk = 9973,
  VFMADDSUB132PHZrkz = 9974,
  VFMADDSUB132PSYm = 9975,
  VFMADDSUB132PSYr = 9976,
  VFMADDSUB132PSZ128m = 9977,
  VFMADDSUB132PSZ128mb = 9978,
  VFMADDSUB132PSZ128mbk = 9979,
  VFMADDSUB132PSZ128mbkz = 9980,
  VFMADDSUB132PSZ128mk = 9981,
  VFMADDSUB132PSZ128mkz = 9982,
  VFMADDSUB132PSZ128r = 9983,
  VFMADDSUB132PSZ128rk = 9984,
  VFMADDSUB132PSZ128rkz = 9985,
  VFMADDSUB132PSZ256m = 9986,
  VFMADDSUB132PSZ256mb = 9987,
  VFMADDSUB132PSZ256mbk = 9988,
  VFMADDSUB132PSZ256mbkz = 9989,
  VFMADDSUB132PSZ256mk = 9990,
  VFMADDSUB132PSZ256mkz = 9991,
  VFMADDSUB132PSZ256r = 9992,
  VFMADDSUB132PSZ256rk = 9993,
  VFMADDSUB132PSZ256rkz = 9994,
  VFMADDSUB132PSZm = 9995,
  VFMADDSUB132PSZmb = 9996,
  VFMADDSUB132PSZmbk = 9997,
  VFMADDSUB132PSZmbkz = 9998,
  VFMADDSUB132PSZmk = 9999,
  VFMADDSUB132PSZmkz = 10000,
  VFMADDSUB132PSZr = 10001,
  VFMADDSUB132PSZrb = 10002,
  VFMADDSUB132PSZrbk = 10003,
  VFMADDSUB132PSZrbkz = 10004,
  VFMADDSUB132PSZrk = 10005,
  VFMADDSUB132PSZrkz = 10006,
  VFMADDSUB132PSm = 10007,
  VFMADDSUB132PSr = 10008,
  VFMADDSUB213PDYm = 10009,
  VFMADDSUB213PDYr = 10010,
  VFMADDSUB213PDZ128m = 10011,
  VFMADDSUB213PDZ128mb = 10012,
  VFMADDSUB213PDZ128mbk = 10013,
  VFMADDSUB213PDZ128mbkz = 10014,
  VFMADDSUB213PDZ128mk = 10015,
  VFMADDSUB213PDZ128mkz = 10016,
  VFMADDSUB213PDZ128r = 10017,
  VFMADDSUB213PDZ128rk = 10018,
  VFMADDSUB213PDZ128rkz = 10019,
  VFMADDSUB213PDZ256m = 10020,
  VFMADDSUB213PDZ256mb = 10021,
  VFMADDSUB213PDZ256mbk = 10022,
  VFMADDSUB213PDZ256mbkz = 10023,
  VFMADDSUB213PDZ256mk = 10024,
  VFMADDSUB213PDZ256mkz = 10025,
  VFMADDSUB213PDZ256r = 10026,
  VFMADDSUB213PDZ256rk = 10027,
  VFMADDSUB213PDZ256rkz = 10028,
  VFMADDSUB213PDZm = 10029,
  VFMADDSUB213PDZmb = 10030,
  VFMADDSUB213PDZmbk = 10031,
  VFMADDSUB213PDZmbkz = 10032,
  VFMADDSUB213PDZmk = 10033,
  VFMADDSUB213PDZmkz = 10034,
  VFMADDSUB213PDZr = 10035,
  VFMADDSUB213PDZrb = 10036,
  VFMADDSUB213PDZrbk = 10037,
  VFMADDSUB213PDZrbkz = 10038,
  VFMADDSUB213PDZrk = 10039,
  VFMADDSUB213PDZrkz = 10040,
  VFMADDSUB213PDm = 10041,
  VFMADDSUB213PDr = 10042,
  VFMADDSUB213PHZ128m = 10043,
  VFMADDSUB213PHZ128mb = 10044,
  VFMADDSUB213PHZ128mbk = 10045,
  VFMADDSUB213PHZ128mbkz = 10046,
  VFMADDSUB213PHZ128mk = 10047,
  VFMADDSUB213PHZ128mkz = 10048,
  VFMADDSUB213PHZ128r = 10049,
  VFMADDSUB213PHZ128rk = 10050,
  VFMADDSUB213PHZ128rkz = 10051,
  VFMADDSUB213PHZ256m = 10052,
  VFMADDSUB213PHZ256mb = 10053,
  VFMADDSUB213PHZ256mbk = 10054,
  VFMADDSUB213PHZ256mbkz = 10055,
  VFMADDSUB213PHZ256mk = 10056,
  VFMADDSUB213PHZ256mkz = 10057,
  VFMADDSUB213PHZ256r = 10058,
  VFMADDSUB213PHZ256rk = 10059,
  VFMADDSUB213PHZ256rkz = 10060,
  VFMADDSUB213PHZm = 10061,
  VFMADDSUB213PHZmb = 10062,
  VFMADDSUB213PHZmbk = 10063,
  VFMADDSUB213PHZmbkz = 10064,
  VFMADDSUB213PHZmk = 10065,
  VFMADDSUB213PHZmkz = 10066,
  VFMADDSUB213PHZr = 10067,
  VFMADDSUB213PHZrb = 10068,
  VFMADDSUB213PHZrbk = 10069,
  VFMADDSUB213PHZrbkz = 10070,
  VFMADDSUB213PHZrk = 10071,
  VFMADDSUB213PHZrkz = 10072,
  VFMADDSUB213PSYm = 10073,
  VFMADDSUB213PSYr = 10074,
  VFMADDSUB213PSZ128m = 10075,
  VFMADDSUB213PSZ128mb = 10076,
  VFMADDSUB213PSZ128mbk = 10077,
  VFMADDSUB213PSZ128mbkz = 10078,
  VFMADDSUB213PSZ128mk = 10079,
  VFMADDSUB213PSZ128mkz = 10080,
  VFMADDSUB213PSZ128r = 10081,
  VFMADDSUB213PSZ128rk = 10082,
  VFMADDSUB213PSZ128rkz = 10083,
  VFMADDSUB213PSZ256m = 10084,
  VFMADDSUB213PSZ256mb = 10085,
  VFMADDSUB213PSZ256mbk = 10086,
  VFMADDSUB213PSZ256mbkz = 10087,
  VFMADDSUB213PSZ256mk = 10088,
  VFMADDSUB213PSZ256mkz = 10089,
  VFMADDSUB213PSZ256r = 10090,
  VFMADDSUB213PSZ256rk = 10091,
  VFMADDSUB213PSZ256rkz = 10092,
  VFMADDSUB213PSZm = 10093,
  VFMADDSUB213PSZmb = 10094,
  VFMADDSUB213PSZmbk = 10095,
  VFMADDSUB213PSZmbkz = 10096,
  VFMADDSUB213PSZmk = 10097,
  VFMADDSUB213PSZmkz = 10098,
  VFMADDSUB213PSZr = 10099,
  VFMADDSUB213PSZrb = 10100,
  VFMADDSUB213PSZrbk = 10101,
  VFMADDSUB213PSZrbkz = 10102,
  VFMADDSUB213PSZrk = 10103,
  VFMADDSUB213PSZrkz = 10104,
  VFMADDSUB213PSm = 10105,
  VFMADDSUB213PSr = 10106,
  VFMADDSUB231PDYm = 10107,
  VFMADDSUB231PDYr = 10108,
  VFMADDSUB231PDZ128m = 10109,
  VFMADDSUB231PDZ128mb = 10110,
  VFMADDSUB231PDZ128mbk = 10111,
  VFMADDSUB231PDZ128mbkz = 10112,
  VFMADDSUB231PDZ128mk = 10113,
  VFMADDSUB231PDZ128mkz = 10114,
  VFMADDSUB231PDZ128r = 10115,
  VFMADDSUB231PDZ128rk = 10116,
  VFMADDSUB231PDZ128rkz = 10117,
  VFMADDSUB231PDZ256m = 10118,
  VFMADDSUB231PDZ256mb = 10119,
  VFMADDSUB231PDZ256mbk = 10120,
  VFMADDSUB231PDZ256mbkz = 10121,
  VFMADDSUB231PDZ256mk = 10122,
  VFMADDSUB231PDZ256mkz = 10123,
  VFMADDSUB231PDZ256r = 10124,
  VFMADDSUB231PDZ256rk = 10125,
  VFMADDSUB231PDZ256rkz = 10126,
  VFMADDSUB231PDZm = 10127,
  VFMADDSUB231PDZmb = 10128,
  VFMADDSUB231PDZmbk = 10129,
  VFMADDSUB231PDZmbkz = 10130,
  VFMADDSUB231PDZmk = 10131,
  VFMADDSUB231PDZmkz = 10132,
  VFMADDSUB231PDZr = 10133,
  VFMADDSUB231PDZrb = 10134,
  VFMADDSUB231PDZrbk = 10135,
  VFMADDSUB231PDZrbkz = 10136,
  VFMADDSUB231PDZrk = 10137,
  VFMADDSUB231PDZrkz = 10138,
  VFMADDSUB231PDm = 10139,
  VFMADDSUB231PDr = 10140,
  VFMADDSUB231PHZ128m = 10141,
  VFMADDSUB231PHZ128mb = 10142,
  VFMADDSUB231PHZ128mbk = 10143,
  VFMADDSUB231PHZ128mbkz = 10144,
  VFMADDSUB231PHZ128mk = 10145,
  VFMADDSUB231PHZ128mkz = 10146,
  VFMADDSUB231PHZ128r = 10147,
  VFMADDSUB231PHZ128rk = 10148,
  VFMADDSUB231PHZ128rkz = 10149,
  VFMADDSUB231PHZ256m = 10150,
  VFMADDSUB231PHZ256mb = 10151,
  VFMADDSUB231PHZ256mbk = 10152,
  VFMADDSUB231PHZ256mbkz = 10153,
  VFMADDSUB231PHZ256mk = 10154,
  VFMADDSUB231PHZ256mkz = 10155,
  VFMADDSUB231PHZ256r = 10156,
  VFMADDSUB231PHZ256rk = 10157,
  VFMADDSUB231PHZ256rkz = 10158,
  VFMADDSUB231PHZm = 10159,
  VFMADDSUB231PHZmb = 10160,
  VFMADDSUB231PHZmbk = 10161,
  VFMADDSUB231PHZmbkz = 10162,
  VFMADDSUB231PHZmk = 10163,
  VFMADDSUB231PHZmkz = 10164,
  VFMADDSUB231PHZr = 10165,
  VFMADDSUB231PHZrb = 10166,
  VFMADDSUB231PHZrbk = 10167,
  VFMADDSUB231PHZrbkz = 10168,
  VFMADDSUB231PHZrk = 10169,
  VFMADDSUB231PHZrkz = 10170,
  VFMADDSUB231PSYm = 10171,
  VFMADDSUB231PSYr = 10172,
  VFMADDSUB231PSZ128m = 10173,
  VFMADDSUB231PSZ128mb = 10174,
  VFMADDSUB231PSZ128mbk = 10175,
  VFMADDSUB231PSZ128mbkz = 10176,
  VFMADDSUB231PSZ128mk = 10177,
  VFMADDSUB231PSZ128mkz = 10178,
  VFMADDSUB231PSZ128r = 10179,
  VFMADDSUB231PSZ128rk = 10180,
  VFMADDSUB231PSZ128rkz = 10181,
  VFMADDSUB231PSZ256m = 10182,
  VFMADDSUB231PSZ256mb = 10183,
  VFMADDSUB231PSZ256mbk = 10184,
  VFMADDSUB231PSZ256mbkz = 10185,
  VFMADDSUB231PSZ256mk = 10186,
  VFMADDSUB231PSZ256mkz = 10187,
  VFMADDSUB231PSZ256r = 10188,
  VFMADDSUB231PSZ256rk = 10189,
  VFMADDSUB231PSZ256rkz = 10190,
  VFMADDSUB231PSZm = 10191,
  VFMADDSUB231PSZmb = 10192,
  VFMADDSUB231PSZmbk = 10193,
  VFMADDSUB231PSZmbkz = 10194,
  VFMADDSUB231PSZmk = 10195,
  VFMADDSUB231PSZmkz = 10196,
  VFMADDSUB231PSZr = 10197,
  VFMADDSUB231PSZrb = 10198,
  VFMADDSUB231PSZrbk = 10199,
  VFMADDSUB231PSZrbkz = 10200,
  VFMADDSUB231PSZrk = 10201,
  VFMADDSUB231PSZrkz = 10202,
  VFMADDSUB231PSm = 10203,
  VFMADDSUB231PSr = 10204,
  VFMADDSUBPD4Ymr = 10205,
  VFMADDSUBPD4Yrm = 10206,
  VFMADDSUBPD4Yrr = 10207,
  VFMADDSUBPD4Yrr_REV = 10208,
  VFMADDSUBPD4mr = 10209,
  VFMADDSUBPD4rm = 10210,
  VFMADDSUBPD4rr = 10211,
  VFMADDSUBPD4rr_REV = 10212,
  VFMADDSUBPS4Ymr = 10213,
  VFMADDSUBPS4Yrm = 10214,
  VFMADDSUBPS4Yrr = 10215,
  VFMADDSUBPS4Yrr_REV = 10216,
  VFMADDSUBPS4mr = 10217,
  VFMADDSUBPS4rm = 10218,
  VFMADDSUBPS4rr = 10219,
  VFMADDSUBPS4rr_REV = 10220,
  VFMSUB132BF16Z128m = 10221,
  VFMSUB132BF16Z128mb = 10222,
  VFMSUB132BF16Z128mbk = 10223,
  VFMSUB132BF16Z128mbkz = 10224,
  VFMSUB132BF16Z128mk = 10225,
  VFMSUB132BF16Z128mkz = 10226,
  VFMSUB132BF16Z128r = 10227,
  VFMSUB132BF16Z128rk = 10228,
  VFMSUB132BF16Z128rkz = 10229,
  VFMSUB132BF16Z256m = 10230,
  VFMSUB132BF16Z256mb = 10231,
  VFMSUB132BF16Z256mbk = 10232,
  VFMSUB132BF16Z256mbkz = 10233,
  VFMSUB132BF16Z256mk = 10234,
  VFMSUB132BF16Z256mkz = 10235,
  VFMSUB132BF16Z256r = 10236,
  VFMSUB132BF16Z256rk = 10237,
  VFMSUB132BF16Z256rkz = 10238,
  VFMSUB132BF16Zm = 10239,
  VFMSUB132BF16Zmb = 10240,
  VFMSUB132BF16Zmbk = 10241,
  VFMSUB132BF16Zmbkz = 10242,
  VFMSUB132BF16Zmk = 10243,
  VFMSUB132BF16Zmkz = 10244,
  VFMSUB132BF16Zr = 10245,
  VFMSUB132BF16Zrk = 10246,
  VFMSUB132BF16Zrkz = 10247,
  VFMSUB132PDYm = 10248,
  VFMSUB132PDYr = 10249,
  VFMSUB132PDZ128m = 10250,
  VFMSUB132PDZ128mb = 10251,
  VFMSUB132PDZ128mbk = 10252,
  VFMSUB132PDZ128mbkz = 10253,
  VFMSUB132PDZ128mk = 10254,
  VFMSUB132PDZ128mkz = 10255,
  VFMSUB132PDZ128r = 10256,
  VFMSUB132PDZ128rk = 10257,
  VFMSUB132PDZ128rkz = 10258,
  VFMSUB132PDZ256m = 10259,
  VFMSUB132PDZ256mb = 10260,
  VFMSUB132PDZ256mbk = 10261,
  VFMSUB132PDZ256mbkz = 10262,
  VFMSUB132PDZ256mk = 10263,
  VFMSUB132PDZ256mkz = 10264,
  VFMSUB132PDZ256r = 10265,
  VFMSUB132PDZ256rk = 10266,
  VFMSUB132PDZ256rkz = 10267,
  VFMSUB132PDZm = 10268,
  VFMSUB132PDZmb = 10269,
  VFMSUB132PDZmbk = 10270,
  VFMSUB132PDZmbkz = 10271,
  VFMSUB132PDZmk = 10272,
  VFMSUB132PDZmkz = 10273,
  VFMSUB132PDZr = 10274,
  VFMSUB132PDZrb = 10275,
  VFMSUB132PDZrbk = 10276,
  VFMSUB132PDZrbkz = 10277,
  VFMSUB132PDZrk = 10278,
  VFMSUB132PDZrkz = 10279,
  VFMSUB132PDm = 10280,
  VFMSUB132PDr = 10281,
  VFMSUB132PHZ128m = 10282,
  VFMSUB132PHZ128mb = 10283,
  VFMSUB132PHZ128mbk = 10284,
  VFMSUB132PHZ128mbkz = 10285,
  VFMSUB132PHZ128mk = 10286,
  VFMSUB132PHZ128mkz = 10287,
  VFMSUB132PHZ128r = 10288,
  VFMSUB132PHZ128rk = 10289,
  VFMSUB132PHZ128rkz = 10290,
  VFMSUB132PHZ256m = 10291,
  VFMSUB132PHZ256mb = 10292,
  VFMSUB132PHZ256mbk = 10293,
  VFMSUB132PHZ256mbkz = 10294,
  VFMSUB132PHZ256mk = 10295,
  VFMSUB132PHZ256mkz = 10296,
  VFMSUB132PHZ256r = 10297,
  VFMSUB132PHZ256rk = 10298,
  VFMSUB132PHZ256rkz = 10299,
  VFMSUB132PHZm = 10300,
  VFMSUB132PHZmb = 10301,
  VFMSUB132PHZmbk = 10302,
  VFMSUB132PHZmbkz = 10303,
  VFMSUB132PHZmk = 10304,
  VFMSUB132PHZmkz = 10305,
  VFMSUB132PHZr = 10306,
  VFMSUB132PHZrb = 10307,
  VFMSUB132PHZrbk = 10308,
  VFMSUB132PHZrbkz = 10309,
  VFMSUB132PHZrk = 10310,
  VFMSUB132PHZrkz = 10311,
  VFMSUB132PSYm = 10312,
  VFMSUB132PSYr = 10313,
  VFMSUB132PSZ128m = 10314,
  VFMSUB132PSZ128mb = 10315,
  VFMSUB132PSZ128mbk = 10316,
  VFMSUB132PSZ128mbkz = 10317,
  VFMSUB132PSZ128mk = 10318,
  VFMSUB132PSZ128mkz = 10319,
  VFMSUB132PSZ128r = 10320,
  VFMSUB132PSZ128rk = 10321,
  VFMSUB132PSZ128rkz = 10322,
  VFMSUB132PSZ256m = 10323,
  VFMSUB132PSZ256mb = 10324,
  VFMSUB132PSZ256mbk = 10325,
  VFMSUB132PSZ256mbkz = 10326,
  VFMSUB132PSZ256mk = 10327,
  VFMSUB132PSZ256mkz = 10328,
  VFMSUB132PSZ256r = 10329,
  VFMSUB132PSZ256rk = 10330,
  VFMSUB132PSZ256rkz = 10331,
  VFMSUB132PSZm = 10332,
  VFMSUB132PSZmb = 10333,
  VFMSUB132PSZmbk = 10334,
  VFMSUB132PSZmbkz = 10335,
  VFMSUB132PSZmk = 10336,
  VFMSUB132PSZmkz = 10337,
  VFMSUB132PSZr = 10338,
  VFMSUB132PSZrb = 10339,
  VFMSUB132PSZrbk = 10340,
  VFMSUB132PSZrbkz = 10341,
  VFMSUB132PSZrk = 10342,
  VFMSUB132PSZrkz = 10343,
  VFMSUB132PSm = 10344,
  VFMSUB132PSr = 10345,
  VFMSUB132SDZm = 10346,
  VFMSUB132SDZm_Int = 10347,
  VFMSUB132SDZmk_Int = 10348,
  VFMSUB132SDZmkz_Int = 10349,
  VFMSUB132SDZr = 10350,
  VFMSUB132SDZr_Int = 10351,
  VFMSUB132SDZrb = 10352,
  VFMSUB132SDZrb_Int = 10353,
  VFMSUB132SDZrbk_Int = 10354,
  VFMSUB132SDZrbkz_Int = 10355,
  VFMSUB132SDZrk_Int = 10356,
  VFMSUB132SDZrkz_Int = 10357,
  VFMSUB132SDm = 10358,
  VFMSUB132SDm_Int = 10359,
  VFMSUB132SDr = 10360,
  VFMSUB132SDr_Int = 10361,
  VFMSUB132SHZm = 10362,
  VFMSUB132SHZm_Int = 10363,
  VFMSUB132SHZmk_Int = 10364,
  VFMSUB132SHZmkz_Int = 10365,
  VFMSUB132SHZr = 10366,
  VFMSUB132SHZr_Int = 10367,
  VFMSUB132SHZrb = 10368,
  VFMSUB132SHZrb_Int = 10369,
  VFMSUB132SHZrbk_Int = 10370,
  VFMSUB132SHZrbkz_Int = 10371,
  VFMSUB132SHZrk_Int = 10372,
  VFMSUB132SHZrkz_Int = 10373,
  VFMSUB132SSZm = 10374,
  VFMSUB132SSZm_Int = 10375,
  VFMSUB132SSZmk_Int = 10376,
  VFMSUB132SSZmkz_Int = 10377,
  VFMSUB132SSZr = 10378,
  VFMSUB132SSZr_Int = 10379,
  VFMSUB132SSZrb = 10380,
  VFMSUB132SSZrb_Int = 10381,
  VFMSUB132SSZrbk_Int = 10382,
  VFMSUB132SSZrbkz_Int = 10383,
  VFMSUB132SSZrk_Int = 10384,
  VFMSUB132SSZrkz_Int = 10385,
  VFMSUB132SSm = 10386,
  VFMSUB132SSm_Int = 10387,
  VFMSUB132SSr = 10388,
  VFMSUB132SSr_Int = 10389,
  VFMSUB213BF16Z128m = 10390,
  VFMSUB213BF16Z128mb = 10391,
  VFMSUB213BF16Z128mbk = 10392,
  VFMSUB213BF16Z128mbkz = 10393,
  VFMSUB213BF16Z128mk = 10394,
  VFMSUB213BF16Z128mkz = 10395,
  VFMSUB213BF16Z128r = 10396,
  VFMSUB213BF16Z128rk = 10397,
  VFMSUB213BF16Z128rkz = 10398,
  VFMSUB213BF16Z256m = 10399,
  VFMSUB213BF16Z256mb = 10400,
  VFMSUB213BF16Z256mbk = 10401,
  VFMSUB213BF16Z256mbkz = 10402,
  VFMSUB213BF16Z256mk = 10403,
  VFMSUB213BF16Z256mkz = 10404,
  VFMSUB213BF16Z256r = 10405,
  VFMSUB213BF16Z256rk = 10406,
  VFMSUB213BF16Z256rkz = 10407,
  VFMSUB213BF16Zm = 10408,
  VFMSUB213BF16Zmb = 10409,
  VFMSUB213BF16Zmbk = 10410,
  VFMSUB213BF16Zmbkz = 10411,
  VFMSUB213BF16Zmk = 10412,
  VFMSUB213BF16Zmkz = 10413,
  VFMSUB213BF16Zr = 10414,
  VFMSUB213BF16Zrk = 10415,
  VFMSUB213BF16Zrkz = 10416,
  VFMSUB213PDYm = 10417,
  VFMSUB213PDYr = 10418,
  VFMSUB213PDZ128m = 10419,
  VFMSUB213PDZ128mb = 10420,
  VFMSUB213PDZ128mbk = 10421,
  VFMSUB213PDZ128mbkz = 10422,
  VFMSUB213PDZ128mk = 10423,
  VFMSUB213PDZ128mkz = 10424,
  VFMSUB213PDZ128r = 10425,
  VFMSUB213PDZ128rk = 10426,
  VFMSUB213PDZ128rkz = 10427,
  VFMSUB213PDZ256m = 10428,
  VFMSUB213PDZ256mb = 10429,
  VFMSUB213PDZ256mbk = 10430,
  VFMSUB213PDZ256mbkz = 10431,
  VFMSUB213PDZ256mk = 10432,
  VFMSUB213PDZ256mkz = 10433,
  VFMSUB213PDZ256r = 10434,
  VFMSUB213PDZ256rk = 10435,
  VFMSUB213PDZ256rkz = 10436,
  VFMSUB213PDZm = 10437,
  VFMSUB213PDZmb = 10438,
  VFMSUB213PDZmbk = 10439,
  VFMSUB213PDZmbkz = 10440,
  VFMSUB213PDZmk = 10441,
  VFMSUB213PDZmkz = 10442,
  VFMSUB213PDZr = 10443,
  VFMSUB213PDZrb = 10444,
  VFMSUB213PDZrbk = 10445,
  VFMSUB213PDZrbkz = 10446,
  VFMSUB213PDZrk = 10447,
  VFMSUB213PDZrkz = 10448,
  VFMSUB213PDm = 10449,
  VFMSUB213PDr = 10450,
  VFMSUB213PHZ128m = 10451,
  VFMSUB213PHZ128mb = 10452,
  VFMSUB213PHZ128mbk = 10453,
  VFMSUB213PHZ128mbkz = 10454,
  VFMSUB213PHZ128mk = 10455,
  VFMSUB213PHZ128mkz = 10456,
  VFMSUB213PHZ128r = 10457,
  VFMSUB213PHZ128rk = 10458,
  VFMSUB213PHZ128rkz = 10459,
  VFMSUB213PHZ256m = 10460,
  VFMSUB213PHZ256mb = 10461,
  VFMSUB213PHZ256mbk = 10462,
  VFMSUB213PHZ256mbkz = 10463,
  VFMSUB213PHZ256mk = 10464,
  VFMSUB213PHZ256mkz = 10465,
  VFMSUB213PHZ256r = 10466,
  VFMSUB213PHZ256rk = 10467,
  VFMSUB213PHZ256rkz = 10468,
  VFMSUB213PHZm = 10469,
  VFMSUB213PHZmb = 10470,
  VFMSUB213PHZmbk = 10471,
  VFMSUB213PHZmbkz = 10472,
  VFMSUB213PHZmk = 10473,
  VFMSUB213PHZmkz = 10474,
  VFMSUB213PHZr = 10475,
  VFMSUB213PHZrb = 10476,
  VFMSUB213PHZrbk = 10477,
  VFMSUB213PHZrbkz = 10478,
  VFMSUB213PHZrk = 10479,
  VFMSUB213PHZrkz = 10480,
  VFMSUB213PSYm = 10481,
  VFMSUB213PSYr = 10482,
  VFMSUB213PSZ128m = 10483,
  VFMSUB213PSZ128mb = 10484,
  VFMSUB213PSZ128mbk = 10485,
  VFMSUB213PSZ128mbkz = 10486,
  VFMSUB213PSZ128mk = 10487,
  VFMSUB213PSZ128mkz = 10488,
  VFMSUB213PSZ128r = 10489,
  VFMSUB213PSZ128rk = 10490,
  VFMSUB213PSZ128rkz = 10491,
  VFMSUB213PSZ256m = 10492,
  VFMSUB213PSZ256mb = 10493,
  VFMSUB213PSZ256mbk = 10494,
  VFMSUB213PSZ256mbkz = 10495,
  VFMSUB213PSZ256mk = 10496,
  VFMSUB213PSZ256mkz = 10497,
  VFMSUB213PSZ256r = 10498,
  VFMSUB213PSZ256rk = 10499,
  VFMSUB213PSZ256rkz = 10500,
  VFMSUB213PSZm = 10501,
  VFMSUB213PSZmb = 10502,
  VFMSUB213PSZmbk = 10503,
  VFMSUB213PSZmbkz = 10504,
  VFMSUB213PSZmk = 10505,
  VFMSUB213PSZmkz = 10506,
  VFMSUB213PSZr = 10507,
  VFMSUB213PSZrb = 10508,
  VFMSUB213PSZrbk = 10509,
  VFMSUB213PSZrbkz = 10510,
  VFMSUB213PSZrk = 10511,
  VFMSUB213PSZrkz = 10512,
  VFMSUB213PSm = 10513,
  VFMSUB213PSr = 10514,
  VFMSUB213SDZm = 10515,
  VFMSUB213SDZm_Int = 10516,
  VFMSUB213SDZmk_Int = 10517,
  VFMSUB213SDZmkz_Int = 10518,
  VFMSUB213SDZr = 10519,
  VFMSUB213SDZr_Int = 10520,
  VFMSUB213SDZrb = 10521,
  VFMSUB213SDZrb_Int = 10522,
  VFMSUB213SDZrbk_Int = 10523,
  VFMSUB213SDZrbkz_Int = 10524,
  VFMSUB213SDZrk_Int = 10525,
  VFMSUB213SDZrkz_Int = 10526,
  VFMSUB213SDm = 10527,
  VFMSUB213SDm_Int = 10528,
  VFMSUB213SDr = 10529,
  VFMSUB213SDr_Int = 10530,
  VFMSUB213SHZm = 10531,
  VFMSUB213SHZm_Int = 10532,
  VFMSUB213SHZmk_Int = 10533,
  VFMSUB213SHZmkz_Int = 10534,
  VFMSUB213SHZr = 10535,
  VFMSUB213SHZr_Int = 10536,
  VFMSUB213SHZrb = 10537,
  VFMSUB213SHZrb_Int = 10538,
  VFMSUB213SHZrbk_Int = 10539,
  VFMSUB213SHZrbkz_Int = 10540,
  VFMSUB213SHZrk_Int = 10541,
  VFMSUB213SHZrkz_Int = 10542,
  VFMSUB213SSZm = 10543,
  VFMSUB213SSZm_Int = 10544,
  VFMSUB213SSZmk_Int = 10545,
  VFMSUB213SSZmkz_Int = 10546,
  VFMSUB213SSZr = 10547,
  VFMSUB213SSZr_Int = 10548,
  VFMSUB213SSZrb = 10549,
  VFMSUB213SSZrb_Int = 10550,
  VFMSUB213SSZrbk_Int = 10551,
  VFMSUB213SSZrbkz_Int = 10552,
  VFMSUB213SSZrk_Int = 10553,
  VFMSUB213SSZrkz_Int = 10554,
  VFMSUB213SSm = 10555,
  VFMSUB213SSm_Int = 10556,
  VFMSUB213SSr = 10557,
  VFMSUB213SSr_Int = 10558,
  VFMSUB231BF16Z128m = 10559,
  VFMSUB231BF16Z128mb = 10560,
  VFMSUB231BF16Z128mbk = 10561,
  VFMSUB231BF16Z128mbkz = 10562,
  VFMSUB231BF16Z128mk = 10563,
  VFMSUB231BF16Z128mkz = 10564,
  VFMSUB231BF16Z128r = 10565,
  VFMSUB231BF16Z128rk = 10566,
  VFMSUB231BF16Z128rkz = 10567,
  VFMSUB231BF16Z256m = 10568,
  VFMSUB231BF16Z256mb = 10569,
  VFMSUB231BF16Z256mbk = 10570,
  VFMSUB231BF16Z256mbkz = 10571,
  VFMSUB231BF16Z256mk = 10572,
  VFMSUB231BF16Z256mkz = 10573,
  VFMSUB231BF16Z256r = 10574,
  VFMSUB231BF16Z256rk = 10575,
  VFMSUB231BF16Z256rkz = 10576,
  VFMSUB231BF16Zm = 10577,
  VFMSUB231BF16Zmb = 10578,
  VFMSUB231BF16Zmbk = 10579,
  VFMSUB231BF16Zmbkz = 10580,
  VFMSUB231BF16Zmk = 10581,
  VFMSUB231BF16Zmkz = 10582,
  VFMSUB231BF16Zr = 10583,
  VFMSUB231BF16Zrk = 10584,
  VFMSUB231BF16Zrkz = 10585,
  VFMSUB231PDYm = 10586,
  VFMSUB231PDYr = 10587,
  VFMSUB231PDZ128m = 10588,
  VFMSUB231PDZ128mb = 10589,
  VFMSUB231PDZ128mbk = 10590,
  VFMSUB231PDZ128mbkz = 10591,
  VFMSUB231PDZ128mk = 10592,
  VFMSUB231PDZ128mkz = 10593,
  VFMSUB231PDZ128r = 10594,
  VFMSUB231PDZ128rk = 10595,
  VFMSUB231PDZ128rkz = 10596,
  VFMSUB231PDZ256m = 10597,
  VFMSUB231PDZ256mb = 10598,
  VFMSUB231PDZ256mbk = 10599,
  VFMSUB231PDZ256mbkz = 10600,
  VFMSUB231PDZ256mk = 10601,
  VFMSUB231PDZ256mkz = 10602,
  VFMSUB231PDZ256r = 10603,
  VFMSUB231PDZ256rk = 10604,
  VFMSUB231PDZ256rkz = 10605,
  VFMSUB231PDZm = 10606,
  VFMSUB231PDZmb = 10607,
  VFMSUB231PDZmbk = 10608,
  VFMSUB231PDZmbkz = 10609,
  VFMSUB231PDZmk = 10610,
  VFMSUB231PDZmkz = 10611,
  VFMSUB231PDZr = 10612,
  VFMSUB231PDZrb = 10613,
  VFMSUB231PDZrbk = 10614,
  VFMSUB231PDZrbkz = 10615,
  VFMSUB231PDZrk = 10616,
  VFMSUB231PDZrkz = 10617,
  VFMSUB231PDm = 10618,
  VFMSUB231PDr = 10619,
  VFMSUB231PHZ128m = 10620,
  VFMSUB231PHZ128mb = 10621,
  VFMSUB231PHZ128mbk = 10622,
  VFMSUB231PHZ128mbkz = 10623,
  VFMSUB231PHZ128mk = 10624,
  VFMSUB231PHZ128mkz = 10625,
  VFMSUB231PHZ128r = 10626,
  VFMSUB231PHZ128rk = 10627,
  VFMSUB231PHZ128rkz = 10628,
  VFMSUB231PHZ256m = 10629,
  VFMSUB231PHZ256mb = 10630,
  VFMSUB231PHZ256mbk = 10631,
  VFMSUB231PHZ256mbkz = 10632,
  VFMSUB231PHZ256mk = 10633,
  VFMSUB231PHZ256mkz = 10634,
  VFMSUB231PHZ256r = 10635,
  VFMSUB231PHZ256rk = 10636,
  VFMSUB231PHZ256rkz = 10637,
  VFMSUB231PHZm = 10638,
  VFMSUB231PHZmb = 10639,
  VFMSUB231PHZmbk = 10640,
  VFMSUB231PHZmbkz = 10641,
  VFMSUB231PHZmk = 10642,
  VFMSUB231PHZmkz = 10643,
  VFMSUB231PHZr = 10644,
  VFMSUB231PHZrb = 10645,
  VFMSUB231PHZrbk = 10646,
  VFMSUB231PHZrbkz = 10647,
  VFMSUB231PHZrk = 10648,
  VFMSUB231PHZrkz = 10649,
  VFMSUB231PSYm = 10650,
  VFMSUB231PSYr = 10651,
  VFMSUB231PSZ128m = 10652,
  VFMSUB231PSZ128mb = 10653,
  VFMSUB231PSZ128mbk = 10654,
  VFMSUB231PSZ128mbkz = 10655,
  VFMSUB231PSZ128mk = 10656,
  VFMSUB231PSZ128mkz = 10657,
  VFMSUB231PSZ128r = 10658,
  VFMSUB231PSZ128rk = 10659,
  VFMSUB231PSZ128rkz = 10660,
  VFMSUB231PSZ256m = 10661,
  VFMSUB231PSZ256mb = 10662,
  VFMSUB231PSZ256mbk = 10663,
  VFMSUB231PSZ256mbkz = 10664,
  VFMSUB231PSZ256mk = 10665,
  VFMSUB231PSZ256mkz = 10666,
  VFMSUB231PSZ256r = 10667,
  VFMSUB231PSZ256rk = 10668,
  VFMSUB231PSZ256rkz = 10669,
  VFMSUB231PSZm = 10670,
  VFMSUB231PSZmb = 10671,
  VFMSUB231PSZmbk = 10672,
  VFMSUB231PSZmbkz = 10673,
  VFMSUB231PSZmk = 10674,
  VFMSUB231PSZmkz = 10675,
  VFMSUB231PSZr = 10676,
  VFMSUB231PSZrb = 10677,
  VFMSUB231PSZrbk = 10678,
  VFMSUB231PSZrbkz = 10679,
  VFMSUB231PSZrk = 10680,
  VFMSUB231PSZrkz = 10681,
  VFMSUB231PSm = 10682,
  VFMSUB231PSr = 10683,
  VFMSUB231SDZm = 10684,
  VFMSUB231SDZm_Int = 10685,
  VFMSUB231SDZmk_Int = 10686,
  VFMSUB231SDZmkz_Int = 10687,
  VFMSUB231SDZr = 10688,
  VFMSUB231SDZr_Int = 10689,
  VFMSUB231SDZrb = 10690,
  VFMSUB231SDZrb_Int = 10691,
  VFMSUB231SDZrbk_Int = 10692,
  VFMSUB231SDZrbkz_Int = 10693,
  VFMSUB231SDZrk_Int = 10694,
  VFMSUB231SDZrkz_Int = 10695,
  VFMSUB231SDm = 10696,
  VFMSUB231SDm_Int = 10697,
  VFMSUB231SDr = 10698,
  VFMSUB231SDr_Int = 10699,
  VFMSUB231SHZm = 10700,
  VFMSUB231SHZm_Int = 10701,
  VFMSUB231SHZmk_Int = 10702,
  VFMSUB231SHZmkz_Int = 10703,
  VFMSUB231SHZr = 10704,
  VFMSUB231SHZr_Int = 10705,
  VFMSUB231SHZrb = 10706,
  VFMSUB231SHZrb_Int = 10707,
  VFMSUB231SHZrbk_Int = 10708,
  VFMSUB231SHZrbkz_Int = 10709,
  VFMSUB231SHZrk_Int = 10710,
  VFMSUB231SHZrkz_Int = 10711,
  VFMSUB231SSZm = 10712,
  VFMSUB231SSZm_Int = 10713,
  VFMSUB231SSZmk_Int = 10714,
  VFMSUB231SSZmkz_Int = 10715,
  VFMSUB231SSZr = 10716,
  VFMSUB231SSZr_Int = 10717,
  VFMSUB231SSZrb = 10718,
  VFMSUB231SSZrb_Int = 10719,
  VFMSUB231SSZrbk_Int = 10720,
  VFMSUB231SSZrbkz_Int = 10721,
  VFMSUB231SSZrk_Int = 10722,
  VFMSUB231SSZrkz_Int = 10723,
  VFMSUB231SSm = 10724,
  VFMSUB231SSm_Int = 10725,
  VFMSUB231SSr = 10726,
  VFMSUB231SSr_Int = 10727,
  VFMSUBADD132PDYm = 10728,
  VFMSUBADD132PDYr = 10729,
  VFMSUBADD132PDZ128m = 10730,
  VFMSUBADD132PDZ128mb = 10731,
  VFMSUBADD132PDZ128mbk = 10732,
  VFMSUBADD132PDZ128mbkz = 10733,
  VFMSUBADD132PDZ128mk = 10734,
  VFMSUBADD132PDZ128mkz = 10735,
  VFMSUBADD132PDZ128r = 10736,
  VFMSUBADD132PDZ128rk = 10737,
  VFMSUBADD132PDZ128rkz = 10738,
  VFMSUBADD132PDZ256m = 10739,
  VFMSUBADD132PDZ256mb = 10740,
  VFMSUBADD132PDZ256mbk = 10741,
  VFMSUBADD132PDZ256mbkz = 10742,
  VFMSUBADD132PDZ256mk = 10743,
  VFMSUBADD132PDZ256mkz = 10744,
  VFMSUBADD132PDZ256r = 10745,
  VFMSUBADD132PDZ256rk = 10746,
  VFMSUBADD132PDZ256rkz = 10747,
  VFMSUBADD132PDZm = 10748,
  VFMSUBADD132PDZmb = 10749,
  VFMSUBADD132PDZmbk = 10750,
  VFMSUBADD132PDZmbkz = 10751,
  VFMSUBADD132PDZmk = 10752,
  VFMSUBADD132PDZmkz = 10753,
  VFMSUBADD132PDZr = 10754,
  VFMSUBADD132PDZrb = 10755,
  VFMSUBADD132PDZrbk = 10756,
  VFMSUBADD132PDZrbkz = 10757,
  VFMSUBADD132PDZrk = 10758,
  VFMSUBADD132PDZrkz = 10759,
  VFMSUBADD132PDm = 10760,
  VFMSUBADD132PDr = 10761,
  VFMSUBADD132PHZ128m = 10762,
  VFMSUBADD132PHZ128mb = 10763,
  VFMSUBADD132PHZ128mbk = 10764,
  VFMSUBADD132PHZ128mbkz = 10765,
  VFMSUBADD132PHZ128mk = 10766,
  VFMSUBADD132PHZ128mkz = 10767,
  VFMSUBADD132PHZ128r = 10768,
  VFMSUBADD132PHZ128rk = 10769,
  VFMSUBADD132PHZ128rkz = 10770,
  VFMSUBADD132PHZ256m = 10771,
  VFMSUBADD132PHZ256mb = 10772,
  VFMSUBADD132PHZ256mbk = 10773,
  VFMSUBADD132PHZ256mbkz = 10774,
  VFMSUBADD132PHZ256mk = 10775,
  VFMSUBADD132PHZ256mkz = 10776,
  VFMSUBADD132PHZ256r = 10777,
  VFMSUBADD132PHZ256rk = 10778,
  VFMSUBADD132PHZ256rkz = 10779,
  VFMSUBADD132PHZm = 10780,
  VFMSUBADD132PHZmb = 10781,
  VFMSUBADD132PHZmbk = 10782,
  VFMSUBADD132PHZmbkz = 10783,
  VFMSUBADD132PHZmk = 10784,
  VFMSUBADD132PHZmkz = 10785,
  VFMSUBADD132PHZr = 10786,
  VFMSUBADD132PHZrb = 10787,
  VFMSUBADD132PHZrbk = 10788,
  VFMSUBADD132PHZrbkz = 10789,
  VFMSUBADD132PHZrk = 10790,
  VFMSUBADD132PHZrkz = 10791,
  VFMSUBADD132PSYm = 10792,
  VFMSUBADD132PSYr = 10793,
  VFMSUBADD132PSZ128m = 10794,
  VFMSUBADD132PSZ128mb = 10795,
  VFMSUBADD132PSZ128mbk = 10796,
  VFMSUBADD132PSZ128mbkz = 10797,
  VFMSUBADD132PSZ128mk = 10798,
  VFMSUBADD132PSZ128mkz = 10799,
  VFMSUBADD132PSZ128r = 10800,
  VFMSUBADD132PSZ128rk = 10801,
  VFMSUBADD132PSZ128rkz = 10802,
  VFMSUBADD132PSZ256m = 10803,
  VFMSUBADD132PSZ256mb = 10804,
  VFMSUBADD132PSZ256mbk = 10805,
  VFMSUBADD132PSZ256mbkz = 10806,
  VFMSUBADD132PSZ256mk = 10807,
  VFMSUBADD132PSZ256mkz = 10808,
  VFMSUBADD132PSZ256r = 10809,
  VFMSUBADD132PSZ256rk = 10810,
  VFMSUBADD132PSZ256rkz = 10811,
  VFMSUBADD132PSZm = 10812,
  VFMSUBADD132PSZmb = 10813,
  VFMSUBADD132PSZmbk = 10814,
  VFMSUBADD132PSZmbkz = 10815,
  VFMSUBADD132PSZmk = 10816,
  VFMSUBADD132PSZmkz = 10817,
  VFMSUBADD132PSZr = 10818,
  VFMSUBADD132PSZrb = 10819,
  VFMSUBADD132PSZrbk = 10820,
  VFMSUBADD132PSZrbkz = 10821,
  VFMSUBADD132PSZrk = 10822,
  VFMSUBADD132PSZrkz = 10823,
  VFMSUBADD132PSm = 10824,
  VFMSUBADD132PSr = 10825,
  VFMSUBADD213PDYm = 10826,
  VFMSUBADD213PDYr = 10827,
  VFMSUBADD213PDZ128m = 10828,
  VFMSUBADD213PDZ128mb = 10829,
  VFMSUBADD213PDZ128mbk = 10830,
  VFMSUBADD213PDZ128mbkz = 10831,
  VFMSUBADD213PDZ128mk = 10832,
  VFMSUBADD213PDZ128mkz = 10833,
  VFMSUBADD213PDZ128r = 10834,
  VFMSUBADD213PDZ128rk = 10835,
  VFMSUBADD213PDZ128rkz = 10836,
  VFMSUBADD213PDZ256m = 10837,
  VFMSUBADD213PDZ256mb = 10838,
  VFMSUBADD213PDZ256mbk = 10839,
  VFMSUBADD213PDZ256mbkz = 10840,
  VFMSUBADD213PDZ256mk = 10841,
  VFMSUBADD213PDZ256mkz = 10842,
  VFMSUBADD213PDZ256r = 10843,
  VFMSUBADD213PDZ256rk = 10844,
  VFMSUBADD213PDZ256rkz = 10845,
  VFMSUBADD213PDZm = 10846,
  VFMSUBADD213PDZmb = 10847,
  VFMSUBADD213PDZmbk = 10848,
  VFMSUBADD213PDZmbkz = 10849,
  VFMSUBADD213PDZmk = 10850,
  VFMSUBADD213PDZmkz = 10851,
  VFMSUBADD213PDZr = 10852,
  VFMSUBADD213PDZrb = 10853,
  VFMSUBADD213PDZrbk = 10854,
  VFMSUBADD213PDZrbkz = 10855,
  VFMSUBADD213PDZrk = 10856,
  VFMSUBADD213PDZrkz = 10857,
  VFMSUBADD213PDm = 10858,
  VFMSUBADD213PDr = 10859,
  VFMSUBADD213PHZ128m = 10860,
  VFMSUBADD213PHZ128mb = 10861,
  VFMSUBADD213PHZ128mbk = 10862,
  VFMSUBADD213PHZ128mbkz = 10863,
  VFMSUBADD213PHZ128mk = 10864,
  VFMSUBADD213PHZ128mkz = 10865,
  VFMSUBADD213PHZ128r = 10866,
  VFMSUBADD213PHZ128rk = 10867,
  VFMSUBADD213PHZ128rkz = 10868,
  VFMSUBADD213PHZ256m = 10869,
  VFMSUBADD213PHZ256mb = 10870,
  VFMSUBADD213PHZ256mbk = 10871,
  VFMSUBADD213PHZ256mbkz = 10872,
  VFMSUBADD213PHZ256mk = 10873,
  VFMSUBADD213PHZ256mkz = 10874,
  VFMSUBADD213PHZ256r = 10875,
  VFMSUBADD213PHZ256rk = 10876,
  VFMSUBADD213PHZ256rkz = 10877,
  VFMSUBADD213PHZm = 10878,
  VFMSUBADD213PHZmb = 10879,
  VFMSUBADD213PHZmbk = 10880,
  VFMSUBADD213PHZmbkz = 10881,
  VFMSUBADD213PHZmk = 10882,
  VFMSUBADD213PHZmkz = 10883,
  VFMSUBADD213PHZr = 10884,
  VFMSUBADD213PHZrb = 10885,
  VFMSUBADD213PHZrbk = 10886,
  VFMSUBADD213PHZrbkz = 10887,
  VFMSUBADD213PHZrk = 10888,
  VFMSUBADD213PHZrkz = 10889,
  VFMSUBADD213PSYm = 10890,
  VFMSUBADD213PSYr = 10891,
  VFMSUBADD213PSZ128m = 10892,
  VFMSUBADD213PSZ128mb = 10893,
  VFMSUBADD213PSZ128mbk = 10894,
  VFMSUBADD213PSZ128mbkz = 10895,
  VFMSUBADD213PSZ128mk = 10896,
  VFMSUBADD213PSZ128mkz = 10897,
  VFMSUBADD213PSZ128r = 10898,
  VFMSUBADD213PSZ128rk = 10899,
  VFMSUBADD213PSZ128rkz = 10900,
  VFMSUBADD213PSZ256m = 10901,
  VFMSUBADD213PSZ256mb = 10902,
  VFMSUBADD213PSZ256mbk = 10903,
  VFMSUBADD213PSZ256mbkz = 10904,
  VFMSUBADD213PSZ256mk = 10905,
  VFMSUBADD213PSZ256mkz = 10906,
  VFMSUBADD213PSZ256r = 10907,
  VFMSUBADD213PSZ256rk = 10908,
  VFMSUBADD213PSZ256rkz = 10909,
  VFMSUBADD213PSZm = 10910,
  VFMSUBADD213PSZmb = 10911,
  VFMSUBADD213PSZmbk = 10912,
  VFMSUBADD213PSZmbkz = 10913,
  VFMSUBADD213PSZmk = 10914,
  VFMSUBADD213PSZmkz = 10915,
  VFMSUBADD213PSZr = 10916,
  VFMSUBADD213PSZrb = 10917,
  VFMSUBADD213PSZrbk = 10918,
  VFMSUBADD213PSZrbkz = 10919,
  VFMSUBADD213PSZrk = 10920,
  VFMSUBADD213PSZrkz = 10921,
  VFMSUBADD213PSm = 10922,
  VFMSUBADD213PSr = 10923,
  VFMSUBADD231PDYm = 10924,
  VFMSUBADD231PDYr = 10925,
  VFMSUBADD231PDZ128m = 10926,
  VFMSUBADD231PDZ128mb = 10927,
  VFMSUBADD231PDZ128mbk = 10928,
  VFMSUBADD231PDZ128mbkz = 10929,
  VFMSUBADD231PDZ128mk = 10930,
  VFMSUBADD231PDZ128mkz = 10931,
  VFMSUBADD231PDZ128r = 10932,
  VFMSUBADD231PDZ128rk = 10933,
  VFMSUBADD231PDZ128rkz = 10934,
  VFMSUBADD231PDZ256m = 10935,
  VFMSUBADD231PDZ256mb = 10936,
  VFMSUBADD231PDZ256mbk = 10937,
  VFMSUBADD231PDZ256mbkz = 10938,
  VFMSUBADD231PDZ256mk = 10939,
  VFMSUBADD231PDZ256mkz = 10940,
  VFMSUBADD231PDZ256r = 10941,
  VFMSUBADD231PDZ256rk = 10942,
  VFMSUBADD231PDZ256rkz = 10943,
  VFMSUBADD231PDZm = 10944,
  VFMSUBADD231PDZmb = 10945,
  VFMSUBADD231PDZmbk = 10946,
  VFMSUBADD231PDZmbkz = 10947,
  VFMSUBADD231PDZmk = 10948,
  VFMSUBADD231PDZmkz = 10949,
  VFMSUBADD231PDZr = 10950,
  VFMSUBADD231PDZrb = 10951,
  VFMSUBADD231PDZrbk = 10952,
  VFMSUBADD231PDZrbkz = 10953,
  VFMSUBADD231PDZrk = 10954,
  VFMSUBADD231PDZrkz = 10955,
  VFMSUBADD231PDm = 10956,
  VFMSUBADD231PDr = 10957,
  VFMSUBADD231PHZ128m = 10958,
  VFMSUBADD231PHZ128mb = 10959,
  VFMSUBADD231PHZ128mbk = 10960,
  VFMSUBADD231PHZ128mbkz = 10961,
  VFMSUBADD231PHZ128mk = 10962,
  VFMSUBADD231PHZ128mkz = 10963,
  VFMSUBADD231PHZ128r = 10964,
  VFMSUBADD231PHZ128rk = 10965,
  VFMSUBADD231PHZ128rkz = 10966,
  VFMSUBADD231PHZ256m = 10967,
  VFMSUBADD231PHZ256mb = 10968,
  VFMSUBADD231PHZ256mbk = 10969,
  VFMSUBADD231PHZ256mbkz = 10970,
  VFMSUBADD231PHZ256mk = 10971,
  VFMSUBADD231PHZ256mkz = 10972,
  VFMSUBADD231PHZ256r = 10973,
  VFMSUBADD231PHZ256rk = 10974,
  VFMSUBADD231PHZ256rkz = 10975,
  VFMSUBADD231PHZm = 10976,
  VFMSUBADD231PHZmb = 10977,
  VFMSUBADD231PHZmbk = 10978,
  VFMSUBADD231PHZmbkz = 10979,
  VFMSUBADD231PHZmk = 10980,
  VFMSUBADD231PHZmkz = 10981,
  VFMSUBADD231PHZr = 10982,
  VFMSUBADD231PHZrb = 10983,
  VFMSUBADD231PHZrbk = 10984,
  VFMSUBADD231PHZrbkz = 10985,
  VFMSUBADD231PHZrk = 10986,
  VFMSUBADD231PHZrkz = 10987,
  VFMSUBADD231PSYm = 10988,
  VFMSUBADD231PSYr = 10989,
  VFMSUBADD231PSZ128m = 10990,
  VFMSUBADD231PSZ128mb = 10991,
  VFMSUBADD231PSZ128mbk = 10992,
  VFMSUBADD231PSZ128mbkz = 10993,
  VFMSUBADD231PSZ128mk = 10994,
  VFMSUBADD231PSZ128mkz = 10995,
  VFMSUBADD231PSZ128r = 10996,
  VFMSUBADD231PSZ128rk = 10997,
  VFMSUBADD231PSZ128rkz = 10998,
  VFMSUBADD231PSZ256m = 10999,
  VFMSUBADD231PSZ256mb = 11000,
  VFMSUBADD231PSZ256mbk = 11001,
  VFMSUBADD231PSZ256mbkz = 11002,
  VFMSUBADD231PSZ256mk = 11003,
  VFMSUBADD231PSZ256mkz = 11004,
  VFMSUBADD231PSZ256r = 11005,
  VFMSUBADD231PSZ256rk = 11006,
  VFMSUBADD231PSZ256rkz = 11007,
  VFMSUBADD231PSZm = 11008,
  VFMSUBADD231PSZmb = 11009,
  VFMSUBADD231PSZmbk = 11010,
  VFMSUBADD231PSZmbkz = 11011,
  VFMSUBADD231PSZmk = 11012,
  VFMSUBADD231PSZmkz = 11013,
  VFMSUBADD231PSZr = 11014,
  VFMSUBADD231PSZrb = 11015,
  VFMSUBADD231PSZrbk = 11016,
  VFMSUBADD231PSZrbkz = 11017,
  VFMSUBADD231PSZrk = 11018,
  VFMSUBADD231PSZrkz = 11019,
  VFMSUBADD231PSm = 11020,
  VFMSUBADD231PSr = 11021,
  VFMSUBADDPD4Ymr = 11022,
  VFMSUBADDPD4Yrm = 11023,
  VFMSUBADDPD4Yrr = 11024,
  VFMSUBADDPD4Yrr_REV = 11025,
  VFMSUBADDPD4mr = 11026,
  VFMSUBADDPD4rm = 11027,
  VFMSUBADDPD4rr = 11028,
  VFMSUBADDPD4rr_REV = 11029,
  VFMSUBADDPS4Ymr = 11030,
  VFMSUBADDPS4Yrm = 11031,
  VFMSUBADDPS4Yrr = 11032,
  VFMSUBADDPS4Yrr_REV = 11033,
  VFMSUBADDPS4mr = 11034,
  VFMSUBADDPS4rm = 11035,
  VFMSUBADDPS4rr = 11036,
  VFMSUBADDPS4rr_REV = 11037,
  VFMSUBPD4Ymr = 11038,
  VFMSUBPD4Yrm = 11039,
  VFMSUBPD4Yrr = 11040,
  VFMSUBPD4Yrr_REV = 11041,
  VFMSUBPD4mr = 11042,
  VFMSUBPD4rm = 11043,
  VFMSUBPD4rr = 11044,
  VFMSUBPD4rr_REV = 11045,
  VFMSUBPS4Ymr = 11046,
  VFMSUBPS4Yrm = 11047,
  VFMSUBPS4Yrr = 11048,
  VFMSUBPS4Yrr_REV = 11049,
  VFMSUBPS4mr = 11050,
  VFMSUBPS4rm = 11051,
  VFMSUBPS4rr = 11052,
  VFMSUBPS4rr_REV = 11053,
  VFMSUBSD4mr = 11054,
  VFMSUBSD4mr_Int = 11055,
  VFMSUBSD4rm = 11056,
  VFMSUBSD4rm_Int = 11057,
  VFMSUBSD4rr = 11058,
  VFMSUBSD4rr_Int = 11059,
  VFMSUBSD4rr_Int_REV = 11060,
  VFMSUBSD4rr_REV = 11061,
  VFMSUBSS4mr = 11062,
  VFMSUBSS4mr_Int = 11063,
  VFMSUBSS4rm = 11064,
  VFMSUBSS4rm_Int = 11065,
  VFMSUBSS4rr = 11066,
  VFMSUBSS4rr_Int = 11067,
  VFMSUBSS4rr_Int_REV = 11068,
  VFMSUBSS4rr_REV = 11069,
  VFMULCPHZ128rm = 11070,
  VFMULCPHZ128rmb = 11071,
  VFMULCPHZ128rmbk = 11072,
  VFMULCPHZ128rmbkz = 11073,
  VFMULCPHZ128rmk = 11074,
  VFMULCPHZ128rmkz = 11075,
  VFMULCPHZ128rr = 11076,
  VFMULCPHZ128rrk = 11077,
  VFMULCPHZ128rrkz = 11078,
  VFMULCPHZ256rm = 11079,
  VFMULCPHZ256rmb = 11080,
  VFMULCPHZ256rmbk = 11081,
  VFMULCPHZ256rmbkz = 11082,
  VFMULCPHZ256rmk = 11083,
  VFMULCPHZ256rmkz = 11084,
  VFMULCPHZ256rr = 11085,
  VFMULCPHZ256rrk = 11086,
  VFMULCPHZ256rrkz = 11087,
  VFMULCPHZrm = 11088,
  VFMULCPHZrmb = 11089,
  VFMULCPHZrmbk = 11090,
  VFMULCPHZrmbkz = 11091,
  VFMULCPHZrmk = 11092,
  VFMULCPHZrmkz = 11093,
  VFMULCPHZrr = 11094,
  VFMULCPHZrrb = 11095,
  VFMULCPHZrrbk = 11096,
  VFMULCPHZrrbkz = 11097,
  VFMULCPHZrrk = 11098,
  VFMULCPHZrrkz = 11099,
  VFMULCSHZrm = 11100,
  VFMULCSHZrmk = 11101,
  VFMULCSHZrmkz = 11102,
  VFMULCSHZrr = 11103,
  VFMULCSHZrrb = 11104,
  VFMULCSHZrrbk = 11105,
  VFMULCSHZrrbkz = 11106,
  VFMULCSHZrrk = 11107,
  VFMULCSHZrrkz = 11108,
  VFNMADD132BF16Z128m = 11109,
  VFNMADD132BF16Z128mb = 11110,
  VFNMADD132BF16Z128mbk = 11111,
  VFNMADD132BF16Z128mbkz = 11112,
  VFNMADD132BF16Z128mk = 11113,
  VFNMADD132BF16Z128mkz = 11114,
  VFNMADD132BF16Z128r = 11115,
  VFNMADD132BF16Z128rk = 11116,
  VFNMADD132BF16Z128rkz = 11117,
  VFNMADD132BF16Z256m = 11118,
  VFNMADD132BF16Z256mb = 11119,
  VFNMADD132BF16Z256mbk = 11120,
  VFNMADD132BF16Z256mbkz = 11121,
  VFNMADD132BF16Z256mk = 11122,
  VFNMADD132BF16Z256mkz = 11123,
  VFNMADD132BF16Z256r = 11124,
  VFNMADD132BF16Z256rk = 11125,
  VFNMADD132BF16Z256rkz = 11126,
  VFNMADD132BF16Zm = 11127,
  VFNMADD132BF16Zmb = 11128,
  VFNMADD132BF16Zmbk = 11129,
  VFNMADD132BF16Zmbkz = 11130,
  VFNMADD132BF16Zmk = 11131,
  VFNMADD132BF16Zmkz = 11132,
  VFNMADD132BF16Zr = 11133,
  VFNMADD132BF16Zrk = 11134,
  VFNMADD132BF16Zrkz = 11135,
  VFNMADD132PDYm = 11136,
  VFNMADD132PDYr = 11137,
  VFNMADD132PDZ128m = 11138,
  VFNMADD132PDZ128mb = 11139,
  VFNMADD132PDZ128mbk = 11140,
  VFNMADD132PDZ128mbkz = 11141,
  VFNMADD132PDZ128mk = 11142,
  VFNMADD132PDZ128mkz = 11143,
  VFNMADD132PDZ128r = 11144,
  VFNMADD132PDZ128rk = 11145,
  VFNMADD132PDZ128rkz = 11146,
  VFNMADD132PDZ256m = 11147,
  VFNMADD132PDZ256mb = 11148,
  VFNMADD132PDZ256mbk = 11149,
  VFNMADD132PDZ256mbkz = 11150,
  VFNMADD132PDZ256mk = 11151,
  VFNMADD132PDZ256mkz = 11152,
  VFNMADD132PDZ256r = 11153,
  VFNMADD132PDZ256rk = 11154,
  VFNMADD132PDZ256rkz = 11155,
  VFNMADD132PDZm = 11156,
  VFNMADD132PDZmb = 11157,
  VFNMADD132PDZmbk = 11158,
  VFNMADD132PDZmbkz = 11159,
  VFNMADD132PDZmk = 11160,
  VFNMADD132PDZmkz = 11161,
  VFNMADD132PDZr = 11162,
  VFNMADD132PDZrb = 11163,
  VFNMADD132PDZrbk = 11164,
  VFNMADD132PDZrbkz = 11165,
  VFNMADD132PDZrk = 11166,
  VFNMADD132PDZrkz = 11167,
  VFNMADD132PDm = 11168,
  VFNMADD132PDr = 11169,
  VFNMADD132PHZ128m = 11170,
  VFNMADD132PHZ128mb = 11171,
  VFNMADD132PHZ128mbk = 11172,
  VFNMADD132PHZ128mbkz = 11173,
  VFNMADD132PHZ128mk = 11174,
  VFNMADD132PHZ128mkz = 11175,
  VFNMADD132PHZ128r = 11176,
  VFNMADD132PHZ128rk = 11177,
  VFNMADD132PHZ128rkz = 11178,
  VFNMADD132PHZ256m = 11179,
  VFNMADD132PHZ256mb = 11180,
  VFNMADD132PHZ256mbk = 11181,
  VFNMADD132PHZ256mbkz = 11182,
  VFNMADD132PHZ256mk = 11183,
  VFNMADD132PHZ256mkz = 11184,
  VFNMADD132PHZ256r = 11185,
  VFNMADD132PHZ256rk = 11186,
  VFNMADD132PHZ256rkz = 11187,
  VFNMADD132PHZm = 11188,
  VFNMADD132PHZmb = 11189,
  VFNMADD132PHZmbk = 11190,
  VFNMADD132PHZmbkz = 11191,
  VFNMADD132PHZmk = 11192,
  VFNMADD132PHZmkz = 11193,
  VFNMADD132PHZr = 11194,
  VFNMADD132PHZrb = 11195,
  VFNMADD132PHZrbk = 11196,
  VFNMADD132PHZrbkz = 11197,
  VFNMADD132PHZrk = 11198,
  VFNMADD132PHZrkz = 11199,
  VFNMADD132PSYm = 11200,
  VFNMADD132PSYr = 11201,
  VFNMADD132PSZ128m = 11202,
  VFNMADD132PSZ128mb = 11203,
  VFNMADD132PSZ128mbk = 11204,
  VFNMADD132PSZ128mbkz = 11205,
  VFNMADD132PSZ128mk = 11206,
  VFNMADD132PSZ128mkz = 11207,
  VFNMADD132PSZ128r = 11208,
  VFNMADD132PSZ128rk = 11209,
  VFNMADD132PSZ128rkz = 11210,
  VFNMADD132PSZ256m = 11211,
  VFNMADD132PSZ256mb = 11212,
  VFNMADD132PSZ256mbk = 11213,
  VFNMADD132PSZ256mbkz = 11214,
  VFNMADD132PSZ256mk = 11215,
  VFNMADD132PSZ256mkz = 11216,
  VFNMADD132PSZ256r = 11217,
  VFNMADD132PSZ256rk = 11218,
  VFNMADD132PSZ256rkz = 11219,
  VFNMADD132PSZm = 11220,
  VFNMADD132PSZmb = 11221,
  VFNMADD132PSZmbk = 11222,
  VFNMADD132PSZmbkz = 11223,
  VFNMADD132PSZmk = 11224,
  VFNMADD132PSZmkz = 11225,
  VFNMADD132PSZr = 11226,
  VFNMADD132PSZrb = 11227,
  VFNMADD132PSZrbk = 11228,
  VFNMADD132PSZrbkz = 11229,
  VFNMADD132PSZrk = 11230,
  VFNMADD132PSZrkz = 11231,
  VFNMADD132PSm = 11232,
  VFNMADD132PSr = 11233,
  VFNMADD132SDZm = 11234,
  VFNMADD132SDZm_Int = 11235,
  VFNMADD132SDZmk_Int = 11236,
  VFNMADD132SDZmkz_Int = 11237,
  VFNMADD132SDZr = 11238,
  VFNMADD132SDZr_Int = 11239,
  VFNMADD132SDZrb = 11240,
  VFNMADD132SDZrb_Int = 11241,
  VFNMADD132SDZrbk_Int = 11242,
  VFNMADD132SDZrbkz_Int = 11243,
  VFNMADD132SDZrk_Int = 11244,
  VFNMADD132SDZrkz_Int = 11245,
  VFNMADD132SDm = 11246,
  VFNMADD132SDm_Int = 11247,
  VFNMADD132SDr = 11248,
  VFNMADD132SDr_Int = 11249,
  VFNMADD132SHZm = 11250,
  VFNMADD132SHZm_Int = 11251,
  VFNMADD132SHZmk_Int = 11252,
  VFNMADD132SHZmkz_Int = 11253,
  VFNMADD132SHZr = 11254,
  VFNMADD132SHZr_Int = 11255,
  VFNMADD132SHZrb = 11256,
  VFNMADD132SHZrb_Int = 11257,
  VFNMADD132SHZrbk_Int = 11258,
  VFNMADD132SHZrbkz_Int = 11259,
  VFNMADD132SHZrk_Int = 11260,
  VFNMADD132SHZrkz_Int = 11261,
  VFNMADD132SSZm = 11262,
  VFNMADD132SSZm_Int = 11263,
  VFNMADD132SSZmk_Int = 11264,
  VFNMADD132SSZmkz_Int = 11265,
  VFNMADD132SSZr = 11266,
  VFNMADD132SSZr_Int = 11267,
  VFNMADD132SSZrb = 11268,
  VFNMADD132SSZrb_Int = 11269,
  VFNMADD132SSZrbk_Int = 11270,
  VFNMADD132SSZrbkz_Int = 11271,
  VFNMADD132SSZrk_Int = 11272,
  VFNMADD132SSZrkz_Int = 11273,
  VFNMADD132SSm = 11274,
  VFNMADD132SSm_Int = 11275,
  VFNMADD132SSr = 11276,
  VFNMADD132SSr_Int = 11277,
  VFNMADD213BF16Z128m = 11278,
  VFNMADD213BF16Z128mb = 11279,
  VFNMADD213BF16Z128mbk = 11280,
  VFNMADD213BF16Z128mbkz = 11281,
  VFNMADD213BF16Z128mk = 11282,
  VFNMADD213BF16Z128mkz = 11283,
  VFNMADD213BF16Z128r = 11284,
  VFNMADD213BF16Z128rk = 11285,
  VFNMADD213BF16Z128rkz = 11286,
  VFNMADD213BF16Z256m = 11287,
  VFNMADD213BF16Z256mb = 11288,
  VFNMADD213BF16Z256mbk = 11289,
  VFNMADD213BF16Z256mbkz = 11290,
  VFNMADD213BF16Z256mk = 11291,
  VFNMADD213BF16Z256mkz = 11292,
  VFNMADD213BF16Z256r = 11293,
  VFNMADD213BF16Z256rk = 11294,
  VFNMADD213BF16Z256rkz = 11295,
  VFNMADD213BF16Zm = 11296,
  VFNMADD213BF16Zmb = 11297,
  VFNMADD213BF16Zmbk = 11298,
  VFNMADD213BF16Zmbkz = 11299,
  VFNMADD213BF16Zmk = 11300,
  VFNMADD213BF16Zmkz = 11301,
  VFNMADD213BF16Zr = 11302,
  VFNMADD213BF16Zrk = 11303,
  VFNMADD213BF16Zrkz = 11304,
  VFNMADD213PDYm = 11305,
  VFNMADD213PDYr = 11306,
  VFNMADD213PDZ128m = 11307,
  VFNMADD213PDZ128mb = 11308,
  VFNMADD213PDZ128mbk = 11309,
  VFNMADD213PDZ128mbkz = 11310,
  VFNMADD213PDZ128mk = 11311,
  VFNMADD213PDZ128mkz = 11312,
  VFNMADD213PDZ128r = 11313,
  VFNMADD213PDZ128rk = 11314,
  VFNMADD213PDZ128rkz = 11315,
  VFNMADD213PDZ256m = 11316,
  VFNMADD213PDZ256mb = 11317,
  VFNMADD213PDZ256mbk = 11318,
  VFNMADD213PDZ256mbkz = 11319,
  VFNMADD213PDZ256mk = 11320,
  VFNMADD213PDZ256mkz = 11321,
  VFNMADD213PDZ256r = 11322,
  VFNMADD213PDZ256rk = 11323,
  VFNMADD213PDZ256rkz = 11324,
  VFNMADD213PDZm = 11325,
  VFNMADD213PDZmb = 11326,
  VFNMADD213PDZmbk = 11327,
  VFNMADD213PDZmbkz = 11328,
  VFNMADD213PDZmk = 11329,
  VFNMADD213PDZmkz = 11330,
  VFNMADD213PDZr = 11331,
  VFNMADD213PDZrb = 11332,
  VFNMADD213PDZrbk = 11333,
  VFNMADD213PDZrbkz = 11334,
  VFNMADD213PDZrk = 11335,
  VFNMADD213PDZrkz = 11336,
  VFNMADD213PDm = 11337,
  VFNMADD213PDr = 11338,
  VFNMADD213PHZ128m = 11339,
  VFNMADD213PHZ128mb = 11340,
  VFNMADD213PHZ128mbk = 11341,
  VFNMADD213PHZ128mbkz = 11342,
  VFNMADD213PHZ128mk = 11343,
  VFNMADD213PHZ128mkz = 11344,
  VFNMADD213PHZ128r = 11345,
  VFNMADD213PHZ128rk = 11346,
  VFNMADD213PHZ128rkz = 11347,
  VFNMADD213PHZ256m = 11348,
  VFNMADD213PHZ256mb = 11349,
  VFNMADD213PHZ256mbk = 11350,
  VFNMADD213PHZ256mbkz = 11351,
  VFNMADD213PHZ256mk = 11352,
  VFNMADD213PHZ256mkz = 11353,
  VFNMADD213PHZ256r = 11354,
  VFNMADD213PHZ256rk = 11355,
  VFNMADD213PHZ256rkz = 11356,
  VFNMADD213PHZm = 11357,
  VFNMADD213PHZmb = 11358,
  VFNMADD213PHZmbk = 11359,
  VFNMADD213PHZmbkz = 11360,
  VFNMADD213PHZmk = 11361,
  VFNMADD213PHZmkz = 11362,
  VFNMADD213PHZr = 11363,
  VFNMADD213PHZrb = 11364,
  VFNMADD213PHZrbk = 11365,
  VFNMADD213PHZrbkz = 11366,
  VFNMADD213PHZrk = 11367,
  VFNMADD213PHZrkz = 11368,
  VFNMADD213PSYm = 11369,
  VFNMADD213PSYr = 11370,
  VFNMADD213PSZ128m = 11371,
  VFNMADD213PSZ128mb = 11372,
  VFNMADD213PSZ128mbk = 11373,
  VFNMADD213PSZ128mbkz = 11374,
  VFNMADD213PSZ128mk = 11375,
  VFNMADD213PSZ128mkz = 11376,
  VFNMADD213PSZ128r = 11377,
  VFNMADD213PSZ128rk = 11378,
  VFNMADD213PSZ128rkz = 11379,
  VFNMADD213PSZ256m = 11380,
  VFNMADD213PSZ256mb = 11381,
  VFNMADD213PSZ256mbk = 11382,
  VFNMADD213PSZ256mbkz = 11383,
  VFNMADD213PSZ256mk = 11384,
  VFNMADD213PSZ256mkz = 11385,
  VFNMADD213PSZ256r = 11386,
  VFNMADD213PSZ256rk = 11387,
  VFNMADD213PSZ256rkz = 11388,
  VFNMADD213PSZm = 11389,
  VFNMADD213PSZmb = 11390,
  VFNMADD213PSZmbk = 11391,
  VFNMADD213PSZmbkz = 11392,
  VFNMADD213PSZmk = 11393,
  VFNMADD213PSZmkz = 11394,
  VFNMADD213PSZr = 11395,
  VFNMADD213PSZrb = 11396,
  VFNMADD213PSZrbk = 11397,
  VFNMADD213PSZrbkz = 11398,
  VFNMADD213PSZrk = 11399,
  VFNMADD213PSZrkz = 11400,
  VFNMADD213PSm = 11401,
  VFNMADD213PSr = 11402,
  VFNMADD213SDZm = 11403,
  VFNMADD213SDZm_Int = 11404,
  VFNMADD213SDZmk_Int = 11405,
  VFNMADD213SDZmkz_Int = 11406,
  VFNMADD213SDZr = 11407,
  VFNMADD213SDZr_Int = 11408,
  VFNMADD213SDZrb = 11409,
  VFNMADD213SDZrb_Int = 11410,
  VFNMADD213SDZrbk_Int = 11411,
  VFNMADD213SDZrbkz_Int = 11412,
  VFNMADD213SDZrk_Int = 11413,
  VFNMADD213SDZrkz_Int = 11414,
  VFNMADD213SDm = 11415,
  VFNMADD213SDm_Int = 11416,
  VFNMADD213SDr = 11417,
  VFNMADD213SDr_Int = 11418,
  VFNMADD213SHZm = 11419,
  VFNMADD213SHZm_Int = 11420,
  VFNMADD213SHZmk_Int = 11421,
  VFNMADD213SHZmkz_Int = 11422,
  VFNMADD213SHZr = 11423,
  VFNMADD213SHZr_Int = 11424,
  VFNMADD213SHZrb = 11425,
  VFNMADD213SHZrb_Int = 11426,
  VFNMADD213SHZrbk_Int = 11427,
  VFNMADD213SHZrbkz_Int = 11428,
  VFNMADD213SHZrk_Int = 11429,
  VFNMADD213SHZrkz_Int = 11430,
  VFNMADD213SSZm = 11431,
  VFNMADD213SSZm_Int = 11432,
  VFNMADD213SSZmk_Int = 11433,
  VFNMADD213SSZmkz_Int = 11434,
  VFNMADD213SSZr = 11435,
  VFNMADD213SSZr_Int = 11436,
  VFNMADD213SSZrb = 11437,
  VFNMADD213SSZrb_Int = 11438,
  VFNMADD213SSZrbk_Int = 11439,
  VFNMADD213SSZrbkz_Int = 11440,
  VFNMADD213SSZrk_Int = 11441,
  VFNMADD213SSZrkz_Int = 11442,
  VFNMADD213SSm = 11443,
  VFNMADD213SSm_Int = 11444,
  VFNMADD213SSr = 11445,
  VFNMADD213SSr_Int = 11446,
  VFNMADD231BF16Z128m = 11447,
  VFNMADD231BF16Z128mb = 11448,
  VFNMADD231BF16Z128mbk = 11449,
  VFNMADD231BF16Z128mbkz = 11450,
  VFNMADD231BF16Z128mk = 11451,
  VFNMADD231BF16Z128mkz = 11452,
  VFNMADD231BF16Z128r = 11453,
  VFNMADD231BF16Z128rk = 11454,
  VFNMADD231BF16Z128rkz = 11455,
  VFNMADD231BF16Z256m = 11456,
  VFNMADD231BF16Z256mb = 11457,
  VFNMADD231BF16Z256mbk = 11458,
  VFNMADD231BF16Z256mbkz = 11459,
  VFNMADD231BF16Z256mk = 11460,
  VFNMADD231BF16Z256mkz = 11461,
  VFNMADD231BF16Z256r = 11462,
  VFNMADD231BF16Z256rk = 11463,
  VFNMADD231BF16Z256rkz = 11464,
  VFNMADD231BF16Zm = 11465,
  VFNMADD231BF16Zmb = 11466,
  VFNMADD231BF16Zmbk = 11467,
  VFNMADD231BF16Zmbkz = 11468,
  VFNMADD231BF16Zmk = 11469,
  VFNMADD231BF16Zmkz = 11470,
  VFNMADD231BF16Zr = 11471,
  VFNMADD231BF16Zrk = 11472,
  VFNMADD231BF16Zrkz = 11473,
  VFNMADD231PDYm = 11474,
  VFNMADD231PDYr = 11475,
  VFNMADD231PDZ128m = 11476,
  VFNMADD231PDZ128mb = 11477,
  VFNMADD231PDZ128mbk = 11478,
  VFNMADD231PDZ128mbkz = 11479,
  VFNMADD231PDZ128mk = 11480,
  VFNMADD231PDZ128mkz = 11481,
  VFNMADD231PDZ128r = 11482,
  VFNMADD231PDZ128rk = 11483,
  VFNMADD231PDZ128rkz = 11484,
  VFNMADD231PDZ256m = 11485,
  VFNMADD231PDZ256mb = 11486,
  VFNMADD231PDZ256mbk = 11487,
  VFNMADD231PDZ256mbkz = 11488,
  VFNMADD231PDZ256mk = 11489,
  VFNMADD231PDZ256mkz = 11490,
  VFNMADD231PDZ256r = 11491,
  VFNMADD231PDZ256rk = 11492,
  VFNMADD231PDZ256rkz = 11493,
  VFNMADD231PDZm = 11494,
  VFNMADD231PDZmb = 11495,
  VFNMADD231PDZmbk = 11496,
  VFNMADD231PDZmbkz = 11497,
  VFNMADD231PDZmk = 11498,
  VFNMADD231PDZmkz = 11499,
  VFNMADD231PDZr = 11500,
  VFNMADD231PDZrb = 11501,
  VFNMADD231PDZrbk = 11502,
  VFNMADD231PDZrbkz = 11503,
  VFNMADD231PDZrk = 11504,
  VFNMADD231PDZrkz = 11505,
  VFNMADD231PDm = 11506,
  VFNMADD231PDr = 11507,
  VFNMADD231PHZ128m = 11508,
  VFNMADD231PHZ128mb = 11509,
  VFNMADD231PHZ128mbk = 11510,
  VFNMADD231PHZ128mbkz = 11511,
  VFNMADD231PHZ128mk = 11512,
  VFNMADD231PHZ128mkz = 11513,
  VFNMADD231PHZ128r = 11514,
  VFNMADD231PHZ128rk = 11515,
  VFNMADD231PHZ128rkz = 11516,
  VFNMADD231PHZ256m = 11517,
  VFNMADD231PHZ256mb = 11518,
  VFNMADD231PHZ256mbk = 11519,
  VFNMADD231PHZ256mbkz = 11520,
  VFNMADD231PHZ256mk = 11521,
  VFNMADD231PHZ256mkz = 11522,
  VFNMADD231PHZ256r = 11523,
  VFNMADD231PHZ256rk = 11524,
  VFNMADD231PHZ256rkz = 11525,
  VFNMADD231PHZm = 11526,
  VFNMADD231PHZmb = 11527,
  VFNMADD231PHZmbk = 11528,
  VFNMADD231PHZmbkz = 11529,
  VFNMADD231PHZmk = 11530,
  VFNMADD231PHZmkz = 11531,
  VFNMADD231PHZr = 11532,
  VFNMADD231PHZrb = 11533,
  VFNMADD231PHZrbk = 11534,
  VFNMADD231PHZrbkz = 11535,
  VFNMADD231PHZrk = 11536,
  VFNMADD231PHZrkz = 11537,
  VFNMADD231PSYm = 11538,
  VFNMADD231PSYr = 11539,
  VFNMADD231PSZ128m = 11540,
  VFNMADD231PSZ128mb = 11541,
  VFNMADD231PSZ128mbk = 11542,
  VFNMADD231PSZ128mbkz = 11543,
  VFNMADD231PSZ128mk = 11544,
  VFNMADD231PSZ128mkz = 11545,
  VFNMADD231PSZ128r = 11546,
  VFNMADD231PSZ128rk = 11547,
  VFNMADD231PSZ128rkz = 11548,
  VFNMADD231PSZ256m = 11549,
  VFNMADD231PSZ256mb = 11550,
  VFNMADD231PSZ256mbk = 11551,
  VFNMADD231PSZ256mbkz = 11552,
  VFNMADD231PSZ256mk = 11553,
  VFNMADD231PSZ256mkz = 11554,
  VFNMADD231PSZ256r = 11555,
  VFNMADD231PSZ256rk = 11556,
  VFNMADD231PSZ256rkz = 11557,
  VFNMADD231PSZm = 11558,
  VFNMADD231PSZmb = 11559,
  VFNMADD231PSZmbk = 11560,
  VFNMADD231PSZmbkz = 11561,
  VFNMADD231PSZmk = 11562,
  VFNMADD231PSZmkz = 11563,
  VFNMADD231PSZr = 11564,
  VFNMADD231PSZrb = 11565,
  VFNMADD231PSZrbk = 11566,
  VFNMADD231PSZrbkz = 11567,
  VFNMADD231PSZrk = 11568,
  VFNMADD231PSZrkz = 11569,
  VFNMADD231PSm = 11570,
  VFNMADD231PSr = 11571,
  VFNMADD231SDZm = 11572,
  VFNMADD231SDZm_Int = 11573,
  VFNMADD231SDZmk_Int = 11574,
  VFNMADD231SDZmkz_Int = 11575,
  VFNMADD231SDZr = 11576,
  VFNMADD231SDZr_Int = 11577,
  VFNMADD231SDZrb = 11578,
  VFNMADD231SDZrb_Int = 11579,
  VFNMADD231SDZrbk_Int = 11580,
  VFNMADD231SDZrbkz_Int = 11581,
  VFNMADD231SDZrk_Int = 11582,
  VFNMADD231SDZrkz_Int = 11583,
  VFNMADD231SDm = 11584,
  VFNMADD231SDm_Int = 11585,
  VFNMADD231SDr = 11586,
  VFNMADD231SDr_Int = 11587,
  VFNMADD231SHZm = 11588,
  VFNMADD231SHZm_Int = 11589,
  VFNMADD231SHZmk_Int = 11590,
  VFNMADD231SHZmkz_Int = 11591,
  VFNMADD231SHZr = 11592,
  VFNMADD231SHZr_Int = 11593,
  VFNMADD231SHZrb = 11594,
  VFNMADD231SHZrb_Int = 11595,
  VFNMADD231SHZrbk_Int = 11596,
  VFNMADD231SHZrbkz_Int = 11597,
  VFNMADD231SHZrk_Int = 11598,
  VFNMADD231SHZrkz_Int = 11599,
  VFNMADD231SSZm = 11600,
  VFNMADD231SSZm_Int = 11601,
  VFNMADD231SSZmk_Int = 11602,
  VFNMADD231SSZmkz_Int = 11603,
  VFNMADD231SSZr = 11604,
  VFNMADD231SSZr_Int = 11605,
  VFNMADD231SSZrb = 11606,
  VFNMADD231SSZrb_Int = 11607,
  VFNMADD231SSZrbk_Int = 11608,
  VFNMADD231SSZrbkz_Int = 11609,
  VFNMADD231SSZrk_Int = 11610,
  VFNMADD231SSZrkz_Int = 11611,
  VFNMADD231SSm = 11612,
  VFNMADD231SSm_Int = 11613,
  VFNMADD231SSr = 11614,
  VFNMADD231SSr_Int = 11615,
  VFNMADDPD4Ymr = 11616,
  VFNMADDPD4Yrm = 11617,
  VFNMADDPD4Yrr = 11618,
  VFNMADDPD4Yrr_REV = 11619,
  VFNMADDPD4mr = 11620,
  VFNMADDPD4rm = 11621,
  VFNMADDPD4rr = 11622,
  VFNMADDPD4rr_REV = 11623,
  VFNMADDPS4Ymr = 11624,
  VFNMADDPS4Yrm = 11625,
  VFNMADDPS4Yrr = 11626,
  VFNMADDPS4Yrr_REV = 11627,
  VFNMADDPS4mr = 11628,
  VFNMADDPS4rm = 11629,
  VFNMADDPS4rr = 11630,
  VFNMADDPS4rr_REV = 11631,
  VFNMADDSD4mr = 11632,
  VFNMADDSD4mr_Int = 11633,
  VFNMADDSD4rm = 11634,
  VFNMADDSD4rm_Int = 11635,
  VFNMADDSD4rr = 11636,
  VFNMADDSD4rr_Int = 11637,
  VFNMADDSD4rr_Int_REV = 11638,
  VFNMADDSD4rr_REV = 11639,
  VFNMADDSS4mr = 11640,
  VFNMADDSS4mr_Int = 11641,
  VFNMADDSS4rm = 11642,
  VFNMADDSS4rm_Int = 11643,
  VFNMADDSS4rr = 11644,
  VFNMADDSS4rr_Int = 11645,
  VFNMADDSS4rr_Int_REV = 11646,
  VFNMADDSS4rr_REV = 11647,
  VFNMSUB132BF16Z128m = 11648,
  VFNMSUB132BF16Z128mb = 11649,
  VFNMSUB132BF16Z128mbk = 11650,
  VFNMSUB132BF16Z128mbkz = 11651,
  VFNMSUB132BF16Z128mk = 11652,
  VFNMSUB132BF16Z128mkz = 11653,
  VFNMSUB132BF16Z128r = 11654,
  VFNMSUB132BF16Z128rk = 11655,
  VFNMSUB132BF16Z128rkz = 11656,
  VFNMSUB132BF16Z256m = 11657,
  VFNMSUB132BF16Z256mb = 11658,
  VFNMSUB132BF16Z256mbk = 11659,
  VFNMSUB132BF16Z256mbkz = 11660,
  VFNMSUB132BF16Z256mk = 11661,
  VFNMSUB132BF16Z256mkz = 11662,
  VFNMSUB132BF16Z256r = 11663,
  VFNMSUB132BF16Z256rk = 11664,
  VFNMSUB132BF16Z256rkz = 11665,
  VFNMSUB132BF16Zm = 11666,
  VFNMSUB132BF16Zmb = 11667,
  VFNMSUB132BF16Zmbk = 11668,
  VFNMSUB132BF16Zmbkz = 11669,
  VFNMSUB132BF16Zmk = 11670,
  VFNMSUB132BF16Zmkz = 11671,
  VFNMSUB132BF16Zr = 11672,
  VFNMSUB132BF16Zrk = 11673,
  VFNMSUB132BF16Zrkz = 11674,
  VFNMSUB132PDYm = 11675,
  VFNMSUB132PDYr = 11676,
  VFNMSUB132PDZ128m = 11677,
  VFNMSUB132PDZ128mb = 11678,
  VFNMSUB132PDZ128mbk = 11679,
  VFNMSUB132PDZ128mbkz = 11680,
  VFNMSUB132PDZ128mk = 11681,
  VFNMSUB132PDZ128mkz = 11682,
  VFNMSUB132PDZ128r = 11683,
  VFNMSUB132PDZ128rk = 11684,
  VFNMSUB132PDZ128rkz = 11685,
  VFNMSUB132PDZ256m = 11686,
  VFNMSUB132PDZ256mb = 11687,
  VFNMSUB132PDZ256mbk = 11688,
  VFNMSUB132PDZ256mbkz = 11689,
  VFNMSUB132PDZ256mk = 11690,
  VFNMSUB132PDZ256mkz = 11691,
  VFNMSUB132PDZ256r = 11692,
  VFNMSUB132PDZ256rk = 11693,
  VFNMSUB132PDZ256rkz = 11694,
  VFNMSUB132PDZm = 11695,
  VFNMSUB132PDZmb = 11696,
  VFNMSUB132PDZmbk = 11697,
  VFNMSUB132PDZmbkz = 11698,
  VFNMSUB132PDZmk = 11699,
  VFNMSUB132PDZmkz = 11700,
  VFNMSUB132PDZr = 11701,
  VFNMSUB132PDZrb = 11702,
  VFNMSUB132PDZrbk = 11703,
  VFNMSUB132PDZrbkz = 11704,
  VFNMSUB132PDZrk = 11705,
  VFNMSUB132PDZrkz = 11706,
  VFNMSUB132PDm = 11707,
  VFNMSUB132PDr = 11708,
  VFNMSUB132PHZ128m = 11709,
  VFNMSUB132PHZ128mb = 11710,
  VFNMSUB132PHZ128mbk = 11711,
  VFNMSUB132PHZ128mbkz = 11712,
  VFNMSUB132PHZ128mk = 11713,
  VFNMSUB132PHZ128mkz = 11714,
  VFNMSUB132PHZ128r = 11715,
  VFNMSUB132PHZ128rk = 11716,
  VFNMSUB132PHZ128rkz = 11717,
  VFNMSUB132PHZ256m = 11718,
  VFNMSUB132PHZ256mb = 11719,
  VFNMSUB132PHZ256mbk = 11720,
  VFNMSUB132PHZ256mbkz = 11721,
  VFNMSUB132PHZ256mk = 11722,
  VFNMSUB132PHZ256mkz = 11723,
  VFNMSUB132PHZ256r = 11724,
  VFNMSUB132PHZ256rk = 11725,
  VFNMSUB132PHZ256rkz = 11726,
  VFNMSUB132PHZm = 11727,
  VFNMSUB132PHZmb = 11728,
  VFNMSUB132PHZmbk = 11729,
  VFNMSUB132PHZmbkz = 11730,
  VFNMSUB132PHZmk = 11731,
  VFNMSUB132PHZmkz = 11732,
  VFNMSUB132PHZr = 11733,
  VFNMSUB132PHZrb = 11734,
  VFNMSUB132PHZrbk = 11735,
  VFNMSUB132PHZrbkz = 11736,
  VFNMSUB132PHZrk = 11737,
  VFNMSUB132PHZrkz = 11738,
  VFNMSUB132PSYm = 11739,
  VFNMSUB132PSYr = 11740,
  VFNMSUB132PSZ128m = 11741,
  VFNMSUB132PSZ128mb = 11742,
  VFNMSUB132PSZ128mbk = 11743,
  VFNMSUB132PSZ128mbkz = 11744,
  VFNMSUB132PSZ128mk = 11745,
  VFNMSUB132PSZ128mkz = 11746,
  VFNMSUB132PSZ128r = 11747,
  VFNMSUB132PSZ128rk = 11748,
  VFNMSUB132PSZ128rkz = 11749,
  VFNMSUB132PSZ256m = 11750,
  VFNMSUB132PSZ256mb = 11751,
  VFNMSUB132PSZ256mbk = 11752,
  VFNMSUB132PSZ256mbkz = 11753,
  VFNMSUB132PSZ256mk = 11754,
  VFNMSUB132PSZ256mkz = 11755,
  VFNMSUB132PSZ256r = 11756,
  VFNMSUB132PSZ256rk = 11757,
  VFNMSUB132PSZ256rkz = 11758,
  VFNMSUB132PSZm = 11759,
  VFNMSUB132PSZmb = 11760,
  VFNMSUB132PSZmbk = 11761,
  VFNMSUB132PSZmbkz = 11762,
  VFNMSUB132PSZmk = 11763,
  VFNMSUB132PSZmkz = 11764,
  VFNMSUB132PSZr = 11765,
  VFNMSUB132PSZrb = 11766,
  VFNMSUB132PSZrbk = 11767,
  VFNMSUB132PSZrbkz = 11768,
  VFNMSUB132PSZrk = 11769,
  VFNMSUB132PSZrkz = 11770,
  VFNMSUB132PSm = 11771,
  VFNMSUB132PSr = 11772,
  VFNMSUB132SDZm = 11773,
  VFNMSUB132SDZm_Int = 11774,
  VFNMSUB132SDZmk_Int = 11775,
  VFNMSUB132SDZmkz_Int = 11776,
  VFNMSUB132SDZr = 11777,
  VFNMSUB132SDZr_Int = 11778,
  VFNMSUB132SDZrb = 11779,
  VFNMSUB132SDZrb_Int = 11780,
  VFNMSUB132SDZrbk_Int = 11781,
  VFNMSUB132SDZrbkz_Int = 11782,
  VFNMSUB132SDZrk_Int = 11783,
  VFNMSUB132SDZrkz_Int = 11784,
  VFNMSUB132SDm = 11785,
  VFNMSUB132SDm_Int = 11786,
  VFNMSUB132SDr = 11787,
  VFNMSUB132SDr_Int = 11788,
  VFNMSUB132SHZm = 11789,
  VFNMSUB132SHZm_Int = 11790,
  VFNMSUB132SHZmk_Int = 11791,
  VFNMSUB132SHZmkz_Int = 11792,
  VFNMSUB132SHZr = 11793,
  VFNMSUB132SHZr_Int = 11794,
  VFNMSUB132SHZrb = 11795,
  VFNMSUB132SHZrb_Int = 11796,
  VFNMSUB132SHZrbk_Int = 11797,
  VFNMSUB132SHZrbkz_Int = 11798,
  VFNMSUB132SHZrk_Int = 11799,
  VFNMSUB132SHZrkz_Int = 11800,
  VFNMSUB132SSZm = 11801,
  VFNMSUB132SSZm_Int = 11802,
  VFNMSUB132SSZmk_Int = 11803,
  VFNMSUB132SSZmkz_Int = 11804,
  VFNMSUB132SSZr = 11805,
  VFNMSUB132SSZr_Int = 11806,
  VFNMSUB132SSZrb = 11807,
  VFNMSUB132SSZrb_Int = 11808,
  VFNMSUB132SSZrbk_Int = 11809,
  VFNMSUB132SSZrbkz_Int = 11810,
  VFNMSUB132SSZrk_Int = 11811,
  VFNMSUB132SSZrkz_Int = 11812,
  VFNMSUB132SSm = 11813,
  VFNMSUB132SSm_Int = 11814,
  VFNMSUB132SSr = 11815,
  VFNMSUB132SSr_Int = 11816,
  VFNMSUB213BF16Z128m = 11817,
  VFNMSUB213BF16Z128mb = 11818,
  VFNMSUB213BF16Z128mbk = 11819,
  VFNMSUB213BF16Z128mbkz = 11820,
  VFNMSUB213BF16Z128mk = 11821,
  VFNMSUB213BF16Z128mkz = 11822,
  VFNMSUB213BF16Z128r = 11823,
  VFNMSUB213BF16Z128rk = 11824,
  VFNMSUB213BF16Z128rkz = 11825,
  VFNMSUB213BF16Z256m = 11826,
  VFNMSUB213BF16Z256mb = 11827,
  VFNMSUB213BF16Z256mbk = 11828,
  VFNMSUB213BF16Z256mbkz = 11829,
  VFNMSUB213BF16Z256mk = 11830,
  VFNMSUB213BF16Z256mkz = 11831,
  VFNMSUB213BF16Z256r = 11832,
  VFNMSUB213BF16Z256rk = 11833,
  VFNMSUB213BF16Z256rkz = 11834,
  VFNMSUB213BF16Zm = 11835,
  VFNMSUB213BF16Zmb = 11836,
  VFNMSUB213BF16Zmbk = 11837,
  VFNMSUB213BF16Zmbkz = 11838,
  VFNMSUB213BF16Zmk = 11839,
  VFNMSUB213BF16Zmkz = 11840,
  VFNMSUB213BF16Zr = 11841,
  VFNMSUB213BF16Zrk = 11842,
  VFNMSUB213BF16Zrkz = 11843,
  VFNMSUB213PDYm = 11844,
  VFNMSUB213PDYr = 11845,
  VFNMSUB213PDZ128m = 11846,
  VFNMSUB213PDZ128mb = 11847,
  VFNMSUB213PDZ128mbk = 11848,
  VFNMSUB213PDZ128mbkz = 11849,
  VFNMSUB213PDZ128mk = 11850,
  VFNMSUB213PDZ128mkz = 11851,
  VFNMSUB213PDZ128r = 11852,
  VFNMSUB213PDZ128rk = 11853,
  VFNMSUB213PDZ128rkz = 11854,
  VFNMSUB213PDZ256m = 11855,
  VFNMSUB213PDZ256mb = 11856,
  VFNMSUB213PDZ256mbk = 11857,
  VFNMSUB213PDZ256mbkz = 11858,
  VFNMSUB213PDZ256mk = 11859,
  VFNMSUB213PDZ256mkz = 11860,
  VFNMSUB213PDZ256r = 11861,
  VFNMSUB213PDZ256rk = 11862,
  VFNMSUB213PDZ256rkz = 11863,
  VFNMSUB213PDZm = 11864,
  VFNMSUB213PDZmb = 11865,
  VFNMSUB213PDZmbk = 11866,
  VFNMSUB213PDZmbkz = 11867,
  VFNMSUB213PDZmk = 11868,
  VFNMSUB213PDZmkz = 11869,
  VFNMSUB213PDZr = 11870,
  VFNMSUB213PDZrb = 11871,
  VFNMSUB213PDZrbk = 11872,
  VFNMSUB213PDZrbkz = 11873,
  VFNMSUB213PDZrk = 11874,
  VFNMSUB213PDZrkz = 11875,
  VFNMSUB213PDm = 11876,
  VFNMSUB213PDr = 11877,
  VFNMSUB213PHZ128m = 11878,
  VFNMSUB213PHZ128mb = 11879,
  VFNMSUB213PHZ128mbk = 11880,
  VFNMSUB213PHZ128mbkz = 11881,
  VFNMSUB213PHZ128mk = 11882,
  VFNMSUB213PHZ128mkz = 11883,
  VFNMSUB213PHZ128r = 11884,
  VFNMSUB213PHZ128rk = 11885,
  VFNMSUB213PHZ128rkz = 11886,
  VFNMSUB213PHZ256m = 11887,
  VFNMSUB213PHZ256mb = 11888,
  VFNMSUB213PHZ256mbk = 11889,
  VFNMSUB213PHZ256mbkz = 11890,
  VFNMSUB213PHZ256mk = 11891,
  VFNMSUB213PHZ256mkz = 11892,
  VFNMSUB213PHZ256r = 11893,
  VFNMSUB213PHZ256rk = 11894,
  VFNMSUB213PHZ256rkz = 11895,
  VFNMSUB213PHZm = 11896,
  VFNMSUB213PHZmb = 11897,
  VFNMSUB213PHZmbk = 11898,
  VFNMSUB213PHZmbkz = 11899,
  VFNMSUB213PHZmk = 11900,
  VFNMSUB213PHZmkz = 11901,
  VFNMSUB213PHZr = 11902,
  VFNMSUB213PHZrb = 11903,
  VFNMSUB213PHZrbk = 11904,
  VFNMSUB213PHZrbkz = 11905,
  VFNMSUB213PHZrk = 11906,
  VFNMSUB213PHZrkz = 11907,
  VFNMSUB213PSYm = 11908,
  VFNMSUB213PSYr = 11909,
  VFNMSUB213PSZ128m = 11910,
  VFNMSUB213PSZ128mb = 11911,
  VFNMSUB213PSZ128mbk = 11912,
  VFNMSUB213PSZ128mbkz = 11913,
  VFNMSUB213PSZ128mk = 11914,
  VFNMSUB213PSZ128mkz = 11915,
  VFNMSUB213PSZ128r = 11916,
  VFNMSUB213PSZ128rk = 11917,
  VFNMSUB213PSZ128rkz = 11918,
  VFNMSUB213PSZ256m = 11919,
  VFNMSUB213PSZ256mb = 11920,
  VFNMSUB213PSZ256mbk = 11921,
  VFNMSUB213PSZ256mbkz = 11922,
  VFNMSUB213PSZ256mk = 11923,
  VFNMSUB213PSZ256mkz = 11924,
  VFNMSUB213PSZ256r = 11925,
  VFNMSUB213PSZ256rk = 11926,
  VFNMSUB213PSZ256rkz = 11927,
  VFNMSUB213PSZm = 11928,
  VFNMSUB213PSZmb = 11929,
  VFNMSUB213PSZmbk = 11930,
  VFNMSUB213PSZmbkz = 11931,
  VFNMSUB213PSZmk = 11932,
  VFNMSUB213PSZmkz = 11933,
  VFNMSUB213PSZr = 11934,
  VFNMSUB213PSZrb = 11935,
  VFNMSUB213PSZrbk = 11936,
  VFNMSUB213PSZrbkz = 11937,
  VFNMSUB213PSZrk = 11938,
  VFNMSUB213PSZrkz = 11939,
  VFNMSUB213PSm = 11940,
  VFNMSUB213PSr = 11941,
  VFNMSUB213SDZm = 11942,
  VFNMSUB213SDZm_Int = 11943,
  VFNMSUB213SDZmk_Int = 11944,
  VFNMSUB213SDZmkz_Int = 11945,
  VFNMSUB213SDZr = 11946,
  VFNMSUB213SDZr_Int = 11947,
  VFNMSUB213SDZrb = 11948,
  VFNMSUB213SDZrb_Int = 11949,
  VFNMSUB213SDZrbk_Int = 11950,
  VFNMSUB213SDZrbkz_Int = 11951,
  VFNMSUB213SDZrk_Int = 11952,
  VFNMSUB213SDZrkz_Int = 11953,
  VFNMSUB213SDm = 11954,
  VFNMSUB213SDm_Int = 11955,
  VFNMSUB213SDr = 11956,
  VFNMSUB213SDr_Int = 11957,
  VFNMSUB213SHZm = 11958,
  VFNMSUB213SHZm_Int = 11959,
  VFNMSUB213SHZmk_Int = 11960,
  VFNMSUB213SHZmkz_Int = 11961,
  VFNMSUB213SHZr = 11962,
  VFNMSUB213SHZr_Int = 11963,
  VFNMSUB213SHZrb = 11964,
  VFNMSUB213SHZrb_Int = 11965,
  VFNMSUB213SHZrbk_Int = 11966,
  VFNMSUB213SHZrbkz_Int = 11967,
  VFNMSUB213SHZrk_Int = 11968,
  VFNMSUB213SHZrkz_Int = 11969,
  VFNMSUB213SSZm = 11970,
  VFNMSUB213SSZm_Int = 11971,
  VFNMSUB213SSZmk_Int = 11972,
  VFNMSUB213SSZmkz_Int = 11973,
  VFNMSUB213SSZr = 11974,
  VFNMSUB213SSZr_Int = 11975,
  VFNMSUB213SSZrb = 11976,
  VFNMSUB213SSZrb_Int = 11977,
  VFNMSUB213SSZrbk_Int = 11978,
  VFNMSUB213SSZrbkz_Int = 11979,
  VFNMSUB213SSZrk_Int = 11980,
  VFNMSUB213SSZrkz_Int = 11981,
  VFNMSUB213SSm = 11982,
  VFNMSUB213SSm_Int = 11983,
  VFNMSUB213SSr = 11984,
  VFNMSUB213SSr_Int = 11985,
  VFNMSUB231BF16Z128m = 11986,
  VFNMSUB231BF16Z128mb = 11987,
  VFNMSUB231BF16Z128mbk = 11988,
  VFNMSUB231BF16Z128mbkz = 11989,
  VFNMSUB231BF16Z128mk = 11990,
  VFNMSUB231BF16Z128mkz = 11991,
  VFNMSUB231BF16Z128r = 11992,
  VFNMSUB231BF16Z128rk = 11993,
  VFNMSUB231BF16Z128rkz = 11994,
  VFNMSUB231BF16Z256m = 11995,
  VFNMSUB231BF16Z256mb = 11996,
  VFNMSUB231BF16Z256mbk = 11997,
  VFNMSUB231BF16Z256mbkz = 11998,
  VFNMSUB231BF16Z256mk = 11999,
  VFNMSUB231BF16Z256mkz = 12000,
  VFNMSUB231BF16Z256r = 12001,
  VFNMSUB231BF16Z256rk = 12002,
  VFNMSUB231BF16Z256rkz = 12003,
  VFNMSUB231BF16Zm = 12004,
  VFNMSUB231BF16Zmb = 12005,
  VFNMSUB231BF16Zmbk = 12006,
  VFNMSUB231BF16Zmbkz = 12007,
  VFNMSUB231BF16Zmk = 12008,
  VFNMSUB231BF16Zmkz = 12009,
  VFNMSUB231BF16Zr = 12010,
  VFNMSUB231BF16Zrk = 12011,
  VFNMSUB231BF16Zrkz = 12012,
  VFNMSUB231PDYm = 12013,
  VFNMSUB231PDYr = 12014,
  VFNMSUB231PDZ128m = 12015,
  VFNMSUB231PDZ128mb = 12016,
  VFNMSUB231PDZ128mbk = 12017,
  VFNMSUB231PDZ128mbkz = 12018,
  VFNMSUB231PDZ128mk = 12019,
  VFNMSUB231PDZ128mkz = 12020,
  VFNMSUB231PDZ128r = 12021,
  VFNMSUB231PDZ128rk = 12022,
  VFNMSUB231PDZ128rkz = 12023,
  VFNMSUB231PDZ256m = 12024,
  VFNMSUB231PDZ256mb = 12025,
  VFNMSUB231PDZ256mbk = 12026,
  VFNMSUB231PDZ256mbkz = 12027,
  VFNMSUB231PDZ256mk = 12028,
  VFNMSUB231PDZ256mkz = 12029,
  VFNMSUB231PDZ256r = 12030,
  VFNMSUB231PDZ256rk = 12031,
  VFNMSUB231PDZ256rkz = 12032,
  VFNMSUB231PDZm = 12033,
  VFNMSUB231PDZmb = 12034,
  VFNMSUB231PDZmbk = 12035,
  VFNMSUB231PDZmbkz = 12036,
  VFNMSUB231PDZmk = 12037,
  VFNMSUB231PDZmkz = 12038,
  VFNMSUB231PDZr = 12039,
  VFNMSUB231PDZrb = 12040,
  VFNMSUB231PDZrbk = 12041,
  VFNMSUB231PDZrbkz = 12042,
  VFNMSUB231PDZrk = 12043,
  VFNMSUB231PDZrkz = 12044,
  VFNMSUB231PDm = 12045,
  VFNMSUB231PDr = 12046,
  VFNMSUB231PHZ128m = 12047,
  VFNMSUB231PHZ128mb = 12048,
  VFNMSUB231PHZ128mbk = 12049,
  VFNMSUB231PHZ128mbkz = 12050,
  VFNMSUB231PHZ128mk = 12051,
  VFNMSUB231PHZ128mkz = 12052,
  VFNMSUB231PHZ128r = 12053,
  VFNMSUB231PHZ128rk = 12054,
  VFNMSUB231PHZ128rkz = 12055,
  VFNMSUB231PHZ256m = 12056,
  VFNMSUB231PHZ256mb = 12057,
  VFNMSUB231PHZ256mbk = 12058,
  VFNMSUB231PHZ256mbkz = 12059,
  VFNMSUB231PHZ256mk = 12060,
  VFNMSUB231PHZ256mkz = 12061,
  VFNMSUB231PHZ256r = 12062,
  VFNMSUB231PHZ256rk = 12063,
  VFNMSUB231PHZ256rkz = 12064,
  VFNMSUB231PHZm = 12065,
  VFNMSUB231PHZmb = 12066,
  VFNMSUB231PHZmbk = 12067,
  VFNMSUB231PHZmbkz = 12068,
  VFNMSUB231PHZmk = 12069,
  VFNMSUB231PHZmkz = 12070,
  VFNMSUB231PHZr = 12071,
  VFNMSUB231PHZrb = 12072,
  VFNMSUB231PHZrbk = 12073,
  VFNMSUB231PHZrbkz = 12074,
  VFNMSUB231PHZrk = 12075,
  VFNMSUB231PHZrkz = 12076,
  VFNMSUB231PSYm = 12077,
  VFNMSUB231PSYr = 12078,
  VFNMSUB231PSZ128m = 12079,
  VFNMSUB231PSZ128mb = 12080,
  VFNMSUB231PSZ128mbk = 12081,
  VFNMSUB231PSZ128mbkz = 12082,
  VFNMSUB231PSZ128mk = 12083,
  VFNMSUB231PSZ128mkz = 12084,
  VFNMSUB231PSZ128r = 12085,
  VFNMSUB231PSZ128rk = 12086,
  VFNMSUB231PSZ128rkz = 12087,
  VFNMSUB231PSZ256m = 12088,
  VFNMSUB231PSZ256mb = 12089,
  VFNMSUB231PSZ256mbk = 12090,
  VFNMSUB231PSZ256mbkz = 12091,
  VFNMSUB231PSZ256mk = 12092,
  VFNMSUB231PSZ256mkz = 12093,
  VFNMSUB231PSZ256r = 12094,
  VFNMSUB231PSZ256rk = 12095,
  VFNMSUB231PSZ256rkz = 12096,
  VFNMSUB231PSZm = 12097,
  VFNMSUB231PSZmb = 12098,
  VFNMSUB231PSZmbk = 12099,
  VFNMSUB231PSZmbkz = 12100,
  VFNMSUB231PSZmk = 12101,
  VFNMSUB231PSZmkz = 12102,
  VFNMSUB231PSZr = 12103,
  VFNMSUB231PSZrb = 12104,
  VFNMSUB231PSZrbk = 12105,
  VFNMSUB231PSZrbkz = 12106,
  VFNMSUB231PSZrk = 12107,
  VFNMSUB231PSZrkz = 12108,
  VFNMSUB231PSm = 12109,
  VFNMSUB231PSr = 12110,
  VFNMSUB231SDZm = 12111,
  VFNMSUB231SDZm_Int = 12112,
  VFNMSUB231SDZmk_Int = 12113,
  VFNMSUB231SDZmkz_Int = 12114,
  VFNMSUB231SDZr = 12115,
  VFNMSUB231SDZr_Int = 12116,
  VFNMSUB231SDZrb = 12117,
  VFNMSUB231SDZrb_Int = 12118,
  VFNMSUB231SDZrbk_Int = 12119,
  VFNMSUB231SDZrbkz_Int = 12120,
  VFNMSUB231SDZrk_Int = 12121,
  VFNMSUB231SDZrkz_Int = 12122,
  VFNMSUB231SDm = 12123,
  VFNMSUB231SDm_Int = 12124,
  VFNMSUB231SDr = 12125,
  VFNMSUB231SDr_Int = 12126,
  VFNMSUB231SHZm = 12127,
  VFNMSUB231SHZm_Int = 12128,
  VFNMSUB231SHZmk_Int = 12129,
  VFNMSUB231SHZmkz_Int = 12130,
  VFNMSUB231SHZr = 12131,
  VFNMSUB231SHZr_Int = 12132,
  VFNMSUB231SHZrb = 12133,
  VFNMSUB231SHZrb_Int = 12134,
  VFNMSUB231SHZrbk_Int = 12135,
  VFNMSUB231SHZrbkz_Int = 12136,
  VFNMSUB231SHZrk_Int = 12137,
  VFNMSUB231SHZrkz_Int = 12138,
  VFNMSUB231SSZm = 12139,
  VFNMSUB231SSZm_Int = 12140,
  VFNMSUB231SSZmk_Int = 12141,
  VFNMSUB231SSZmkz_Int = 12142,
  VFNMSUB231SSZr = 12143,
  VFNMSUB231SSZr_Int = 12144,
  VFNMSUB231SSZrb = 12145,
  VFNMSUB231SSZrb_Int = 12146,
  VFNMSUB231SSZrbk_Int = 12147,
  VFNMSUB231SSZrbkz_Int = 12148,
  VFNMSUB231SSZrk_Int = 12149,
  VFNMSUB231SSZrkz_Int = 12150,
  VFNMSUB231SSm = 12151,
  VFNMSUB231SSm_Int = 12152,
  VFNMSUB231SSr = 12153,
  VFNMSUB231SSr_Int = 12154,
  VFNMSUBPD4Ymr = 12155,
  VFNMSUBPD4Yrm = 12156,
  VFNMSUBPD4Yrr = 12157,
  VFNMSUBPD4Yrr_REV = 12158,
  VFNMSUBPD4mr = 12159,
  VFNMSUBPD4rm = 12160,
  VFNMSUBPD4rr = 12161,
  VFNMSUBPD4rr_REV = 12162,
  VFNMSUBPS4Ymr = 12163,
  VFNMSUBPS4Yrm = 12164,
  VFNMSUBPS4Yrr = 12165,
  VFNMSUBPS4Yrr_REV = 12166,
  VFNMSUBPS4mr = 12167,
  VFNMSUBPS4rm = 12168,
  VFNMSUBPS4rr = 12169,
  VFNMSUBPS4rr_REV = 12170,
  VFNMSUBSD4mr = 12171,
  VFNMSUBSD4mr_Int = 12172,
  VFNMSUBSD4rm = 12173,
  VFNMSUBSD4rm_Int = 12174,
  VFNMSUBSD4rr = 12175,
  VFNMSUBSD4rr_Int = 12176,
  VFNMSUBSD4rr_Int_REV = 12177,
  VFNMSUBSD4rr_REV = 12178,
  VFNMSUBSS4mr = 12179,
  VFNMSUBSS4mr_Int = 12180,
  VFNMSUBSS4rm = 12181,
  VFNMSUBSS4rm_Int = 12182,
  VFNMSUBSS4rr = 12183,
  VFNMSUBSS4rr_Int = 12184,
  VFNMSUBSS4rr_Int_REV = 12185,
  VFNMSUBSS4rr_REV = 12186,
  VFPCLASSBF16Z128mbi = 12187,
  VFPCLASSBF16Z128mbik = 12188,
  VFPCLASSBF16Z128mi = 12189,
  VFPCLASSBF16Z128mik = 12190,
  VFPCLASSBF16Z128ri = 12191,
  VFPCLASSBF16Z128rik = 12192,
  VFPCLASSBF16Z256mbi = 12193,
  VFPCLASSBF16Z256mbik = 12194,
  VFPCLASSBF16Z256mi = 12195,
  VFPCLASSBF16Z256mik = 12196,
  VFPCLASSBF16Z256ri = 12197,
  VFPCLASSBF16Z256rik = 12198,
  VFPCLASSBF16Zmbi = 12199,
  VFPCLASSBF16Zmbik = 12200,
  VFPCLASSBF16Zmi = 12201,
  VFPCLASSBF16Zmik = 12202,
  VFPCLASSBF16Zri = 12203,
  VFPCLASSBF16Zrik = 12204,
  VFPCLASSPDZ128mbi = 12205,
  VFPCLASSPDZ128mbik = 12206,
  VFPCLASSPDZ128mi = 12207,
  VFPCLASSPDZ128mik = 12208,
  VFPCLASSPDZ128ri = 12209,
  VFPCLASSPDZ128rik = 12210,
  VFPCLASSPDZ256mbi = 12211,
  VFPCLASSPDZ256mbik = 12212,
  VFPCLASSPDZ256mi = 12213,
  VFPCLASSPDZ256mik = 12214,
  VFPCLASSPDZ256ri = 12215,
  VFPCLASSPDZ256rik = 12216,
  VFPCLASSPDZmbi = 12217,
  VFPCLASSPDZmbik = 12218,
  VFPCLASSPDZmi = 12219,
  VFPCLASSPDZmik = 12220,
  VFPCLASSPDZri = 12221,
  VFPCLASSPDZrik = 12222,
  VFPCLASSPHZ128mbi = 12223,
  VFPCLASSPHZ128mbik = 12224,
  VFPCLASSPHZ128mi = 12225,
  VFPCLASSPHZ128mik = 12226,
  VFPCLASSPHZ128ri = 12227,
  VFPCLASSPHZ128rik = 12228,
  VFPCLASSPHZ256mbi = 12229,
  VFPCLASSPHZ256mbik = 12230,
  VFPCLASSPHZ256mi = 12231,
  VFPCLASSPHZ256mik = 12232,
  VFPCLASSPHZ256ri = 12233,
  VFPCLASSPHZ256rik = 12234,
  VFPCLASSPHZmbi = 12235,
  VFPCLASSPHZmbik = 12236,
  VFPCLASSPHZmi = 12237,
  VFPCLASSPHZmik = 12238,
  VFPCLASSPHZri = 12239,
  VFPCLASSPHZrik = 12240,
  VFPCLASSPSZ128mbi = 12241,
  VFPCLASSPSZ128mbik = 12242,
  VFPCLASSPSZ128mi = 12243,
  VFPCLASSPSZ128mik = 12244,
  VFPCLASSPSZ128ri = 12245,
  VFPCLASSPSZ128rik = 12246,
  VFPCLASSPSZ256mbi = 12247,
  VFPCLASSPSZ256mbik = 12248,
  VFPCLASSPSZ256mi = 12249,
  VFPCLASSPSZ256mik = 12250,
  VFPCLASSPSZ256ri = 12251,
  VFPCLASSPSZ256rik = 12252,
  VFPCLASSPSZmbi = 12253,
  VFPCLASSPSZmbik = 12254,
  VFPCLASSPSZmi = 12255,
  VFPCLASSPSZmik = 12256,
  VFPCLASSPSZri = 12257,
  VFPCLASSPSZrik = 12258,
  VFPCLASSSDZmi = 12259,
  VFPCLASSSDZmik = 12260,
  VFPCLASSSDZri = 12261,
  VFPCLASSSDZrik = 12262,
  VFPCLASSSHZmi = 12263,
  VFPCLASSSHZmik = 12264,
  VFPCLASSSHZri = 12265,
  VFPCLASSSHZrik = 12266,
  VFPCLASSSSZmi = 12267,
  VFPCLASSSSZmik = 12268,
  VFPCLASSSSZri = 12269,
  VFPCLASSSSZrik = 12270,
  VFRCZPDYrm = 12271,
  VFRCZPDYrr = 12272,
  VFRCZPDrm = 12273,
  VFRCZPDrr = 12274,
  VFRCZPSYrm = 12275,
  VFRCZPSYrr = 12276,
  VFRCZPSrm = 12277,
  VFRCZPSrr = 12278,
  VFRCZSDrm = 12279,
  VFRCZSDrr = 12280,
  VFRCZSSrm = 12281,
  VFRCZSSrr = 12282,
  VGATHERDPDYrm = 12283,
  VGATHERDPDZ128rm = 12284,
  VGATHERDPDZ256rm = 12285,
  VGATHERDPDZrm = 12286,
  VGATHERDPDrm = 12287,
  VGATHERDPSYrm = 12288,
  VGATHERDPSZ128rm = 12289,
  VGATHERDPSZ256rm = 12290,
  VGATHERDPSZrm = 12291,
  VGATHERDPSrm = 12292,
  VGATHERPF0DPDm = 12293,
  VGATHERPF0DPSm = 12294,
  VGATHERPF0QPDm = 12295,
  VGATHERPF0QPSm = 12296,
  VGATHERPF1DPDm = 12297,
  VGATHERPF1DPSm = 12298,
  VGATHERPF1QPDm = 12299,
  VGATHERPF1QPSm = 12300,
  VGATHERQPDYrm = 12301,
  VGATHERQPDZ128rm = 12302,
  VGATHERQPDZ256rm = 12303,
  VGATHERQPDZrm = 12304,
  VGATHERQPDrm = 12305,
  VGATHERQPSYrm = 12306,
  VGATHERQPSZ128rm = 12307,
  VGATHERQPSZ256rm = 12308,
  VGATHERQPSZrm = 12309,
  VGATHERQPSrm = 12310,
  VGETEXPBF16Z128m = 12311,
  VGETEXPBF16Z128mb = 12312,
  VGETEXPBF16Z128mbk = 12313,
  VGETEXPBF16Z128mbkz = 12314,
  VGETEXPBF16Z128mk = 12315,
  VGETEXPBF16Z128mkz = 12316,
  VGETEXPBF16Z128r = 12317,
  VGETEXPBF16Z128rk = 12318,
  VGETEXPBF16Z128rkz = 12319,
  VGETEXPBF16Z256m = 12320,
  VGETEXPBF16Z256mb = 12321,
  VGETEXPBF16Z256mbk = 12322,
  VGETEXPBF16Z256mbkz = 12323,
  VGETEXPBF16Z256mk = 12324,
  VGETEXPBF16Z256mkz = 12325,
  VGETEXPBF16Z256r = 12326,
  VGETEXPBF16Z256rk = 12327,
  VGETEXPBF16Z256rkz = 12328,
  VGETEXPBF16Zm = 12329,
  VGETEXPBF16Zmb = 12330,
  VGETEXPBF16Zmbk = 12331,
  VGETEXPBF16Zmbkz = 12332,
  VGETEXPBF16Zmk = 12333,
  VGETEXPBF16Zmkz = 12334,
  VGETEXPBF16Zr = 12335,
  VGETEXPBF16Zrk = 12336,
  VGETEXPBF16Zrkz = 12337,
  VGETEXPPDZ128m = 12338,
  VGETEXPPDZ128mb = 12339,
  VGETEXPPDZ128mbk = 12340,
  VGETEXPPDZ128mbkz = 12341,
  VGETEXPPDZ128mk = 12342,
  VGETEXPPDZ128mkz = 12343,
  VGETEXPPDZ128r = 12344,
  VGETEXPPDZ128rk = 12345,
  VGETEXPPDZ128rkz = 12346,
  VGETEXPPDZ256m = 12347,
  VGETEXPPDZ256mb = 12348,
  VGETEXPPDZ256mbk = 12349,
  VGETEXPPDZ256mbkz = 12350,
  VGETEXPPDZ256mk = 12351,
  VGETEXPPDZ256mkz = 12352,
  VGETEXPPDZ256r = 12353,
  VGETEXPPDZ256rk = 12354,
  VGETEXPPDZ256rkz = 12355,
  VGETEXPPDZm = 12356,
  VGETEXPPDZmb = 12357,
  VGETEXPPDZmbk = 12358,
  VGETEXPPDZmbkz = 12359,
  VGETEXPPDZmk = 12360,
  VGETEXPPDZmkz = 12361,
  VGETEXPPDZr = 12362,
  VGETEXPPDZrb = 12363,
  VGETEXPPDZrbk = 12364,
  VGETEXPPDZrbkz = 12365,
  VGETEXPPDZrk = 12366,
  VGETEXPPDZrkz = 12367,
  VGETEXPPHZ128m = 12368,
  VGETEXPPHZ128mb = 12369,
  VGETEXPPHZ128mbk = 12370,
  VGETEXPPHZ128mbkz = 12371,
  VGETEXPPHZ128mk = 12372,
  VGETEXPPHZ128mkz = 12373,
  VGETEXPPHZ128r = 12374,
  VGETEXPPHZ128rk = 12375,
  VGETEXPPHZ128rkz = 12376,
  VGETEXPPHZ256m = 12377,
  VGETEXPPHZ256mb = 12378,
  VGETEXPPHZ256mbk = 12379,
  VGETEXPPHZ256mbkz = 12380,
  VGETEXPPHZ256mk = 12381,
  VGETEXPPHZ256mkz = 12382,
  VGETEXPPHZ256r = 12383,
  VGETEXPPHZ256rk = 12384,
  VGETEXPPHZ256rkz = 12385,
  VGETEXPPHZm = 12386,
  VGETEXPPHZmb = 12387,
  VGETEXPPHZmbk = 12388,
  VGETEXPPHZmbkz = 12389,
  VGETEXPPHZmk = 12390,
  VGETEXPPHZmkz = 12391,
  VGETEXPPHZr = 12392,
  VGETEXPPHZrb = 12393,
  VGETEXPPHZrbk = 12394,
  VGETEXPPHZrbkz = 12395,
  VGETEXPPHZrk = 12396,
  VGETEXPPHZrkz = 12397,
  VGETEXPPSZ128m = 12398,
  VGETEXPPSZ128mb = 12399,
  VGETEXPPSZ128mbk = 12400,
  VGETEXPPSZ128mbkz = 12401,
  VGETEXPPSZ128mk = 12402,
  VGETEXPPSZ128mkz = 12403,
  VGETEXPPSZ128r = 12404,
  VGETEXPPSZ128rk = 12405,
  VGETEXPPSZ128rkz = 12406,
  VGETEXPPSZ256m = 12407,
  VGETEXPPSZ256mb = 12408,
  VGETEXPPSZ256mbk = 12409,
  VGETEXPPSZ256mbkz = 12410,
  VGETEXPPSZ256mk = 12411,
  VGETEXPPSZ256mkz = 12412,
  VGETEXPPSZ256r = 12413,
  VGETEXPPSZ256rk = 12414,
  VGETEXPPSZ256rkz = 12415,
  VGETEXPPSZm = 12416,
  VGETEXPPSZmb = 12417,
  VGETEXPPSZmbk = 12418,
  VGETEXPPSZmbkz = 12419,
  VGETEXPPSZmk = 12420,
  VGETEXPPSZmkz = 12421,
  VGETEXPPSZr = 12422,
  VGETEXPPSZrb = 12423,
  VGETEXPPSZrbk = 12424,
  VGETEXPPSZrbkz = 12425,
  VGETEXPPSZrk = 12426,
  VGETEXPPSZrkz = 12427,
  VGETEXPSDZm = 12428,
  VGETEXPSDZmk = 12429,
  VGETEXPSDZmkz = 12430,
  VGETEXPSDZr = 12431,
  VGETEXPSDZrb = 12432,
  VGETEXPSDZrbk = 12433,
  VGETEXPSDZrbkz = 12434,
  VGETEXPSDZrk = 12435,
  VGETEXPSDZrkz = 12436,
  VGETEXPSHZm = 12437,
  VGETEXPSHZmk = 12438,
  VGETEXPSHZmkz = 12439,
  VGETEXPSHZr = 12440,
  VGETEXPSHZrb = 12441,
  VGETEXPSHZrbk = 12442,
  VGETEXPSHZrbkz = 12443,
  VGETEXPSHZrk = 12444,
  VGETEXPSHZrkz = 12445,
  VGETEXPSSZm = 12446,
  VGETEXPSSZmk = 12447,
  VGETEXPSSZmkz = 12448,
  VGETEXPSSZr = 12449,
  VGETEXPSSZrb = 12450,
  VGETEXPSSZrbk = 12451,
  VGETEXPSSZrbkz = 12452,
  VGETEXPSSZrk = 12453,
  VGETEXPSSZrkz = 12454,
  VGETMANTBF16Z128rmbi = 12455,
  VGETMANTBF16Z128rmbik = 12456,
  VGETMANTBF16Z128rmbikz = 12457,
  VGETMANTBF16Z128rmi = 12458,
  VGETMANTBF16Z128rmik = 12459,
  VGETMANTBF16Z128rmikz = 12460,
  VGETMANTBF16Z128rri = 12461,
  VGETMANTBF16Z128rrik = 12462,
  VGETMANTBF16Z128rrikz = 12463,
  VGETMANTBF16Z256rmbi = 12464,
  VGETMANTBF16Z256rmbik = 12465,
  VGETMANTBF16Z256rmbikz = 12466,
  VGETMANTBF16Z256rmi = 12467,
  VGETMANTBF16Z256rmik = 12468,
  VGETMANTBF16Z256rmikz = 12469,
  VGETMANTBF16Z256rri = 12470,
  VGETMANTBF16Z256rrik = 12471,
  VGETMANTBF16Z256rrikz = 12472,
  VGETMANTBF16Zrmbi = 12473,
  VGETMANTBF16Zrmbik = 12474,
  VGETMANTBF16Zrmbikz = 12475,
  VGETMANTBF16Zrmi = 12476,
  VGETMANTBF16Zrmik = 12477,
  VGETMANTBF16Zrmikz = 12478,
  VGETMANTBF16Zrri = 12479,
  VGETMANTBF16Zrrik = 12480,
  VGETMANTBF16Zrrikz = 12481,
  VGETMANTPDZ128rmbi = 12482,
  VGETMANTPDZ128rmbik = 12483,
  VGETMANTPDZ128rmbikz = 12484,
  VGETMANTPDZ128rmi = 12485,
  VGETMANTPDZ128rmik = 12486,
  VGETMANTPDZ128rmikz = 12487,
  VGETMANTPDZ128rri = 12488,
  VGETMANTPDZ128rrik = 12489,
  VGETMANTPDZ128rrikz = 12490,
  VGETMANTPDZ256rmbi = 12491,
  VGETMANTPDZ256rmbik = 12492,
  VGETMANTPDZ256rmbikz = 12493,
  VGETMANTPDZ256rmi = 12494,
  VGETMANTPDZ256rmik = 12495,
  VGETMANTPDZ256rmikz = 12496,
  VGETMANTPDZ256rri = 12497,
  VGETMANTPDZ256rrik = 12498,
  VGETMANTPDZ256rrikz = 12499,
  VGETMANTPDZrmbi = 12500,
  VGETMANTPDZrmbik = 12501,
  VGETMANTPDZrmbikz = 12502,
  VGETMANTPDZrmi = 12503,
  VGETMANTPDZrmik = 12504,
  VGETMANTPDZrmikz = 12505,
  VGETMANTPDZrri = 12506,
  VGETMANTPDZrrib = 12507,
  VGETMANTPDZrribk = 12508,
  VGETMANTPDZrribkz = 12509,
  VGETMANTPDZrrik = 12510,
  VGETMANTPDZrrikz = 12511,
  VGETMANTPHZ128rmbi = 12512,
  VGETMANTPHZ128rmbik = 12513,
  VGETMANTPHZ128rmbikz = 12514,
  VGETMANTPHZ128rmi = 12515,
  VGETMANTPHZ128rmik = 12516,
  VGETMANTPHZ128rmikz = 12517,
  VGETMANTPHZ128rri = 12518,
  VGETMANTPHZ128rrik = 12519,
  VGETMANTPHZ128rrikz = 12520,
  VGETMANTPHZ256rmbi = 12521,
  VGETMANTPHZ256rmbik = 12522,
  VGETMANTPHZ256rmbikz = 12523,
  VGETMANTPHZ256rmi = 12524,
  VGETMANTPHZ256rmik = 12525,
  VGETMANTPHZ256rmikz = 12526,
  VGETMANTPHZ256rri = 12527,
  VGETMANTPHZ256rrik = 12528,
  VGETMANTPHZ256rrikz = 12529,
  VGETMANTPHZrmbi = 12530,
  VGETMANTPHZrmbik = 12531,
  VGETMANTPHZrmbikz = 12532,
  VGETMANTPHZrmi = 12533,
  VGETMANTPHZrmik = 12534,
  VGETMANTPHZrmikz = 12535,
  VGETMANTPHZrri = 12536,
  VGETMANTPHZrrib = 12537,
  VGETMANTPHZrribk = 12538,
  VGETMANTPHZrribkz = 12539,
  VGETMANTPHZrrik = 12540,
  VGETMANTPHZrrikz = 12541,
  VGETMANTPSZ128rmbi = 12542,
  VGETMANTPSZ128rmbik = 12543,
  VGETMANTPSZ128rmbikz = 12544,
  VGETMANTPSZ128rmi = 12545,
  VGETMANTPSZ128rmik = 12546,
  VGETMANTPSZ128rmikz = 12547,
  VGETMANTPSZ128rri = 12548,
  VGETMANTPSZ128rrik = 12549,
  VGETMANTPSZ128rrikz = 12550,
  VGETMANTPSZ256rmbi = 12551,
  VGETMANTPSZ256rmbik = 12552,
  VGETMANTPSZ256rmbikz = 12553,
  VGETMANTPSZ256rmi = 12554,
  VGETMANTPSZ256rmik = 12555,
  VGETMANTPSZ256rmikz = 12556,
  VGETMANTPSZ256rri = 12557,
  VGETMANTPSZ256rrik = 12558,
  VGETMANTPSZ256rrikz = 12559,
  VGETMANTPSZrmbi = 12560,
  VGETMANTPSZrmbik = 12561,
  VGETMANTPSZrmbikz = 12562,
  VGETMANTPSZrmi = 12563,
  VGETMANTPSZrmik = 12564,
  VGETMANTPSZrmikz = 12565,
  VGETMANTPSZrri = 12566,
  VGETMANTPSZrrib = 12567,
  VGETMANTPSZrribk = 12568,
  VGETMANTPSZrribkz = 12569,
  VGETMANTPSZrrik = 12570,
  VGETMANTPSZrrikz = 12571,
  VGETMANTSDZrmi = 12572,
  VGETMANTSDZrmik = 12573,
  VGETMANTSDZrmikz = 12574,
  VGETMANTSDZrri = 12575,
  VGETMANTSDZrrib = 12576,
  VGETMANTSDZrribk = 12577,
  VGETMANTSDZrribkz = 12578,
  VGETMANTSDZrrik = 12579,
  VGETMANTSDZrrikz = 12580,
  VGETMANTSHZrmi = 12581,
  VGETMANTSHZrmik = 12582,
  VGETMANTSHZrmikz = 12583,
  VGETMANTSHZrri = 12584,
  VGETMANTSHZrrib = 12585,
  VGETMANTSHZrribk = 12586,
  VGETMANTSHZrribkz = 12587,
  VGETMANTSHZrrik = 12588,
  VGETMANTSHZrrikz = 12589,
  VGETMANTSSZrmi = 12590,
  VGETMANTSSZrmik = 12591,
  VGETMANTSSZrmikz = 12592,
  VGETMANTSSZrri = 12593,
  VGETMANTSSZrrib = 12594,
  VGETMANTSSZrribk = 12595,
  VGETMANTSSZrribkz = 12596,
  VGETMANTSSZrrik = 12597,
  VGETMANTSSZrrikz = 12598,
  VGF2P8AFFINEINVQBYrmi = 12599,
  VGF2P8AFFINEINVQBYrri = 12600,
  VGF2P8AFFINEINVQBZ128rmbi = 12601,
  VGF2P8AFFINEINVQBZ128rmbik = 12602,
  VGF2P8AFFINEINVQBZ128rmbikz = 12603,
  VGF2P8AFFINEINVQBZ128rmi = 12604,
  VGF2P8AFFINEINVQBZ128rmik = 12605,
  VGF2P8AFFINEINVQBZ128rmikz = 12606,
  VGF2P8AFFINEINVQBZ128rri = 12607,
  VGF2P8AFFINEINVQBZ128rrik = 12608,
  VGF2P8AFFINEINVQBZ128rrikz = 12609,
  VGF2P8AFFINEINVQBZ256rmbi = 12610,
  VGF2P8AFFINEINVQBZ256rmbik = 12611,
  VGF2P8AFFINEINVQBZ256rmbikz = 12612,
  VGF2P8AFFINEINVQBZ256rmi = 12613,
  VGF2P8AFFINEINVQBZ256rmik = 12614,
  VGF2P8AFFINEINVQBZ256rmikz = 12615,
  VGF2P8AFFINEINVQBZ256rri = 12616,
  VGF2P8AFFINEINVQBZ256rrik = 12617,
  VGF2P8AFFINEINVQBZ256rrikz = 12618,
  VGF2P8AFFINEINVQBZrmbi = 12619,
  VGF2P8AFFINEINVQBZrmbik = 12620,
  VGF2P8AFFINEINVQBZrmbikz = 12621,
  VGF2P8AFFINEINVQBZrmi = 12622,
  VGF2P8AFFINEINVQBZrmik = 12623,
  VGF2P8AFFINEINVQBZrmikz = 12624,
  VGF2P8AFFINEINVQBZrri = 12625,
  VGF2P8AFFINEINVQBZrrik = 12626,
  VGF2P8AFFINEINVQBZrrikz = 12627,
  VGF2P8AFFINEINVQBrmi = 12628,
  VGF2P8AFFINEINVQBrri = 12629,
  VGF2P8AFFINEQBYrmi = 12630,
  VGF2P8AFFINEQBYrri = 12631,
  VGF2P8AFFINEQBZ128rmbi = 12632,
  VGF2P8AFFINEQBZ128rmbik = 12633,
  VGF2P8AFFINEQBZ128rmbikz = 12634,
  VGF2P8AFFINEQBZ128rmi = 12635,
  VGF2P8AFFINEQBZ128rmik = 12636,
  VGF2P8AFFINEQBZ128rmikz = 12637,
  VGF2P8AFFINEQBZ128rri = 12638,
  VGF2P8AFFINEQBZ128rrik = 12639,
  VGF2P8AFFINEQBZ128rrikz = 12640,
  VGF2P8AFFINEQBZ256rmbi = 12641,
  VGF2P8AFFINEQBZ256rmbik = 12642,
  VGF2P8AFFINEQBZ256rmbikz = 12643,
  VGF2P8AFFINEQBZ256rmi = 12644,
  VGF2P8AFFINEQBZ256rmik = 12645,
  VGF2P8AFFINEQBZ256rmikz = 12646,
  VGF2P8AFFINEQBZ256rri = 12647,
  VGF2P8AFFINEQBZ256rrik = 12648,
  VGF2P8AFFINEQBZ256rrikz = 12649,
  VGF2P8AFFINEQBZrmbi = 12650,
  VGF2P8AFFINEQBZrmbik = 12651,
  VGF2P8AFFINEQBZrmbikz = 12652,
  VGF2P8AFFINEQBZrmi = 12653,
  VGF2P8AFFINEQBZrmik = 12654,
  VGF2P8AFFINEQBZrmikz = 12655,
  VGF2P8AFFINEQBZrri = 12656,
  VGF2P8AFFINEQBZrrik = 12657,
  VGF2P8AFFINEQBZrrikz = 12658,
  VGF2P8AFFINEQBrmi = 12659,
  VGF2P8AFFINEQBrri = 12660,
  VGF2P8MULBYrm = 12661,
  VGF2P8MULBYrr = 12662,
  VGF2P8MULBZ128rm = 12663,
  VGF2P8MULBZ128rmk = 12664,
  VGF2P8MULBZ128rmkz = 12665,
  VGF2P8MULBZ128rr = 12666,
  VGF2P8MULBZ128rrk = 12667,
  VGF2P8MULBZ128rrkz = 12668,
  VGF2P8MULBZ256rm = 12669,
  VGF2P8MULBZ256rmk = 12670,
  VGF2P8MULBZ256rmkz = 12671,
  VGF2P8MULBZ256rr = 12672,
  VGF2P8MULBZ256rrk = 12673,
  VGF2P8MULBZ256rrkz = 12674,
  VGF2P8MULBZrm = 12675,
  VGF2P8MULBZrmk = 12676,
  VGF2P8MULBZrmkz = 12677,
  VGF2P8MULBZrr = 12678,
  VGF2P8MULBZrrk = 12679,
  VGF2P8MULBZrrkz = 12680,
  VGF2P8MULBrm = 12681,
  VGF2P8MULBrr = 12682,
  VHADDPDYrm = 12683,
  VHADDPDYrr = 12684,
  VHADDPDrm = 12685,
  VHADDPDrr = 12686,
  VHADDPSYrm = 12687,
  VHADDPSYrr = 12688,
  VHADDPSrm = 12689,
  VHADDPSrr = 12690,
  VHSUBPDYrm = 12691,
  VHSUBPDYrr = 12692,
  VHSUBPDrm = 12693,
  VHSUBPDrr = 12694,
  VHSUBPSYrm = 12695,
  VHSUBPSYrr = 12696,
  VHSUBPSrm = 12697,
  VHSUBPSrr = 12698,
  VINSERTF128rmi = 12699,
  VINSERTF128rri = 12700,
  VINSERTF32X4Z256rmi = 12701,
  VINSERTF32X4Z256rmik = 12702,
  VINSERTF32X4Z256rmikz = 12703,
  VINSERTF32X4Z256rri = 12704,
  VINSERTF32X4Z256rrik = 12705,
  VINSERTF32X4Z256rrikz = 12706,
  VINSERTF32X4Zrmi = 12707,
  VINSERTF32X4Zrmik = 12708,
  VINSERTF32X4Zrmikz = 12709,
  VINSERTF32X4Zrri = 12710,
  VINSERTF32X4Zrrik = 12711,
  VINSERTF32X4Zrrikz = 12712,
  VINSERTF32X8Zrmi = 12713,
  VINSERTF32X8Zrmik = 12714,
  VINSERTF32X8Zrmikz = 12715,
  VINSERTF32X8Zrri = 12716,
  VINSERTF32X8Zrrik = 12717,
  VINSERTF32X8Zrrikz = 12718,
  VINSERTF64X2Z256rmi = 12719,
  VINSERTF64X2Z256rmik = 12720,
  VINSERTF64X2Z256rmikz = 12721,
  VINSERTF64X2Z256rri = 12722,
  VINSERTF64X2Z256rrik = 12723,
  VINSERTF64X2Z256rrikz = 12724,
  VINSERTF64X2Zrmi = 12725,
  VINSERTF64X2Zrmik = 12726,
  VINSERTF64X2Zrmikz = 12727,
  VINSERTF64X2Zrri = 12728,
  VINSERTF64X2Zrrik = 12729,
  VINSERTF64X2Zrrikz = 12730,
  VINSERTF64X4Zrmi = 12731,
  VINSERTF64X4Zrmik = 12732,
  VINSERTF64X4Zrmikz = 12733,
  VINSERTF64X4Zrri = 12734,
  VINSERTF64X4Zrrik = 12735,
  VINSERTF64X4Zrrikz = 12736,
  VINSERTI128rmi = 12737,
  VINSERTI128rri = 12738,
  VINSERTI32X4Z256rmi = 12739,
  VINSERTI32X4Z256rmik = 12740,
  VINSERTI32X4Z256rmikz = 12741,
  VINSERTI32X4Z256rri = 12742,
  VINSERTI32X4Z256rrik = 12743,
  VINSERTI32X4Z256rrikz = 12744,
  VINSERTI32X4Zrmi = 12745,
  VINSERTI32X4Zrmik = 12746,
  VINSERTI32X4Zrmikz = 12747,
  VINSERTI32X4Zrri = 12748,
  VINSERTI32X4Zrrik = 12749,
  VINSERTI32X4Zrrikz = 12750,
  VINSERTI32X8Zrmi = 12751,
  VINSERTI32X8Zrmik = 12752,
  VINSERTI32X8Zrmikz = 12753,
  VINSERTI32X8Zrri = 12754,
  VINSERTI32X8Zrrik = 12755,
  VINSERTI32X8Zrrikz = 12756,
  VINSERTI64X2Z256rmi = 12757,
  VINSERTI64X2Z256rmik = 12758,
  VINSERTI64X2Z256rmikz = 12759,
  VINSERTI64X2Z256rri = 12760,
  VINSERTI64X2Z256rrik = 12761,
  VINSERTI64X2Z256rrikz = 12762,
  VINSERTI64X2Zrmi = 12763,
  VINSERTI64X2Zrmik = 12764,
  VINSERTI64X2Zrmikz = 12765,
  VINSERTI64X2Zrri = 12766,
  VINSERTI64X2Zrrik = 12767,
  VINSERTI64X2Zrrikz = 12768,
  VINSERTI64X4Zrmi = 12769,
  VINSERTI64X4Zrmik = 12770,
  VINSERTI64X4Zrmikz = 12771,
  VINSERTI64X4Zrri = 12772,
  VINSERTI64X4Zrrik = 12773,
  VINSERTI64X4Zrrikz = 12774,
  VINSERTPSZrmi = 12775,
  VINSERTPSZrri = 12776,
  VINSERTPSrmi = 12777,
  VINSERTPSrri = 12778,
  VLDDQUYrm = 12779,
  VLDDQUrm = 12780,
  VLDMXCSR = 12781,
  VMASKMOVDQU = 12782,
  VMASKMOVDQU64 = 12783,
  VMASKMOVPDYmr = 12784,
  VMASKMOVPDYrm = 12785,
  VMASKMOVPDmr = 12786,
  VMASKMOVPDrm = 12787,
  VMASKMOVPSYmr = 12788,
  VMASKMOVPSYrm = 12789,
  VMASKMOVPSmr = 12790,
  VMASKMOVPSrm = 12791,
  VMAXBF16Z128rm = 12792,
  VMAXBF16Z128rmb = 12793,
  VMAXBF16Z128rmbk = 12794,
  VMAXBF16Z128rmbkz = 12795,
  VMAXBF16Z128rmk = 12796,
  VMAXBF16Z128rmkz = 12797,
  VMAXBF16Z128rr = 12798,
  VMAXBF16Z128rrk = 12799,
  VMAXBF16Z128rrkz = 12800,
  VMAXBF16Z256rm = 12801,
  VMAXBF16Z256rmb = 12802,
  VMAXBF16Z256rmbk = 12803,
  VMAXBF16Z256rmbkz = 12804,
  VMAXBF16Z256rmk = 12805,
  VMAXBF16Z256rmkz = 12806,
  VMAXBF16Z256rr = 12807,
  VMAXBF16Z256rrk = 12808,
  VMAXBF16Z256rrkz = 12809,
  VMAXBF16Zrm = 12810,
  VMAXBF16Zrmb = 12811,
  VMAXBF16Zrmbk = 12812,
  VMAXBF16Zrmbkz = 12813,
  VMAXBF16Zrmk = 12814,
  VMAXBF16Zrmkz = 12815,
  VMAXBF16Zrr = 12816,
  VMAXBF16Zrrk = 12817,
  VMAXBF16Zrrkz = 12818,
  VMAXCPDYrm = 12819,
  VMAXCPDYrr = 12820,
  VMAXCPDZ128rm = 12821,
  VMAXCPDZ128rmb = 12822,
  VMAXCPDZ128rmbk = 12823,
  VMAXCPDZ128rmbkz = 12824,
  VMAXCPDZ128rmk = 12825,
  VMAXCPDZ128rmkz = 12826,
  VMAXCPDZ128rr = 12827,
  VMAXCPDZ128rrk = 12828,
  VMAXCPDZ128rrkz = 12829,
  VMAXCPDZ256rm = 12830,
  VMAXCPDZ256rmb = 12831,
  VMAXCPDZ256rmbk = 12832,
  VMAXCPDZ256rmbkz = 12833,
  VMAXCPDZ256rmk = 12834,
  VMAXCPDZ256rmkz = 12835,
  VMAXCPDZ256rr = 12836,
  VMAXCPDZ256rrk = 12837,
  VMAXCPDZ256rrkz = 12838,
  VMAXCPDZrm = 12839,
  VMAXCPDZrmb = 12840,
  VMAXCPDZrmbk = 12841,
  VMAXCPDZrmbkz = 12842,
  VMAXCPDZrmk = 12843,
  VMAXCPDZrmkz = 12844,
  VMAXCPDZrr = 12845,
  VMAXCPDZrrk = 12846,
  VMAXCPDZrrkz = 12847,
  VMAXCPDrm = 12848,
  VMAXCPDrr = 12849,
  VMAXCPHZ128rm = 12850,
  VMAXCPHZ128rmb = 12851,
  VMAXCPHZ128rmbk = 12852,
  VMAXCPHZ128rmbkz = 12853,
  VMAXCPHZ128rmk = 12854,
  VMAXCPHZ128rmkz = 12855,
  VMAXCPHZ128rr = 12856,
  VMAXCPHZ128rrk = 12857,
  VMAXCPHZ128rrkz = 12858,
  VMAXCPHZ256rm = 12859,
  VMAXCPHZ256rmb = 12860,
  VMAXCPHZ256rmbk = 12861,
  VMAXCPHZ256rmbkz = 12862,
  VMAXCPHZ256rmk = 12863,
  VMAXCPHZ256rmkz = 12864,
  VMAXCPHZ256rr = 12865,
  VMAXCPHZ256rrk = 12866,
  VMAXCPHZ256rrkz = 12867,
  VMAXCPHZrm = 12868,
  VMAXCPHZrmb = 12869,
  VMAXCPHZrmbk = 12870,
  VMAXCPHZrmbkz = 12871,
  VMAXCPHZrmk = 12872,
  VMAXCPHZrmkz = 12873,
  VMAXCPHZrr = 12874,
  VMAXCPHZrrk = 12875,
  VMAXCPHZrrkz = 12876,
  VMAXCPSYrm = 12877,
  VMAXCPSYrr = 12878,
  VMAXCPSZ128rm = 12879,
  VMAXCPSZ128rmb = 12880,
  VMAXCPSZ128rmbk = 12881,
  VMAXCPSZ128rmbkz = 12882,
  VMAXCPSZ128rmk = 12883,
  VMAXCPSZ128rmkz = 12884,
  VMAXCPSZ128rr = 12885,
  VMAXCPSZ128rrk = 12886,
  VMAXCPSZ128rrkz = 12887,
  VMAXCPSZ256rm = 12888,
  VMAXCPSZ256rmb = 12889,
  VMAXCPSZ256rmbk = 12890,
  VMAXCPSZ256rmbkz = 12891,
  VMAXCPSZ256rmk = 12892,
  VMAXCPSZ256rmkz = 12893,
  VMAXCPSZ256rr = 12894,
  VMAXCPSZ256rrk = 12895,
  VMAXCPSZ256rrkz = 12896,
  VMAXCPSZrm = 12897,
  VMAXCPSZrmb = 12898,
  VMAXCPSZrmbk = 12899,
  VMAXCPSZrmbkz = 12900,
  VMAXCPSZrmk = 12901,
  VMAXCPSZrmkz = 12902,
  VMAXCPSZrr = 12903,
  VMAXCPSZrrk = 12904,
  VMAXCPSZrrkz = 12905,
  VMAXCPSrm = 12906,
  VMAXCPSrr = 12907,
  VMAXCSDZrm = 12908,
  VMAXCSDZrr = 12909,
  VMAXCSDrm = 12910,
  VMAXCSDrr = 12911,
  VMAXCSHZrm = 12912,
  VMAXCSHZrr = 12913,
  VMAXCSSZrm = 12914,
  VMAXCSSZrr = 12915,
  VMAXCSSrm = 12916,
  VMAXCSSrr = 12917,
  VMAXPDYrm = 12918,
  VMAXPDYrr = 12919,
  VMAXPDZ128rm = 12920,
  VMAXPDZ128rmb = 12921,
  VMAXPDZ128rmbk = 12922,
  VMAXPDZ128rmbkz = 12923,
  VMAXPDZ128rmk = 12924,
  VMAXPDZ128rmkz = 12925,
  VMAXPDZ128rr = 12926,
  VMAXPDZ128rrk = 12927,
  VMAXPDZ128rrkz = 12928,
  VMAXPDZ256rm = 12929,
  VMAXPDZ256rmb = 12930,
  VMAXPDZ256rmbk = 12931,
  VMAXPDZ256rmbkz = 12932,
  VMAXPDZ256rmk = 12933,
  VMAXPDZ256rmkz = 12934,
  VMAXPDZ256rr = 12935,
  VMAXPDZ256rrk = 12936,
  VMAXPDZ256rrkz = 12937,
  VMAXPDZrm = 12938,
  VMAXPDZrmb = 12939,
  VMAXPDZrmbk = 12940,
  VMAXPDZrmbkz = 12941,
  VMAXPDZrmk = 12942,
  VMAXPDZrmkz = 12943,
  VMAXPDZrr = 12944,
  VMAXPDZrrb = 12945,
  VMAXPDZrrbk = 12946,
  VMAXPDZrrbkz = 12947,
  VMAXPDZrrk = 12948,
  VMAXPDZrrkz = 12949,
  VMAXPDrm = 12950,
  VMAXPDrr = 12951,
  VMAXPHZ128rm = 12952,
  VMAXPHZ128rmb = 12953,
  VMAXPHZ128rmbk = 12954,
  VMAXPHZ128rmbkz = 12955,
  VMAXPHZ128rmk = 12956,
  VMAXPHZ128rmkz = 12957,
  VMAXPHZ128rr = 12958,
  VMAXPHZ128rrk = 12959,
  VMAXPHZ128rrkz = 12960,
  VMAXPHZ256rm = 12961,
  VMAXPHZ256rmb = 12962,
  VMAXPHZ256rmbk = 12963,
  VMAXPHZ256rmbkz = 12964,
  VMAXPHZ256rmk = 12965,
  VMAXPHZ256rmkz = 12966,
  VMAXPHZ256rr = 12967,
  VMAXPHZ256rrk = 12968,
  VMAXPHZ256rrkz = 12969,
  VMAXPHZrm = 12970,
  VMAXPHZrmb = 12971,
  VMAXPHZrmbk = 12972,
  VMAXPHZrmbkz = 12973,
  VMAXPHZrmk = 12974,
  VMAXPHZrmkz = 12975,
  VMAXPHZrr = 12976,
  VMAXPHZrrb = 12977,
  VMAXPHZrrbk = 12978,
  VMAXPHZrrbkz = 12979,
  VMAXPHZrrk = 12980,
  VMAXPHZrrkz = 12981,
  VMAXPSYrm = 12982,
  VMAXPSYrr = 12983,
  VMAXPSZ128rm = 12984,
  VMAXPSZ128rmb = 12985,
  VMAXPSZ128rmbk = 12986,
  VMAXPSZ128rmbkz = 12987,
  VMAXPSZ128rmk = 12988,
  VMAXPSZ128rmkz = 12989,
  VMAXPSZ128rr = 12990,
  VMAXPSZ128rrk = 12991,
  VMAXPSZ128rrkz = 12992,
  VMAXPSZ256rm = 12993,
  VMAXPSZ256rmb = 12994,
  VMAXPSZ256rmbk = 12995,
  VMAXPSZ256rmbkz = 12996,
  VMAXPSZ256rmk = 12997,
  VMAXPSZ256rmkz = 12998,
  VMAXPSZ256rr = 12999,
  VMAXPSZ256rrk = 13000,
  VMAXPSZ256rrkz = 13001,
  VMAXPSZrm = 13002,
  VMAXPSZrmb = 13003,
  VMAXPSZrmbk = 13004,
  VMAXPSZrmbkz = 13005,
  VMAXPSZrmk = 13006,
  VMAXPSZrmkz = 13007,
  VMAXPSZrr = 13008,
  VMAXPSZrrb = 13009,
  VMAXPSZrrbk = 13010,
  VMAXPSZrrbkz = 13011,
  VMAXPSZrrk = 13012,
  VMAXPSZrrkz = 13013,
  VMAXPSrm = 13014,
  VMAXPSrr = 13015,
  VMAXSDZrm = 13016,
  VMAXSDZrm_Int = 13017,
  VMAXSDZrmk_Int = 13018,
  VMAXSDZrmkz_Int = 13019,
  VMAXSDZrr = 13020,
  VMAXSDZrr_Int = 13021,
  VMAXSDZrrb_Int = 13022,
  VMAXSDZrrbk_Int = 13023,
  VMAXSDZrrbkz_Int = 13024,
  VMAXSDZrrk_Int = 13025,
  VMAXSDZrrkz_Int = 13026,
  VMAXSDrm = 13027,
  VMAXSDrm_Int = 13028,
  VMAXSDrr = 13029,
  VMAXSDrr_Int = 13030,
  VMAXSHZrm = 13031,
  VMAXSHZrm_Int = 13032,
  VMAXSHZrmk_Int = 13033,
  VMAXSHZrmkz_Int = 13034,
  VMAXSHZrr = 13035,
  VMAXSHZrr_Int = 13036,
  VMAXSHZrrb_Int = 13037,
  VMAXSHZrrbk_Int = 13038,
  VMAXSHZrrbkz_Int = 13039,
  VMAXSHZrrk_Int = 13040,
  VMAXSHZrrkz_Int = 13041,
  VMAXSSZrm = 13042,
  VMAXSSZrm_Int = 13043,
  VMAXSSZrmk_Int = 13044,
  VMAXSSZrmkz_Int = 13045,
  VMAXSSZrr = 13046,
  VMAXSSZrr_Int = 13047,
  VMAXSSZrrb_Int = 13048,
  VMAXSSZrrbk_Int = 13049,
  VMAXSSZrrbkz_Int = 13050,
  VMAXSSZrrk_Int = 13051,
  VMAXSSZrrkz_Int = 13052,
  VMAXSSrm = 13053,
  VMAXSSrm_Int = 13054,
  VMAXSSrr = 13055,
  VMAXSSrr_Int = 13056,
  VMCALL = 13057,
  VMCLEARm = 13058,
  VMFUNC = 13059,
  VMINBF16Z128rm = 13060,
  VMINBF16Z128rmb = 13061,
  VMINBF16Z128rmbk = 13062,
  VMINBF16Z128rmbkz = 13063,
  VMINBF16Z128rmk = 13064,
  VMINBF16Z128rmkz = 13065,
  VMINBF16Z128rr = 13066,
  VMINBF16Z128rrk = 13067,
  VMINBF16Z128rrkz = 13068,
  VMINBF16Z256rm = 13069,
  VMINBF16Z256rmb = 13070,
  VMINBF16Z256rmbk = 13071,
  VMINBF16Z256rmbkz = 13072,
  VMINBF16Z256rmk = 13073,
  VMINBF16Z256rmkz = 13074,
  VMINBF16Z256rr = 13075,
  VMINBF16Z256rrk = 13076,
  VMINBF16Z256rrkz = 13077,
  VMINBF16Zrm = 13078,
  VMINBF16Zrmb = 13079,
  VMINBF16Zrmbk = 13080,
  VMINBF16Zrmbkz = 13081,
  VMINBF16Zrmk = 13082,
  VMINBF16Zrmkz = 13083,
  VMINBF16Zrr = 13084,
  VMINBF16Zrrk = 13085,
  VMINBF16Zrrkz = 13086,
  VMINCPDYrm = 13087,
  VMINCPDYrr = 13088,
  VMINCPDZ128rm = 13089,
  VMINCPDZ128rmb = 13090,
  VMINCPDZ128rmbk = 13091,
  VMINCPDZ128rmbkz = 13092,
  VMINCPDZ128rmk = 13093,
  VMINCPDZ128rmkz = 13094,
  VMINCPDZ128rr = 13095,
  VMINCPDZ128rrk = 13096,
  VMINCPDZ128rrkz = 13097,
  VMINCPDZ256rm = 13098,
  VMINCPDZ256rmb = 13099,
  VMINCPDZ256rmbk = 13100,
  VMINCPDZ256rmbkz = 13101,
  VMINCPDZ256rmk = 13102,
  VMINCPDZ256rmkz = 13103,
  VMINCPDZ256rr = 13104,
  VMINCPDZ256rrk = 13105,
  VMINCPDZ256rrkz = 13106,
  VMINCPDZrm = 13107,
  VMINCPDZrmb = 13108,
  VMINCPDZrmbk = 13109,
  VMINCPDZrmbkz = 13110,
  VMINCPDZrmk = 13111,
  VMINCPDZrmkz = 13112,
  VMINCPDZrr = 13113,
  VMINCPDZrrk = 13114,
  VMINCPDZrrkz = 13115,
  VMINCPDrm = 13116,
  VMINCPDrr = 13117,
  VMINCPHZ128rm = 13118,
  VMINCPHZ128rmb = 13119,
  VMINCPHZ128rmbk = 13120,
  VMINCPHZ128rmbkz = 13121,
  VMINCPHZ128rmk = 13122,
  VMINCPHZ128rmkz = 13123,
  VMINCPHZ128rr = 13124,
  VMINCPHZ128rrk = 13125,
  VMINCPHZ128rrkz = 13126,
  VMINCPHZ256rm = 13127,
  VMINCPHZ256rmb = 13128,
  VMINCPHZ256rmbk = 13129,
  VMINCPHZ256rmbkz = 13130,
  VMINCPHZ256rmk = 13131,
  VMINCPHZ256rmkz = 13132,
  VMINCPHZ256rr = 13133,
  VMINCPHZ256rrk = 13134,
  VMINCPHZ256rrkz = 13135,
  VMINCPHZrm = 13136,
  VMINCPHZrmb = 13137,
  VMINCPHZrmbk = 13138,
  VMINCPHZrmbkz = 13139,
  VMINCPHZrmk = 13140,
  VMINCPHZrmkz = 13141,
  VMINCPHZrr = 13142,
  VMINCPHZrrk = 13143,
  VMINCPHZrrkz = 13144,
  VMINCPSYrm = 13145,
  VMINCPSYrr = 13146,
  VMINCPSZ128rm = 13147,
  VMINCPSZ128rmb = 13148,
  VMINCPSZ128rmbk = 13149,
  VMINCPSZ128rmbkz = 13150,
  VMINCPSZ128rmk = 13151,
  VMINCPSZ128rmkz = 13152,
  VMINCPSZ128rr = 13153,
  VMINCPSZ128rrk = 13154,
  VMINCPSZ128rrkz = 13155,
  VMINCPSZ256rm = 13156,
  VMINCPSZ256rmb = 13157,
  VMINCPSZ256rmbk = 13158,
  VMINCPSZ256rmbkz = 13159,
  VMINCPSZ256rmk = 13160,
  VMINCPSZ256rmkz = 13161,
  VMINCPSZ256rr = 13162,
  VMINCPSZ256rrk = 13163,
  VMINCPSZ256rrkz = 13164,
  VMINCPSZrm = 13165,
  VMINCPSZrmb = 13166,
  VMINCPSZrmbk = 13167,
  VMINCPSZrmbkz = 13168,
  VMINCPSZrmk = 13169,
  VMINCPSZrmkz = 13170,
  VMINCPSZrr = 13171,
  VMINCPSZrrk = 13172,
  VMINCPSZrrkz = 13173,
  VMINCPSrm = 13174,
  VMINCPSrr = 13175,
  VMINCSDZrm = 13176,
  VMINCSDZrr = 13177,
  VMINCSDrm = 13178,
  VMINCSDrr = 13179,
  VMINCSHZrm = 13180,
  VMINCSHZrr = 13181,
  VMINCSSZrm = 13182,
  VMINCSSZrr = 13183,
  VMINCSSrm = 13184,
  VMINCSSrr = 13185,
  VMINMAXBF16Z128rmbi = 13186,
  VMINMAXBF16Z128rmbik = 13187,
  VMINMAXBF16Z128rmbikz = 13188,
  VMINMAXBF16Z128rmi = 13189,
  VMINMAXBF16Z128rmik = 13190,
  VMINMAXBF16Z128rmikz = 13191,
  VMINMAXBF16Z128rri = 13192,
  VMINMAXBF16Z128rrik = 13193,
  VMINMAXBF16Z128rrikz = 13194,
  VMINMAXBF16Z256rmbi = 13195,
  VMINMAXBF16Z256rmbik = 13196,
  VMINMAXBF16Z256rmbikz = 13197,
  VMINMAXBF16Z256rmi = 13198,
  VMINMAXBF16Z256rmik = 13199,
  VMINMAXBF16Z256rmikz = 13200,
  VMINMAXBF16Z256rri = 13201,
  VMINMAXBF16Z256rrik = 13202,
  VMINMAXBF16Z256rrikz = 13203,
  VMINMAXBF16Zrmbi = 13204,
  VMINMAXBF16Zrmbik = 13205,
  VMINMAXBF16Zrmbikz = 13206,
  VMINMAXBF16Zrmi = 13207,
  VMINMAXBF16Zrmik = 13208,
  VMINMAXBF16Zrmikz = 13209,
  VMINMAXBF16Zrri = 13210,
  VMINMAXBF16Zrrik = 13211,
  VMINMAXBF16Zrrikz = 13212,
  VMINMAXPDZ128rmbi = 13213,
  VMINMAXPDZ128rmbik = 13214,
  VMINMAXPDZ128rmbikz = 13215,
  VMINMAXPDZ128rmi = 13216,
  VMINMAXPDZ128rmik = 13217,
  VMINMAXPDZ128rmikz = 13218,
  VMINMAXPDZ128rri = 13219,
  VMINMAXPDZ128rrik = 13220,
  VMINMAXPDZ128rrikz = 13221,
  VMINMAXPDZ256rmbi = 13222,
  VMINMAXPDZ256rmbik = 13223,
  VMINMAXPDZ256rmbikz = 13224,
  VMINMAXPDZ256rmi = 13225,
  VMINMAXPDZ256rmik = 13226,
  VMINMAXPDZ256rmikz = 13227,
  VMINMAXPDZ256rri = 13228,
  VMINMAXPDZ256rrik = 13229,
  VMINMAXPDZ256rrikz = 13230,
  VMINMAXPDZrmbi = 13231,
  VMINMAXPDZrmbik = 13232,
  VMINMAXPDZrmbikz = 13233,
  VMINMAXPDZrmi = 13234,
  VMINMAXPDZrmik = 13235,
  VMINMAXPDZrmikz = 13236,
  VMINMAXPDZrri = 13237,
  VMINMAXPDZrrib = 13238,
  VMINMAXPDZrribk = 13239,
  VMINMAXPDZrribkz = 13240,
  VMINMAXPDZrrik = 13241,
  VMINMAXPDZrrikz = 13242,
  VMINMAXPHZ128rmbi = 13243,
  VMINMAXPHZ128rmbik = 13244,
  VMINMAXPHZ128rmbikz = 13245,
  VMINMAXPHZ128rmi = 13246,
  VMINMAXPHZ128rmik = 13247,
  VMINMAXPHZ128rmikz = 13248,
  VMINMAXPHZ128rri = 13249,
  VMINMAXPHZ128rrik = 13250,
  VMINMAXPHZ128rrikz = 13251,
  VMINMAXPHZ256rmbi = 13252,
  VMINMAXPHZ256rmbik = 13253,
  VMINMAXPHZ256rmbikz = 13254,
  VMINMAXPHZ256rmi = 13255,
  VMINMAXPHZ256rmik = 13256,
  VMINMAXPHZ256rmikz = 13257,
  VMINMAXPHZ256rri = 13258,
  VMINMAXPHZ256rrik = 13259,
  VMINMAXPHZ256rrikz = 13260,
  VMINMAXPHZrmbi = 13261,
  VMINMAXPHZrmbik = 13262,
  VMINMAXPHZrmbikz = 13263,
  VMINMAXPHZrmi = 13264,
  VMINMAXPHZrmik = 13265,
  VMINMAXPHZrmikz = 13266,
  VMINMAXPHZrri = 13267,
  VMINMAXPHZrrib = 13268,
  VMINMAXPHZrribk = 13269,
  VMINMAXPHZrribkz = 13270,
  VMINMAXPHZrrik = 13271,
  VMINMAXPHZrrikz = 13272,
  VMINMAXPSZ128rmbi = 13273,
  VMINMAXPSZ128rmbik = 13274,
  VMINMAXPSZ128rmbikz = 13275,
  VMINMAXPSZ128rmi = 13276,
  VMINMAXPSZ128rmik = 13277,
  VMINMAXPSZ128rmikz = 13278,
  VMINMAXPSZ128rri = 13279,
  VMINMAXPSZ128rrik = 13280,
  VMINMAXPSZ128rrikz = 13281,
  VMINMAXPSZ256rmbi = 13282,
  VMINMAXPSZ256rmbik = 13283,
  VMINMAXPSZ256rmbikz = 13284,
  VMINMAXPSZ256rmi = 13285,
  VMINMAXPSZ256rmik = 13286,
  VMINMAXPSZ256rmikz = 13287,
  VMINMAXPSZ256rri = 13288,
  VMINMAXPSZ256rrik = 13289,
  VMINMAXPSZ256rrikz = 13290,
  VMINMAXPSZrmbi = 13291,
  VMINMAXPSZrmbik = 13292,
  VMINMAXPSZrmbikz = 13293,
  VMINMAXPSZrmi = 13294,
  VMINMAXPSZrmik = 13295,
  VMINMAXPSZrmikz = 13296,
  VMINMAXPSZrri = 13297,
  VMINMAXPSZrrib = 13298,
  VMINMAXPSZrribk = 13299,
  VMINMAXPSZrribkz = 13300,
  VMINMAXPSZrrik = 13301,
  VMINMAXPSZrrikz = 13302,
  VMINMAXSDrmi = 13303,
  VMINMAXSDrmi_Int = 13304,
  VMINMAXSDrmik_Int = 13305,
  VMINMAXSDrmikz_Int = 13306,
  VMINMAXSDrri = 13307,
  VMINMAXSDrri_Int = 13308,
  VMINMAXSDrrib_Int = 13309,
  VMINMAXSDrribk_Int = 13310,
  VMINMAXSDrribkz_Int = 13311,
  VMINMAXSDrrik_Int = 13312,
  VMINMAXSDrrikz_Int = 13313,
  VMINMAXSHrmi = 13314,
  VMINMAXSHrmi_Int = 13315,
  VMINMAXSHrmik_Int = 13316,
  VMINMAXSHrmikz_Int = 13317,
  VMINMAXSHrri = 13318,
  VMINMAXSHrri_Int = 13319,
  VMINMAXSHrrib_Int = 13320,
  VMINMAXSHrribk_Int = 13321,
  VMINMAXSHrribkz_Int = 13322,
  VMINMAXSHrrik_Int = 13323,
  VMINMAXSHrrikz_Int = 13324,
  VMINMAXSSrmi = 13325,
  VMINMAXSSrmi_Int = 13326,
  VMINMAXSSrmik_Int = 13327,
  VMINMAXSSrmikz_Int = 13328,
  VMINMAXSSrri = 13329,
  VMINMAXSSrri_Int = 13330,
  VMINMAXSSrrib_Int = 13331,
  VMINMAXSSrribk_Int = 13332,
  VMINMAXSSrribkz_Int = 13333,
  VMINMAXSSrrik_Int = 13334,
  VMINMAXSSrrikz_Int = 13335,
  VMINPDYrm = 13336,
  VMINPDYrr = 13337,
  VMINPDZ128rm = 13338,
  VMINPDZ128rmb = 13339,
  VMINPDZ128rmbk = 13340,
  VMINPDZ128rmbkz = 13341,
  VMINPDZ128rmk = 13342,
  VMINPDZ128rmkz = 13343,
  VMINPDZ128rr = 13344,
  VMINPDZ128rrk = 13345,
  VMINPDZ128rrkz = 13346,
  VMINPDZ256rm = 13347,
  VMINPDZ256rmb = 13348,
  VMINPDZ256rmbk = 13349,
  VMINPDZ256rmbkz = 13350,
  VMINPDZ256rmk = 13351,
  VMINPDZ256rmkz = 13352,
  VMINPDZ256rr = 13353,
  VMINPDZ256rrk = 13354,
  VMINPDZ256rrkz = 13355,
  VMINPDZrm = 13356,
  VMINPDZrmb = 13357,
  VMINPDZrmbk = 13358,
  VMINPDZrmbkz = 13359,
  VMINPDZrmk = 13360,
  VMINPDZrmkz = 13361,
  VMINPDZrr = 13362,
  VMINPDZrrb = 13363,
  VMINPDZrrbk = 13364,
  VMINPDZrrbkz = 13365,
  VMINPDZrrk = 13366,
  VMINPDZrrkz = 13367,
  VMINPDrm = 13368,
  VMINPDrr = 13369,
  VMINPHZ128rm = 13370,
  VMINPHZ128rmb = 13371,
  VMINPHZ128rmbk = 13372,
  VMINPHZ128rmbkz = 13373,
  VMINPHZ128rmk = 13374,
  VMINPHZ128rmkz = 13375,
  VMINPHZ128rr = 13376,
  VMINPHZ128rrk = 13377,
  VMINPHZ128rrkz = 13378,
  VMINPHZ256rm = 13379,
  VMINPHZ256rmb = 13380,
  VMINPHZ256rmbk = 13381,
  VMINPHZ256rmbkz = 13382,
  VMINPHZ256rmk = 13383,
  VMINPHZ256rmkz = 13384,
  VMINPHZ256rr = 13385,
  VMINPHZ256rrk = 13386,
  VMINPHZ256rrkz = 13387,
  VMINPHZrm = 13388,
  VMINPHZrmb = 13389,
  VMINPHZrmbk = 13390,
  VMINPHZrmbkz = 13391,
  VMINPHZrmk = 13392,
  VMINPHZrmkz = 13393,
  VMINPHZrr = 13394,
  VMINPHZrrb = 13395,
  VMINPHZrrbk = 13396,
  VMINPHZrrbkz = 13397,
  VMINPHZrrk = 13398,
  VMINPHZrrkz = 13399,
  VMINPSYrm = 13400,
  VMINPSYrr = 13401,
  VMINPSZ128rm = 13402,
  VMINPSZ128rmb = 13403,
  VMINPSZ128rmbk = 13404,
  VMINPSZ128rmbkz = 13405,
  VMINPSZ128rmk = 13406,
  VMINPSZ128rmkz = 13407,
  VMINPSZ128rr = 13408,
  VMINPSZ128rrk = 13409,
  VMINPSZ128rrkz = 13410,
  VMINPSZ256rm = 13411,
  VMINPSZ256rmb = 13412,
  VMINPSZ256rmbk = 13413,
  VMINPSZ256rmbkz = 13414,
  VMINPSZ256rmk = 13415,
  VMINPSZ256rmkz = 13416,
  VMINPSZ256rr = 13417,
  VMINPSZ256rrk = 13418,
  VMINPSZ256rrkz = 13419,
  VMINPSZrm = 13420,
  VMINPSZrmb = 13421,
  VMINPSZrmbk = 13422,
  VMINPSZrmbkz = 13423,
  VMINPSZrmk = 13424,
  VMINPSZrmkz = 13425,
  VMINPSZrr = 13426,
  VMINPSZrrb = 13427,
  VMINPSZrrbk = 13428,
  VMINPSZrrbkz = 13429,
  VMINPSZrrk = 13430,
  VMINPSZrrkz = 13431,
  VMINPSrm = 13432,
  VMINPSrr = 13433,
  VMINSDZrm = 13434,
  VMINSDZrm_Int = 13435,
  VMINSDZrmk_Int = 13436,
  VMINSDZrmkz_Int = 13437,
  VMINSDZrr = 13438,
  VMINSDZrr_Int = 13439,
  VMINSDZrrb_Int = 13440,
  VMINSDZrrbk_Int = 13441,
  VMINSDZrrbkz_Int = 13442,
  VMINSDZrrk_Int = 13443,
  VMINSDZrrkz_Int = 13444,
  VMINSDrm = 13445,
  VMINSDrm_Int = 13446,
  VMINSDrr = 13447,
  VMINSDrr_Int = 13448,
  VMINSHZrm = 13449,
  VMINSHZrm_Int = 13450,
  VMINSHZrmk_Int = 13451,
  VMINSHZrmkz_Int = 13452,
  VMINSHZrr = 13453,
  VMINSHZrr_Int = 13454,
  VMINSHZrrb_Int = 13455,
  VMINSHZrrbk_Int = 13456,
  VMINSHZrrbkz_Int = 13457,
  VMINSHZrrk_Int = 13458,
  VMINSHZrrkz_Int = 13459,
  VMINSSZrm = 13460,
  VMINSSZrm_Int = 13461,
  VMINSSZrmk_Int = 13462,
  VMINSSZrmkz_Int = 13463,
  VMINSSZrr = 13464,
  VMINSSZrr_Int = 13465,
  VMINSSZrrb_Int = 13466,
  VMINSSZrrbk_Int = 13467,
  VMINSSZrrbkz_Int = 13468,
  VMINSSZrrk_Int = 13469,
  VMINSSZrrkz_Int = 13470,
  VMINSSrm = 13471,
  VMINSSrm_Int = 13472,
  VMINSSrr = 13473,
  VMINSSrr_Int = 13474,
  VMLAUNCH = 13475,
  VMLOAD32 = 13476,
  VMLOAD64 = 13477,
  VMMCALL = 13478,
  VMOV64toPQIZrm = 13479,
  VMOV64toPQIZrr = 13480,
  VMOV64toPQIrm = 13481,
  VMOV64toPQIrr = 13482,
  VMOV64toSDZrr = 13483,
  VMOV64toSDrr = 13484,
  VMOVAPDYmr = 13485,
  VMOVAPDYrm = 13486,
  VMOVAPDYrr = 13487,
  VMOVAPDYrr_REV = 13488,
  VMOVAPDZ128mr = 13489,
  VMOVAPDZ128mrk = 13490,
  VMOVAPDZ128rm = 13491,
  VMOVAPDZ128rmk = 13492,
  VMOVAPDZ128rmkz = 13493,
  VMOVAPDZ128rr = 13494,
  VMOVAPDZ128rr_REV = 13495,
  VMOVAPDZ128rrk = 13496,
  VMOVAPDZ128rrk_REV = 13497,
  VMOVAPDZ128rrkz = 13498,
  VMOVAPDZ128rrkz_REV = 13499,
  VMOVAPDZ256mr = 13500,
  VMOVAPDZ256mrk = 13501,
  VMOVAPDZ256rm = 13502,
  VMOVAPDZ256rmk = 13503,
  VMOVAPDZ256rmkz = 13504,
  VMOVAPDZ256rr = 13505,
  VMOVAPDZ256rr_REV = 13506,
  VMOVAPDZ256rrk = 13507,
  VMOVAPDZ256rrk_REV = 13508,
  VMOVAPDZ256rrkz = 13509,
  VMOVAPDZ256rrkz_REV = 13510,
  VMOVAPDZmr = 13511,
  VMOVAPDZmrk = 13512,
  VMOVAPDZrm = 13513,
  VMOVAPDZrmk = 13514,
  VMOVAPDZrmkz = 13515,
  VMOVAPDZrr = 13516,
  VMOVAPDZrr_REV = 13517,
  VMOVAPDZrrk = 13518,
  VMOVAPDZrrk_REV = 13519,
  VMOVAPDZrrkz = 13520,
  VMOVAPDZrrkz_REV = 13521,
  VMOVAPDmr = 13522,
  VMOVAPDrm = 13523,
  VMOVAPDrr = 13524,
  VMOVAPDrr_REV = 13525,
  VMOVAPSYmr = 13526,
  VMOVAPSYrm = 13527,
  VMOVAPSYrr = 13528,
  VMOVAPSYrr_REV = 13529,
  VMOVAPSZ128mr = 13530,
  VMOVAPSZ128mrk = 13531,
  VMOVAPSZ128rm = 13532,
  VMOVAPSZ128rmk = 13533,
  VMOVAPSZ128rmkz = 13534,
  VMOVAPSZ128rr = 13535,
  VMOVAPSZ128rr_REV = 13536,
  VMOVAPSZ128rrk = 13537,
  VMOVAPSZ128rrk_REV = 13538,
  VMOVAPSZ128rrkz = 13539,
  VMOVAPSZ128rrkz_REV = 13540,
  VMOVAPSZ256mr = 13541,
  VMOVAPSZ256mrk = 13542,
  VMOVAPSZ256rm = 13543,
  VMOVAPSZ256rmk = 13544,
  VMOVAPSZ256rmkz = 13545,
  VMOVAPSZ256rr = 13546,
  VMOVAPSZ256rr_REV = 13547,
  VMOVAPSZ256rrk = 13548,
  VMOVAPSZ256rrk_REV = 13549,
  VMOVAPSZ256rrkz = 13550,
  VMOVAPSZ256rrkz_REV = 13551,
  VMOVAPSZmr = 13552,
  VMOVAPSZmrk = 13553,
  VMOVAPSZrm = 13554,
  VMOVAPSZrmk = 13555,
  VMOVAPSZrmkz = 13556,
  VMOVAPSZrr = 13557,
  VMOVAPSZrr_REV = 13558,
  VMOVAPSZrrk = 13559,
  VMOVAPSZrrk_REV = 13560,
  VMOVAPSZrrkz = 13561,
  VMOVAPSZrrkz_REV = 13562,
  VMOVAPSmr = 13563,
  VMOVAPSrm = 13564,
  VMOVAPSrr = 13565,
  VMOVAPSrr_REV = 13566,
  VMOVDDUPYrm = 13567,
  VMOVDDUPYrr = 13568,
  VMOVDDUPZ128rm = 13569,
  VMOVDDUPZ128rmk = 13570,
  VMOVDDUPZ128rmkz = 13571,
  VMOVDDUPZ128rr = 13572,
  VMOVDDUPZ128rrk = 13573,
  VMOVDDUPZ128rrkz = 13574,
  VMOVDDUPZ256rm = 13575,
  VMOVDDUPZ256rmk = 13576,
  VMOVDDUPZ256rmkz = 13577,
  VMOVDDUPZ256rr = 13578,
  VMOVDDUPZ256rrk = 13579,
  VMOVDDUPZ256rrkz = 13580,
  VMOVDDUPZrm = 13581,
  VMOVDDUPZrmk = 13582,
  VMOVDDUPZrmkz = 13583,
  VMOVDDUPZrr = 13584,
  VMOVDDUPZrrk = 13585,
  VMOVDDUPZrrkz = 13586,
  VMOVDDUPrm = 13587,
  VMOVDDUPrr = 13588,
  VMOVDI2PDIZrm = 13589,
  VMOVDI2PDIZrr = 13590,
  VMOVDI2PDIrm = 13591,
  VMOVDI2PDIrr = 13592,
  VMOVDI2SSZrr = 13593,
  VMOVDI2SSrr = 13594,
  VMOVDQA32Z128mr = 13595,
  VMOVDQA32Z128mrk = 13596,
  VMOVDQA32Z128rm = 13597,
  VMOVDQA32Z128rmk = 13598,
  VMOVDQA32Z128rmkz = 13599,
  VMOVDQA32Z128rr = 13600,
  VMOVDQA32Z128rr_REV = 13601,
  VMOVDQA32Z128rrk = 13602,
  VMOVDQA32Z128rrk_REV = 13603,
  VMOVDQA32Z128rrkz = 13604,
  VMOVDQA32Z128rrkz_REV = 13605,
  VMOVDQA32Z256mr = 13606,
  VMOVDQA32Z256mrk = 13607,
  VMOVDQA32Z256rm = 13608,
  VMOVDQA32Z256rmk = 13609,
  VMOVDQA32Z256rmkz = 13610,
  VMOVDQA32Z256rr = 13611,
  VMOVDQA32Z256rr_REV = 13612,
  VMOVDQA32Z256rrk = 13613,
  VMOVDQA32Z256rrk_REV = 13614,
  VMOVDQA32Z256rrkz = 13615,
  VMOVDQA32Z256rrkz_REV = 13616,
  VMOVDQA32Zmr = 13617,
  VMOVDQA32Zmrk = 13618,
  VMOVDQA32Zrm = 13619,
  VMOVDQA32Zrmk = 13620,
  VMOVDQA32Zrmkz = 13621,
  VMOVDQA32Zrr = 13622,
  VMOVDQA32Zrr_REV = 13623,
  VMOVDQA32Zrrk = 13624,
  VMOVDQA32Zrrk_REV = 13625,
  VMOVDQA32Zrrkz = 13626,
  VMOVDQA32Zrrkz_REV = 13627,
  VMOVDQA64Z128mr = 13628,
  VMOVDQA64Z128mrk = 13629,
  VMOVDQA64Z128rm = 13630,
  VMOVDQA64Z128rmk = 13631,
  VMOVDQA64Z128rmkz = 13632,
  VMOVDQA64Z128rr = 13633,
  VMOVDQA64Z128rr_REV = 13634,
  VMOVDQA64Z128rrk = 13635,
  VMOVDQA64Z128rrk_REV = 13636,
  VMOVDQA64Z128rrkz = 13637,
  VMOVDQA64Z128rrkz_REV = 13638,
  VMOVDQA64Z256mr = 13639,
  VMOVDQA64Z256mrk = 13640,
  VMOVDQA64Z256rm = 13641,
  VMOVDQA64Z256rmk = 13642,
  VMOVDQA64Z256rmkz = 13643,
  VMOVDQA64Z256rr = 13644,
  VMOVDQA64Z256rr_REV = 13645,
  VMOVDQA64Z256rrk = 13646,
  VMOVDQA64Z256rrk_REV = 13647,
  VMOVDQA64Z256rrkz = 13648,
  VMOVDQA64Z256rrkz_REV = 13649,
  VMOVDQA64Zmr = 13650,
  VMOVDQA64Zmrk = 13651,
  VMOVDQA64Zrm = 13652,
  VMOVDQA64Zrmk = 13653,
  VMOVDQA64Zrmkz = 13654,
  VMOVDQA64Zrr = 13655,
  VMOVDQA64Zrr_REV = 13656,
  VMOVDQA64Zrrk = 13657,
  VMOVDQA64Zrrk_REV = 13658,
  VMOVDQA64Zrrkz = 13659,
  VMOVDQA64Zrrkz_REV = 13660,
  VMOVDQAYmr = 13661,
  VMOVDQAYrm = 13662,
  VMOVDQAYrr = 13663,
  VMOVDQAYrr_REV = 13664,
  VMOVDQAmr = 13665,
  VMOVDQArm = 13666,
  VMOVDQArr = 13667,
  VMOVDQArr_REV = 13668,
  VMOVDQU16Z128mr = 13669,
  VMOVDQU16Z128mrk = 13670,
  VMOVDQU16Z128rm = 13671,
  VMOVDQU16Z128rmk = 13672,
  VMOVDQU16Z128rmkz = 13673,
  VMOVDQU16Z128rr = 13674,
  VMOVDQU16Z128rr_REV = 13675,
  VMOVDQU16Z128rrk = 13676,
  VMOVDQU16Z128rrk_REV = 13677,
  VMOVDQU16Z128rrkz = 13678,
  VMOVDQU16Z128rrkz_REV = 13679,
  VMOVDQU16Z256mr = 13680,
  VMOVDQU16Z256mrk = 13681,
  VMOVDQU16Z256rm = 13682,
  VMOVDQU16Z256rmk = 13683,
  VMOVDQU16Z256rmkz = 13684,
  VMOVDQU16Z256rr = 13685,
  VMOVDQU16Z256rr_REV = 13686,
  VMOVDQU16Z256rrk = 13687,
  VMOVDQU16Z256rrk_REV = 13688,
  VMOVDQU16Z256rrkz = 13689,
  VMOVDQU16Z256rrkz_REV = 13690,
  VMOVDQU16Zmr = 13691,
  VMOVDQU16Zmrk = 13692,
  VMOVDQU16Zrm = 13693,
  VMOVDQU16Zrmk = 13694,
  VMOVDQU16Zrmkz = 13695,
  VMOVDQU16Zrr = 13696,
  VMOVDQU16Zrr_REV = 13697,
  VMOVDQU16Zrrk = 13698,
  VMOVDQU16Zrrk_REV = 13699,
  VMOVDQU16Zrrkz = 13700,
  VMOVDQU16Zrrkz_REV = 13701,
  VMOVDQU32Z128mr = 13702,
  VMOVDQU32Z128mrk = 13703,
  VMOVDQU32Z128rm = 13704,
  VMOVDQU32Z128rmk = 13705,
  VMOVDQU32Z128rmkz = 13706,
  VMOVDQU32Z128rr = 13707,
  VMOVDQU32Z128rr_REV = 13708,
  VMOVDQU32Z128rrk = 13709,
  VMOVDQU32Z128rrk_REV = 13710,
  VMOVDQU32Z128rrkz = 13711,
  VMOVDQU32Z128rrkz_REV = 13712,
  VMOVDQU32Z256mr = 13713,
  VMOVDQU32Z256mrk = 13714,
  VMOVDQU32Z256rm = 13715,
  VMOVDQU32Z256rmk = 13716,
  VMOVDQU32Z256rmkz = 13717,
  VMOVDQU32Z256rr = 13718,
  VMOVDQU32Z256rr_REV = 13719,
  VMOVDQU32Z256rrk = 13720,
  VMOVDQU32Z256rrk_REV = 13721,
  VMOVDQU32Z256rrkz = 13722,
  VMOVDQU32Z256rrkz_REV = 13723,
  VMOVDQU32Zmr = 13724,
  VMOVDQU32Zmrk = 13725,
  VMOVDQU32Zrm = 13726,
  VMOVDQU32Zrmk = 13727,
  VMOVDQU32Zrmkz = 13728,
  VMOVDQU32Zrr = 13729,
  VMOVDQU32Zrr_REV = 13730,
  VMOVDQU32Zrrk = 13731,
  VMOVDQU32Zrrk_REV = 13732,
  VMOVDQU32Zrrkz = 13733,
  VMOVDQU32Zrrkz_REV = 13734,
  VMOVDQU64Z128mr = 13735,
  VMOVDQU64Z128mrk = 13736,
  VMOVDQU64Z128rm = 13737,
  VMOVDQU64Z128rmk = 13738,
  VMOVDQU64Z128rmkz = 13739,
  VMOVDQU64Z128rr = 13740,
  VMOVDQU64Z128rr_REV = 13741,
  VMOVDQU64Z128rrk = 13742,
  VMOVDQU64Z128rrk_REV = 13743,
  VMOVDQU64Z128rrkz = 13744,
  VMOVDQU64Z128rrkz_REV = 13745,
  VMOVDQU64Z256mr = 13746,
  VMOVDQU64Z256mrk = 13747,
  VMOVDQU64Z256rm = 13748,
  VMOVDQU64Z256rmk = 13749,
  VMOVDQU64Z256rmkz = 13750,
  VMOVDQU64Z256rr = 13751,
  VMOVDQU64Z256rr_REV = 13752,
  VMOVDQU64Z256rrk = 13753,
  VMOVDQU64Z256rrk_REV = 13754,
  VMOVDQU64Z256rrkz = 13755,
  VMOVDQU64Z256rrkz_REV = 13756,
  VMOVDQU64Zmr = 13757,
  VMOVDQU64Zmrk = 13758,
  VMOVDQU64Zrm = 13759,
  VMOVDQU64Zrmk = 13760,
  VMOVDQU64Zrmkz = 13761,
  VMOVDQU64Zrr = 13762,
  VMOVDQU64Zrr_REV = 13763,
  VMOVDQU64Zrrk = 13764,
  VMOVDQU64Zrrk_REV = 13765,
  VMOVDQU64Zrrkz = 13766,
  VMOVDQU64Zrrkz_REV = 13767,
  VMOVDQU8Z128mr = 13768,
  VMOVDQU8Z128mrk = 13769,
  VMOVDQU8Z128rm = 13770,
  VMOVDQU8Z128rmk = 13771,
  VMOVDQU8Z128rmkz = 13772,
  VMOVDQU8Z128rr = 13773,
  VMOVDQU8Z128rr_REV = 13774,
  VMOVDQU8Z128rrk = 13775,
  VMOVDQU8Z128rrk_REV = 13776,
  VMOVDQU8Z128rrkz = 13777,
  VMOVDQU8Z128rrkz_REV = 13778,
  VMOVDQU8Z256mr = 13779,
  VMOVDQU8Z256mrk = 13780,
  VMOVDQU8Z256rm = 13781,
  VMOVDQU8Z256rmk = 13782,
  VMOVDQU8Z256rmkz = 13783,
  VMOVDQU8Z256rr = 13784,
  VMOVDQU8Z256rr_REV = 13785,
  VMOVDQU8Z256rrk = 13786,
  VMOVDQU8Z256rrk_REV = 13787,
  VMOVDQU8Z256rrkz = 13788,
  VMOVDQU8Z256rrkz_REV = 13789,
  VMOVDQU8Zmr = 13790,
  VMOVDQU8Zmrk = 13791,
  VMOVDQU8Zrm = 13792,
  VMOVDQU8Zrmk = 13793,
  VMOVDQU8Zrmkz = 13794,
  VMOVDQU8Zrr = 13795,
  VMOVDQU8Zrr_REV = 13796,
  VMOVDQU8Zrrk = 13797,
  VMOVDQU8Zrrk_REV = 13798,
  VMOVDQU8Zrrkz = 13799,
  VMOVDQU8Zrrkz_REV = 13800,
  VMOVDQUYmr = 13801,
  VMOVDQUYrm = 13802,
  VMOVDQUYrr = 13803,
  VMOVDQUYrr_REV = 13804,
  VMOVDQUmr = 13805,
  VMOVDQUrm = 13806,
  VMOVDQUrr = 13807,
  VMOVDQUrr_REV = 13808,
  VMOVHLPSZrr = 13809,
  VMOVHLPSrr = 13810,
  VMOVHPDZ128mr = 13811,
  VMOVHPDZ128rm = 13812,
  VMOVHPDmr = 13813,
  VMOVHPDrm = 13814,
  VMOVHPSZ128mr = 13815,
  VMOVHPSZ128rm = 13816,
  VMOVHPSmr = 13817,
  VMOVHPSrm = 13818,
  VMOVLHPSZrr = 13819,
  VMOVLHPSrr = 13820,
  VMOVLPDZ128mr = 13821,
  VMOVLPDZ128rm = 13822,
  VMOVLPDmr = 13823,
  VMOVLPDrm = 13824,
  VMOVLPSZ128mr = 13825,
  VMOVLPSZ128rm = 13826,
  VMOVLPSmr = 13827,
  VMOVLPSrm = 13828,
  VMOVMSKPDYrr = 13829,
  VMOVMSKPDrr = 13830,
  VMOVMSKPSYrr = 13831,
  VMOVMSKPSrr = 13832,
  VMOVNTDQAYrm = 13833,
  VMOVNTDQAZ128rm = 13834,
  VMOVNTDQAZ256rm = 13835,
  VMOVNTDQAZrm = 13836,
  VMOVNTDQArm = 13837,
  VMOVNTDQYmr = 13838,
  VMOVNTDQZ128mr = 13839,
  VMOVNTDQZ256mr = 13840,
  VMOVNTDQZmr = 13841,
  VMOVNTDQmr = 13842,
  VMOVNTPDYmr = 13843,
  VMOVNTPDZ128mr = 13844,
  VMOVNTPDZ256mr = 13845,
  VMOVNTPDZmr = 13846,
  VMOVNTPDmr = 13847,
  VMOVNTPSYmr = 13848,
  VMOVNTPSZ128mr = 13849,
  VMOVNTPSZ256mr = 13850,
  VMOVNTPSZmr = 13851,
  VMOVNTPSmr = 13852,
  VMOVPDI2DIZmr = 13853,
  VMOVPDI2DIZrr = 13854,
  VMOVPDI2DImr = 13855,
  VMOVPDI2DIrr = 13856,
  VMOVPQI2QIZmr = 13857,
  VMOVPQI2QIZrr = 13858,
  VMOVPQI2QImr = 13859,
  VMOVPQI2QIrr = 13860,
  VMOVPQIto64Zmr = 13861,
  VMOVPQIto64Zrr = 13862,
  VMOVPQIto64mr = 13863,
  VMOVPQIto64rr = 13864,
  VMOVQI2PQIZrm = 13865,
  VMOVQI2PQIrm = 13866,
  VMOVRSBZ128m = 13867,
  VMOVRSBZ128mk = 13868,
  VMOVRSBZ128mkz = 13869,
  VMOVRSBZ256m = 13870,
  VMOVRSBZ256mk = 13871,
  VMOVRSBZ256mkz = 13872,
  VMOVRSBZm = 13873,
  VMOVRSBZmk = 13874,
  VMOVRSBZmkz = 13875,
  VMOVRSDZ128m = 13876,
  VMOVRSDZ128mk = 13877,
  VMOVRSDZ128mkz = 13878,
  VMOVRSDZ256m = 13879,
  VMOVRSDZ256mk = 13880,
  VMOVRSDZ256mkz = 13881,
  VMOVRSDZm = 13882,
  VMOVRSDZmk = 13883,
  VMOVRSDZmkz = 13884,
  VMOVRSQZ128m = 13885,
  VMOVRSQZ128mk = 13886,
  VMOVRSQZ128mkz = 13887,
  VMOVRSQZ256m = 13888,
  VMOVRSQZ256mk = 13889,
  VMOVRSQZ256mkz = 13890,
  VMOVRSQZm = 13891,
  VMOVRSQZmk = 13892,
  VMOVRSQZmkz = 13893,
  VMOVRSWZ128m = 13894,
  VMOVRSWZ128mk = 13895,
  VMOVRSWZ128mkz = 13896,
  VMOVRSWZ256m = 13897,
  VMOVRSWZ256mk = 13898,
  VMOVRSWZ256mkz = 13899,
  VMOVRSWZm = 13900,
  VMOVRSWZmk = 13901,
  VMOVRSWZmkz = 13902,
  VMOVSDZmr = 13903,
  VMOVSDZmrk = 13904,
  VMOVSDZrm = 13905,
  VMOVSDZrm_alt = 13906,
  VMOVSDZrmk = 13907,
  VMOVSDZrmkz = 13908,
  VMOVSDZrr = 13909,
  VMOVSDZrr_REV = 13910,
  VMOVSDZrrk = 13911,
  VMOVSDZrrk_REV = 13912,
  VMOVSDZrrkz = 13913,
  VMOVSDZrrkz_REV = 13914,
  VMOVSDmr = 13915,
  VMOVSDrm = 13916,
  VMOVSDrm_alt = 13917,
  VMOVSDrr = 13918,
  VMOVSDrr_REV = 13919,
  VMOVSDto64Zrr = 13920,
  VMOVSDto64rr = 13921,
  VMOVSH2Wrr = 13922,
  VMOVSHDUPYrm = 13923,
  VMOVSHDUPYrr = 13924,
  VMOVSHDUPZ128rm = 13925,
  VMOVSHDUPZ128rmk = 13926,
  VMOVSHDUPZ128rmkz = 13927,
  VMOVSHDUPZ128rr = 13928,
  VMOVSHDUPZ128rrk = 13929,
  VMOVSHDUPZ128rrkz = 13930,
  VMOVSHDUPZ256rm = 13931,
  VMOVSHDUPZ256rmk = 13932,
  VMOVSHDUPZ256rmkz = 13933,
  VMOVSHDUPZ256rr = 13934,
  VMOVSHDUPZ256rrk = 13935,
  VMOVSHDUPZ256rrkz = 13936,
  VMOVSHDUPZrm = 13937,
  VMOVSHDUPZrmk = 13938,
  VMOVSHDUPZrmkz = 13939,
  VMOVSHDUPZrr = 13940,
  VMOVSHDUPZrrk = 13941,
  VMOVSHDUPZrrkz = 13942,
  VMOVSHDUPrm = 13943,
  VMOVSHDUPrr = 13944,
  VMOVSHZmr = 13945,
  VMOVSHZmrk = 13946,
  VMOVSHZrm = 13947,
  VMOVSHZrm_alt = 13948,
  VMOVSHZrmk = 13949,
  VMOVSHZrmkz = 13950,
  VMOVSHZrr = 13951,
  VMOVSHZrr_REV = 13952,
  VMOVSHZrrk = 13953,
  VMOVSHZrrk_REV = 13954,
  VMOVSHZrrkz = 13955,
  VMOVSHZrrkz_REV = 13956,
  VMOVSHtoW64rr = 13957,
  VMOVSLDUPYrm = 13958,
  VMOVSLDUPYrr = 13959,
  VMOVSLDUPZ128rm = 13960,
  VMOVSLDUPZ128rmk = 13961,
  VMOVSLDUPZ128rmkz = 13962,
  VMOVSLDUPZ128rr = 13963,
  VMOVSLDUPZ128rrk = 13964,
  VMOVSLDUPZ128rrkz = 13965,
  VMOVSLDUPZ256rm = 13966,
  VMOVSLDUPZ256rmk = 13967,
  VMOVSLDUPZ256rmkz = 13968,
  VMOVSLDUPZ256rr = 13969,
  VMOVSLDUPZ256rrk = 13970,
  VMOVSLDUPZ256rrkz = 13971,
  VMOVSLDUPZrm = 13972,
  VMOVSLDUPZrmk = 13973,
  VMOVSLDUPZrmkz = 13974,
  VMOVSLDUPZrr = 13975,
  VMOVSLDUPZrrk = 13976,
  VMOVSLDUPZrrkz = 13977,
  VMOVSLDUPrm = 13978,
  VMOVSLDUPrr = 13979,
  VMOVSS2DIZrr = 13980,
  VMOVSS2DIrr = 13981,
  VMOVSSZmr = 13982,
  VMOVSSZmrk = 13983,
  VMOVSSZrm = 13984,
  VMOVSSZrm_alt = 13985,
  VMOVSSZrmk = 13986,
  VMOVSSZrmkz = 13987,
  VMOVSSZrr = 13988,
  VMOVSSZrr_REV = 13989,
  VMOVSSZrrk = 13990,
  VMOVSSZrrk_REV = 13991,
  VMOVSSZrrkz = 13992,
  VMOVSSZrrkz_REV = 13993,
  VMOVSSmr = 13994,
  VMOVSSrm = 13995,
  VMOVSSrm_alt = 13996,
  VMOVSSrr = 13997,
  VMOVSSrr_REV = 13998,
  VMOVUPDYmr = 13999,
  VMOVUPDYrm = 14000,
  VMOVUPDYrr = 14001,
  VMOVUPDYrr_REV = 14002,
  VMOVUPDZ128mr = 14003,
  VMOVUPDZ128mrk = 14004,
  VMOVUPDZ128rm = 14005,
  VMOVUPDZ128rmk = 14006,
  VMOVUPDZ128rmkz = 14007,
  VMOVUPDZ128rr = 14008,
  VMOVUPDZ128rr_REV = 14009,
  VMOVUPDZ128rrk = 14010,
  VMOVUPDZ128rrk_REV = 14011,
  VMOVUPDZ128rrkz = 14012,
  VMOVUPDZ128rrkz_REV = 14013,
  VMOVUPDZ256mr = 14014,
  VMOVUPDZ256mrk = 14015,
  VMOVUPDZ256rm = 14016,
  VMOVUPDZ256rmk = 14017,
  VMOVUPDZ256rmkz = 14018,
  VMOVUPDZ256rr = 14019,
  VMOVUPDZ256rr_REV = 14020,
  VMOVUPDZ256rrk = 14021,
  VMOVUPDZ256rrk_REV = 14022,
  VMOVUPDZ256rrkz = 14023,
  VMOVUPDZ256rrkz_REV = 14024,
  VMOVUPDZmr = 14025,
  VMOVUPDZmrk = 14026,
  VMOVUPDZrm = 14027,
  VMOVUPDZrmk = 14028,
  VMOVUPDZrmkz = 14029,
  VMOVUPDZrr = 14030,
  VMOVUPDZrr_REV = 14031,
  VMOVUPDZrrk = 14032,
  VMOVUPDZrrk_REV = 14033,
  VMOVUPDZrrkz = 14034,
  VMOVUPDZrrkz_REV = 14035,
  VMOVUPDmr = 14036,
  VMOVUPDrm = 14037,
  VMOVUPDrr = 14038,
  VMOVUPDrr_REV = 14039,
  VMOVUPSYmr = 14040,
  VMOVUPSYrm = 14041,
  VMOVUPSYrr = 14042,
  VMOVUPSYrr_REV = 14043,
  VMOVUPSZ128mr = 14044,
  VMOVUPSZ128mrk = 14045,
  VMOVUPSZ128rm = 14046,
  VMOVUPSZ128rmk = 14047,
  VMOVUPSZ128rmkz = 14048,
  VMOVUPSZ128rr = 14049,
  VMOVUPSZ128rr_REV = 14050,
  VMOVUPSZ128rrk = 14051,
  VMOVUPSZ128rrk_REV = 14052,
  VMOVUPSZ128rrkz = 14053,
  VMOVUPSZ128rrkz_REV = 14054,
  VMOVUPSZ256mr = 14055,
  VMOVUPSZ256mrk = 14056,
  VMOVUPSZ256rm = 14057,
  VMOVUPSZ256rmk = 14058,
  VMOVUPSZ256rmkz = 14059,
  VMOVUPSZ256rr = 14060,
  VMOVUPSZ256rr_REV = 14061,
  VMOVUPSZ256rrk = 14062,
  VMOVUPSZ256rrk_REV = 14063,
  VMOVUPSZ256rrkz = 14064,
  VMOVUPSZ256rrkz_REV = 14065,
  VMOVUPSZmr = 14066,
  VMOVUPSZmrk = 14067,
  VMOVUPSZrm = 14068,
  VMOVUPSZrmk = 14069,
  VMOVUPSZrmkz = 14070,
  VMOVUPSZrr = 14071,
  VMOVUPSZrr_REV = 14072,
  VMOVUPSZrrk = 14073,
  VMOVUPSZrrk_REV = 14074,
  VMOVUPSZrrkz = 14075,
  VMOVUPSZrrkz_REV = 14076,
  VMOVUPSmr = 14077,
  VMOVUPSrm = 14078,
  VMOVUPSrr = 14079,
  VMOVUPSrr_REV = 14080,
  VMOVW2SHrr = 14081,
  VMOVW64toSHrr = 14082,
  VMOVWmr = 14083,
  VMOVWrm = 14084,
  VMOVZPDILo2PDIZmr = 14085,
  VMOVZPDILo2PDIZrm = 14086,
  VMOVZPDILo2PDIZrr = 14087,
  VMOVZPDILo2PDIZrr2 = 14088,
  VMOVZPQILo2PQIZrr = 14089,
  VMOVZPQILo2PQIrr = 14090,
  VMOVZPWILo2PWIZmr = 14091,
  VMOVZPWILo2PWIZrm = 14092,
  VMOVZPWILo2PWIZrr = 14093,
  VMOVZPWILo2PWIZrr2 = 14094,
  VMPSADBWYrmi = 14095,
  VMPSADBWYrri = 14096,
  VMPSADBWZ128rmi = 14097,
  VMPSADBWZ128rmik = 14098,
  VMPSADBWZ128rmikz = 14099,
  VMPSADBWZ128rri = 14100,
  VMPSADBWZ128rrik = 14101,
  VMPSADBWZ128rrikz = 14102,
  VMPSADBWZ256rmi = 14103,
  VMPSADBWZ256rmik = 14104,
  VMPSADBWZ256rmikz = 14105,
  VMPSADBWZ256rri = 14106,
  VMPSADBWZ256rrik = 14107,
  VMPSADBWZ256rrikz = 14108,
  VMPSADBWZrmi = 14109,
  VMPSADBWZrmik = 14110,
  VMPSADBWZrmikz = 14111,
  VMPSADBWZrri = 14112,
  VMPSADBWZrrik = 14113,
  VMPSADBWZrrikz = 14114,
  VMPSADBWrmi = 14115,
  VMPSADBWrri = 14116,
  VMPTRLDm = 14117,
  VMPTRSTm = 14118,
  VMREAD32mr = 14119,
  VMREAD32rr = 14120,
  VMREAD64mr = 14121,
  VMREAD64rr = 14122,
  VMRESUME = 14123,
  VMRUN32 = 14124,
  VMRUN64 = 14125,
  VMSAVE32 = 14126,
  VMSAVE64 = 14127,
  VMULBF16Z128rm = 14128,
  VMULBF16Z128rmb = 14129,
  VMULBF16Z128rmbk = 14130,
  VMULBF16Z128rmbkz = 14131,
  VMULBF16Z128rmk = 14132,
  VMULBF16Z128rmkz = 14133,
  VMULBF16Z128rr = 14134,
  VMULBF16Z128rrk = 14135,
  VMULBF16Z128rrkz = 14136,
  VMULBF16Z256rm = 14137,
  VMULBF16Z256rmb = 14138,
  VMULBF16Z256rmbk = 14139,
  VMULBF16Z256rmbkz = 14140,
  VMULBF16Z256rmk = 14141,
  VMULBF16Z256rmkz = 14142,
  VMULBF16Z256rr = 14143,
  VMULBF16Z256rrk = 14144,
  VMULBF16Z256rrkz = 14145,
  VMULBF16Zrm = 14146,
  VMULBF16Zrmb = 14147,
  VMULBF16Zrmbk = 14148,
  VMULBF16Zrmbkz = 14149,
  VMULBF16Zrmk = 14150,
  VMULBF16Zrmkz = 14151,
  VMULBF16Zrr = 14152,
  VMULBF16Zrrk = 14153,
  VMULBF16Zrrkz = 14154,
  VMULPDYrm = 14155,
  VMULPDYrr = 14156,
  VMULPDZ128rm = 14157,
  VMULPDZ128rmb = 14158,
  VMULPDZ128rmbk = 14159,
  VMULPDZ128rmbkz = 14160,
  VMULPDZ128rmk = 14161,
  VMULPDZ128rmkz = 14162,
  VMULPDZ128rr = 14163,
  VMULPDZ128rrk = 14164,
  VMULPDZ128rrkz = 14165,
  VMULPDZ256rm = 14166,
  VMULPDZ256rmb = 14167,
  VMULPDZ256rmbk = 14168,
  VMULPDZ256rmbkz = 14169,
  VMULPDZ256rmk = 14170,
  VMULPDZ256rmkz = 14171,
  VMULPDZ256rr = 14172,
  VMULPDZ256rrk = 14173,
  VMULPDZ256rrkz = 14174,
  VMULPDZrm = 14175,
  VMULPDZrmb = 14176,
  VMULPDZrmbk = 14177,
  VMULPDZrmbkz = 14178,
  VMULPDZrmk = 14179,
  VMULPDZrmkz = 14180,
  VMULPDZrr = 14181,
  VMULPDZrrb = 14182,
  VMULPDZrrbk = 14183,
  VMULPDZrrbkz = 14184,
  VMULPDZrrk = 14185,
  VMULPDZrrkz = 14186,
  VMULPDrm = 14187,
  VMULPDrr = 14188,
  VMULPHZ128rm = 14189,
  VMULPHZ128rmb = 14190,
  VMULPHZ128rmbk = 14191,
  VMULPHZ128rmbkz = 14192,
  VMULPHZ128rmk = 14193,
  VMULPHZ128rmkz = 14194,
  VMULPHZ128rr = 14195,
  VMULPHZ128rrk = 14196,
  VMULPHZ128rrkz = 14197,
  VMULPHZ256rm = 14198,
  VMULPHZ256rmb = 14199,
  VMULPHZ256rmbk = 14200,
  VMULPHZ256rmbkz = 14201,
  VMULPHZ256rmk = 14202,
  VMULPHZ256rmkz = 14203,
  VMULPHZ256rr = 14204,
  VMULPHZ256rrk = 14205,
  VMULPHZ256rrkz = 14206,
  VMULPHZrm = 14207,
  VMULPHZrmb = 14208,
  VMULPHZrmbk = 14209,
  VMULPHZrmbkz = 14210,
  VMULPHZrmk = 14211,
  VMULPHZrmkz = 14212,
  VMULPHZrr = 14213,
  VMULPHZrrb = 14214,
  VMULPHZrrbk = 14215,
  VMULPHZrrbkz = 14216,
  VMULPHZrrk = 14217,
  VMULPHZrrkz = 14218,
  VMULPSYrm = 14219,
  VMULPSYrr = 14220,
  VMULPSZ128rm = 14221,
  VMULPSZ128rmb = 14222,
  VMULPSZ128rmbk = 14223,
  VMULPSZ128rmbkz = 14224,
  VMULPSZ128rmk = 14225,
  VMULPSZ128rmkz = 14226,
  VMULPSZ128rr = 14227,
  VMULPSZ128rrk = 14228,
  VMULPSZ128rrkz = 14229,
  VMULPSZ256rm = 14230,
  VMULPSZ256rmb = 14231,
  VMULPSZ256rmbk = 14232,
  VMULPSZ256rmbkz = 14233,
  VMULPSZ256rmk = 14234,
  VMULPSZ256rmkz = 14235,
  VMULPSZ256rr = 14236,
  VMULPSZ256rrk = 14237,
  VMULPSZ256rrkz = 14238,
  VMULPSZrm = 14239,
  VMULPSZrmb = 14240,
  VMULPSZrmbk = 14241,
  VMULPSZrmbkz = 14242,
  VMULPSZrmk = 14243,
  VMULPSZrmkz = 14244,
  VMULPSZrr = 14245,
  VMULPSZrrb = 14246,
  VMULPSZrrbk = 14247,
  VMULPSZrrbkz = 14248,
  VMULPSZrrk = 14249,
  VMULPSZrrkz = 14250,
  VMULPSrm = 14251,
  VMULPSrr = 14252,
  VMULSDZrm = 14253,
  VMULSDZrm_Int = 14254,
  VMULSDZrmk_Int = 14255,
  VMULSDZrmkz_Int = 14256,
  VMULSDZrr = 14257,
  VMULSDZrr_Int = 14258,
  VMULSDZrrb_Int = 14259,
  VMULSDZrrbk_Int = 14260,
  VMULSDZrrbkz_Int = 14261,
  VMULSDZrrk_Int = 14262,
  VMULSDZrrkz_Int = 14263,
  VMULSDrm = 14264,
  VMULSDrm_Int = 14265,
  VMULSDrr = 14266,
  VMULSDrr_Int = 14267,
  VMULSHZrm = 14268,
  VMULSHZrm_Int = 14269,
  VMULSHZrmk_Int = 14270,
  VMULSHZrmkz_Int = 14271,
  VMULSHZrr = 14272,
  VMULSHZrr_Int = 14273,
  VMULSHZrrb_Int = 14274,
  VMULSHZrrbk_Int = 14275,
  VMULSHZrrbkz_Int = 14276,
  VMULSHZrrk_Int = 14277,
  VMULSHZrrkz_Int = 14278,
  VMULSSZrm = 14279,
  VMULSSZrm_Int = 14280,
  VMULSSZrmk_Int = 14281,
  VMULSSZrmkz_Int = 14282,
  VMULSSZrr = 14283,
  VMULSSZrr_Int = 14284,
  VMULSSZrrb_Int = 14285,
  VMULSSZrrbk_Int = 14286,
  VMULSSZrrbkz_Int = 14287,
  VMULSSZrrk_Int = 14288,
  VMULSSZrrkz_Int = 14289,
  VMULSSrm = 14290,
  VMULSSrm_Int = 14291,
  VMULSSrr = 14292,
  VMULSSrr_Int = 14293,
  VMWRITE32rm = 14294,
  VMWRITE32rr = 14295,
  VMWRITE64rm = 14296,
  VMWRITE64rr = 14297,
  VMXOFF = 14298,
  VMXON = 14299,
  VORPDYrm = 14300,
  VORPDYrr = 14301,
  VORPDZ128rm = 14302,
  VORPDZ128rmb = 14303,
  VORPDZ128rmbk = 14304,
  VORPDZ128rmbkz = 14305,
  VORPDZ128rmk = 14306,
  VORPDZ128rmkz = 14307,
  VORPDZ128rr = 14308,
  VORPDZ128rrk = 14309,
  VORPDZ128rrkz = 14310,
  VORPDZ256rm = 14311,
  VORPDZ256rmb = 14312,
  VORPDZ256rmbk = 14313,
  VORPDZ256rmbkz = 14314,
  VORPDZ256rmk = 14315,
  VORPDZ256rmkz = 14316,
  VORPDZ256rr = 14317,
  VORPDZ256rrk = 14318,
  VORPDZ256rrkz = 14319,
  VORPDZrm = 14320,
  VORPDZrmb = 14321,
  VORPDZrmbk = 14322,
  VORPDZrmbkz = 14323,
  VORPDZrmk = 14324,
  VORPDZrmkz = 14325,
  VORPDZrr = 14326,
  VORPDZrrk = 14327,
  VORPDZrrkz = 14328,
  VORPDrm = 14329,
  VORPDrr = 14330,
  VORPSYrm = 14331,
  VORPSYrr = 14332,
  VORPSZ128rm = 14333,
  VORPSZ128rmb = 14334,
  VORPSZ128rmbk = 14335,
  VORPSZ128rmbkz = 14336,
  VORPSZ128rmk = 14337,
  VORPSZ128rmkz = 14338,
  VORPSZ128rr = 14339,
  VORPSZ128rrk = 14340,
  VORPSZ128rrkz = 14341,
  VORPSZ256rm = 14342,
  VORPSZ256rmb = 14343,
  VORPSZ256rmbk = 14344,
  VORPSZ256rmbkz = 14345,
  VORPSZ256rmk = 14346,
  VORPSZ256rmkz = 14347,
  VORPSZ256rr = 14348,
  VORPSZ256rrk = 14349,
  VORPSZ256rrkz = 14350,
  VORPSZrm = 14351,
  VORPSZrmb = 14352,
  VORPSZrmbk = 14353,
  VORPSZrmbkz = 14354,
  VORPSZrmk = 14355,
  VORPSZrmkz = 14356,
  VORPSZrr = 14357,
  VORPSZrrk = 14358,
  VORPSZrrkz = 14359,
  VORPSrm = 14360,
  VORPSrr = 14361,
  VP2INTERSECTDZ128rm = 14362,
  VP2INTERSECTDZ128rmb = 14363,
  VP2INTERSECTDZ128rr = 14364,
  VP2INTERSECTDZ256rm = 14365,
  VP2INTERSECTDZ256rmb = 14366,
  VP2INTERSECTDZ256rr = 14367,
  VP2INTERSECTDZrm = 14368,
  VP2INTERSECTDZrmb = 14369,
  VP2INTERSECTDZrr = 14370,
  VP2INTERSECTQZ128rm = 14371,
  VP2INTERSECTQZ128rmb = 14372,
  VP2INTERSECTQZ128rr = 14373,
  VP2INTERSECTQZ256rm = 14374,
  VP2INTERSECTQZ256rmb = 14375,
  VP2INTERSECTQZ256rr = 14376,
  VP2INTERSECTQZrm = 14377,
  VP2INTERSECTQZrmb = 14378,
  VP2INTERSECTQZrr = 14379,
  VP4DPWSSDSrm = 14380,
  VP4DPWSSDSrmk = 14381,
  VP4DPWSSDSrmkz = 14382,
  VP4DPWSSDrm = 14383,
  VP4DPWSSDrmk = 14384,
  VP4DPWSSDrmkz = 14385,
  VPABSBYrm = 14386,
  VPABSBYrr = 14387,
  VPABSBZ128rm = 14388,
  VPABSBZ128rmk = 14389,
  VPABSBZ128rmkz = 14390,
  VPABSBZ128rr = 14391,
  VPABSBZ128rrk = 14392,
  VPABSBZ128rrkz = 14393,
  VPABSBZ256rm = 14394,
  VPABSBZ256rmk = 14395,
  VPABSBZ256rmkz = 14396,
  VPABSBZ256rr = 14397,
  VPABSBZ256rrk = 14398,
  VPABSBZ256rrkz = 14399,
  VPABSBZrm = 14400,
  VPABSBZrmk = 14401,
  VPABSBZrmkz = 14402,
  VPABSBZrr = 14403,
  VPABSBZrrk = 14404,
  VPABSBZrrkz = 14405,
  VPABSBrm = 14406,
  VPABSBrr = 14407,
  VPABSDYrm = 14408,
  VPABSDYrr = 14409,
  VPABSDZ128rm = 14410,
  VPABSDZ128rmb = 14411,
  VPABSDZ128rmbk = 14412,
  VPABSDZ128rmbkz = 14413,
  VPABSDZ128rmk = 14414,
  VPABSDZ128rmkz = 14415,
  VPABSDZ128rr = 14416,
  VPABSDZ128rrk = 14417,
  VPABSDZ128rrkz = 14418,
  VPABSDZ256rm = 14419,
  VPABSDZ256rmb = 14420,
  VPABSDZ256rmbk = 14421,
  VPABSDZ256rmbkz = 14422,
  VPABSDZ256rmk = 14423,
  VPABSDZ256rmkz = 14424,
  VPABSDZ256rr = 14425,
  VPABSDZ256rrk = 14426,
  VPABSDZ256rrkz = 14427,
  VPABSDZrm = 14428,
  VPABSDZrmb = 14429,
  VPABSDZrmbk = 14430,
  VPABSDZrmbkz = 14431,
  VPABSDZrmk = 14432,
  VPABSDZrmkz = 14433,
  VPABSDZrr = 14434,
  VPABSDZrrk = 14435,
  VPABSDZrrkz = 14436,
  VPABSDrm = 14437,
  VPABSDrr = 14438,
  VPABSQZ128rm = 14439,
  VPABSQZ128rmb = 14440,
  VPABSQZ128rmbk = 14441,
  VPABSQZ128rmbkz = 14442,
  VPABSQZ128rmk = 14443,
  VPABSQZ128rmkz = 14444,
  VPABSQZ128rr = 14445,
  VPABSQZ128rrk = 14446,
  VPABSQZ128rrkz = 14447,
  VPABSQZ256rm = 14448,
  VPABSQZ256rmb = 14449,
  VPABSQZ256rmbk = 14450,
  VPABSQZ256rmbkz = 14451,
  VPABSQZ256rmk = 14452,
  VPABSQZ256rmkz = 14453,
  VPABSQZ256rr = 14454,
  VPABSQZ256rrk = 14455,
  VPABSQZ256rrkz = 14456,
  VPABSQZrm = 14457,
  VPABSQZrmb = 14458,
  VPABSQZrmbk = 14459,
  VPABSQZrmbkz = 14460,
  VPABSQZrmk = 14461,
  VPABSQZrmkz = 14462,
  VPABSQZrr = 14463,
  VPABSQZrrk = 14464,
  VPABSQZrrkz = 14465,
  VPABSWYrm = 14466,
  VPABSWYrr = 14467,
  VPABSWZ128rm = 14468,
  VPABSWZ128rmk = 14469,
  VPABSWZ128rmkz = 14470,
  VPABSWZ128rr = 14471,
  VPABSWZ128rrk = 14472,
  VPABSWZ128rrkz = 14473,
  VPABSWZ256rm = 14474,
  VPABSWZ256rmk = 14475,
  VPABSWZ256rmkz = 14476,
  VPABSWZ256rr = 14477,
  VPABSWZ256rrk = 14478,
  VPABSWZ256rrkz = 14479,
  VPABSWZrm = 14480,
  VPABSWZrmk = 14481,
  VPABSWZrmkz = 14482,
  VPABSWZrr = 14483,
  VPABSWZrrk = 14484,
  VPABSWZrrkz = 14485,
  VPABSWrm = 14486,
  VPABSWrr = 14487,
  VPACKSSDWYrm = 14488,
  VPACKSSDWYrr = 14489,
  VPACKSSDWZ128rm = 14490,
  VPACKSSDWZ128rmb = 14491,
  VPACKSSDWZ128rmbk = 14492,
  VPACKSSDWZ128rmbkz = 14493,
  VPACKSSDWZ128rmk = 14494,
  VPACKSSDWZ128rmkz = 14495,
  VPACKSSDWZ128rr = 14496,
  VPACKSSDWZ128rrk = 14497,
  VPACKSSDWZ128rrkz = 14498,
  VPACKSSDWZ256rm = 14499,
  VPACKSSDWZ256rmb = 14500,
  VPACKSSDWZ256rmbk = 14501,
  VPACKSSDWZ256rmbkz = 14502,
  VPACKSSDWZ256rmk = 14503,
  VPACKSSDWZ256rmkz = 14504,
  VPACKSSDWZ256rr = 14505,
  VPACKSSDWZ256rrk = 14506,
  VPACKSSDWZ256rrkz = 14507,
  VPACKSSDWZrm = 14508,
  VPACKSSDWZrmb = 14509,
  VPACKSSDWZrmbk = 14510,
  VPACKSSDWZrmbkz = 14511,
  VPACKSSDWZrmk = 14512,
  VPACKSSDWZrmkz = 14513,
  VPACKSSDWZrr = 14514,
  VPACKSSDWZrrk = 14515,
  VPACKSSDWZrrkz = 14516,
  VPACKSSDWrm = 14517,
  VPACKSSDWrr = 14518,
  VPACKSSWBYrm = 14519,
  VPACKSSWBYrr = 14520,
  VPACKSSWBZ128rm = 14521,
  VPACKSSWBZ128rmk = 14522,
  VPACKSSWBZ128rmkz = 14523,
  VPACKSSWBZ128rr = 14524,
  VPACKSSWBZ128rrk = 14525,
  VPACKSSWBZ128rrkz = 14526,
  VPACKSSWBZ256rm = 14527,
  VPACKSSWBZ256rmk = 14528,
  VPACKSSWBZ256rmkz = 14529,
  VPACKSSWBZ256rr = 14530,
  VPACKSSWBZ256rrk = 14531,
  VPACKSSWBZ256rrkz = 14532,
  VPACKSSWBZrm = 14533,
  VPACKSSWBZrmk = 14534,
  VPACKSSWBZrmkz = 14535,
  VPACKSSWBZrr = 14536,
  VPACKSSWBZrrk = 14537,
  VPACKSSWBZrrkz = 14538,
  VPACKSSWBrm = 14539,
  VPACKSSWBrr = 14540,
  VPACKUSDWYrm = 14541,
  VPACKUSDWYrr = 14542,
  VPACKUSDWZ128rm = 14543,
  VPACKUSDWZ128rmb = 14544,
  VPACKUSDWZ128rmbk = 14545,
  VPACKUSDWZ128rmbkz = 14546,
  VPACKUSDWZ128rmk = 14547,
  VPACKUSDWZ128rmkz = 14548,
  VPACKUSDWZ128rr = 14549,
  VPACKUSDWZ128rrk = 14550,
  VPACKUSDWZ128rrkz = 14551,
  VPACKUSDWZ256rm = 14552,
  VPACKUSDWZ256rmb = 14553,
  VPACKUSDWZ256rmbk = 14554,
  VPACKUSDWZ256rmbkz = 14555,
  VPACKUSDWZ256rmk = 14556,
  VPACKUSDWZ256rmkz = 14557,
  VPACKUSDWZ256rr = 14558,
  VPACKUSDWZ256rrk = 14559,
  VPACKUSDWZ256rrkz = 14560,
  VPACKUSDWZrm = 14561,
  VPACKUSDWZrmb = 14562,
  VPACKUSDWZrmbk = 14563,
  VPACKUSDWZrmbkz = 14564,
  VPACKUSDWZrmk = 14565,
  VPACKUSDWZrmkz = 14566,
  VPACKUSDWZrr = 14567,
  VPACKUSDWZrrk = 14568,
  VPACKUSDWZrrkz = 14569,
  VPACKUSDWrm = 14570,
  VPACKUSDWrr = 14571,
  VPACKUSWBYrm = 14572,
  VPACKUSWBYrr = 14573,
  VPACKUSWBZ128rm = 14574,
  VPACKUSWBZ128rmk = 14575,
  VPACKUSWBZ128rmkz = 14576,
  VPACKUSWBZ128rr = 14577,
  VPACKUSWBZ128rrk = 14578,
  VPACKUSWBZ128rrkz = 14579,
  VPACKUSWBZ256rm = 14580,
  VPACKUSWBZ256rmk = 14581,
  VPACKUSWBZ256rmkz = 14582,
  VPACKUSWBZ256rr = 14583,
  VPACKUSWBZ256rrk = 14584,
  VPACKUSWBZ256rrkz = 14585,
  VPACKUSWBZrm = 14586,
  VPACKUSWBZrmk = 14587,
  VPACKUSWBZrmkz = 14588,
  VPACKUSWBZrr = 14589,
  VPACKUSWBZrrk = 14590,
  VPACKUSWBZrrkz = 14591,
  VPACKUSWBrm = 14592,
  VPACKUSWBrr = 14593,
  VPADDBYrm = 14594,
  VPADDBYrr = 14595,
  VPADDBZ128rm = 14596,
  VPADDBZ128rmk = 14597,
  VPADDBZ128rmkz = 14598,
  VPADDBZ128rr = 14599,
  VPADDBZ128rrk = 14600,
  VPADDBZ128rrkz = 14601,
  VPADDBZ256rm = 14602,
  VPADDBZ256rmk = 14603,
  VPADDBZ256rmkz = 14604,
  VPADDBZ256rr = 14605,
  VPADDBZ256rrk = 14606,
  VPADDBZ256rrkz = 14607,
  VPADDBZrm = 14608,
  VPADDBZrmk = 14609,
  VPADDBZrmkz = 14610,
  VPADDBZrr = 14611,
  VPADDBZrrk = 14612,
  VPADDBZrrkz = 14613,
  VPADDBrm = 14614,
  VPADDBrr = 14615,
  VPADDDYrm = 14616,
  VPADDDYrr = 14617,
  VPADDDZ128rm = 14618,
  VPADDDZ128rmb = 14619,
  VPADDDZ128rmbk = 14620,
  VPADDDZ128rmbkz = 14621,
  VPADDDZ128rmk = 14622,
  VPADDDZ128rmkz = 14623,
  VPADDDZ128rr = 14624,
  VPADDDZ128rrk = 14625,
  VPADDDZ128rrkz = 14626,
  VPADDDZ256rm = 14627,
  VPADDDZ256rmb = 14628,
  VPADDDZ256rmbk = 14629,
  VPADDDZ256rmbkz = 14630,
  VPADDDZ256rmk = 14631,
  VPADDDZ256rmkz = 14632,
  VPADDDZ256rr = 14633,
  VPADDDZ256rrk = 14634,
  VPADDDZ256rrkz = 14635,
  VPADDDZrm = 14636,
  VPADDDZrmb = 14637,
  VPADDDZrmbk = 14638,
  VPADDDZrmbkz = 14639,
  VPADDDZrmk = 14640,
  VPADDDZrmkz = 14641,
  VPADDDZrr = 14642,
  VPADDDZrrk = 14643,
  VPADDDZrrkz = 14644,
  VPADDDrm = 14645,
  VPADDDrr = 14646,
  VPADDQYrm = 14647,
  VPADDQYrr = 14648,
  VPADDQZ128rm = 14649,
  VPADDQZ128rmb = 14650,
  VPADDQZ128rmbk = 14651,
  VPADDQZ128rmbkz = 14652,
  VPADDQZ128rmk = 14653,
  VPADDQZ128rmkz = 14654,
  VPADDQZ128rr = 14655,
  VPADDQZ128rrk = 14656,
  VPADDQZ128rrkz = 14657,
  VPADDQZ256rm = 14658,
  VPADDQZ256rmb = 14659,
  VPADDQZ256rmbk = 14660,
  VPADDQZ256rmbkz = 14661,
  VPADDQZ256rmk = 14662,
  VPADDQZ256rmkz = 14663,
  VPADDQZ256rr = 14664,
  VPADDQZ256rrk = 14665,
  VPADDQZ256rrkz = 14666,
  VPADDQZrm = 14667,
  VPADDQZrmb = 14668,
  VPADDQZrmbk = 14669,
  VPADDQZrmbkz = 14670,
  VPADDQZrmk = 14671,
  VPADDQZrmkz = 14672,
  VPADDQZrr = 14673,
  VPADDQZrrk = 14674,
  VPADDQZrrkz = 14675,
  VPADDQrm = 14676,
  VPADDQrr = 14677,
  VPADDSBYrm = 14678,
  VPADDSBYrr = 14679,
  VPADDSBZ128rm = 14680,
  VPADDSBZ128rmk = 14681,
  VPADDSBZ128rmkz = 14682,
  VPADDSBZ128rr = 14683,
  VPADDSBZ128rrk = 14684,
  VPADDSBZ128rrkz = 14685,
  VPADDSBZ256rm = 14686,
  VPADDSBZ256rmk = 14687,
  VPADDSBZ256rmkz = 14688,
  VPADDSBZ256rr = 14689,
  VPADDSBZ256rrk = 14690,
  VPADDSBZ256rrkz = 14691,
  VPADDSBZrm = 14692,
  VPADDSBZrmk = 14693,
  VPADDSBZrmkz = 14694,
  VPADDSBZrr = 14695,
  VPADDSBZrrk = 14696,
  VPADDSBZrrkz = 14697,
  VPADDSBrm = 14698,
  VPADDSBrr = 14699,
  VPADDSWYrm = 14700,
  VPADDSWYrr = 14701,
  VPADDSWZ128rm = 14702,
  VPADDSWZ128rmk = 14703,
  VPADDSWZ128rmkz = 14704,
  VPADDSWZ128rr = 14705,
  VPADDSWZ128rrk = 14706,
  VPADDSWZ128rrkz = 14707,
  VPADDSWZ256rm = 14708,
  VPADDSWZ256rmk = 14709,
  VPADDSWZ256rmkz = 14710,
  VPADDSWZ256rr = 14711,
  VPADDSWZ256rrk = 14712,
  VPADDSWZ256rrkz = 14713,
  VPADDSWZrm = 14714,
  VPADDSWZrmk = 14715,
  VPADDSWZrmkz = 14716,
  VPADDSWZrr = 14717,
  VPADDSWZrrk = 14718,
  VPADDSWZrrkz = 14719,
  VPADDSWrm = 14720,
  VPADDSWrr = 14721,
  VPADDUSBYrm = 14722,
  VPADDUSBYrr = 14723,
  VPADDUSBZ128rm = 14724,
  VPADDUSBZ128rmk = 14725,
  VPADDUSBZ128rmkz = 14726,
  VPADDUSBZ128rr = 14727,
  VPADDUSBZ128rrk = 14728,
  VPADDUSBZ128rrkz = 14729,
  VPADDUSBZ256rm = 14730,
  VPADDUSBZ256rmk = 14731,
  VPADDUSBZ256rmkz = 14732,
  VPADDUSBZ256rr = 14733,
  VPADDUSBZ256rrk = 14734,
  VPADDUSBZ256rrkz = 14735,
  VPADDUSBZrm = 14736,
  VPADDUSBZrmk = 14737,
  VPADDUSBZrmkz = 14738,
  VPADDUSBZrr = 14739,
  VPADDUSBZrrk = 14740,
  VPADDUSBZrrkz = 14741,
  VPADDUSBrm = 14742,
  VPADDUSBrr = 14743,
  VPADDUSWYrm = 14744,
  VPADDUSWYrr = 14745,
  VPADDUSWZ128rm = 14746,
  VPADDUSWZ128rmk = 14747,
  VPADDUSWZ128rmkz = 14748,
  VPADDUSWZ128rr = 14749,
  VPADDUSWZ128rrk = 14750,
  VPADDUSWZ128rrkz = 14751,
  VPADDUSWZ256rm = 14752,
  VPADDUSWZ256rmk = 14753,
  VPADDUSWZ256rmkz = 14754,
  VPADDUSWZ256rr = 14755,
  VPADDUSWZ256rrk = 14756,
  VPADDUSWZ256rrkz = 14757,
  VPADDUSWZrm = 14758,
  VPADDUSWZrmk = 14759,
  VPADDUSWZrmkz = 14760,
  VPADDUSWZrr = 14761,
  VPADDUSWZrrk = 14762,
  VPADDUSWZrrkz = 14763,
  VPADDUSWrm = 14764,
  VPADDUSWrr = 14765,
  VPADDWYrm = 14766,
  VPADDWYrr = 14767,
  VPADDWZ128rm = 14768,
  VPADDWZ128rmk = 14769,
  VPADDWZ128rmkz = 14770,
  VPADDWZ128rr = 14771,
  VPADDWZ128rrk = 14772,
  VPADDWZ128rrkz = 14773,
  VPADDWZ256rm = 14774,
  VPADDWZ256rmk = 14775,
  VPADDWZ256rmkz = 14776,
  VPADDWZ256rr = 14777,
  VPADDWZ256rrk = 14778,
  VPADDWZ256rrkz = 14779,
  VPADDWZrm = 14780,
  VPADDWZrmk = 14781,
  VPADDWZrmkz = 14782,
  VPADDWZrr = 14783,
  VPADDWZrrk = 14784,
  VPADDWZrrkz = 14785,
  VPADDWrm = 14786,
  VPADDWrr = 14787,
  VPALIGNRYrmi = 14788,
  VPALIGNRYrri = 14789,
  VPALIGNRZ128rmi = 14790,
  VPALIGNRZ128rmik = 14791,
  VPALIGNRZ128rmikz = 14792,
  VPALIGNRZ128rri = 14793,
  VPALIGNRZ128rrik = 14794,
  VPALIGNRZ128rrikz = 14795,
  VPALIGNRZ256rmi = 14796,
  VPALIGNRZ256rmik = 14797,
  VPALIGNRZ256rmikz = 14798,
  VPALIGNRZ256rri = 14799,
  VPALIGNRZ256rrik = 14800,
  VPALIGNRZ256rrikz = 14801,
  VPALIGNRZrmi = 14802,
  VPALIGNRZrmik = 14803,
  VPALIGNRZrmikz = 14804,
  VPALIGNRZrri = 14805,
  VPALIGNRZrrik = 14806,
  VPALIGNRZrrikz = 14807,
  VPALIGNRrmi = 14808,
  VPALIGNRrri = 14809,
  VPANDDZ128rm = 14810,
  VPANDDZ128rmb = 14811,
  VPANDDZ128rmbk = 14812,
  VPANDDZ128rmbkz = 14813,
  VPANDDZ128rmk = 14814,
  VPANDDZ128rmkz = 14815,
  VPANDDZ128rr = 14816,
  VPANDDZ128rrk = 14817,
  VPANDDZ128rrkz = 14818,
  VPANDDZ256rm = 14819,
  VPANDDZ256rmb = 14820,
  VPANDDZ256rmbk = 14821,
  VPANDDZ256rmbkz = 14822,
  VPANDDZ256rmk = 14823,
  VPANDDZ256rmkz = 14824,
  VPANDDZ256rr = 14825,
  VPANDDZ256rrk = 14826,
  VPANDDZ256rrkz = 14827,
  VPANDDZrm = 14828,
  VPANDDZrmb = 14829,
  VPANDDZrmbk = 14830,
  VPANDDZrmbkz = 14831,
  VPANDDZrmk = 14832,
  VPANDDZrmkz = 14833,
  VPANDDZrr = 14834,
  VPANDDZrrk = 14835,
  VPANDDZrrkz = 14836,
  VPANDNDZ128rm = 14837,
  VPANDNDZ128rmb = 14838,
  VPANDNDZ128rmbk = 14839,
  VPANDNDZ128rmbkz = 14840,
  VPANDNDZ128rmk = 14841,
  VPANDNDZ128rmkz = 14842,
  VPANDNDZ128rr = 14843,
  VPANDNDZ128rrk = 14844,
  VPANDNDZ128rrkz = 14845,
  VPANDNDZ256rm = 14846,
  VPANDNDZ256rmb = 14847,
  VPANDNDZ256rmbk = 14848,
  VPANDNDZ256rmbkz = 14849,
  VPANDNDZ256rmk = 14850,
  VPANDNDZ256rmkz = 14851,
  VPANDNDZ256rr = 14852,
  VPANDNDZ256rrk = 14853,
  VPANDNDZ256rrkz = 14854,
  VPANDNDZrm = 14855,
  VPANDNDZrmb = 14856,
  VPANDNDZrmbk = 14857,
  VPANDNDZrmbkz = 14858,
  VPANDNDZrmk = 14859,
  VPANDNDZrmkz = 14860,
  VPANDNDZrr = 14861,
  VPANDNDZrrk = 14862,
  VPANDNDZrrkz = 14863,
  VPANDNQZ128rm = 14864,
  VPANDNQZ128rmb = 14865,
  VPANDNQZ128rmbk = 14866,
  VPANDNQZ128rmbkz = 14867,
  VPANDNQZ128rmk = 14868,
  VPANDNQZ128rmkz = 14869,
  VPANDNQZ128rr = 14870,
  VPANDNQZ128rrk = 14871,
  VPANDNQZ128rrkz = 14872,
  VPANDNQZ256rm = 14873,
  VPANDNQZ256rmb = 14874,
  VPANDNQZ256rmbk = 14875,
  VPANDNQZ256rmbkz = 14876,
  VPANDNQZ256rmk = 14877,
  VPANDNQZ256rmkz = 14878,
  VPANDNQZ256rr = 14879,
  VPANDNQZ256rrk = 14880,
  VPANDNQZ256rrkz = 14881,
  VPANDNQZrm = 14882,
  VPANDNQZrmb = 14883,
  VPANDNQZrmbk = 14884,
  VPANDNQZrmbkz = 14885,
  VPANDNQZrmk = 14886,
  VPANDNQZrmkz = 14887,
  VPANDNQZrr = 14888,
  VPANDNQZrrk = 14889,
  VPANDNQZrrkz = 14890,
  VPANDNYrm = 14891,
  VPANDNYrr = 14892,
  VPANDNrm = 14893,
  VPANDNrr = 14894,
  VPANDQZ128rm = 14895,
  VPANDQZ128rmb = 14896,
  VPANDQZ128rmbk = 14897,
  VPANDQZ128rmbkz = 14898,
  VPANDQZ128rmk = 14899,
  VPANDQZ128rmkz = 14900,
  VPANDQZ128rr = 14901,
  VPANDQZ128rrk = 14902,
  VPANDQZ128rrkz = 14903,
  VPANDQZ256rm = 14904,
  VPANDQZ256rmb = 14905,
  VPANDQZ256rmbk = 14906,
  VPANDQZ256rmbkz = 14907,
  VPANDQZ256rmk = 14908,
  VPANDQZ256rmkz = 14909,
  VPANDQZ256rr = 14910,
  VPANDQZ256rrk = 14911,
  VPANDQZ256rrkz = 14912,
  VPANDQZrm = 14913,
  VPANDQZrmb = 14914,
  VPANDQZrmbk = 14915,
  VPANDQZrmbkz = 14916,
  VPANDQZrmk = 14917,
  VPANDQZrmkz = 14918,
  VPANDQZrr = 14919,
  VPANDQZrrk = 14920,
  VPANDQZrrkz = 14921,
  VPANDYrm = 14922,
  VPANDYrr = 14923,
  VPANDrm = 14924,
  VPANDrr = 14925,
  VPAVGBYrm = 14926,
  VPAVGBYrr = 14927,
  VPAVGBZ128rm = 14928,
  VPAVGBZ128rmk = 14929,
  VPAVGBZ128rmkz = 14930,
  VPAVGBZ128rr = 14931,
  VPAVGBZ128rrk = 14932,
  VPAVGBZ128rrkz = 14933,
  VPAVGBZ256rm = 14934,
  VPAVGBZ256rmk = 14935,
  VPAVGBZ256rmkz = 14936,
  VPAVGBZ256rr = 14937,
  VPAVGBZ256rrk = 14938,
  VPAVGBZ256rrkz = 14939,
  VPAVGBZrm = 14940,
  VPAVGBZrmk = 14941,
  VPAVGBZrmkz = 14942,
  VPAVGBZrr = 14943,
  VPAVGBZrrk = 14944,
  VPAVGBZrrkz = 14945,
  VPAVGBrm = 14946,
  VPAVGBrr = 14947,
  VPAVGWYrm = 14948,
  VPAVGWYrr = 14949,
  VPAVGWZ128rm = 14950,
  VPAVGWZ128rmk = 14951,
  VPAVGWZ128rmkz = 14952,
  VPAVGWZ128rr = 14953,
  VPAVGWZ128rrk = 14954,
  VPAVGWZ128rrkz = 14955,
  VPAVGWZ256rm = 14956,
  VPAVGWZ256rmk = 14957,
  VPAVGWZ256rmkz = 14958,
  VPAVGWZ256rr = 14959,
  VPAVGWZ256rrk = 14960,
  VPAVGWZ256rrkz = 14961,
  VPAVGWZrm = 14962,
  VPAVGWZrmk = 14963,
  VPAVGWZrmkz = 14964,
  VPAVGWZrr = 14965,
  VPAVGWZrrk = 14966,
  VPAVGWZrrkz = 14967,
  VPAVGWrm = 14968,
  VPAVGWrr = 14969,
  VPBLENDDYrmi = 14970,
  VPBLENDDYrri = 14971,
  VPBLENDDrmi = 14972,
  VPBLENDDrri = 14973,
  VPBLENDMBZ128rm = 14974,
  VPBLENDMBZ128rmk = 14975,
  VPBLENDMBZ128rmkz = 14976,
  VPBLENDMBZ128rr = 14977,
  VPBLENDMBZ128rrk = 14978,
  VPBLENDMBZ128rrkz = 14979,
  VPBLENDMBZ256rm = 14980,
  VPBLENDMBZ256rmk = 14981,
  VPBLENDMBZ256rmkz = 14982,
  VPBLENDMBZ256rr = 14983,
  VPBLENDMBZ256rrk = 14984,
  VPBLENDMBZ256rrkz = 14985,
  VPBLENDMBZrm = 14986,
  VPBLENDMBZrmk = 14987,
  VPBLENDMBZrmkz = 14988,
  VPBLENDMBZrr = 14989,
  VPBLENDMBZrrk = 14990,
  VPBLENDMBZrrkz = 14991,
  VPBLENDMDZ128rm = 14992,
  VPBLENDMDZ128rmb = 14993,
  VPBLENDMDZ128rmbk = 14994,
  VPBLENDMDZ128rmbkz = 14995,
  VPBLENDMDZ128rmk = 14996,
  VPBLENDMDZ128rmkz = 14997,
  VPBLENDMDZ128rr = 14998,
  VPBLENDMDZ128rrk = 14999,
  VPBLENDMDZ128rrkz = 15000,
  VPBLENDMDZ256rm = 15001,
  VPBLENDMDZ256rmb = 15002,
  VPBLENDMDZ256rmbk = 15003,
  VPBLENDMDZ256rmbkz = 15004,
  VPBLENDMDZ256rmk = 15005,
  VPBLENDMDZ256rmkz = 15006,
  VPBLENDMDZ256rr = 15007,
  VPBLENDMDZ256rrk = 15008,
  VPBLENDMDZ256rrkz = 15009,
  VPBLENDMDZrm = 15010,
  VPBLENDMDZrmb = 15011,
  VPBLENDMDZrmbk = 15012,
  VPBLENDMDZrmbkz = 15013,
  VPBLENDMDZrmk = 15014,
  VPBLENDMDZrmkz = 15015,
  VPBLENDMDZrr = 15016,
  VPBLENDMDZrrk = 15017,
  VPBLENDMDZrrkz = 15018,
  VPBLENDMQZ128rm = 15019,
  VPBLENDMQZ128rmb = 15020,
  VPBLENDMQZ128rmbk = 15021,
  VPBLENDMQZ128rmbkz = 15022,
  VPBLENDMQZ128rmk = 15023,
  VPBLENDMQZ128rmkz = 15024,
  VPBLENDMQZ128rr = 15025,
  VPBLENDMQZ128rrk = 15026,
  VPBLENDMQZ128rrkz = 15027,
  VPBLENDMQZ256rm = 15028,
  VPBLENDMQZ256rmb = 15029,
  VPBLENDMQZ256rmbk = 15030,
  VPBLENDMQZ256rmbkz = 15031,
  VPBLENDMQZ256rmk = 15032,
  VPBLENDMQZ256rmkz = 15033,
  VPBLENDMQZ256rr = 15034,
  VPBLENDMQZ256rrk = 15035,
  VPBLENDMQZ256rrkz = 15036,
  VPBLENDMQZrm = 15037,
  VPBLENDMQZrmb = 15038,
  VPBLENDMQZrmbk = 15039,
  VPBLENDMQZrmbkz = 15040,
  VPBLENDMQZrmk = 15041,
  VPBLENDMQZrmkz = 15042,
  VPBLENDMQZrr = 15043,
  VPBLENDMQZrrk = 15044,
  VPBLENDMQZrrkz = 15045,
  VPBLENDMWZ128rm = 15046,
  VPBLENDMWZ128rmk = 15047,
  VPBLENDMWZ128rmkz = 15048,
  VPBLENDMWZ128rr = 15049,
  VPBLENDMWZ128rrk = 15050,
  VPBLENDMWZ128rrkz = 15051,
  VPBLENDMWZ256rm = 15052,
  VPBLENDMWZ256rmk = 15053,
  VPBLENDMWZ256rmkz = 15054,
  VPBLENDMWZ256rr = 15055,
  VPBLENDMWZ256rrk = 15056,
  VPBLENDMWZ256rrkz = 15057,
  VPBLENDMWZrm = 15058,
  VPBLENDMWZrmk = 15059,
  VPBLENDMWZrmkz = 15060,
  VPBLENDMWZrr = 15061,
  VPBLENDMWZrrk = 15062,
  VPBLENDMWZrrkz = 15063,
  VPBLENDVBYrmr = 15064,
  VPBLENDVBYrrr = 15065,
  VPBLENDVBrmr = 15066,
  VPBLENDVBrrr = 15067,
  VPBLENDWYrmi = 15068,
  VPBLENDWYrri = 15069,
  VPBLENDWrmi = 15070,
  VPBLENDWrri = 15071,
  VPBROADCASTBYrm = 15072,
  VPBROADCASTBYrr = 15073,
  VPBROADCASTBZ128rm = 15074,
  VPBROADCASTBZ128rmk = 15075,
  VPBROADCASTBZ128rmkz = 15076,
  VPBROADCASTBZ128rr = 15077,
  VPBROADCASTBZ128rrk = 15078,
  VPBROADCASTBZ128rrkz = 15079,
  VPBROADCASTBZ256rm = 15080,
  VPBROADCASTBZ256rmk = 15081,
  VPBROADCASTBZ256rmkz = 15082,
  VPBROADCASTBZ256rr = 15083,
  VPBROADCASTBZ256rrk = 15084,
  VPBROADCASTBZ256rrkz = 15085,
  VPBROADCASTBZrm = 15086,
  VPBROADCASTBZrmk = 15087,
  VPBROADCASTBZrmkz = 15088,
  VPBROADCASTBZrr = 15089,
  VPBROADCASTBZrrk = 15090,
  VPBROADCASTBZrrkz = 15091,
  VPBROADCASTBrZ128rr = 15092,
  VPBROADCASTBrZ128rrk = 15093,
  VPBROADCASTBrZ128rrkz = 15094,
  VPBROADCASTBrZ256rr = 15095,
  VPBROADCASTBrZ256rrk = 15096,
  VPBROADCASTBrZ256rrkz = 15097,
  VPBROADCASTBrZrr = 15098,
  VPBROADCASTBrZrrk = 15099,
  VPBROADCASTBrZrrkz = 15100,
  VPBROADCASTBrm = 15101,
  VPBROADCASTBrr = 15102,
  VPBROADCASTDYrm = 15103,
  VPBROADCASTDYrr = 15104,
  VPBROADCASTDZ128rm = 15105,
  VPBROADCASTDZ128rmk = 15106,
  VPBROADCASTDZ128rmkz = 15107,
  VPBROADCASTDZ128rr = 15108,
  VPBROADCASTDZ128rrk = 15109,
  VPBROADCASTDZ128rrkz = 15110,
  VPBROADCASTDZ256rm = 15111,
  VPBROADCASTDZ256rmk = 15112,
  VPBROADCASTDZ256rmkz = 15113,
  VPBROADCASTDZ256rr = 15114,
  VPBROADCASTDZ256rrk = 15115,
  VPBROADCASTDZ256rrkz = 15116,
  VPBROADCASTDZrm = 15117,
  VPBROADCASTDZrmk = 15118,
  VPBROADCASTDZrmkz = 15119,
  VPBROADCASTDZrr = 15120,
  VPBROADCASTDZrrk = 15121,
  VPBROADCASTDZrrkz = 15122,
  VPBROADCASTDrZ128rr = 15123,
  VPBROADCASTDrZ128rrk = 15124,
  VPBROADCASTDrZ128rrkz = 15125,
  VPBROADCASTDrZ256rr = 15126,
  VPBROADCASTDrZ256rrk = 15127,
  VPBROADCASTDrZ256rrkz = 15128,
  VPBROADCASTDrZrr = 15129,
  VPBROADCASTDrZrrk = 15130,
  VPBROADCASTDrZrrkz = 15131,
  VPBROADCASTDrm = 15132,
  VPBROADCASTDrr = 15133,
  VPBROADCASTMB2QZ128rr = 15134,
  VPBROADCASTMB2QZ256rr = 15135,
  VPBROADCASTMB2QZrr = 15136,
  VPBROADCASTMW2DZ128rr = 15137,
  VPBROADCASTMW2DZ256rr = 15138,
  VPBROADCASTMW2DZrr = 15139,
  VPBROADCASTQYrm = 15140,
  VPBROADCASTQYrr = 15141,
  VPBROADCASTQZ128rm = 15142,
  VPBROADCASTQZ128rmk = 15143,
  VPBROADCASTQZ128rmkz = 15144,
  VPBROADCASTQZ128rr = 15145,
  VPBROADCASTQZ128rrk = 15146,
  VPBROADCASTQZ128rrkz = 15147,
  VPBROADCASTQZ256rm = 15148,
  VPBROADCASTQZ256rmk = 15149,
  VPBROADCASTQZ256rmkz = 15150,
  VPBROADCASTQZ256rr = 15151,
  VPBROADCASTQZ256rrk = 15152,
  VPBROADCASTQZ256rrkz = 15153,
  VPBROADCASTQZrm = 15154,
  VPBROADCASTQZrmk = 15155,
  VPBROADCASTQZrmkz = 15156,
  VPBROADCASTQZrr = 15157,
  VPBROADCASTQZrrk = 15158,
  VPBROADCASTQZrrkz = 15159,
  VPBROADCASTQrZ128rr = 15160,
  VPBROADCASTQrZ128rrk = 15161,
  VPBROADCASTQrZ128rrkz = 15162,
  VPBROADCASTQrZ256rr = 15163,
  VPBROADCASTQrZ256rrk = 15164,
  VPBROADCASTQrZ256rrkz = 15165,
  VPBROADCASTQrZrr = 15166,
  VPBROADCASTQrZrrk = 15167,
  VPBROADCASTQrZrrkz = 15168,
  VPBROADCASTQrm = 15169,
  VPBROADCASTQrr = 15170,
  VPBROADCASTWYrm = 15171,
  VPBROADCASTWYrr = 15172,
  VPBROADCASTWZ128rm = 15173,
  VPBROADCASTWZ128rmk = 15174,
  VPBROADCASTWZ128rmkz = 15175,
  VPBROADCASTWZ128rr = 15176,
  VPBROADCASTWZ128rrk = 15177,
  VPBROADCASTWZ128rrkz = 15178,
  VPBROADCASTWZ256rm = 15179,
  VPBROADCASTWZ256rmk = 15180,
  VPBROADCASTWZ256rmkz = 15181,
  VPBROADCASTWZ256rr = 15182,
  VPBROADCASTWZ256rrk = 15183,
  VPBROADCASTWZ256rrkz = 15184,
  VPBROADCASTWZrm = 15185,
  VPBROADCASTWZrmk = 15186,
  VPBROADCASTWZrmkz = 15187,
  VPBROADCASTWZrr = 15188,
  VPBROADCASTWZrrk = 15189,
  VPBROADCASTWZrrkz = 15190,
  VPBROADCASTWrZ128rr = 15191,
  VPBROADCASTWrZ128rrk = 15192,
  VPBROADCASTWrZ128rrkz = 15193,
  VPBROADCASTWrZ256rr = 15194,
  VPBROADCASTWrZ256rrk = 15195,
  VPBROADCASTWrZ256rrkz = 15196,
  VPBROADCASTWrZrr = 15197,
  VPBROADCASTWrZrrk = 15198,
  VPBROADCASTWrZrrkz = 15199,
  VPBROADCASTWrm = 15200,
  VPBROADCASTWrr = 15201,
  VPCLMULQDQYrmi = 15202,
  VPCLMULQDQYrri = 15203,
  VPCLMULQDQZ128rmi = 15204,
  VPCLMULQDQZ128rri = 15205,
  VPCLMULQDQZ256rmi = 15206,
  VPCLMULQDQZ256rri = 15207,
  VPCLMULQDQZrmi = 15208,
  VPCLMULQDQZrri = 15209,
  VPCLMULQDQrmi = 15210,
  VPCLMULQDQrri = 15211,
  VPCMOVYrmr = 15212,
  VPCMOVYrrm = 15213,
  VPCMOVYrrr = 15214,
  VPCMOVYrrr_REV = 15215,
  VPCMOVrmr = 15216,
  VPCMOVrrm = 15217,
  VPCMOVrrr = 15218,
  VPCMOVrrr_REV = 15219,
  VPCMPBZ128rmi = 15220,
  VPCMPBZ128rmik = 15221,
  VPCMPBZ128rri = 15222,
  VPCMPBZ128rrik = 15223,
  VPCMPBZ256rmi = 15224,
  VPCMPBZ256rmik = 15225,
  VPCMPBZ256rri = 15226,
  VPCMPBZ256rrik = 15227,
  VPCMPBZrmi = 15228,
  VPCMPBZrmik = 15229,
  VPCMPBZrri = 15230,
  VPCMPBZrrik = 15231,
  VPCMPDZ128rmbi = 15232,
  VPCMPDZ128rmbik = 15233,
  VPCMPDZ128rmi = 15234,
  VPCMPDZ128rmik = 15235,
  VPCMPDZ128rri = 15236,
  VPCMPDZ128rrik = 15237,
  VPCMPDZ256rmbi = 15238,
  VPCMPDZ256rmbik = 15239,
  VPCMPDZ256rmi = 15240,
  VPCMPDZ256rmik = 15241,
  VPCMPDZ256rri = 15242,
  VPCMPDZ256rrik = 15243,
  VPCMPDZrmbi = 15244,
  VPCMPDZrmbik = 15245,
  VPCMPDZrmi = 15246,
  VPCMPDZrmik = 15247,
  VPCMPDZrri = 15248,
  VPCMPDZrrik = 15249,
  VPCMPEQBYrm = 15250,
  VPCMPEQBYrr = 15251,
  VPCMPEQBZ128rm = 15252,
  VPCMPEQBZ128rmk = 15253,
  VPCMPEQBZ128rr = 15254,
  VPCMPEQBZ128rrk = 15255,
  VPCMPEQBZ256rm = 15256,
  VPCMPEQBZ256rmk = 15257,
  VPCMPEQBZ256rr = 15258,
  VPCMPEQBZ256rrk = 15259,
  VPCMPEQBZrm = 15260,
  VPCMPEQBZrmk = 15261,
  VPCMPEQBZrr = 15262,
  VPCMPEQBZrrk = 15263,
  VPCMPEQBrm = 15264,
  VPCMPEQBrr = 15265,
  VPCMPEQDYrm = 15266,
  VPCMPEQDYrr = 15267,
  VPCMPEQDZ128rm = 15268,
  VPCMPEQDZ128rmb = 15269,
  VPCMPEQDZ128rmbk = 15270,
  VPCMPEQDZ128rmk = 15271,
  VPCMPEQDZ128rr = 15272,
  VPCMPEQDZ128rrk = 15273,
  VPCMPEQDZ256rm = 15274,
  VPCMPEQDZ256rmb = 15275,
  VPCMPEQDZ256rmbk = 15276,
  VPCMPEQDZ256rmk = 15277,
  VPCMPEQDZ256rr = 15278,
  VPCMPEQDZ256rrk = 15279,
  VPCMPEQDZrm = 15280,
  VPCMPEQDZrmb = 15281,
  VPCMPEQDZrmbk = 15282,
  VPCMPEQDZrmk = 15283,
  VPCMPEQDZrr = 15284,
  VPCMPEQDZrrk = 15285,
  VPCMPEQDrm = 15286,
  VPCMPEQDrr = 15287,
  VPCMPEQQYrm = 15288,
  VPCMPEQQYrr = 15289,
  VPCMPEQQZ128rm = 15290,
  VPCMPEQQZ128rmb = 15291,
  VPCMPEQQZ128rmbk = 15292,
  VPCMPEQQZ128rmk = 15293,
  VPCMPEQQZ128rr = 15294,
  VPCMPEQQZ128rrk = 15295,
  VPCMPEQQZ256rm = 15296,
  VPCMPEQQZ256rmb = 15297,
  VPCMPEQQZ256rmbk = 15298,
  VPCMPEQQZ256rmk = 15299,
  VPCMPEQQZ256rr = 15300,
  VPCMPEQQZ256rrk = 15301,
  VPCMPEQQZrm = 15302,
  VPCMPEQQZrmb = 15303,
  VPCMPEQQZrmbk = 15304,
  VPCMPEQQZrmk = 15305,
  VPCMPEQQZrr = 15306,
  VPCMPEQQZrrk = 15307,
  VPCMPEQQrm = 15308,
  VPCMPEQQrr = 15309,
  VPCMPEQWYrm = 15310,
  VPCMPEQWYrr = 15311,
  VPCMPEQWZ128rm = 15312,
  VPCMPEQWZ128rmk = 15313,
  VPCMPEQWZ128rr = 15314,
  VPCMPEQWZ128rrk = 15315,
  VPCMPEQWZ256rm = 15316,
  VPCMPEQWZ256rmk = 15317,
  VPCMPEQWZ256rr = 15318,
  VPCMPEQWZ256rrk = 15319,
  VPCMPEQWZrm = 15320,
  VPCMPEQWZrmk = 15321,
  VPCMPEQWZrr = 15322,
  VPCMPEQWZrrk = 15323,
  VPCMPEQWrm = 15324,
  VPCMPEQWrr = 15325,
  VPCMPESTRIrmi = 15326,
  VPCMPESTRIrri = 15327,
  VPCMPESTRMrmi = 15328,
  VPCMPESTRMrri = 15329,
  VPCMPGTBYrm = 15330,
  VPCMPGTBYrr = 15331,
  VPCMPGTBZ128rm = 15332,
  VPCMPGTBZ128rmk = 15333,
  VPCMPGTBZ128rr = 15334,
  VPCMPGTBZ128rrk = 15335,
  VPCMPGTBZ256rm = 15336,
  VPCMPGTBZ256rmk = 15337,
  VPCMPGTBZ256rr = 15338,
  VPCMPGTBZ256rrk = 15339,
  VPCMPGTBZrm = 15340,
  VPCMPGTBZrmk = 15341,
  VPCMPGTBZrr = 15342,
  VPCMPGTBZrrk = 15343,
  VPCMPGTBrm = 15344,
  VPCMPGTBrr = 15345,
  VPCMPGTDYrm = 15346,
  VPCMPGTDYrr = 15347,
  VPCMPGTDZ128rm = 15348,
  VPCMPGTDZ128rmb = 15349,
  VPCMPGTDZ128rmbk = 15350,
  VPCMPGTDZ128rmk = 15351,
  VPCMPGTDZ128rr = 15352,
  VPCMPGTDZ128rrk = 15353,
  VPCMPGTDZ256rm = 15354,
  VPCMPGTDZ256rmb = 15355,
  VPCMPGTDZ256rmbk = 15356,
  VPCMPGTDZ256rmk = 15357,
  VPCMPGTDZ256rr = 15358,
  VPCMPGTDZ256rrk = 15359,
  VPCMPGTDZrm = 15360,
  VPCMPGTDZrmb = 15361,
  VPCMPGTDZrmbk = 15362,
  VPCMPGTDZrmk = 15363,
  VPCMPGTDZrr = 15364,
  VPCMPGTDZrrk = 15365,
  VPCMPGTDrm = 15366,
  VPCMPGTDrr = 15367,
  VPCMPGTQYrm = 15368,
  VPCMPGTQYrr = 15369,
  VPCMPGTQZ128rm = 15370,
  VPCMPGTQZ128rmb = 15371,
  VPCMPGTQZ128rmbk = 15372,
  VPCMPGTQZ128rmk = 15373,
  VPCMPGTQZ128rr = 15374,
  VPCMPGTQZ128rrk = 15375,
  VPCMPGTQZ256rm = 15376,
  VPCMPGTQZ256rmb = 15377,
  VPCMPGTQZ256rmbk = 15378,
  VPCMPGTQZ256rmk = 15379,
  VPCMPGTQZ256rr = 15380,
  VPCMPGTQZ256rrk = 15381,
  VPCMPGTQZrm = 15382,
  VPCMPGTQZrmb = 15383,
  VPCMPGTQZrmbk = 15384,
  VPCMPGTQZrmk = 15385,
  VPCMPGTQZrr = 15386,
  VPCMPGTQZrrk = 15387,
  VPCMPGTQrm = 15388,
  VPCMPGTQrr = 15389,
  VPCMPGTWYrm = 15390,
  VPCMPGTWYrr = 15391,
  VPCMPGTWZ128rm = 15392,
  VPCMPGTWZ128rmk = 15393,
  VPCMPGTWZ128rr = 15394,
  VPCMPGTWZ128rrk = 15395,
  VPCMPGTWZ256rm = 15396,
  VPCMPGTWZ256rmk = 15397,
  VPCMPGTWZ256rr = 15398,
  VPCMPGTWZ256rrk = 15399,
  VPCMPGTWZrm = 15400,
  VPCMPGTWZrmk = 15401,
  VPCMPGTWZrr = 15402,
  VPCMPGTWZrrk = 15403,
  VPCMPGTWrm = 15404,
  VPCMPGTWrr = 15405,
  VPCMPISTRIrmi = 15406,
  VPCMPISTRIrri = 15407,
  VPCMPISTRMrmi = 15408,
  VPCMPISTRMrri = 15409,
  VPCMPQZ128rmbi = 15410,
  VPCMPQZ128rmbik = 15411,
  VPCMPQZ128rmi = 15412,
  VPCMPQZ128rmik = 15413,
  VPCMPQZ128rri = 15414,
  VPCMPQZ128rrik = 15415,
  VPCMPQZ256rmbi = 15416,
  VPCMPQZ256rmbik = 15417,
  VPCMPQZ256rmi = 15418,
  VPCMPQZ256rmik = 15419,
  VPCMPQZ256rri = 15420,
  VPCMPQZ256rrik = 15421,
  VPCMPQZrmbi = 15422,
  VPCMPQZrmbik = 15423,
  VPCMPQZrmi = 15424,
  VPCMPQZrmik = 15425,
  VPCMPQZrri = 15426,
  VPCMPQZrrik = 15427,
  VPCMPUBZ128rmi = 15428,
  VPCMPUBZ128rmik = 15429,
  VPCMPUBZ128rri = 15430,
  VPCMPUBZ128rrik = 15431,
  VPCMPUBZ256rmi = 15432,
  VPCMPUBZ256rmik = 15433,
  VPCMPUBZ256rri = 15434,
  VPCMPUBZ256rrik = 15435,
  VPCMPUBZrmi = 15436,
  VPCMPUBZrmik = 15437,
  VPCMPUBZrri = 15438,
  VPCMPUBZrrik = 15439,
  VPCMPUDZ128rmbi = 15440,
  VPCMPUDZ128rmbik = 15441,
  VPCMPUDZ128rmi = 15442,
  VPCMPUDZ128rmik = 15443,
  VPCMPUDZ128rri = 15444,
  VPCMPUDZ128rrik = 15445,
  VPCMPUDZ256rmbi = 15446,
  VPCMPUDZ256rmbik = 15447,
  VPCMPUDZ256rmi = 15448,
  VPCMPUDZ256rmik = 15449,
  VPCMPUDZ256rri = 15450,
  VPCMPUDZ256rrik = 15451,
  VPCMPUDZrmbi = 15452,
  VPCMPUDZrmbik = 15453,
  VPCMPUDZrmi = 15454,
  VPCMPUDZrmik = 15455,
  VPCMPUDZrri = 15456,
  VPCMPUDZrrik = 15457,
  VPCMPUQZ128rmbi = 15458,
  VPCMPUQZ128rmbik = 15459,
  VPCMPUQZ128rmi = 15460,
  VPCMPUQZ128rmik = 15461,
  VPCMPUQZ128rri = 15462,
  VPCMPUQZ128rrik = 15463,
  VPCMPUQZ256rmbi = 15464,
  VPCMPUQZ256rmbik = 15465,
  VPCMPUQZ256rmi = 15466,
  VPCMPUQZ256rmik = 15467,
  VPCMPUQZ256rri = 15468,
  VPCMPUQZ256rrik = 15469,
  VPCMPUQZrmbi = 15470,
  VPCMPUQZrmbik = 15471,
  VPCMPUQZrmi = 15472,
  VPCMPUQZrmik = 15473,
  VPCMPUQZrri = 15474,
  VPCMPUQZrrik = 15475,
  VPCMPUWZ128rmi = 15476,
  VPCMPUWZ128rmik = 15477,
  VPCMPUWZ128rri = 15478,
  VPCMPUWZ128rrik = 15479,
  VPCMPUWZ256rmi = 15480,
  VPCMPUWZ256rmik = 15481,
  VPCMPUWZ256rri = 15482,
  VPCMPUWZ256rrik = 15483,
  VPCMPUWZrmi = 15484,
  VPCMPUWZrmik = 15485,
  VPCMPUWZrri = 15486,
  VPCMPUWZrrik = 15487,
  VPCMPWZ128rmi = 15488,
  VPCMPWZ128rmik = 15489,
  VPCMPWZ128rri = 15490,
  VPCMPWZ128rrik = 15491,
  VPCMPWZ256rmi = 15492,
  VPCMPWZ256rmik = 15493,
  VPCMPWZ256rri = 15494,
  VPCMPWZ256rrik = 15495,
  VPCMPWZrmi = 15496,
  VPCMPWZrmik = 15497,
  VPCMPWZrri = 15498,
  VPCMPWZrrik = 15499,
  VPCOMBmi = 15500,
  VPCOMBri = 15501,
  VPCOMDmi = 15502,
  VPCOMDri = 15503,
  VPCOMPRESSBZ128mr = 15504,
  VPCOMPRESSBZ128mrk = 15505,
  VPCOMPRESSBZ128rr = 15506,
  VPCOMPRESSBZ128rrk = 15507,
  VPCOMPRESSBZ128rrkz = 15508,
  VPCOMPRESSBZ256mr = 15509,
  VPCOMPRESSBZ256mrk = 15510,
  VPCOMPRESSBZ256rr = 15511,
  VPCOMPRESSBZ256rrk = 15512,
  VPCOMPRESSBZ256rrkz = 15513,
  VPCOMPRESSBZmr = 15514,
  VPCOMPRESSBZmrk = 15515,
  VPCOMPRESSBZrr = 15516,
  VPCOMPRESSBZrrk = 15517,
  VPCOMPRESSBZrrkz = 15518,
  VPCOMPRESSDZ128mr = 15519,
  VPCOMPRESSDZ128mrk = 15520,
  VPCOMPRESSDZ128rr = 15521,
  VPCOMPRESSDZ128rrk = 15522,
  VPCOMPRESSDZ128rrkz = 15523,
  VPCOMPRESSDZ256mr = 15524,
  VPCOMPRESSDZ256mrk = 15525,
  VPCOMPRESSDZ256rr = 15526,
  VPCOMPRESSDZ256rrk = 15527,
  VPCOMPRESSDZ256rrkz = 15528,
  VPCOMPRESSDZmr = 15529,
  VPCOMPRESSDZmrk = 15530,
  VPCOMPRESSDZrr = 15531,
  VPCOMPRESSDZrrk = 15532,
  VPCOMPRESSDZrrkz = 15533,
  VPCOMPRESSQZ128mr = 15534,
  VPCOMPRESSQZ128mrk = 15535,
  VPCOMPRESSQZ128rr = 15536,
  VPCOMPRESSQZ128rrk = 15537,
  VPCOMPRESSQZ128rrkz = 15538,
  VPCOMPRESSQZ256mr = 15539,
  VPCOMPRESSQZ256mrk = 15540,
  VPCOMPRESSQZ256rr = 15541,
  VPCOMPRESSQZ256rrk = 15542,
  VPCOMPRESSQZ256rrkz = 15543,
  VPCOMPRESSQZmr = 15544,
  VPCOMPRESSQZmrk = 15545,
  VPCOMPRESSQZrr = 15546,
  VPCOMPRESSQZrrk = 15547,
  VPCOMPRESSQZrrkz = 15548,
  VPCOMPRESSWZ128mr = 15549,
  VPCOMPRESSWZ128mrk = 15550,
  VPCOMPRESSWZ128rr = 15551,
  VPCOMPRESSWZ128rrk = 15552,
  VPCOMPRESSWZ128rrkz = 15553,
  VPCOMPRESSWZ256mr = 15554,
  VPCOMPRESSWZ256mrk = 15555,
  VPCOMPRESSWZ256rr = 15556,
  VPCOMPRESSWZ256rrk = 15557,
  VPCOMPRESSWZ256rrkz = 15558,
  VPCOMPRESSWZmr = 15559,
  VPCOMPRESSWZmrk = 15560,
  VPCOMPRESSWZrr = 15561,
  VPCOMPRESSWZrrk = 15562,
  VPCOMPRESSWZrrkz = 15563,
  VPCOMQmi = 15564,
  VPCOMQri = 15565,
  VPCOMUBmi = 15566,
  VPCOMUBri = 15567,
  VPCOMUDmi = 15568,
  VPCOMUDri = 15569,
  VPCOMUQmi = 15570,
  VPCOMUQri = 15571,
  VPCOMUWmi = 15572,
  VPCOMUWri = 15573,
  VPCOMWmi = 15574,
  VPCOMWri = 15575,
  VPCONFLICTDZ128rm = 15576,
  VPCONFLICTDZ128rmb = 15577,
  VPCONFLICTDZ128rmbk = 15578,
  VPCONFLICTDZ128rmbkz = 15579,
  VPCONFLICTDZ128rmk = 15580,
  VPCONFLICTDZ128rmkz = 15581,
  VPCONFLICTDZ128rr = 15582,
  VPCONFLICTDZ128rrk = 15583,
  VPCONFLICTDZ128rrkz = 15584,
  VPCONFLICTDZ256rm = 15585,
  VPCONFLICTDZ256rmb = 15586,
  VPCONFLICTDZ256rmbk = 15587,
  VPCONFLICTDZ256rmbkz = 15588,
  VPCONFLICTDZ256rmk = 15589,
  VPCONFLICTDZ256rmkz = 15590,
  VPCONFLICTDZ256rr = 15591,
  VPCONFLICTDZ256rrk = 15592,
  VPCONFLICTDZ256rrkz = 15593,
  VPCONFLICTDZrm = 15594,
  VPCONFLICTDZrmb = 15595,
  VPCONFLICTDZrmbk = 15596,
  VPCONFLICTDZrmbkz = 15597,
  VPCONFLICTDZrmk = 15598,
  VPCONFLICTDZrmkz = 15599,
  VPCONFLICTDZrr = 15600,
  VPCONFLICTDZrrk = 15601,
  VPCONFLICTDZrrkz = 15602,
  VPCONFLICTQZ128rm = 15603,
  VPCONFLICTQZ128rmb = 15604,
  VPCONFLICTQZ128rmbk = 15605,
  VPCONFLICTQZ128rmbkz = 15606,
  VPCONFLICTQZ128rmk = 15607,
  VPCONFLICTQZ128rmkz = 15608,
  VPCONFLICTQZ128rr = 15609,
  VPCONFLICTQZ128rrk = 15610,
  VPCONFLICTQZ128rrkz = 15611,
  VPCONFLICTQZ256rm = 15612,
  VPCONFLICTQZ256rmb = 15613,
  VPCONFLICTQZ256rmbk = 15614,
  VPCONFLICTQZ256rmbkz = 15615,
  VPCONFLICTQZ256rmk = 15616,
  VPCONFLICTQZ256rmkz = 15617,
  VPCONFLICTQZ256rr = 15618,
  VPCONFLICTQZ256rrk = 15619,
  VPCONFLICTQZ256rrkz = 15620,
  VPCONFLICTQZrm = 15621,
  VPCONFLICTQZrmb = 15622,
  VPCONFLICTQZrmbk = 15623,
  VPCONFLICTQZrmbkz = 15624,
  VPCONFLICTQZrmk = 15625,
  VPCONFLICTQZrmkz = 15626,
  VPCONFLICTQZrr = 15627,
  VPCONFLICTQZrrk = 15628,
  VPCONFLICTQZrrkz = 15629,
  VPDPBSSDSYrm = 15630,
  VPDPBSSDSYrr = 15631,
  VPDPBSSDSZ128m = 15632,
  VPDPBSSDSZ128mb = 15633,
  VPDPBSSDSZ128mbk = 15634,
  VPDPBSSDSZ128mbkz = 15635,
  VPDPBSSDSZ128mk = 15636,
  VPDPBSSDSZ128mkz = 15637,
  VPDPBSSDSZ128r = 15638,
  VPDPBSSDSZ128rk = 15639,
  VPDPBSSDSZ128rkz = 15640,
  VPDPBSSDSZ256m = 15641,
  VPDPBSSDSZ256mb = 15642,
  VPDPBSSDSZ256mbk = 15643,
  VPDPBSSDSZ256mbkz = 15644,
  VPDPBSSDSZ256mk = 15645,
  VPDPBSSDSZ256mkz = 15646,
  VPDPBSSDSZ256r = 15647,
  VPDPBSSDSZ256rk = 15648,
  VPDPBSSDSZ256rkz = 15649,
  VPDPBSSDSZm = 15650,
  VPDPBSSDSZmb = 15651,
  VPDPBSSDSZmbk = 15652,
  VPDPBSSDSZmbkz = 15653,
  VPDPBSSDSZmk = 15654,
  VPDPBSSDSZmkz = 15655,
  VPDPBSSDSZr = 15656,
  VPDPBSSDSZrk = 15657,
  VPDPBSSDSZrkz = 15658,
  VPDPBSSDSrm = 15659,
  VPDPBSSDSrr = 15660,
  VPDPBSSDYrm = 15661,
  VPDPBSSDYrr = 15662,
  VPDPBSSDZ128m = 15663,
  VPDPBSSDZ128mb = 15664,
  VPDPBSSDZ128mbk = 15665,
  VPDPBSSDZ128mbkz = 15666,
  VPDPBSSDZ128mk = 15667,
  VPDPBSSDZ128mkz = 15668,
  VPDPBSSDZ128r = 15669,
  VPDPBSSDZ128rk = 15670,
  VPDPBSSDZ128rkz = 15671,
  VPDPBSSDZ256m = 15672,
  VPDPBSSDZ256mb = 15673,
  VPDPBSSDZ256mbk = 15674,
  VPDPBSSDZ256mbkz = 15675,
  VPDPBSSDZ256mk = 15676,
  VPDPBSSDZ256mkz = 15677,
  VPDPBSSDZ256r = 15678,
  VPDPBSSDZ256rk = 15679,
  VPDPBSSDZ256rkz = 15680,
  VPDPBSSDZm = 15681,
  VPDPBSSDZmb = 15682,
  VPDPBSSDZmbk = 15683,
  VPDPBSSDZmbkz = 15684,
  VPDPBSSDZmk = 15685,
  VPDPBSSDZmkz = 15686,
  VPDPBSSDZr = 15687,
  VPDPBSSDZrk = 15688,
  VPDPBSSDZrkz = 15689,
  VPDPBSSDrm = 15690,
  VPDPBSSDrr = 15691,
  VPDPBSUDSYrm = 15692,
  VPDPBSUDSYrr = 15693,
  VPDPBSUDSZ128m = 15694,
  VPDPBSUDSZ128mb = 15695,
  VPDPBSUDSZ128mbk = 15696,
  VPDPBSUDSZ128mbkz = 15697,
  VPDPBSUDSZ128mk = 15698,
  VPDPBSUDSZ128mkz = 15699,
  VPDPBSUDSZ128r = 15700,
  VPDPBSUDSZ128rk = 15701,
  VPDPBSUDSZ128rkz = 15702,
  VPDPBSUDSZ256m = 15703,
  VPDPBSUDSZ256mb = 15704,
  VPDPBSUDSZ256mbk = 15705,
  VPDPBSUDSZ256mbkz = 15706,
  VPDPBSUDSZ256mk = 15707,
  VPDPBSUDSZ256mkz = 15708,
  VPDPBSUDSZ256r = 15709,
  VPDPBSUDSZ256rk = 15710,
  VPDPBSUDSZ256rkz = 15711,
  VPDPBSUDSZm = 15712,
  VPDPBSUDSZmb = 15713,
  VPDPBSUDSZmbk = 15714,
  VPDPBSUDSZmbkz = 15715,
  VPDPBSUDSZmk = 15716,
  VPDPBSUDSZmkz = 15717,
  VPDPBSUDSZr = 15718,
  VPDPBSUDSZrk = 15719,
  VPDPBSUDSZrkz = 15720,
  VPDPBSUDSrm = 15721,
  VPDPBSUDSrr = 15722,
  VPDPBSUDYrm = 15723,
  VPDPBSUDYrr = 15724,
  VPDPBSUDZ128m = 15725,
  VPDPBSUDZ128mb = 15726,
  VPDPBSUDZ128mbk = 15727,
  VPDPBSUDZ128mbkz = 15728,
  VPDPBSUDZ128mk = 15729,
  VPDPBSUDZ128mkz = 15730,
  VPDPBSUDZ128r = 15731,
  VPDPBSUDZ128rk = 15732,
  VPDPBSUDZ128rkz = 15733,
  VPDPBSUDZ256m = 15734,
  VPDPBSUDZ256mb = 15735,
  VPDPBSUDZ256mbk = 15736,
  VPDPBSUDZ256mbkz = 15737,
  VPDPBSUDZ256mk = 15738,
  VPDPBSUDZ256mkz = 15739,
  VPDPBSUDZ256r = 15740,
  VPDPBSUDZ256rk = 15741,
  VPDPBSUDZ256rkz = 15742,
  VPDPBSUDZm = 15743,
  VPDPBSUDZmb = 15744,
  VPDPBSUDZmbk = 15745,
  VPDPBSUDZmbkz = 15746,
  VPDPBSUDZmk = 15747,
  VPDPBSUDZmkz = 15748,
  VPDPBSUDZr = 15749,
  VPDPBSUDZrk = 15750,
  VPDPBSUDZrkz = 15751,
  VPDPBSUDrm = 15752,
  VPDPBSUDrr = 15753,
  VPDPBUSDSYrm = 15754,
  VPDPBUSDSYrr = 15755,
  VPDPBUSDSZ128m = 15756,
  VPDPBUSDSZ128mb = 15757,
  VPDPBUSDSZ128mbk = 15758,
  VPDPBUSDSZ128mbkz = 15759,
  VPDPBUSDSZ128mk = 15760,
  VPDPBUSDSZ128mkz = 15761,
  VPDPBUSDSZ128r = 15762,
  VPDPBUSDSZ128rk = 15763,
  VPDPBUSDSZ128rkz = 15764,
  VPDPBUSDSZ256m = 15765,
  VPDPBUSDSZ256mb = 15766,
  VPDPBUSDSZ256mbk = 15767,
  VPDPBUSDSZ256mbkz = 15768,
  VPDPBUSDSZ256mk = 15769,
  VPDPBUSDSZ256mkz = 15770,
  VPDPBUSDSZ256r = 15771,
  VPDPBUSDSZ256rk = 15772,
  VPDPBUSDSZ256rkz = 15773,
  VPDPBUSDSZm = 15774,
  VPDPBUSDSZmb = 15775,
  VPDPBUSDSZmbk = 15776,
  VPDPBUSDSZmbkz = 15777,
  VPDPBUSDSZmk = 15778,
  VPDPBUSDSZmkz = 15779,
  VPDPBUSDSZr = 15780,
  VPDPBUSDSZrk = 15781,
  VPDPBUSDSZrkz = 15782,
  VPDPBUSDSrm = 15783,
  VPDPBUSDSrr = 15784,
  VPDPBUSDYrm = 15785,
  VPDPBUSDYrr = 15786,
  VPDPBUSDZ128m = 15787,
  VPDPBUSDZ128mb = 15788,
  VPDPBUSDZ128mbk = 15789,
  VPDPBUSDZ128mbkz = 15790,
  VPDPBUSDZ128mk = 15791,
  VPDPBUSDZ128mkz = 15792,
  VPDPBUSDZ128r = 15793,
  VPDPBUSDZ128rk = 15794,
  VPDPBUSDZ128rkz = 15795,
  VPDPBUSDZ256m = 15796,
  VPDPBUSDZ256mb = 15797,
  VPDPBUSDZ256mbk = 15798,
  VPDPBUSDZ256mbkz = 15799,
  VPDPBUSDZ256mk = 15800,
  VPDPBUSDZ256mkz = 15801,
  VPDPBUSDZ256r = 15802,
  VPDPBUSDZ256rk = 15803,
  VPDPBUSDZ256rkz = 15804,
  VPDPBUSDZm = 15805,
  VPDPBUSDZmb = 15806,
  VPDPBUSDZmbk = 15807,
  VPDPBUSDZmbkz = 15808,
  VPDPBUSDZmk = 15809,
  VPDPBUSDZmkz = 15810,
  VPDPBUSDZr = 15811,
  VPDPBUSDZrk = 15812,
  VPDPBUSDZrkz = 15813,
  VPDPBUSDrm = 15814,
  VPDPBUSDrr = 15815,
  VPDPBUUDSYrm = 15816,
  VPDPBUUDSYrr = 15817,
  VPDPBUUDSZ128m = 15818,
  VPDPBUUDSZ128mb = 15819,
  VPDPBUUDSZ128mbk = 15820,
  VPDPBUUDSZ128mbkz = 15821,
  VPDPBUUDSZ128mk = 15822,
  VPDPBUUDSZ128mkz = 15823,
  VPDPBUUDSZ128r = 15824,
  VPDPBUUDSZ128rk = 15825,
  VPDPBUUDSZ128rkz = 15826,
  VPDPBUUDSZ256m = 15827,
  VPDPBUUDSZ256mb = 15828,
  VPDPBUUDSZ256mbk = 15829,
  VPDPBUUDSZ256mbkz = 15830,
  VPDPBUUDSZ256mk = 15831,
  VPDPBUUDSZ256mkz = 15832,
  VPDPBUUDSZ256r = 15833,
  VPDPBUUDSZ256rk = 15834,
  VPDPBUUDSZ256rkz = 15835,
  VPDPBUUDSZm = 15836,
  VPDPBUUDSZmb = 15837,
  VPDPBUUDSZmbk = 15838,
  VPDPBUUDSZmbkz = 15839,
  VPDPBUUDSZmk = 15840,
  VPDPBUUDSZmkz = 15841,
  VPDPBUUDSZr = 15842,
  VPDPBUUDSZrk = 15843,
  VPDPBUUDSZrkz = 15844,
  VPDPBUUDSrm = 15845,
  VPDPBUUDSrr = 15846,
  VPDPBUUDYrm = 15847,
  VPDPBUUDYrr = 15848,
  VPDPBUUDZ128m = 15849,
  VPDPBUUDZ128mb = 15850,
  VPDPBUUDZ128mbk = 15851,
  VPDPBUUDZ128mbkz = 15852,
  VPDPBUUDZ128mk = 15853,
  VPDPBUUDZ128mkz = 15854,
  VPDPBUUDZ128r = 15855,
  VPDPBUUDZ128rk = 15856,
  VPDPBUUDZ128rkz = 15857,
  VPDPBUUDZ256m = 15858,
  VPDPBUUDZ256mb = 15859,
  VPDPBUUDZ256mbk = 15860,
  VPDPBUUDZ256mbkz = 15861,
  VPDPBUUDZ256mk = 15862,
  VPDPBUUDZ256mkz = 15863,
  VPDPBUUDZ256r = 15864,
  VPDPBUUDZ256rk = 15865,
  VPDPBUUDZ256rkz = 15866,
  VPDPBUUDZm = 15867,
  VPDPBUUDZmb = 15868,
  VPDPBUUDZmbk = 15869,
  VPDPBUUDZmbkz = 15870,
  VPDPBUUDZmk = 15871,
  VPDPBUUDZmkz = 15872,
  VPDPBUUDZr = 15873,
  VPDPBUUDZrk = 15874,
  VPDPBUUDZrkz = 15875,
  VPDPBUUDrm = 15876,
  VPDPBUUDrr = 15877,
  VPDPWSSDSYrm = 15878,
  VPDPWSSDSYrr = 15879,
  VPDPWSSDSZ128m = 15880,
  VPDPWSSDSZ128mb = 15881,
  VPDPWSSDSZ128mbk = 15882,
  VPDPWSSDSZ128mbkz = 15883,
  VPDPWSSDSZ128mk = 15884,
  VPDPWSSDSZ128mkz = 15885,
  VPDPWSSDSZ128r = 15886,
  VPDPWSSDSZ128rk = 15887,
  VPDPWSSDSZ128rkz = 15888,
  VPDPWSSDSZ256m = 15889,
  VPDPWSSDSZ256mb = 15890,
  VPDPWSSDSZ256mbk = 15891,
  VPDPWSSDSZ256mbkz = 15892,
  VPDPWSSDSZ256mk = 15893,
  VPDPWSSDSZ256mkz = 15894,
  VPDPWSSDSZ256r = 15895,
  VPDPWSSDSZ256rk = 15896,
  VPDPWSSDSZ256rkz = 15897,
  VPDPWSSDSZm = 15898,
  VPDPWSSDSZmb = 15899,
  VPDPWSSDSZmbk = 15900,
  VPDPWSSDSZmbkz = 15901,
  VPDPWSSDSZmk = 15902,
  VPDPWSSDSZmkz = 15903,
  VPDPWSSDSZr = 15904,
  VPDPWSSDSZrk = 15905,
  VPDPWSSDSZrkz = 15906,
  VPDPWSSDSrm = 15907,
  VPDPWSSDSrr = 15908,
  VPDPWSSDYrm = 15909,
  VPDPWSSDYrr = 15910,
  VPDPWSSDZ128m = 15911,
  VPDPWSSDZ128mb = 15912,
  VPDPWSSDZ128mbk = 15913,
  VPDPWSSDZ128mbkz = 15914,
  VPDPWSSDZ128mk = 15915,
  VPDPWSSDZ128mkz = 15916,
  VPDPWSSDZ128r = 15917,
  VPDPWSSDZ128rk = 15918,
  VPDPWSSDZ128rkz = 15919,
  VPDPWSSDZ256m = 15920,
  VPDPWSSDZ256mb = 15921,
  VPDPWSSDZ256mbk = 15922,
  VPDPWSSDZ256mbkz = 15923,
  VPDPWSSDZ256mk = 15924,
  VPDPWSSDZ256mkz = 15925,
  VPDPWSSDZ256r = 15926,
  VPDPWSSDZ256rk = 15927,
  VPDPWSSDZ256rkz = 15928,
  VPDPWSSDZm = 15929,
  VPDPWSSDZmb = 15930,
  VPDPWSSDZmbk = 15931,
  VPDPWSSDZmbkz = 15932,
  VPDPWSSDZmk = 15933,
  VPDPWSSDZmkz = 15934,
  VPDPWSSDZr = 15935,
  VPDPWSSDZrk = 15936,
  VPDPWSSDZrkz = 15937,
  VPDPWSSDrm = 15938,
  VPDPWSSDrr = 15939,
  VPDPWSUDSYrm = 15940,
  VPDPWSUDSYrr = 15941,
  VPDPWSUDSZ128m = 15942,
  VPDPWSUDSZ128mb = 15943,
  VPDPWSUDSZ128mbk = 15944,
  VPDPWSUDSZ128mbkz = 15945,
  VPDPWSUDSZ128mk = 15946,
  VPDPWSUDSZ128mkz = 15947,
  VPDPWSUDSZ128r = 15948,
  VPDPWSUDSZ128rk = 15949,
  VPDPWSUDSZ128rkz = 15950,
  VPDPWSUDSZ256m = 15951,
  VPDPWSUDSZ256mb = 15952,
  VPDPWSUDSZ256mbk = 15953,
  VPDPWSUDSZ256mbkz = 15954,
  VPDPWSUDSZ256mk = 15955,
  VPDPWSUDSZ256mkz = 15956,
  VPDPWSUDSZ256r = 15957,
  VPDPWSUDSZ256rk = 15958,
  VPDPWSUDSZ256rkz = 15959,
  VPDPWSUDSZm = 15960,
  VPDPWSUDSZmb = 15961,
  VPDPWSUDSZmbk = 15962,
  VPDPWSUDSZmbkz = 15963,
  VPDPWSUDSZmk = 15964,
  VPDPWSUDSZmkz = 15965,
  VPDPWSUDSZr = 15966,
  VPDPWSUDSZrk = 15967,
  VPDPWSUDSZrkz = 15968,
  VPDPWSUDSrm = 15969,
  VPDPWSUDSrr = 15970,
  VPDPWSUDYrm = 15971,
  VPDPWSUDYrr = 15972,
  VPDPWSUDZ128m = 15973,
  VPDPWSUDZ128mb = 15974,
  VPDPWSUDZ128mbk = 15975,
  VPDPWSUDZ128mbkz = 15976,
  VPDPWSUDZ128mk = 15977,
  VPDPWSUDZ128mkz = 15978,
  VPDPWSUDZ128r = 15979,
  VPDPWSUDZ128rk = 15980,
  VPDPWSUDZ128rkz = 15981,
  VPDPWSUDZ256m = 15982,
  VPDPWSUDZ256mb = 15983,
  VPDPWSUDZ256mbk = 15984,
  VPDPWSUDZ256mbkz = 15985,
  VPDPWSUDZ256mk = 15986,
  VPDPWSUDZ256mkz = 15987,
  VPDPWSUDZ256r = 15988,
  VPDPWSUDZ256rk = 15989,
  VPDPWSUDZ256rkz = 15990,
  VPDPWSUDZm = 15991,
  VPDPWSUDZmb = 15992,
  VPDPWSUDZmbk = 15993,
  VPDPWSUDZmbkz = 15994,
  VPDPWSUDZmk = 15995,
  VPDPWSUDZmkz = 15996,
  VPDPWSUDZr = 15997,
  VPDPWSUDZrk = 15998,
  VPDPWSUDZrkz = 15999,
  VPDPWSUDrm = 16000,
  VPDPWSUDrr = 16001,
  VPDPWUSDSYrm = 16002,
  VPDPWUSDSYrr = 16003,
  VPDPWUSDSZ128m = 16004,
  VPDPWUSDSZ128mb = 16005,
  VPDPWUSDSZ128mbk = 16006,
  VPDPWUSDSZ128mbkz = 16007,
  VPDPWUSDSZ128mk = 16008,
  VPDPWUSDSZ128mkz = 16009,
  VPDPWUSDSZ128r = 16010,
  VPDPWUSDSZ128rk = 16011,
  VPDPWUSDSZ128rkz = 16012,
  VPDPWUSDSZ256m = 16013,
  VPDPWUSDSZ256mb = 16014,
  VPDPWUSDSZ256mbk = 16015,
  VPDPWUSDSZ256mbkz = 16016,
  VPDPWUSDSZ256mk = 16017,
  VPDPWUSDSZ256mkz = 16018,
  VPDPWUSDSZ256r = 16019,
  VPDPWUSDSZ256rk = 16020,
  VPDPWUSDSZ256rkz = 16021,
  VPDPWUSDSZm = 16022,
  VPDPWUSDSZmb = 16023,
  VPDPWUSDSZmbk = 16024,
  VPDPWUSDSZmbkz = 16025,
  VPDPWUSDSZmk = 16026,
  VPDPWUSDSZmkz = 16027,
  VPDPWUSDSZr = 16028,
  VPDPWUSDSZrk = 16029,
  VPDPWUSDSZrkz = 16030,
  VPDPWUSDSrm = 16031,
  VPDPWUSDSrr = 16032,
  VPDPWUSDYrm = 16033,
  VPDPWUSDYrr = 16034,
  VPDPWUSDZ128m = 16035,
  VPDPWUSDZ128mb = 16036,
  VPDPWUSDZ128mbk = 16037,
  VPDPWUSDZ128mbkz = 16038,
  VPDPWUSDZ128mk = 16039,
  VPDPWUSDZ128mkz = 16040,
  VPDPWUSDZ128r = 16041,
  VPDPWUSDZ128rk = 16042,
  VPDPWUSDZ128rkz = 16043,
  VPDPWUSDZ256m = 16044,
  VPDPWUSDZ256mb = 16045,
  VPDPWUSDZ256mbk = 16046,
  VPDPWUSDZ256mbkz = 16047,
  VPDPWUSDZ256mk = 16048,
  VPDPWUSDZ256mkz = 16049,
  VPDPWUSDZ256r = 16050,
  VPDPWUSDZ256rk = 16051,
  VPDPWUSDZ256rkz = 16052,
  VPDPWUSDZm = 16053,
  VPDPWUSDZmb = 16054,
  VPDPWUSDZmbk = 16055,
  VPDPWUSDZmbkz = 16056,
  VPDPWUSDZmk = 16057,
  VPDPWUSDZmkz = 16058,
  VPDPWUSDZr = 16059,
  VPDPWUSDZrk = 16060,
  VPDPWUSDZrkz = 16061,
  VPDPWUSDrm = 16062,
  VPDPWUSDrr = 16063,
  VPDPWUUDSYrm = 16064,
  VPDPWUUDSYrr = 16065,
  VPDPWUUDSZ128m = 16066,
  VPDPWUUDSZ128mb = 16067,
  VPDPWUUDSZ128mbk = 16068,
  VPDPWUUDSZ128mbkz = 16069,
  VPDPWUUDSZ128mk = 16070,
  VPDPWUUDSZ128mkz = 16071,
  VPDPWUUDSZ128r = 16072,
  VPDPWUUDSZ128rk = 16073,
  VPDPWUUDSZ128rkz = 16074,
  VPDPWUUDSZ256m = 16075,
  VPDPWUUDSZ256mb = 16076,
  VPDPWUUDSZ256mbk = 16077,
  VPDPWUUDSZ256mbkz = 16078,
  VPDPWUUDSZ256mk = 16079,
  VPDPWUUDSZ256mkz = 16080,
  VPDPWUUDSZ256r = 16081,
  VPDPWUUDSZ256rk = 16082,
  VPDPWUUDSZ256rkz = 16083,
  VPDPWUUDSZm = 16084,
  VPDPWUUDSZmb = 16085,
  VPDPWUUDSZmbk = 16086,
  VPDPWUUDSZmbkz = 16087,
  VPDPWUUDSZmk = 16088,
  VPDPWUUDSZmkz = 16089,
  VPDPWUUDSZr = 16090,
  VPDPWUUDSZrk = 16091,
  VPDPWUUDSZrkz = 16092,
  VPDPWUUDSrm = 16093,
  VPDPWUUDSrr = 16094,
  VPDPWUUDYrm = 16095,
  VPDPWUUDYrr = 16096,
  VPDPWUUDZ128m = 16097,
  VPDPWUUDZ128mb = 16098,
  VPDPWUUDZ128mbk = 16099,
  VPDPWUUDZ128mbkz = 16100,
  VPDPWUUDZ128mk = 16101,
  VPDPWUUDZ128mkz = 16102,
  VPDPWUUDZ128r = 16103,
  VPDPWUUDZ128rk = 16104,
  VPDPWUUDZ128rkz = 16105,
  VPDPWUUDZ256m = 16106,
  VPDPWUUDZ256mb = 16107,
  VPDPWUUDZ256mbk = 16108,
  VPDPWUUDZ256mbkz = 16109,
  VPDPWUUDZ256mk = 16110,
  VPDPWUUDZ256mkz = 16111,
  VPDPWUUDZ256r = 16112,
  VPDPWUUDZ256rk = 16113,
  VPDPWUUDZ256rkz = 16114,
  VPDPWUUDZm = 16115,
  VPDPWUUDZmb = 16116,
  VPDPWUUDZmbk = 16117,
  VPDPWUUDZmbkz = 16118,
  VPDPWUUDZmk = 16119,
  VPDPWUUDZmkz = 16120,
  VPDPWUUDZr = 16121,
  VPDPWUUDZrk = 16122,
  VPDPWUUDZrkz = 16123,
  VPDPWUUDrm = 16124,
  VPDPWUUDrr = 16125,
  VPERM2F128rmi = 16126,
  VPERM2F128rri = 16127,
  VPERM2I128rmi = 16128,
  VPERM2I128rri = 16129,
  VPERMBZ128rm = 16130,
  VPERMBZ128rmk = 16131,
  VPERMBZ128rmkz = 16132,
  VPERMBZ128rr = 16133,
  VPERMBZ128rrk = 16134,
  VPERMBZ128rrkz = 16135,
  VPERMBZ256rm = 16136,
  VPERMBZ256rmk = 16137,
  VPERMBZ256rmkz = 16138,
  VPERMBZ256rr = 16139,
  VPERMBZ256rrk = 16140,
  VPERMBZ256rrkz = 16141,
  VPERMBZrm = 16142,
  VPERMBZrmk = 16143,
  VPERMBZrmkz = 16144,
  VPERMBZrr = 16145,
  VPERMBZrrk = 16146,
  VPERMBZrrkz = 16147,
  VPERMDYrm = 16148,
  VPERMDYrr = 16149,
  VPERMDZ256rm = 16150,
  VPERMDZ256rmb = 16151,
  VPERMDZ256rmbk = 16152,
  VPERMDZ256rmbkz = 16153,
  VPERMDZ256rmk = 16154,
  VPERMDZ256rmkz = 16155,
  VPERMDZ256rr = 16156,
  VPERMDZ256rrk = 16157,
  VPERMDZ256rrkz = 16158,
  VPERMDZrm = 16159,
  VPERMDZrmb = 16160,
  VPERMDZrmbk = 16161,
  VPERMDZrmbkz = 16162,
  VPERMDZrmk = 16163,
  VPERMDZrmkz = 16164,
  VPERMDZrr = 16165,
  VPERMDZrrk = 16166,
  VPERMDZrrkz = 16167,
  VPERMI2BZ128rm = 16168,
  VPERMI2BZ128rmk = 16169,
  VPERMI2BZ128rmkz = 16170,
  VPERMI2BZ128rr = 16171,
  VPERMI2BZ128rrk = 16172,
  VPERMI2BZ128rrkz = 16173,
  VPERMI2BZ256rm = 16174,
  VPERMI2BZ256rmk = 16175,
  VPERMI2BZ256rmkz = 16176,
  VPERMI2BZ256rr = 16177,
  VPERMI2BZ256rrk = 16178,
  VPERMI2BZ256rrkz = 16179,
  VPERMI2BZrm = 16180,
  VPERMI2BZrmk = 16181,
  VPERMI2BZrmkz = 16182,
  VPERMI2BZrr = 16183,
  VPERMI2BZrrk = 16184,
  VPERMI2BZrrkz = 16185,
  VPERMI2DZ128rm = 16186,
  VPERMI2DZ128rmb = 16187,
  VPERMI2DZ128rmbk = 16188,
  VPERMI2DZ128rmbkz = 16189,
  VPERMI2DZ128rmk = 16190,
  VPERMI2DZ128rmkz = 16191,
  VPERMI2DZ128rr = 16192,
  VPERMI2DZ128rrk = 16193,
  VPERMI2DZ128rrkz = 16194,
  VPERMI2DZ256rm = 16195,
  VPERMI2DZ256rmb = 16196,
  VPERMI2DZ256rmbk = 16197,
  VPERMI2DZ256rmbkz = 16198,
  VPERMI2DZ256rmk = 16199,
  VPERMI2DZ256rmkz = 16200,
  VPERMI2DZ256rr = 16201,
  VPERMI2DZ256rrk = 16202,
  VPERMI2DZ256rrkz = 16203,
  VPERMI2DZrm = 16204,
  VPERMI2DZrmb = 16205,
  VPERMI2DZrmbk = 16206,
  VPERMI2DZrmbkz = 16207,
  VPERMI2DZrmk = 16208,
  VPERMI2DZrmkz = 16209,
  VPERMI2DZrr = 16210,
  VPERMI2DZrrk = 16211,
  VPERMI2DZrrkz = 16212,
  VPERMI2PDZ128rm = 16213,
  VPERMI2PDZ128rmb = 16214,
  VPERMI2PDZ128rmbk = 16215,
  VPERMI2PDZ128rmbkz = 16216,
  VPERMI2PDZ128rmk = 16217,
  VPERMI2PDZ128rmkz = 16218,
  VPERMI2PDZ128rr = 16219,
  VPERMI2PDZ128rrk = 16220,
  VPERMI2PDZ128rrkz = 16221,
  VPERMI2PDZ256rm = 16222,
  VPERMI2PDZ256rmb = 16223,
  VPERMI2PDZ256rmbk = 16224,
  VPERMI2PDZ256rmbkz = 16225,
  VPERMI2PDZ256rmk = 16226,
  VPERMI2PDZ256rmkz = 16227,
  VPERMI2PDZ256rr = 16228,
  VPERMI2PDZ256rrk = 16229,
  VPERMI2PDZ256rrkz = 16230,
  VPERMI2PDZrm = 16231,
  VPERMI2PDZrmb = 16232,
  VPERMI2PDZrmbk = 16233,
  VPERMI2PDZrmbkz = 16234,
  VPERMI2PDZrmk = 16235,
  VPERMI2PDZrmkz = 16236,
  VPERMI2PDZrr = 16237,
  VPERMI2PDZrrk = 16238,
  VPERMI2PDZrrkz = 16239,
  VPERMI2PSZ128rm = 16240,
  VPERMI2PSZ128rmb = 16241,
  VPERMI2PSZ128rmbk = 16242,
  VPERMI2PSZ128rmbkz = 16243,
  VPERMI2PSZ128rmk = 16244,
  VPERMI2PSZ128rmkz = 16245,
  VPERMI2PSZ128rr = 16246,
  VPERMI2PSZ128rrk = 16247,
  VPERMI2PSZ128rrkz = 16248,
  VPERMI2PSZ256rm = 16249,
  VPERMI2PSZ256rmb = 16250,
  VPERMI2PSZ256rmbk = 16251,
  VPERMI2PSZ256rmbkz = 16252,
  VPERMI2PSZ256rmk = 16253,
  VPERMI2PSZ256rmkz = 16254,
  VPERMI2PSZ256rr = 16255,
  VPERMI2PSZ256rrk = 16256,
  VPERMI2PSZ256rrkz = 16257,
  VPERMI2PSZrm = 16258,
  VPERMI2PSZrmb = 16259,
  VPERMI2PSZrmbk = 16260,
  VPERMI2PSZrmbkz = 16261,
  VPERMI2PSZrmk = 16262,
  VPERMI2PSZrmkz = 16263,
  VPERMI2PSZrr = 16264,
  VPERMI2PSZrrk = 16265,
  VPERMI2PSZrrkz = 16266,
  VPERMI2QZ128rm = 16267,
  VPERMI2QZ128rmb = 16268,
  VPERMI2QZ128rmbk = 16269,
  VPERMI2QZ128rmbkz = 16270,
  VPERMI2QZ128rmk = 16271,
  VPERMI2QZ128rmkz = 16272,
  VPERMI2QZ128rr = 16273,
  VPERMI2QZ128rrk = 16274,
  VPERMI2QZ128rrkz = 16275,
  VPERMI2QZ256rm = 16276,
  VPERMI2QZ256rmb = 16277,
  VPERMI2QZ256rmbk = 16278,
  VPERMI2QZ256rmbkz = 16279,
  VPERMI2QZ256rmk = 16280,
  VPERMI2QZ256rmkz = 16281,
  VPERMI2QZ256rr = 16282,
  VPERMI2QZ256rrk = 16283,
  VPERMI2QZ256rrkz = 16284,
  VPERMI2QZrm = 16285,
  VPERMI2QZrmb = 16286,
  VPERMI2QZrmbk = 16287,
  VPERMI2QZrmbkz = 16288,
  VPERMI2QZrmk = 16289,
  VPERMI2QZrmkz = 16290,
  VPERMI2QZrr = 16291,
  VPERMI2QZrrk = 16292,
  VPERMI2QZrrkz = 16293,
  VPERMI2WZ128rm = 16294,
  VPERMI2WZ128rmk = 16295,
  VPERMI2WZ128rmkz = 16296,
  VPERMI2WZ128rr = 16297,
  VPERMI2WZ128rrk = 16298,
  VPERMI2WZ128rrkz = 16299,
  VPERMI2WZ256rm = 16300,
  VPERMI2WZ256rmk = 16301,
  VPERMI2WZ256rmkz = 16302,
  VPERMI2WZ256rr = 16303,
  VPERMI2WZ256rrk = 16304,
  VPERMI2WZ256rrkz = 16305,
  VPERMI2WZrm = 16306,
  VPERMI2WZrmk = 16307,
  VPERMI2WZrmkz = 16308,
  VPERMI2WZrr = 16309,
  VPERMI2WZrrk = 16310,
  VPERMI2WZrrkz = 16311,
  VPERMIL2PDYmr = 16312,
  VPERMIL2PDYrm = 16313,
  VPERMIL2PDYrr = 16314,
  VPERMIL2PDYrr_REV = 16315,
  VPERMIL2PDmr = 16316,
  VPERMIL2PDrm = 16317,
  VPERMIL2PDrr = 16318,
  VPERMIL2PDrr_REV = 16319,
  VPERMIL2PSYmr = 16320,
  VPERMIL2PSYrm = 16321,
  VPERMIL2PSYrr = 16322,
  VPERMIL2PSYrr_REV = 16323,
  VPERMIL2PSmr = 16324,
  VPERMIL2PSrm = 16325,
  VPERMIL2PSrr = 16326,
  VPERMIL2PSrr_REV = 16327,
  VPERMILPDYmi = 16328,
  VPERMILPDYri = 16329,
  VPERMILPDYrm = 16330,
  VPERMILPDYrr = 16331,
  VPERMILPDZ128mbi = 16332,
  VPERMILPDZ128mbik = 16333,
  VPERMILPDZ128mbikz = 16334,
  VPERMILPDZ128mi = 16335,
  VPERMILPDZ128mik = 16336,
  VPERMILPDZ128mikz = 16337,
  VPERMILPDZ128ri = 16338,
  VPERMILPDZ128rik = 16339,
  VPERMILPDZ128rikz = 16340,
  VPERMILPDZ128rm = 16341,
  VPERMILPDZ128rmb = 16342,
  VPERMILPDZ128rmbk = 16343,
  VPERMILPDZ128rmbkz = 16344,
  VPERMILPDZ128rmk = 16345,
  VPERMILPDZ128rmkz = 16346,
  VPERMILPDZ128rr = 16347,
  VPERMILPDZ128rrk = 16348,
  VPERMILPDZ128rrkz = 16349,
  VPERMILPDZ256mbi = 16350,
  VPERMILPDZ256mbik = 16351,
  VPERMILPDZ256mbikz = 16352,
  VPERMILPDZ256mi = 16353,
  VPERMILPDZ256mik = 16354,
  VPERMILPDZ256mikz = 16355,
  VPERMILPDZ256ri = 16356,
  VPERMILPDZ256rik = 16357,
  VPERMILPDZ256rikz = 16358,
  VPERMILPDZ256rm = 16359,
  VPERMILPDZ256rmb = 16360,
  VPERMILPDZ256rmbk = 16361,
  VPERMILPDZ256rmbkz = 16362,
  VPERMILPDZ256rmk = 16363,
  VPERMILPDZ256rmkz = 16364,
  VPERMILPDZ256rr = 16365,
  VPERMILPDZ256rrk = 16366,
  VPERMILPDZ256rrkz = 16367,
  VPERMILPDZmbi = 16368,
  VPERMILPDZmbik = 16369,
  VPERMILPDZmbikz = 16370,
  VPERMILPDZmi = 16371,
  VPERMILPDZmik = 16372,
  VPERMILPDZmikz = 16373,
  VPERMILPDZri = 16374,
  VPERMILPDZrik = 16375,
  VPERMILPDZrikz = 16376,
  VPERMILPDZrm = 16377,
  VPERMILPDZrmb = 16378,
  VPERMILPDZrmbk = 16379,
  VPERMILPDZrmbkz = 16380,
  VPERMILPDZrmk = 16381,
  VPERMILPDZrmkz = 16382,
  VPERMILPDZrr = 16383,
  VPERMILPDZrrk = 16384,
  VPERMILPDZrrkz = 16385,
  VPERMILPDmi = 16386,
  VPERMILPDri = 16387,
  VPERMILPDrm = 16388,
  VPERMILPDrr = 16389,
  VPERMILPSYmi = 16390,
  VPERMILPSYri = 16391,
  VPERMILPSYrm = 16392,
  VPERMILPSYrr = 16393,
  VPERMILPSZ128mbi = 16394,
  VPERMILPSZ128mbik = 16395,
  VPERMILPSZ128mbikz = 16396,
  VPERMILPSZ128mi = 16397,
  VPERMILPSZ128mik = 16398,
  VPERMILPSZ128mikz = 16399,
  VPERMILPSZ128ri = 16400,
  VPERMILPSZ128rik = 16401,
  VPERMILPSZ128rikz = 16402,
  VPERMILPSZ128rm = 16403,
  VPERMILPSZ128rmb = 16404,
  VPERMILPSZ128rmbk = 16405,
  VPERMILPSZ128rmbkz = 16406,
  VPERMILPSZ128rmk = 16407,
  VPERMILPSZ128rmkz = 16408,
  VPERMILPSZ128rr = 16409,
  VPERMILPSZ128rrk = 16410,
  VPERMILPSZ128rrkz = 16411,
  VPERMILPSZ256mbi = 16412,
  VPERMILPSZ256mbik = 16413,
  VPERMILPSZ256mbikz = 16414,
  VPERMILPSZ256mi = 16415,
  VPERMILPSZ256mik = 16416,
  VPERMILPSZ256mikz = 16417,
  VPERMILPSZ256ri = 16418,
  VPERMILPSZ256rik = 16419,
  VPERMILPSZ256rikz = 16420,
  VPERMILPSZ256rm = 16421,
  VPERMILPSZ256rmb = 16422,
  VPERMILPSZ256rmbk = 16423,
  VPERMILPSZ256rmbkz = 16424,
  VPERMILPSZ256rmk = 16425,
  VPERMILPSZ256rmkz = 16426,
  VPERMILPSZ256rr = 16427,
  VPERMILPSZ256rrk = 16428,
  VPERMILPSZ256rrkz = 16429,
  VPERMILPSZmbi = 16430,
  VPERMILPSZmbik = 16431,
  VPERMILPSZmbikz = 16432,
  VPERMILPSZmi = 16433,
  VPERMILPSZmik = 16434,
  VPERMILPSZmikz = 16435,
  VPERMILPSZri = 16436,
  VPERMILPSZrik = 16437,
  VPERMILPSZrikz = 16438,
  VPERMILPSZrm = 16439,
  VPERMILPSZrmb = 16440,
  VPERMILPSZrmbk = 16441,
  VPERMILPSZrmbkz = 16442,
  VPERMILPSZrmk = 16443,
  VPERMILPSZrmkz = 16444,
  VPERMILPSZrr = 16445,
  VPERMILPSZrrk = 16446,
  VPERMILPSZrrkz = 16447,
  VPERMILPSmi = 16448,
  VPERMILPSri = 16449,
  VPERMILPSrm = 16450,
  VPERMILPSrr = 16451,
  VPERMPDYmi = 16452,
  VPERMPDYri = 16453,
  VPERMPDZ256mbi = 16454,
  VPERMPDZ256mbik = 16455,
  VPERMPDZ256mbikz = 16456,
  VPERMPDZ256mi = 16457,
  VPERMPDZ256mik = 16458,
  VPERMPDZ256mikz = 16459,
  VPERMPDZ256ri = 16460,
  VPERMPDZ256rik = 16461,
  VPERMPDZ256rikz = 16462,
  VPERMPDZ256rm = 16463,
  VPERMPDZ256rmb = 16464,
  VPERMPDZ256rmbk = 16465,
  VPERMPDZ256rmbkz = 16466,
  VPERMPDZ256rmk = 16467,
  VPERMPDZ256rmkz = 16468,
  VPERMPDZ256rr = 16469,
  VPERMPDZ256rrk = 16470,
  VPERMPDZ256rrkz = 16471,
  VPERMPDZmbi = 16472,
  VPERMPDZmbik = 16473,
  VPERMPDZmbikz = 16474,
  VPERMPDZmi = 16475,
  VPERMPDZmik = 16476,
  VPERMPDZmikz = 16477,
  VPERMPDZri = 16478,
  VPERMPDZrik = 16479,
  VPERMPDZrikz = 16480,
  VPERMPDZrm = 16481,
  VPERMPDZrmb = 16482,
  VPERMPDZrmbk = 16483,
  VPERMPDZrmbkz = 16484,
  VPERMPDZrmk = 16485,
  VPERMPDZrmkz = 16486,
  VPERMPDZrr = 16487,
  VPERMPDZrrk = 16488,
  VPERMPDZrrkz = 16489,
  VPERMPSYrm = 16490,
  VPERMPSYrr = 16491,
  VPERMPSZ256rm = 16492,
  VPERMPSZ256rmb = 16493,
  VPERMPSZ256rmbk = 16494,
  VPERMPSZ256rmbkz = 16495,
  VPERMPSZ256rmk = 16496,
  VPERMPSZ256rmkz = 16497,
  VPERMPSZ256rr = 16498,
  VPERMPSZ256rrk = 16499,
  VPERMPSZ256rrkz = 16500,
  VPERMPSZrm = 16501,
  VPERMPSZrmb = 16502,
  VPERMPSZrmbk = 16503,
  VPERMPSZrmbkz = 16504,
  VPERMPSZrmk = 16505,
  VPERMPSZrmkz = 16506,
  VPERMPSZrr = 16507,
  VPERMPSZrrk = 16508,
  VPERMPSZrrkz = 16509,
  VPERMQYmi = 16510,
  VPERMQYri = 16511,
  VPERMQZ256mbi = 16512,
  VPERMQZ256mbik = 16513,
  VPERMQZ256mbikz = 16514,
  VPERMQZ256mi = 16515,
  VPERMQZ256mik = 16516,
  VPERMQZ256mikz = 16517,
  VPERMQZ256ri = 16518,
  VPERMQZ256rik = 16519,
  VPERMQZ256rikz = 16520,
  VPERMQZ256rm = 16521,
  VPERMQZ256rmb = 16522,
  VPERMQZ256rmbk = 16523,
  VPERMQZ256rmbkz = 16524,
  VPERMQZ256rmk = 16525,
  VPERMQZ256rmkz = 16526,
  VPERMQZ256rr = 16527,
  VPERMQZ256rrk = 16528,
  VPERMQZ256rrkz = 16529,
  VPERMQZmbi = 16530,
  VPERMQZmbik = 16531,
  VPERMQZmbikz = 16532,
  VPERMQZmi = 16533,
  VPERMQZmik = 16534,
  VPERMQZmikz = 16535,
  VPERMQZri = 16536,
  VPERMQZrik = 16537,
  VPERMQZrikz = 16538,
  VPERMQZrm = 16539,
  VPERMQZrmb = 16540,
  VPERMQZrmbk = 16541,
  VPERMQZrmbkz = 16542,
  VPERMQZrmk = 16543,
  VPERMQZrmkz = 16544,
  VPERMQZrr = 16545,
  VPERMQZrrk = 16546,
  VPERMQZrrkz = 16547,
  VPERMT2BZ128rm = 16548,
  VPERMT2BZ128rmk = 16549,
  VPERMT2BZ128rmkz = 16550,
  VPERMT2BZ128rr = 16551,
  VPERMT2BZ128rrk = 16552,
  VPERMT2BZ128rrkz = 16553,
  VPERMT2BZ256rm = 16554,
  VPERMT2BZ256rmk = 16555,
  VPERMT2BZ256rmkz = 16556,
  VPERMT2BZ256rr = 16557,
  VPERMT2BZ256rrk = 16558,
  VPERMT2BZ256rrkz = 16559,
  VPERMT2BZrm = 16560,
  VPERMT2BZrmk = 16561,
  VPERMT2BZrmkz = 16562,
  VPERMT2BZrr = 16563,
  VPERMT2BZrrk = 16564,
  VPERMT2BZrrkz = 16565,
  VPERMT2DZ128rm = 16566,
  VPERMT2DZ128rmb = 16567,
  VPERMT2DZ128rmbk = 16568,
  VPERMT2DZ128rmbkz = 16569,
  VPERMT2DZ128rmk = 16570,
  VPERMT2DZ128rmkz = 16571,
  VPERMT2DZ128rr = 16572,
  VPERMT2DZ128rrk = 16573,
  VPERMT2DZ128rrkz = 16574,
  VPERMT2DZ256rm = 16575,
  VPERMT2DZ256rmb = 16576,
  VPERMT2DZ256rmbk = 16577,
  VPERMT2DZ256rmbkz = 16578,
  VPERMT2DZ256rmk = 16579,
  VPERMT2DZ256rmkz = 16580,
  VPERMT2DZ256rr = 16581,
  VPERMT2DZ256rrk = 16582,
  VPERMT2DZ256rrkz = 16583,
  VPERMT2DZrm = 16584,
  VPERMT2DZrmb = 16585,
  VPERMT2DZrmbk = 16586,
  VPERMT2DZrmbkz = 16587,
  VPERMT2DZrmk = 16588,
  VPERMT2DZrmkz = 16589,
  VPERMT2DZrr = 16590,
  VPERMT2DZrrk = 16591,
  VPERMT2DZrrkz = 16592,
  VPERMT2PDZ128rm = 16593,
  VPERMT2PDZ128rmb = 16594,
  VPERMT2PDZ128rmbk = 16595,
  VPERMT2PDZ128rmbkz = 16596,
  VPERMT2PDZ128rmk = 16597,
  VPERMT2PDZ128rmkz = 16598,
  VPERMT2PDZ128rr = 16599,
  VPERMT2PDZ128rrk = 16600,
  VPERMT2PDZ128rrkz = 16601,
  VPERMT2PDZ256rm = 16602,
  VPERMT2PDZ256rmb = 16603,
  VPERMT2PDZ256rmbk = 16604,
  VPERMT2PDZ256rmbkz = 16605,
  VPERMT2PDZ256rmk = 16606,
  VPERMT2PDZ256rmkz = 16607,
  VPERMT2PDZ256rr = 16608,
  VPERMT2PDZ256rrk = 16609,
  VPERMT2PDZ256rrkz = 16610,
  VPERMT2PDZrm = 16611,
  VPERMT2PDZrmb = 16612,
  VPERMT2PDZrmbk = 16613,
  VPERMT2PDZrmbkz = 16614,
  VPERMT2PDZrmk = 16615,
  VPERMT2PDZrmkz = 16616,
  VPERMT2PDZrr = 16617,
  VPERMT2PDZrrk = 16618,
  VPERMT2PDZrrkz = 16619,
  VPERMT2PSZ128rm = 16620,
  VPERMT2PSZ128rmb = 16621,
  VPERMT2PSZ128rmbk = 16622,
  VPERMT2PSZ128rmbkz = 16623,
  VPERMT2PSZ128rmk = 16624,
  VPERMT2PSZ128rmkz = 16625,
  VPERMT2PSZ128rr = 16626,
  VPERMT2PSZ128rrk = 16627,
  VPERMT2PSZ128rrkz = 16628,
  VPERMT2PSZ256rm = 16629,
  VPERMT2PSZ256rmb = 16630,
  VPERMT2PSZ256rmbk = 16631,
  VPERMT2PSZ256rmbkz = 16632,
  VPERMT2PSZ256rmk = 16633,
  VPERMT2PSZ256rmkz = 16634,
  VPERMT2PSZ256rr = 16635,
  VPERMT2PSZ256rrk = 16636,
  VPERMT2PSZ256rrkz = 16637,
  VPERMT2PSZrm = 16638,
  VPERMT2PSZrmb = 16639,
  VPERMT2PSZrmbk = 16640,
  VPERMT2PSZrmbkz = 16641,
  VPERMT2PSZrmk = 16642,
  VPERMT2PSZrmkz = 16643,
  VPERMT2PSZrr = 16644,
  VPERMT2PSZrrk = 16645,
  VPERMT2PSZrrkz = 16646,
  VPERMT2QZ128rm = 16647,
  VPERMT2QZ128rmb = 16648,
  VPERMT2QZ128rmbk = 16649,
  VPERMT2QZ128rmbkz = 16650,
  VPERMT2QZ128rmk = 16651,
  VPERMT2QZ128rmkz = 16652,
  VPERMT2QZ128rr = 16653,
  VPERMT2QZ128rrk = 16654,
  VPERMT2QZ128rrkz = 16655,
  VPERMT2QZ256rm = 16656,
  VPERMT2QZ256rmb = 16657,
  VPERMT2QZ256rmbk = 16658,
  VPERMT2QZ256rmbkz = 16659,
  VPERMT2QZ256rmk = 16660,
  VPERMT2QZ256rmkz = 16661,
  VPERMT2QZ256rr = 16662,
  VPERMT2QZ256rrk = 16663,
  VPERMT2QZ256rrkz = 16664,
  VPERMT2QZrm = 16665,
  VPERMT2QZrmb = 16666,
  VPERMT2QZrmbk = 16667,
  VPERMT2QZrmbkz = 16668,
  VPERMT2QZrmk = 16669,
  VPERMT2QZrmkz = 16670,
  VPERMT2QZrr = 16671,
  VPERMT2QZrrk = 16672,
  VPERMT2QZrrkz = 16673,
  VPERMT2WZ128rm = 16674,
  VPERMT2WZ128rmk = 16675,
  VPERMT2WZ128rmkz = 16676,
  VPERMT2WZ128rr = 16677,
  VPERMT2WZ128rrk = 16678,
  VPERMT2WZ128rrkz = 16679,
  VPERMT2WZ256rm = 16680,
  VPERMT2WZ256rmk = 16681,
  VPERMT2WZ256rmkz = 16682,
  VPERMT2WZ256rr = 16683,
  VPERMT2WZ256rrk = 16684,
  VPERMT2WZ256rrkz = 16685,
  VPERMT2WZrm = 16686,
  VPERMT2WZrmk = 16687,
  VPERMT2WZrmkz = 16688,
  VPERMT2WZrr = 16689,
  VPERMT2WZrrk = 16690,
  VPERMT2WZrrkz = 16691,
  VPERMWZ128rm = 16692,
  VPERMWZ128rmk = 16693,
  VPERMWZ128rmkz = 16694,
  VPERMWZ128rr = 16695,
  VPERMWZ128rrk = 16696,
  VPERMWZ128rrkz = 16697,
  VPERMWZ256rm = 16698,
  VPERMWZ256rmk = 16699,
  VPERMWZ256rmkz = 16700,
  VPERMWZ256rr = 16701,
  VPERMWZ256rrk = 16702,
  VPERMWZ256rrkz = 16703,
  VPERMWZrm = 16704,
  VPERMWZrmk = 16705,
  VPERMWZrmkz = 16706,
  VPERMWZrr = 16707,
  VPERMWZrrk = 16708,
  VPERMWZrrkz = 16709,
  VPEXPANDBZ128rm = 16710,
  VPEXPANDBZ128rmk = 16711,
  VPEXPANDBZ128rmkz = 16712,
  VPEXPANDBZ128rr = 16713,
  VPEXPANDBZ128rrk = 16714,
  VPEXPANDBZ128rrkz = 16715,
  VPEXPANDBZ256rm = 16716,
  VPEXPANDBZ256rmk = 16717,
  VPEXPANDBZ256rmkz = 16718,
  VPEXPANDBZ256rr = 16719,
  VPEXPANDBZ256rrk = 16720,
  VPEXPANDBZ256rrkz = 16721,
  VPEXPANDBZrm = 16722,
  VPEXPANDBZrmk = 16723,
  VPEXPANDBZrmkz = 16724,
  VPEXPANDBZrr = 16725,
  VPEXPANDBZrrk = 16726,
  VPEXPANDBZrrkz = 16727,
  VPEXPANDDZ128rm = 16728,
  VPEXPANDDZ128rmk = 16729,
  VPEXPANDDZ128rmkz = 16730,
  VPEXPANDDZ128rr = 16731,
  VPEXPANDDZ128rrk = 16732,
  VPEXPANDDZ128rrkz = 16733,
  VPEXPANDDZ256rm = 16734,
  VPEXPANDDZ256rmk = 16735,
  VPEXPANDDZ256rmkz = 16736,
  VPEXPANDDZ256rr = 16737,
  VPEXPANDDZ256rrk = 16738,
  VPEXPANDDZ256rrkz = 16739,
  VPEXPANDDZrm = 16740,
  VPEXPANDDZrmk = 16741,
  VPEXPANDDZrmkz = 16742,
  VPEXPANDDZrr = 16743,
  VPEXPANDDZrrk = 16744,
  VPEXPANDDZrrkz = 16745,
  VPEXPANDQZ128rm = 16746,
  VPEXPANDQZ128rmk = 16747,
  VPEXPANDQZ128rmkz = 16748,
  VPEXPANDQZ128rr = 16749,
  VPEXPANDQZ128rrk = 16750,
  VPEXPANDQZ128rrkz = 16751,
  VPEXPANDQZ256rm = 16752,
  VPEXPANDQZ256rmk = 16753,
  VPEXPANDQZ256rmkz = 16754,
  VPEXPANDQZ256rr = 16755,
  VPEXPANDQZ256rrk = 16756,
  VPEXPANDQZ256rrkz = 16757,
  VPEXPANDQZrm = 16758,
  VPEXPANDQZrmk = 16759,
  VPEXPANDQZrmkz = 16760,
  VPEXPANDQZrr = 16761,
  VPEXPANDQZrrk = 16762,
  VPEXPANDQZrrkz = 16763,
  VPEXPANDWZ128rm = 16764,
  VPEXPANDWZ128rmk = 16765,
  VPEXPANDWZ128rmkz = 16766,
  VPEXPANDWZ128rr = 16767,
  VPEXPANDWZ128rrk = 16768,
  VPEXPANDWZ128rrkz = 16769,
  VPEXPANDWZ256rm = 16770,
  VPEXPANDWZ256rmk = 16771,
  VPEXPANDWZ256rmkz = 16772,
  VPEXPANDWZ256rr = 16773,
  VPEXPANDWZ256rrk = 16774,
  VPEXPANDWZ256rrkz = 16775,
  VPEXPANDWZrm = 16776,
  VPEXPANDWZrmk = 16777,
  VPEXPANDWZrmkz = 16778,
  VPEXPANDWZrr = 16779,
  VPEXPANDWZrrk = 16780,
  VPEXPANDWZrrkz = 16781,
  VPEXTRBZmri = 16782,
  VPEXTRBZrri = 16783,
  VPEXTRBmri = 16784,
  VPEXTRBrri = 16785,
  VPEXTRDZmri = 16786,
  VPEXTRDZrri = 16787,
  VPEXTRDmri = 16788,
  VPEXTRDrri = 16789,
  VPEXTRQZmri = 16790,
  VPEXTRQZrri = 16791,
  VPEXTRQmri = 16792,
  VPEXTRQrri = 16793,
  VPEXTRWZmri = 16794,
  VPEXTRWZrri = 16795,
  VPEXTRWZrri_REV = 16796,
  VPEXTRWmri = 16797,
  VPEXTRWrri = 16798,
  VPEXTRWrri_REV = 16799,
  VPGATHERDDYrm = 16800,
  VPGATHERDDZ128rm = 16801,
  VPGATHERDDZ256rm = 16802,
  VPGATHERDDZrm = 16803,
  VPGATHERDDrm = 16804,
  VPGATHERDQYrm = 16805,
  VPGATHERDQZ128rm = 16806,
  VPGATHERDQZ256rm = 16807,
  VPGATHERDQZrm = 16808,
  VPGATHERDQrm = 16809,
  VPGATHERQDYrm = 16810,
  VPGATHERQDZ128rm = 16811,
  VPGATHERQDZ256rm = 16812,
  VPGATHERQDZrm = 16813,
  VPGATHERQDrm = 16814,
  VPGATHERQQYrm = 16815,
  VPGATHERQQZ128rm = 16816,
  VPGATHERQQZ256rm = 16817,
  VPGATHERQQZrm = 16818,
  VPGATHERQQrm = 16819,
  VPHADDBDrm = 16820,
  VPHADDBDrr = 16821,
  VPHADDBQrm = 16822,
  VPHADDBQrr = 16823,
  VPHADDBWrm = 16824,
  VPHADDBWrr = 16825,
  VPHADDDQrm = 16826,
  VPHADDDQrr = 16827,
  VPHADDDYrm = 16828,
  VPHADDDYrr = 16829,
  VPHADDDrm = 16830,
  VPHADDDrr = 16831,
  VPHADDSWYrm = 16832,
  VPHADDSWYrr = 16833,
  VPHADDSWrm = 16834,
  VPHADDSWrr = 16835,
  VPHADDUBDrm = 16836,
  VPHADDUBDrr = 16837,
  VPHADDUBQrm = 16838,
  VPHADDUBQrr = 16839,
  VPHADDUBWrm = 16840,
  VPHADDUBWrr = 16841,
  VPHADDUDQrm = 16842,
  VPHADDUDQrr = 16843,
  VPHADDUWDrm = 16844,
  VPHADDUWDrr = 16845,
  VPHADDUWQrm = 16846,
  VPHADDUWQrr = 16847,
  VPHADDWDrm = 16848,
  VPHADDWDrr = 16849,
  VPHADDWQrm = 16850,
  VPHADDWQrr = 16851,
  VPHADDWYrm = 16852,
  VPHADDWYrr = 16853,
  VPHADDWrm = 16854,
  VPHADDWrr = 16855,
  VPHMINPOSUWrm = 16856,
  VPHMINPOSUWrr = 16857,
  VPHSUBBWrm = 16858,
  VPHSUBBWrr = 16859,
  VPHSUBDQrm = 16860,
  VPHSUBDQrr = 16861,
  VPHSUBDYrm = 16862,
  VPHSUBDYrr = 16863,
  VPHSUBDrm = 16864,
  VPHSUBDrr = 16865,
  VPHSUBSWYrm = 16866,
  VPHSUBSWYrr = 16867,
  VPHSUBSWrm = 16868,
  VPHSUBSWrr = 16869,
  VPHSUBWDrm = 16870,
  VPHSUBWDrr = 16871,
  VPHSUBWYrm = 16872,
  VPHSUBWYrr = 16873,
  VPHSUBWrm = 16874,
  VPHSUBWrr = 16875,
  VPINSRBZrmi = 16876,
  VPINSRBZrri = 16877,
  VPINSRBrmi = 16878,
  VPINSRBrri = 16879,
  VPINSRDZrmi = 16880,
  VPINSRDZrri = 16881,
  VPINSRDrmi = 16882,
  VPINSRDrri = 16883,
  VPINSRQZrmi = 16884,
  VPINSRQZrri = 16885,
  VPINSRQrmi = 16886,
  VPINSRQrri = 16887,
  VPINSRWZrmi = 16888,
  VPINSRWZrri = 16889,
  VPINSRWrmi = 16890,
  VPINSRWrri = 16891,
  VPLZCNTDZ128rm = 16892,
  VPLZCNTDZ128rmb = 16893,
  VPLZCNTDZ128rmbk = 16894,
  VPLZCNTDZ128rmbkz = 16895,
  VPLZCNTDZ128rmk = 16896,
  VPLZCNTDZ128rmkz = 16897,
  VPLZCNTDZ128rr = 16898,
  VPLZCNTDZ128rrk = 16899,
  VPLZCNTDZ128rrkz = 16900,
  VPLZCNTDZ256rm = 16901,
  VPLZCNTDZ256rmb = 16902,
  VPLZCNTDZ256rmbk = 16903,
  VPLZCNTDZ256rmbkz = 16904,
  VPLZCNTDZ256rmk = 16905,
  VPLZCNTDZ256rmkz = 16906,
  VPLZCNTDZ256rr = 16907,
  VPLZCNTDZ256rrk = 16908,
  VPLZCNTDZ256rrkz = 16909,
  VPLZCNTDZrm = 16910,
  VPLZCNTDZrmb = 16911,
  VPLZCNTDZrmbk = 16912,
  VPLZCNTDZrmbkz = 16913,
  VPLZCNTDZrmk = 16914,
  VPLZCNTDZrmkz = 16915,
  VPLZCNTDZrr = 16916,
  VPLZCNTDZrrk = 16917,
  VPLZCNTDZrrkz = 16918,
  VPLZCNTQZ128rm = 16919,
  VPLZCNTQZ128rmb = 16920,
  VPLZCNTQZ128rmbk = 16921,
  VPLZCNTQZ128rmbkz = 16922,
  VPLZCNTQZ128rmk = 16923,
  VPLZCNTQZ128rmkz = 16924,
  VPLZCNTQZ128rr = 16925,
  VPLZCNTQZ128rrk = 16926,
  VPLZCNTQZ128rrkz = 16927,
  VPLZCNTQZ256rm = 16928,
  VPLZCNTQZ256rmb = 16929,
  VPLZCNTQZ256rmbk = 16930,
  VPLZCNTQZ256rmbkz = 16931,
  VPLZCNTQZ256rmk = 16932,
  VPLZCNTQZ256rmkz = 16933,
  VPLZCNTQZ256rr = 16934,
  VPLZCNTQZ256rrk = 16935,
  VPLZCNTQZ256rrkz = 16936,
  VPLZCNTQZrm = 16937,
  VPLZCNTQZrmb = 16938,
  VPLZCNTQZrmbk = 16939,
  VPLZCNTQZrmbkz = 16940,
  VPLZCNTQZrmk = 16941,
  VPLZCNTQZrmkz = 16942,
  VPLZCNTQZrr = 16943,
  VPLZCNTQZrrk = 16944,
  VPLZCNTQZrrkz = 16945,
  VPMACSDDrm = 16946,
  VPMACSDDrr = 16947,
  VPMACSDQHrm = 16948,
  VPMACSDQHrr = 16949,
  VPMACSDQLrm = 16950,
  VPMACSDQLrr = 16951,
  VPMACSSDDrm = 16952,
  VPMACSSDDrr = 16953,
  VPMACSSDQHrm = 16954,
  VPMACSSDQHrr = 16955,
  VPMACSSDQLrm = 16956,
  VPMACSSDQLrr = 16957,
  VPMACSSWDrm = 16958,
  VPMACSSWDrr = 16959,
  VPMACSSWWrm = 16960,
  VPMACSSWWrr = 16961,
  VPMACSWDrm = 16962,
  VPMACSWDrr = 16963,
  VPMACSWWrm = 16964,
  VPMACSWWrr = 16965,
  VPMADCSSWDrm = 16966,
  VPMADCSSWDrr = 16967,
  VPMADCSWDrm = 16968,
  VPMADCSWDrr = 16969,
  VPMADD52HUQYrm = 16970,
  VPMADD52HUQYrr = 16971,
  VPMADD52HUQZ128m = 16972,
  VPMADD52HUQZ128mb = 16973,
  VPMADD52HUQZ128mbk = 16974,
  VPMADD52HUQZ128mbkz = 16975,
  VPMADD52HUQZ128mk = 16976,
  VPMADD52HUQZ128mkz = 16977,
  VPMADD52HUQZ128r = 16978,
  VPMADD52HUQZ128rk = 16979,
  VPMADD52HUQZ128rkz = 16980,
  VPMADD52HUQZ256m = 16981,
  VPMADD52HUQZ256mb = 16982,
  VPMADD52HUQZ256mbk = 16983,
  VPMADD52HUQZ256mbkz = 16984,
  VPMADD52HUQZ256mk = 16985,
  VPMADD52HUQZ256mkz = 16986,
  VPMADD52HUQZ256r = 16987,
  VPMADD52HUQZ256rk = 16988,
  VPMADD52HUQZ256rkz = 16989,
  VPMADD52HUQZm = 16990,
  VPMADD52HUQZmb = 16991,
  VPMADD52HUQZmbk = 16992,
  VPMADD52HUQZmbkz = 16993,
  VPMADD52HUQZmk = 16994,
  VPMADD52HUQZmkz = 16995,
  VPMADD52HUQZr = 16996,
  VPMADD52HUQZrk = 16997,
  VPMADD52HUQZrkz = 16998,
  VPMADD52HUQrm = 16999,
  VPMADD52HUQrr = 17000,
  VPMADD52LUQYrm = 17001,
  VPMADD52LUQYrr = 17002,
  VPMADD52LUQZ128m = 17003,
  VPMADD52LUQZ128mb = 17004,
  VPMADD52LUQZ128mbk = 17005,
  VPMADD52LUQZ128mbkz = 17006,
  VPMADD52LUQZ128mk = 17007,
  VPMADD52LUQZ128mkz = 17008,
  VPMADD52LUQZ128r = 17009,
  VPMADD52LUQZ128rk = 17010,
  VPMADD52LUQZ128rkz = 17011,
  VPMADD52LUQZ256m = 17012,
  VPMADD52LUQZ256mb = 17013,
  VPMADD52LUQZ256mbk = 17014,
  VPMADD52LUQZ256mbkz = 17015,
  VPMADD52LUQZ256mk = 17016,
  VPMADD52LUQZ256mkz = 17017,
  VPMADD52LUQZ256r = 17018,
  VPMADD52LUQZ256rk = 17019,
  VPMADD52LUQZ256rkz = 17020,
  VPMADD52LUQZm = 17021,
  VPMADD52LUQZmb = 17022,
  VPMADD52LUQZmbk = 17023,
  VPMADD52LUQZmbkz = 17024,
  VPMADD52LUQZmk = 17025,
  VPMADD52LUQZmkz = 17026,
  VPMADD52LUQZr = 17027,
  VPMADD52LUQZrk = 17028,
  VPMADD52LUQZrkz = 17029,
  VPMADD52LUQrm = 17030,
  VPMADD52LUQrr = 17031,
  VPMADDUBSWYrm = 17032,
  VPMADDUBSWYrr = 17033,
  VPMADDUBSWZ128rm = 17034,
  VPMADDUBSWZ128rmk = 17035,
  VPMADDUBSWZ128rmkz = 17036,
  VPMADDUBSWZ128rr = 17037,
  VPMADDUBSWZ128rrk = 17038,
  VPMADDUBSWZ128rrkz = 17039,
  VPMADDUBSWZ256rm = 17040,
  VPMADDUBSWZ256rmk = 17041,
  VPMADDUBSWZ256rmkz = 17042,
  VPMADDUBSWZ256rr = 17043,
  VPMADDUBSWZ256rrk = 17044,
  VPMADDUBSWZ256rrkz = 17045,
  VPMADDUBSWZrm = 17046,
  VPMADDUBSWZrmk = 17047,
  VPMADDUBSWZrmkz = 17048,
  VPMADDUBSWZrr = 17049,
  VPMADDUBSWZrrk = 17050,
  VPMADDUBSWZrrkz = 17051,
  VPMADDUBSWrm = 17052,
  VPMADDUBSWrr = 17053,
  VPMADDWDYrm = 17054,
  VPMADDWDYrr = 17055,
  VPMADDWDZ128rm = 17056,
  VPMADDWDZ128rmk = 17057,
  VPMADDWDZ128rmkz = 17058,
  VPMADDWDZ128rr = 17059,
  VPMADDWDZ128rrk = 17060,
  VPMADDWDZ128rrkz = 17061,
  VPMADDWDZ256rm = 17062,
  VPMADDWDZ256rmk = 17063,
  VPMADDWDZ256rmkz = 17064,
  VPMADDWDZ256rr = 17065,
  VPMADDWDZ256rrk = 17066,
  VPMADDWDZ256rrkz = 17067,
  VPMADDWDZrm = 17068,
  VPMADDWDZrmk = 17069,
  VPMADDWDZrmkz = 17070,
  VPMADDWDZrr = 17071,
  VPMADDWDZrrk = 17072,
  VPMADDWDZrrkz = 17073,
  VPMADDWDrm = 17074,
  VPMADDWDrr = 17075,
  VPMASKMOVDYmr = 17076,
  VPMASKMOVDYrm = 17077,
  VPMASKMOVDmr = 17078,
  VPMASKMOVDrm = 17079,
  VPMASKMOVQYmr = 17080,
  VPMASKMOVQYrm = 17081,
  VPMASKMOVQmr = 17082,
  VPMASKMOVQrm = 17083,
  VPMAXSBYrm = 17084,
  VPMAXSBYrr = 17085,
  VPMAXSBZ128rm = 17086,
  VPMAXSBZ128rmk = 17087,
  VPMAXSBZ128rmkz = 17088,
  VPMAXSBZ128rr = 17089,
  VPMAXSBZ128rrk = 17090,
  VPMAXSBZ128rrkz = 17091,
  VPMAXSBZ256rm = 17092,
  VPMAXSBZ256rmk = 17093,
  VPMAXSBZ256rmkz = 17094,
  VPMAXSBZ256rr = 17095,
  VPMAXSBZ256rrk = 17096,
  VPMAXSBZ256rrkz = 17097,
  VPMAXSBZrm = 17098,
  VPMAXSBZrmk = 17099,
  VPMAXSBZrmkz = 17100,
  VPMAXSBZrr = 17101,
  VPMAXSBZrrk = 17102,
  VPMAXSBZrrkz = 17103,
  VPMAXSBrm = 17104,
  VPMAXSBrr = 17105,
  VPMAXSDYrm = 17106,
  VPMAXSDYrr = 17107,
  VPMAXSDZ128rm = 17108,
  VPMAXSDZ128rmb = 17109,
  VPMAXSDZ128rmbk = 17110,
  VPMAXSDZ128rmbkz = 17111,
  VPMAXSDZ128rmk = 17112,
  VPMAXSDZ128rmkz = 17113,
  VPMAXSDZ128rr = 17114,
  VPMAXSDZ128rrk = 17115,
  VPMAXSDZ128rrkz = 17116,
  VPMAXSDZ256rm = 17117,
  VPMAXSDZ256rmb = 17118,
  VPMAXSDZ256rmbk = 17119,
  VPMAXSDZ256rmbkz = 17120,
  VPMAXSDZ256rmk = 17121,
  VPMAXSDZ256rmkz = 17122,
  VPMAXSDZ256rr = 17123,
  VPMAXSDZ256rrk = 17124,
  VPMAXSDZ256rrkz = 17125,
  VPMAXSDZrm = 17126,
  VPMAXSDZrmb = 17127,
  VPMAXSDZrmbk = 17128,
  VPMAXSDZrmbkz = 17129,
  VPMAXSDZrmk = 17130,
  VPMAXSDZrmkz = 17131,
  VPMAXSDZrr = 17132,
  VPMAXSDZrrk = 17133,
  VPMAXSDZrrkz = 17134,
  VPMAXSDrm = 17135,
  VPMAXSDrr = 17136,
  VPMAXSQZ128rm = 17137,
  VPMAXSQZ128rmb = 17138,
  VPMAXSQZ128rmbk = 17139,
  VPMAXSQZ128rmbkz = 17140,
  VPMAXSQZ128rmk = 17141,
  VPMAXSQZ128rmkz = 17142,
  VPMAXSQZ128rr = 17143,
  VPMAXSQZ128rrk = 17144,
  VPMAXSQZ128rrkz = 17145,
  VPMAXSQZ256rm = 17146,
  VPMAXSQZ256rmb = 17147,
  VPMAXSQZ256rmbk = 17148,
  VPMAXSQZ256rmbkz = 17149,
  VPMAXSQZ256rmk = 17150,
  VPMAXSQZ256rmkz = 17151,
  VPMAXSQZ256rr = 17152,
  VPMAXSQZ256rrk = 17153,
  VPMAXSQZ256rrkz = 17154,
  VPMAXSQZrm = 17155,
  VPMAXSQZrmb = 17156,
  VPMAXSQZrmbk = 17157,
  VPMAXSQZrmbkz = 17158,
  VPMAXSQZrmk = 17159,
  VPMAXSQZrmkz = 17160,
  VPMAXSQZrr = 17161,
  VPMAXSQZrrk = 17162,
  VPMAXSQZrrkz = 17163,
  VPMAXSWYrm = 17164,
  VPMAXSWYrr = 17165,
  VPMAXSWZ128rm = 17166,
  VPMAXSWZ128rmk = 17167,
  VPMAXSWZ128rmkz = 17168,
  VPMAXSWZ128rr = 17169,
  VPMAXSWZ128rrk = 17170,
  VPMAXSWZ128rrkz = 17171,
  VPMAXSWZ256rm = 17172,
  VPMAXSWZ256rmk = 17173,
  VPMAXSWZ256rmkz = 17174,
  VPMAXSWZ256rr = 17175,
  VPMAXSWZ256rrk = 17176,
  VPMAXSWZ256rrkz = 17177,
  VPMAXSWZrm = 17178,
  VPMAXSWZrmk = 17179,
  VPMAXSWZrmkz = 17180,
  VPMAXSWZrr = 17181,
  VPMAXSWZrrk = 17182,
  VPMAXSWZrrkz = 17183,
  VPMAXSWrm = 17184,
  VPMAXSWrr = 17185,
  VPMAXUBYrm = 17186,
  VPMAXUBYrr = 17187,
  VPMAXUBZ128rm = 17188,
  VPMAXUBZ128rmk = 17189,
  VPMAXUBZ128rmkz = 17190,
  VPMAXUBZ128rr = 17191,
  VPMAXUBZ128rrk = 17192,
  VPMAXUBZ128rrkz = 17193,
  VPMAXUBZ256rm = 17194,
  VPMAXUBZ256rmk = 17195,
  VPMAXUBZ256rmkz = 17196,
  VPMAXUBZ256rr = 17197,
  VPMAXUBZ256rrk = 17198,
  VPMAXUBZ256rrkz = 17199,
  VPMAXUBZrm = 17200,
  VPMAXUBZrmk = 17201,
  VPMAXUBZrmkz = 17202,
  VPMAXUBZrr = 17203,
  VPMAXUBZrrk = 17204,
  VPMAXUBZrrkz = 17205,
  VPMAXUBrm = 17206,
  VPMAXUBrr = 17207,
  VPMAXUDYrm = 17208,
  VPMAXUDYrr = 17209,
  VPMAXUDZ128rm = 17210,
  VPMAXUDZ128rmb = 17211,
  VPMAXUDZ128rmbk = 17212,
  VPMAXUDZ128rmbkz = 17213,
  VPMAXUDZ128rmk = 17214,
  VPMAXUDZ128rmkz = 17215,
  VPMAXUDZ128rr = 17216,
  VPMAXUDZ128rrk = 17217,
  VPMAXUDZ128rrkz = 17218,
  VPMAXUDZ256rm = 17219,
  VPMAXUDZ256rmb = 17220,
  VPMAXUDZ256rmbk = 17221,
  VPMAXUDZ256rmbkz = 17222,
  VPMAXUDZ256rmk = 17223,
  VPMAXUDZ256rmkz = 17224,
  VPMAXUDZ256rr = 17225,
  VPMAXUDZ256rrk = 17226,
  VPMAXUDZ256rrkz = 17227,
  VPMAXUDZrm = 17228,
  VPMAXUDZrmb = 17229,
  VPMAXUDZrmbk = 17230,
  VPMAXUDZrmbkz = 17231,
  VPMAXUDZrmk = 17232,
  VPMAXUDZrmkz = 17233,
  VPMAXUDZrr = 17234,
  VPMAXUDZrrk = 17235,
  VPMAXUDZrrkz = 17236,
  VPMAXUDrm = 17237,
  VPMAXUDrr = 17238,
  VPMAXUQZ128rm = 17239,
  VPMAXUQZ128rmb = 17240,
  VPMAXUQZ128rmbk = 17241,
  VPMAXUQZ128rmbkz = 17242,
  VPMAXUQZ128rmk = 17243,
  VPMAXUQZ128rmkz = 17244,
  VPMAXUQZ128rr = 17245,
  VPMAXUQZ128rrk = 17246,
  VPMAXUQZ128rrkz = 17247,
  VPMAXUQZ256rm = 17248,
  VPMAXUQZ256rmb = 17249,
  VPMAXUQZ256rmbk = 17250,
  VPMAXUQZ256rmbkz = 17251,
  VPMAXUQZ256rmk = 17252,
  VPMAXUQZ256rmkz = 17253,
  VPMAXUQZ256rr = 17254,
  VPMAXUQZ256rrk = 17255,
  VPMAXUQZ256rrkz = 17256,
  VPMAXUQZrm = 17257,
  VPMAXUQZrmb = 17258,
  VPMAXUQZrmbk = 17259,
  VPMAXUQZrmbkz = 17260,
  VPMAXUQZrmk = 17261,
  VPMAXUQZrmkz = 17262,
  VPMAXUQZrr = 17263,
  VPMAXUQZrrk = 17264,
  VPMAXUQZrrkz = 17265,
  VPMAXUWYrm = 17266,
  VPMAXUWYrr = 17267,
  VPMAXUWZ128rm = 17268,
  VPMAXUWZ128rmk = 17269,
  VPMAXUWZ128rmkz = 17270,
  VPMAXUWZ128rr = 17271,
  VPMAXUWZ128rrk = 17272,
  VPMAXUWZ128rrkz = 17273,
  VPMAXUWZ256rm = 17274,
  VPMAXUWZ256rmk = 17275,
  VPMAXUWZ256rmkz = 17276,
  VPMAXUWZ256rr = 17277,
  VPMAXUWZ256rrk = 17278,
  VPMAXUWZ256rrkz = 17279,
  VPMAXUWZrm = 17280,
  VPMAXUWZrmk = 17281,
  VPMAXUWZrmkz = 17282,
  VPMAXUWZrr = 17283,
  VPMAXUWZrrk = 17284,
  VPMAXUWZrrkz = 17285,
  VPMAXUWrm = 17286,
  VPMAXUWrr = 17287,
  VPMINSBYrm = 17288,
  VPMINSBYrr = 17289,
  VPMINSBZ128rm = 17290,
  VPMINSBZ128rmk = 17291,
  VPMINSBZ128rmkz = 17292,
  VPMINSBZ128rr = 17293,
  VPMINSBZ128rrk = 17294,
  VPMINSBZ128rrkz = 17295,
  VPMINSBZ256rm = 17296,
  VPMINSBZ256rmk = 17297,
  VPMINSBZ256rmkz = 17298,
  VPMINSBZ256rr = 17299,
  VPMINSBZ256rrk = 17300,
  VPMINSBZ256rrkz = 17301,
  VPMINSBZrm = 17302,
  VPMINSBZrmk = 17303,
  VPMINSBZrmkz = 17304,
  VPMINSBZrr = 17305,
  VPMINSBZrrk = 17306,
  VPMINSBZrrkz = 17307,
  VPMINSBrm = 17308,
  VPMINSBrr = 17309,
  VPMINSDYrm = 17310,
  VPMINSDYrr = 17311,
  VPMINSDZ128rm = 17312,
  VPMINSDZ128rmb = 17313,
  VPMINSDZ128rmbk = 17314,
  VPMINSDZ128rmbkz = 17315,
  VPMINSDZ128rmk = 17316,
  VPMINSDZ128rmkz = 17317,
  VPMINSDZ128rr = 17318,
  VPMINSDZ128rrk = 17319,
  VPMINSDZ128rrkz = 17320,
  VPMINSDZ256rm = 17321,
  VPMINSDZ256rmb = 17322,
  VPMINSDZ256rmbk = 17323,
  VPMINSDZ256rmbkz = 17324,
  VPMINSDZ256rmk = 17325,
  VPMINSDZ256rmkz = 17326,
  VPMINSDZ256rr = 17327,
  VPMINSDZ256rrk = 17328,
  VPMINSDZ256rrkz = 17329,
  VPMINSDZrm = 17330,
  VPMINSDZrmb = 17331,
  VPMINSDZrmbk = 17332,
  VPMINSDZrmbkz = 17333,
  VPMINSDZrmk = 17334,
  VPMINSDZrmkz = 17335,
  VPMINSDZrr = 17336,
  VPMINSDZrrk = 17337,
  VPMINSDZrrkz = 17338,
  VPMINSDrm = 17339,
  VPMINSDrr = 17340,
  VPMINSQZ128rm = 17341,
  VPMINSQZ128rmb = 17342,
  VPMINSQZ128rmbk = 17343,
  VPMINSQZ128rmbkz = 17344,
  VPMINSQZ128rmk = 17345,
  VPMINSQZ128rmkz = 17346,
  VPMINSQZ128rr = 17347,
  VPMINSQZ128rrk = 17348,
  VPMINSQZ128rrkz = 17349,
  VPMINSQZ256rm = 17350,
  VPMINSQZ256rmb = 17351,
  VPMINSQZ256rmbk = 17352,
  VPMINSQZ256rmbkz = 17353,
  VPMINSQZ256rmk = 17354,
  VPMINSQZ256rmkz = 17355,
  VPMINSQZ256rr = 17356,
  VPMINSQZ256rrk = 17357,
  VPMINSQZ256rrkz = 17358,
  VPMINSQZrm = 17359,
  VPMINSQZrmb = 17360,
  VPMINSQZrmbk = 17361,
  VPMINSQZrmbkz = 17362,
  VPMINSQZrmk = 17363,
  VPMINSQZrmkz = 17364,
  VPMINSQZrr = 17365,
  VPMINSQZrrk = 17366,
  VPMINSQZrrkz = 17367,
  VPMINSWYrm = 17368,
  VPMINSWYrr = 17369,
  VPMINSWZ128rm = 17370,
  VPMINSWZ128rmk = 17371,
  VPMINSWZ128rmkz = 17372,
  VPMINSWZ128rr = 17373,
  VPMINSWZ128rrk = 17374,
  VPMINSWZ128rrkz = 17375,
  VPMINSWZ256rm = 17376,
  VPMINSWZ256rmk = 17377,
  VPMINSWZ256rmkz = 17378,
  VPMINSWZ256rr = 17379,
  VPMINSWZ256rrk = 17380,
  VPMINSWZ256rrkz = 17381,
  VPMINSWZrm = 17382,
  VPMINSWZrmk = 17383,
  VPMINSWZrmkz = 17384,
  VPMINSWZrr = 17385,
  VPMINSWZrrk = 17386,
  VPMINSWZrrkz = 17387,
  VPMINSWrm = 17388,
  VPMINSWrr = 17389,
  VPMINUBYrm = 17390,
  VPMINUBYrr = 17391,
  VPMINUBZ128rm = 17392,
  VPMINUBZ128rmk = 17393,
  VPMINUBZ128rmkz = 17394,
  VPMINUBZ128rr = 17395,
  VPMINUBZ128rrk = 17396,
  VPMINUBZ128rrkz = 17397,
  VPMINUBZ256rm = 17398,
  VPMINUBZ256rmk = 17399,
  VPMINUBZ256rmkz = 17400,
  VPMINUBZ256rr = 17401,
  VPMINUBZ256rrk = 17402,
  VPMINUBZ256rrkz = 17403,
  VPMINUBZrm = 17404,
  VPMINUBZrmk = 17405,
  VPMINUBZrmkz = 17406,
  VPMINUBZrr = 17407,
  VPMINUBZrrk = 17408,
  VPMINUBZrrkz = 17409,
  VPMINUBrm = 17410,
  VPMINUBrr = 17411,
  VPMINUDYrm = 17412,
  VPMINUDYrr = 17413,
  VPMINUDZ128rm = 17414,
  VPMINUDZ128rmb = 17415,
  VPMINUDZ128rmbk = 17416,
  VPMINUDZ128rmbkz = 17417,
  VPMINUDZ128rmk = 17418,
  VPMINUDZ128rmkz = 17419,
  VPMINUDZ128rr = 17420,
  VPMINUDZ128rrk = 17421,
  VPMINUDZ128rrkz = 17422,
  VPMINUDZ256rm = 17423,
  VPMINUDZ256rmb = 17424,
  VPMINUDZ256rmbk = 17425,
  VPMINUDZ256rmbkz = 17426,
  VPMINUDZ256rmk = 17427,
  VPMINUDZ256rmkz = 17428,
  VPMINUDZ256rr = 17429,
  VPMINUDZ256rrk = 17430,
  VPMINUDZ256rrkz = 17431,
  VPMINUDZrm = 17432,
  VPMINUDZrmb = 17433,
  VPMINUDZrmbk = 17434,
  VPMINUDZrmbkz = 17435,
  VPMINUDZrmk = 17436,
  VPMINUDZrmkz = 17437,
  VPMINUDZrr = 17438,
  VPMINUDZrrk = 17439,
  VPMINUDZrrkz = 17440,
  VPMINUDrm = 17441,
  VPMINUDrr = 17442,
  VPMINUQZ128rm = 17443,
  VPMINUQZ128rmb = 17444,
  VPMINUQZ128rmbk = 17445,
  VPMINUQZ128rmbkz = 17446,
  VPMINUQZ128rmk = 17447,
  VPMINUQZ128rmkz = 17448,
  VPMINUQZ128rr = 17449,
  VPMINUQZ128rrk = 17450,
  VPMINUQZ128rrkz = 17451,
  VPMINUQZ256rm = 17452,
  VPMINUQZ256rmb = 17453,
  VPMINUQZ256rmbk = 17454,
  VPMINUQZ256rmbkz = 17455,
  VPMINUQZ256rmk = 17456,
  VPMINUQZ256rmkz = 17457,
  VPMINUQZ256rr = 17458,
  VPMINUQZ256rrk = 17459,
  VPMINUQZ256rrkz = 17460,
  VPMINUQZrm = 17461,
  VPMINUQZrmb = 17462,
  VPMINUQZrmbk = 17463,
  VPMINUQZrmbkz = 17464,
  VPMINUQZrmk = 17465,
  VPMINUQZrmkz = 17466,
  VPMINUQZrr = 17467,
  VPMINUQZrrk = 17468,
  VPMINUQZrrkz = 17469,
  VPMINUWYrm = 17470,
  VPMINUWYrr = 17471,
  VPMINUWZ128rm = 17472,
  VPMINUWZ128rmk = 17473,
  VPMINUWZ128rmkz = 17474,
  VPMINUWZ128rr = 17475,
  VPMINUWZ128rrk = 17476,
  VPMINUWZ128rrkz = 17477,
  VPMINUWZ256rm = 17478,
  VPMINUWZ256rmk = 17479,
  VPMINUWZ256rmkz = 17480,
  VPMINUWZ256rr = 17481,
  VPMINUWZ256rrk = 17482,
  VPMINUWZ256rrkz = 17483,
  VPMINUWZrm = 17484,
  VPMINUWZrmk = 17485,
  VPMINUWZrmkz = 17486,
  VPMINUWZrr = 17487,
  VPMINUWZrrk = 17488,
  VPMINUWZrrkz = 17489,
  VPMINUWrm = 17490,
  VPMINUWrr = 17491,
  VPMOVB2MZ128kr = 17492,
  VPMOVB2MZ256kr = 17493,
  VPMOVB2MZkr = 17494,
  VPMOVD2MZ128kr = 17495,
  VPMOVD2MZ256kr = 17496,
  VPMOVD2MZkr = 17497,
  VPMOVDBZ128mr = 17498,
  VPMOVDBZ128mrk = 17499,
  VPMOVDBZ128rr = 17500,
  VPMOVDBZ128rrk = 17501,
  VPMOVDBZ128rrkz = 17502,
  VPMOVDBZ256mr = 17503,
  VPMOVDBZ256mrk = 17504,
  VPMOVDBZ256rr = 17505,
  VPMOVDBZ256rrk = 17506,
  VPMOVDBZ256rrkz = 17507,
  VPMOVDBZmr = 17508,
  VPMOVDBZmrk = 17509,
  VPMOVDBZrr = 17510,
  VPMOVDBZrrk = 17511,
  VPMOVDBZrrkz = 17512,
  VPMOVDWZ128mr = 17513,
  VPMOVDWZ128mrk = 17514,
  VPMOVDWZ128rr = 17515,
  VPMOVDWZ128rrk = 17516,
  VPMOVDWZ128rrkz = 17517,
  VPMOVDWZ256mr = 17518,
  VPMOVDWZ256mrk = 17519,
  VPMOVDWZ256rr = 17520,
  VPMOVDWZ256rrk = 17521,
  VPMOVDWZ256rrkz = 17522,
  VPMOVDWZmr = 17523,
  VPMOVDWZmrk = 17524,
  VPMOVDWZrr = 17525,
  VPMOVDWZrrk = 17526,
  VPMOVDWZrrkz = 17527,
  VPMOVM2BZ128rk = 17528,
  VPMOVM2BZ256rk = 17529,
  VPMOVM2BZrk = 17530,
  VPMOVM2DZ128rk = 17531,
  VPMOVM2DZ256rk = 17532,
  VPMOVM2DZrk = 17533,
  VPMOVM2QZ128rk = 17534,
  VPMOVM2QZ256rk = 17535,
  VPMOVM2QZrk = 17536,
  VPMOVM2WZ128rk = 17537,
  VPMOVM2WZ256rk = 17538,
  VPMOVM2WZrk = 17539,
  VPMOVMSKBYrr = 17540,
  VPMOVMSKBrr = 17541,
  VPMOVQ2MZ128kr = 17542,
  VPMOVQ2MZ256kr = 17543,
  VPMOVQ2MZkr = 17544,
  VPMOVQBZ128mr = 17545,
  VPMOVQBZ128mrk = 17546,
  VPMOVQBZ128rr = 17547,
  VPMOVQBZ128rrk = 17548,
  VPMOVQBZ128rrkz = 17549,
  VPMOVQBZ256mr = 17550,
  VPMOVQBZ256mrk = 17551,
  VPMOVQBZ256rr = 17552,
  VPMOVQBZ256rrk = 17553,
  VPMOVQBZ256rrkz = 17554,
  VPMOVQBZmr = 17555,
  VPMOVQBZmrk = 17556,
  VPMOVQBZrr = 17557,
  VPMOVQBZrrk = 17558,
  VPMOVQBZrrkz = 17559,
  VPMOVQDZ128mr = 17560,
  VPMOVQDZ128mrk = 17561,
  VPMOVQDZ128rr = 17562,
  VPMOVQDZ128rrk = 17563,
  VPMOVQDZ128rrkz = 17564,
  VPMOVQDZ256mr = 17565,
  VPMOVQDZ256mrk = 17566,
  VPMOVQDZ256rr = 17567,
  VPMOVQDZ256rrk = 17568,
  VPMOVQDZ256rrkz = 17569,
  VPMOVQDZmr = 17570,
  VPMOVQDZmrk = 17571,
  VPMOVQDZrr = 17572,
  VPMOVQDZrrk = 17573,
  VPMOVQDZrrkz = 17574,
  VPMOVQWZ128mr = 17575,
  VPMOVQWZ128mrk = 17576,
  VPMOVQWZ128rr = 17577,
  VPMOVQWZ128rrk = 17578,
  VPMOVQWZ128rrkz = 17579,
  VPMOVQWZ256mr = 17580,
  VPMOVQWZ256mrk = 17581,
  VPMOVQWZ256rr = 17582,
  VPMOVQWZ256rrk = 17583,
  VPMOVQWZ256rrkz = 17584,
  VPMOVQWZmr = 17585,
  VPMOVQWZmrk = 17586,
  VPMOVQWZrr = 17587,
  VPMOVQWZrrk = 17588,
  VPMOVQWZrrkz = 17589,
  VPMOVSDBZ128mr = 17590,
  VPMOVSDBZ128mrk = 17591,
  VPMOVSDBZ128rr = 17592,
  VPMOVSDBZ128rrk = 17593,
  VPMOVSDBZ128rrkz = 17594,
  VPMOVSDBZ256mr = 17595,
  VPMOVSDBZ256mrk = 17596,
  VPMOVSDBZ256rr = 17597,
  VPMOVSDBZ256rrk = 17598,
  VPMOVSDBZ256rrkz = 17599,
  VPMOVSDBZmr = 17600,
  VPMOVSDBZmrk = 17601,
  VPMOVSDBZrr = 17602,
  VPMOVSDBZrrk = 17603,
  VPMOVSDBZrrkz = 17604,
  VPMOVSDWZ128mr = 17605,
  VPMOVSDWZ128mrk = 17606,
  VPMOVSDWZ128rr = 17607,
  VPMOVSDWZ128rrk = 17608,
  VPMOVSDWZ128rrkz = 17609,
  VPMOVSDWZ256mr = 17610,
  VPMOVSDWZ256mrk = 17611,
  VPMOVSDWZ256rr = 17612,
  VPMOVSDWZ256rrk = 17613,
  VPMOVSDWZ256rrkz = 17614,
  VPMOVSDWZmr = 17615,
  VPMOVSDWZmrk = 17616,
  VPMOVSDWZrr = 17617,
  VPMOVSDWZrrk = 17618,
  VPMOVSDWZrrkz = 17619,
  VPMOVSQBZ128mr = 17620,
  VPMOVSQBZ128mrk = 17621,
  VPMOVSQBZ128rr = 17622,
  VPMOVSQBZ128rrk = 17623,
  VPMOVSQBZ128rrkz = 17624,
  VPMOVSQBZ256mr = 17625,
  VPMOVSQBZ256mrk = 17626,
  VPMOVSQBZ256rr = 17627,
  VPMOVSQBZ256rrk = 17628,
  VPMOVSQBZ256rrkz = 17629,
  VPMOVSQBZmr = 17630,
  VPMOVSQBZmrk = 17631,
  VPMOVSQBZrr = 17632,
  VPMOVSQBZrrk = 17633,
  VPMOVSQBZrrkz = 17634,
  VPMOVSQDZ128mr = 17635,
  VPMOVSQDZ128mrk = 17636,
  VPMOVSQDZ128rr = 17637,
  VPMOVSQDZ128rrk = 17638,
  VPMOVSQDZ128rrkz = 17639,
  VPMOVSQDZ256mr = 17640,
  VPMOVSQDZ256mrk = 17641,
  VPMOVSQDZ256rr = 17642,
  VPMOVSQDZ256rrk = 17643,
  VPMOVSQDZ256rrkz = 17644,
  VPMOVSQDZmr = 17645,
  VPMOVSQDZmrk = 17646,
  VPMOVSQDZrr = 17647,
  VPMOVSQDZrrk = 17648,
  VPMOVSQDZrrkz = 17649,
  VPMOVSQWZ128mr = 17650,
  VPMOVSQWZ128mrk = 17651,
  VPMOVSQWZ128rr = 17652,
  VPMOVSQWZ128rrk = 17653,
  VPMOVSQWZ128rrkz = 17654,
  VPMOVSQWZ256mr = 17655,
  VPMOVSQWZ256mrk = 17656,
  VPMOVSQWZ256rr = 17657,
  VPMOVSQWZ256rrk = 17658,
  VPMOVSQWZ256rrkz = 17659,
  VPMOVSQWZmr = 17660,
  VPMOVSQWZmrk = 17661,
  VPMOVSQWZrr = 17662,
  VPMOVSQWZrrk = 17663,
  VPMOVSQWZrrkz = 17664,
  VPMOVSWBZ128mr = 17665,
  VPMOVSWBZ128mrk = 17666,
  VPMOVSWBZ128rr = 17667,
  VPMOVSWBZ128rrk = 17668,
  VPMOVSWBZ128rrkz = 17669,
  VPMOVSWBZ256mr = 17670,
  VPMOVSWBZ256mrk = 17671,
  VPMOVSWBZ256rr = 17672,
  VPMOVSWBZ256rrk = 17673,
  VPMOVSWBZ256rrkz = 17674,
  VPMOVSWBZmr = 17675,
  VPMOVSWBZmrk = 17676,
  VPMOVSWBZrr = 17677,
  VPMOVSWBZrrk = 17678,
  VPMOVSWBZrrkz = 17679,
  VPMOVSXBDYrm = 17680,
  VPMOVSXBDYrr = 17681,
  VPMOVSXBDZ128rm = 17682,
  VPMOVSXBDZ128rmk = 17683,
  VPMOVSXBDZ128rmkz = 17684,
  VPMOVSXBDZ128rr = 17685,
  VPMOVSXBDZ128rrk = 17686,
  VPMOVSXBDZ128rrkz = 17687,
  VPMOVSXBDZ256rm = 17688,
  VPMOVSXBDZ256rmk = 17689,
  VPMOVSXBDZ256rmkz = 17690,
  VPMOVSXBDZ256rr = 17691,
  VPMOVSXBDZ256rrk = 17692,
  VPMOVSXBDZ256rrkz = 17693,
  VPMOVSXBDZrm = 17694,
  VPMOVSXBDZrmk = 17695,
  VPMOVSXBDZrmkz = 17696,
  VPMOVSXBDZrr = 17697,
  VPMOVSXBDZrrk = 17698,
  VPMOVSXBDZrrkz = 17699,
  VPMOVSXBDrm = 17700,
  VPMOVSXBDrr = 17701,
  VPMOVSXBQYrm = 17702,
  VPMOVSXBQYrr = 17703,
  VPMOVSXBQZ128rm = 17704,
  VPMOVSXBQZ128rmk = 17705,
  VPMOVSXBQZ128rmkz = 17706,
  VPMOVSXBQZ128rr = 17707,
  VPMOVSXBQZ128rrk = 17708,
  VPMOVSXBQZ128rrkz = 17709,
  VPMOVSXBQZ256rm = 17710,
  VPMOVSXBQZ256rmk = 17711,
  VPMOVSXBQZ256rmkz = 17712,
  VPMOVSXBQZ256rr = 17713,
  VPMOVSXBQZ256rrk = 17714,
  VPMOVSXBQZ256rrkz = 17715,
  VPMOVSXBQZrm = 17716,
  VPMOVSXBQZrmk = 17717,
  VPMOVSXBQZrmkz = 17718,
  VPMOVSXBQZrr = 17719,
  VPMOVSXBQZrrk = 17720,
  VPMOVSXBQZrrkz = 17721,
  VPMOVSXBQrm = 17722,
  VPMOVSXBQrr = 17723,
  VPMOVSXBWYrm = 17724,
  VPMOVSXBWYrr = 17725,
  VPMOVSXBWZ128rm = 17726,
  VPMOVSXBWZ128rmk = 17727,
  VPMOVSXBWZ128rmkz = 17728,
  VPMOVSXBWZ128rr = 17729,
  VPMOVSXBWZ128rrk = 17730,
  VPMOVSXBWZ128rrkz = 17731,
  VPMOVSXBWZ256rm = 17732,
  VPMOVSXBWZ256rmk = 17733,
  VPMOVSXBWZ256rmkz = 17734,
  VPMOVSXBWZ256rr = 17735,
  VPMOVSXBWZ256rrk = 17736,
  VPMOVSXBWZ256rrkz = 17737,
  VPMOVSXBWZrm = 17738,
  VPMOVSXBWZrmk = 17739,
  VPMOVSXBWZrmkz = 17740,
  VPMOVSXBWZrr = 17741,
  VPMOVSXBWZrrk = 17742,
  VPMOVSXBWZrrkz = 17743,
  VPMOVSXBWrm = 17744,
  VPMOVSXBWrr = 17745,
  VPMOVSXDQYrm = 17746,
  VPMOVSXDQYrr = 17747,
  VPMOVSXDQZ128rm = 17748,
  VPMOVSXDQZ128rmk = 17749,
  VPMOVSXDQZ128rmkz = 17750,
  VPMOVSXDQZ128rr = 17751,
  VPMOVSXDQZ128rrk = 17752,
  VPMOVSXDQZ128rrkz = 17753,
  VPMOVSXDQZ256rm = 17754,
  VPMOVSXDQZ256rmk = 17755,
  VPMOVSXDQZ256rmkz = 17756,
  VPMOVSXDQZ256rr = 17757,
  VPMOVSXDQZ256rrk = 17758,
  VPMOVSXDQZ256rrkz = 17759,
  VPMOVSXDQZrm = 17760,
  VPMOVSXDQZrmk = 17761,
  VPMOVSXDQZrmkz = 17762,
  VPMOVSXDQZrr = 17763,
  VPMOVSXDQZrrk = 17764,
  VPMOVSXDQZrrkz = 17765,
  VPMOVSXDQrm = 17766,
  VPMOVSXDQrr = 17767,
  VPMOVSXWDYrm = 17768,
  VPMOVSXWDYrr = 17769,
  VPMOVSXWDZ128rm = 17770,
  VPMOVSXWDZ128rmk = 17771,
  VPMOVSXWDZ128rmkz = 17772,
  VPMOVSXWDZ128rr = 17773,
  VPMOVSXWDZ128rrk = 17774,
  VPMOVSXWDZ128rrkz = 17775,
  VPMOVSXWDZ256rm = 17776,
  VPMOVSXWDZ256rmk = 17777,
  VPMOVSXWDZ256rmkz = 17778,
  VPMOVSXWDZ256rr = 17779,
  VPMOVSXWDZ256rrk = 17780,
  VPMOVSXWDZ256rrkz = 17781,
  VPMOVSXWDZrm = 17782,
  VPMOVSXWDZrmk = 17783,
  VPMOVSXWDZrmkz = 17784,
  VPMOVSXWDZrr = 17785,
  VPMOVSXWDZrrk = 17786,
  VPMOVSXWDZrrkz = 17787,
  VPMOVSXWDrm = 17788,
  VPMOVSXWDrr = 17789,
  VPMOVSXWQYrm = 17790,
  VPMOVSXWQYrr = 17791,
  VPMOVSXWQZ128rm = 17792,
  VPMOVSXWQZ128rmk = 17793,
  VPMOVSXWQZ128rmkz = 17794,
  VPMOVSXWQZ128rr = 17795,
  VPMOVSXWQZ128rrk = 17796,
  VPMOVSXWQZ128rrkz = 17797,
  VPMOVSXWQZ256rm = 17798,
  VPMOVSXWQZ256rmk = 17799,
  VPMOVSXWQZ256rmkz = 17800,
  VPMOVSXWQZ256rr = 17801,
  VPMOVSXWQZ256rrk = 17802,
  VPMOVSXWQZ256rrkz = 17803,
  VPMOVSXWQZrm = 17804,
  VPMOVSXWQZrmk = 17805,
  VPMOVSXWQZrmkz = 17806,
  VPMOVSXWQZrr = 17807,
  VPMOVSXWQZrrk = 17808,
  VPMOVSXWQZrrkz = 17809,
  VPMOVSXWQrm = 17810,
  VPMOVSXWQrr = 17811,
  VPMOVUSDBZ128mr = 17812,
  VPMOVUSDBZ128mrk = 17813,
  VPMOVUSDBZ128rr = 17814,
  VPMOVUSDBZ128rrk = 17815,
  VPMOVUSDBZ128rrkz = 17816,
  VPMOVUSDBZ256mr = 17817,
  VPMOVUSDBZ256mrk = 17818,
  VPMOVUSDBZ256rr = 17819,
  VPMOVUSDBZ256rrk = 17820,
  VPMOVUSDBZ256rrkz = 17821,
  VPMOVUSDBZmr = 17822,
  VPMOVUSDBZmrk = 17823,
  VPMOVUSDBZrr = 17824,
  VPMOVUSDBZrrk = 17825,
  VPMOVUSDBZrrkz = 17826,
  VPMOVUSDWZ128mr = 17827,
  VPMOVUSDWZ128mrk = 17828,
  VPMOVUSDWZ128rr = 17829,
  VPMOVUSDWZ128rrk = 17830,
  VPMOVUSDWZ128rrkz = 17831,
  VPMOVUSDWZ256mr = 17832,
  VPMOVUSDWZ256mrk = 17833,
  VPMOVUSDWZ256rr = 17834,
  VPMOVUSDWZ256rrk = 17835,
  VPMOVUSDWZ256rrkz = 17836,
  VPMOVUSDWZmr = 17837,
  VPMOVUSDWZmrk = 17838,
  VPMOVUSDWZrr = 17839,
  VPMOVUSDWZrrk = 17840,
  VPMOVUSDWZrrkz = 17841,
  VPMOVUSQBZ128mr = 17842,
  VPMOVUSQBZ128mrk = 17843,
  VPMOVUSQBZ128rr = 17844,
  VPMOVUSQBZ128rrk = 17845,
  VPMOVUSQBZ128rrkz = 17846,
  VPMOVUSQBZ256mr = 17847,
  VPMOVUSQBZ256mrk = 17848,
  VPMOVUSQBZ256rr = 17849,
  VPMOVUSQBZ256rrk = 17850,
  VPMOVUSQBZ256rrkz = 17851,
  VPMOVUSQBZmr = 17852,
  VPMOVUSQBZmrk = 17853,
  VPMOVUSQBZrr = 17854,
  VPMOVUSQBZrrk = 17855,
  VPMOVUSQBZrrkz = 17856,
  VPMOVUSQDZ128mr = 17857,
  VPMOVUSQDZ128mrk = 17858,
  VPMOVUSQDZ128rr = 17859,
  VPMOVUSQDZ128rrk = 17860,
  VPMOVUSQDZ128rrkz = 17861,
  VPMOVUSQDZ256mr = 17862,
  VPMOVUSQDZ256mrk = 17863,
  VPMOVUSQDZ256rr = 17864,
  VPMOVUSQDZ256rrk = 17865,
  VPMOVUSQDZ256rrkz = 17866,
  VPMOVUSQDZmr = 17867,
  VPMOVUSQDZmrk = 17868,
  VPMOVUSQDZrr = 17869,
  VPMOVUSQDZrrk = 17870,
  VPMOVUSQDZrrkz = 17871,
  VPMOVUSQWZ128mr = 17872,
  VPMOVUSQWZ128mrk = 17873,
  VPMOVUSQWZ128rr = 17874,
  VPMOVUSQWZ128rrk = 17875,
  VPMOVUSQWZ128rrkz = 17876,
  VPMOVUSQWZ256mr = 17877,
  VPMOVUSQWZ256mrk = 17878,
  VPMOVUSQWZ256rr = 17879,
  VPMOVUSQWZ256rrk = 17880,
  VPMOVUSQWZ256rrkz = 17881,
  VPMOVUSQWZmr = 17882,
  VPMOVUSQWZmrk = 17883,
  VPMOVUSQWZrr = 17884,
  VPMOVUSQWZrrk = 17885,
  VPMOVUSQWZrrkz = 17886,
  VPMOVUSWBZ128mr = 17887,
  VPMOVUSWBZ128mrk = 17888,
  VPMOVUSWBZ128rr = 17889,
  VPMOVUSWBZ128rrk = 17890,
  VPMOVUSWBZ128rrkz = 17891,
  VPMOVUSWBZ256mr = 17892,
  VPMOVUSWBZ256mrk = 17893,
  VPMOVUSWBZ256rr = 17894,
  VPMOVUSWBZ256rrk = 17895,
  VPMOVUSWBZ256rrkz = 17896,
  VPMOVUSWBZmr = 17897,
  VPMOVUSWBZmrk = 17898,
  VPMOVUSWBZrr = 17899,
  VPMOVUSWBZrrk = 17900,
  VPMOVUSWBZrrkz = 17901,
  VPMOVW2MZ128kr = 17902,
  VPMOVW2MZ256kr = 17903,
  VPMOVW2MZkr = 17904,
  VPMOVWBZ128mr = 17905,
  VPMOVWBZ128mrk = 17906,
  VPMOVWBZ128rr = 17907,
  VPMOVWBZ128rrk = 17908,
  VPMOVWBZ128rrkz = 17909,
  VPMOVWBZ256mr = 17910,
  VPMOVWBZ256mrk = 17911,
  VPMOVWBZ256rr = 17912,
  VPMOVWBZ256rrk = 17913,
  VPMOVWBZ256rrkz = 17914,
  VPMOVWBZmr = 17915,
  VPMOVWBZmrk = 17916,
  VPMOVWBZrr = 17917,
  VPMOVWBZrrk = 17918,
  VPMOVWBZrrkz = 17919,
  VPMOVZXBDYrm = 17920,
  VPMOVZXBDYrr = 17921,
  VPMOVZXBDZ128rm = 17922,
  VPMOVZXBDZ128rmk = 17923,
  VPMOVZXBDZ128rmkz = 17924,
  VPMOVZXBDZ128rr = 17925,
  VPMOVZXBDZ128rrk = 17926,
  VPMOVZXBDZ128rrkz = 17927,
  VPMOVZXBDZ256rm = 17928,
  VPMOVZXBDZ256rmk = 17929,
  VPMOVZXBDZ256rmkz = 17930,
  VPMOVZXBDZ256rr = 17931,
  VPMOVZXBDZ256rrk = 17932,
  VPMOVZXBDZ256rrkz = 17933,
  VPMOVZXBDZrm = 17934,
  VPMOVZXBDZrmk = 17935,
  VPMOVZXBDZrmkz = 17936,
  VPMOVZXBDZrr = 17937,
  VPMOVZXBDZrrk = 17938,
  VPMOVZXBDZrrkz = 17939,
  VPMOVZXBDrm = 17940,
  VPMOVZXBDrr = 17941,
  VPMOVZXBQYrm = 17942,
  VPMOVZXBQYrr = 17943,
  VPMOVZXBQZ128rm = 17944,
  VPMOVZXBQZ128rmk = 17945,
  VPMOVZXBQZ128rmkz = 17946,
  VPMOVZXBQZ128rr = 17947,
  VPMOVZXBQZ128rrk = 17948,
  VPMOVZXBQZ128rrkz = 17949,
  VPMOVZXBQZ256rm = 17950,
  VPMOVZXBQZ256rmk = 17951,
  VPMOVZXBQZ256rmkz = 17952,
  VPMOVZXBQZ256rr = 17953,
  VPMOVZXBQZ256rrk = 17954,
  VPMOVZXBQZ256rrkz = 17955,
  VPMOVZXBQZrm = 17956,
  VPMOVZXBQZrmk = 17957,
  VPMOVZXBQZrmkz = 17958,
  VPMOVZXBQZrr = 17959,
  VPMOVZXBQZrrk = 17960,
  VPMOVZXBQZrrkz = 17961,
  VPMOVZXBQrm = 17962,
  VPMOVZXBQrr = 17963,
  VPMOVZXBWYrm = 17964,
  VPMOVZXBWYrr = 17965,
  VPMOVZXBWZ128rm = 17966,
  VPMOVZXBWZ128rmk = 17967,
  VPMOVZXBWZ128rmkz = 17968,
  VPMOVZXBWZ128rr = 17969,
  VPMOVZXBWZ128rrk = 17970,
  VPMOVZXBWZ128rrkz = 17971,
  VPMOVZXBWZ256rm = 17972,
  VPMOVZXBWZ256rmk = 17973,
  VPMOVZXBWZ256rmkz = 17974,
  VPMOVZXBWZ256rr = 17975,
  VPMOVZXBWZ256rrk = 17976,
  VPMOVZXBWZ256rrkz = 17977,
  VPMOVZXBWZrm = 17978,
  VPMOVZXBWZrmk = 17979,
  VPMOVZXBWZrmkz = 17980,
  VPMOVZXBWZrr = 17981,
  VPMOVZXBWZrrk = 17982,
  VPMOVZXBWZrrkz = 17983,
  VPMOVZXBWrm = 17984,
  VPMOVZXBWrr = 17985,
  VPMOVZXDQYrm = 17986,
  VPMOVZXDQYrr = 17987,
  VPMOVZXDQZ128rm = 17988,
  VPMOVZXDQZ128rmk = 17989,
  VPMOVZXDQZ128rmkz = 17990,
  VPMOVZXDQZ128rr = 17991,
  VPMOVZXDQZ128rrk = 17992,
  VPMOVZXDQZ128rrkz = 17993,
  VPMOVZXDQZ256rm = 17994,
  VPMOVZXDQZ256rmk = 17995,
  VPMOVZXDQZ256rmkz = 17996,
  VPMOVZXDQZ256rr = 17997,
  VPMOVZXDQZ256rrk = 17998,
  VPMOVZXDQZ256rrkz = 17999,
  VPMOVZXDQZrm = 18000,
  VPMOVZXDQZrmk = 18001,
  VPMOVZXDQZrmkz = 18002,
  VPMOVZXDQZrr = 18003,
  VPMOVZXDQZrrk = 18004,
  VPMOVZXDQZrrkz = 18005,
  VPMOVZXDQrm = 18006,
  VPMOVZXDQrr = 18007,
  VPMOVZXWDYrm = 18008,
  VPMOVZXWDYrr = 18009,
  VPMOVZXWDZ128rm = 18010,
  VPMOVZXWDZ128rmk = 18011,
  VPMOVZXWDZ128rmkz = 18012,
  VPMOVZXWDZ128rr = 18013,
  VPMOVZXWDZ128rrk = 18014,
  VPMOVZXWDZ128rrkz = 18015,
  VPMOVZXWDZ256rm = 18016,
  VPMOVZXWDZ256rmk = 18017,
  VPMOVZXWDZ256rmkz = 18018,
  VPMOVZXWDZ256rr = 18019,
  VPMOVZXWDZ256rrk = 18020,
  VPMOVZXWDZ256rrkz = 18021,
  VPMOVZXWDZrm = 18022,
  VPMOVZXWDZrmk = 18023,
  VPMOVZXWDZrmkz = 18024,
  VPMOVZXWDZrr = 18025,
  VPMOVZXWDZrrk = 18026,
  VPMOVZXWDZrrkz = 18027,
  VPMOVZXWDrm = 18028,
  VPMOVZXWDrr = 18029,
  VPMOVZXWQYrm = 18030,
  VPMOVZXWQYrr = 18031,
  VPMOVZXWQZ128rm = 18032,
  VPMOVZXWQZ128rmk = 18033,
  VPMOVZXWQZ128rmkz = 18034,
  VPMOVZXWQZ128rr = 18035,
  VPMOVZXWQZ128rrk = 18036,
  VPMOVZXWQZ128rrkz = 18037,
  VPMOVZXWQZ256rm = 18038,
  VPMOVZXWQZ256rmk = 18039,
  VPMOVZXWQZ256rmkz = 18040,
  VPMOVZXWQZ256rr = 18041,
  VPMOVZXWQZ256rrk = 18042,
  VPMOVZXWQZ256rrkz = 18043,
  VPMOVZXWQZrm = 18044,
  VPMOVZXWQZrmk = 18045,
  VPMOVZXWQZrmkz = 18046,
  VPMOVZXWQZrr = 18047,
  VPMOVZXWQZrrk = 18048,
  VPMOVZXWQZrrkz = 18049,
  VPMOVZXWQrm = 18050,
  VPMOVZXWQrr = 18051,
  VPMULDQYrm = 18052,
  VPMULDQYrr = 18053,
  VPMULDQZ128rm = 18054,
  VPMULDQZ128rmb = 18055,
  VPMULDQZ128rmbk = 18056,
  VPMULDQZ128rmbkz = 18057,
  VPMULDQZ128rmk = 18058,
  VPMULDQZ128rmkz = 18059,
  VPMULDQZ128rr = 18060,
  VPMULDQZ128rrk = 18061,
  VPMULDQZ128rrkz = 18062,
  VPMULDQZ256rm = 18063,
  VPMULDQZ256rmb = 18064,
  VPMULDQZ256rmbk = 18065,
  VPMULDQZ256rmbkz = 18066,
  VPMULDQZ256rmk = 18067,
  VPMULDQZ256rmkz = 18068,
  VPMULDQZ256rr = 18069,
  VPMULDQZ256rrk = 18070,
  VPMULDQZ256rrkz = 18071,
  VPMULDQZrm = 18072,
  VPMULDQZrmb = 18073,
  VPMULDQZrmbk = 18074,
  VPMULDQZrmbkz = 18075,
  VPMULDQZrmk = 18076,
  VPMULDQZrmkz = 18077,
  VPMULDQZrr = 18078,
  VPMULDQZrrk = 18079,
  VPMULDQZrrkz = 18080,
  VPMULDQrm = 18081,
  VPMULDQrr = 18082,
  VPMULHRSWYrm = 18083,
  VPMULHRSWYrr = 18084,
  VPMULHRSWZ128rm = 18085,
  VPMULHRSWZ128rmk = 18086,
  VPMULHRSWZ128rmkz = 18087,
  VPMULHRSWZ128rr = 18088,
  VPMULHRSWZ128rrk = 18089,
  VPMULHRSWZ128rrkz = 18090,
  VPMULHRSWZ256rm = 18091,
  VPMULHRSWZ256rmk = 18092,
  VPMULHRSWZ256rmkz = 18093,
  VPMULHRSWZ256rr = 18094,
  VPMULHRSWZ256rrk = 18095,
  VPMULHRSWZ256rrkz = 18096,
  VPMULHRSWZrm = 18097,
  VPMULHRSWZrmk = 18098,
  VPMULHRSWZrmkz = 18099,
  VPMULHRSWZrr = 18100,
  VPMULHRSWZrrk = 18101,
  VPMULHRSWZrrkz = 18102,
  VPMULHRSWrm = 18103,
  VPMULHRSWrr = 18104,
  VPMULHUWYrm = 18105,
  VPMULHUWYrr = 18106,
  VPMULHUWZ128rm = 18107,
  VPMULHUWZ128rmk = 18108,
  VPMULHUWZ128rmkz = 18109,
  VPMULHUWZ128rr = 18110,
  VPMULHUWZ128rrk = 18111,
  VPMULHUWZ128rrkz = 18112,
  VPMULHUWZ256rm = 18113,
  VPMULHUWZ256rmk = 18114,
  VPMULHUWZ256rmkz = 18115,
  VPMULHUWZ256rr = 18116,
  VPMULHUWZ256rrk = 18117,
  VPMULHUWZ256rrkz = 18118,
  VPMULHUWZrm = 18119,
  VPMULHUWZrmk = 18120,
  VPMULHUWZrmkz = 18121,
  VPMULHUWZrr = 18122,
  VPMULHUWZrrk = 18123,
  VPMULHUWZrrkz = 18124,
  VPMULHUWrm = 18125,
  VPMULHUWrr = 18126,
  VPMULHWYrm = 18127,
  VPMULHWYrr = 18128,
  VPMULHWZ128rm = 18129,
  VPMULHWZ128rmk = 18130,
  VPMULHWZ128rmkz = 18131,
  VPMULHWZ128rr = 18132,
  VPMULHWZ128rrk = 18133,
  VPMULHWZ128rrkz = 18134,
  VPMULHWZ256rm = 18135,
  VPMULHWZ256rmk = 18136,
  VPMULHWZ256rmkz = 18137,
  VPMULHWZ256rr = 18138,
  VPMULHWZ256rrk = 18139,
  VPMULHWZ256rrkz = 18140,
  VPMULHWZrm = 18141,
  VPMULHWZrmk = 18142,
  VPMULHWZrmkz = 18143,
  VPMULHWZrr = 18144,
  VPMULHWZrrk = 18145,
  VPMULHWZrrkz = 18146,
  VPMULHWrm = 18147,
  VPMULHWrr = 18148,
  VPMULLDYrm = 18149,
  VPMULLDYrr = 18150,
  VPMULLDZ128rm = 18151,
  VPMULLDZ128rmb = 18152,
  VPMULLDZ128rmbk = 18153,
  VPMULLDZ128rmbkz = 18154,
  VPMULLDZ128rmk = 18155,
  VPMULLDZ128rmkz = 18156,
  VPMULLDZ128rr = 18157,
  VPMULLDZ128rrk = 18158,
  VPMULLDZ128rrkz = 18159,
  VPMULLDZ256rm = 18160,
  VPMULLDZ256rmb = 18161,
  VPMULLDZ256rmbk = 18162,
  VPMULLDZ256rmbkz = 18163,
  VPMULLDZ256rmk = 18164,
  VPMULLDZ256rmkz = 18165,
  VPMULLDZ256rr = 18166,
  VPMULLDZ256rrk = 18167,
  VPMULLDZ256rrkz = 18168,
  VPMULLDZrm = 18169,
  VPMULLDZrmb = 18170,
  VPMULLDZrmbk = 18171,
  VPMULLDZrmbkz = 18172,
  VPMULLDZrmk = 18173,
  VPMULLDZrmkz = 18174,
  VPMULLDZrr = 18175,
  VPMULLDZrrk = 18176,
  VPMULLDZrrkz = 18177,
  VPMULLDrm = 18178,
  VPMULLDrr = 18179,
  VPMULLQZ128rm = 18180,
  VPMULLQZ128rmb = 18181,
  VPMULLQZ128rmbk = 18182,
  VPMULLQZ128rmbkz = 18183,
  VPMULLQZ128rmk = 18184,
  VPMULLQZ128rmkz = 18185,
  VPMULLQZ128rr = 18186,
  VPMULLQZ128rrk = 18187,
  VPMULLQZ128rrkz = 18188,
  VPMULLQZ256rm = 18189,
  VPMULLQZ256rmb = 18190,
  VPMULLQZ256rmbk = 18191,
  VPMULLQZ256rmbkz = 18192,
  VPMULLQZ256rmk = 18193,
  VPMULLQZ256rmkz = 18194,
  VPMULLQZ256rr = 18195,
  VPMULLQZ256rrk = 18196,
  VPMULLQZ256rrkz = 18197,
  VPMULLQZrm = 18198,
  VPMULLQZrmb = 18199,
  VPMULLQZrmbk = 18200,
  VPMULLQZrmbkz = 18201,
  VPMULLQZrmk = 18202,
  VPMULLQZrmkz = 18203,
  VPMULLQZrr = 18204,
  VPMULLQZrrk = 18205,
  VPMULLQZrrkz = 18206,
  VPMULLWYrm = 18207,
  VPMULLWYrr = 18208,
  VPMULLWZ128rm = 18209,
  VPMULLWZ128rmk = 18210,
  VPMULLWZ128rmkz = 18211,
  VPMULLWZ128rr = 18212,
  VPMULLWZ128rrk = 18213,
  VPMULLWZ128rrkz = 18214,
  VPMULLWZ256rm = 18215,
  VPMULLWZ256rmk = 18216,
  VPMULLWZ256rmkz = 18217,
  VPMULLWZ256rr = 18218,
  VPMULLWZ256rrk = 18219,
  VPMULLWZ256rrkz = 18220,
  VPMULLWZrm = 18221,
  VPMULLWZrmk = 18222,
  VPMULLWZrmkz = 18223,
  VPMULLWZrr = 18224,
  VPMULLWZrrk = 18225,
  VPMULLWZrrkz = 18226,
  VPMULLWrm = 18227,
  VPMULLWrr = 18228,
  VPMULTISHIFTQBZ128rm = 18229,
  VPMULTISHIFTQBZ128rmb = 18230,
  VPMULTISHIFTQBZ128rmbk = 18231,
  VPMULTISHIFTQBZ128rmbkz = 18232,
  VPMULTISHIFTQBZ128rmk = 18233,
  VPMULTISHIFTQBZ128rmkz = 18234,
  VPMULTISHIFTQBZ128rr = 18235,
  VPMULTISHIFTQBZ128rrk = 18236,
  VPMULTISHIFTQBZ128rrkz = 18237,
  VPMULTISHIFTQBZ256rm = 18238,
  VPMULTISHIFTQBZ256rmb = 18239,
  VPMULTISHIFTQBZ256rmbk = 18240,
  VPMULTISHIFTQBZ256rmbkz = 18241,
  VPMULTISHIFTQBZ256rmk = 18242,
  VPMULTISHIFTQBZ256rmkz = 18243,
  VPMULTISHIFTQBZ256rr = 18244,
  VPMULTISHIFTQBZ256rrk = 18245,
  VPMULTISHIFTQBZ256rrkz = 18246,
  VPMULTISHIFTQBZrm = 18247,
  VPMULTISHIFTQBZrmb = 18248,
  VPMULTISHIFTQBZrmbk = 18249,
  VPMULTISHIFTQBZrmbkz = 18250,
  VPMULTISHIFTQBZrmk = 18251,
  VPMULTISHIFTQBZrmkz = 18252,
  VPMULTISHIFTQBZrr = 18253,
  VPMULTISHIFTQBZrrk = 18254,
  VPMULTISHIFTQBZrrkz = 18255,
  VPMULUDQYrm = 18256,
  VPMULUDQYrr = 18257,
  VPMULUDQZ128rm = 18258,
  VPMULUDQZ128rmb = 18259,
  VPMULUDQZ128rmbk = 18260,
  VPMULUDQZ128rmbkz = 18261,
  VPMULUDQZ128rmk = 18262,
  VPMULUDQZ128rmkz = 18263,
  VPMULUDQZ128rr = 18264,
  VPMULUDQZ128rrk = 18265,
  VPMULUDQZ128rrkz = 18266,
  VPMULUDQZ256rm = 18267,
  VPMULUDQZ256rmb = 18268,
  VPMULUDQZ256rmbk = 18269,
  VPMULUDQZ256rmbkz = 18270,
  VPMULUDQZ256rmk = 18271,
  VPMULUDQZ256rmkz = 18272,
  VPMULUDQZ256rr = 18273,
  VPMULUDQZ256rrk = 18274,
  VPMULUDQZ256rrkz = 18275,
  VPMULUDQZrm = 18276,
  VPMULUDQZrmb = 18277,
  VPMULUDQZrmbk = 18278,
  VPMULUDQZrmbkz = 18279,
  VPMULUDQZrmk = 18280,
  VPMULUDQZrmkz = 18281,
  VPMULUDQZrr = 18282,
  VPMULUDQZrrk = 18283,
  VPMULUDQZrrkz = 18284,
  VPMULUDQrm = 18285,
  VPMULUDQrr = 18286,
  VPOPCNTBZ128rm = 18287,
  VPOPCNTBZ128rmk = 18288,
  VPOPCNTBZ128rmkz = 18289,
  VPOPCNTBZ128rr = 18290,
  VPOPCNTBZ128rrk = 18291,
  VPOPCNTBZ128rrkz = 18292,
  VPOPCNTBZ256rm = 18293,
  VPOPCNTBZ256rmk = 18294,
  VPOPCNTBZ256rmkz = 18295,
  VPOPCNTBZ256rr = 18296,
  VPOPCNTBZ256rrk = 18297,
  VPOPCNTBZ256rrkz = 18298,
  VPOPCNTBZrm = 18299,
  VPOPCNTBZrmk = 18300,
  VPOPCNTBZrmkz = 18301,
  VPOPCNTBZrr = 18302,
  VPOPCNTBZrrk = 18303,
  VPOPCNTBZrrkz = 18304,
  VPOPCNTDZ128rm = 18305,
  VPOPCNTDZ128rmb = 18306,
  VPOPCNTDZ128rmbk = 18307,
  VPOPCNTDZ128rmbkz = 18308,
  VPOPCNTDZ128rmk = 18309,
  VPOPCNTDZ128rmkz = 18310,
  VPOPCNTDZ128rr = 18311,
  VPOPCNTDZ128rrk = 18312,
  VPOPCNTDZ128rrkz = 18313,
  VPOPCNTDZ256rm = 18314,
  VPOPCNTDZ256rmb = 18315,
  VPOPCNTDZ256rmbk = 18316,
  VPOPCNTDZ256rmbkz = 18317,
  VPOPCNTDZ256rmk = 18318,
  VPOPCNTDZ256rmkz = 18319,
  VPOPCNTDZ256rr = 18320,
  VPOPCNTDZ256rrk = 18321,
  VPOPCNTDZ256rrkz = 18322,
  VPOPCNTDZrm = 18323,
  VPOPCNTDZrmb = 18324,
  VPOPCNTDZrmbk = 18325,
  VPOPCNTDZrmbkz = 18326,
  VPOPCNTDZrmk = 18327,
  VPOPCNTDZrmkz = 18328,
  VPOPCNTDZrr = 18329,
  VPOPCNTDZrrk = 18330,
  VPOPCNTDZrrkz = 18331,
  VPOPCNTQZ128rm = 18332,
  VPOPCNTQZ128rmb = 18333,
  VPOPCNTQZ128rmbk = 18334,
  VPOPCNTQZ128rmbkz = 18335,
  VPOPCNTQZ128rmk = 18336,
  VPOPCNTQZ128rmkz = 18337,
  VPOPCNTQZ128rr = 18338,
  VPOPCNTQZ128rrk = 18339,
  VPOPCNTQZ128rrkz = 18340,
  VPOPCNTQZ256rm = 18341,
  VPOPCNTQZ256rmb = 18342,
  VPOPCNTQZ256rmbk = 18343,
  VPOPCNTQZ256rmbkz = 18344,
  VPOPCNTQZ256rmk = 18345,
  VPOPCNTQZ256rmkz = 18346,
  VPOPCNTQZ256rr = 18347,
  VPOPCNTQZ256rrk = 18348,
  VPOPCNTQZ256rrkz = 18349,
  VPOPCNTQZrm = 18350,
  VPOPCNTQZrmb = 18351,
  VPOPCNTQZrmbk = 18352,
  VPOPCNTQZrmbkz = 18353,
  VPOPCNTQZrmk = 18354,
  VPOPCNTQZrmkz = 18355,
  VPOPCNTQZrr = 18356,
  VPOPCNTQZrrk = 18357,
  VPOPCNTQZrrkz = 18358,
  VPOPCNTWZ128rm = 18359,
  VPOPCNTWZ128rmk = 18360,
  VPOPCNTWZ128rmkz = 18361,
  VPOPCNTWZ128rr = 18362,
  VPOPCNTWZ128rrk = 18363,
  VPOPCNTWZ128rrkz = 18364,
  VPOPCNTWZ256rm = 18365,
  VPOPCNTWZ256rmk = 18366,
  VPOPCNTWZ256rmkz = 18367,
  VPOPCNTWZ256rr = 18368,
  VPOPCNTWZ256rrk = 18369,
  VPOPCNTWZ256rrkz = 18370,
  VPOPCNTWZrm = 18371,
  VPOPCNTWZrmk = 18372,
  VPOPCNTWZrmkz = 18373,
  VPOPCNTWZrr = 18374,
  VPOPCNTWZrrk = 18375,
  VPOPCNTWZrrkz = 18376,
  VPORDZ128rm = 18377,
  VPORDZ128rmb = 18378,
  VPORDZ128rmbk = 18379,
  VPORDZ128rmbkz = 18380,
  VPORDZ128rmk = 18381,
  VPORDZ128rmkz = 18382,
  VPORDZ128rr = 18383,
  VPORDZ128rrk = 18384,
  VPORDZ128rrkz = 18385,
  VPORDZ256rm = 18386,
  VPORDZ256rmb = 18387,
  VPORDZ256rmbk = 18388,
  VPORDZ256rmbkz = 18389,
  VPORDZ256rmk = 18390,
  VPORDZ256rmkz = 18391,
  VPORDZ256rr = 18392,
  VPORDZ256rrk = 18393,
  VPORDZ256rrkz = 18394,
  VPORDZrm = 18395,
  VPORDZrmb = 18396,
  VPORDZrmbk = 18397,
  VPORDZrmbkz = 18398,
  VPORDZrmk = 18399,
  VPORDZrmkz = 18400,
  VPORDZrr = 18401,
  VPORDZrrk = 18402,
  VPORDZrrkz = 18403,
  VPORQZ128rm = 18404,
  VPORQZ128rmb = 18405,
  VPORQZ128rmbk = 18406,
  VPORQZ128rmbkz = 18407,
  VPORQZ128rmk = 18408,
  VPORQZ128rmkz = 18409,
  VPORQZ128rr = 18410,
  VPORQZ128rrk = 18411,
  VPORQZ128rrkz = 18412,
  VPORQZ256rm = 18413,
  VPORQZ256rmb = 18414,
  VPORQZ256rmbk = 18415,
  VPORQZ256rmbkz = 18416,
  VPORQZ256rmk = 18417,
  VPORQZ256rmkz = 18418,
  VPORQZ256rr = 18419,
  VPORQZ256rrk = 18420,
  VPORQZ256rrkz = 18421,
  VPORQZrm = 18422,
  VPORQZrmb = 18423,
  VPORQZrmbk = 18424,
  VPORQZrmbkz = 18425,
  VPORQZrmk = 18426,
  VPORQZrmkz = 18427,
  VPORQZrr = 18428,
  VPORQZrrk = 18429,
  VPORQZrrkz = 18430,
  VPORYrm = 18431,
  VPORYrr = 18432,
  VPORrm = 18433,
  VPORrr = 18434,
  VPPERMrmr = 18435,
  VPPERMrrm = 18436,
  VPPERMrrr = 18437,
  VPPERMrrr_REV = 18438,
  VPROLDZ128mbi = 18439,
  VPROLDZ128mbik = 18440,
  VPROLDZ128mbikz = 18441,
  VPROLDZ128mi = 18442,
  VPROLDZ128mik = 18443,
  VPROLDZ128mikz = 18444,
  VPROLDZ128ri = 18445,
  VPROLDZ128rik = 18446,
  VPROLDZ128rikz = 18447,
  VPROLDZ256mbi = 18448,
  VPROLDZ256mbik = 18449,
  VPROLDZ256mbikz = 18450,
  VPROLDZ256mi = 18451,
  VPROLDZ256mik = 18452,
  VPROLDZ256mikz = 18453,
  VPROLDZ256ri = 18454,
  VPROLDZ256rik = 18455,
  VPROLDZ256rikz = 18456,
  VPROLDZmbi = 18457,
  VPROLDZmbik = 18458,
  VPROLDZmbikz = 18459,
  VPROLDZmi = 18460,
  VPROLDZmik = 18461,
  VPROLDZmikz = 18462,
  VPROLDZri = 18463,
  VPROLDZrik = 18464,
  VPROLDZrikz = 18465,
  VPROLQZ128mbi = 18466,
  VPROLQZ128mbik = 18467,
  VPROLQZ128mbikz = 18468,
  VPROLQZ128mi = 18469,
  VPROLQZ128mik = 18470,
  VPROLQZ128mikz = 18471,
  VPROLQZ128ri = 18472,
  VPROLQZ128rik = 18473,
  VPROLQZ128rikz = 18474,
  VPROLQZ256mbi = 18475,
  VPROLQZ256mbik = 18476,
  VPROLQZ256mbikz = 18477,
  VPROLQZ256mi = 18478,
  VPROLQZ256mik = 18479,
  VPROLQZ256mikz = 18480,
  VPROLQZ256ri = 18481,
  VPROLQZ256rik = 18482,
  VPROLQZ256rikz = 18483,
  VPROLQZmbi = 18484,
  VPROLQZmbik = 18485,
  VPROLQZmbikz = 18486,
  VPROLQZmi = 18487,
  VPROLQZmik = 18488,
  VPROLQZmikz = 18489,
  VPROLQZri = 18490,
  VPROLQZrik = 18491,
  VPROLQZrikz = 18492,
  VPROLVDZ128rm = 18493,
  VPROLVDZ128rmb = 18494,
  VPROLVDZ128rmbk = 18495,
  VPROLVDZ128rmbkz = 18496,
  VPROLVDZ128rmk = 18497,
  VPROLVDZ128rmkz = 18498,
  VPROLVDZ128rr = 18499,
  VPROLVDZ128rrk = 18500,
  VPROLVDZ128rrkz = 18501,
  VPROLVDZ256rm = 18502,
  VPROLVDZ256rmb = 18503,
  VPROLVDZ256rmbk = 18504,
  VPROLVDZ256rmbkz = 18505,
  VPROLVDZ256rmk = 18506,
  VPROLVDZ256rmkz = 18507,
  VPROLVDZ256rr = 18508,
  VPROLVDZ256rrk = 18509,
  VPROLVDZ256rrkz = 18510,
  VPROLVDZrm = 18511,
  VPROLVDZrmb = 18512,
  VPROLVDZrmbk = 18513,
  VPROLVDZrmbkz = 18514,
  VPROLVDZrmk = 18515,
  VPROLVDZrmkz = 18516,
  VPROLVDZrr = 18517,
  VPROLVDZrrk = 18518,
  VPROLVDZrrkz = 18519,
  VPROLVQZ128rm = 18520,
  VPROLVQZ128rmb = 18521,
  VPROLVQZ128rmbk = 18522,
  VPROLVQZ128rmbkz = 18523,
  VPROLVQZ128rmk = 18524,
  VPROLVQZ128rmkz = 18525,
  VPROLVQZ128rr = 18526,
  VPROLVQZ128rrk = 18527,
  VPROLVQZ128rrkz = 18528,
  VPROLVQZ256rm = 18529,
  VPROLVQZ256rmb = 18530,
  VPROLVQZ256rmbk = 18531,
  VPROLVQZ256rmbkz = 18532,
  VPROLVQZ256rmk = 18533,
  VPROLVQZ256rmkz = 18534,
  VPROLVQZ256rr = 18535,
  VPROLVQZ256rrk = 18536,
  VPROLVQZ256rrkz = 18537,
  VPROLVQZrm = 18538,
  VPROLVQZrmb = 18539,
  VPROLVQZrmbk = 18540,
  VPROLVQZrmbkz = 18541,
  VPROLVQZrmk = 18542,
  VPROLVQZrmkz = 18543,
  VPROLVQZrr = 18544,
  VPROLVQZrrk = 18545,
  VPROLVQZrrkz = 18546,
  VPRORDZ128mbi = 18547,
  VPRORDZ128mbik = 18548,
  VPRORDZ128mbikz = 18549,
  VPRORDZ128mi = 18550,
  VPRORDZ128mik = 18551,
  VPRORDZ128mikz = 18552,
  VPRORDZ128ri = 18553,
  VPRORDZ128rik = 18554,
  VPRORDZ128rikz = 18555,
  VPRORDZ256mbi = 18556,
  VPRORDZ256mbik = 18557,
  VPRORDZ256mbikz = 18558,
  VPRORDZ256mi = 18559,
  VPRORDZ256mik = 18560,
  VPRORDZ256mikz = 18561,
  VPRORDZ256ri = 18562,
  VPRORDZ256rik = 18563,
  VPRORDZ256rikz = 18564,
  VPRORDZmbi = 18565,
  VPRORDZmbik = 18566,
  VPRORDZmbikz = 18567,
  VPRORDZmi = 18568,
  VPRORDZmik = 18569,
  VPRORDZmikz = 18570,
  VPRORDZri = 18571,
  VPRORDZrik = 18572,
  VPRORDZrikz = 18573,
  VPRORQZ128mbi = 18574,
  VPRORQZ128mbik = 18575,
  VPRORQZ128mbikz = 18576,
  VPRORQZ128mi = 18577,
  VPRORQZ128mik = 18578,
  VPRORQZ128mikz = 18579,
  VPRORQZ128ri = 18580,
  VPRORQZ128rik = 18581,
  VPRORQZ128rikz = 18582,
  VPRORQZ256mbi = 18583,
  VPRORQZ256mbik = 18584,
  VPRORQZ256mbikz = 18585,
  VPRORQZ256mi = 18586,
  VPRORQZ256mik = 18587,
  VPRORQZ256mikz = 18588,
  VPRORQZ256ri = 18589,
  VPRORQZ256rik = 18590,
  VPRORQZ256rikz = 18591,
  VPRORQZmbi = 18592,
  VPRORQZmbik = 18593,
  VPRORQZmbikz = 18594,
  VPRORQZmi = 18595,
  VPRORQZmik = 18596,
  VPRORQZmikz = 18597,
  VPRORQZri = 18598,
  VPRORQZrik = 18599,
  VPRORQZrikz = 18600,
  VPRORVDZ128rm = 18601,
  VPRORVDZ128rmb = 18602,
  VPRORVDZ128rmbk = 18603,
  VPRORVDZ128rmbkz = 18604,
  VPRORVDZ128rmk = 18605,
  VPRORVDZ128rmkz = 18606,
  VPRORVDZ128rr = 18607,
  VPRORVDZ128rrk = 18608,
  VPRORVDZ128rrkz = 18609,
  VPRORVDZ256rm = 18610,
  VPRORVDZ256rmb = 18611,
  VPRORVDZ256rmbk = 18612,
  VPRORVDZ256rmbkz = 18613,
  VPRORVDZ256rmk = 18614,
  VPRORVDZ256rmkz = 18615,
  VPRORVDZ256rr = 18616,
  VPRORVDZ256rrk = 18617,
  VPRORVDZ256rrkz = 18618,
  VPRORVDZrm = 18619,
  VPRORVDZrmb = 18620,
  VPRORVDZrmbk = 18621,
  VPRORVDZrmbkz = 18622,
  VPRORVDZrmk = 18623,
  VPRORVDZrmkz = 18624,
  VPRORVDZrr = 18625,
  VPRORVDZrrk = 18626,
  VPRORVDZrrkz = 18627,
  VPRORVQZ128rm = 18628,
  VPRORVQZ128rmb = 18629,
  VPRORVQZ128rmbk = 18630,
  VPRORVQZ128rmbkz = 18631,
  VPRORVQZ128rmk = 18632,
  VPRORVQZ128rmkz = 18633,
  VPRORVQZ128rr = 18634,
  VPRORVQZ128rrk = 18635,
  VPRORVQZ128rrkz = 18636,
  VPRORVQZ256rm = 18637,
  VPRORVQZ256rmb = 18638,
  VPRORVQZ256rmbk = 18639,
  VPRORVQZ256rmbkz = 18640,
  VPRORVQZ256rmk = 18641,
  VPRORVQZ256rmkz = 18642,
  VPRORVQZ256rr = 18643,
  VPRORVQZ256rrk = 18644,
  VPRORVQZ256rrkz = 18645,
  VPRORVQZrm = 18646,
  VPRORVQZrmb = 18647,
  VPRORVQZrmbk = 18648,
  VPRORVQZrmbkz = 18649,
  VPRORVQZrmk = 18650,
  VPRORVQZrmkz = 18651,
  VPRORVQZrr = 18652,
  VPRORVQZrrk = 18653,
  VPRORVQZrrkz = 18654,
  VPROTBmi = 18655,
  VPROTBmr = 18656,
  VPROTBri = 18657,
  VPROTBrm = 18658,
  VPROTBrr = 18659,
  VPROTBrr_REV = 18660,
  VPROTDmi = 18661,
  VPROTDmr = 18662,
  VPROTDri = 18663,
  VPROTDrm = 18664,
  VPROTDrr = 18665,
  VPROTDrr_REV = 18666,
  VPROTQmi = 18667,
  VPROTQmr = 18668,
  VPROTQri = 18669,
  VPROTQrm = 18670,
  VPROTQrr = 18671,
  VPROTQrr_REV = 18672,
  VPROTWmi = 18673,
  VPROTWmr = 18674,
  VPROTWri = 18675,
  VPROTWrm = 18676,
  VPROTWrr = 18677,
  VPROTWrr_REV = 18678,
  VPSADBWYrm = 18679,
  VPSADBWYrr = 18680,
  VPSADBWZ128rm = 18681,
  VPSADBWZ128rr = 18682,
  VPSADBWZ256rm = 18683,
  VPSADBWZ256rr = 18684,
  VPSADBWZrm = 18685,
  VPSADBWZrr = 18686,
  VPSADBWrm = 18687,
  VPSADBWrr = 18688,
  VPSCATTERDDZ128mr = 18689,
  VPSCATTERDDZ256mr = 18690,
  VPSCATTERDDZmr = 18691,
  VPSCATTERDQZ128mr = 18692,
  VPSCATTERDQZ256mr = 18693,
  VPSCATTERDQZmr = 18694,
  VPSCATTERQDZ128mr = 18695,
  VPSCATTERQDZ256mr = 18696,
  VPSCATTERQDZmr = 18697,
  VPSCATTERQQZ128mr = 18698,
  VPSCATTERQQZ256mr = 18699,
  VPSCATTERQQZmr = 18700,
  VPSHABmr = 18701,
  VPSHABrm = 18702,
  VPSHABrr = 18703,
  VPSHABrr_REV = 18704,
  VPSHADmr = 18705,
  VPSHADrm = 18706,
  VPSHADrr = 18707,
  VPSHADrr_REV = 18708,
  VPSHAQmr = 18709,
  VPSHAQrm = 18710,
  VPSHAQrr = 18711,
  VPSHAQrr_REV = 18712,
  VPSHAWmr = 18713,
  VPSHAWrm = 18714,
  VPSHAWrr = 18715,
  VPSHAWrr_REV = 18716,
  VPSHLBmr = 18717,
  VPSHLBrm = 18718,
  VPSHLBrr = 18719,
  VPSHLBrr_REV = 18720,
  VPSHLDDZ128rmbi = 18721,
  VPSHLDDZ128rmbik = 18722,
  VPSHLDDZ128rmbikz = 18723,
  VPSHLDDZ128rmi = 18724,
  VPSHLDDZ128rmik = 18725,
  VPSHLDDZ128rmikz = 18726,
  VPSHLDDZ128rri = 18727,
  VPSHLDDZ128rrik = 18728,
  VPSHLDDZ128rrikz = 18729,
  VPSHLDDZ256rmbi = 18730,
  VPSHLDDZ256rmbik = 18731,
  VPSHLDDZ256rmbikz = 18732,
  VPSHLDDZ256rmi = 18733,
  VPSHLDDZ256rmik = 18734,
  VPSHLDDZ256rmikz = 18735,
  VPSHLDDZ256rri = 18736,
  VPSHLDDZ256rrik = 18737,
  VPSHLDDZ256rrikz = 18738,
  VPSHLDDZrmbi = 18739,
  VPSHLDDZrmbik = 18740,
  VPSHLDDZrmbikz = 18741,
  VPSHLDDZrmi = 18742,
  VPSHLDDZrmik = 18743,
  VPSHLDDZrmikz = 18744,
  VPSHLDDZrri = 18745,
  VPSHLDDZrrik = 18746,
  VPSHLDDZrrikz = 18747,
  VPSHLDQZ128rmbi = 18748,
  VPSHLDQZ128rmbik = 18749,
  VPSHLDQZ128rmbikz = 18750,
  VPSHLDQZ128rmi = 18751,
  VPSHLDQZ128rmik = 18752,
  VPSHLDQZ128rmikz = 18753,
  VPSHLDQZ128rri = 18754,
  VPSHLDQZ128rrik = 18755,
  VPSHLDQZ128rrikz = 18756,
  VPSHLDQZ256rmbi = 18757,
  VPSHLDQZ256rmbik = 18758,
  VPSHLDQZ256rmbikz = 18759,
  VPSHLDQZ256rmi = 18760,
  VPSHLDQZ256rmik = 18761,
  VPSHLDQZ256rmikz = 18762,
  VPSHLDQZ256rri = 18763,
  VPSHLDQZ256rrik = 18764,
  VPSHLDQZ256rrikz = 18765,
  VPSHLDQZrmbi = 18766,
  VPSHLDQZrmbik = 18767,
  VPSHLDQZrmbikz = 18768,
  VPSHLDQZrmi = 18769,
  VPSHLDQZrmik = 18770,
  VPSHLDQZrmikz = 18771,
  VPSHLDQZrri = 18772,
  VPSHLDQZrrik = 18773,
  VPSHLDQZrrikz = 18774,
  VPSHLDVDZ128m = 18775,
  VPSHLDVDZ128mb = 18776,
  VPSHLDVDZ128mbk = 18777,
  VPSHLDVDZ128mbkz = 18778,
  VPSHLDVDZ128mk = 18779,
  VPSHLDVDZ128mkz = 18780,
  VPSHLDVDZ128r = 18781,
  VPSHLDVDZ128rk = 18782,
  VPSHLDVDZ128rkz = 18783,
  VPSHLDVDZ256m = 18784,
  VPSHLDVDZ256mb = 18785,
  VPSHLDVDZ256mbk = 18786,
  VPSHLDVDZ256mbkz = 18787,
  VPSHLDVDZ256mk = 18788,
  VPSHLDVDZ256mkz = 18789,
  VPSHLDVDZ256r = 18790,
  VPSHLDVDZ256rk = 18791,
  VPSHLDVDZ256rkz = 18792,
  VPSHLDVDZm = 18793,
  VPSHLDVDZmb = 18794,
  VPSHLDVDZmbk = 18795,
  VPSHLDVDZmbkz = 18796,
  VPSHLDVDZmk = 18797,
  VPSHLDVDZmkz = 18798,
  VPSHLDVDZr = 18799,
  VPSHLDVDZrk = 18800,
  VPSHLDVDZrkz = 18801,
  VPSHLDVQZ128m = 18802,
  VPSHLDVQZ128mb = 18803,
  VPSHLDVQZ128mbk = 18804,
  VPSHLDVQZ128mbkz = 18805,
  VPSHLDVQZ128mk = 18806,
  VPSHLDVQZ128mkz = 18807,
  VPSHLDVQZ128r = 18808,
  VPSHLDVQZ128rk = 18809,
  VPSHLDVQZ128rkz = 18810,
  VPSHLDVQZ256m = 18811,
  VPSHLDVQZ256mb = 18812,
  VPSHLDVQZ256mbk = 18813,
  VPSHLDVQZ256mbkz = 18814,
  VPSHLDVQZ256mk = 18815,
  VPSHLDVQZ256mkz = 18816,
  VPSHLDVQZ256r = 18817,
  VPSHLDVQZ256rk = 18818,
  VPSHLDVQZ256rkz = 18819,
  VPSHLDVQZm = 18820,
  VPSHLDVQZmb = 18821,
  VPSHLDVQZmbk = 18822,
  VPSHLDVQZmbkz = 18823,
  VPSHLDVQZmk = 18824,
  VPSHLDVQZmkz = 18825,
  VPSHLDVQZr = 18826,
  VPSHLDVQZrk = 18827,
  VPSHLDVQZrkz = 18828,
  VPSHLDVWZ128m = 18829,
  VPSHLDVWZ128mk = 18830,
  VPSHLDVWZ128mkz = 18831,
  VPSHLDVWZ128r = 18832,
  VPSHLDVWZ128rk = 18833,
  VPSHLDVWZ128rkz = 18834,
  VPSHLDVWZ256m = 18835,
  VPSHLDVWZ256mk = 18836,
  VPSHLDVWZ256mkz = 18837,
  VPSHLDVWZ256r = 18838,
  VPSHLDVWZ256rk = 18839,
  VPSHLDVWZ256rkz = 18840,
  VPSHLDVWZm = 18841,
  VPSHLDVWZmk = 18842,
  VPSHLDVWZmkz = 18843,
  VPSHLDVWZr = 18844,
  VPSHLDVWZrk = 18845,
  VPSHLDVWZrkz = 18846,
  VPSHLDWZ128rmi = 18847,
  VPSHLDWZ128rmik = 18848,
  VPSHLDWZ128rmikz = 18849,
  VPSHLDWZ128rri = 18850,
  VPSHLDWZ128rrik = 18851,
  VPSHLDWZ128rrikz = 18852,
  VPSHLDWZ256rmi = 18853,
  VPSHLDWZ256rmik = 18854,
  VPSHLDWZ256rmikz = 18855,
  VPSHLDWZ256rri = 18856,
  VPSHLDWZ256rrik = 18857,
  VPSHLDWZ256rrikz = 18858,
  VPSHLDWZrmi = 18859,
  VPSHLDWZrmik = 18860,
  VPSHLDWZrmikz = 18861,
  VPSHLDWZrri = 18862,
  VPSHLDWZrrik = 18863,
  VPSHLDWZrrikz = 18864,
  VPSHLDmr = 18865,
  VPSHLDrm = 18866,
  VPSHLDrr = 18867,
  VPSHLDrr_REV = 18868,
  VPSHLQmr = 18869,
  VPSHLQrm = 18870,
  VPSHLQrr = 18871,
  VPSHLQrr_REV = 18872,
  VPSHLWmr = 18873,
  VPSHLWrm = 18874,
  VPSHLWrr = 18875,
  VPSHLWrr_REV = 18876,
  VPSHRDDZ128rmbi = 18877,
  VPSHRDDZ128rmbik = 18878,
  VPSHRDDZ128rmbikz = 18879,
  VPSHRDDZ128rmi = 18880,
  VPSHRDDZ128rmik = 18881,
  VPSHRDDZ128rmikz = 18882,
  VPSHRDDZ128rri = 18883,
  VPSHRDDZ128rrik = 18884,
  VPSHRDDZ128rrikz = 18885,
  VPSHRDDZ256rmbi = 18886,
  VPSHRDDZ256rmbik = 18887,
  VPSHRDDZ256rmbikz = 18888,
  VPSHRDDZ256rmi = 18889,
  VPSHRDDZ256rmik = 18890,
  VPSHRDDZ256rmikz = 18891,
  VPSHRDDZ256rri = 18892,
  VPSHRDDZ256rrik = 18893,
  VPSHRDDZ256rrikz = 18894,
  VPSHRDDZrmbi = 18895,
  VPSHRDDZrmbik = 18896,
  VPSHRDDZrmbikz = 18897,
  VPSHRDDZrmi = 18898,
  VPSHRDDZrmik = 18899,
  VPSHRDDZrmikz = 18900,
  VPSHRDDZrri = 18901,
  VPSHRDDZrrik = 18902,
  VPSHRDDZrrikz = 18903,
  VPSHRDQZ128rmbi = 18904,
  VPSHRDQZ128rmbik = 18905,
  VPSHRDQZ128rmbikz = 18906,
  VPSHRDQZ128rmi = 18907,
  VPSHRDQZ128rmik = 18908,
  VPSHRDQZ128rmikz = 18909,
  VPSHRDQZ128rri = 18910,
  VPSHRDQZ128rrik = 18911,
  VPSHRDQZ128rrikz = 18912,
  VPSHRDQZ256rmbi = 18913,
  VPSHRDQZ256rmbik = 18914,
  VPSHRDQZ256rmbikz = 18915,
  VPSHRDQZ256rmi = 18916,
  VPSHRDQZ256rmik = 18917,
  VPSHRDQZ256rmikz = 18918,
  VPSHRDQZ256rri = 18919,
  VPSHRDQZ256rrik = 18920,
  VPSHRDQZ256rrikz = 18921,
  VPSHRDQZrmbi = 18922,
  VPSHRDQZrmbik = 18923,
  VPSHRDQZrmbikz = 18924,
  VPSHRDQZrmi = 18925,
  VPSHRDQZrmik = 18926,
  VPSHRDQZrmikz = 18927,
  VPSHRDQZrri = 18928,
  VPSHRDQZrrik = 18929,
  VPSHRDQZrrikz = 18930,
  VPSHRDVDZ128m = 18931,
  VPSHRDVDZ128mb = 18932,
  VPSHRDVDZ128mbk = 18933,
  VPSHRDVDZ128mbkz = 18934,
  VPSHRDVDZ128mk = 18935,
  VPSHRDVDZ128mkz = 18936,
  VPSHRDVDZ128r = 18937,
  VPSHRDVDZ128rk = 18938,
  VPSHRDVDZ128rkz = 18939,
  VPSHRDVDZ256m = 18940,
  VPSHRDVDZ256mb = 18941,
  VPSHRDVDZ256mbk = 18942,
  VPSHRDVDZ256mbkz = 18943,
  VPSHRDVDZ256mk = 18944,
  VPSHRDVDZ256mkz = 18945,
  VPSHRDVDZ256r = 18946,
  VPSHRDVDZ256rk = 18947,
  VPSHRDVDZ256rkz = 18948,
  VPSHRDVDZm = 18949,
  VPSHRDVDZmb = 18950,
  VPSHRDVDZmbk = 18951,
  VPSHRDVDZmbkz = 18952,
  VPSHRDVDZmk = 18953,
  VPSHRDVDZmkz = 18954,
  VPSHRDVDZr = 18955,
  VPSHRDVDZrk = 18956,
  VPSHRDVDZrkz = 18957,
  VPSHRDVQZ128m = 18958,
  VPSHRDVQZ128mb = 18959,
  VPSHRDVQZ128mbk = 18960,
  VPSHRDVQZ128mbkz = 18961,
  VPSHRDVQZ128mk = 18962,
  VPSHRDVQZ128mkz = 18963,
  VPSHRDVQZ128r = 18964,
  VPSHRDVQZ128rk = 18965,
  VPSHRDVQZ128rkz = 18966,
  VPSHRDVQZ256m = 18967,
  VPSHRDVQZ256mb = 18968,
  VPSHRDVQZ256mbk = 18969,
  VPSHRDVQZ256mbkz = 18970,
  VPSHRDVQZ256mk = 18971,
  VPSHRDVQZ256mkz = 18972,
  VPSHRDVQZ256r = 18973,
  VPSHRDVQZ256rk = 18974,
  VPSHRDVQZ256rkz = 18975,
  VPSHRDVQZm = 18976,
  VPSHRDVQZmb = 18977,
  VPSHRDVQZmbk = 18978,
  VPSHRDVQZmbkz = 18979,
  VPSHRDVQZmk = 18980,
  VPSHRDVQZmkz = 18981,
  VPSHRDVQZr = 18982,
  VPSHRDVQZrk = 18983,
  VPSHRDVQZrkz = 18984,
  VPSHRDVWZ128m = 18985,
  VPSHRDVWZ128mk = 18986,
  VPSHRDVWZ128mkz = 18987,
  VPSHRDVWZ128r = 18988,
  VPSHRDVWZ128rk = 18989,
  VPSHRDVWZ128rkz = 18990,
  VPSHRDVWZ256m = 18991,
  VPSHRDVWZ256mk = 18992,
  VPSHRDVWZ256mkz = 18993,
  VPSHRDVWZ256r = 18994,
  VPSHRDVWZ256rk = 18995,
  VPSHRDVWZ256rkz = 18996,
  VPSHRDVWZm = 18997,
  VPSHRDVWZmk = 18998,
  VPSHRDVWZmkz = 18999,
  VPSHRDVWZr = 19000,
  VPSHRDVWZrk = 19001,
  VPSHRDVWZrkz = 19002,
  VPSHRDWZ128rmi = 19003,
  VPSHRDWZ128rmik = 19004,
  VPSHRDWZ128rmikz = 19005,
  VPSHRDWZ128rri = 19006,
  VPSHRDWZ128rrik = 19007,
  VPSHRDWZ128rrikz = 19008,
  VPSHRDWZ256rmi = 19009,
  VPSHRDWZ256rmik = 19010,
  VPSHRDWZ256rmikz = 19011,
  VPSHRDWZ256rri = 19012,
  VPSHRDWZ256rrik = 19013,
  VPSHRDWZ256rrikz = 19014,
  VPSHRDWZrmi = 19015,
  VPSHRDWZrmik = 19016,
  VPSHRDWZrmikz = 19017,
  VPSHRDWZrri = 19018,
  VPSHRDWZrrik = 19019,
  VPSHRDWZrrikz = 19020,
  VPSHUFBITQMBZ128rm = 19021,
  VPSHUFBITQMBZ128rmk = 19022,
  VPSHUFBITQMBZ128rr = 19023,
  VPSHUFBITQMBZ128rrk = 19024,
  VPSHUFBITQMBZ256rm = 19025,
  VPSHUFBITQMBZ256rmk = 19026,
  VPSHUFBITQMBZ256rr = 19027,
  VPSHUFBITQMBZ256rrk = 19028,
  VPSHUFBITQMBZrm = 19029,
  VPSHUFBITQMBZrmk = 19030,
  VPSHUFBITQMBZrr = 19031,
  VPSHUFBITQMBZrrk = 19032,
  VPSHUFBYrm = 19033,
  VPSHUFBYrr = 19034,
  VPSHUFBZ128rm = 19035,
  VPSHUFBZ128rmk = 19036,
  VPSHUFBZ128rmkz = 19037,
  VPSHUFBZ128rr = 19038,
  VPSHUFBZ128rrk = 19039,
  VPSHUFBZ128rrkz = 19040,
  VPSHUFBZ256rm = 19041,
  VPSHUFBZ256rmk = 19042,
  VPSHUFBZ256rmkz = 19043,
  VPSHUFBZ256rr = 19044,
  VPSHUFBZ256rrk = 19045,
  VPSHUFBZ256rrkz = 19046,
  VPSHUFBZrm = 19047,
  VPSHUFBZrmk = 19048,
  VPSHUFBZrmkz = 19049,
  VPSHUFBZrr = 19050,
  VPSHUFBZrrk = 19051,
  VPSHUFBZrrkz = 19052,
  VPSHUFBrm = 19053,
  VPSHUFBrr = 19054,
  VPSHUFDYmi = 19055,
  VPSHUFDYri = 19056,
  VPSHUFDZ128mbi = 19057,
  VPSHUFDZ128mbik = 19058,
  VPSHUFDZ128mbikz = 19059,
  VPSHUFDZ128mi = 19060,
  VPSHUFDZ128mik = 19061,
  VPSHUFDZ128mikz = 19062,
  VPSHUFDZ128ri = 19063,
  VPSHUFDZ128rik = 19064,
  VPSHUFDZ128rikz = 19065,
  VPSHUFDZ256mbi = 19066,
  VPSHUFDZ256mbik = 19067,
  VPSHUFDZ256mbikz = 19068,
  VPSHUFDZ256mi = 19069,
  VPSHUFDZ256mik = 19070,
  VPSHUFDZ256mikz = 19071,
  VPSHUFDZ256ri = 19072,
  VPSHUFDZ256rik = 19073,
  VPSHUFDZ256rikz = 19074,
  VPSHUFDZmbi = 19075,
  VPSHUFDZmbik = 19076,
  VPSHUFDZmbikz = 19077,
  VPSHUFDZmi = 19078,
  VPSHUFDZmik = 19079,
  VPSHUFDZmikz = 19080,
  VPSHUFDZri = 19081,
  VPSHUFDZrik = 19082,
  VPSHUFDZrikz = 19083,
  VPSHUFDmi = 19084,
  VPSHUFDri = 19085,
  VPSHUFHWYmi = 19086,
  VPSHUFHWYri = 19087,
  VPSHUFHWZ128mi = 19088,
  VPSHUFHWZ128mik = 19089,
  VPSHUFHWZ128mikz = 19090,
  VPSHUFHWZ128ri = 19091,
  VPSHUFHWZ128rik = 19092,
  VPSHUFHWZ128rikz = 19093,
  VPSHUFHWZ256mi = 19094,
  VPSHUFHWZ256mik = 19095,
  VPSHUFHWZ256mikz = 19096,
  VPSHUFHWZ256ri = 19097,
  VPSHUFHWZ256rik = 19098,
  VPSHUFHWZ256rikz = 19099,
  VPSHUFHWZmi = 19100,
  VPSHUFHWZmik = 19101,
  VPSHUFHWZmikz = 19102,
  VPSHUFHWZri = 19103,
  VPSHUFHWZrik = 19104,
  VPSHUFHWZrikz = 19105,
  VPSHUFHWmi = 19106,
  VPSHUFHWri = 19107,
  VPSHUFLWYmi = 19108,
  VPSHUFLWYri = 19109,
  VPSHUFLWZ128mi = 19110,
  VPSHUFLWZ128mik = 19111,
  VPSHUFLWZ128mikz = 19112,
  VPSHUFLWZ128ri = 19113,
  VPSHUFLWZ128rik = 19114,
  VPSHUFLWZ128rikz = 19115,
  VPSHUFLWZ256mi = 19116,
  VPSHUFLWZ256mik = 19117,
  VPSHUFLWZ256mikz = 19118,
  VPSHUFLWZ256ri = 19119,
  VPSHUFLWZ256rik = 19120,
  VPSHUFLWZ256rikz = 19121,
  VPSHUFLWZmi = 19122,
  VPSHUFLWZmik = 19123,
  VPSHUFLWZmikz = 19124,
  VPSHUFLWZri = 19125,
  VPSHUFLWZrik = 19126,
  VPSHUFLWZrikz = 19127,
  VPSHUFLWmi = 19128,
  VPSHUFLWri = 19129,
  VPSIGNBYrm = 19130,
  VPSIGNBYrr = 19131,
  VPSIGNBrm = 19132,
  VPSIGNBrr = 19133,
  VPSIGNDYrm = 19134,
  VPSIGNDYrr = 19135,
  VPSIGNDrm = 19136,
  VPSIGNDrr = 19137,
  VPSIGNWYrm = 19138,
  VPSIGNWYrr = 19139,
  VPSIGNWrm = 19140,
  VPSIGNWrr = 19141,
  VPSLLDQYri = 19142,
  VPSLLDQZ128mi = 19143,
  VPSLLDQZ128ri = 19144,
  VPSLLDQZ256mi = 19145,
  VPSLLDQZ256ri = 19146,
  VPSLLDQZmi = 19147,
  VPSLLDQZri = 19148,
  VPSLLDQri = 19149,
  VPSLLDYri = 19150,
  VPSLLDYrm = 19151,
  VPSLLDYrr = 19152,
  VPSLLDZ128mbi = 19153,
  VPSLLDZ128mbik = 19154,
  VPSLLDZ128mbikz = 19155,
  VPSLLDZ128mi = 19156,
  VPSLLDZ128mik = 19157,
  VPSLLDZ128mikz = 19158,
  VPSLLDZ128ri = 19159,
  VPSLLDZ128rik = 19160,
  VPSLLDZ128rikz = 19161,
  VPSLLDZ128rm = 19162,
  VPSLLDZ128rmk = 19163,
  VPSLLDZ128rmkz = 19164,
  VPSLLDZ128rr = 19165,
  VPSLLDZ128rrk = 19166,
  VPSLLDZ128rrkz = 19167,
  VPSLLDZ256mbi = 19168,
  VPSLLDZ256mbik = 19169,
  VPSLLDZ256mbikz = 19170,
  VPSLLDZ256mi = 19171,
  VPSLLDZ256mik = 19172,
  VPSLLDZ256mikz = 19173,
  VPSLLDZ256ri = 19174,
  VPSLLDZ256rik = 19175,
  VPSLLDZ256rikz = 19176,
  VPSLLDZ256rm = 19177,
  VPSLLDZ256rmk = 19178,
  VPSLLDZ256rmkz = 19179,
  VPSLLDZ256rr = 19180,
  VPSLLDZ256rrk = 19181,
  VPSLLDZ256rrkz = 19182,
  VPSLLDZmbi = 19183,
  VPSLLDZmbik = 19184,
  VPSLLDZmbikz = 19185,
  VPSLLDZmi = 19186,
  VPSLLDZmik = 19187,
  VPSLLDZmikz = 19188,
  VPSLLDZri = 19189,
  VPSLLDZrik = 19190,
  VPSLLDZrikz = 19191,
  VPSLLDZrm = 19192,
  VPSLLDZrmk = 19193,
  VPSLLDZrmkz = 19194,
  VPSLLDZrr = 19195,
  VPSLLDZrrk = 19196,
  VPSLLDZrrkz = 19197,
  VPSLLDri = 19198,
  VPSLLDrm = 19199,
  VPSLLDrr = 19200,
  VPSLLQYri = 19201,
  VPSLLQYrm = 19202,
  VPSLLQYrr = 19203,
  VPSLLQZ128mbi = 19204,
  VPSLLQZ128mbik = 19205,
  VPSLLQZ128mbikz = 19206,
  VPSLLQZ128mi = 19207,
  VPSLLQZ128mik = 19208,
  VPSLLQZ128mikz = 19209,
  VPSLLQZ128ri = 19210,
  VPSLLQZ128rik = 19211,
  VPSLLQZ128rikz = 19212,
  VPSLLQZ128rm = 19213,
  VPSLLQZ128rmk = 19214,
  VPSLLQZ128rmkz = 19215,
  VPSLLQZ128rr = 19216,
  VPSLLQZ128rrk = 19217,
  VPSLLQZ128rrkz = 19218,
  VPSLLQZ256mbi = 19219,
  VPSLLQZ256mbik = 19220,
  VPSLLQZ256mbikz = 19221,
  VPSLLQZ256mi = 19222,
  VPSLLQZ256mik = 19223,
  VPSLLQZ256mikz = 19224,
  VPSLLQZ256ri = 19225,
  VPSLLQZ256rik = 19226,
  VPSLLQZ256rikz = 19227,
  VPSLLQZ256rm = 19228,
  VPSLLQZ256rmk = 19229,
  VPSLLQZ256rmkz = 19230,
  VPSLLQZ256rr = 19231,
  VPSLLQZ256rrk = 19232,
  VPSLLQZ256rrkz = 19233,
  VPSLLQZmbi = 19234,
  VPSLLQZmbik = 19235,
  VPSLLQZmbikz = 19236,
  VPSLLQZmi = 19237,
  VPSLLQZmik = 19238,
  VPSLLQZmikz = 19239,
  VPSLLQZri = 19240,
  VPSLLQZrik = 19241,
  VPSLLQZrikz = 19242,
  VPSLLQZrm = 19243,
  VPSLLQZrmk = 19244,
  VPSLLQZrmkz = 19245,
  VPSLLQZrr = 19246,
  VPSLLQZrrk = 19247,
  VPSLLQZrrkz = 19248,
  VPSLLQri = 19249,
  VPSLLQrm = 19250,
  VPSLLQrr = 19251,
  VPSLLVDYrm = 19252,
  VPSLLVDYrr = 19253,
  VPSLLVDZ128rm = 19254,
  VPSLLVDZ128rmb = 19255,
  VPSLLVDZ128rmbk = 19256,
  VPSLLVDZ128rmbkz = 19257,
  VPSLLVDZ128rmk = 19258,
  VPSLLVDZ128rmkz = 19259,
  VPSLLVDZ128rr = 19260,
  VPSLLVDZ128rrk = 19261,
  VPSLLVDZ128rrkz = 19262,
  VPSLLVDZ256rm = 19263,
  VPSLLVDZ256rmb = 19264,
  VPSLLVDZ256rmbk = 19265,
  VPSLLVDZ256rmbkz = 19266,
  VPSLLVDZ256rmk = 19267,
  VPSLLVDZ256rmkz = 19268,
  VPSLLVDZ256rr = 19269,
  VPSLLVDZ256rrk = 19270,
  VPSLLVDZ256rrkz = 19271,
  VPSLLVDZrm = 19272,
  VPSLLVDZrmb = 19273,
  VPSLLVDZrmbk = 19274,
  VPSLLVDZrmbkz = 19275,
  VPSLLVDZrmk = 19276,
  VPSLLVDZrmkz = 19277,
  VPSLLVDZrr = 19278,
  VPSLLVDZrrk = 19279,
  VPSLLVDZrrkz = 19280,
  VPSLLVDrm = 19281,
  VPSLLVDrr = 19282,
  VPSLLVQYrm = 19283,
  VPSLLVQYrr = 19284,
  VPSLLVQZ128rm = 19285,
  VPSLLVQZ128rmb = 19286,
  VPSLLVQZ128rmbk = 19287,
  VPSLLVQZ128rmbkz = 19288,
  VPSLLVQZ128rmk = 19289,
  VPSLLVQZ128rmkz = 19290,
  VPSLLVQZ128rr = 19291,
  VPSLLVQZ128rrk = 19292,
  VPSLLVQZ128rrkz = 19293,
  VPSLLVQZ256rm = 19294,
  VPSLLVQZ256rmb = 19295,
  VPSLLVQZ256rmbk = 19296,
  VPSLLVQZ256rmbkz = 19297,
  VPSLLVQZ256rmk = 19298,
  VPSLLVQZ256rmkz = 19299,
  VPSLLVQZ256rr = 19300,
  VPSLLVQZ256rrk = 19301,
  VPSLLVQZ256rrkz = 19302,
  VPSLLVQZrm = 19303,
  VPSLLVQZrmb = 19304,
  VPSLLVQZrmbk = 19305,
  VPSLLVQZrmbkz = 19306,
  VPSLLVQZrmk = 19307,
  VPSLLVQZrmkz = 19308,
  VPSLLVQZrr = 19309,
  VPSLLVQZrrk = 19310,
  VPSLLVQZrrkz = 19311,
  VPSLLVQrm = 19312,
  VPSLLVQrr = 19313,
  VPSLLVWZ128rm = 19314,
  VPSLLVWZ128rmk = 19315,
  VPSLLVWZ128rmkz = 19316,
  VPSLLVWZ128rr = 19317,
  VPSLLVWZ128rrk = 19318,
  VPSLLVWZ128rrkz = 19319,
  VPSLLVWZ256rm = 19320,
  VPSLLVWZ256rmk = 19321,
  VPSLLVWZ256rmkz = 19322,
  VPSLLVWZ256rr = 19323,
  VPSLLVWZ256rrk = 19324,
  VPSLLVWZ256rrkz = 19325,
  VPSLLVWZrm = 19326,
  VPSLLVWZrmk = 19327,
  VPSLLVWZrmkz = 19328,
  VPSLLVWZrr = 19329,
  VPSLLVWZrrk = 19330,
  VPSLLVWZrrkz = 19331,
  VPSLLWYri = 19332,
  VPSLLWYrm = 19333,
  VPSLLWYrr = 19334,
  VPSLLWZ128mi = 19335,
  VPSLLWZ128mik = 19336,
  VPSLLWZ128mikz = 19337,
  VPSLLWZ128ri = 19338,
  VPSLLWZ128rik = 19339,
  VPSLLWZ128rikz = 19340,
  VPSLLWZ128rm = 19341,
  VPSLLWZ128rmk = 19342,
  VPSLLWZ128rmkz = 19343,
  VPSLLWZ128rr = 19344,
  VPSLLWZ128rrk = 19345,
  VPSLLWZ128rrkz = 19346,
  VPSLLWZ256mi = 19347,
  VPSLLWZ256mik = 19348,
  VPSLLWZ256mikz = 19349,
  VPSLLWZ256ri = 19350,
  VPSLLWZ256rik = 19351,
  VPSLLWZ256rikz = 19352,
  VPSLLWZ256rm = 19353,
  VPSLLWZ256rmk = 19354,
  VPSLLWZ256rmkz = 19355,
  VPSLLWZ256rr = 19356,
  VPSLLWZ256rrk = 19357,
  VPSLLWZ256rrkz = 19358,
  VPSLLWZmi = 19359,
  VPSLLWZmik = 19360,
  VPSLLWZmikz = 19361,
  VPSLLWZri = 19362,
  VPSLLWZrik = 19363,
  VPSLLWZrikz = 19364,
  VPSLLWZrm = 19365,
  VPSLLWZrmk = 19366,
  VPSLLWZrmkz = 19367,
  VPSLLWZrr = 19368,
  VPSLLWZrrk = 19369,
  VPSLLWZrrkz = 19370,
  VPSLLWri = 19371,
  VPSLLWrm = 19372,
  VPSLLWrr = 19373,
  VPSRADYri = 19374,
  VPSRADYrm = 19375,
  VPSRADYrr = 19376,
  VPSRADZ128mbi = 19377,
  VPSRADZ128mbik = 19378,
  VPSRADZ128mbikz = 19379,
  VPSRADZ128mi = 19380,
  VPSRADZ128mik = 19381,
  VPSRADZ128mikz = 19382,
  VPSRADZ128ri = 19383,
  VPSRADZ128rik = 19384,
  VPSRADZ128rikz = 19385,
  VPSRADZ128rm = 19386,
  VPSRADZ128rmk = 19387,
  VPSRADZ128rmkz = 19388,
  VPSRADZ128rr = 19389,
  VPSRADZ128rrk = 19390,
  VPSRADZ128rrkz = 19391,
  VPSRADZ256mbi = 19392,
  VPSRADZ256mbik = 19393,
  VPSRADZ256mbikz = 19394,
  VPSRADZ256mi = 19395,
  VPSRADZ256mik = 19396,
  VPSRADZ256mikz = 19397,
  VPSRADZ256ri = 19398,
  VPSRADZ256rik = 19399,
  VPSRADZ256rikz = 19400,
  VPSRADZ256rm = 19401,
  VPSRADZ256rmk = 19402,
  VPSRADZ256rmkz = 19403,
  VPSRADZ256rr = 19404,
  VPSRADZ256rrk = 19405,
  VPSRADZ256rrkz = 19406,
  VPSRADZmbi = 19407,
  VPSRADZmbik = 19408,
  VPSRADZmbikz = 19409,
  VPSRADZmi = 19410,
  VPSRADZmik = 19411,
  VPSRADZmikz = 19412,
  VPSRADZri = 19413,
  VPSRADZrik = 19414,
  VPSRADZrikz = 19415,
  VPSRADZrm = 19416,
  VPSRADZrmk = 19417,
  VPSRADZrmkz = 19418,
  VPSRADZrr = 19419,
  VPSRADZrrk = 19420,
  VPSRADZrrkz = 19421,
  VPSRADri = 19422,
  VPSRADrm = 19423,
  VPSRADrr = 19424,
  VPSRAQZ128mbi = 19425,
  VPSRAQZ128mbik = 19426,
  VPSRAQZ128mbikz = 19427,
  VPSRAQZ128mi = 19428,
  VPSRAQZ128mik = 19429,
  VPSRAQZ128mikz = 19430,
  VPSRAQZ128ri = 19431,
  VPSRAQZ128rik = 19432,
  VPSRAQZ128rikz = 19433,
  VPSRAQZ128rm = 19434,
  VPSRAQZ128rmk = 19435,
  VPSRAQZ128rmkz = 19436,
  VPSRAQZ128rr = 19437,
  VPSRAQZ128rrk = 19438,
  VPSRAQZ128rrkz = 19439,
  VPSRAQZ256mbi = 19440,
  VPSRAQZ256mbik = 19441,
  VPSRAQZ256mbikz = 19442,
  VPSRAQZ256mi = 19443,
  VPSRAQZ256mik = 19444,
  VPSRAQZ256mikz = 19445,
  VPSRAQZ256ri = 19446,
  VPSRAQZ256rik = 19447,
  VPSRAQZ256rikz = 19448,
  VPSRAQZ256rm = 19449,
  VPSRAQZ256rmk = 19450,
  VPSRAQZ256rmkz = 19451,
  VPSRAQZ256rr = 19452,
  VPSRAQZ256rrk = 19453,
  VPSRAQZ256rrkz = 19454,
  VPSRAQZmbi = 19455,
  VPSRAQZmbik = 19456,
  VPSRAQZmbikz = 19457,
  VPSRAQZmi = 19458,
  VPSRAQZmik = 19459,
  VPSRAQZmikz = 19460,
  VPSRAQZri = 19461,
  VPSRAQZrik = 19462,
  VPSRAQZrikz = 19463,
  VPSRAQZrm = 19464,
  VPSRAQZrmk = 19465,
  VPSRAQZrmkz = 19466,
  VPSRAQZrr = 19467,
  VPSRAQZrrk = 19468,
  VPSRAQZrrkz = 19469,
  VPSRAVDYrm = 19470,
  VPSRAVDYrr = 19471,
  VPSRAVDZ128rm = 19472,
  VPSRAVDZ128rmb = 19473,
  VPSRAVDZ128rmbk = 19474,
  VPSRAVDZ128rmbkz = 19475,
  VPSRAVDZ128rmk = 19476,
  VPSRAVDZ128rmkz = 19477,
  VPSRAVDZ128rr = 19478,
  VPSRAVDZ128rrk = 19479,
  VPSRAVDZ128rrkz = 19480,
  VPSRAVDZ256rm = 19481,
  VPSRAVDZ256rmb = 19482,
  VPSRAVDZ256rmbk = 19483,
  VPSRAVDZ256rmbkz = 19484,
  VPSRAVDZ256rmk = 19485,
  VPSRAVDZ256rmkz = 19486,
  VPSRAVDZ256rr = 19487,
  VPSRAVDZ256rrk = 19488,
  VPSRAVDZ256rrkz = 19489,
  VPSRAVDZrm = 19490,
  VPSRAVDZrmb = 19491,
  VPSRAVDZrmbk = 19492,
  VPSRAVDZrmbkz = 19493,
  VPSRAVDZrmk = 19494,
  VPSRAVDZrmkz = 19495,
  VPSRAVDZrr = 19496,
  VPSRAVDZrrk = 19497,
  VPSRAVDZrrkz = 19498,
  VPSRAVDrm = 19499,
  VPSRAVDrr = 19500,
  VPSRAVQZ128rm = 19501,
  VPSRAVQZ128rmb = 19502,
  VPSRAVQZ128rmbk = 19503,
  VPSRAVQZ128rmbkz = 19504,
  VPSRAVQZ128rmk = 19505,
  VPSRAVQZ128rmkz = 19506,
  VPSRAVQZ128rr = 19507,
  VPSRAVQZ128rrk = 19508,
  VPSRAVQZ128rrkz = 19509,
  VPSRAVQZ256rm = 19510,
  VPSRAVQZ256rmb = 19511,
  VPSRAVQZ256rmbk = 19512,
  VPSRAVQZ256rmbkz = 19513,
  VPSRAVQZ256rmk = 19514,
  VPSRAVQZ256rmkz = 19515,
  VPSRAVQZ256rr = 19516,
  VPSRAVQZ256rrk = 19517,
  VPSRAVQZ256rrkz = 19518,
  VPSRAVQZrm = 19519,
  VPSRAVQZrmb = 19520,
  VPSRAVQZrmbk = 19521,
  VPSRAVQZrmbkz = 19522,
  VPSRAVQZrmk = 19523,
  VPSRAVQZrmkz = 19524,
  VPSRAVQZrr = 19525,
  VPSRAVQZrrk = 19526,
  VPSRAVQZrrkz = 19527,
  VPSRAVWZ128rm = 19528,
  VPSRAVWZ128rmk = 19529,
  VPSRAVWZ128rmkz = 19530,
  VPSRAVWZ128rr = 19531,
  VPSRAVWZ128rrk = 19532,
  VPSRAVWZ128rrkz = 19533,
  VPSRAVWZ256rm = 19534,
  VPSRAVWZ256rmk = 19535,
  VPSRAVWZ256rmkz = 19536,
  VPSRAVWZ256rr = 19537,
  VPSRAVWZ256rrk = 19538,
  VPSRAVWZ256rrkz = 19539,
  VPSRAVWZrm = 19540,
  VPSRAVWZrmk = 19541,
  VPSRAVWZrmkz = 19542,
  VPSRAVWZrr = 19543,
  VPSRAVWZrrk = 19544,
  VPSRAVWZrrkz = 19545,
  VPSRAWYri = 19546,
  VPSRAWYrm = 19547,
  VPSRAWYrr = 19548,
  VPSRAWZ128mi = 19549,
  VPSRAWZ128mik = 19550,
  VPSRAWZ128mikz = 19551,
  VPSRAWZ128ri = 19552,
  VPSRAWZ128rik = 19553,
  VPSRAWZ128rikz = 19554,
  VPSRAWZ128rm = 19555,
  VPSRAWZ128rmk = 19556,
  VPSRAWZ128rmkz = 19557,
  VPSRAWZ128rr = 19558,
  VPSRAWZ128rrk = 19559,
  VPSRAWZ128rrkz = 19560,
  VPSRAWZ256mi = 19561,
  VPSRAWZ256mik = 19562,
  VPSRAWZ256mikz = 19563,
  VPSRAWZ256ri = 19564,
  VPSRAWZ256rik = 19565,
  VPSRAWZ256rikz = 19566,
  VPSRAWZ256rm = 19567,
  VPSRAWZ256rmk = 19568,
  VPSRAWZ256rmkz = 19569,
  VPSRAWZ256rr = 19570,
  VPSRAWZ256rrk = 19571,
  VPSRAWZ256rrkz = 19572,
  VPSRAWZmi = 19573,
  VPSRAWZmik = 19574,
  VPSRAWZmikz = 19575,
  VPSRAWZri = 19576,
  VPSRAWZrik = 19577,
  VPSRAWZrikz = 19578,
  VPSRAWZrm = 19579,
  VPSRAWZrmk = 19580,
  VPSRAWZrmkz = 19581,
  VPSRAWZrr = 19582,
  VPSRAWZrrk = 19583,
  VPSRAWZrrkz = 19584,
  VPSRAWri = 19585,
  VPSRAWrm = 19586,
  VPSRAWrr = 19587,
  VPSRLDQYri = 19588,
  VPSRLDQZ128mi = 19589,
  VPSRLDQZ128ri = 19590,
  VPSRLDQZ256mi = 19591,
  VPSRLDQZ256ri = 19592,
  VPSRLDQZmi = 19593,
  VPSRLDQZri = 19594,
  VPSRLDQri = 19595,
  VPSRLDYri = 19596,
  VPSRLDYrm = 19597,
  VPSRLDYrr = 19598,
  VPSRLDZ128mbi = 19599,
  VPSRLDZ128mbik = 19600,
  VPSRLDZ128mbikz = 19601,
  VPSRLDZ128mi = 19602,
  VPSRLDZ128mik = 19603,
  VPSRLDZ128mikz = 19604,
  VPSRLDZ128ri = 19605,
  VPSRLDZ128rik = 19606,
  VPSRLDZ128rikz = 19607,
  VPSRLDZ128rm = 19608,
  VPSRLDZ128rmk = 19609,
  VPSRLDZ128rmkz = 19610,
  VPSRLDZ128rr = 19611,
  VPSRLDZ128rrk = 19612,
  VPSRLDZ128rrkz = 19613,
  VPSRLDZ256mbi = 19614,
  VPSRLDZ256mbik = 19615,
  VPSRLDZ256mbikz = 19616,
  VPSRLDZ256mi = 19617,
  VPSRLDZ256mik = 19618,
  VPSRLDZ256mikz = 19619,
  VPSRLDZ256ri = 19620,
  VPSRLDZ256rik = 19621,
  VPSRLDZ256rikz = 19622,
  VPSRLDZ256rm = 19623,
  VPSRLDZ256rmk = 19624,
  VPSRLDZ256rmkz = 19625,
  VPSRLDZ256rr = 19626,
  VPSRLDZ256rrk = 19627,
  VPSRLDZ256rrkz = 19628,
  VPSRLDZmbi = 19629,
  VPSRLDZmbik = 19630,
  VPSRLDZmbikz = 19631,
  VPSRLDZmi = 19632,
  VPSRLDZmik = 19633,
  VPSRLDZmikz = 19634,
  VPSRLDZri = 19635,
  VPSRLDZrik = 19636,
  VPSRLDZrikz = 19637,
  VPSRLDZrm = 19638,
  VPSRLDZrmk = 19639,
  VPSRLDZrmkz = 19640,
  VPSRLDZrr = 19641,
  VPSRLDZrrk = 19642,
  VPSRLDZrrkz = 19643,
  VPSRLDri = 19644,
  VPSRLDrm = 19645,
  VPSRLDrr = 19646,
  VPSRLQYri = 19647,
  VPSRLQYrm = 19648,
  VPSRLQYrr = 19649,
  VPSRLQZ128mbi = 19650,
  VPSRLQZ128mbik = 19651,
  VPSRLQZ128mbikz = 19652,
  VPSRLQZ128mi = 19653,
  VPSRLQZ128mik = 19654,
  VPSRLQZ128mikz = 19655,
  VPSRLQZ128ri = 19656,
  VPSRLQZ128rik = 19657,
  VPSRLQZ128rikz = 19658,
  VPSRLQZ128rm = 19659,
  VPSRLQZ128rmk = 19660,
  VPSRLQZ128rmkz = 19661,
  VPSRLQZ128rr = 19662,
  VPSRLQZ128rrk = 19663,
  VPSRLQZ128rrkz = 19664,
  VPSRLQZ256mbi = 19665,
  VPSRLQZ256mbik = 19666,
  VPSRLQZ256mbikz = 19667,
  VPSRLQZ256mi = 19668,
  VPSRLQZ256mik = 19669,
  VPSRLQZ256mikz = 19670,
  VPSRLQZ256ri = 19671,
  VPSRLQZ256rik = 19672,
  VPSRLQZ256rikz = 19673,
  VPSRLQZ256rm = 19674,
  VPSRLQZ256rmk = 19675,
  VPSRLQZ256rmkz = 19676,
  VPSRLQZ256rr = 19677,
  VPSRLQZ256rrk = 19678,
  VPSRLQZ256rrkz = 19679,
  VPSRLQZmbi = 19680,
  VPSRLQZmbik = 19681,
  VPSRLQZmbikz = 19682,
  VPSRLQZmi = 19683,
  VPSRLQZmik = 19684,
  VPSRLQZmikz = 19685,
  VPSRLQZri = 19686,
  VPSRLQZrik = 19687,
  VPSRLQZrikz = 19688,
  VPSRLQZrm = 19689,
  VPSRLQZrmk = 19690,
  VPSRLQZrmkz = 19691,
  VPSRLQZrr = 19692,
  VPSRLQZrrk = 19693,
  VPSRLQZrrkz = 19694,
  VPSRLQri = 19695,
  VPSRLQrm = 19696,
  VPSRLQrr = 19697,
  VPSRLVDYrm = 19698,
  VPSRLVDYrr = 19699,
  VPSRLVDZ128rm = 19700,
  VPSRLVDZ128rmb = 19701,
  VPSRLVDZ128rmbk = 19702,
  VPSRLVDZ128rmbkz = 19703,
  VPSRLVDZ128rmk = 19704,
  VPSRLVDZ128rmkz = 19705,
  VPSRLVDZ128rr = 19706,
  VPSRLVDZ128rrk = 19707,
  VPSRLVDZ128rrkz = 19708,
  VPSRLVDZ256rm = 19709,
  VPSRLVDZ256rmb = 19710,
  VPSRLVDZ256rmbk = 19711,
  VPSRLVDZ256rmbkz = 19712,
  VPSRLVDZ256rmk = 19713,
  VPSRLVDZ256rmkz = 19714,
  VPSRLVDZ256rr = 19715,
  VPSRLVDZ256rrk = 19716,
  VPSRLVDZ256rrkz = 19717,
  VPSRLVDZrm = 19718,
  VPSRLVDZrmb = 19719,
  VPSRLVDZrmbk = 19720,
  VPSRLVDZrmbkz = 19721,
  VPSRLVDZrmk = 19722,
  VPSRLVDZrmkz = 19723,
  VPSRLVDZrr = 19724,
  VPSRLVDZrrk = 19725,
  VPSRLVDZrrkz = 19726,
  VPSRLVDrm = 19727,
  VPSRLVDrr = 19728,
  VPSRLVQYrm = 19729,
  VPSRLVQYrr = 19730,
  VPSRLVQZ128rm = 19731,
  VPSRLVQZ128rmb = 19732,
  VPSRLVQZ128rmbk = 19733,
  VPSRLVQZ128rmbkz = 19734,
  VPSRLVQZ128rmk = 19735,
  VPSRLVQZ128rmkz = 19736,
  VPSRLVQZ128rr = 19737,
  VPSRLVQZ128rrk = 19738,
  VPSRLVQZ128rrkz = 19739,
  VPSRLVQZ256rm = 19740,
  VPSRLVQZ256rmb = 19741,
  VPSRLVQZ256rmbk = 19742,
  VPSRLVQZ256rmbkz = 19743,
  VPSRLVQZ256rmk = 19744,
  VPSRLVQZ256rmkz = 19745,
  VPSRLVQZ256rr = 19746,
  VPSRLVQZ256rrk = 19747,
  VPSRLVQZ256rrkz = 19748,
  VPSRLVQZrm = 19749,
  VPSRLVQZrmb = 19750,
  VPSRLVQZrmbk = 19751,
  VPSRLVQZrmbkz = 19752,
  VPSRLVQZrmk = 19753,
  VPSRLVQZrmkz = 19754,
  VPSRLVQZrr = 19755,
  VPSRLVQZrrk = 19756,
  VPSRLVQZrrkz = 19757,
  VPSRLVQrm = 19758,
  VPSRLVQrr = 19759,
  VPSRLVWZ128rm = 19760,
  VPSRLVWZ128rmk = 19761,
  VPSRLVWZ128rmkz = 19762,
  VPSRLVWZ128rr = 19763,
  VPSRLVWZ128rrk = 19764,
  VPSRLVWZ128rrkz = 19765,
  VPSRLVWZ256rm = 19766,
  VPSRLVWZ256rmk = 19767,
  VPSRLVWZ256rmkz = 19768,
  VPSRLVWZ256rr = 19769,
  VPSRLVWZ256rrk = 19770,
  VPSRLVWZ256rrkz = 19771,
  VPSRLVWZrm = 19772,
  VPSRLVWZrmk = 19773,
  VPSRLVWZrmkz = 19774,
  VPSRLVWZrr = 19775,
  VPSRLVWZrrk = 19776,
  VPSRLVWZrrkz = 19777,
  VPSRLWYri = 19778,
  VPSRLWYrm = 19779,
  VPSRLWYrr = 19780,
  VPSRLWZ128mi = 19781,
  VPSRLWZ128mik = 19782,
  VPSRLWZ128mikz = 19783,
  VPSRLWZ128ri = 19784,
  VPSRLWZ128rik = 19785,
  VPSRLWZ128rikz = 19786,
  VPSRLWZ128rm = 19787,
  VPSRLWZ128rmk = 19788,
  VPSRLWZ128rmkz = 19789,
  VPSRLWZ128rr = 19790,
  VPSRLWZ128rrk = 19791,
  VPSRLWZ128rrkz = 19792,
  VPSRLWZ256mi = 19793,
  VPSRLWZ256mik = 19794,
  VPSRLWZ256mikz = 19795,
  VPSRLWZ256ri = 19796,
  VPSRLWZ256rik = 19797,
  VPSRLWZ256rikz = 19798,
  VPSRLWZ256rm = 19799,
  VPSRLWZ256rmk = 19800,
  VPSRLWZ256rmkz = 19801,
  VPSRLWZ256rr = 19802,
  VPSRLWZ256rrk = 19803,
  VPSRLWZ256rrkz = 19804,
  VPSRLWZmi = 19805,
  VPSRLWZmik = 19806,
  VPSRLWZmikz = 19807,
  VPSRLWZri = 19808,
  VPSRLWZrik = 19809,
  VPSRLWZrikz = 19810,
  VPSRLWZrm = 19811,
  VPSRLWZrmk = 19812,
  VPSRLWZrmkz = 19813,
  VPSRLWZrr = 19814,
  VPSRLWZrrk = 19815,
  VPSRLWZrrkz = 19816,
  VPSRLWri = 19817,
  VPSRLWrm = 19818,
  VPSRLWrr = 19819,
  VPSUBBYrm = 19820,
  VPSUBBYrr = 19821,
  VPSUBBZ128rm = 19822,
  VPSUBBZ128rmk = 19823,
  VPSUBBZ128rmkz = 19824,
  VPSUBBZ128rr = 19825,
  VPSUBBZ128rrk = 19826,
  VPSUBBZ128rrkz = 19827,
  VPSUBBZ256rm = 19828,
  VPSUBBZ256rmk = 19829,
  VPSUBBZ256rmkz = 19830,
  VPSUBBZ256rr = 19831,
  VPSUBBZ256rrk = 19832,
  VPSUBBZ256rrkz = 19833,
  VPSUBBZrm = 19834,
  VPSUBBZrmk = 19835,
  VPSUBBZrmkz = 19836,
  VPSUBBZrr = 19837,
  VPSUBBZrrk = 19838,
  VPSUBBZrrkz = 19839,
  VPSUBBrm = 19840,
  VPSUBBrr = 19841,
  VPSUBDYrm = 19842,
  VPSUBDYrr = 19843,
  VPSUBDZ128rm = 19844,
  VPSUBDZ128rmb = 19845,
  VPSUBDZ128rmbk = 19846,
  VPSUBDZ128rmbkz = 19847,
  VPSUBDZ128rmk = 19848,
  VPSUBDZ128rmkz = 19849,
  VPSUBDZ128rr = 19850,
  VPSUBDZ128rrk = 19851,
  VPSUBDZ128rrkz = 19852,
  VPSUBDZ256rm = 19853,
  VPSUBDZ256rmb = 19854,
  VPSUBDZ256rmbk = 19855,
  VPSUBDZ256rmbkz = 19856,
  VPSUBDZ256rmk = 19857,
  VPSUBDZ256rmkz = 19858,
  VPSUBDZ256rr = 19859,
  VPSUBDZ256rrk = 19860,
  VPSUBDZ256rrkz = 19861,
  VPSUBDZrm = 19862,
  VPSUBDZrmb = 19863,
  VPSUBDZrmbk = 19864,
  VPSUBDZrmbkz = 19865,
  VPSUBDZrmk = 19866,
  VPSUBDZrmkz = 19867,
  VPSUBDZrr = 19868,
  VPSUBDZrrk = 19869,
  VPSUBDZrrkz = 19870,
  VPSUBDrm = 19871,
  VPSUBDrr = 19872,
  VPSUBQYrm = 19873,
  VPSUBQYrr = 19874,
  VPSUBQZ128rm = 19875,
  VPSUBQZ128rmb = 19876,
  VPSUBQZ128rmbk = 19877,
  VPSUBQZ128rmbkz = 19878,
  VPSUBQZ128rmk = 19879,
  VPSUBQZ128rmkz = 19880,
  VPSUBQZ128rr = 19881,
  VPSUBQZ128rrk = 19882,
  VPSUBQZ128rrkz = 19883,
  VPSUBQZ256rm = 19884,
  VPSUBQZ256rmb = 19885,
  VPSUBQZ256rmbk = 19886,
  VPSUBQZ256rmbkz = 19887,
  VPSUBQZ256rmk = 19888,
  VPSUBQZ256rmkz = 19889,
  VPSUBQZ256rr = 19890,
  VPSUBQZ256rrk = 19891,
  VPSUBQZ256rrkz = 19892,
  VPSUBQZrm = 19893,
  VPSUBQZrmb = 19894,
  VPSUBQZrmbk = 19895,
  VPSUBQZrmbkz = 19896,
  VPSUBQZrmk = 19897,
  VPSUBQZrmkz = 19898,
  VPSUBQZrr = 19899,
  VPSUBQZrrk = 19900,
  VPSUBQZrrkz = 19901,
  VPSUBQrm = 19902,
  VPSUBQrr = 19903,
  VPSUBSBYrm = 19904,
  VPSUBSBYrr = 19905,
  VPSUBSBZ128rm = 19906,
  VPSUBSBZ128rmk = 19907,
  VPSUBSBZ128rmkz = 19908,
  VPSUBSBZ128rr = 19909,
  VPSUBSBZ128rrk = 19910,
  VPSUBSBZ128rrkz = 19911,
  VPSUBSBZ256rm = 19912,
  VPSUBSBZ256rmk = 19913,
  VPSUBSBZ256rmkz = 19914,
  VPSUBSBZ256rr = 19915,
  VPSUBSBZ256rrk = 19916,
  VPSUBSBZ256rrkz = 19917,
  VPSUBSBZrm = 19918,
  VPSUBSBZrmk = 19919,
  VPSUBSBZrmkz = 19920,
  VPSUBSBZrr = 19921,
  VPSUBSBZrrk = 19922,
  VPSUBSBZrrkz = 19923,
  VPSUBSBrm = 19924,
  VPSUBSBrr = 19925,
  VPSUBSWYrm = 19926,
  VPSUBSWYrr = 19927,
  VPSUBSWZ128rm = 19928,
  VPSUBSWZ128rmk = 19929,
  VPSUBSWZ128rmkz = 19930,
  VPSUBSWZ128rr = 19931,
  VPSUBSWZ128rrk = 19932,
  VPSUBSWZ128rrkz = 19933,
  VPSUBSWZ256rm = 19934,
  VPSUBSWZ256rmk = 19935,
  VPSUBSWZ256rmkz = 19936,
  VPSUBSWZ256rr = 19937,
  VPSUBSWZ256rrk = 19938,
  VPSUBSWZ256rrkz = 19939,
  VPSUBSWZrm = 19940,
  VPSUBSWZrmk = 19941,
  VPSUBSWZrmkz = 19942,
  VPSUBSWZrr = 19943,
  VPSUBSWZrrk = 19944,
  VPSUBSWZrrkz = 19945,
  VPSUBSWrm = 19946,
  VPSUBSWrr = 19947,
  VPSUBUSBYrm = 19948,
  VPSUBUSBYrr = 19949,
  VPSUBUSBZ128rm = 19950,
  VPSUBUSBZ128rmk = 19951,
  VPSUBUSBZ128rmkz = 19952,
  VPSUBUSBZ128rr = 19953,
  VPSUBUSBZ128rrk = 19954,
  VPSUBUSBZ128rrkz = 19955,
  VPSUBUSBZ256rm = 19956,
  VPSUBUSBZ256rmk = 19957,
  VPSUBUSBZ256rmkz = 19958,
  VPSUBUSBZ256rr = 19959,
  VPSUBUSBZ256rrk = 19960,
  VPSUBUSBZ256rrkz = 19961,
  VPSUBUSBZrm = 19962,
  VPSUBUSBZrmk = 19963,
  VPSUBUSBZrmkz = 19964,
  VPSUBUSBZrr = 19965,
  VPSUBUSBZrrk = 19966,
  VPSUBUSBZrrkz = 19967,
  VPSUBUSBrm = 19968,
  VPSUBUSBrr = 19969,
  VPSUBUSWYrm = 19970,
  VPSUBUSWYrr = 19971,
  VPSUBUSWZ128rm = 19972,
  VPSUBUSWZ128rmk = 19973,
  VPSUBUSWZ128rmkz = 19974,
  VPSUBUSWZ128rr = 19975,
  VPSUBUSWZ128rrk = 19976,
  VPSUBUSWZ128rrkz = 19977,
  VPSUBUSWZ256rm = 19978,
  VPSUBUSWZ256rmk = 19979,
  VPSUBUSWZ256rmkz = 19980,
  VPSUBUSWZ256rr = 19981,
  VPSUBUSWZ256rrk = 19982,
  VPSUBUSWZ256rrkz = 19983,
  VPSUBUSWZrm = 19984,
  VPSUBUSWZrmk = 19985,
  VPSUBUSWZrmkz = 19986,
  VPSUBUSWZrr = 19987,
  VPSUBUSWZrrk = 19988,
  VPSUBUSWZrrkz = 19989,
  VPSUBUSWrm = 19990,
  VPSUBUSWrr = 19991,
  VPSUBWYrm = 19992,
  VPSUBWYrr = 19993,
  VPSUBWZ128rm = 19994,
  VPSUBWZ128rmk = 19995,
  VPSUBWZ128rmkz = 19996,
  VPSUBWZ128rr = 19997,
  VPSUBWZ128rrk = 19998,
  VPSUBWZ128rrkz = 19999,
  VPSUBWZ256rm = 20000,
  VPSUBWZ256rmk = 20001,
  VPSUBWZ256rmkz = 20002,
  VPSUBWZ256rr = 20003,
  VPSUBWZ256rrk = 20004,
  VPSUBWZ256rrkz = 20005,
  VPSUBWZrm = 20006,
  VPSUBWZrmk = 20007,
  VPSUBWZrmkz = 20008,
  VPSUBWZrr = 20009,
  VPSUBWZrrk = 20010,
  VPSUBWZrrkz = 20011,
  VPSUBWrm = 20012,
  VPSUBWrr = 20013,
  VPTERNLOGDZ128rmbi = 20014,
  VPTERNLOGDZ128rmbik = 20015,
  VPTERNLOGDZ128rmbikz = 20016,
  VPTERNLOGDZ128rmi = 20017,
  VPTERNLOGDZ128rmik = 20018,
  VPTERNLOGDZ128rmikz = 20019,
  VPTERNLOGDZ128rri = 20020,
  VPTERNLOGDZ128rrik = 20021,
  VPTERNLOGDZ128rrikz = 20022,
  VPTERNLOGDZ256rmbi = 20023,
  VPTERNLOGDZ256rmbik = 20024,
  VPTERNLOGDZ256rmbikz = 20025,
  VPTERNLOGDZ256rmi = 20026,
  VPTERNLOGDZ256rmik = 20027,
  VPTERNLOGDZ256rmikz = 20028,
  VPTERNLOGDZ256rri = 20029,
  VPTERNLOGDZ256rrik = 20030,
  VPTERNLOGDZ256rrikz = 20031,
  VPTERNLOGDZrmbi = 20032,
  VPTERNLOGDZrmbik = 20033,
  VPTERNLOGDZrmbikz = 20034,
  VPTERNLOGDZrmi = 20035,
  VPTERNLOGDZrmik = 20036,
  VPTERNLOGDZrmikz = 20037,
  VPTERNLOGDZrri = 20038,
  VPTERNLOGDZrrik = 20039,
  VPTERNLOGDZrrikz = 20040,
  VPTERNLOGQZ128rmbi = 20041,
  VPTERNLOGQZ128rmbik = 20042,
  VPTERNLOGQZ128rmbikz = 20043,
  VPTERNLOGQZ128rmi = 20044,
  VPTERNLOGQZ128rmik = 20045,
  VPTERNLOGQZ128rmikz = 20046,
  VPTERNLOGQZ128rri = 20047,
  VPTERNLOGQZ128rrik = 20048,
  VPTERNLOGQZ128rrikz = 20049,
  VPTERNLOGQZ256rmbi = 20050,
  VPTERNLOGQZ256rmbik = 20051,
  VPTERNLOGQZ256rmbikz = 20052,
  VPTERNLOGQZ256rmi = 20053,
  VPTERNLOGQZ256rmik = 20054,
  VPTERNLOGQZ256rmikz = 20055,
  VPTERNLOGQZ256rri = 20056,
  VPTERNLOGQZ256rrik = 20057,
  VPTERNLOGQZ256rrikz = 20058,
  VPTERNLOGQZrmbi = 20059,
  VPTERNLOGQZrmbik = 20060,
  VPTERNLOGQZrmbikz = 20061,
  VPTERNLOGQZrmi = 20062,
  VPTERNLOGQZrmik = 20063,
  VPTERNLOGQZrmikz = 20064,
  VPTERNLOGQZrri = 20065,
  VPTERNLOGQZrrik = 20066,
  VPTERNLOGQZrrikz = 20067,
  VPTESTMBZ128rm = 20068,
  VPTESTMBZ128rmk = 20069,
  VPTESTMBZ128rr = 20070,
  VPTESTMBZ128rrk = 20071,
  VPTESTMBZ256rm = 20072,
  VPTESTMBZ256rmk = 20073,
  VPTESTMBZ256rr = 20074,
  VPTESTMBZ256rrk = 20075,
  VPTESTMBZrm = 20076,
  VPTESTMBZrmk = 20077,
  VPTESTMBZrr = 20078,
  VPTESTMBZrrk = 20079,
  VPTESTMDZ128rm = 20080,
  VPTESTMDZ128rmb = 20081,
  VPTESTMDZ128rmbk = 20082,
  VPTESTMDZ128rmk = 20083,
  VPTESTMDZ128rr = 20084,
  VPTESTMDZ128rrk = 20085,
  VPTESTMDZ256rm = 20086,
  VPTESTMDZ256rmb = 20087,
  VPTESTMDZ256rmbk = 20088,
  VPTESTMDZ256rmk = 20089,
  VPTESTMDZ256rr = 20090,
  VPTESTMDZ256rrk = 20091,
  VPTESTMDZrm = 20092,
  VPTESTMDZrmb = 20093,
  VPTESTMDZrmbk = 20094,
  VPTESTMDZrmk = 20095,
  VPTESTMDZrr = 20096,
  VPTESTMDZrrk = 20097,
  VPTESTMQZ128rm = 20098,
  VPTESTMQZ128rmb = 20099,
  VPTESTMQZ128rmbk = 20100,
  VPTESTMQZ128rmk = 20101,
  VPTESTMQZ128rr = 20102,
  VPTESTMQZ128rrk = 20103,
  VPTESTMQZ256rm = 20104,
  VPTESTMQZ256rmb = 20105,
  VPTESTMQZ256rmbk = 20106,
  VPTESTMQZ256rmk = 20107,
  VPTESTMQZ256rr = 20108,
  VPTESTMQZ256rrk = 20109,
  VPTESTMQZrm = 20110,
  VPTESTMQZrmb = 20111,
  VPTESTMQZrmbk = 20112,
  VPTESTMQZrmk = 20113,
  VPTESTMQZrr = 20114,
  VPTESTMQZrrk = 20115,
  VPTESTMWZ128rm = 20116,
  VPTESTMWZ128rmk = 20117,
  VPTESTMWZ128rr = 20118,
  VPTESTMWZ128rrk = 20119,
  VPTESTMWZ256rm = 20120,
  VPTESTMWZ256rmk = 20121,
  VPTESTMWZ256rr = 20122,
  VPTESTMWZ256rrk = 20123,
  VPTESTMWZrm = 20124,
  VPTESTMWZrmk = 20125,
  VPTESTMWZrr = 20126,
  VPTESTMWZrrk = 20127,
  VPTESTNMBZ128rm = 20128,
  VPTESTNMBZ128rmk = 20129,
  VPTESTNMBZ128rr = 20130,
  VPTESTNMBZ128rrk = 20131,
  VPTESTNMBZ256rm = 20132,
  VPTESTNMBZ256rmk = 20133,
  VPTESTNMBZ256rr = 20134,
  VPTESTNMBZ256rrk = 20135,
  VPTESTNMBZrm = 20136,
  VPTESTNMBZrmk = 20137,
  VPTESTNMBZrr = 20138,
  VPTESTNMBZrrk = 20139,
  VPTESTNMDZ128rm = 20140,
  VPTESTNMDZ128rmb = 20141,
  VPTESTNMDZ128rmbk = 20142,
  VPTESTNMDZ128rmk = 20143,
  VPTESTNMDZ128rr = 20144,
  VPTESTNMDZ128rrk = 20145,
  VPTESTNMDZ256rm = 20146,
  VPTESTNMDZ256rmb = 20147,
  VPTESTNMDZ256rmbk = 20148,
  VPTESTNMDZ256rmk = 20149,
  VPTESTNMDZ256rr = 20150,
  VPTESTNMDZ256rrk = 20151,
  VPTESTNMDZrm = 20152,
  VPTESTNMDZrmb = 20153,
  VPTESTNMDZrmbk = 20154,
  VPTESTNMDZrmk = 20155,
  VPTESTNMDZrr = 20156,
  VPTESTNMDZrrk = 20157,
  VPTESTNMQZ128rm = 20158,
  VPTESTNMQZ128rmb = 20159,
  VPTESTNMQZ128rmbk = 20160,
  VPTESTNMQZ128rmk = 20161,
  VPTESTNMQZ128rr = 20162,
  VPTESTNMQZ128rrk = 20163,
  VPTESTNMQZ256rm = 20164,
  VPTESTNMQZ256rmb = 20165,
  VPTESTNMQZ256rmbk = 20166,
  VPTESTNMQZ256rmk = 20167,
  VPTESTNMQZ256rr = 20168,
  VPTESTNMQZ256rrk = 20169,
  VPTESTNMQZrm = 20170,
  VPTESTNMQZrmb = 20171,
  VPTESTNMQZrmbk = 20172,
  VPTESTNMQZrmk = 20173,
  VPTESTNMQZrr = 20174,
  VPTESTNMQZrrk = 20175,
  VPTESTNMWZ128rm = 20176,
  VPTESTNMWZ128rmk = 20177,
  VPTESTNMWZ128rr = 20178,
  VPTESTNMWZ128rrk = 20179,
  VPTESTNMWZ256rm = 20180,
  VPTESTNMWZ256rmk = 20181,
  VPTESTNMWZ256rr = 20182,
  VPTESTNMWZ256rrk = 20183,
  VPTESTNMWZrm = 20184,
  VPTESTNMWZrmk = 20185,
  VPTESTNMWZrr = 20186,
  VPTESTNMWZrrk = 20187,
  VPTESTYrm = 20188,
  VPTESTYrr = 20189,
  VPTESTrm = 20190,
  VPTESTrr = 20191,
  VPUNPCKHBWYrm = 20192,
  VPUNPCKHBWYrr = 20193,
  VPUNPCKHBWZ128rm = 20194,
  VPUNPCKHBWZ128rmk = 20195,
  VPUNPCKHBWZ128rmkz = 20196,
  VPUNPCKHBWZ128rr = 20197,
  VPUNPCKHBWZ128rrk = 20198,
  VPUNPCKHBWZ128rrkz = 20199,
  VPUNPCKHBWZ256rm = 20200,
  VPUNPCKHBWZ256rmk = 20201,
  VPUNPCKHBWZ256rmkz = 20202,
  VPUNPCKHBWZ256rr = 20203,
  VPUNPCKHBWZ256rrk = 20204,
  VPUNPCKHBWZ256rrkz = 20205,
  VPUNPCKHBWZrm = 20206,
  VPUNPCKHBWZrmk = 20207,
  VPUNPCKHBWZrmkz = 20208,
  VPUNPCKHBWZrr = 20209,
  VPUNPCKHBWZrrk = 20210,
  VPUNPCKHBWZrrkz = 20211,
  VPUNPCKHBWrm = 20212,
  VPUNPCKHBWrr = 20213,
  VPUNPCKHDQYrm = 20214,
  VPUNPCKHDQYrr = 20215,
  VPUNPCKHDQZ128rm = 20216,
  VPUNPCKHDQZ128rmb = 20217,
  VPUNPCKHDQZ128rmbk = 20218,
  VPUNPCKHDQZ128rmbkz = 20219,
  VPUNPCKHDQZ128rmk = 20220,
  VPUNPCKHDQZ128rmkz = 20221,
  VPUNPCKHDQZ128rr = 20222,
  VPUNPCKHDQZ128rrk = 20223,
  VPUNPCKHDQZ128rrkz = 20224,
  VPUNPCKHDQZ256rm = 20225,
  VPUNPCKHDQZ256rmb = 20226,
  VPUNPCKHDQZ256rmbk = 20227,
  VPUNPCKHDQZ256rmbkz = 20228,
  VPUNPCKHDQZ256rmk = 20229,
  VPUNPCKHDQZ256rmkz = 20230,
  VPUNPCKHDQZ256rr = 20231,
  VPUNPCKHDQZ256rrk = 20232,
  VPUNPCKHDQZ256rrkz = 20233,
  VPUNPCKHDQZrm = 20234,
  VPUNPCKHDQZrmb = 20235,
  VPUNPCKHDQZrmbk = 20236,
  VPUNPCKHDQZrmbkz = 20237,
  VPUNPCKHDQZrmk = 20238,
  VPUNPCKHDQZrmkz = 20239,
  VPUNPCKHDQZrr = 20240,
  VPUNPCKHDQZrrk = 20241,
  VPUNPCKHDQZrrkz = 20242,
  VPUNPCKHDQrm = 20243,
  VPUNPCKHDQrr = 20244,
  VPUNPCKHQDQYrm = 20245,
  VPUNPCKHQDQYrr = 20246,
  VPUNPCKHQDQZ128rm = 20247,
  VPUNPCKHQDQZ128rmb = 20248,
  VPUNPCKHQDQZ128rmbk = 20249,
  VPUNPCKHQDQZ128rmbkz = 20250,
  VPUNPCKHQDQZ128rmk = 20251,
  VPUNPCKHQDQZ128rmkz = 20252,
  VPUNPCKHQDQZ128rr = 20253,
  VPUNPCKHQDQZ128rrk = 20254,
  VPUNPCKHQDQZ128rrkz = 20255,
  VPUNPCKHQDQZ256rm = 20256,
  VPUNPCKHQDQZ256rmb = 20257,
  VPUNPCKHQDQZ256rmbk = 20258,
  VPUNPCKHQDQZ256rmbkz = 20259,
  VPUNPCKHQDQZ256rmk = 20260,
  VPUNPCKHQDQZ256rmkz = 20261,
  VPUNPCKHQDQZ256rr = 20262,
  VPUNPCKHQDQZ256rrk = 20263,
  VPUNPCKHQDQZ256rrkz = 20264,
  VPUNPCKHQDQZrm = 20265,
  VPUNPCKHQDQZrmb = 20266,
  VPUNPCKHQDQZrmbk = 20267,
  VPUNPCKHQDQZrmbkz = 20268,
  VPUNPCKHQDQZrmk = 20269,
  VPUNPCKHQDQZrmkz = 20270,
  VPUNPCKHQDQZrr = 20271,
  VPUNPCKHQDQZrrk = 20272,
  VPUNPCKHQDQZrrkz = 20273,
  VPUNPCKHQDQrm = 20274,
  VPUNPCKHQDQrr = 20275,
  VPUNPCKHWDYrm = 20276,
  VPUNPCKHWDYrr = 20277,
  VPUNPCKHWDZ128rm = 20278,
  VPUNPCKHWDZ128rmk = 20279,
  VPUNPCKHWDZ128rmkz = 20280,
  VPUNPCKHWDZ128rr = 20281,
  VPUNPCKHWDZ128rrk = 20282,
  VPUNPCKHWDZ128rrkz = 20283,
  VPUNPCKHWDZ256rm = 20284,
  VPUNPCKHWDZ256rmk = 20285,
  VPUNPCKHWDZ256rmkz = 20286,
  VPUNPCKHWDZ256rr = 20287,
  VPUNPCKHWDZ256rrk = 20288,
  VPUNPCKHWDZ256rrkz = 20289,
  VPUNPCKHWDZrm = 20290,
  VPUNPCKHWDZrmk = 20291,
  VPUNPCKHWDZrmkz = 20292,
  VPUNPCKHWDZrr = 20293,
  VPUNPCKHWDZrrk = 20294,
  VPUNPCKHWDZrrkz = 20295,
  VPUNPCKHWDrm = 20296,
  VPUNPCKHWDrr = 20297,
  VPUNPCKLBWYrm = 20298,
  VPUNPCKLBWYrr = 20299,
  VPUNPCKLBWZ128rm = 20300,
  VPUNPCKLBWZ128rmk = 20301,
  VPUNPCKLBWZ128rmkz = 20302,
  VPUNPCKLBWZ128rr = 20303,
  VPUNPCKLBWZ128rrk = 20304,
  VPUNPCKLBWZ128rrkz = 20305,
  VPUNPCKLBWZ256rm = 20306,
  VPUNPCKLBWZ256rmk = 20307,
  VPUNPCKLBWZ256rmkz = 20308,
  VPUNPCKLBWZ256rr = 20309,
  VPUNPCKLBWZ256rrk = 20310,
  VPUNPCKLBWZ256rrkz = 20311,
  VPUNPCKLBWZrm = 20312,
  VPUNPCKLBWZrmk = 20313,
  VPUNPCKLBWZrmkz = 20314,
  VPUNPCKLBWZrr = 20315,
  VPUNPCKLBWZrrk = 20316,
  VPUNPCKLBWZrrkz = 20317,
  VPUNPCKLBWrm = 20318,
  VPUNPCKLBWrr = 20319,
  VPUNPCKLDQYrm = 20320,
  VPUNPCKLDQYrr = 20321,
  VPUNPCKLDQZ128rm = 20322,
  VPUNPCKLDQZ128rmb = 20323,
  VPUNPCKLDQZ128rmbk = 20324,
  VPUNPCKLDQZ128rmbkz = 20325,
  VPUNPCKLDQZ128rmk = 20326,
  VPUNPCKLDQZ128rmkz = 20327,
  VPUNPCKLDQZ128rr = 20328,
  VPUNPCKLDQZ128rrk = 20329,
  VPUNPCKLDQZ128rrkz = 20330,
  VPUNPCKLDQZ256rm = 20331,
  VPUNPCKLDQZ256rmb = 20332,
  VPUNPCKLDQZ256rmbk = 20333,
  VPUNPCKLDQZ256rmbkz = 20334,
  VPUNPCKLDQZ256rmk = 20335,
  VPUNPCKLDQZ256rmkz = 20336,
  VPUNPCKLDQZ256rr = 20337,
  VPUNPCKLDQZ256rrk = 20338,
  VPUNPCKLDQZ256rrkz = 20339,
  VPUNPCKLDQZrm = 20340,
  VPUNPCKLDQZrmb = 20341,
  VPUNPCKLDQZrmbk = 20342,
  VPUNPCKLDQZrmbkz = 20343,
  VPUNPCKLDQZrmk = 20344,
  VPUNPCKLDQZrmkz = 20345,
  VPUNPCKLDQZrr = 20346,
  VPUNPCKLDQZrrk = 20347,
  VPUNPCKLDQZrrkz = 20348,
  VPUNPCKLDQrm = 20349,
  VPUNPCKLDQrr = 20350,
  VPUNPCKLQDQYrm = 20351,
  VPUNPCKLQDQYrr = 20352,
  VPUNPCKLQDQZ128rm = 20353,
  VPUNPCKLQDQZ128rmb = 20354,
  VPUNPCKLQDQZ128rmbk = 20355,
  VPUNPCKLQDQZ128rmbkz = 20356,
  VPUNPCKLQDQZ128rmk = 20357,
  VPUNPCKLQDQZ128rmkz = 20358,
  VPUNPCKLQDQZ128rr = 20359,
  VPUNPCKLQDQZ128rrk = 20360,
  VPUNPCKLQDQZ128rrkz = 20361,
  VPUNPCKLQDQZ256rm = 20362,
  VPUNPCKLQDQZ256rmb = 20363,
  VPUNPCKLQDQZ256rmbk = 20364,
  VPUNPCKLQDQZ256rmbkz = 20365,
  VPUNPCKLQDQZ256rmk = 20366,
  VPUNPCKLQDQZ256rmkz = 20367,
  VPUNPCKLQDQZ256rr = 20368,
  VPUNPCKLQDQZ256rrk = 20369,
  VPUNPCKLQDQZ256rrkz = 20370,
  VPUNPCKLQDQZrm = 20371,
  VPUNPCKLQDQZrmb = 20372,
  VPUNPCKLQDQZrmbk = 20373,
  VPUNPCKLQDQZrmbkz = 20374,
  VPUNPCKLQDQZrmk = 20375,
  VPUNPCKLQDQZrmkz = 20376,
  VPUNPCKLQDQZrr = 20377,
  VPUNPCKLQDQZrrk = 20378,
  VPUNPCKLQDQZrrkz = 20379,
  VPUNPCKLQDQrm = 20380,
  VPUNPCKLQDQrr = 20381,
  VPUNPCKLWDYrm = 20382,
  VPUNPCKLWDYrr = 20383,
  VPUNPCKLWDZ128rm = 20384,
  VPUNPCKLWDZ128rmk = 20385,
  VPUNPCKLWDZ128rmkz = 20386,
  VPUNPCKLWDZ128rr = 20387,
  VPUNPCKLWDZ128rrk = 20388,
  VPUNPCKLWDZ128rrkz = 20389,
  VPUNPCKLWDZ256rm = 20390,
  VPUNPCKLWDZ256rmk = 20391,
  VPUNPCKLWDZ256rmkz = 20392,
  VPUNPCKLWDZ256rr = 20393,
  VPUNPCKLWDZ256rrk = 20394,
  VPUNPCKLWDZ256rrkz = 20395,
  VPUNPCKLWDZrm = 20396,
  VPUNPCKLWDZrmk = 20397,
  VPUNPCKLWDZrmkz = 20398,
  VPUNPCKLWDZrr = 20399,
  VPUNPCKLWDZrrk = 20400,
  VPUNPCKLWDZrrkz = 20401,
  VPUNPCKLWDrm = 20402,
  VPUNPCKLWDrr = 20403,
  VPXORDZ128rm = 20404,
  VPXORDZ128rmb = 20405,
  VPXORDZ128rmbk = 20406,
  VPXORDZ128rmbkz = 20407,
  VPXORDZ128rmk = 20408,
  VPXORDZ128rmkz = 20409,
  VPXORDZ128rr = 20410,
  VPXORDZ128rrk = 20411,
  VPXORDZ128rrkz = 20412,
  VPXORDZ256rm = 20413,
  VPXORDZ256rmb = 20414,
  VPXORDZ256rmbk = 20415,
  VPXORDZ256rmbkz = 20416,
  VPXORDZ256rmk = 20417,
  VPXORDZ256rmkz = 20418,
  VPXORDZ256rr = 20419,
  VPXORDZ256rrk = 20420,
  VPXORDZ256rrkz = 20421,
  VPXORDZrm = 20422,
  VPXORDZrmb = 20423,
  VPXORDZrmbk = 20424,
  VPXORDZrmbkz = 20425,
  VPXORDZrmk = 20426,
  VPXORDZrmkz = 20427,
  VPXORDZrr = 20428,
  VPXORDZrrk = 20429,
  VPXORDZrrkz = 20430,
  VPXORQZ128rm = 20431,
  VPXORQZ128rmb = 20432,
  VPXORQZ128rmbk = 20433,
  VPXORQZ128rmbkz = 20434,
  VPXORQZ128rmk = 20435,
  VPXORQZ128rmkz = 20436,
  VPXORQZ128rr = 20437,
  VPXORQZ128rrk = 20438,
  VPXORQZ128rrkz = 20439,
  VPXORQZ256rm = 20440,
  VPXORQZ256rmb = 20441,
  VPXORQZ256rmbk = 20442,
  VPXORQZ256rmbkz = 20443,
  VPXORQZ256rmk = 20444,
  VPXORQZ256rmkz = 20445,
  VPXORQZ256rr = 20446,
  VPXORQZ256rrk = 20447,
  VPXORQZ256rrkz = 20448,
  VPXORQZrm = 20449,
  VPXORQZrmb = 20450,
  VPXORQZrmbk = 20451,
  VPXORQZrmbkz = 20452,
  VPXORQZrmk = 20453,
  VPXORQZrmkz = 20454,
  VPXORQZrr = 20455,
  VPXORQZrrk = 20456,
  VPXORQZrrkz = 20457,
  VPXORYrm = 20458,
  VPXORYrr = 20459,
  VPXORrm = 20460,
  VPXORrr = 20461,
  VRANGEPDZ128rmbi = 20462,
  VRANGEPDZ128rmbik = 20463,
  VRANGEPDZ128rmbikz = 20464,
  VRANGEPDZ128rmi = 20465,
  VRANGEPDZ128rmik = 20466,
  VRANGEPDZ128rmikz = 20467,
  VRANGEPDZ128rri = 20468,
  VRANGEPDZ128rrik = 20469,
  VRANGEPDZ128rrikz = 20470,
  VRANGEPDZ256rmbi = 20471,
  VRANGEPDZ256rmbik = 20472,
  VRANGEPDZ256rmbikz = 20473,
  VRANGEPDZ256rmi = 20474,
  VRANGEPDZ256rmik = 20475,
  VRANGEPDZ256rmikz = 20476,
  VRANGEPDZ256rri = 20477,
  VRANGEPDZ256rrik = 20478,
  VRANGEPDZ256rrikz = 20479,
  VRANGEPDZrmbi = 20480,
  VRANGEPDZrmbik = 20481,
  VRANGEPDZrmbikz = 20482,
  VRANGEPDZrmi = 20483,
  VRANGEPDZrmik = 20484,
  VRANGEPDZrmikz = 20485,
  VRANGEPDZrri = 20486,
  VRANGEPDZrrib = 20487,
  VRANGEPDZrribk = 20488,
  VRANGEPDZrribkz = 20489,
  VRANGEPDZrrik = 20490,
  VRANGEPDZrrikz = 20491,
  VRANGEPSZ128rmbi = 20492,
  VRANGEPSZ128rmbik = 20493,
  VRANGEPSZ128rmbikz = 20494,
  VRANGEPSZ128rmi = 20495,
  VRANGEPSZ128rmik = 20496,
  VRANGEPSZ128rmikz = 20497,
  VRANGEPSZ128rri = 20498,
  VRANGEPSZ128rrik = 20499,
  VRANGEPSZ128rrikz = 20500,
  VRANGEPSZ256rmbi = 20501,
  VRANGEPSZ256rmbik = 20502,
  VRANGEPSZ256rmbikz = 20503,
  VRANGEPSZ256rmi = 20504,
  VRANGEPSZ256rmik = 20505,
  VRANGEPSZ256rmikz = 20506,
  VRANGEPSZ256rri = 20507,
  VRANGEPSZ256rrik = 20508,
  VRANGEPSZ256rrikz = 20509,
  VRANGEPSZrmbi = 20510,
  VRANGEPSZrmbik = 20511,
  VRANGEPSZrmbikz = 20512,
  VRANGEPSZrmi = 20513,
  VRANGEPSZrmik = 20514,
  VRANGEPSZrmikz = 20515,
  VRANGEPSZrri = 20516,
  VRANGEPSZrrib = 20517,
  VRANGEPSZrribk = 20518,
  VRANGEPSZrribkz = 20519,
  VRANGEPSZrrik = 20520,
  VRANGEPSZrrikz = 20521,
  VRANGESDZrmi = 20522,
  VRANGESDZrmik = 20523,
  VRANGESDZrmikz = 20524,
  VRANGESDZrri = 20525,
  VRANGESDZrrib = 20526,
  VRANGESDZrribk = 20527,
  VRANGESDZrribkz = 20528,
  VRANGESDZrrik = 20529,
  VRANGESDZrrikz = 20530,
  VRANGESSZrmi = 20531,
  VRANGESSZrmik = 20532,
  VRANGESSZrmikz = 20533,
  VRANGESSZrri = 20534,
  VRANGESSZrrib = 20535,
  VRANGESSZrribk = 20536,
  VRANGESSZrribkz = 20537,
  VRANGESSZrrik = 20538,
  VRANGESSZrrikz = 20539,
  VRCP14PDZ128m = 20540,
  VRCP14PDZ128mb = 20541,
  VRCP14PDZ128mbk = 20542,
  VRCP14PDZ128mbkz = 20543,
  VRCP14PDZ128mk = 20544,
  VRCP14PDZ128mkz = 20545,
  VRCP14PDZ128r = 20546,
  VRCP14PDZ128rk = 20547,
  VRCP14PDZ128rkz = 20548,
  VRCP14PDZ256m = 20549,
  VRCP14PDZ256mb = 20550,
  VRCP14PDZ256mbk = 20551,
  VRCP14PDZ256mbkz = 20552,
  VRCP14PDZ256mk = 20553,
  VRCP14PDZ256mkz = 20554,
  VRCP14PDZ256r = 20555,
  VRCP14PDZ256rk = 20556,
  VRCP14PDZ256rkz = 20557,
  VRCP14PDZm = 20558,
  VRCP14PDZmb = 20559,
  VRCP14PDZmbk = 20560,
  VRCP14PDZmbkz = 20561,
  VRCP14PDZmk = 20562,
  VRCP14PDZmkz = 20563,
  VRCP14PDZr = 20564,
  VRCP14PDZrk = 20565,
  VRCP14PDZrkz = 20566,
  VRCP14PSZ128m = 20567,
  VRCP14PSZ128mb = 20568,
  VRCP14PSZ128mbk = 20569,
  VRCP14PSZ128mbkz = 20570,
  VRCP14PSZ128mk = 20571,
  VRCP14PSZ128mkz = 20572,
  VRCP14PSZ128r = 20573,
  VRCP14PSZ128rk = 20574,
  VRCP14PSZ128rkz = 20575,
  VRCP14PSZ256m = 20576,
  VRCP14PSZ256mb = 20577,
  VRCP14PSZ256mbk = 20578,
  VRCP14PSZ256mbkz = 20579,
  VRCP14PSZ256mk = 20580,
  VRCP14PSZ256mkz = 20581,
  VRCP14PSZ256r = 20582,
  VRCP14PSZ256rk = 20583,
  VRCP14PSZ256rkz = 20584,
  VRCP14PSZm = 20585,
  VRCP14PSZmb = 20586,
  VRCP14PSZmbk = 20587,
  VRCP14PSZmbkz = 20588,
  VRCP14PSZmk = 20589,
  VRCP14PSZmkz = 20590,
  VRCP14PSZr = 20591,
  VRCP14PSZrk = 20592,
  VRCP14PSZrkz = 20593,
  VRCP14SDZrm = 20594,
  VRCP14SDZrmk = 20595,
  VRCP14SDZrmkz = 20596,
  VRCP14SDZrr = 20597,
  VRCP14SDZrrk = 20598,
  VRCP14SDZrrkz = 20599,
  VRCP14SSZrm = 20600,
  VRCP14SSZrmk = 20601,
  VRCP14SSZrmkz = 20602,
  VRCP14SSZrr = 20603,
  VRCP14SSZrrk = 20604,
  VRCP14SSZrrkz = 20605,
  VRCP28PDZm = 20606,
  VRCP28PDZmb = 20607,
  VRCP28PDZmbk = 20608,
  VRCP28PDZmbkz = 20609,
  VRCP28PDZmk = 20610,
  VRCP28PDZmkz = 20611,
  VRCP28PDZr = 20612,
  VRCP28PDZrb = 20613,
  VRCP28PDZrbk = 20614,
  VRCP28PDZrbkz = 20615,
  VRCP28PDZrk = 20616,
  VRCP28PDZrkz = 20617,
  VRCP28PSZm = 20618,
  VRCP28PSZmb = 20619,
  VRCP28PSZmbk = 20620,
  VRCP28PSZmbkz = 20621,
  VRCP28PSZmk = 20622,
  VRCP28PSZmkz = 20623,
  VRCP28PSZr = 20624,
  VRCP28PSZrb = 20625,
  VRCP28PSZrbk = 20626,
  VRCP28PSZrbkz = 20627,
  VRCP28PSZrk = 20628,
  VRCP28PSZrkz = 20629,
  VRCP28SDZm = 20630,
  VRCP28SDZmk = 20631,
  VRCP28SDZmkz = 20632,
  VRCP28SDZr = 20633,
  VRCP28SDZrb = 20634,
  VRCP28SDZrbk = 20635,
  VRCP28SDZrbkz = 20636,
  VRCP28SDZrk = 20637,
  VRCP28SDZrkz = 20638,
  VRCP28SSZm = 20639,
  VRCP28SSZmk = 20640,
  VRCP28SSZmkz = 20641,
  VRCP28SSZr = 20642,
  VRCP28SSZrb = 20643,
  VRCP28SSZrbk = 20644,
  VRCP28SSZrbkz = 20645,
  VRCP28SSZrk = 20646,
  VRCP28SSZrkz = 20647,
  VRCPBF16Z128m = 20648,
  VRCPBF16Z128mb = 20649,
  VRCPBF16Z128mbk = 20650,
  VRCPBF16Z128mbkz = 20651,
  VRCPBF16Z128mk = 20652,
  VRCPBF16Z128mkz = 20653,
  VRCPBF16Z128r = 20654,
  VRCPBF16Z128rk = 20655,
  VRCPBF16Z128rkz = 20656,
  VRCPBF16Z256m = 20657,
  VRCPBF16Z256mb = 20658,
  VRCPBF16Z256mbk = 20659,
  VRCPBF16Z256mbkz = 20660,
  VRCPBF16Z256mk = 20661,
  VRCPBF16Z256mkz = 20662,
  VRCPBF16Z256r = 20663,
  VRCPBF16Z256rk = 20664,
  VRCPBF16Z256rkz = 20665,
  VRCPBF16Zm = 20666,
  VRCPBF16Zmb = 20667,
  VRCPBF16Zmbk = 20668,
  VRCPBF16Zmbkz = 20669,
  VRCPBF16Zmk = 20670,
  VRCPBF16Zmkz = 20671,
  VRCPBF16Zr = 20672,
  VRCPBF16Zrk = 20673,
  VRCPBF16Zrkz = 20674,
  VRCPPHZ128m = 20675,
  VRCPPHZ128mb = 20676,
  VRCPPHZ128mbk = 20677,
  VRCPPHZ128mbkz = 20678,
  VRCPPHZ128mk = 20679,
  VRCPPHZ128mkz = 20680,
  VRCPPHZ128r = 20681,
  VRCPPHZ128rk = 20682,
  VRCPPHZ128rkz = 20683,
  VRCPPHZ256m = 20684,
  VRCPPHZ256mb = 20685,
  VRCPPHZ256mbk = 20686,
  VRCPPHZ256mbkz = 20687,
  VRCPPHZ256mk = 20688,
  VRCPPHZ256mkz = 20689,
  VRCPPHZ256r = 20690,
  VRCPPHZ256rk = 20691,
  VRCPPHZ256rkz = 20692,
  VRCPPHZm = 20693,
  VRCPPHZmb = 20694,
  VRCPPHZmbk = 20695,
  VRCPPHZmbkz = 20696,
  VRCPPHZmk = 20697,
  VRCPPHZmkz = 20698,
  VRCPPHZr = 20699,
  VRCPPHZrk = 20700,
  VRCPPHZrkz = 20701,
  VRCPPSYm = 20702,
  VRCPPSYr = 20703,
  VRCPPSm = 20704,
  VRCPPSr = 20705,
  VRCPSHZrm = 20706,
  VRCPSHZrmk = 20707,
  VRCPSHZrmkz = 20708,
  VRCPSHZrr = 20709,
  VRCPSHZrrk = 20710,
  VRCPSHZrrkz = 20711,
  VRCPSSm = 20712,
  VRCPSSm_Int = 20713,
  VRCPSSr = 20714,
  VRCPSSr_Int = 20715,
  VREDUCEBF16Z128rmbi = 20716,
  VREDUCEBF16Z128rmbik = 20717,
  VREDUCEBF16Z128rmbikz = 20718,
  VREDUCEBF16Z128rmi = 20719,
  VREDUCEBF16Z128rmik = 20720,
  VREDUCEBF16Z128rmikz = 20721,
  VREDUCEBF16Z128rri = 20722,
  VREDUCEBF16Z128rrik = 20723,
  VREDUCEBF16Z128rrikz = 20724,
  VREDUCEBF16Z256rmbi = 20725,
  VREDUCEBF16Z256rmbik = 20726,
  VREDUCEBF16Z256rmbikz = 20727,
  VREDUCEBF16Z256rmi = 20728,
  VREDUCEBF16Z256rmik = 20729,
  VREDUCEBF16Z256rmikz = 20730,
  VREDUCEBF16Z256rri = 20731,
  VREDUCEBF16Z256rrik = 20732,
  VREDUCEBF16Z256rrikz = 20733,
  VREDUCEBF16Zrmbi = 20734,
  VREDUCEBF16Zrmbik = 20735,
  VREDUCEBF16Zrmbikz = 20736,
  VREDUCEBF16Zrmi = 20737,
  VREDUCEBF16Zrmik = 20738,
  VREDUCEBF16Zrmikz = 20739,
  VREDUCEBF16Zrri = 20740,
  VREDUCEBF16Zrrik = 20741,
  VREDUCEBF16Zrrikz = 20742,
  VREDUCEPDZ128rmbi = 20743,
  VREDUCEPDZ128rmbik = 20744,
  VREDUCEPDZ128rmbikz = 20745,
  VREDUCEPDZ128rmi = 20746,
  VREDUCEPDZ128rmik = 20747,
  VREDUCEPDZ128rmikz = 20748,
  VREDUCEPDZ128rri = 20749,
  VREDUCEPDZ128rrik = 20750,
  VREDUCEPDZ128rrikz = 20751,
  VREDUCEPDZ256rmbi = 20752,
  VREDUCEPDZ256rmbik = 20753,
  VREDUCEPDZ256rmbikz = 20754,
  VREDUCEPDZ256rmi = 20755,
  VREDUCEPDZ256rmik = 20756,
  VREDUCEPDZ256rmikz = 20757,
  VREDUCEPDZ256rri = 20758,
  VREDUCEPDZ256rrik = 20759,
  VREDUCEPDZ256rrikz = 20760,
  VREDUCEPDZrmbi = 20761,
  VREDUCEPDZrmbik = 20762,
  VREDUCEPDZrmbikz = 20763,
  VREDUCEPDZrmi = 20764,
  VREDUCEPDZrmik = 20765,
  VREDUCEPDZrmikz = 20766,
  VREDUCEPDZrri = 20767,
  VREDUCEPDZrrib = 20768,
  VREDUCEPDZrribk = 20769,
  VREDUCEPDZrribkz = 20770,
  VREDUCEPDZrrik = 20771,
  VREDUCEPDZrrikz = 20772,
  VREDUCEPHZ128rmbi = 20773,
  VREDUCEPHZ128rmbik = 20774,
  VREDUCEPHZ128rmbikz = 20775,
  VREDUCEPHZ128rmi = 20776,
  VREDUCEPHZ128rmik = 20777,
  VREDUCEPHZ128rmikz = 20778,
  VREDUCEPHZ128rri = 20779,
  VREDUCEPHZ128rrik = 20780,
  VREDUCEPHZ128rrikz = 20781,
  VREDUCEPHZ256rmbi = 20782,
  VREDUCEPHZ256rmbik = 20783,
  VREDUCEPHZ256rmbikz = 20784,
  VREDUCEPHZ256rmi = 20785,
  VREDUCEPHZ256rmik = 20786,
  VREDUCEPHZ256rmikz = 20787,
  VREDUCEPHZ256rri = 20788,
  VREDUCEPHZ256rrik = 20789,
  VREDUCEPHZ256rrikz = 20790,
  VREDUCEPHZrmbi = 20791,
  VREDUCEPHZrmbik = 20792,
  VREDUCEPHZrmbikz = 20793,
  VREDUCEPHZrmi = 20794,
  VREDUCEPHZrmik = 20795,
  VREDUCEPHZrmikz = 20796,
  VREDUCEPHZrri = 20797,
  VREDUCEPHZrrib = 20798,
  VREDUCEPHZrribk = 20799,
  VREDUCEPHZrribkz = 20800,
  VREDUCEPHZrrik = 20801,
  VREDUCEPHZrrikz = 20802,
  VREDUCEPSZ128rmbi = 20803,
  VREDUCEPSZ128rmbik = 20804,
  VREDUCEPSZ128rmbikz = 20805,
  VREDUCEPSZ128rmi = 20806,
  VREDUCEPSZ128rmik = 20807,
  VREDUCEPSZ128rmikz = 20808,
  VREDUCEPSZ128rri = 20809,
  VREDUCEPSZ128rrik = 20810,
  VREDUCEPSZ128rrikz = 20811,
  VREDUCEPSZ256rmbi = 20812,
  VREDUCEPSZ256rmbik = 20813,
  VREDUCEPSZ256rmbikz = 20814,
  VREDUCEPSZ256rmi = 20815,
  VREDUCEPSZ256rmik = 20816,
  VREDUCEPSZ256rmikz = 20817,
  VREDUCEPSZ256rri = 20818,
  VREDUCEPSZ256rrik = 20819,
  VREDUCEPSZ256rrikz = 20820,
  VREDUCEPSZrmbi = 20821,
  VREDUCEPSZrmbik = 20822,
  VREDUCEPSZrmbikz = 20823,
  VREDUCEPSZrmi = 20824,
  VREDUCEPSZrmik = 20825,
  VREDUCEPSZrmikz = 20826,
  VREDUCEPSZrri = 20827,
  VREDUCEPSZrrib = 20828,
  VREDUCEPSZrribk = 20829,
  VREDUCEPSZrribkz = 20830,
  VREDUCEPSZrrik = 20831,
  VREDUCEPSZrrikz = 20832,
  VREDUCESDZrmi = 20833,
  VREDUCESDZrmik = 20834,
  VREDUCESDZrmikz = 20835,
  VREDUCESDZrri = 20836,
  VREDUCESDZrrib = 20837,
  VREDUCESDZrribk = 20838,
  VREDUCESDZrribkz = 20839,
  VREDUCESDZrrik = 20840,
  VREDUCESDZrrikz = 20841,
  VREDUCESHZrmi = 20842,
  VREDUCESHZrmik = 20843,
  VREDUCESHZrmikz = 20844,
  VREDUCESHZrri = 20845,
  VREDUCESHZrrib = 20846,
  VREDUCESHZrribk = 20847,
  VREDUCESHZrribkz = 20848,
  VREDUCESHZrrik = 20849,
  VREDUCESHZrrikz = 20850,
  VREDUCESSZrmi = 20851,
  VREDUCESSZrmik = 20852,
  VREDUCESSZrmikz = 20853,
  VREDUCESSZrri = 20854,
  VREDUCESSZrrib = 20855,
  VREDUCESSZrribk = 20856,
  VREDUCESSZrribkz = 20857,
  VREDUCESSZrrik = 20858,
  VREDUCESSZrrikz = 20859,
  VRNDSCALEBF16Z128rmbi = 20860,
  VRNDSCALEBF16Z128rmbik = 20861,
  VRNDSCALEBF16Z128rmbikz = 20862,
  VRNDSCALEBF16Z128rmi = 20863,
  VRNDSCALEBF16Z128rmik = 20864,
  VRNDSCALEBF16Z128rmikz = 20865,
  VRNDSCALEBF16Z128rri = 20866,
  VRNDSCALEBF16Z128rrik = 20867,
  VRNDSCALEBF16Z128rrikz = 20868,
  VRNDSCALEBF16Z256rmbi = 20869,
  VRNDSCALEBF16Z256rmbik = 20870,
  VRNDSCALEBF16Z256rmbikz = 20871,
  VRNDSCALEBF16Z256rmi = 20872,
  VRNDSCALEBF16Z256rmik = 20873,
  VRNDSCALEBF16Z256rmikz = 20874,
  VRNDSCALEBF16Z256rri = 20875,
  VRNDSCALEBF16Z256rrik = 20876,
  VRNDSCALEBF16Z256rrikz = 20877,
  VRNDSCALEBF16Zrmbi = 20878,
  VRNDSCALEBF16Zrmbik = 20879,
  VRNDSCALEBF16Zrmbikz = 20880,
  VRNDSCALEBF16Zrmi = 20881,
  VRNDSCALEBF16Zrmik = 20882,
  VRNDSCALEBF16Zrmikz = 20883,
  VRNDSCALEBF16Zrri = 20884,
  VRNDSCALEBF16Zrrik = 20885,
  VRNDSCALEBF16Zrrikz = 20886,
  VRNDSCALEPDZ128rmbi = 20887,
  VRNDSCALEPDZ128rmbik = 20888,
  VRNDSCALEPDZ128rmbikz = 20889,
  VRNDSCALEPDZ128rmi = 20890,
  VRNDSCALEPDZ128rmik = 20891,
  VRNDSCALEPDZ128rmikz = 20892,
  VRNDSCALEPDZ128rri = 20893,
  VRNDSCALEPDZ128rrik = 20894,
  VRNDSCALEPDZ128rrikz = 20895,
  VRNDSCALEPDZ256rmbi = 20896,
  VRNDSCALEPDZ256rmbik = 20897,
  VRNDSCALEPDZ256rmbikz = 20898,
  VRNDSCALEPDZ256rmi = 20899,
  VRNDSCALEPDZ256rmik = 20900,
  VRNDSCALEPDZ256rmikz = 20901,
  VRNDSCALEPDZ256rri = 20902,
  VRNDSCALEPDZ256rrik = 20903,
  VRNDSCALEPDZ256rrikz = 20904,
  VRNDSCALEPDZrmbi = 20905,
  VRNDSCALEPDZrmbik = 20906,
  VRNDSCALEPDZrmbikz = 20907,
  VRNDSCALEPDZrmi = 20908,
  VRNDSCALEPDZrmik = 20909,
  VRNDSCALEPDZrmikz = 20910,
  VRNDSCALEPDZrri = 20911,
  VRNDSCALEPDZrrib = 20912,
  VRNDSCALEPDZrribk = 20913,
  VRNDSCALEPDZrribkz = 20914,
  VRNDSCALEPDZrrik = 20915,
  VRNDSCALEPDZrrikz = 20916,
  VRNDSCALEPHZ128rmbi = 20917,
  VRNDSCALEPHZ128rmbik = 20918,
  VRNDSCALEPHZ128rmbikz = 20919,
  VRNDSCALEPHZ128rmi = 20920,
  VRNDSCALEPHZ128rmik = 20921,
  VRNDSCALEPHZ128rmikz = 20922,
  VRNDSCALEPHZ128rri = 20923,
  VRNDSCALEPHZ128rrik = 20924,
  VRNDSCALEPHZ128rrikz = 20925,
  VRNDSCALEPHZ256rmbi = 20926,
  VRNDSCALEPHZ256rmbik = 20927,
  VRNDSCALEPHZ256rmbikz = 20928,
  VRNDSCALEPHZ256rmi = 20929,
  VRNDSCALEPHZ256rmik = 20930,
  VRNDSCALEPHZ256rmikz = 20931,
  VRNDSCALEPHZ256rri = 20932,
  VRNDSCALEPHZ256rrik = 20933,
  VRNDSCALEPHZ256rrikz = 20934,
  VRNDSCALEPHZrmbi = 20935,
  VRNDSCALEPHZrmbik = 20936,
  VRNDSCALEPHZrmbikz = 20937,
  VRNDSCALEPHZrmi = 20938,
  VRNDSCALEPHZrmik = 20939,
  VRNDSCALEPHZrmikz = 20940,
  VRNDSCALEPHZrri = 20941,
  VRNDSCALEPHZrrib = 20942,
  VRNDSCALEPHZrribk = 20943,
  VRNDSCALEPHZrribkz = 20944,
  VRNDSCALEPHZrrik = 20945,
  VRNDSCALEPHZrrikz = 20946,
  VRNDSCALEPSZ128rmbi = 20947,
  VRNDSCALEPSZ128rmbik = 20948,
  VRNDSCALEPSZ128rmbikz = 20949,
  VRNDSCALEPSZ128rmi = 20950,
  VRNDSCALEPSZ128rmik = 20951,
  VRNDSCALEPSZ128rmikz = 20952,
  VRNDSCALEPSZ128rri = 20953,
  VRNDSCALEPSZ128rrik = 20954,
  VRNDSCALEPSZ128rrikz = 20955,
  VRNDSCALEPSZ256rmbi = 20956,
  VRNDSCALEPSZ256rmbik = 20957,
  VRNDSCALEPSZ256rmbikz = 20958,
  VRNDSCALEPSZ256rmi = 20959,
  VRNDSCALEPSZ256rmik = 20960,
  VRNDSCALEPSZ256rmikz = 20961,
  VRNDSCALEPSZ256rri = 20962,
  VRNDSCALEPSZ256rrik = 20963,
  VRNDSCALEPSZ256rrikz = 20964,
  VRNDSCALEPSZrmbi = 20965,
  VRNDSCALEPSZrmbik = 20966,
  VRNDSCALEPSZrmbikz = 20967,
  VRNDSCALEPSZrmi = 20968,
  VRNDSCALEPSZrmik = 20969,
  VRNDSCALEPSZrmikz = 20970,
  VRNDSCALEPSZrri = 20971,
  VRNDSCALEPSZrrib = 20972,
  VRNDSCALEPSZrribk = 20973,
  VRNDSCALEPSZrribkz = 20974,
  VRNDSCALEPSZrrik = 20975,
  VRNDSCALEPSZrrikz = 20976,
  VRNDSCALESDZrmi = 20977,
  VRNDSCALESDZrmi_Int = 20978,
  VRNDSCALESDZrmik_Int = 20979,
  VRNDSCALESDZrmikz_Int = 20980,
  VRNDSCALESDZrri = 20981,
  VRNDSCALESDZrri_Int = 20982,
  VRNDSCALESDZrrib_Int = 20983,
  VRNDSCALESDZrribk_Int = 20984,
  VRNDSCALESDZrribkz_Int = 20985,
  VRNDSCALESDZrrik_Int = 20986,
  VRNDSCALESDZrrikz_Int = 20987,
  VRNDSCALESHZrmi = 20988,
  VRNDSCALESHZrmi_Int = 20989,
  VRNDSCALESHZrmik_Int = 20990,
  VRNDSCALESHZrmikz_Int = 20991,
  VRNDSCALESHZrri = 20992,
  VRNDSCALESHZrri_Int = 20993,
  VRNDSCALESHZrrib_Int = 20994,
  VRNDSCALESHZrribk_Int = 20995,
  VRNDSCALESHZrribkz_Int = 20996,
  VRNDSCALESHZrrik_Int = 20997,
  VRNDSCALESHZrrikz_Int = 20998,
  VRNDSCALESSZrmi = 20999,
  VRNDSCALESSZrmi_Int = 21000,
  VRNDSCALESSZrmik_Int = 21001,
  VRNDSCALESSZrmikz_Int = 21002,
  VRNDSCALESSZrri = 21003,
  VRNDSCALESSZrri_Int = 21004,
  VRNDSCALESSZrrib_Int = 21005,
  VRNDSCALESSZrribk_Int = 21006,
  VRNDSCALESSZrribkz_Int = 21007,
  VRNDSCALESSZrrik_Int = 21008,
  VRNDSCALESSZrrikz_Int = 21009,
  VROUNDPDYmi = 21010,
  VROUNDPDYri = 21011,
  VROUNDPDmi = 21012,
  VROUNDPDri = 21013,
  VROUNDPSYmi = 21014,
  VROUNDPSYri = 21015,
  VROUNDPSmi = 21016,
  VROUNDPSri = 21017,
  VROUNDSDmi = 21018,
  VROUNDSDmi_Int = 21019,
  VROUNDSDri = 21020,
  VROUNDSDri_Int = 21021,
  VROUNDSSmi = 21022,
  VROUNDSSmi_Int = 21023,
  VROUNDSSri = 21024,
  VROUNDSSri_Int = 21025,
  VRSQRT14PDZ128m = 21026,
  VRSQRT14PDZ128mb = 21027,
  VRSQRT14PDZ128mbk = 21028,
  VRSQRT14PDZ128mbkz = 21029,
  VRSQRT14PDZ128mk = 21030,
  VRSQRT14PDZ128mkz = 21031,
  VRSQRT14PDZ128r = 21032,
  VRSQRT14PDZ128rk = 21033,
  VRSQRT14PDZ128rkz = 21034,
  VRSQRT14PDZ256m = 21035,
  VRSQRT14PDZ256mb = 21036,
  VRSQRT14PDZ256mbk = 21037,
  VRSQRT14PDZ256mbkz = 21038,
  VRSQRT14PDZ256mk = 21039,
  VRSQRT14PDZ256mkz = 21040,
  VRSQRT14PDZ256r = 21041,
  VRSQRT14PDZ256rk = 21042,
  VRSQRT14PDZ256rkz = 21043,
  VRSQRT14PDZm = 21044,
  VRSQRT14PDZmb = 21045,
  VRSQRT14PDZmbk = 21046,
  VRSQRT14PDZmbkz = 21047,
  VRSQRT14PDZmk = 21048,
  VRSQRT14PDZmkz = 21049,
  VRSQRT14PDZr = 21050,
  VRSQRT14PDZrk = 21051,
  VRSQRT14PDZrkz = 21052,
  VRSQRT14PSZ128m = 21053,
  VRSQRT14PSZ128mb = 21054,
  VRSQRT14PSZ128mbk = 21055,
  VRSQRT14PSZ128mbkz = 21056,
  VRSQRT14PSZ128mk = 21057,
  VRSQRT14PSZ128mkz = 21058,
  VRSQRT14PSZ128r = 21059,
  VRSQRT14PSZ128rk = 21060,
  VRSQRT14PSZ128rkz = 21061,
  VRSQRT14PSZ256m = 21062,
  VRSQRT14PSZ256mb = 21063,
  VRSQRT14PSZ256mbk = 21064,
  VRSQRT14PSZ256mbkz = 21065,
  VRSQRT14PSZ256mk = 21066,
  VRSQRT14PSZ256mkz = 21067,
  VRSQRT14PSZ256r = 21068,
  VRSQRT14PSZ256rk = 21069,
  VRSQRT14PSZ256rkz = 21070,
  VRSQRT14PSZm = 21071,
  VRSQRT14PSZmb = 21072,
  VRSQRT14PSZmbk = 21073,
  VRSQRT14PSZmbkz = 21074,
  VRSQRT14PSZmk = 21075,
  VRSQRT14PSZmkz = 21076,
  VRSQRT14PSZr = 21077,
  VRSQRT14PSZrk = 21078,
  VRSQRT14PSZrkz = 21079,
  VRSQRT14SDZrm = 21080,
  VRSQRT14SDZrmk = 21081,
  VRSQRT14SDZrmkz = 21082,
  VRSQRT14SDZrr = 21083,
  VRSQRT14SDZrrk = 21084,
  VRSQRT14SDZrrkz = 21085,
  VRSQRT14SSZrm = 21086,
  VRSQRT14SSZrmk = 21087,
  VRSQRT14SSZrmkz = 21088,
  VRSQRT14SSZrr = 21089,
  VRSQRT14SSZrrk = 21090,
  VRSQRT14SSZrrkz = 21091,
  VRSQRT28PDZm = 21092,
  VRSQRT28PDZmb = 21093,
  VRSQRT28PDZmbk = 21094,
  VRSQRT28PDZmbkz = 21095,
  VRSQRT28PDZmk = 21096,
  VRSQRT28PDZmkz = 21097,
  VRSQRT28PDZr = 21098,
  VRSQRT28PDZrb = 21099,
  VRSQRT28PDZrbk = 21100,
  VRSQRT28PDZrbkz = 21101,
  VRSQRT28PDZrk = 21102,
  VRSQRT28PDZrkz = 21103,
  VRSQRT28PSZm = 21104,
  VRSQRT28PSZmb = 21105,
  VRSQRT28PSZmbk = 21106,
  VRSQRT28PSZmbkz = 21107,
  VRSQRT28PSZmk = 21108,
  VRSQRT28PSZmkz = 21109,
  VRSQRT28PSZr = 21110,
  VRSQRT28PSZrb = 21111,
  VRSQRT28PSZrbk = 21112,
  VRSQRT28PSZrbkz = 21113,
  VRSQRT28PSZrk = 21114,
  VRSQRT28PSZrkz = 21115,
  VRSQRT28SDZm = 21116,
  VRSQRT28SDZmk = 21117,
  VRSQRT28SDZmkz = 21118,
  VRSQRT28SDZr = 21119,
  VRSQRT28SDZrb = 21120,
  VRSQRT28SDZrbk = 21121,
  VRSQRT28SDZrbkz = 21122,
  VRSQRT28SDZrk = 21123,
  VRSQRT28SDZrkz = 21124,
  VRSQRT28SSZm = 21125,
  VRSQRT28SSZmk = 21126,
  VRSQRT28SSZmkz = 21127,
  VRSQRT28SSZr = 21128,
  VRSQRT28SSZrb = 21129,
  VRSQRT28SSZrbk = 21130,
  VRSQRT28SSZrbkz = 21131,
  VRSQRT28SSZrk = 21132,
  VRSQRT28SSZrkz = 21133,
  VRSQRTBF16Z128m = 21134,
  VRSQRTBF16Z128mb = 21135,
  VRSQRTBF16Z128mbk = 21136,
  VRSQRTBF16Z128mbkz = 21137,
  VRSQRTBF16Z128mk = 21138,
  VRSQRTBF16Z128mkz = 21139,
  VRSQRTBF16Z128r = 21140,
  VRSQRTBF16Z128rk = 21141,
  VRSQRTBF16Z128rkz = 21142,
  VRSQRTBF16Z256m = 21143,
  VRSQRTBF16Z256mb = 21144,
  VRSQRTBF16Z256mbk = 21145,
  VRSQRTBF16Z256mbkz = 21146,
  VRSQRTBF16Z256mk = 21147,
  VRSQRTBF16Z256mkz = 21148,
  VRSQRTBF16Z256r = 21149,
  VRSQRTBF16Z256rk = 21150,
  VRSQRTBF16Z256rkz = 21151,
  VRSQRTBF16Zm = 21152,
  VRSQRTBF16Zmb = 21153,
  VRSQRTBF16Zmbk = 21154,
  VRSQRTBF16Zmbkz = 21155,
  VRSQRTBF16Zmk = 21156,
  VRSQRTBF16Zmkz = 21157,
  VRSQRTBF16Zr = 21158,
  VRSQRTBF16Zrk = 21159,
  VRSQRTBF16Zrkz = 21160,
  VRSQRTPHZ128m = 21161,
  VRSQRTPHZ128mb = 21162,
  VRSQRTPHZ128mbk = 21163,
  VRSQRTPHZ128mbkz = 21164,
  VRSQRTPHZ128mk = 21165,
  VRSQRTPHZ128mkz = 21166,
  VRSQRTPHZ128r = 21167,
  VRSQRTPHZ128rk = 21168,
  VRSQRTPHZ128rkz = 21169,
  VRSQRTPHZ256m = 21170,
  VRSQRTPHZ256mb = 21171,
  VRSQRTPHZ256mbk = 21172,
  VRSQRTPHZ256mbkz = 21173,
  VRSQRTPHZ256mk = 21174,
  VRSQRTPHZ256mkz = 21175,
  VRSQRTPHZ256r = 21176,
  VRSQRTPHZ256rk = 21177,
  VRSQRTPHZ256rkz = 21178,
  VRSQRTPHZm = 21179,
  VRSQRTPHZmb = 21180,
  VRSQRTPHZmbk = 21181,
  VRSQRTPHZmbkz = 21182,
  VRSQRTPHZmk = 21183,
  VRSQRTPHZmkz = 21184,
  VRSQRTPHZr = 21185,
  VRSQRTPHZrk = 21186,
  VRSQRTPHZrkz = 21187,
  VRSQRTPSYm = 21188,
  VRSQRTPSYr = 21189,
  VRSQRTPSm = 21190,
  VRSQRTPSr = 21191,
  VRSQRTSHZrm = 21192,
  VRSQRTSHZrmk = 21193,
  VRSQRTSHZrmkz = 21194,
  VRSQRTSHZrr = 21195,
  VRSQRTSHZrrk = 21196,
  VRSQRTSHZrrkz = 21197,
  VRSQRTSSm = 21198,
  VRSQRTSSm_Int = 21199,
  VRSQRTSSr = 21200,
  VRSQRTSSr_Int = 21201,
  VSCALEFBF16Z128rm = 21202,
  VSCALEFBF16Z128rmb = 21203,
  VSCALEFBF16Z128rmbk = 21204,
  VSCALEFBF16Z128rmbkz = 21205,
  VSCALEFBF16Z128rmk = 21206,
  VSCALEFBF16Z128rmkz = 21207,
  VSCALEFBF16Z128rr = 21208,
  VSCALEFBF16Z128rrk = 21209,
  VSCALEFBF16Z128rrkz = 21210,
  VSCALEFBF16Z256rm = 21211,
  VSCALEFBF16Z256rmb = 21212,
  VSCALEFBF16Z256rmbk = 21213,
  VSCALEFBF16Z256rmbkz = 21214,
  VSCALEFBF16Z256rmk = 21215,
  VSCALEFBF16Z256rmkz = 21216,
  VSCALEFBF16Z256rr = 21217,
  VSCALEFBF16Z256rrk = 21218,
  VSCALEFBF16Z256rrkz = 21219,
  VSCALEFBF16Zrm = 21220,
  VSCALEFBF16Zrmb = 21221,
  VSCALEFBF16Zrmbk = 21222,
  VSCALEFBF16Zrmbkz = 21223,
  VSCALEFBF16Zrmk = 21224,
  VSCALEFBF16Zrmkz = 21225,
  VSCALEFBF16Zrr = 21226,
  VSCALEFBF16Zrrk = 21227,
  VSCALEFBF16Zrrkz = 21228,
  VSCALEFPDZ128rm = 21229,
  VSCALEFPDZ128rmb = 21230,
  VSCALEFPDZ128rmbk = 21231,
  VSCALEFPDZ128rmbkz = 21232,
  VSCALEFPDZ128rmk = 21233,
  VSCALEFPDZ128rmkz = 21234,
  VSCALEFPDZ128rr = 21235,
  VSCALEFPDZ128rrk = 21236,
  VSCALEFPDZ128rrkz = 21237,
  VSCALEFPDZ256rm = 21238,
  VSCALEFPDZ256rmb = 21239,
  VSCALEFPDZ256rmbk = 21240,
  VSCALEFPDZ256rmbkz = 21241,
  VSCALEFPDZ256rmk = 21242,
  VSCALEFPDZ256rmkz = 21243,
  VSCALEFPDZ256rr = 21244,
  VSCALEFPDZ256rrk = 21245,
  VSCALEFPDZ256rrkz = 21246,
  VSCALEFPDZrm = 21247,
  VSCALEFPDZrmb = 21248,
  VSCALEFPDZrmbk = 21249,
  VSCALEFPDZrmbkz = 21250,
  VSCALEFPDZrmk = 21251,
  VSCALEFPDZrmkz = 21252,
  VSCALEFPDZrr = 21253,
  VSCALEFPDZrrb = 21254,
  VSCALEFPDZrrbk = 21255,
  VSCALEFPDZrrbkz = 21256,
  VSCALEFPDZrrk = 21257,
  VSCALEFPDZrrkz = 21258,
  VSCALEFPHZ128rm = 21259,
  VSCALEFPHZ128rmb = 21260,
  VSCALEFPHZ128rmbk = 21261,
  VSCALEFPHZ128rmbkz = 21262,
  VSCALEFPHZ128rmk = 21263,
  VSCALEFPHZ128rmkz = 21264,
  VSCALEFPHZ128rr = 21265,
  VSCALEFPHZ128rrk = 21266,
  VSCALEFPHZ128rrkz = 21267,
  VSCALEFPHZ256rm = 21268,
  VSCALEFPHZ256rmb = 21269,
  VSCALEFPHZ256rmbk = 21270,
  VSCALEFPHZ256rmbkz = 21271,
  VSCALEFPHZ256rmk = 21272,
  VSCALEFPHZ256rmkz = 21273,
  VSCALEFPHZ256rr = 21274,
  VSCALEFPHZ256rrk = 21275,
  VSCALEFPHZ256rrkz = 21276,
  VSCALEFPHZrm = 21277,
  VSCALEFPHZrmb = 21278,
  VSCALEFPHZrmbk = 21279,
  VSCALEFPHZrmbkz = 21280,
  VSCALEFPHZrmk = 21281,
  VSCALEFPHZrmkz = 21282,
  VSCALEFPHZrr = 21283,
  VSCALEFPHZrrb = 21284,
  VSCALEFPHZrrbk = 21285,
  VSCALEFPHZrrbkz = 21286,
  VSCALEFPHZrrk = 21287,
  VSCALEFPHZrrkz = 21288,
  VSCALEFPSZ128rm = 21289,
  VSCALEFPSZ128rmb = 21290,
  VSCALEFPSZ128rmbk = 21291,
  VSCALEFPSZ128rmbkz = 21292,
  VSCALEFPSZ128rmk = 21293,
  VSCALEFPSZ128rmkz = 21294,
  VSCALEFPSZ128rr = 21295,
  VSCALEFPSZ128rrk = 21296,
  VSCALEFPSZ128rrkz = 21297,
  VSCALEFPSZ256rm = 21298,
  VSCALEFPSZ256rmb = 21299,
  VSCALEFPSZ256rmbk = 21300,
  VSCALEFPSZ256rmbkz = 21301,
  VSCALEFPSZ256rmk = 21302,
  VSCALEFPSZ256rmkz = 21303,
  VSCALEFPSZ256rr = 21304,
  VSCALEFPSZ256rrk = 21305,
  VSCALEFPSZ256rrkz = 21306,
  VSCALEFPSZrm = 21307,
  VSCALEFPSZrmb = 21308,
  VSCALEFPSZrmbk = 21309,
  VSCALEFPSZrmbkz = 21310,
  VSCALEFPSZrmk = 21311,
  VSCALEFPSZrmkz = 21312,
  VSCALEFPSZrr = 21313,
  VSCALEFPSZrrb = 21314,
  VSCALEFPSZrrbk = 21315,
  VSCALEFPSZrrbkz = 21316,
  VSCALEFPSZrrk = 21317,
  VSCALEFPSZrrkz = 21318,
  VSCALEFSDZrm = 21319,
  VSCALEFSDZrmk = 21320,
  VSCALEFSDZrmkz = 21321,
  VSCALEFSDZrr = 21322,
  VSCALEFSDZrrb_Int = 21323,
  VSCALEFSDZrrbk_Int = 21324,
  VSCALEFSDZrrbkz_Int = 21325,
  VSCALEFSDZrrk = 21326,
  VSCALEFSDZrrkz = 21327,
  VSCALEFSHZrm = 21328,
  VSCALEFSHZrmk = 21329,
  VSCALEFSHZrmkz = 21330,
  VSCALEFSHZrr = 21331,
  VSCALEFSHZrrb_Int = 21332,
  VSCALEFSHZrrbk_Int = 21333,
  VSCALEFSHZrrbkz_Int = 21334,
  VSCALEFSHZrrk = 21335,
  VSCALEFSHZrrkz = 21336,
  VSCALEFSSZrm = 21337,
  VSCALEFSSZrmk = 21338,
  VSCALEFSSZrmkz = 21339,
  VSCALEFSSZrr = 21340,
  VSCALEFSSZrrb_Int = 21341,
  VSCALEFSSZrrbk_Int = 21342,
  VSCALEFSSZrrbkz_Int = 21343,
  VSCALEFSSZrrk = 21344,
  VSCALEFSSZrrkz = 21345,
  VSCATTERDPDZ128mr = 21346,
  VSCATTERDPDZ256mr = 21347,
  VSCATTERDPDZmr = 21348,
  VSCATTERDPSZ128mr = 21349,
  VSCATTERDPSZ256mr = 21350,
  VSCATTERDPSZmr = 21351,
  VSCATTERPF0DPDm = 21352,
  VSCATTERPF0DPSm = 21353,
  VSCATTERPF0QPDm = 21354,
  VSCATTERPF0QPSm = 21355,
  VSCATTERPF1DPDm = 21356,
  VSCATTERPF1DPSm = 21357,
  VSCATTERPF1QPDm = 21358,
  VSCATTERPF1QPSm = 21359,
  VSCATTERQPDZ128mr = 21360,
  VSCATTERQPDZ256mr = 21361,
  VSCATTERQPDZmr = 21362,
  VSCATTERQPSZ128mr = 21363,
  VSCATTERQPSZ256mr = 21364,
  VSCATTERQPSZmr = 21365,
  VSHA512MSG1rr = 21366,
  VSHA512MSG2rr = 21367,
  VSHA512RNDS2rr = 21368,
  VSHUFF32X4Z256rmbi = 21369,
  VSHUFF32X4Z256rmbik = 21370,
  VSHUFF32X4Z256rmbikz = 21371,
  VSHUFF32X4Z256rmi = 21372,
  VSHUFF32X4Z256rmik = 21373,
  VSHUFF32X4Z256rmikz = 21374,
  VSHUFF32X4Z256rri = 21375,
  VSHUFF32X4Z256rrik = 21376,
  VSHUFF32X4Z256rrikz = 21377,
  VSHUFF32X4Zrmbi = 21378,
  VSHUFF32X4Zrmbik = 21379,
  VSHUFF32X4Zrmbikz = 21380,
  VSHUFF32X4Zrmi = 21381,
  VSHUFF32X4Zrmik = 21382,
  VSHUFF32X4Zrmikz = 21383,
  VSHUFF32X4Zrri = 21384,
  VSHUFF32X4Zrrik = 21385,
  VSHUFF32X4Zrrikz = 21386,
  VSHUFF64X2Z256rmbi = 21387,
  VSHUFF64X2Z256rmbik = 21388,
  VSHUFF64X2Z256rmbikz = 21389,
  VSHUFF64X2Z256rmi = 21390,
  VSHUFF64X2Z256rmik = 21391,
  VSHUFF64X2Z256rmikz = 21392,
  VSHUFF64X2Z256rri = 21393,
  VSHUFF64X2Z256rrik = 21394,
  VSHUFF64X2Z256rrikz = 21395,
  VSHUFF64X2Zrmbi = 21396,
  VSHUFF64X2Zrmbik = 21397,
  VSHUFF64X2Zrmbikz = 21398,
  VSHUFF64X2Zrmi = 21399,
  VSHUFF64X2Zrmik = 21400,
  VSHUFF64X2Zrmikz = 21401,
  VSHUFF64X2Zrri = 21402,
  VSHUFF64X2Zrrik = 21403,
  VSHUFF64X2Zrrikz = 21404,
  VSHUFI32X4Z256rmbi = 21405,
  VSHUFI32X4Z256rmbik = 21406,
  VSHUFI32X4Z256rmbikz = 21407,
  VSHUFI32X4Z256rmi = 21408,
  VSHUFI32X4Z256rmik = 21409,
  VSHUFI32X4Z256rmikz = 21410,
  VSHUFI32X4Z256rri = 21411,
  VSHUFI32X4Z256rrik = 21412,
  VSHUFI32X4Z256rrikz = 21413,
  VSHUFI32X4Zrmbi = 21414,
  VSHUFI32X4Zrmbik = 21415,
  VSHUFI32X4Zrmbikz = 21416,
  VSHUFI32X4Zrmi = 21417,
  VSHUFI32X4Zrmik = 21418,
  VSHUFI32X4Zrmikz = 21419,
  VSHUFI32X4Zrri = 21420,
  VSHUFI32X4Zrrik = 21421,
  VSHUFI32X4Zrrikz = 21422,
  VSHUFI64X2Z256rmbi = 21423,
  VSHUFI64X2Z256rmbik = 21424,
  VSHUFI64X2Z256rmbikz = 21425,
  VSHUFI64X2Z256rmi = 21426,
  VSHUFI64X2Z256rmik = 21427,
  VSHUFI64X2Z256rmikz = 21428,
  VSHUFI64X2Z256rri = 21429,
  VSHUFI64X2Z256rrik = 21430,
  VSHUFI64X2Z256rrikz = 21431,
  VSHUFI64X2Zrmbi = 21432,
  VSHUFI64X2Zrmbik = 21433,
  VSHUFI64X2Zrmbikz = 21434,
  VSHUFI64X2Zrmi = 21435,
  VSHUFI64X2Zrmik = 21436,
  VSHUFI64X2Zrmikz = 21437,
  VSHUFI64X2Zrri = 21438,
  VSHUFI64X2Zrrik = 21439,
  VSHUFI64X2Zrrikz = 21440,
  VSHUFPDYrmi = 21441,
  VSHUFPDYrri = 21442,
  VSHUFPDZ128rmbi = 21443,
  VSHUFPDZ128rmbik = 21444,
  VSHUFPDZ128rmbikz = 21445,
  VSHUFPDZ128rmi = 21446,
  VSHUFPDZ128rmik = 21447,
  VSHUFPDZ128rmikz = 21448,
  VSHUFPDZ128rri = 21449,
  VSHUFPDZ128rrik = 21450,
  VSHUFPDZ128rrikz = 21451,
  VSHUFPDZ256rmbi = 21452,
  VSHUFPDZ256rmbik = 21453,
  VSHUFPDZ256rmbikz = 21454,
  VSHUFPDZ256rmi = 21455,
  VSHUFPDZ256rmik = 21456,
  VSHUFPDZ256rmikz = 21457,
  VSHUFPDZ256rri = 21458,
  VSHUFPDZ256rrik = 21459,
  VSHUFPDZ256rrikz = 21460,
  VSHUFPDZrmbi = 21461,
  VSHUFPDZrmbik = 21462,
  VSHUFPDZrmbikz = 21463,
  VSHUFPDZrmi = 21464,
  VSHUFPDZrmik = 21465,
  VSHUFPDZrmikz = 21466,
  VSHUFPDZrri = 21467,
  VSHUFPDZrrik = 21468,
  VSHUFPDZrrikz = 21469,
  VSHUFPDrmi = 21470,
  VSHUFPDrri = 21471,
  VSHUFPSYrmi = 21472,
  VSHUFPSYrri = 21473,
  VSHUFPSZ128rmbi = 21474,
  VSHUFPSZ128rmbik = 21475,
  VSHUFPSZ128rmbikz = 21476,
  VSHUFPSZ128rmi = 21477,
  VSHUFPSZ128rmik = 21478,
  VSHUFPSZ128rmikz = 21479,
  VSHUFPSZ128rri = 21480,
  VSHUFPSZ128rrik = 21481,
  VSHUFPSZ128rrikz = 21482,
  VSHUFPSZ256rmbi = 21483,
  VSHUFPSZ256rmbik = 21484,
  VSHUFPSZ256rmbikz = 21485,
  VSHUFPSZ256rmi = 21486,
  VSHUFPSZ256rmik = 21487,
  VSHUFPSZ256rmikz = 21488,
  VSHUFPSZ256rri = 21489,
  VSHUFPSZ256rrik = 21490,
  VSHUFPSZ256rrikz = 21491,
  VSHUFPSZrmbi = 21492,
  VSHUFPSZrmbik = 21493,
  VSHUFPSZrmbikz = 21494,
  VSHUFPSZrmi = 21495,
  VSHUFPSZrmik = 21496,
  VSHUFPSZrmikz = 21497,
  VSHUFPSZrri = 21498,
  VSHUFPSZrrik = 21499,
  VSHUFPSZrrikz = 21500,
  VSHUFPSrmi = 21501,
  VSHUFPSrri = 21502,
  VSM3MSG1rm = 21503,
  VSM3MSG1rr = 21504,
  VSM3MSG2rm = 21505,
  VSM3MSG2rr = 21506,
  VSM3RNDS2rmi = 21507,
  VSM3RNDS2rri = 21508,
  VSM4KEY4Yrm = 21509,
  VSM4KEY4Yrr = 21510,
  VSM4KEY4Z128rm = 21511,
  VSM4KEY4Z128rr = 21512,
  VSM4KEY4Z256rm = 21513,
  VSM4KEY4Z256rr = 21514,
  VSM4KEY4Zrm = 21515,
  VSM4KEY4Zrr = 21516,
  VSM4KEY4rm = 21517,
  VSM4KEY4rr = 21518,
  VSM4RNDS4Yrm = 21519,
  VSM4RNDS4Yrr = 21520,
  VSM4RNDS4Z128rm = 21521,
  VSM4RNDS4Z128rr = 21522,
  VSM4RNDS4Z256rm = 21523,
  VSM4RNDS4Z256rr = 21524,
  VSM4RNDS4Zrm = 21525,
  VSM4RNDS4Zrr = 21526,
  VSM4RNDS4rm = 21527,
  VSM4RNDS4rr = 21528,
  VSQRTBF16Z128m = 21529,
  VSQRTBF16Z128mb = 21530,
  VSQRTBF16Z128mbk = 21531,
  VSQRTBF16Z128mbkz = 21532,
  VSQRTBF16Z128mk = 21533,
  VSQRTBF16Z128mkz = 21534,
  VSQRTBF16Z128r = 21535,
  VSQRTBF16Z128rk = 21536,
  VSQRTBF16Z128rkz = 21537,
  VSQRTBF16Z256m = 21538,
  VSQRTBF16Z256mb = 21539,
  VSQRTBF16Z256mbk = 21540,
  VSQRTBF16Z256mbkz = 21541,
  VSQRTBF16Z256mk = 21542,
  VSQRTBF16Z256mkz = 21543,
  VSQRTBF16Z256r = 21544,
  VSQRTBF16Z256rk = 21545,
  VSQRTBF16Z256rkz = 21546,
  VSQRTBF16Zm = 21547,
  VSQRTBF16Zmb = 21548,
  VSQRTBF16Zmbk = 21549,
  VSQRTBF16Zmbkz = 21550,
  VSQRTBF16Zmk = 21551,
  VSQRTBF16Zmkz = 21552,
  VSQRTBF16Zr = 21553,
  VSQRTBF16Zrk = 21554,
  VSQRTBF16Zrkz = 21555,
  VSQRTPDYm = 21556,
  VSQRTPDYr = 21557,
  VSQRTPDZ128m = 21558,
  VSQRTPDZ128mb = 21559,
  VSQRTPDZ128mbk = 21560,
  VSQRTPDZ128mbkz = 21561,
  VSQRTPDZ128mk = 21562,
  VSQRTPDZ128mkz = 21563,
  VSQRTPDZ128r = 21564,
  VSQRTPDZ128rk = 21565,
  VSQRTPDZ128rkz = 21566,
  VSQRTPDZ256m = 21567,
  VSQRTPDZ256mb = 21568,
  VSQRTPDZ256mbk = 21569,
  VSQRTPDZ256mbkz = 21570,
  VSQRTPDZ256mk = 21571,
  VSQRTPDZ256mkz = 21572,
  VSQRTPDZ256r = 21573,
  VSQRTPDZ256rk = 21574,
  VSQRTPDZ256rkz = 21575,
  VSQRTPDZm = 21576,
  VSQRTPDZmb = 21577,
  VSQRTPDZmbk = 21578,
  VSQRTPDZmbkz = 21579,
  VSQRTPDZmk = 21580,
  VSQRTPDZmkz = 21581,
  VSQRTPDZr = 21582,
  VSQRTPDZrb = 21583,
  VSQRTPDZrbk = 21584,
  VSQRTPDZrbkz = 21585,
  VSQRTPDZrk = 21586,
  VSQRTPDZrkz = 21587,
  VSQRTPDm = 21588,
  VSQRTPDr = 21589,
  VSQRTPHZ128m = 21590,
  VSQRTPHZ128mb = 21591,
  VSQRTPHZ128mbk = 21592,
  VSQRTPHZ128mbkz = 21593,
  VSQRTPHZ128mk = 21594,
  VSQRTPHZ128mkz = 21595,
  VSQRTPHZ128r = 21596,
  VSQRTPHZ128rk = 21597,
  VSQRTPHZ128rkz = 21598,
  VSQRTPHZ256m = 21599,
  VSQRTPHZ256mb = 21600,
  VSQRTPHZ256mbk = 21601,
  VSQRTPHZ256mbkz = 21602,
  VSQRTPHZ256mk = 21603,
  VSQRTPHZ256mkz = 21604,
  VSQRTPHZ256r = 21605,
  VSQRTPHZ256rk = 21606,
  VSQRTPHZ256rkz = 21607,
  VSQRTPHZm = 21608,
  VSQRTPHZmb = 21609,
  VSQRTPHZmbk = 21610,
  VSQRTPHZmbkz = 21611,
  VSQRTPHZmk = 21612,
  VSQRTPHZmkz = 21613,
  VSQRTPHZr = 21614,
  VSQRTPHZrb = 21615,
  VSQRTPHZrbk = 21616,
  VSQRTPHZrbkz = 21617,
  VSQRTPHZrk = 21618,
  VSQRTPHZrkz = 21619,
  VSQRTPSYm = 21620,
  VSQRTPSYr = 21621,
  VSQRTPSZ128m = 21622,
  VSQRTPSZ128mb = 21623,
  VSQRTPSZ128mbk = 21624,
  VSQRTPSZ128mbkz = 21625,
  VSQRTPSZ128mk = 21626,
  VSQRTPSZ128mkz = 21627,
  VSQRTPSZ128r = 21628,
  VSQRTPSZ128rk = 21629,
  VSQRTPSZ128rkz = 21630,
  VSQRTPSZ256m = 21631,
  VSQRTPSZ256mb = 21632,
  VSQRTPSZ256mbk = 21633,
  VSQRTPSZ256mbkz = 21634,
  VSQRTPSZ256mk = 21635,
  VSQRTPSZ256mkz = 21636,
  VSQRTPSZ256r = 21637,
  VSQRTPSZ256rk = 21638,
  VSQRTPSZ256rkz = 21639,
  VSQRTPSZm = 21640,
  VSQRTPSZmb = 21641,
  VSQRTPSZmbk = 21642,
  VSQRTPSZmbkz = 21643,
  VSQRTPSZmk = 21644,
  VSQRTPSZmkz = 21645,
  VSQRTPSZr = 21646,
  VSQRTPSZrb = 21647,
  VSQRTPSZrbk = 21648,
  VSQRTPSZrbkz = 21649,
  VSQRTPSZrk = 21650,
  VSQRTPSZrkz = 21651,
  VSQRTPSm = 21652,
  VSQRTPSr = 21653,
  VSQRTSDZm = 21654,
  VSQRTSDZm_Int = 21655,
  VSQRTSDZmk_Int = 21656,
  VSQRTSDZmkz_Int = 21657,
  VSQRTSDZr = 21658,
  VSQRTSDZr_Int = 21659,
  VSQRTSDZrb_Int = 21660,
  VSQRTSDZrbk_Int = 21661,
  VSQRTSDZrbkz_Int = 21662,
  VSQRTSDZrk_Int = 21663,
  VSQRTSDZrkz_Int = 21664,
  VSQRTSDm = 21665,
  VSQRTSDm_Int = 21666,
  VSQRTSDr = 21667,
  VSQRTSDr_Int = 21668,
  VSQRTSHZm = 21669,
  VSQRTSHZm_Int = 21670,
  VSQRTSHZmk_Int = 21671,
  VSQRTSHZmkz_Int = 21672,
  VSQRTSHZr = 21673,
  VSQRTSHZr_Int = 21674,
  VSQRTSHZrb_Int = 21675,
  VSQRTSHZrbk_Int = 21676,
  VSQRTSHZrbkz_Int = 21677,
  VSQRTSHZrk_Int = 21678,
  VSQRTSHZrkz_Int = 21679,
  VSQRTSSZm = 21680,
  VSQRTSSZm_Int = 21681,
  VSQRTSSZmk_Int = 21682,
  VSQRTSSZmkz_Int = 21683,
  VSQRTSSZr = 21684,
  VSQRTSSZr_Int = 21685,
  VSQRTSSZrb_Int = 21686,
  VSQRTSSZrbk_Int = 21687,
  VSQRTSSZrbkz_Int = 21688,
  VSQRTSSZrk_Int = 21689,
  VSQRTSSZrkz_Int = 21690,
  VSQRTSSm = 21691,
  VSQRTSSm_Int = 21692,
  VSQRTSSr = 21693,
  VSQRTSSr_Int = 21694,
  VSTMXCSR = 21695,
  VSUBBF16Z128rm = 21696,
  VSUBBF16Z128rmb = 21697,
  VSUBBF16Z128rmbk = 21698,
  VSUBBF16Z128rmbkz = 21699,
  VSUBBF16Z128rmk = 21700,
  VSUBBF16Z128rmkz = 21701,
  VSUBBF16Z128rr = 21702,
  VSUBBF16Z128rrk = 21703,
  VSUBBF16Z128rrkz = 21704,
  VSUBBF16Z256rm = 21705,
  VSUBBF16Z256rmb = 21706,
  VSUBBF16Z256rmbk = 21707,
  VSUBBF16Z256rmbkz = 21708,
  VSUBBF16Z256rmk = 21709,
  VSUBBF16Z256rmkz = 21710,
  VSUBBF16Z256rr = 21711,
  VSUBBF16Z256rrk = 21712,
  VSUBBF16Z256rrkz = 21713,
  VSUBBF16Zrm = 21714,
  VSUBBF16Zrmb = 21715,
  VSUBBF16Zrmbk = 21716,
  VSUBBF16Zrmbkz = 21717,
  VSUBBF16Zrmk = 21718,
  VSUBBF16Zrmkz = 21719,
  VSUBBF16Zrr = 21720,
  VSUBBF16Zrrk = 21721,
  VSUBBF16Zrrkz = 21722,
  VSUBPDYrm = 21723,
  VSUBPDYrr = 21724,
  VSUBPDZ128rm = 21725,
  VSUBPDZ128rmb = 21726,
  VSUBPDZ128rmbk = 21727,
  VSUBPDZ128rmbkz = 21728,
  VSUBPDZ128rmk = 21729,
  VSUBPDZ128rmkz = 21730,
  VSUBPDZ128rr = 21731,
  VSUBPDZ128rrk = 21732,
  VSUBPDZ128rrkz = 21733,
  VSUBPDZ256rm = 21734,
  VSUBPDZ256rmb = 21735,
  VSUBPDZ256rmbk = 21736,
  VSUBPDZ256rmbkz = 21737,
  VSUBPDZ256rmk = 21738,
  VSUBPDZ256rmkz = 21739,
  VSUBPDZ256rr = 21740,
  VSUBPDZ256rrk = 21741,
  VSUBPDZ256rrkz = 21742,
  VSUBPDZrm = 21743,
  VSUBPDZrmb = 21744,
  VSUBPDZrmbk = 21745,
  VSUBPDZrmbkz = 21746,
  VSUBPDZrmk = 21747,
  VSUBPDZrmkz = 21748,
  VSUBPDZrr = 21749,
  VSUBPDZrrb = 21750,
  VSUBPDZrrbk = 21751,
  VSUBPDZrrbkz = 21752,
  VSUBPDZrrk = 21753,
  VSUBPDZrrkz = 21754,
  VSUBPDrm = 21755,
  VSUBPDrr = 21756,
  VSUBPHZ128rm = 21757,
  VSUBPHZ128rmb = 21758,
  VSUBPHZ128rmbk = 21759,
  VSUBPHZ128rmbkz = 21760,
  VSUBPHZ128rmk = 21761,
  VSUBPHZ128rmkz = 21762,
  VSUBPHZ128rr = 21763,
  VSUBPHZ128rrk = 21764,
  VSUBPHZ128rrkz = 21765,
  VSUBPHZ256rm = 21766,
  VSUBPHZ256rmb = 21767,
  VSUBPHZ256rmbk = 21768,
  VSUBPHZ256rmbkz = 21769,
  VSUBPHZ256rmk = 21770,
  VSUBPHZ256rmkz = 21771,
  VSUBPHZ256rr = 21772,
  VSUBPHZ256rrk = 21773,
  VSUBPHZ256rrkz = 21774,
  VSUBPHZrm = 21775,
  VSUBPHZrmb = 21776,
  VSUBPHZrmbk = 21777,
  VSUBPHZrmbkz = 21778,
  VSUBPHZrmk = 21779,
  VSUBPHZrmkz = 21780,
  VSUBPHZrr = 21781,
  VSUBPHZrrb = 21782,
  VSUBPHZrrbk = 21783,
  VSUBPHZrrbkz = 21784,
  VSUBPHZrrk = 21785,
  VSUBPHZrrkz = 21786,
  VSUBPSYrm = 21787,
  VSUBPSYrr = 21788,
  VSUBPSZ128rm = 21789,
  VSUBPSZ128rmb = 21790,
  VSUBPSZ128rmbk = 21791,
  VSUBPSZ128rmbkz = 21792,
  VSUBPSZ128rmk = 21793,
  VSUBPSZ128rmkz = 21794,
  VSUBPSZ128rr = 21795,
  VSUBPSZ128rrk = 21796,
  VSUBPSZ128rrkz = 21797,
  VSUBPSZ256rm = 21798,
  VSUBPSZ256rmb = 21799,
  VSUBPSZ256rmbk = 21800,
  VSUBPSZ256rmbkz = 21801,
  VSUBPSZ256rmk = 21802,
  VSUBPSZ256rmkz = 21803,
  VSUBPSZ256rr = 21804,
  VSUBPSZ256rrk = 21805,
  VSUBPSZ256rrkz = 21806,
  VSUBPSZrm = 21807,
  VSUBPSZrmb = 21808,
  VSUBPSZrmbk = 21809,
  VSUBPSZrmbkz = 21810,
  VSUBPSZrmk = 21811,
  VSUBPSZrmkz = 21812,
  VSUBPSZrr = 21813,
  VSUBPSZrrb = 21814,
  VSUBPSZrrbk = 21815,
  VSUBPSZrrbkz = 21816,
  VSUBPSZrrk = 21817,
  VSUBPSZrrkz = 21818,
  VSUBPSrm = 21819,
  VSUBPSrr = 21820,
  VSUBSDZrm = 21821,
  VSUBSDZrm_Int = 21822,
  VSUBSDZrmk_Int = 21823,
  VSUBSDZrmkz_Int = 21824,
  VSUBSDZrr = 21825,
  VSUBSDZrr_Int = 21826,
  VSUBSDZrrb_Int = 21827,
  VSUBSDZrrbk_Int = 21828,
  VSUBSDZrrbkz_Int = 21829,
  VSUBSDZrrk_Int = 21830,
  VSUBSDZrrkz_Int = 21831,
  VSUBSDrm = 21832,
  VSUBSDrm_Int = 21833,
  VSUBSDrr = 21834,
  VSUBSDrr_Int = 21835,
  VSUBSHZrm = 21836,
  VSUBSHZrm_Int = 21837,
  VSUBSHZrmk_Int = 21838,
  VSUBSHZrmkz_Int = 21839,
  VSUBSHZrr = 21840,
  VSUBSHZrr_Int = 21841,
  VSUBSHZrrb_Int = 21842,
  VSUBSHZrrbk_Int = 21843,
  VSUBSHZrrbkz_Int = 21844,
  VSUBSHZrrk_Int = 21845,
  VSUBSHZrrkz_Int = 21846,
  VSUBSSZrm = 21847,
  VSUBSSZrm_Int = 21848,
  VSUBSSZrmk_Int = 21849,
  VSUBSSZrmkz_Int = 21850,
  VSUBSSZrr = 21851,
  VSUBSSZrr_Int = 21852,
  VSUBSSZrrb_Int = 21853,
  VSUBSSZrrbk_Int = 21854,
  VSUBSSZrrbkz_Int = 21855,
  VSUBSSZrrk_Int = 21856,
  VSUBSSZrrkz_Int = 21857,
  VSUBSSrm = 21858,
  VSUBSSrm_Int = 21859,
  VSUBSSrr = 21860,
  VSUBSSrr_Int = 21861,
  VTESTPDYrm = 21862,
  VTESTPDYrr = 21863,
  VTESTPDrm = 21864,
  VTESTPDrr = 21865,
  VTESTPSYrm = 21866,
  VTESTPSYrr = 21867,
  VTESTPSrm = 21868,
  VTESTPSrr = 21869,
  VUCOMISDZrm = 21870,
  VUCOMISDZrm_Int = 21871,
  VUCOMISDZrr = 21872,
  VUCOMISDZrr_Int = 21873,
  VUCOMISDZrrb = 21874,
  VUCOMISDrm = 21875,
  VUCOMISDrm_Int = 21876,
  VUCOMISDrr = 21877,
  VUCOMISDrr_Int = 21878,
  VUCOMISHZrm = 21879,
  VUCOMISHZrm_Int = 21880,
  VUCOMISHZrr = 21881,
  VUCOMISHZrr_Int = 21882,
  VUCOMISHZrrb = 21883,
  VUCOMISSZrm = 21884,
  VUCOMISSZrm_Int = 21885,
  VUCOMISSZrr = 21886,
  VUCOMISSZrr_Int = 21887,
  VUCOMISSZrrb = 21888,
  VUCOMISSrm = 21889,
  VUCOMISSrm_Int = 21890,
  VUCOMISSrr = 21891,
  VUCOMISSrr_Int = 21892,
  VUCOMXSDZrm = 21893,
  VUCOMXSDZrm_Int = 21894,
  VUCOMXSDZrr = 21895,
  VUCOMXSDZrr_Int = 21896,
  VUCOMXSDZrrb_Int = 21897,
  VUCOMXSHZrm = 21898,
  VUCOMXSHZrm_Int = 21899,
  VUCOMXSHZrr = 21900,
  VUCOMXSHZrr_Int = 21901,
  VUCOMXSHZrrb_Int = 21902,
  VUCOMXSSZrm = 21903,
  VUCOMXSSZrm_Int = 21904,
  VUCOMXSSZrr = 21905,
  VUCOMXSSZrr_Int = 21906,
  VUCOMXSSZrrb_Int = 21907,
  VUNPCKHPDYrm = 21908,
  VUNPCKHPDYrr = 21909,
  VUNPCKHPDZ128rm = 21910,
  VUNPCKHPDZ128rmb = 21911,
  VUNPCKHPDZ128rmbk = 21912,
  VUNPCKHPDZ128rmbkz = 21913,
  VUNPCKHPDZ128rmk = 21914,
  VUNPCKHPDZ128rmkz = 21915,
  VUNPCKHPDZ128rr = 21916,
  VUNPCKHPDZ128rrk = 21917,
  VUNPCKHPDZ128rrkz = 21918,
  VUNPCKHPDZ256rm = 21919,
  VUNPCKHPDZ256rmb = 21920,
  VUNPCKHPDZ256rmbk = 21921,
  VUNPCKHPDZ256rmbkz = 21922,
  VUNPCKHPDZ256rmk = 21923,
  VUNPCKHPDZ256rmkz = 21924,
  VUNPCKHPDZ256rr = 21925,
  VUNPCKHPDZ256rrk = 21926,
  VUNPCKHPDZ256rrkz = 21927,
  VUNPCKHPDZrm = 21928,
  VUNPCKHPDZrmb = 21929,
  VUNPCKHPDZrmbk = 21930,
  VUNPCKHPDZrmbkz = 21931,
  VUNPCKHPDZrmk = 21932,
  VUNPCKHPDZrmkz = 21933,
  VUNPCKHPDZrr = 21934,
  VUNPCKHPDZrrk = 21935,
  VUNPCKHPDZrrkz = 21936,
  VUNPCKHPDrm = 21937,
  VUNPCKHPDrr = 21938,
  VUNPCKHPSYrm = 21939,
  VUNPCKHPSYrr = 21940,
  VUNPCKHPSZ128rm = 21941,
  VUNPCKHPSZ128rmb = 21942,
  VUNPCKHPSZ128rmbk = 21943,
  VUNPCKHPSZ128rmbkz = 21944,
  VUNPCKHPSZ128rmk = 21945,
  VUNPCKHPSZ128rmkz = 21946,
  VUNPCKHPSZ128rr = 21947,
  VUNPCKHPSZ128rrk = 21948,
  VUNPCKHPSZ128rrkz = 21949,
  VUNPCKHPSZ256rm = 21950,
  VUNPCKHPSZ256rmb = 21951,
  VUNPCKHPSZ256rmbk = 21952,
  VUNPCKHPSZ256rmbkz = 21953,
  VUNPCKHPSZ256rmk = 21954,
  VUNPCKHPSZ256rmkz = 21955,
  VUNPCKHPSZ256rr = 21956,
  VUNPCKHPSZ256rrk = 21957,
  VUNPCKHPSZ256rrkz = 21958,
  VUNPCKHPSZrm = 21959,
  VUNPCKHPSZrmb = 21960,
  VUNPCKHPSZrmbk = 21961,
  VUNPCKHPSZrmbkz = 21962,
  VUNPCKHPSZrmk = 21963,
  VUNPCKHPSZrmkz = 21964,
  VUNPCKHPSZrr = 21965,
  VUNPCKHPSZrrk = 21966,
  VUNPCKHPSZrrkz = 21967,
  VUNPCKHPSrm = 21968,
  VUNPCKHPSrr = 21969,
  VUNPCKLPDYrm = 21970,
  VUNPCKLPDYrr = 21971,
  VUNPCKLPDZ128rm = 21972,
  VUNPCKLPDZ128rmb = 21973,
  VUNPCKLPDZ128rmbk = 21974,
  VUNPCKLPDZ128rmbkz = 21975,
  VUNPCKLPDZ128rmk = 21976,
  VUNPCKLPDZ128rmkz = 21977,
  VUNPCKLPDZ128rr = 21978,
  VUNPCKLPDZ128rrk = 21979,
  VUNPCKLPDZ128rrkz = 21980,
  VUNPCKLPDZ256rm = 21981,
  VUNPCKLPDZ256rmb = 21982,
  VUNPCKLPDZ256rmbk = 21983,
  VUNPCKLPDZ256rmbkz = 21984,
  VUNPCKLPDZ256rmk = 21985,
  VUNPCKLPDZ256rmkz = 21986,
  VUNPCKLPDZ256rr = 21987,
  VUNPCKLPDZ256rrk = 21988,
  VUNPCKLPDZ256rrkz = 21989,
  VUNPCKLPDZrm = 21990,
  VUNPCKLPDZrmb = 21991,
  VUNPCKLPDZrmbk = 21992,
  VUNPCKLPDZrmbkz = 21993,
  VUNPCKLPDZrmk = 21994,
  VUNPCKLPDZrmkz = 21995,
  VUNPCKLPDZrr = 21996,
  VUNPCKLPDZrrk = 21997,
  VUNPCKLPDZrrkz = 21998,
  VUNPCKLPDrm = 21999,
  VUNPCKLPDrr = 22000,
  VUNPCKLPSYrm = 22001,
  VUNPCKLPSYrr = 22002,
  VUNPCKLPSZ128rm = 22003,
  VUNPCKLPSZ128rmb = 22004,
  VUNPCKLPSZ128rmbk = 22005,
  VUNPCKLPSZ128rmbkz = 22006,
  VUNPCKLPSZ128rmk = 22007,
  VUNPCKLPSZ128rmkz = 22008,
  VUNPCKLPSZ128rr = 22009,
  VUNPCKLPSZ128rrk = 22010,
  VUNPCKLPSZ128rrkz = 22011,
  VUNPCKLPSZ256rm = 22012,
  VUNPCKLPSZ256rmb = 22013,
  VUNPCKLPSZ256rmbk = 22014,
  VUNPCKLPSZ256rmbkz = 22015,
  VUNPCKLPSZ256rmk = 22016,
  VUNPCKLPSZ256rmkz = 22017,
  VUNPCKLPSZ256rr = 22018,
  VUNPCKLPSZ256rrk = 22019,
  VUNPCKLPSZ256rrkz = 22020,
  VUNPCKLPSZrm = 22021,
  VUNPCKLPSZrmb = 22022,
  VUNPCKLPSZrmbk = 22023,
  VUNPCKLPSZrmbkz = 22024,
  VUNPCKLPSZrmk = 22025,
  VUNPCKLPSZrmkz = 22026,
  VUNPCKLPSZrr = 22027,
  VUNPCKLPSZrrk = 22028,
  VUNPCKLPSZrrkz = 22029,
  VUNPCKLPSrm = 22030,
  VUNPCKLPSrr = 22031,
  VXORPDYrm = 22032,
  VXORPDYrr = 22033,
  VXORPDZ128rm = 22034,
  VXORPDZ128rmb = 22035,
  VXORPDZ128rmbk = 22036,
  VXORPDZ128rmbkz = 22037,
  VXORPDZ128rmk = 22038,
  VXORPDZ128rmkz = 22039,
  VXORPDZ128rr = 22040,
  VXORPDZ128rrk = 22041,
  VXORPDZ128rrkz = 22042,
  VXORPDZ256rm = 22043,
  VXORPDZ256rmb = 22044,
  VXORPDZ256rmbk = 22045,
  VXORPDZ256rmbkz = 22046,
  VXORPDZ256rmk = 22047,
  VXORPDZ256rmkz = 22048,
  VXORPDZ256rr = 22049,
  VXORPDZ256rrk = 22050,
  VXORPDZ256rrkz = 22051,
  VXORPDZrm = 22052,
  VXORPDZrmb = 22053,
  VXORPDZrmbk = 22054,
  VXORPDZrmbkz = 22055,
  VXORPDZrmk = 22056,
  VXORPDZrmkz = 22057,
  VXORPDZrr = 22058,
  VXORPDZrrk = 22059,
  VXORPDZrrkz = 22060,
  VXORPDrm = 22061,
  VXORPDrr = 22062,
  VXORPSYrm = 22063,
  VXORPSYrr = 22064,
  VXORPSZ128rm = 22065,
  VXORPSZ128rmb = 22066,
  VXORPSZ128rmbk = 22067,
  VXORPSZ128rmbkz = 22068,
  VXORPSZ128rmk = 22069,
  VXORPSZ128rmkz = 22070,
  VXORPSZ128rr = 22071,
  VXORPSZ128rrk = 22072,
  VXORPSZ128rrkz = 22073,
  VXORPSZ256rm = 22074,
  VXORPSZ256rmb = 22075,
  VXORPSZ256rmbk = 22076,
  VXORPSZ256rmbkz = 22077,
  VXORPSZ256rmk = 22078,
  VXORPSZ256rmkz = 22079,
  VXORPSZ256rr = 22080,
  VXORPSZ256rrk = 22081,
  VXORPSZ256rrkz = 22082,
  VXORPSZrm = 22083,
  VXORPSZrmb = 22084,
  VXORPSZrmbk = 22085,
  VXORPSZrmbkz = 22086,
  VXORPSZrmk = 22087,
  VXORPSZrmkz = 22088,
  VXORPSZrr = 22089,
  VXORPSZrrk = 22090,
  VXORPSZrrkz = 22091,
  VXORPSrm = 22092,
  VXORPSrr = 22093,
  VZEROALL = 22094,
  VZEROUPPER = 22095,
  WAIT = 22096,
  WBINVD = 22097,
  WBNOINVD = 22098,
  WRFSBASE = 22099,
  WRFSBASE64 = 22100,
  WRGSBASE = 22101,
  WRGSBASE64 = 22102,
  WRMSR = 22103,
  WRMSRLIST = 22104,
  WRMSRNS = 22105,
  WRMSRNSir = 22106,
  WRMSRNSir_EVEX = 22107,
  WRPKRUr = 22108,
  WRSSD = 22109,
  WRSSD_EVEX = 22110,
  WRSSQ = 22111,
  WRSSQ_EVEX = 22112,
  WRUSSD = 22113,
  WRUSSD_EVEX = 22114,
  WRUSSQ = 22115,
  WRUSSQ_EVEX = 22116,
  XABORT = 22117,
  XACQUIRE_PREFIX = 22118,
  XADD16rm = 22119,
  XADD16rr = 22120,
  XADD32rm = 22121,
  XADD32rr = 22122,
  XADD64rm = 22123,
  XADD64rr = 22124,
  XADD8rm = 22125,
  XADD8rr = 22126,
  XAM_F = 22127,
  XAM_Fp32 = 22128,
  XAM_Fp64 = 22129,
  XAM_Fp80 = 22130,
  XBEGIN = 22131,
  XBEGIN_2 = 22132,
  XBEGIN_4 = 22133,
  XCHG16ar = 22134,
  XCHG16rm = 22135,
  XCHG16rr = 22136,
  XCHG32ar = 22137,
  XCHG32rm = 22138,
  XCHG32rr = 22139,
  XCHG64ar = 22140,
  XCHG64rm = 22141,
  XCHG64rr = 22142,
  XCHG8rm = 22143,
  XCHG8rr = 22144,
  XCH_F = 22145,
  XCRYPTCBC = 22146,
  XCRYPTCFB = 22147,
  XCRYPTCTR = 22148,
  XCRYPTECB = 22149,
  XCRYPTOFB = 22150,
  XEND = 22151,
  XGETBV = 22152,
  XLAT = 22153,
  XOR16i16 = 22154,
  XOR16mi = 22155,
  XOR16mi8 = 22156,
  XOR16mi8_EVEX = 22157,
  XOR16mi8_ND = 22158,
  XOR16mi8_NF = 22159,
  XOR16mi8_NF_ND = 22160,
  XOR16mi_EVEX = 22161,
  XOR16mi_ND = 22162,
  XOR16mi_NF = 22163,
  XOR16mi_NF_ND = 22164,
  XOR16mr = 22165,
  XOR16mr_EVEX = 22166,
  XOR16mr_ND = 22167,
  XOR16mr_NF = 22168,
  XOR16mr_NF_ND = 22169,
  XOR16ri = 22170,
  XOR16ri8 = 22171,
  XOR16ri8_EVEX = 22172,
  XOR16ri8_ND = 22173,
  XOR16ri8_NF = 22174,
  XOR16ri8_NF_ND = 22175,
  XOR16ri_EVEX = 22176,
  XOR16ri_ND = 22177,
  XOR16ri_NF = 22178,
  XOR16ri_NF_ND = 22179,
  XOR16rm = 22180,
  XOR16rm_EVEX = 22181,
  XOR16rm_ND = 22182,
  XOR16rm_NF = 22183,
  XOR16rm_NF_ND = 22184,
  XOR16rr = 22185,
  XOR16rr_EVEX = 22186,
  XOR16rr_EVEX_REV = 22187,
  XOR16rr_ND = 22188,
  XOR16rr_ND_REV = 22189,
  XOR16rr_NF = 22190,
  XOR16rr_NF_ND = 22191,
  XOR16rr_NF_ND_REV = 22192,
  XOR16rr_NF_REV = 22193,
  XOR16rr_REV = 22194,
  XOR32i32 = 22195,
  XOR32mi = 22196,
  XOR32mi8 = 22197,
  XOR32mi8_EVEX = 22198,
  XOR32mi8_ND = 22199,
  XOR32mi8_NF = 22200,
  XOR32mi8_NF_ND = 22201,
  XOR32mi_EVEX = 22202,
  XOR32mi_ND = 22203,
  XOR32mi_NF = 22204,
  XOR32mi_NF_ND = 22205,
  XOR32mr = 22206,
  XOR32mr_EVEX = 22207,
  XOR32mr_ND = 22208,
  XOR32mr_NF = 22209,
  XOR32mr_NF_ND = 22210,
  XOR32ri = 22211,
  XOR32ri8 = 22212,
  XOR32ri8_EVEX = 22213,
  XOR32ri8_ND = 22214,
  XOR32ri8_NF = 22215,
  XOR32ri8_NF_ND = 22216,
  XOR32ri_EVEX = 22217,
  XOR32ri_ND = 22218,
  XOR32ri_NF = 22219,
  XOR32ri_NF_ND = 22220,
  XOR32rm = 22221,
  XOR32rm_EVEX = 22222,
  XOR32rm_ND = 22223,
  XOR32rm_NF = 22224,
  XOR32rm_NF_ND = 22225,
  XOR32rr = 22226,
  XOR32rr_EVEX = 22227,
  XOR32rr_EVEX_REV = 22228,
  XOR32rr_ND = 22229,
  XOR32rr_ND_REV = 22230,
  XOR32rr_NF = 22231,
  XOR32rr_NF_ND = 22232,
  XOR32rr_NF_ND_REV = 22233,
  XOR32rr_NF_REV = 22234,
  XOR32rr_REV = 22235,
  XOR64i32 = 22236,
  XOR64mi32 = 22237,
  XOR64mi32_EVEX = 22238,
  XOR64mi32_ND = 22239,
  XOR64mi32_NF = 22240,
  XOR64mi32_NF_ND = 22241,
  XOR64mi8 = 22242,
  XOR64mi8_EVEX = 22243,
  XOR64mi8_ND = 22244,
  XOR64mi8_NF = 22245,
  XOR64mi8_NF_ND = 22246,
  XOR64mr = 22247,
  XOR64mr_EVEX = 22248,
  XOR64mr_ND = 22249,
  XOR64mr_NF = 22250,
  XOR64mr_NF_ND = 22251,
  XOR64ri32 = 22252,
  XOR64ri32_EVEX = 22253,
  XOR64ri32_ND = 22254,
  XOR64ri32_NF = 22255,
  XOR64ri32_NF_ND = 22256,
  XOR64ri8 = 22257,
  XOR64ri8_EVEX = 22258,
  XOR64ri8_ND = 22259,
  XOR64ri8_NF = 22260,
  XOR64ri8_NF_ND = 22261,
  XOR64rm = 22262,
  XOR64rm_EVEX = 22263,
  XOR64rm_ND = 22264,
  XOR64rm_NF = 22265,
  XOR64rm_NF_ND = 22266,
  XOR64rr = 22267,
  XOR64rr_EVEX = 22268,
  XOR64rr_EVEX_REV = 22269,
  XOR64rr_ND = 22270,
  XOR64rr_ND_REV = 22271,
  XOR64rr_NF = 22272,
  XOR64rr_NF_ND = 22273,
  XOR64rr_NF_ND_REV = 22274,
  XOR64rr_NF_REV = 22275,
  XOR64rr_REV = 22276,
  XOR8i8 = 22277,
  XOR8mi = 22278,
  XOR8mi8 = 22279,
  XOR8mi_EVEX = 22280,
  XOR8mi_ND = 22281,
  XOR8mi_NF = 22282,
  XOR8mi_NF_ND = 22283,
  XOR8mr = 22284,
  XOR8mr_EVEX = 22285,
  XOR8mr_ND = 22286,
  XOR8mr_NF = 22287,
  XOR8mr_NF_ND = 22288,
  XOR8ri = 22289,
  XOR8ri8 = 22290,
  XOR8ri_EVEX = 22291,
  XOR8ri_ND = 22292,
  XOR8ri_NF = 22293,
  XOR8ri_NF_ND = 22294,
  XOR8rm = 22295,
  XOR8rm_EVEX = 22296,
  XOR8rm_ND = 22297,
  XOR8rm_NF = 22298,
  XOR8rm_NF_ND = 22299,
  XOR8rr = 22300,
  XOR8rr_EVEX = 22301,
  XOR8rr_EVEX_REV = 22302,
  XOR8rr_ND = 22303,
  XOR8rr_ND_REV = 22304,
  XOR8rr_NF = 22305,
  XOR8rr_NF_ND = 22306,
  XOR8rr_NF_ND_REV = 22307,
  XOR8rr_NF_REV = 22308,
  XOR8rr_NOREX = 22309,
  XOR8rr_REV = 22310,
  XORPDrm = 22311,
  XORPDrr = 22312,
  XORPSrm = 22313,
  XORPSrr = 22314,
  XRELEASE_PREFIX = 22315,
  XRESLDTRK = 22316,
  XRSTOR = 22317,
  XRSTOR64 = 22318,
  XRSTORS = 22319,
  XRSTORS64 = 22320,
  XSAVE = 22321,
  XSAVE64 = 22322,
  XSAVEC = 22323,
  XSAVEC64 = 22324,
  XSAVEOPT = 22325,
  XSAVEOPT64 = 22326,
  XSAVES = 22327,
  XSAVES64 = 22328,
  XSETBV = 22329,
  XSHA1 = 22330,
  XSHA256 = 22331,
  XSTORE = 22332,
  XSUSLDTRK = 22333,
  XTEST = 22334,
  INSTRUCTION_LIST_END = 22335,
};
}
}
}
#endif
